////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube AppTask
//
// ghmGcAppTask.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/09/2002
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCAPPTASK_H__
#define	__GHMGCAPPTASK_H__

#include "ghmGcTypes.h"
#include "ghmGcList.h"
#include "ghmGcMem.h"

class ghmGcApp;

#define	GHMGCAPPTASK_PRERENDERPROCESS		(1<<0)
#define	GHMGCAPPTASK_RENDERPROCESS			(1<<1)
#define	GHMGCAPPTASK_POSTRENDERPROCESS		(1<<2)
#define	GHMGCAPPTASK_PREFRAMEPROCESS		(1<<3)
#define	GHMGCAPPTASK_FRAMEPROCESS			(1<<4)
#define	GHMGCAPPTASK_POSTFRAMEPROCESS		(1<<5)

#define	GHMGCAPPTASK_PROCESSFLAGALL			(GHMGCAPPTASK_PRERENDERPROCESS | \
                                             GHMGCAPPTASK_RENDERPROCESS    | \
                                             GHMGCAPPTASK_POSTRENDERPROCESS| \
                                             GHMGCAPPTASK_PREFRAMEPROCESS  | \
                                             GHMGCAPPTASK_FRAMEPROCESS     | \
                                             GHMGCAPPTASK_POSTFRAMEPROCESS )

//
// AppTask
//
class ghmGcAppTask : public ghmGcListObj{
public:
	enum State{
		INIT = 0,
		EXEC,
		TERM,
		DELETE
	};
private:
	State							mState;
	ghmGcApp						*mpApplication;
	int								mID;
	
	union{
		struct{
			u_int					mPreRenderProcess : 1;
			u_int					mRenderProcess    : 1;
			u_int					mPostRenderProcess: 1;
			u_int					mPreFrameProcess  : 1;
			u_int					mFrameProcess     : 1;
			u_int					mPostFrameProcess : 1;
			u_int					mFlagPad          :26;
		};
		u_int						mFlag;
	};
	
public:
	ghmGcAppTask();												// AJ[쐬p
	ghmGcAppTask(ghmGcApp *pApp,int prio=GHMGCLIST_DEFAULT_PRIO);
	virtual ~ghmGcAppTask();
	
	inline ghmGcApp* GetApplication(void){ return mpApplication; }
	inline State GetState(void){ return mState; }
	inline u_int GetFlag(void){ return mFlag; }
	inline void SetFlag(u_int flag){ mFlag |= flag; }
	inline void ResetFlag(u_int flag){ mFlag &= ~flag; }
	inline int GetID(void){ return mID; }
	
	// I[o[Ch֎~
	// R[obN֐ŌĂяoĂ̂ŁA
	// R[obN֐I[o[Ch邱ƁI
	// 
	bool Initialize(void);
	// IiR[obNtrueԂ܂Ń^XN͍폜Ȃj
	bool Terminate(void);
	// IiP񂵂R[obNĂ΂Ȃj
	bool DeleteTerminate(void);
	// I[o[Ch֎~
	
	// R[obN
	virtual bool InitializeCallBack(void){ return true; }
	// IR[obN
	virtual bool TerminateCallBack(void){ return true; }
	
	// t[
	virtual bool PreFrameProcess(void){ return true; }
	virtual bool FrameProcess(void){ return true; }
	virtual bool PostFrameProcess(void){ return true; }
	
	// _O
	virtual bool PreRenderProcess(void){ return true; }
	virtual bool RenderProcess(void){ return true; }
	virtual bool PostRenderProcess(void){ return true; }
	
	virtual bool Join(ghmGcListObj */*pAnchor*/,int /*prio*/){ return false; }
	virtual bool Remove(void){ return Terminate(); }
	
	ghmGcAppTask* GetNext(void){ return (ghmGcAppTask *)ghmGcListObj::GetNext(); }
	ghmGcAppTask* GetPrev(void){ return (ghmGcAppTask *)ghmGcListObj::GetPrev(); }
};

#endif	//__GHMGCAPPTASK_H__

