////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Memory Manager
//
// ghmGcDefine.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/07/2002
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCDEFINE_H__
#define	__GHMGCDEFINE_H__

#include <dolphin.h>

#ifndef	PI
#define	PI									(3.14159265358979f)
#endif
#ifndef	HALF_PI
#define	HALF_PI								(PI * 0.50f)
#endif
#ifndef	TWO_PI
#define	TWO_PI								(2.0f * PI)
#endif
#ifndef	INV_PI
#define	INV_PI								(1.0f / PI)
#endif

#define	D2R(deg)							((deg) * (PI/180.0f))
#define	R2D(rad)							((rad) * (180.0f/PI))

#define	GHM_VEC_SET_XYZ(vec,nx,ny,nz)		(vec)->x=(nx);(vec)->y=(ny);(vec)->z=(nz);
#define	GHM_VEC_SET_XYZW(vec,nx,ny,nz,nw)	(vec)->x=(nx);(vec)->y=(ny);(vec)->z=(nz);(vec)->w=(nw);
#define	GHM_VEC_SET_XYZ0(vec)				(vec)->x=0.0f;(vec)->y=0.0f;(vec)->z=0.0f;
#define	GHM_VEC_SET_CLEAR(vec)				GHM_VEC_SET_XYZ0(vec)
#define	GHM_VEC_SET_XYZ0W1(vec)				(vec)->x=0.0f;(vec)->y=0.0f;(vec)->z=0.0f;(vec)->w=1.0f;
#define	GHM_VEC_SET_XYZ0W0(vec)				(vec)->x=0.0f;(vec)->y=0.0f;(vec)->z=0.0f;(vec)->w=0.0f;

#define	GHM_VEC_COPY_XYZ(dst,src)			(dst)->x=(src)->x;(dst)->y=(src)->y;(dst)->z=(src)->z;
#define	GHM_VEC_COPY_XYZW(dst,src)			(dst)->x=(src)->x;(dst)->y=(src)->y;(dst)->z=(src)->z;(dst)->w=(src)->w;

//#define	GHM_VEC_GET_LEN(vec)				sqrtf((vec)->x * (vec)->x + (vec)->y * (vec)->y + (vec)->z * (vec)->z)
#define	GHM_VEC_GET_LEN(vec)				VECMag((vec))

#define	GHM_VEC_DIST(vec0,vec1)				VECDistance((vec0),(vec1))
//#define	GHM_VEC_DIST(vec0,vec1)		\
//					sqrtf(((vec1)->x - (vec0)->x) * ((vec1)->x - (vec0)->x) \
//					     +((vec1)->y - (vec0)->y) * ((vec1)->y - (vec0)->y) \
//					     +((vec1)->z - (vec0)->z) * ((vec1)->z - (vec0)->z))
#define	GHM_VEC_GET_DIST(vec0,vec1)		GHM_VEC_DIST(vec0,vec1)
#define	GHM_VEC2_GET_DIST(vec0,vec1)	\
					sqrtf(((vec1)->x - (vec0)->x) * ((vec1)->x - (vec0)->x) \
					     +((vec1)->y - (vec0)->y) * ((vec1)->y - (vec0)->y))

#define	GHM_LIMIT(val,max,min)	\
					if( (val) > (max) ){ \
						(val) = (max); \
					}else if( (val) < (min) ){ \
						(val) = (min); \
					}
#define	GHM_CLAMP(val,max,min)	(((val) > (max))? max : (((val) < (min))? min : val))

#define	GHM_LIMIT_RANGE(val,max,min)	\
					if( (val) > (max) ){ \
						while( (val) >= (max) ){ \
							(val) -= (max) - (min); \
						} \
					}else if( (val) < (min) ){ \
						while( (val) <= (min) ){ \
							(val) += (max) - (min); \
						} \
					}

#define	GHM_SAFE_DELETE(p)		\
					if( (p) != NULL ){ \
						delete (p); \
						(p)  = NULL; \
					}

#define	GHM_SAFE_DELETE_ARRAY(p)		\
					if( (p) != NULL ){ \
						delete[] (p); \
						(p)  = NULL; \
					}

#define	GHM_ROUNDUP(val,align)			(((u32)(val) + align - 1) & ~(align - 1))
#define	GHM_ROUNDDOWN(val,align)		(((u32)(val)) & ~(align - 1))
#define	GHM_ROUNDUP_QUOT(val,align)		(((u32)(val) + align - 1) / align)
#define	GHM_ROUNDDOWN_QUOT(val,align)	(((u32)(val)) / align)

#define	CONV_INT_ENDIAN(val)		((((int)(val)<<24)&0xff000000)|(((int)(val)<< 8)&0x00ff0000)| \
									 (((int)(val)>> 8)&0x0000ff00)|(((int)(val)>>24)&0x000000ff))
#define	CONV_SHORT_ENDIAN(val)		((((int)(val)<<8)&0xff00)|(((int)(val)>>8)&0x00ff))

#define	CONV_UINT_ENDIAN(val)		((u_int)CONV_INT_ENDIAN(val))
#define	CONV_USHORT_ENDIAN(val)		((u_short)CONV_SHORT_ENDIAN(val))

#define	CONV_S32_ENDIAN(val)		CONV_INT_ENDIAN(val)
#define	CONV_S16_ENDIAN(val)		CONV_SHORT_ENDIAN(val)
#define	CONV_U32_ENDIAN(val)		CONV_UINT_ENDIAN(val)
#define	CONV_U16_ENDIAN(val)		CONV_USHORT_ENDIAN(val)

#define	GHM_ABS(val)				((val)>=0   ? (val) : -(val))
#define	GHM_FABS(val)				((val)>=0.0f? (val) : -(val))

#ifdef	_DEBUG
#define	OSDebugReport(...)			OSReport(__VA_ARGS__)
#define	OSDebugReportExp(exp,...)	if( (exp)==true ){ OSReport(__VA_ARGS__); }
#else
#define	OSDebugReport(...)			
#define	OSDebugReportExp(exp,...)	
#endif

#endif	//__GHMGCDEFINE_H__

