////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube File Manager
//
// ghmGcMem.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/07/2002
//
////////////////////////////////////////////////////////////////////////////////

#define	__GHMGCFILE_CPP__

#include <string.h>
#include <dolphin.h>
#include <sysdolphin\baselib.h>
#include <sysdolphin\audiolib_ax.h>
#include "ghmGcFile.h"

//#define	GHMGCFILE_DEBUG

#ifndef	GHMGCFILE_DISKNUM
#define	GHMGCFILE_DISKNUM			(2)
#endif

#if	GHMGCFILE_DISKNUM <= 0
#undef	GHMGCFILE_DISKNUM
#define	GHMGCFILE_DISKNUM			(1)
#endif

#ifndef	GHMGCFILE_GAMENAME
//#define	GHMGCFILE_GAMENAME			"TEST"
//#define	GHMGCFILE_GAMENAME			"KLR7"
#define	GHMGCFILE_GAMENAME			"GK7J"
#endif
#ifndef	GHMGCFILE_COMPANYNAME
//#define	GHMGCFILE_COMPANYNAME		"GH"
#define	GHMGCFILE_COMPANYNAME		"08"
#endif
#ifndef	GHMGCFILE_VERSION
#define	GHMGCFILE_VERSION			(2)
#endif

//#define	GHMGCFILE_ACCESSREPORT
#ifndef	_DEBUG
#undef	GHMGCFILE_ACCESSREPORT
#endif

ghmGcFile*										ghmGcFile::mpFileTop      = NULL;
ghmGcMem*										ghmGcFile::mpAllocHeap    = &ghmGcMem::mDefaultHeap;
bool											ghmGcFile::mUseDevCom     = false;
//int											ghmGcFile_mReadRequest = 0;
//int											ghmGcFile_mReadEnd     = 0;
DVDDiskID * 									ghmGcFile::mpDiskID       = NULL;
int												ghmGcFile::mDiskNum       = 0;
s32												ghmGcFile::mDriveState    = DVD_STATE_END;
volatile s32									ghmGcFile::mCurrentDiskID = -1;
s32												ghmGcFile::mReserveDiskID = -1;
volatile ghmGcFile::ghmGcFileChangeDiskState	ghmGcFile::mChangeDiskState = CHANGED;
DVDCommandBlock									ghmGcFile::mChangeDiskCommandBlock;

ghmGcFile::ghmGcFile()
{
	memset(&mInfo,0,sizeof(mInfo));
	mInfo.mState = CLOSED;
	mpPrev       = NULL;
	mpNext       = mpFileTop;
	if( mpNext!=NULL ){
		mpNext->mpPrev = this;
	}
	mpFileTop = this;
}

ghmGcFile::~ghmGcFile()
{
	switch( mInfo.mState ){
	case	READ:
	case	READRETRY:
		CancelB();
	default:
		Close();
		break;
	}
	if( mpPrev != NULL ){
		mpPrev->mpNext = mpNext;
	}else{
		mpFileTop = mpNext;
	}
	if( mpNext != NULL ){
		mpNext->mpPrev = mpPrev;
	}
}

int ghmGcFile::GetDiskNum(void)
{
	return mDiskNum;
}

int ghmGcFile::_GetCurrentDiskID(void)
{
	DVDDiskID	*pDiskID;
	if( (pDiskID = DVDGetCurrentDiskID())==NULL ){
		return -1;
	}
	int		id;
	for(id=mDiskNum-1 ; id>=0 ; id--){
		if( DVDCompareDiskID(pDiskID,mpDiskID+id)==TRUE ){
			break;
		}
	}
	return id;
}

bool ghmGcFile::Initialize(void)
{
	u8	i;
	GHM_SAFE_DELETE_ARRAY(mpDiskID);
	ghmGcMemSetAllocName("DiskID");
	if( GHMGCFILE_DISKNUM <= 0 || (mpDiskID = new DVDDiskID[GHMGCFILE_DISKNUM])==NULL ){
		OSPanic(__FILE__, __LINE__,"DiskID Work Allocation Error\n");
	}
	mDiskNum = GHMGCFILE_DISKNUM;
	for(i=0 ; i<GHMGCFILE_DISKNUM ; i++){
		if( DVDGenerateDiskID(mpDiskID + i,
		                      GHMGCFILE_GAMENAME,
		                      GHMGCFILE_COMPANYNAME,
		                      i,
		                      (u8)GHMGCFILE_VERSION)!=(mpDiskID + i) ){
			OSPanic(__FILE__, __LINE__,"DiskID Generate Error : %d\n",i);
		}
	}
	mReserveDiskID =
	mCurrentDiskID = _GetCurrentDiskID();
	return true;
}

bool ghmGcFile::Terminate(void)
{
	GHM_SAFE_DELETE_ARRAY(mpDiskID);
	return true;
}

void ghmGcFile::ChangeDiskCallback(s32 result, DVDCommandBlock* block)
{
#pragma unused(block)
	switch( result ){
	case	0:
		mCurrentDiskID   = _GetCurrentDiskID();
		mChangeDiskState = CHANGED;
		break;
	case	DVD_RESULT_FATAL_ERROR:
		mChangeDiskState = FATAL_ERROR;
		break;
	case	DVD_RESULT_CANCELED:
		mChangeDiskState = CHANGECANCELED;
		break;
	default:
		mChangeDiskState = UNKNOWN;
		break;
	}
}

bool ghmGcFile::ChangeDisk(s32 id)
{
	if( id < 0 || id >= mDiskNum ){
		return false;
	}
	if( mReserveDiskID==id ){
		return true;
	}
	switch( mChangeDiskState ){
	case	IDLE:
		break;
	default:
		return false;
	}
	mReserveDiskID = id;
	int	intflag = OSDisableInterrupts();
	if( DVDChangeDiskAsync(&mChangeDiskCommandBlock,
	                        mpDiskID + mReserveDiskID,
	                        ChangeDiskCallback)==FALSE ){
		OSRestoreInterrupts(intflag);
		return false;
	}
	OSRestoreInterrupts(intflag);
	mChangeDiskState = CHANGEING;
	return true;
}

bool ghmGcFile::IsDiskStandby(void)
{
	switch( mDriveState ){
	case	DVD_STATE_FATAL_ERROR:
	case	DVD_STATE_NO_DISK:
	case	DVD_STATE_COVER_OPEN:
	case	DVD_STATE_WRONG_DISK:
	case	DVD_STATE_RETRY:
		return false;
	default:
		if( GetCurrentDiskID()!=GetReserveDiskID() ){
			return false;
		}
		break;
	}
	return true;
}

bool ghmGcFile::FrameProcessAll(void)
{
	ghmGcFile		*pFile;
	
	switch( (mDriveState = DVDGetDriveStatus()) ){
	case	DVD_STATE_FATAL_ERROR:
	case	DVD_STATE_NO_DISK:
	case	DVD_STATE_COVER_OPEN:
	case	DVD_STATE_WRONG_DISK:
	case	DVD_STATE_RETRY:
		mCurrentDiskID   = -1;
//		mCurrentDiskID   = _GetCurrentDiskID();
		break;
	case	DVD_STATE_END:
		if( mChangeDiskState==CHANGED ){
			mCurrentDiskID   = _GetCurrentDiskID();
		}
		break;
	case	DVD_STATE_BUSY:
	case	DVD_STATE_MOTOR_STOPPED:
	case	DVD_STATE_PAUSING:
	default:
		break;
	}
	
	switch( mChangeDiskState ){
	case	IDLE:
#ifdef	GHMGCFILE_DEBUG
		if( HSD_PadGetTrigger(1) & HSD_PAD_Y ){
			ChangeDisk(mCurrentDiskID ^ 1);
		}
#endif
		break;
	case	CHANGEING:
		// fBXNւ҂iR[obNŏj
		break;
	case	FATAL_ERROR:
	default:
		break;
	}
	
	for(pFile=mpFileTop ; pFile!=NULL ; pFile=pFile->mpNext){
		pFile->FrameProcess();
	}
	
	return true;
}

bool ghmGcFile::FrameProcess(void)
{
#ifndef	FILE_USB
	if( ghmGcFile::mUseDevCom==true ){
		return true;
	}
	switch( mInfo.mState ){
	case	READRETRY:
		mInfo.mReadStatus = DVDGetFileInfoStatus(&mInfo.mFileInfo);
		break;
	case	READ:
		mInfo.mReadStatus = DVDGetFileInfoStatus(&mInfo.mFileInfo);
		switch( mInfo.mReadStatus ){
		case	DVD_STATE_RETRY:
			mInfo.mState = READRETRY;
			break;
		}
		break;
	case	OPENED:
	case	CLOSED:
	default:
		break;
	}
#endif
	return true;
}

#ifdef	_DEBUG
//#ifdef	GHMGCFILE_DEBUG
#include "..\Fujita\C_Font.h"
#endif
bool ghmGcFile::RenderProcessAll(void)
{
	ghmGcFile		*pFile;
	
#ifdef	GHMGCFILE_DEBUG
	C_Paper		paper(32,40);
	
	PRINTF(">Drive State\n");
	switch( mDriveState ){
	case	DVD_STATE_FATAL_ERROR:
		PRINTF(" -> Fatal Error\n");
		break;
	case	DVD_STATE_END:
		PRINTF(" -> End\n");
		break;
	case	DVD_STATE_BUSY:
		PRINTF(" -> Busy\n");
		break;
	case	DVD_STATE_NO_DISK:
		PRINTF(" -> No Disk\n");
		break;
	case	DVD_STATE_COVER_OPEN:
		PRINTF(" -> Cover Open\n");
		break;
	case	DVD_STATE_WRONG_DISK:
		PRINTF(" -> Wrong Disk\n");
		break;
	case	DVD_STATE_PAUSING:
		PRINTF(" -> Pausing\n");
		break;
	case	DVD_STATE_MOTOR_STOPPED:
		PRINTF(" -> Motor Stopped\n");
		break;
	case	DVD_STATE_RETRY:
		PRINTF(" -> Retry\n");
		break;
	default:
		PRINTF(" -> Unknown\n");
		break;
	}
	
	PRINTF(">Current DiskID : %d\n",mCurrentDiskID);
	PRINTF(">Change Disk State\n");
	switch( mChangeDiskState ){
	case	0:
		PRINTF(" -> No Disk Change\n");
		break;
	case	1:
		PRINTF(" -> Now Disk Changing\n");
		break;
	case	2:
		PRINTF(" -> Disk Change Fatal Error\n");
		break;
	case	3:
		PRINTF(" -> Disk Change Canceled\n");
		break;
	default:
		PRINTF(" -> Unknown\n");
		break;
	}
	
	PRINTF(">Reading Files\n");
#endif
	
#ifndef	K7APP_MASTER
	if( mChangeDiskState==CHANGEING ){
		C_Paper		paper(32,40);
		PRINTF(" Insert Disk %d\n",mReserveDiskID + 1);
		switch( mDriveState ){
		case	DVD_STATE_NO_DISK:
			PRINTF(" -> No Disk\n");
			break;
		case	DVD_STATE_WRONG_DISK:
			PRINTF(" -> Wrong Disk\n");
			break;
		default:
			break;
		}
	}
#endif
	
	for(pFile=mpFileTop ; pFile!=NULL ; pFile=pFile->mpNext){
		pFile->RenderProcess();
	}
	
	return true;
}

bool ghmGcFile::RenderProcess(void)
{
#ifdef	GHMGCFILE_DEBUG
	if( mInfo.mState==READ ){
		PRINTF(" -> %s\n",mFileName);
	}
#endif
	return true;
}

bool ghmGcFile::Open(char *pFileName)
{
	if( mInfo.mState != CLOSED ){
		return false;
	}
//#ifdef	_DEBUG
	strncpy(mFileName,pFileName,sizeof(mFileName)-1);
#ifdef	GHMGCFILE_ACCESSREPORT
	OSDebugReport("File Open  : %s\n",mFileName);
#endif
//#endif
#ifdef	FILE_USB
		if( mUsb.Open(pFileName,FIO_OPEN_RDONLY)==false ){
			return false;
		}
	#ifdef	USE_ISDOLVIEWER
		mUsb.Close();
	#endif
#else
		if( DVDOpen(pFileName,&mInfo.mFileInfo)==false ){
			return false;
		}
#endif
	mInfo.mState = OPENED;
	return true;
}

bool ghmGcFile::Close(void)
{
	switch( mInfo.mState ){
	case	OPENED:
		break;
	case	READ:
	case	READRETRY:
		Cancel();
		break;
	default:
		return false;
	}
#ifdef	FILE_USB
	#ifndef	USE_ISDOLVIEWER
		if( mUsb.Close()==false ){
			return false;
		}
	#endif
#else
		if( DVDClose(&mInfo.mFileInfo)==false ){
			return false;
		}
#endif
	mInfo.mState = CLOSED;
	return true;
}

u32 ghmGcFile::GetLength(bool roundup)
{
	if( mInfo.mState==CLOSED ){
		return 0;
	}
	u32		length;
#ifdef	FILE_USB
	#ifdef	USE_ISDOLVIEWER
		if( mUsb.Open(mFileName,FIO_OPEN_RDONLY)==false ){
			return 0;
		}
	#endif
		if( (length=mUsb.GetLength())==0xffffffff ){
	#ifdef	USE_ISDOLVIEWER
			mUsb.Close();
	#endif
			return 0;
		}
	#ifdef	USE_ISDOLVIEWER
		mUsb.Close();
	#endif
#else
		length = DVDGetLength(&mInfo.mFileInfo);
#endif
	if( roundup==true ){
		length = OSRoundUp32B(length);
	}
	return length;
}

bool ghmGcFile::CancelB(void)
{
	switch( mInfo.mState ){
	case	CLOSED:
		return false;
	case	READ:
	case	READRETRY:
		break;
	case	OPENED:
	default:
		return true;
	}
#ifdef	FILE_USB
	return false;
#else
	if( mUseDevCom==true ){
		int		reqid;
		if( (reqid=HSD_DevComCancelEx(mInfo.mRequestNo,
		                              HSD_DEVCOM_CANCEL_CB_SUCCEED,
		                              NULL,
		                              NULL)) < 0 ){
			return false;
		}
		while( HSD_DevComIsBusy(1) || HSD_DevComIsBusy(2) ){}
	}else{
		if( DVDCancel((DVDCommandBlock*)&mInfo.mFileInfo) < 0 ){
			return false;
		}
	}
//	if( DVDCancel((DVDCommandBlock*)&mInfo.mFileInfo) < 0 ){
//		return false;
//	}
#endif
	return true;
}

bool ghmGcFile::Cancel(void)
{
	switch( mInfo.mState ){
	case	CLOSED:
		return false;
	case	READ:
	case	READRETRY:
		break;
	case	OPENED:
	default:
		return true;
	}
#ifdef	FILE_USB
	return false;
#else
	if( mUseDevCom==true ){
		int		reqid;
		if( (reqid=HSD_DevComCancelEx(mInfo.mRequestNo,
		                              HSD_DEVCOM_CANCEL_CB_SUCCEED,
		                              NULL,
		                              NULL)) < 0 ){
			return false;
		}
	}else{
		if( DVDCancelAsync((DVDCommandBlock *)&mInfo.mFileInfo,CancelCallBack)==false ){
			return false;
		}
	}
//	if( DVDCancelAsync((DVDCommandBlock *)&mInfo.mFileInfo,CancelCallBack)==false ){
//		return false;
//	}
#endif
	return true;
}

void* ghmGcFile::ReadB(void *pAddr,s32 offset,s32 length)
{
	switch( mInfo.mState ){
	case	OPENED:
		break;
	case	CLOSED:
	case	READ:
	case	READRETRY:
	default:
		return NULL;
	}
#ifdef	GHMGCFILE_ACCESSREPORT
	OSReport("File ReadB : %s\n",mFileName);
#endif
#ifdef	FILE_USB
	if( offset!=0 ){
		return NULL;
	}
#endif
	void		*pReadAddr;
	s32			filelen;
	filelen = (s32)GetLength();
	if( length < 0 ){
		length = filelen;
	}else{
		length = (s32)OSRoundUp32B(length);
	}
	if( offset + length > filelen || filelen <= 0 ){
		return NULL;
	}
	if( pAddr==NULL ){
#ifdef	_DEBUG
		const char	*pIdent;
		u32			line;
		if( (pIdent=ghmGcMemGetAllocName())==NULL ){
			pIdent = "FileRB";
		}
		line = ghmGcMem::GetNewLine();
		if( (pReadAddr = mpAllocHeap->Alloc(length,pIdent,line==0xffffffff? __LINE__:line))==NULL ){
#else
		if( (pReadAddr = mpAllocHeap->Alloc(length))==NULL ){
#endif
			return NULL;
		}
	}else{
		pReadAddr = pAddr;
	}
#ifdef	FILE_USB
	#ifdef	USE_ISDOLVIEWER
		if( mUsb.Open(mFileName,FIO_OPEN_RDONLY)==false ){
			return NULL;
		}
	#endif
		if( mUsb.Read(pReadAddr,(u32)length)==NULL ){
			if( pAddr==NULL ){
				mpAllocHeap->Free(pReadAddr);
			}
	#ifdef	USE_ISDOLVIEWER
			mUsb.Close();
	#endif
			return NULL;
		}
	#ifdef	USE_ISDOLVIEWER
		mUsb.Close();
	#endif
#else
		if( mUseDevCom==true ){
			mInfo.mReadResult = length;
			mInfo.mLength     = length;
			mInfo.mOffset     = 0;
			int	reqid = 
			HSD_DevComRequest((s32)DVDConvertPathToEntrynum(mFileName),
			                  0UL,
			                  (u32)pReadAddr,
			                  (u32)length,
			                  DEVCOM_DVD_TO_MEM,
			                  1,
			                  0,
			                  0);
			while( HSD_DevComIsBusy(1) || HSD_DevComIsBusy(2) ){}
		}else{
			switch( DVDRead(&mInfo.mFileInfo,pReadAddr,length,offset) ){
			case	DVD_RESULT_FATAL_ERROR:
			case	DVD_RESULT_CANCELED:
				if( pAddr==NULL ){
					mpAllocHeap->Free(pReadAddr);
				}
				return NULL;
			default:
				break;
			}
		}
#endif
	return pReadAddr;
}

void* ghmGcFile::Read(void *pAddr,s32 offset,s32 length)
{
	switch( mInfo.mState ){
	case	OPENED:
		break;
	case	CLOSED:
	case	READ:
	case	READRETRY:
	default:
		return NULL;
	}
#ifdef	GHMGCFILE_ACCESSREPORT
	OSReport("File Read  : %s\n",mFileName);
#endif
#ifdef	FILE_USB
	if( offset!=0 ){
		return NULL;
	}
#endif
	void		*pReadAddr;
	s32			filelen;
	filelen = (s32)GetLength();
	if( length < 0 ){
		length = filelen;
	}else{
		length = (s32)OSRoundUp32B(length);
	}
	if( offset + length > filelen || filelen <= 0 ){
		return NULL;
	}
	if( pAddr==NULL ){
#ifdef	_DEBUG
		const char	*pIdent;
		u32			line;
		if( (pIdent=ghmGcMemGetAllocName())==NULL ){
			pIdent = "FileRead";
		}
		line = ghmGcMem::GetNewLine();
		if( (pReadAddr = mpAllocHeap->Alloc(length,pIdent,line==0xffffffff? __LINE__:line))==NULL ){
#else
		if( (pReadAddr = mpAllocHeap->Alloc(length))==NULL ){
#endif
			return NULL;
		}
	}else{
		pReadAddr = pAddr;
	}
	mInfo.mState = READ;
#ifdef	FILE_USB
		mInfo.mState = OPENED;
	#ifdef	USE_ISDOLVIEWER
		if( mUsb.Open(mFileName,FIO_OPEN_RDONLY)==false ){
			return NULL;
		}
	#endif
		if( mUsb.Read(pReadAddr,(u32)length)==NULL ){
			if( pAddr==NULL ){
				mpAllocHeap->Free(pReadAddr);
			}
	#ifdef	USE_ISDOLVIEWER
			mUsb.Close();
	#endif
			return NULL;
		}
	#ifdef	USE_ISDOLVIEWER
		mUsb.Close();
	#endif
#else
		if( mUseDevCom==true ){
			mInfo.mReadResult = length;
			mInfo.mLength     = length;
			mInfo.mOffset     = 0;
//			ghmGcFile_mReadRequest++;
			mInfo.mRequestNo = HSD_DevComRequest((s32)DVDConvertPathToEntrynum(mFileName),
			                                     0UL,
			                                     (u32)pReadAddr,
			                                     (u32)length,
			                                     DEVCOM_DVD_TO_MEM,
			                                     1,
			                                     DevComCallback,
			                                     (void *)&mInfo);
		}else{
			if( DVDReadAsync(&mInfo.mFileInfo,pReadAddr,length,offset,ReadCallBack)==false ){
				if( pAddr==NULL ){
					mpAllocHeap->Free(pReadAddr);
				}
				mInfo.mState = OPENED;
				return NULL;
			}
		}
#endif
	return pReadAddr;
}

void ghmGcFile::DevComCallback(int dcReq,void *args,void *sbuf,int cancelflag)
{
#pragma unused(sbuf)
#ifndef	_DEBUG
#pragma unused(dcReq)
#endif
	ghmGcFileInfo	*pFileInfo = (ghmGcFileInfo *)args;
#ifdef	_DEBUG
	if( pFileInfo->mRequestNo!=dcReq ){
		OSReport("Error : Wrong File Request No. %d -> %d\n",dcReq,pFileInfo->mRequestNo);
	}
#endif
	switch( cancelflag ){
	case	0:
		// ǂݍ݊
//		ghmGcFile_mReadEnd++;
		pFileInfo->mState      = READFINISH;
		pFileInfo->mReadStatus = DVD_STATE_END;
//		pFileInfo->mReadResult = DVD_RESULT_GOOD;	// t@C̒
		break;
	case	1:
		// ǂݍ݃LZ
		pFileInfo->mState      = CANCELED;
		pFileInfo->mReadStatus = DVD_STATE_END;
		pFileInfo->mReadResult = DVD_RESULT_CANCELED;
		break;
	default:
#ifdef	_DEBUG
		OSPanic(__FILE__, __LINE__,"DevCom Callback cancelflag error : %08x\n",cancelflag);
#endif
		break;
	}
}

void ghmGcFile::ReadCallBack(s32 result,DVDFileInfo *pInfo)
{
	ghmGcFileInfo	*pFileInfo = (ghmGcFileInfo *)pInfo;
	s32				status     = DVDGetFileInfoStatus(pInfo);
	pFileInfo->mState      = OPENED;
	pFileInfo->mReadStatus = status;
	pFileInfo->mReadResult = result;
//	switch( result ){
//	case	DVD_RESULT_FATAL_ERROR:
//	case	DVD_RESULT_CANCELED:
//		break;
//	default:
//		// ǂݍݐ
//		break;
//	}
}

void ghmGcFile::CancelCallBack(s32 result,DVDCommandBlock *pBlock)
{
	ghmGcFileInfo	*pFileInfo = (ghmGcFileInfo *)pBlock;
	s32				status     = DVDGetFileInfoStatus(&pFileInfo->mFileInfo);
	pFileInfo->mState      = OPENED;
	pFileInfo->mReadStatus = status;
	pFileInfo->mReadResult = result;
}

