////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Memory Manager
//
// ghmGcMem.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/07/2002
//
////////////////////////////////////////////////////////////////////////////////

#define	__GHMGCMEM_CPP__

#include <string.h>
#include <dolphin.h>
#include "ghmGcMem.h"
#include "ghmGcThread.h"

//#define	GHMGCMEM_HEAP_24M
//
//#if	defined(K7APP_MASTER) || defined(USE_ISDOLVIEWER)
//#undef	GHMGCMEM_HEAP_24M
//#endif

//#define	GHMGCMEM_DEFAULTHEAP_MESSAGE

#ifndef	_DEBUG
#undef	GHMGCMEM_DEFAULTHEAP_MESSAGE
#endif

#define	GHMGCMEM_DEFAULT_RENDERPOSX			(32)
#define	GHMGCMEM_DEFAULT_RENDERPOSY			(20)
#define	GHMGCMEM_DEFAULT_RENDERWIDTH		(GHM_SCREEN_WIDTH - GHMGCMEM_DEFAULT_RENDERPOSX * 2)
#define	GHMGCMEM_DEFAULT_RENDERHEIGHT		(12)
#define	GHMGCMEM_DEFAULT_RENDERHEIGHT_USE	(10)

bool        ghmGcMem::mInitialized     = false;
int         ghmGcMem::mTotalHeapNum    = GHMGCMEM_DEFAULT_TOTAL_HEAP_NUM;
void*       ghmGcMem::mpArenaInitLo    = NULL;
void*       ghmGcMem::mpArenaInitHi    = NULL;
u32         ghmGcMem::mTotalArenaSize  = 0;
void*       ghmGcMem::mpArenaLo        = NULL;
void*       ghmGcMem::mpArenaHi        = NULL;
ghmGcMem*   ghmGcMem::mpHeapTop        = NULL;
u16         ghmGcMem::mRenderPosX      = GHMGCMEM_DEFAULT_RENDERPOSX;
u16         ghmGcMem::mRenderPosY      = GHMGCMEM_DEFAULT_RENDERPOSY;
u16         ghmGcMem::mRenderWidth     = GHMGCMEM_DEFAULT_RENDERWIDTH;
u16         ghmGcMem::mRenderHeight    = GHMGCMEM_DEFAULT_RENDERHEIGHT;
u16         ghmGcMem::mRenderHeightUse = GHMGCMEM_DEFAULT_RENDERHEIGHT_USE;
bool        ghmGcMem::mRender          = false;
//OSMutex     ghmGcMem::mMutex;
//OSSemaphore ghmGcMem::mSema;
//bool        ghmGcMem::mLock = false;

#ifndef	GHMGCMEM_DEFAULT_HEAP_SIZE
#define	GHMGCMEM_DEFAULT_HEAP_SIZE		(-1)
#endif

ghmGcMem	ghmGcMem::mDefaultHeap;
ghmGcMem*	ghmGcMem::mpDefaultAllocHeap = &ghmGcMem::mDefaultHeap;
#ifdef	_DEBUG
const char*	ghmGcMem::mpNewIdent = NULL;
u32			ghmGcMem::mNewLine   = 0xffffffff;
u16         ghmGcMem::mMinWidth  = 3;
#endif

void* operator new(u32 blocksize)
{
#ifdef	_DEBUG
	const char	*pIdent;
	u32			line;
	line = ghmGcMemGetNewLine();
	if( (pIdent = ghmGcMem::GetNewIdent())==NULL ){
		pIdent = "new";
	}else{
		ghmGcMem::SetNewIdent(NULL,0xffffffff);
	}
	return ghmGcMem::GetNewHeap()->Alloc((int)blocksize,pIdent,line==0xffffffff? __LINE__ : line);
#else
	return ghmGcMem::GetNewHeap()->Alloc((int)blocksize);
#endif
}
 
void* operator new[](u32 blocksize)
{
#ifdef	_DEBUG
	const char	*pIdent;
	u32			line;
	line = ghmGcMemGetNewLine();
	if( (pIdent = ghmGcMem::GetNewIdent())==NULL ){
		pIdent = "new[]";
	}else{
		ghmGcMem::SetNewIdent(NULL,0xffffffff);
	}
	return ghmGcMem::GetNewHeap()->Alloc((int)blocksize,pIdent,line==0xffffffff? __LINE__ : line);
#else
	return ghmGcMem::GetNewHeap()->Alloc((int)blocksize);
#endif
}
 
void operator delete(void *pblock)
{
	ghmGcMem::Free(pblock);
}
 
void operator delete[](void *pblock)
{
	ghmGcMem::Free(pblock);
}

#ifdef	_DEBUG
void ghmGcMem::SetNewIdent(const char *pIdent,u32 line)
{
	mpNewIdent = pIdent;
	mNewLine   = line;
}

const char* ghmGcMem::GetNewIdent(void)
{
	return mpNewIdent;
}

u32 ghmGcMem::GetNewLine(void)
{
	return mNewLine;
}
#endif

void ghmGcMem::SetNewHeap(ghmGcMem *pMem)
{
	if( pMem==NULL ){
		mpDefaultAllocHeap = &mDefaultHeap;
	}else{
		mpDefaultAllocHeap = pMem;
	}
}

ghmGcMem* ghmGcMem::GetNewHeap(void)
{
	return mpDefaultAllocHeap;
}

ghmGcMem::ghmGcMem()
{
	mHandle      = -1;
	mAllocCount  = 0;
	mHeapSize    = 0;
	mpTopAddr    = 
	mpEndAddr    = NULL;
	mpNextHeap   = NULL;
	mpPrevHeap   = NULL;
	mpAllocTop   = NULL;
	mRenderColor = 0xff0000ff;
	mUseMutex    = false;
}

ghmGcMem::~ghmGcMem()
{
	DestroyHeap(true);
}

bool ghmGcMem::Initialize(bool manual)
{
	if( mInitialized==true ){
		return false;
	}
	
	mpArenaLo = OSGetArenaLo();
	mpArenaHi = OSGetArenaHi();
//#ifdef	GHMGCMEM_HEAP_24M
////	mpArenaHi = (void *)((u32)mpArenaHi - 0x1800000);
////	OSSetArenaHi(mpArenaHi);
//	mpArenaHi = OSAllocFromArenaHi(0x1800000,32);
//#endif
#ifdef	_DEBUG
	{
		u32		arenahi = (u32)mpArenaHi & 0x0fffffff;
		if( arenahi >= 0x1800000 ){
			mpArenaHi = OSAllocFromArenaHi(0x1800000,32);
		}
	}
#endif
	
	mpArenaLo = OSInitAlloc(mpArenaLo,mpArenaHi,mTotalHeapNum);
	OSSetArenaLo(mpArenaLo);
	mpArenaInitLo = mpArenaLo;
	mpArenaInitHi = mpArenaHi;
	mTotalArenaSize = (u32)mpArenaInitHi - (u32)mpArenaInitLo;
	
	if( manual==false ){
		OSReport("Warning !! : ----- ghmGcMem auto Initialize -----");
		if( mDefaultHeap.CreateHeap(GHMGCMEM_DEFAULT_HEAP_SIZE)==false ){
			ASSERTMSG1(0,
			           "ghmGcMem->Initialize : default heap not created 0x%08x",
			           (int)&mDefaultHeap);
			return false;
		}
	}
	
	mInitialized = true;
	
//	OSInitMutex(&mMutex);
//	OSInitSemaphore(&mSema,1);
	
	return true;
}

bool ghmGcMem::Lock(void)
{
//	if( ghmGcThread::GetThreadNum()==0 ){
//		return true;
//	}
	if( mUseMutex==false ){
		return true;
	}
	OSLockMutex(&mMutex);
//	OSWaitSemaphore(&mSema);
//	mLock = true;
	return true;
}

bool ghmGcMem::UnLock(void)
{
//	if( ghmGcThread::GetThreadNum()==0 ){
//		return true;
//	}
	if( mUseMutex==false ){
		return true;
	}
//	mLock = false;
	OSUnlockMutex(&mMutex);
//	OSSignalSemaphore(&mSema);
	return true;
}

void ghmGcMem::GetCurrentArena(void)
{
	mpArenaLo = OSGetArenaLo();
	mpArenaHi = OSGetArenaHi();
}

bool ghmGcMem::SetTotalHeapNum(int num)
{
	if( mInitialized==true ){
		return false;
	}
	if( num <= 0 ){
		return false;
	}
	mTotalHeapNum = num;
	return true;
}

bool ghmGcMem::ClearHeap(bool forced)
{
	while( mpHeapTop !=NULL ){
		if( mpHeapTop->DestroyHeap(forced)==false ){
			return false;
		}
	}
	return true;
}

bool ghmGcMem::CreateHeap(int size,bool useMutex)
{
	bool	link;
	
	if( (mUseMutex = useMutex)==true ){
		OSInitMutex(&mMutex);
	}
	
	if( mHandle!=GHMGCMEM_ASSIGNED_HEAP_HANDLE ){
		if( _AssignHeap(size)==false ){
#ifdef	_DEBUG
			OSPanic(__FILE__, __LINE__,"Assign Heap Failed : 0x%08x\n",size);
#endif
			return false;
		}
		link = false;
	}else{
		link = true;
	}
	
	Lock();
	
	OSSetArenaLo(mpTopAddr);
	OSSetArenaHi(mpEndAddr);
	
	if( (mHandle=OSCreateHeap(mpTopAddr,mpEndAddr)) < 0 ){
		UnLock();
#ifdef	_DEBUG
		OSPanic(__FILE__, __LINE__,"Create Heap Failed : 0x%08x\n",size);
#endif
		return false;
	}
	
	OSSetArenaLo(mpEndAddr);
	
	// q[v̈A
	if( link==false ){
		_LinkHeap();
	}
	
	mAllocCount = 0;
	
	long	freesize = GetFreeSize();
	if( freesize > 0 ){
		void	*pMem = Alloc(freesize);
		if( pMem!=NULL ){
//#ifdef	_DEBUG
#ifndef	K7APP_MASTER
			memset(pMem,'U',(u32)freesize);
#else
			memset(pMem,0,(u32)freesize);
#endif
			Free(pMem);
		}
	}
	
	UnLock();
	return true;
}

bool ghmGcMem::_AssignHeap(int size)
{
	if( mpArenaLo==NULL || mpArenaHi==NULL ){
		if( Initialize(false)==false ){
			return false;
		}
	}
	
	ghmGcMem	*pPrevHeap;
	
	if( mHandle >= 0 ){
		// ς
		return false;
	}
	
	Lock();
	
	if( size < 0 ){
		// őTCY擾
		size = 0;
		if( (pPrevHeap = mpHeapTop)!=NULL ){
			while( pPrevHeap != NULL ){
				int	freesize = pPrevHeap->GetNextFreeSize();
				if( size < freesize ){
					size = freesize;
				}
				pPrevHeap = pPrevHeap->mpNextHeap;
			}
		}else{
			size = (int)((u32)mpArenaHi - (u32)mpArenaLo);
		}
		
		if( size <= 0 ){
			UnLock();
			return false;
		}
		
	}else{
		// RQoCgACg␳
		size = (int)OSRoundUp32B(size);
	}
	
	u32			top;
	if( (pPrevHeap = mpHeapTop) != NULL ){
		while( pPrevHeap->mpNextHeap != NULL ){
			top = OSRoundUp32B(pPrevHeap->mpEndAddr);
			if( top + size < (u32)pPrevHeap->mpNextHeap->mpTopAddr ){
				break;
			}
			pPrevHeap = pPrevHeap->mpNextHeap;
		}
		if( pPrevHeap->mpNextHeap==NULL ){
			top = OSRoundUp32B(pPrevHeap->mpEndAddr);
			if( top + size > (u32)mpArenaHi ){
				return false;
			}
		}
	}else{
		top = (u32)mpArenaLo;
		if( top + size > (u32)mpArenaHi ){
			UnLock();
			return false;
		}
	}
	
	mpPrevHeap = pPrevHeap;
	mpTopAddr  = (void *) top;
	mpEndAddr  = (void *)(top + size);
	mHeapSize  = size;
	
	UnLock();
	return true;
}

void ghmGcMem::_LinkHeap(void)
{
	// q[v̈A
	if( mpPrevHeap != NULL ){
		mpNextHeap = mpPrevHeap->mpNextHeap;
	}else{
		mpNextHeap = NULL;
	}
	if( mpPrevHeap != NULL ){
		mpPrevHeap->mpNextHeap = this;
	}else{
		mpHeapTop = this;
	}
	if( mpNextHeap != NULL ){
		mpNextHeap->mpPrevHeap = this;
	}
}

bool ghmGcMem::AssignHeap(int size)
{
	if( _AssignHeap(size)==false ){
		return false;
	}
	mExternHandle = -1;
	mHandle       = GHMGCMEM_ASSIGNED_HEAP_HANDLE;
	_LinkHeap();
	return true;
}

bool ghmGcMem::ReportAssignHeapHandle(OSHeapHandle externHandle)
{
	if( mHandle!=GHMGCMEM_ASSIGNED_HEAP_HANDLE ){
		return false;
	}
	mExternHandle = externHandle;
	return true;
}

bool ghmGcMem::DestroyHeap(bool forced)
{
	if( mHandle < 0 ){
		// ς
		return true;
	}
	if( mAllocCount ){
		// 蓖ă
		if( forced==true ){
			if( FreeAll()==false ){
				return false;
			}
		}else{
			return false;
		}
	}
	// q[v̈
	Lock();
	OSDestroyHeap(mHandle);
	mHandle   = -1;
	mHeapSize = 0;
	mpTopAddr = NULL;
	mpEndAddr = NULL;
	// Ö̘Ä̐؂藣
	if( mpPrevHeap != NULL ){
		mpPrevHeap->mpNextHeap = mpNextHeap;
	}else{
		mpHeapTop = mpNextHeap;
	}
	if( mpNextHeap != NULL ){
		mpNextHeap->mpPrevHeap = mpPrevHeap;
	}
	mpNextHeap = NULL;
	mpPrevHeap = NULL;
	UnLock();
	return true;
}

#ifdef	_DEBUG
void* ghmGcMem::Alloc(int size,const char *pident,u32 line)
#else
void* ghmGcMem::Alloc(int size)
#endif
{
//	if ( size == (0xc0-0x20) ){
///		OSReport("Omaeda\n");
//	}
	Lock();
	if( mHandle < 0 ){
		// q[v
		if( mInitialized==false ){
			// 
			if( Initialize(false)==false ){
				UnLock();
				return NULL;
			}
		}else{
#ifdef	_DEBUG
			if( pident!=NULL ){
				ASSERTMSG3(mHandle>=0,
				           "ghmGcMem->Alloc : heap not created 0x%08x %s(0x%08x)",
				           (int)this,pident,line);
			}else{
				ASSERTMSG2(mHandle>=0,
				           "ghmGcMem->Alloc : heap not created 0x%08x line(0x%08x)",
				           (int)this,line);
			}
#endif
		}
	}
#ifdef	GHMGCMEM_DEFAULTHEAP_MESSAGE
//	if( this==&mDefaultHeap ){
//		if( mLock==false ){
//			OSReport("Alloc DefaultHeap\n");
//		}
//	}
#endif
	size += sizeof(ghmGcMemAllocAreaHeader);
//	if( this!=&mDefaultHeap ){
//		if( size==0x2020 || size==0x84 ){
//			OSReport("Alloc SmalHeap\n");
//		}
//	}
	ghmGcMemAllocArea	*pAllocMem = (ghmGcMemAllocArea *)OSAllocFromHeap(mHandle,OSRoundUp32B(size));
	if( pAllocMem==NULL ){
		long		rest = GetFreeSize();
		UnLock();
		OSDebugReport("Rest Memory = %d\n",rest);
		return NULL;
	}
	// JE^CNg
	mAllocCount++;
	// wb_ݒ
	pAllocMem->mHeader.mMagicNo  = GHMGCMEM_MAGINNO;
	pAllocMem->mHeader.mHandle   = mHandle;
	pAllocMem->mHeader.mSize     = (u32)size;
// Yam
#ifdef	_DEBUG
	pAllocMem->mHeader.mLine     = line;
#else
	pAllocMem->mHeader.mLine     = 0;
#endif
	pAllocMem->mHeader.mpNext    = mpAllocTop;
	pAllocMem->mHeader.mpPrev    = NULL;
	memset(pAllocMem->mHeader.mIdent,0,sizeof(pAllocMem->mHeader.mIdent));
#ifdef	_DEBUG
	if( pident!=NULL ){
		for(int i=0 ; i<8 && pident[i]!='\0' ; i++){
			pAllocMem->mHeader.mIdent[i] = pident[i];
		}
	}else{
		for(int i=0 ; i<8 ; i++){
			pAllocMem->mHeader.mIdent[i] = '-';
		}
	}
#else
	memset(pAllocMem->mHeader.mIdent,0,sizeof(pAllocMem->mHeader.mIdent));
#endif
	if( mpAllocTop!=NULL ){
		mpAllocTop->mpPrev = &pAllocMem->mHeader;
	}
	mpAllocTop                   = &pAllocMem->mHeader;
	UnLock();
	return (void *)pAllocMem->mData;
}

bool ghmGcMem::Free(void *pPtr)
{
#ifdef	_DEBUG
	if( pPtr==NULL ){
		OSReport("ghmGcMem->Free : NULL ptr\n");
		return false;
	}
#endif
	
	ghmGcMemAllocAreaHeader	*pAllocMem = (ghmGcMemAllocAreaHeader *)((u32)pPtr - sizeof(ghmGcMemAllocAreaHeader));
	
	// 蓖ăO
	ASSERTMSG3(pAllocMem->mMagicNo==GHMGCMEM_MAGINNO,"ghmGcMem->Free : not allocated memory Handle %x Line(0x%08x) Addr 0x%08x",
	           pAllocMem->mHandle,pAllocMem->mLine,(u32)pPtr);
	
	ghmGcMem		*pHeap = Find(pAllocMem->mHandle);
	
#ifdef	GHMGCMEM_DEFAULTHEAP_MESSAGE
//	if( pHeap==&mDefaultHeap ){
//		if( mLock==false ){
//			OSReport("Free DefaultHeap\n");
//		}
//	}
#endif
	
	pHeap->Lock();
	
//#ifndef	K7APP_MASTER
//#ifdef	_DEBUG
	u32		size = pAllocMem->mSize;
//#endif
	
	// wb_NA
	pAllocMem->mMagicNo  = 0;
	pAllocMem->mHandle   = -1;
	pAllocMem->mSize     = 0;
	pAllocMem->mLine     = 0;
	if( pAllocMem->mpNext!=NULL ){
		pAllocMem->mpNext->mpPrev = pAllocMem->mpPrev;
	}
	if( pAllocMem->mpPrev!=NULL ){
		pAllocMem->mpPrev->mpNext = pAllocMem->mpNext;
	}else{
		pHeap->mpAllocTop = pAllocMem->mpNext;
	}
	// JE^fNg
	pHeap->mAllocCount--;
	
#ifndef	K7APP_MASTER
//#ifdef	_DEBUG
	// U=0x55=01010101b
	memset(pAllocMem,'U',size);
#else
	memset(pAllocMem,0,size);
#endif
	
	// 
	OSFreeToHeap(pHeap->mHandle,pAllocMem);
	
	pHeap->UnLock();
	
	return true;
}

bool ghmGcMem::FreeAll(void)
{
	while( mpAllocTop!=NULL ){
		if( Free(mpAllocTop)==false ){
			return false;
		}
	}
	return true;
}

ghmGcMem* ghmGcMem::Find(OSHeapHandle handle)
{
	ghmGcMem	*pHeap = mpHeapTop;
	while( pHeap!=NULL ){
		if( pHeap->mHandle==handle ){
			return pHeap;
		}
		pHeap = pHeap->mpNextHeap;
	}
	return NULL;
}

long ghmGcMem::GetFreeSize(void)
{
	if( mHandle < 0 ){
		return -1;
	}
	Lock();
	long	size = OSCheckHeap(mHandle);
	UnLock();
	if( size < sizeof(ghmGcMemAllocAreaHeader) ){
		return 0;
	}
	return size - (long)sizeof(ghmGcMemAllocAreaHeader);
}

int ghmGcMem::GetNextFreeSize(void)
{
	u32		size;
	if( mHandle < 0 ){
		return 0;
	}
	Lock();
	if( mpNextHeap != NULL ){
		size = (u32)mpNextHeap->mpTopAddr;
	}else{
		size = OSRoundDown32B((u32)mpArenaHi);
	}
	size -= OSRoundUp32B((u32)mpEndAddr);
	UnLock();
	return (int)size;
}

bool ghmGcMem::SetRenderPosition(u16 x,u16 y,u16 w,u16 h)
{
	mRenderPosX   = x;
	mRenderPosY   = y;
	mRenderWidth  = w;
	mRenderHeight = h;
	return true;
}

bool ghmGcMem::Render(void)
{
	if( mHandle < 0 && mHandle!=GHMGCMEM_ASSIGNED_HEAP_HANDLE ){
		return false;
	}
	
	Lock();
	
	u16						x0,x1,y0,y1;
	u32						total   = mTotalArenaSize >> 5;
	u32						size    = ((u32)ghmGcMem::mpArenaInitHi - (u32)ghmGcMem::mpArenaInitLo) >> 5;
	u32						top     = GHMGCMEM_GET_PHYSICAL_ADDR(ghmGcMem::mpArenaInitLo)>>5;
	ghmGcMemAllocAreaHeader	*pAlloc = mpAllocTop;
	u16						vertnum;
	
	GXSetChanMatColor(GX_COLOR0A0,*((GXColor *)&mRenderColor));
	
	size = OSRoundUp32B(size);
	
	x0 = (u16)(mRenderPosX + ((GHMGCMEM_GET_PHYSICAL_ADDR(mpTopAddr)>>5) - top) * mRenderWidth / total);
	x1 = (u16)(mRenderPosX + ((GHMGCMEM_GET_PHYSICAL_ADDR(mpEndAddr)>>5) - top) * mRenderWidth / total);
	y0 = (u16)(mRenderPosY +           mRenderHeightUse                                               );
	y1 = (u16)(mRenderPosY + mRenderHeight                                                  );
	
	if( mHandle!=GHMGCMEM_ASSIGNED_HEAP_HANDLE ){
		
		vertnum = 4;
		if( mAllocCount > 0 ){
			vertnum += mAllocCount * 4;
		}
		
		GXBegin(GX_QUADS,GX_VTXFMT0,vertnum);
			
			GXPosition2u16(x0,y0);
			GXPosition2u16(x1,y0);
			GXPosition2u16(x1,y1);
			GXPosition2u16(x0,y1);
			
			y1 = y0;
			y0 = ghmGcMem::mRenderPosY;
			
			if( vertnum > 4 ){
				
				while( pAlloc != NULL ){
					
					x0 = (u16)(mRenderPosX + (((GHMGCMEM_GET_PHYSICAL_ADDR(pAlloc)                              )>>5) - top) * mRenderWidth / total);
					x1 = (u16)(mRenderPosX + (((GHMGCMEM_GET_PHYSICAL_ADDR(pAlloc) + OSRoundUp32B(pAlloc->mSize))>>5) - top) * mRenderWidth / total);
					
static	u16			minWidth = 3;
					if( x1 < (u16)(x0 + minWidth) ){
						x1 = (u16)(x0 + minWidth);
					}
					
					GXPosition2u16(x0,y0);
					GXPosition2u16(x1,y0);
					GXPosition2u16(x1,y1);
					GXPosition2u16(x0,y1);
					
					pAlloc = pAlloc->mpNext;
					
				}
				
			}
			
		GXEnd();
		
	}else{
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4);
			
			GXPosition2u16(x0,y0);
			GXPosition2u16(x1,y0);
			GXPosition2u16(x1,y1);
			GXPosition2u16(x0,y1);
			
		GXEnd();
		
		if( mExternHandle >= 0 ){
			
			long	freesize = OSCheckHeap(mExternHandle);
			
			y1 = y0;
			y0 = ghmGcMem::mRenderPosY;
			
			if( freesize >= 0 ){
			
				x1 = (u16)(mRenderPosX + (((GHMGCMEM_GET_PHYSICAL_ADDR(mpEndAddr) - (u32)freesize)>>5) - top) * mRenderWidth / total);
				
			}else{
				
				GXColor	errorColor = { 0xff, 0xff, 0x00, 0xff };
				
				GXSetChanMatColor(GX_COLOR0A0,errorColor);
				
			}
			
			GXBegin(GX_QUADS,GX_VTXFMT0,4);
				
				GXPosition2u16(x0,y0);
				GXPosition2u16(x1,y0);
				GXPosition2u16(x1,y1);
				GXPosition2u16(x0,y1);
				
			GXEnd();
			
		}
		
	}
	
	UnLock();
	
	return true;
}

bool ghmGcMem::SetupRenderState(void)
{
	Mtx			view;
	Mtx44		proj;
	Vec			vPos;
	Vec			vUp;
	Vec			vAt;
	
	vUp .x =    0.0f;
	vUp .y = -  1.0f;
	vUp .z =    0.0f;
	vPos.x =    0.0f;
	vPos.y =    0.0f;
	vPos.z =    0.0f;
	vAt .x =  vPos.x;
	vAt .y =  vPos.y;
	vAt .z =  300.0f;
	
	// [Jr[ϊ}gNXݒ
	MTXLookAt        (view,&vPos,&vUp,&vAt);
	MTXOrtho         (proj,0,-GHM_SCREEN_HEIGHT,0,GHM_SCREEN_WIDTH,0.0f,1024.0f);
	
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	GXSetScissor     (0,0,GHM_SCREEN_WIDTH,GHM_SCREEN_HEIGHT);
	GXSetViewport    (0.0f,0.0f,GHM_SCREEN_WIDTH,GHM_SCREEN_HEIGHT,0.0f,1.0f);
	
	// _OXe[gݒ
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc     (GX_VA_NRM ,GX_NONE  );
	GXSetVtxDesc     (GX_VA_CLR0,GX_NONE  );
	GXSetVtxDesc     (GX_VA_TEX0,GX_NONE  );
	
	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetVtxAttrFmt(GX_VTXFMT0,GX_VA_POS,GX_POS_XY,GX_U16,0);
	
	return true;
}

#define	GHMGCMEM_WIN_MARGIN			(4)
bool ghmGcMem::RenderAll(void)
{
#if !defined(K7APP_MASTER_NINTENDO) && !defined(_DEBUG)
	if( HSD_PadGetTrigger(1) & HSD_PAD_A ){
		mRender = (mRender==true? false : true);
	}
#endif
	if( mRender==false ){
		return true;
	}
	
	ghmGcMem	*pHeap = mpHeapTop;
	u16			x0,y0,x1,y1;
	GXColor		wincol = { 0x00,0x00,0x00,0x80 };
	
	// `Ԑݒ
	SetupRenderState();
	
	GXSetChanMatColor(GX_COLOR0A0,wincol);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
		
		x0 = (u16)(ghmGcMem::mRenderPosX - GHMGCMEM_WIN_MARGIN);
		x1 = (u16)(ghmGcMem::mRenderPosX + ghmGcMem::mRenderWidth + GHMGCMEM_WIN_MARGIN);
		y0 = (u16)(ghmGcMem::mRenderPosY - GHMGCMEM_WIN_MARGIN);
		y1 = (u16)(ghmGcMem::mRenderPosY + ghmGcMem::mRenderHeight + GHMGCMEM_WIN_MARGIN);
		
		GXPosition2u16(x0,y0);
		GXPosition2u16(x1,y0);
		GXPosition2u16(x1,y1);
		GXPosition2u16(x0,y1);
		
	GXEnd();
	
	// q[v\
	while( pHeap != NULL ){
		pHeap->Render();
		pHeap = pHeap->mpNextHeap;
	}
	
	return true;
}

asm u32 GetReturnAddress(int up)
{
#pragma unused(up)
	nofralloc
	
	lwz		r4,0(rsp)
loop:
	or.		r3,r3,r3
	bne		step
	lwz		r3,4(r4)
	blr
step:
	lwz		r4,0(r4)
	addi	r0,r4,1
	or.		r0,r0,r0
	beq		bottom
	addi	r3,r3,-1
	b		loop
bottom:
	xor		r3,r3,r3
	blr
}

void* ghmGcMem::Check(void *pPtr)
{
	ghmGcMemAllocArea	*pAllocMem;
#ifdef	_DEBUG
	int					count = 0;
#endif
	Lock();
	if( pPtr==NULL ){
		pAllocMem = (ghmGcMemAllocArea *)mpAllocTop;
	}else{
		pAllocMem = (ghmGcMemAllocArea *)((u32)pPtr - sizeof(ghmGcMemAllocAreaHeader));
		pAllocMem = (ghmGcMemAllocArea *)pAllocMem->mHeader.mpNext;
	}
	while( pAllocMem!=NULL ){
		if( pAllocMem->mHeader.mMagicNo!=GHMGCMEM_MAGINNO ){
#ifdef	_DEBUG
			OSReport("%08x : Broken Memory !\n",(u32)pAllocMem);
#endif
			UnLock();
			return pAllocMem->mData;
		}
		pAllocMem = (ghmGcMemAllocArea *)pAllocMem->mHeader.mpNext;
#ifdef	_DEBUG
		count++;
#endif
	}
	if( pPtr==NULL ){
#ifdef	_DEBUG
		if( count!=mAllocCount ){
			OSReport("Alloc Count Error : %08x\n",(u32)this);
		}
#endif
	}
	UnLock();
	return NULL;
}

bool ghmGcMem::OSDump(void)
{
	if( mHandle < 0 ){
		return false;
	}
	OSDumpHeap(mHandle);
	return true;
}

int ghmGcMem::Dump(void)
{
	Lock();
	ghmGcMemAllocArea	*pAllocMem;
	int					count = 0;
	pAllocMem = (ghmGcMemAllocArea *)mpAllocTop;
	while( pAllocMem!=NULL ){
		if( pAllocMem->mHeader.mMagicNo!=GHMGCMEM_MAGINNO ){
			OSReport("%08x : Broken Memory !\n",(u32)pAllocMem->mData);
		}else{
			char	str[16];
			u32		len;
			len = strlen(pAllocMem->mHeader.mIdent);
			if( len > sizeof(str) ){
				len = sizeof(str);
			}
			if( len > sizeof(pAllocMem->mHeader.mIdent) ){
				len = sizeof(pAllocMem->mHeader.mIdent);
			}
			memset (str,0,sizeof(str));
			strncpy(str,pAllocMem->mHeader.mIdent,len);
			OSReport("%08x : %08x %s(%d)\n",(u32)pAllocMem->mData,pAllocMem->mHeader.mSize,str,pAllocMem->mHeader.mLine);
		}
		pAllocMem = (ghmGcMemAllocArea *)pAllocMem->mHeader.mpNext;
		count++;
	}
	UnLock();
	return count;
}

char	ghmGcMem::mIdentTemp[GHMGCMEM_IDENTTEMP_SIZE];
const char* ghmGcMem::GetIdent(void *pPtr)
{
	ghmGcMemAllocAreaHeader		*pAllocMem = (ghmGcMemAllocAreaHeader *)((u32)pPtr - sizeof(ghmGcMemAllocAreaHeader));
	if( pAllocMem->mMagicNo!=GHMGCMEM_MAGINNO ){
		return NULL;
	}
	strncpy(ghmGcMem::mIdentTemp,pAllocMem->mIdent,GHMGCMEM_IDENTTEMP_SIZE);
	return ghmGcMem::mIdentTemp;
}

