////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Memory Manager
//
// ghmGcMem.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/07/2002
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCMEM_H__
#define	__GHMGCMEM_H__

#include <dolphin.h>
#include <dolphin\os.h>

#define	GHMGCMEM_HEAPSIZE_MAX				(-1)

#include "ghmGcLib.ini"

#ifndef	GHMGCMEM_DEFAULT_TOTAL_HEAP_NUM
#define	GHMGCMEM_DEFAULT_TOTAL_HEAP_NUM		(1)
#endif

#define	GHMGCMEM_GET_PHYSICAL_ADDR(addr)	(((u32)(addr)) & 0x0fffffff)

//                                             G H M 0
#define	GHMGCMEM_MAGINNO					(0x47484D30)

#define	GHMGCMEM_NON_ASSIGNED_HEAP_HANDLE	(-1)
#define	GHMGCMEM_ASSIGNED_HEAP_HANDLE		(-2)

#define	GHMGCMEM_SAFE_FREE(p)				\
					if( (p) != NULL ){ \
						ghmGcMem::Free((void *)(p)); \
						(p)  = NULL; \
					}

#define	GHMGCMEM_IDENTTEMP_SIZE		(16)

struct ghmGcMemAllocAreaHeader{
	union{
		char				mMagicNoChar[4];
		u32					mMagicNo;
	};
	OSHeapHandle			mHandle;
	u32						mSize;
	u32						mLine;
	
	ghmGcMemAllocAreaHeader	*mpNext;
	ghmGcMemAllocAreaHeader	*mpPrev;
	char					mIdent[8];
}ATTRIBUTE_ALIGN(32);

struct ghmGcMemAllocArea{
	ghmGcMemAllocAreaHeader	mHeader;
	u32						mData[1];
}ATTRIBUTE_ALIGN(32);

class ghmGcMem{
protected:
	static bool				mInitialized;
	static int				mTotalHeapNum;
	static void				*mpArenaInitLo;
	static void				*mpArenaInitHi;
	static u32				mTotalArenaSize;
	static void				*mpArenaLo;
	static void				*mpArenaHi;
	static ghmGcMem			*mpHeapTop;
	static u16				mRenderPosX;
	static u16				mRenderPosY;
	static u16				mRenderWidth;
	static u16				mRenderHeight;
	static u16				mRenderHeightUse;
	static bool				mRender;
	static ghmGcMem			*mpDefaultAllocHeap;
	static char				mIdentTemp[GHMGCMEM_IDENTTEMP_SIZE];
#ifdef	_DEBUG
	static const char*		mpNewIdent;
	static u32				mNewLine;
	static u16				mMinWidth;
#endif
//	static OSMutex			mMutex;
//	static OSSemaphore		mSema;
//	static bool				mLock;
	
	OSHeapHandle			mHandle;
	int						mHeapSize;
	void					*mpTopAddr;
	void					*mpEndAddr;
	
	union{
		int					mAllocCount;
		OSHeapHandle		mExternHandle;
	};
	ghmGcMem				*mpNextHeap;
	ghmGcMem				*mpPrevHeap;
	ghmGcMemAllocAreaHeader	*mpAllocTop;
	
	u32						mRenderColor;
	OSMutex					mMutex;
	bool					mUseMutex;
	
	int GetNextFreeSize(void);
	bool _AssignHeap(int size);
	void _LinkHeap(void);
	
	bool Lock(void);
	bool UnLock(void);
//	static bool Lock(void);
//	static bool UnLock(void);
public:
	
	static	ghmGcMem		mDefaultHeap;
	
	ghmGcMem();
	~ghmGcMem();
	
	static void* GetArenaLo(void){ return mpArenaLo; }
	static void* GetArenaHi(void){ return mpArenaHi; }
	
	// 
	static bool Initialize(bool manual=true);
	static void GetCurrentArena(void);
	// q[vݒ
	static bool SetTotalHeapNum(int num);
	inline static int GetTotalHeapNum(void){ return mTotalHeapNum; }
	// q[viSq[vj
	static bool ClearHeap(bool forced=false);
	
	// Sq[vgp󋵕\
	static bool RenderAll(void);
	// q[vgp󋵕\
	bool Render(void);
	// `Ԑݒ
	static bool SetupRenderState(void);
	// `Fݒ
	inline void SetRenderColor(u32 col){ mRenderColor=col; }
	inline u32 GetRenderColor(void){ return mRenderColor; }
	// `nm^nee
	inline static void SetRender(bool flag){ mRender=flag; }
	inline static bool GetRender(void){ return mRender; }
	
	// q[v
	bool CreateHeap(int size=GHMGCMEM_HEAPSIZE_MAX,bool useMutex=true);
	// q[v
	bool DestroyHeap(bool forced=false);
	// q[v蓖
	bool AssignHeap(int size=GHMGCMEM_HEAPSIZE_MAX);
	bool ReportAssignHeapHandle(OSHeapHandle externHandle);
	
	// 蓖
#ifdef	_DEBUG
	void* Alloc(int size,const char *pident=NULL,u32 line=0);
#else
	void* Alloc(int size);
#endif
	// 
	static bool Free(void *pPtr);
	// 蓖ăS
	bool FreeAll(void);
	
	// t[TCY擾
	long GetFreeSize(void);
	
	// q[v擪擾
	inline void* GetTopAddr(void){ return mpTopAddr; }
	// q[vI[擾
	inline void* GetEndAddr(void){ return mpEndAddr; }
	
	static bool SetRenderPosition(u16 x,u16 y,u16 w,u16 h);
	
	void* Check(void *pPtr=NULL);
	static const char* GetIdent(void *pPtr);
	
	static void SetNewHeap(ghmGcMem *pMem);
	static ghmGcMem* GetNewHeap(void);
	
	bool OSDump(void);
	int Dump(void);
	static ghmGcMem* Find(OSHeapHandle handle);
		
#ifdef	_DEBUG
	static void SetNewIdent(const char *pIdent,u32 line);
	static const char* GetNewIdent(void);
	static u32 GetNewLine(void);
	static bool SetMinWidth(u16 width){ mMinWidth = width; return true; }
	static u32 GetMinWidth(void){ return mMinWidth; }
#endif
};

//#define	ghmMemAlloc(size)			ghmGcMem::mDefaultHeap.Alloc((size))
//#define	ghmMemFree(ptr)				ghmGcMem::mDefaultHeap.Free((ptr))
//#define	ghmMemGetFreeSize()			ghmGcMem::mDefaultHeap.GetFreeSize()
#ifdef	_DEBUG
#define	ghmGcMemAlloc(size)				ghmGcMem::mDefaultHeap.Alloc((size),__FILE__,__LINE__)
#define	ghmGcMemAllocName(size,name)	ghmGcMem::mDefaultHeap.Alloc((size),(name),__LINE__)
#define	ghmGcMemSetAllocName(name)		ghmGcMem::SetNewIdent((name),__LINE__)
#define	ghmGcMemGetAllocName()			ghmGcMem::GetNewIdent()
#define	ghmGcMemGetNewLine()			ghmGcMem::GetNewLine()
#else
#define	ghmGcMemAlloc(size)				ghmGcMem::mDefaultHeap.Alloc((size))
#define	ghmGcMemAllocName(size,name)	ghmGcMem::mDefaultHeap.Alloc((size))
#define	ghmGcMemSetAllocName(name)		
#define	ghmGcMemGetAllocName()			NULL
#define	ghmGcMemGetNewLine()			0
#endif
#define	ghmGcMemFree(ptr)				ghmGcMem::Free((ptr))
#define	ghmGcMemGetFreeSize()			ghmGcMem::mDefaultHeap.GetFreeSize()

extern	asm u32 GetReturnAddress(int up);

extern void* operator new(u32 blocksize);
extern void* operator new[](u32 blocksize);
extern void operator delete(void *pblock);
extern void operator delete[](void *pblock);

#endif	//__GHMGCMEM_H__

