////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Module
//
// ghmGcModule.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 04/17/2004
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCMODULE_H__
#define	__GHMGCMODULE_H__

#include ".\ghmGcList.h"
#include ".\ghmGcFile.h"

class ghmGcModule : public ghmGcListObj{
public:
	enum State{
		IDLE = 0,
		INIT,
		LOAD,
		LOADING,
		ALLOCBSS,
		LOADED,
		AVAILABLE = LOADED,
		RELEASE,
	};
protected:
	static ghmGcListObj		mAnchor;
#ifdef	_DEBUG
	static ghmGcFile		*mpDebugFile;
	static int				mDebugProc;
	static void				*mpStringTable;
#endif
	ghmGcFile				*mpFile;
	State					mState;
	char					mFileName[64];		// ܂t@C𒷂ȂƁI
	int						mRefCount;
	OSModuleHeader			*mpModuleHeader;
	u8						*mpBss;
	int						mWorkSize;
	
	void IncRefCount(void);
	void DecRefCount(void);
	bool Release(void);
	ghmGcModule(const char *pFileName,void *pLoadAddr,int size);
	~ghmGcModule();
public:
	
	inline ghmGcModule* GetNext(void){ return (ghmGcModule *)ghmGcListObj::GetNext(); }
	inline ghmGcModule* GetPrev(void){ return (ghmGcModule *)ghmGcListObj::GetPrev(); }
	inline const char* GetFileName(void){ return mFileName; }
	inline int GetRefCount(void){ return mRefCount; }
	inline ghmGcModule* GetTopModule(void){ return (ghmGcModule *)mAnchor.GetNext(); }
	inline State GetState(void){ return mState; }
	
	static ghmGcModule* Create(const char *pModuleName,void *pLoadAddr=NULL,int size=-1);
	static bool Release(ghmGcModule *pModule);
	static bool ReleaseAll(void);
	static ghmGcModule* Find(const char *pModuleName);
	static bool IsValid(ghmGcModule *pModule);
	static bool FrameProcessAll(void);
	static bool RenderProcessAll(void);
	static void* InitializeAll(const char *pStringTblFileName=NULL);
	static bool IsInitialized(void);
	static bool TerminateAll(void);
	
	virtual bool FrameProcess(void);
	virtual bool RenderProcess(void);
};

#endif	//__GHMGCMODULE_H__

