////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Nurbs
//
// ghmGcNurbs.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 12/12/2002
//
////////////////////////////////////////////////////////////////////////////////

#define	__GHMGCNURBS_CPP__

#include <dolphin.h>

#include "ghmGcNurbs.h"
#include "ghmGcDefine.h"

//#define	GHMGCNURBS_PATH_BASISDIST			(100.0f)
//#define	GHMGCNURBS_CALCDIST_PATHDIVMAXNUM	(128)

ghmGcNurbs::ghmGcNurbs()
{
	mpDesc = NULL;
//	mPathDivNum = GHMGCNURBS_CALCDIST_PATHDIVNUM;
}

ghmGcNurbs::~ghmGcNurbs()
{
}

bool ghmGcNurbs::SetDesc(ghmNurbsPathDesc *pDesc)
{
	if( (mpDesc = pDesc)==NULL ){
		return true;
	}
//	mPathDivNum = (int)(GHMGCNURBS_CALCDIST_PATHDIVNUM * mpDesc->mTotalDistance / GHMGCNURBS_PATH_BASISDIST);
//	if( mPathDivNum < GHMGCNURBS_CALCDIST_PATHDIVNUM ){
//		mPathDivNum = GHMGCNURBS_CALCDIST_PATHDIVNUM;
//	}else if( mPathDivNum > GHMGCNURBS_CALCDIST_PATHDIVMAXNUM ){
//		mPathDivNum = GHMGCNURBS_CALCDIST_PATHDIVMAXNUM;
//	}
	mpDesc->mTotalDistance = CalcDistance(mpDesc->mEndParam);
	return true;
}

int ghmGcNurbs::GetDegree(void)
{
	if( mpDesc==NULL ){
		return -1;
	}
	return mpDesc->mDegree;
}

int ghmGcNurbs::GetSpanCount(void)
{
	if( mpDesc==NULL ){
		return -1;
	}
	return mpDesc->mSpanCount;
}

int ghmGcNurbs::GetCVCount(void)
{
	if( mpDesc==NULL ){
		return -1;
	}
	return mpDesc->mCVCount;
}

int ghmGcNurbs::GetKnotCount(void)
{
	if( mpDesc==NULL ){
		return -1;
	}
	return mpDesc->mKnotCount;
}

f32 ghmGcNurbs::GetStartParam(void)
{
	if( mpDesc==NULL ){
		return 0.0f;
	}
	return mpDesc->mStartParam;
}

f32 ghmGcNurbs::GetEndParam(void)
{
	if( mpDesc==NULL ){
		return 0.0f;
	}
	return mpDesc->mEndParam;
}

f32 ghmGcNurbs::GetTotalDistance(void)
{
	if( mpDesc==NULL ){
		return 0.0f;
	}
	return mpDesc->mTotalDistance;
}

bool ghmGcNurbs::GetCV(int index,Vec4 &cv)
{
	if( mpDesc==NULL ){
		return false;
	}
	if( index < 0 || index >= mpDesc->mCVCount ){
		return false;
	}
	cv = mpDesc->mpCV[index];
	return true;
}

float ghmGcNurbs::GetKnot(int index)
{
	if( mpDesc==NULL ){
		return 0.0f;
	}
	if( index < 0 || index >= mpDesc->mKnotCount ){
		return 0.0f;
	}
	return mpDesc->mpKnot[index];
}

bool ghmGcNurbs::GetPoint(Vec4 &point,f32 param)
{
	if( GetPoint((Vec&)point,param)==false ){
		return false;
	}
	point.w = 1.0f;
	return true;
}

bool ghmGcNurbs::GetPoint(Vec &point,f32 param)
{
	if( mpDesc==NULL ){
		return false;
	}
	Vec4		*pCV = mpDesc->mpCV;
	if( param < mpDesc->mStartParam ){
		point.x = pCV->x;
		point.y = pCV->y;
		point.z = pCV->z;
		return true;
	}else if( param >= mpDesc->mEndParam ){
		pCV += mpDesc->mCVCount - 1;
		point.x = pCV->x;
		point.y = pCV->y;
		point.z = pCV->z;
		return true;
	}
	int			i;
	float		coeff;
	float		weight = 0.0f;
	point.x = 0.0f;
	point.y = 0.0f;
	point.z = 0.0f;
	for(i=0 ; i<mpDesc->mCVCount ; i++,pCV++){
//		coeff = BSplineBasisFunc(param,i) * pCV->w;
		coeff = BSplineBasisFunc(param,i);
		weight  += coeff;
		point.x += coeff * pCV->x;
		point.y += coeff * pCV->y;
		point.z += coeff * pCV->z;
	}
//	if( weight==0.0f ){
//		return false;
//	}
//	weight   = 1.0f / weight;
//	point.x *= weight;
//	point.y *= weight;
//	point.z *= weight;
	return true;
}

float ghmGcNurbs::BSplineBasisFunc(f32 param,int cvid)
{
	if( mpDesc==NULL || mpDesc->mDegree < 0 || mpDesc->mCVCount <= 0 ){
		return 0.0f;
	}
	return BSplineBasisFunc(param,mpDesc->mDegree,cvid,mpDesc->mpKnot);
}

float ghmGcNurbs::BSplineBasisFunc(f32 param,int degree,int cvid,f32 *pKnotTop)
{
	f32		*pKnot;
	pKnot = pKnotTop + cvid;
#ifdef _DEBUG
	if( degree < 0 ){
		OSReport("B-Spline Basis Func : degree < 0\n");
		return 0.0f;
	}else
#endif
	if( degree==0 ){
//		if( param < pKnot[0] || param > pKnot[1] ){
		if( param < pKnot[0] || param >= pKnot[1] ){
			return 0.0f;
		}else{
			return 1.0f;
		}
	}
	// ċA
	float	coeff;
	float	result = 0.0f;
	coeff = pKnot[degree] - pKnot[0];
	if( coeff > FLT_EPSILON || coeff < -FLT_EPSILON ){
		coeff = (param - pKnot[0]) / coeff;
		if( coeff > FLT_EPSILON || coeff < -FLT_EPSILON ){
			result += BSplineBasisFunc(param,degree-1,cvid,pKnotTop) * coeff;
		}
	}
	coeff = pKnot[degree+1] - pKnot[1];
	if( coeff > FLT_EPSILON || coeff < -FLT_EPSILON ){
		coeff = (pKnot[degree+1] - param) / coeff;
		if( coeff > FLT_EPSILON || coeff < -FLT_EPSILON ){
			result += BSplineBasisFunc(param,degree-1,cvid+1,pKnotTop) * coeff;
		}
	}
	return result;
}

float ghmGcNurbs::BSplineBasisDiffFunc(int rank,f32 param,int cvid)
{
	if( mpDesc==NULL || mpDesc->mDegree < 0 || mpDesc->mCVCount <= 0 ){
		return 0.0f;
	}
	return BSplineBasisDiffFunc(rank,param,mpDesc->mDegree,cvid,mpDesc->mpKnot);
}

float ghmGcNurbs::BSplineBasisDiffFunc(int rank,f32 param,int degree,int cvid,f32 *pKnotTop)
{
	f32		*pKnot;
	pKnot = pKnotTop + cvid;
#ifdef _DEBUG
	if( rank <= 0 ){
		OSReport("B-Spline Basis Diff Func : rank <= 0\n");
		return 0.0f;
	}
#endif
	float	coeff0,coeff1;
	float	result = 0.0f;
	
	coeff0 = pKnot[degree] - pKnot[0];
	if( coeff0 > FLT_EPSILON || coeff0 < -FLT_EPSILON ){
		coeff0 = degree / coeff0;
	}else{
		coeff0 = 0.0f;
	}
	
	coeff1 = pKnot[degree+1] - pKnot[1];
	if( coeff1 > FLT_EPSILON || coeff1 < -FLT_EPSILON ){
		coeff1 = degree / coeff1;
	}else{
		coeff1 = 0.0f;
	}
	
	// ċA
	degree--;
	if( rank==1 ){
		if( coeff0 != 0.0f ){
			result += BSplineBasisFunc(param,degree,cvid  ,pKnotTop) * coeff0;
		}
		if( coeff1 != 0.0f ){
			result -= BSplineBasisFunc(param,degree,cvid+1,pKnotTop) * coeff1;
		}
	}else{
		rank--;
		if( coeff0 != 0.0f ){
			result += BSplineBasisDiffFunc(rank,param,degree,cvid  ,pKnotTop) * coeff0;
		}
		if( coeff1 != 0.0f ){
			result -= BSplineBasisDiffFunc(rank,param,degree,cvid+1,pKnotTop) * coeff1;
		}
	}
	return result;
}

bool ghmGcNurbs::GetDiffVec(Vec4 &vec,f32 param,int rank)
{
	if( GetDiffVec((Vec&)vec,param,rank)==false ){
		return false;
	}
	vec.w = 1.0f;
	return true;
}

bool ghmGcNurbs::GetDiffVec(Vec &vec,f32 param,int rank)
{
	if( mpDesc==NULL ){
		return false;
	}
	int			i;
	float		coeff;
	float		weight = 0.0f;
	Vec4		*pCV = mpDesc->mpCV;
	if( param >= mpDesc->mEndParam ){
		param  = mpDesc->mEndParam - 0.00001f;
	}
	vec.x = 0.0f;
	vec.y = 0.0f;
	vec.z = 0.0f;
	for(i=0 ; i<mpDesc->mCVCount ; i++,pCV++){
		coeff   = BSplineBasisDiffFunc(rank,param,i);
//		coeff   = BSplineBasisDiffFunc(rank,param,i) * pCV->w;
		weight += coeff;
		vec.x  += coeff * pCV->x;
		vec.y  += coeff * pCV->y;
		vec.z  += coeff * pCV->z;
	}
//	if( weight==0.0f ){
//		return false;
//	}
//	weight  = 1.0f / weight;
//	vec.x  *= weight;
//	vec.y  *= weight;
//	vec.z  *= weight;
	return true;
}

f32 ghmGcNurbs::CalcDistance(f32 param)
{
	if( mpDesc==NULL ){
		return -1.0f;
	}
	
	Vec		cur,pre;
	f32		dist = 0.0f;
	
	int		i;
	f32		sp = mpDesc->mStartParam;
	f32		ep = mpDesc->mEndParam;
//	f32		dp = (ep - sp) / mPathDivNum;
	f32		dp = (ep - sp) / GHMGCNURBS_CALCDIST_PATHDIVNUM;
	f32		p;
	
	GetPoint(pre,sp);
//	for(i=1 ; i<=mPathDivNum ; i++){
	for(i=1 ; i<=GHMGCNURBS_CALCDIST_PATHDIVNUM ; i++){
		p = sp + (f32)i * dp;
		if( param < p ){
//			GetPoint(cur,param);
//			dist += GHM_VEC_GET_DIST(&cur,&pre);
			GetPoint(cur,p);
			dist += GHM_VEC_GET_DIST(&cur,&pre) * (param + dp - p) / dp;
			return dist;
		}
		GetPoint(cur,p);
		dist += GHM_VEC_GET_DIST(&cur,&pre);
		pre = cur;
	}
	
	return dist;
}

f32 ghmGcNurbs::CalcParam(f32 dist)
{
	if( mpDesc==NULL ){
		return -1.0f;
	}
	
	if( dist < 0.0f ){
		return mpDesc->mStartParam;
	}
	if( dist > mpDesc->mTotalDistance ){
		return mpDesc->mEndParam;
	}
	
	Vec		cur,pre;
	f32		tdist = 0.0f;
	
	int		i;
	f32		sp = mpDesc->mStartParam;
	f32		ep = mpDesc->mEndParam;
//	f32		dp = (ep - sp) / mPathDivNum;
	f32		dp = (ep - sp) / GHMGCNURBS_CALCDIST_PATHDIVNUM;
	
	GetPoint(pre,sp);
//	for(i=1 ; i<=mPathDivNum ; i++){
	for(i=1 ; i<=GHMGCNURBS_CALCDIST_PATHDIVNUM ; i++){
		f32		p = sp + (f32)i * dp;
		f32		cdist;
		GetPoint(cur,p);
		cdist = GHM_VEC_GET_DIST(&cur,&pre);
		tdist += cdist;
		if( dist < tdist ){
			p = p - dp * (tdist - dist) / cdist;
			if( p < sp ){
				return sp;
			}else if( p > ep ){
				return ep;
			}
			return p;
		}
		pre = cur;
	}
	
	return ep;
}

