////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Nurbs
//
// ghmGcNurbs.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/28/2002
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCNURBS_H__
#define	__GHMGCNURBS_H__

#include <dolphin.h>
#include "ghmGcTypes.h"

#define	GHMGCNURBS_CALCDIST_PATHDIVNUM		(32)

struct ghmNurbsPathDesc{
	u16		mDegree;
	u16		mSpanCount;
	u16		mCVCount;
	u16		mKnotCount;
	f32		mStartParam;
	f32		mEndParam;
	f32		mTotalDistance;
	Vec4	*mpCV;
	f32		*mpKnot;
};

struct ghmScenePathSet{
	ghmNurbsPathDesc		*mpDesc;
	const char				*mpName;
	void					*mpData;
};

class ghmGcNurbs{
protected:
	ghmNurbsPathDesc			*mpDesc;
//	int							mPathDivNum;
	
public:
	ghmGcNurbs();
	virtual ~ghmGcNurbs();
	
	bool SetDesc(ghmNurbsPathDesc *pDesc);
	inline ghmNurbsPathDesc* GetDesc(void){ return mpDesc; }
	
	float BSplineBasisFunc(f32 param,int cvid);
	static float BSplineBasisFunc(f32 param,int degree,int cvid,f32 *pKnot);	// cvid < knotcount - degree
	float BSplineBasisDiffFunc(int rank,f32 param,int cvid);
	static float BSplineBasisDiffFunc(int rank,f32 param,int degree,int cvid,f32 *pKnot);
	
	int GetDegree(void);
	int GetSpanCount(void);
	int GetCVCount(void);
	int GetKnotCount(void);
	f32 GetStartParam(void);
	f32 GetEndParam(void);
	f32 GetTotalDistance(void);
	bool GetCV(int index,Vec4 &cv);
	float GetKnot(int index);
	
	bool GetPoint(Vec4 &point,f32 param);
	bool GetPoint(Vec  &point,f32 param);
	bool GetDiffVec(Vec4 &vec,f32 param,int rank=1);
	bool GetDiffVec(Vec  &vec,f32 param,int rank=1);
	
	inline bool GetStartPoint(Vec4 &point){ return GetPoint(point,GetStartParam()); }
	inline bool GetStartPoint(Vec  &point){ return GetPoint(point,GetStartParam()); }
	inline bool GetStartDiffVec(Vec4 &vec,int rank=1){ return GetDiffVec(vec,GetStartParam(),rank); }
	inline bool GetStartDiffVec(Vec  &vec,int rank=1){ return GetDiffVec(vec,GetStartParam(),rank); }
	
	inline bool GetEndPoint(Vec4 &point){ return GetPoint(point,GetEndParam()); }
	inline bool GetEndPoint(Vec  &point){ return GetPoint(point,GetEndParam()); }
	inline bool GetEndDiffVec(Vec4 &vec,int rank=1){ return GetDiffVec(vec,GetEndParam(),rank); }
	inline bool GetEndDiffVec(Vec  &vec,int rank=1){ return GetDiffVec(vec,GetEndParam(),rank); }
	
	f32 CalcDistance(f32 param);
	f32 CalcParam(f32 dist);
};

#endif	//__GHMGCNURBS_H__

