////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Task
//
// ghmGcTask.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/04/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__GHMGCTASK_CPP__

#include "ghmGcTask.h"

ghmGcTask*         ghmGcTaskController::mpCurrentTask = NULL;
#ifdef	GHMGCTASK_DEBUG
char        		ghmGcTask::mCurrentTaskName[16] = {0,};
#endif

void ghmGcTaskReturn(void);
void ghmGcTaskTerm(void);

ghmGcMem*	ghmGcTask::mpAllocHeap        = &ghmGcMem::mDefaultHeap;
ghmGcMem*	ghmGcTask::mpReserveAllocHeap = NULL;

ghmGcTaskController::ghmGcTaskController()
{
	int			i;
	u32			*pGPR;
	f64			*pFPR;
	mIDSeed = 0;
	mSP     = 0;
	mCR     = 0;
	for(i=0,pGPR=mGPR ; i<GHMGCTASK_SAVE_GPRNUM ; i++,pGPR++){
		*pGPR = 0;
	}
	for(i=0,pFPR=mFPR ; i<GHMGCTASK_SAVE_FPRNUM ; i++,pFPR++){
		*pFPR = 0.0;
	}
}

ghmGcTaskController::~ghmGcTaskController()
{
}

bool ghmGcTaskController::Initialize(void)
{
#ifdef	GHMGCTASK_DEBUG
	ghmGcTask::SetCurrentTaskName();
#endif
	return true;
}

bool ghmGcTaskController::Terminate(void)
{
	ghmGcTask	*pTask = (ghmGcTask *)mTaskAnchor.GetNext();
	ghmGcTask	*pNextTask;
	while( pTask!=NULL ){
		pNextTask = pTask->GetNext();
		delete pTask;
		pTask = pNextTask;
	}
	return true;
}

asm void ghmGcTaskController::CallHandler(register ghmGcTask *pTask)
{
	nofralloc
	
	mflr	r11
	stw		r11,GHMGCTASK_LR_ADDR(pTask)
	
	// WX^ޔ
	bl		StoreMainRegister;
	
	// r4͂̃^XÑ|C^Ă̂ŁÂ܂܈ɓnăR[
	bl		LoadTaskRegister
	
	// vOJE^WX^Ƀ[h
	lwz		r11,GHMGCTASK_PC_ADDR(pTask)
	mtlr	r11
	
	// X^bN|C^[h
	lwz		r1 ,GHMGCTASK_SP_ADDR(pTask)
	
	mr		r3,pTask
	
	// nhR[i߂AhXNWX^ɃZbgj
	blrl
	
	// nh𒼐ڔĂ܂ۂ̏
	
	// X^bNɃ^XÑ|C^ŏɂł
	lwz		r3,(sizeof(u32) * 3)(r1)
	
	// I
	entry	ghmGcTaskTerm
	li		r12,GHMGCTASK_IDLE
	stw		r12,GHMGCTASK_STATE_ADDR(r3)
	mr		r4,r3
	b		@callhandler_exit
	
	entry	ghmGcTaskReturn
	
	mr		r4,r3
	// WX^ޔ
	bl		StoreTaskRegister
	
@callhandler_exit:
	// Rg[{̂̃|C^擾
	lwz		r3,GHMGCTASK_CONTROLLER_ADDR(r3)
	
	// WX^
	bl		LoadMainRegister;
	
	// nhĂяo߂AhXZbg
	lwz		r11,GHMGCTASK_LR_ADDR(r4)
	mtlr	r11
	
	blr
}

bool ghmGcTaskController::FrameProcess(void)
{
	ghmGcTask	*pTask = (ghmGcTask *)mTaskAnchor.GetNext();
	ghmGcTask	*pNextTask;
	while( pTask!=NULL ){
		pNextTask = pTask->GetNext();
#ifdef	_DEBUG
		pTask->Check();
#endif
		switch( pTask->GetState() ){
		case	GHMGCTASK_READY:
			if( pTask->Run()==true ){
				// ^XNĂяo
				
				mpCurrentTask = pTask;
				
				CallHandler(pTask);
				
				mpCurrentTask = NULL;
				
				if( pTask->GetState()==GHMGCTASK_IDLE ){
					delete pTask;
				}else{
					pTask->Ready();
				}
			}
			break;
		case	GHMGCTASK_IDLE:
			delete pTask;
			break;
		case	GHMGCTASK_SUSPEND:
		default:
			break;
		}
		pTask = pNextTask;
	}
	return true;
}

ghmGcTask* ghmGcTaskController::GetTask(int id)
{
	ghmGcTask	*pTask = (ghmGcTask *)mTaskAnchor.GetNext();
	while( pTask!=NULL ){
		if( pTask->GetID()==id ){
			return pTask;
		}
		pTask = pTask->GetNext();
	}
	return NULL;
}

int ghmGcTaskController::Start(ghmGcTaskHandler *pHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( pHandler==NULL || (pStack==NULL && stackSize==0) ){
		return -1;
	}
	ghmGcMemSetAllocName("TSKCST0");
	ghmGcTask	*pTask = new ghmGcTask(&mTaskAnchor,this,pHandler,NULL,pStack,stackSize,prio);
	if( pTask==NULL ){
		return -1;
	}
	int			id;
	pTask->SetParam(0,param0);
	pTask->SetParam(1,param1);
	if( (id=pTask->GetID())==GHMGCTASK_STACKALLOC_ERRORID ){
		delete pTask;
		return -1;
	}
	return id;
}

int ghmGcTaskController::Start(ghmGcTaskHandler *pHandler,ghmGcTaskHandler *pTermHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( pHandler==NULL || (pStack==NULL && stackSize==0) ){
		return -1;
	}
	ghmGcMemSetAllocName("TSKCST1");
	ghmGcTask	*pTask = new ghmGcTask(&mTaskAnchor,this,pHandler,pTermHandler,pStack,stackSize,prio);
	if( pTask==NULL ){
		return -1;
	}
	int			id;
	pTask->SetParam(0,param0);
	pTask->SetParam(1,param1);
	if( (id=pTask->GetID())==GHMGCTASK_STACKALLOC_ERRORID ){
		delete pTask;
		return -1;
	}
	return id;
}

asm void ghmGcTaskController::CallHandler2(register ghmGcTask *pTask,register ghmGcTaskRegister *pRegister)
{
// ۂ͗ϐƂgpĂ邪AeXr3,r4Ɋi[ĂƂďĂ邪
// RpC̃[jOoȂ߂̑Ώ
#pragma unused(pTask)
#pragma unused(pRegister)
	nofralloc
	
	stwu	sp,-16(sp)
	mflr	r0
	stw		r0,20(sp)
	stw		r5,12(sp)
	
	// WX^ޔ
	bl		StoreRegister2;
	
	bl		CallHandler
	
	lwz		r5,12(sp)
	
	// WX^
	bl		LoadRegister2;
	
	lwz		r0,20(sp)
	mtlr	r0
	addi	sp,sp,16
	
	blr
}

int ghmGcTaskController::StartIM(ghmGcTaskHandler *pHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( pHandler==NULL || (pStack==NULL && stackSize==0) ){
		return -1;
	}
	ghmGcMemSetAllocName("TSKCSTI0");
	ghmGcTask	*pTask = new ghmGcTask(&mTaskAnchor,this,pHandler,NULL,pStack,stackSize,prio);
	if( pTask==NULL ){
		return -1;
	}
	int			id;
	pTask->SetParam(0,param0);
	pTask->SetParam(1,param1);
	if( (id=pTask->GetID())==GHMGCTASK_STACKALLOC_ERRORID ){
		delete pTask;
		return -1;
	}
	if( pTask->Run()==true ){
		// ^XNĂяo
		ghmGcTask			*pPrevTask = mpCurrentTask;
		ghmGcTaskRegister	registers;
		mpCurrentTask = pTask;
		CallHandler2(pTask,&registers);
		mpCurrentTask = pPrevTask;
		if( pTask->GetState()==GHMGCTASK_IDLE ){
			delete pTask;
//			return -1;
		}else{
			pTask->Ready();
		}
	}
	return id;
}

int ghmGcTaskController::StartIM(ghmGcTaskHandler *pHandler,ghmGcTaskHandler *pTermHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( pHandler==NULL || (pStack==NULL && stackSize==0)  ){
		return -1;
	}
	ghmGcMemSetAllocName("TSKCSTI1");
	ghmGcTask	*pTask = new ghmGcTask(&mTaskAnchor,this,pHandler,pTermHandler,pStack,stackSize,prio);
	if( pTask==NULL ){
		return -1;
	}
	int			id;
	pTask->SetParam(0,param0);
	pTask->SetParam(1,param1);
	if( (id=pTask->GetID())==GHMGCTASK_STACKALLOC_ERRORID ){
		delete pTask;
		return -1;
	}
	if( pTask->Run()==true ){
		// ^XNĂяo
		ghmGcTask			*pPrevTask = mpCurrentTask;
		ghmGcTaskRegister	registers;
		mpCurrentTask = pTask;
		CallHandler2(pTask,&registers);
		mpCurrentTask = pPrevTask;
		if( pTask->GetState()==GHMGCTASK_IDLE ){
			delete pTask;
//			return -1;
		}else{
			pTask->Ready();
		}
	}
	return id;
}

bool ghmGcTaskController::Stop(int id)
{
	ghmGcTask	*pTask;
	if( (pTask=GetTask(id))==NULL ){
		return false;
	}
	delete pTask;
	return true;
}

bool ghmGcTaskController::Suspend(int id)
{
	ghmGcTask	*pTask;
	if( (pTask=GetTask(id))==NULL ){
		return false;
	}
	return pTask->Suspend();
}

bool ghmGcTaskController::Resume(int id)
{
	ghmGcTask	*pTask;
	if( (pTask=GetTask(id))==NULL ){
		return false;
	}
	return pTask->Resume();
}

u32 ghmGcTaskController::GetState(int id)
{
	ghmGcTask	*pTask;
	if( (pTask=GetTask(id))==NULL ){
		return GHMGCTASK_INVALID_STATE;
	}
	return pTask->GetState();
}

int ghmGcTaskController::GetUniqueID(void)
{
	ghmGcTask	*pTask;
	int			id = mIDSeed;
	
	do{
		pTask = (ghmGcTask *)mTaskAnchor.GetNext();
		while( pTask!=NULL ){
			if( pTask->GetID()==id ){
				id++;
				if( id < 0 ){
					id = 0;
				}
				break;
			}
			pTask = pTask->GetNext();
		}
	}while( pTask!=NULL );
	mIDSeed = id + 1;
	if( mIDSeed==0x65536 ){
		mIDSeed = 0;
	}
	return id;
}

asm void ghmGcTaskController::LoadRegister2(void)
{
	nofralloc
	
	// SP + CR + GPR + FPR
	addi	r12,r3,GHMGCTASKCONTROLLER_HI_ADDR
	addi	r11,r5,GHMGCTASKCONTROLLER_HI_ADDR
	
	// 2 + 18 + 18 * 2 = 56
	li		r9,56
@loop_start:
	subi	r9,r9,1
	lwzu	r10,-sizeof(u32)(r11)
	stwu	r10,-sizeof(u32)(r12)
	cmpwi	r9,0
	bgt		@loop_start
	
	blr
}

asm void ghmGcTaskController::StoreRegister2(void)
{
	nofralloc
	
	// SP + CR + GPR + FPR
	addi	r12,r3,GHMGCTASKCONTROLLER_HI_ADDR
	addi	r11,r5,GHMGCTASKCONTROLLER_HI_ADDR
	
	// 2 + 18 + 18 * 2 = 56
	li		r9,56
@loop_start:
	subi	r9,r9,1
	lwzu	r10,-sizeof(u32)(r12)
	stwu	r10,-sizeof(u32)(r11)
	cmpwi	r9,0
	bgt		@loop_start
	
	blr
}

asm void ghmGcTaskController::LoadMainRegister(void)
{
	
	nofralloc
	
	// SP + CR + GPR + FPR
	addi	r12,r3,GHMGCTASKCONTROLLER_HI_ADDR
	
	// Load SP
	lwzu	r1 ,-sizeof(u32)(r12)
	
	// CR -> GPR
	lwzu	r11,-sizeof(u32)(r12)
	
	// Load CR
	mtcr	r11
	
	// Load GPR
	lwzu	r31,-sizeof(u32)(r12)
	lwzu	r30,-sizeof(u32)(r12)
	lwzu	r29,-sizeof(u32)(r12)
	lwzu	r28,-sizeof(u32)(r12)
	lwzu	r27,-sizeof(u32)(r12)
	lwzu	r26,-sizeof(u32)(r12)
	lwzu	r25,-sizeof(u32)(r12)
	lwzu	r24,-sizeof(u32)(r12)
	lwzu	r23,-sizeof(u32)(r12)
	lwzu	r22,-sizeof(u32)(r12)
	lwzu	r21,-sizeof(u32)(r12)
	lwzu	r20,-sizeof(u32)(r12)
	lwzu	r19,-sizeof(u32)(r12)
	lwzu	r18,-sizeof(u32)(r12)
	lwzu	r17,-sizeof(u32)(r12)
	lwzu	r16,-sizeof(u32)(r12)
	lwzu	r15,-sizeof(u32)(r12)
	lwzu	r14,-sizeof(u32)(r12)
	
	// Load FPR
	lfdu	f31,-sizeof(f64)(r12)
	lfdu	f30,-sizeof(f64)(r12)
	lfdu	f29,-sizeof(f64)(r12)
	lfdu	f28,-sizeof(f64)(r12)
	lfdu	f27,-sizeof(f64)(r12)
	lfdu	f26,-sizeof(f64)(r12)
	lfdu	f25,-sizeof(f64)(r12)
	lfdu	f24,-sizeof(f64)(r12)
	lfdu	f23,-sizeof(f64)(r12)
	lfdu	f22,-sizeof(f64)(r12)
	lfdu	f21,-sizeof(f64)(r12)
	lfdu	f20,-sizeof(f64)(r12)
	lfdu	f19,-sizeof(f64)(r12)
	lfdu	f18,-sizeof(f64)(r12)
	lfdu	f17,-sizeof(f64)(r12)
	lfdu	f16,-sizeof(f64)(r12)
	lfdu	f15,-sizeof(f64)(r12)
	lfdu	f14,-sizeof(f64)(r12)
	
	blr
}

asm void ghmGcTaskController::StoreMainRegister(void)
{
	
	nofralloc
	
	// SP + CR + GPR + FPR
	addi	r12,r3,GHMGCTASKCONTROLLER_HI_ADDR
	
	// Store SP
	stwu	r1 ,-sizeof(u32)(r12)
	
	// CR -> GPR
	mfcr	r11
	
	// Store CR
	stwu	r11,-sizeof(u32)(r12)
	
	// Store GPR
	stwu	r31,-sizeof(u32)(r12)
	stwu	r30,-sizeof(u32)(r12)
	stwu	r29,-sizeof(u32)(r12)
	stwu	r28,-sizeof(u32)(r12)
	stwu	r27,-sizeof(u32)(r12)
	stwu	r26,-sizeof(u32)(r12)
	stwu	r25,-sizeof(u32)(r12)
	stwu	r24,-sizeof(u32)(r12)
	stwu	r23,-sizeof(u32)(r12)
	stwu	r22,-sizeof(u32)(r12)
	stwu	r21,-sizeof(u32)(r12)
	stwu	r20,-sizeof(u32)(r12)
	stwu	r19,-sizeof(u32)(r12)
	stwu	r18,-sizeof(u32)(r12)
	stwu	r17,-sizeof(u32)(r12)
	stwu	r16,-sizeof(u32)(r12)
	stwu	r15,-sizeof(u32)(r12)
	stwu	r14,-sizeof(u32)(r12)
	
	// Store FPR
	stfdu	f31,-sizeof(f64)(r12)
	stfdu	f30,-sizeof(f64)(r12)
	stfdu	f29,-sizeof(f64)(r12)
	stfdu	f28,-sizeof(f64)(r12)
	stfdu	f27,-sizeof(f64)(r12)
	stfdu	f26,-sizeof(f64)(r12)
	stfdu	f25,-sizeof(f64)(r12)
	stfdu	f24,-sizeof(f64)(r12)
	stfdu	f23,-sizeof(f64)(r12)
	stfdu	f22,-sizeof(f64)(r12)
	stfdu	f21,-sizeof(f64)(r12)
	stfdu	f20,-sizeof(f64)(r12)
	stfdu	f19,-sizeof(f64)(r12)
	stfdu	f18,-sizeof(f64)(r12)
	stfdu	f17,-sizeof(f64)(r12)
	stfdu	f16,-sizeof(f64)(r12)
	stfdu	f15,-sizeof(f64)(r12)
	stfdu	f14,-sizeof(f64)(r12)
	
	blr
}

asm void ghmGcTaskController::LoadTaskRegister(void)
{
	
	nofralloc
	
	addi	r12,r4,GHMGCTASK_SP_ADDR
	
	// CR -> GPR
	lwzu	r11,-sizeof(u32)(r12)
	
	// Load CR
	mtcr	r11
	
	// Load GPR
	lwzu	r31,-sizeof(u32)(r12)
	lwzu	r30,-sizeof(u32)(r12)
	lwzu	r29,-sizeof(u32)(r12)
	lwzu	r28,-sizeof(u32)(r12)
	lwzu	r27,-sizeof(u32)(r12)
	lwzu	r26,-sizeof(u32)(r12)
	lwzu	r25,-sizeof(u32)(r12)
	lwzu	r24,-sizeof(u32)(r12)
	lwzu	r23,-sizeof(u32)(r12)
	lwzu	r22,-sizeof(u32)(r12)
	lwzu	r21,-sizeof(u32)(r12)
	lwzu	r20,-sizeof(u32)(r12)
	lwzu	r19,-sizeof(u32)(r12)
	lwzu	r18,-sizeof(u32)(r12)
	lwzu	r17,-sizeof(u32)(r12)
	lwzu	r16,-sizeof(u32)(r12)
	lwzu	r15,-sizeof(u32)(r12)
	lwzu	r14,-sizeof(u32)(r12)
	
	// Load FPR
	lfdu	f31,-sizeof(f64)(r12)
	lfdu	f30,-sizeof(f64)(r12)
	lfdu	f29,-sizeof(f64)(r12)
	lfdu	f28,-sizeof(f64)(r12)
	lfdu	f27,-sizeof(f64)(r12)
	lfdu	f26,-sizeof(f64)(r12)
	lfdu	f25,-sizeof(f64)(r12)
	lfdu	f24,-sizeof(f64)(r12)
	lfdu	f23,-sizeof(f64)(r12)
	lfdu	f22,-sizeof(f64)(r12)
	lfdu	f21,-sizeof(f64)(r12)
	lfdu	f20,-sizeof(f64)(r12)
	lfdu	f19,-sizeof(f64)(r12)
	lfdu	f18,-sizeof(f64)(r12)
	lfdu	f17,-sizeof(f64)(r12)
	lfdu	f16,-sizeof(f64)(r12)
	lfdu	f15,-sizeof(f64)(r12)
	lfdu	f14,-sizeof(f64)(r12)
	
	blr
}

asm void ghmGcTaskController::StoreTaskRegister(void)
{
	
	nofralloc
	
	addi	r12,r4,GHMGCTASK_SP_ADDR
	
	// CR -> GPR
	mfcr	r11
	
	// Save CR
	stwu	r11,-sizeof(u32)(r12)
	
	// Load GPR
	stwu	r31,-sizeof(u32)(r12)
	stwu	r30,-sizeof(u32)(r12)
	stwu	r29,-sizeof(u32)(r12)
	stwu	r28,-sizeof(u32)(r12)
	stwu	r27,-sizeof(u32)(r12)
	stwu	r26,-sizeof(u32)(r12)
	stwu	r25,-sizeof(u32)(r12)
	stwu	r24,-sizeof(u32)(r12)
	stwu	r23,-sizeof(u32)(r12)
	stwu	r22,-sizeof(u32)(r12)
	stwu	r21,-sizeof(u32)(r12)
	stwu	r20,-sizeof(u32)(r12)
	stwu	r19,-sizeof(u32)(r12)
	stwu	r18,-sizeof(u32)(r12)
	stwu	r17,-sizeof(u32)(r12)
	stwu	r16,-sizeof(u32)(r12)
	stwu	r15,-sizeof(u32)(r12)
	stwu	r14,-sizeof(u32)(r12)
	
	// Load FPR
	stfdu	f31,-sizeof(f64)(r12)
	stfdu	f30,-sizeof(f64)(r12)
	stfdu	f29,-sizeof(f64)(r12)
	stfdu	f28,-sizeof(f64)(r12)
	stfdu	f27,-sizeof(f64)(r12)
	stfdu	f26,-sizeof(f64)(r12)
	stfdu	f25,-sizeof(f64)(r12)
	stfdu	f24,-sizeof(f64)(r12)
	stfdu	f23,-sizeof(f64)(r12)
	stfdu	f22,-sizeof(f64)(r12)
	stfdu	f21,-sizeof(f64)(r12)
	stfdu	f20,-sizeof(f64)(r12)
	stfdu	f19,-sizeof(f64)(r12)
	stfdu	f18,-sizeof(f64)(r12)
	stfdu	f17,-sizeof(f64)(r12)
	stfdu	f16,-sizeof(f64)(r12)
	stfdu	f15,-sizeof(f64)(r12)
	stfdu	f14,-sizeof(f64)(r12)
	
	blr
}

ghmGcTask::ghmGcTask(ghmGcListObj *pAnchor,
                     ghmGcTaskController *pController,
                     ghmGcTaskHandler *pHandler,
                     ghmGcTaskHandler *pTermHandler,
                     void *pStack,int stackSize,int prio) : ghmGcListObj(pAnchor,prio<0? GHMGCLIST_DEFAULT_PRIO:prio)
{
	int			i;
	u32			*pSP;
	u32			*pGPR;
	f64			*pFPR;
	mpHandler     = pHandler;
	mpTermHandler = pTermHandler;
	mpController  = pController;
	mFlag         = 0;
	mState        = GHMGCTASK_IDLE;
	mID           = GHMGCTASK_NONUNIQUE_ID;
	if( pController!=NULL ){
		mID = pController->GetUniqueID();
	}
	if( stackSize < 0 ){
		// ftHgTCY
		stackSize = GHMGCTASK_DEFAULT_STACK_SIZE;
		pStack    = NULL;
	}
	if( pStack==NULL ){
		if( stackSize < 32 ){
			stackSize = 32;
		}
		if( mpAllocHeap!=NULL ){
#ifdef	_DEBUG
			mpStack = mpAllocHeap->Alloc(stackSize,"TskStack");
#else
			mpStack = mpAllocHeap->Alloc(stackSize);
#endif
		}
		if( mpStack==NULL && mpReserveAllocHeap!=NULL ){
#ifdef	_DEBUG
			mpStack = mpReserveAllocHeap->Alloc(stackSize,"TskStack");
#else
			mpStack = mpReserveAllocHeap->Alloc(stackSize);
#endif
		}
		if( mpStack==NULL ){
			mID = GHMGCTASK_STACKALLOC_ERRORID;
		}
		mStackAlloc = 1;
	}else{
		mpStack = pStack;
	}
	mStackSize = (u32)stackSize;
	
	mParam[0] = 
	mParam[1] = 0;
	
	if( mpStack!=NULL ){
		mSP = (u32)mpStack + stackSize - sizeof(u32) * 4;
#ifdef	_DEBUG
		pSP = (u32 *)mpStack;
		*pSP = GHMGCTASK_MAGICNO;
#endif
		pSP = (u32 *)mSP;
		pSP[0] = 0xffffffff;
		pSP[1] = 0;
		pSP[2] = 0;
		pSP[3] = (u32)this;
	}
	mPC = (u32)mpHandler;
	mCR = 0;
	for(i=0,pGPR=mGPR ; i<GHMGCTASK_SAVE_GPRNUM ; i++,pGPR++){
		*pGPR = 0;
	}
	for(i=0,pFPR=mFPR ; i<GHMGCTASK_SAVE_FPRNUM ; i++,pFPR++){
		*pFPR = 0.0;
	}
	if( mID!=GHMGCTASK_STACKALLOC_ERRORID ){
		mState = GHMGCTASK_READY;
	}
	mGPR[GHMGCTASK_SAVED_TASKPTR_REGID] = (u32)this;
	
#ifdef	GHMGCTASK_DEBUG
	strncpy(mTaskName,mCurrentTaskName,sizeof(mCurrentTaskName)-1);
#endif
}

ghmGcTask::~ghmGcTask()
{
	if( mpTermHandler!=NULL ){
		ghmGcTask		*pOldTask = ghmGcTaskController::mpCurrentTask;
		ghmGcTaskController::mpCurrentTask = this;
		mpTermHandler(this);
		ghmGcTaskController::mpCurrentTask = pOldTask;
	}
	if( mStackAlloc ){
		if( mpStack!=NULL ){
			mpAllocHeap->Free(mpStack);
			mpStack = NULL;
		}
	}
}

#ifdef	GHMGCTASK_DEBUG
void ghmGcTask::SetCurrentTaskName(const char *pName)
{
	if( pName==NULL ){
		strcpy(mCurrentTaskName,"No Task Name");
		return;
	}
	strncpy(mCurrentTaskName,pName,sizeof(mCurrentTaskName)-1);
}
#endif

#ifdef	_DEBUG
bool ghmGcTask::Check(void)
{
	u32		*pSP = (u32 *)mpStack;
	if( *pSP!=GHMGCTASK_MAGICNO ){
#ifdef	GHMGCTASK_DEBUG
		OSReport("Task Stack Area Broken !! : %s\n",mTaskName);
#else
		OSReport("Task Stack Area Broken !!\n");
#endif
		return false;
	}
	return true;
}
#endif

bool ghmGcTask::SetParam(u32 id,u32 param)
{
	if( id >= (sizeof(mParam) / sizeof(u32)) ){
		return false;
	}
	mParam[id] = param;
	return true;
}

bool ghmGcTask::Run(void)
{
	if( mState!=GHMGCTASK_READY ){
		return false;
	}
	mState = GHMGCTASK_RUN;
	return true;
}

bool ghmGcTask::Ready(void)
{
	if( mState != GHMGCTASK_RUN ){
		return false;
	}
	mState = GHMGCTASK_READY;
	return true;
}

bool ghmGcTask::Suspend(void)
{
	if( mState != GHMGCTASK_READY ){
		return false;
	}
	mState = GHMGCTASK_SUSPEND;
	return true;
}

bool ghmGcTask::Resume(void)
{
	switch( mState ){
	case	GHMGCTASK_SUSPEND:
		break;
	case	GHMGCTASK_READY:
	case	GHMGCTASK_RUN:
		return true;
	default:
		return false;
	}
	mState = GHMGCTASK_READY;
	return true;
}

int ghmGcTask::Start(ghmGcTaskHandler *pHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( mpController==NULL || pHandler==NULL ){
		return -1;
	}
	return mpController->Start(pHandler,NULL,param0,param1,pStack,stackSize,prio);
}

int ghmGcTask::Start(ghmGcTaskHandler *pHandler,ghmGcTaskHandler *pTermHandler,u32 param0,u32 param1,void *pStack,int stackSize,int prio)
{
	if( mpController==NULL || pHandler==NULL ){
		return -1;
	}
	return mpController->Start(pHandler,pTermHandler,param0,param1,pStack,stackSize,prio);
}

bool ghmGcTask::Stop(int id)
{
	if( mpController==NULL || mID==id ){
		return false;
	}
	return mpController->Stop(id);
}

asm void ghmGcTask::RetSys(void)
{
	nofralloc
	
	// ֐^[AhXivOJE^jXgA
	mflr	r11
	stw		r11,GHMGCTASK_PC_ADDR(r3)
	
	// X^bN|C^XgA
	stw		r1 ,GHMGCTASK_SP_ADDR(r3)
	
	b		ghmGcTaskReturn
}

asm void ghmGcTask::Exit(void)
{
	nofralloc
	mr		GHMGCTASK_SAVED_TASKPTR_REG,r3
	b		ghmGcTaskTerm
}

