////////////////////////////////////////////////////////////////////////////////
//
// GrassHopperManufacture GameCube Thread
//
// ghmGcThread.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 05/17/2004
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__GHMGCTHREAD_H__
#define	__GHMGCTHREAD_H__

#include "ghmGcTypes.h"
#include "ghmGcList.h"
#include "ghmGcMem.h"

typedef	void* ghmGcThreadHandler(void *);

#define	GHMGCTHREAD_PRIO_DEFAULT		(16)
#define	GHMGCTHREAD_PRIO_MIN			(0)
#define	GHMGCTHREAD_PRIO_MAX			(31)

#define	GHMGCTHREAD_STACKSIZE_DEFAULT	(4096)

class ghmGcThread : public ghmGcListObj{
protected:
	static	ghmGcListObj	mAnchor;
	static	ghmGcMem		*mpHeap;
	
	OSThread				mThread;
	void					*mpStack;
	int						mStackSize;
	bool					mStackAlloc;
	int						mThreadPriority;
	u32						mParam;
	ghmGcThreadHandler		*mpHandler;
	static void* ThreadHandler(void *pParam);
public:
	inline ghmGcThread* GetNext(void){ return (ghmGcThread *)ghmGcListObj::GetNext(); }
	inline ghmGcThread* GetPrev(void){ return (ghmGcThread *)ghmGcListObj::GetPrev(); }
	
	static bool Initialize(void);
	static bool Terminate(void);
	static int GetThreadNum(void);
	static int GetActiveThreadNum(void);
	
	ghmGcThread(ghmGcThreadHandler *pHandler,u32 param,int prio=-1,int stackSize=-1,void *pStack=NULL);
	virtual ~ghmGcThread();
	
	static ghmGcMem* SetAllocHeap(ghmGcMem *pHeap);
	inline static ghmGcMem* GetAllocHeap(void){ return mpHeap; }
	
	bool SetThreadPriority(int prio);
	inline int GetThreadPriority(void){ return mThreadPriority; }
	
	inline int GetStackSize(void){ return mStackSize; }
	
	bool IsCreated(void);
	bool IsTerminated(void);
	bool IsSuspended(void);
	
	bool Suspend(void);
	bool Resume(void);
	
	bool Cancel(void);
};


#endif	//__GHMGCTHREAD_H__

