////////////////////////////////////////////////////////////////////////////////
//
// Killer7 LoadProcess
//
// SK7LoadProcess.cpp
//
// Creator       : Sato Shinya (GHM)
// Creation Date : 02/12/2004
//
////////////////////////////////////////////////////////////////////////////////

#define	__SK7LOADPROCESS_CPP__

#include <limits.h>
#include "SK7LoadProcess.h"
#include "../matsu/mK7Sound.h"
#include "../wat/K7GlobalFlag.h"
#include "../Yam/YK7Effect.h"
#include "../wat/WK7GameInfo.h"
#include "../Yam/YK7EffectNoise.h"
#include "../yam/YK7CharChange.h"


//========================================================================
//					RXgN^
//========================================================================
SK7LoadProcess::SK7LoadProcess()
{
	m_eProc			= eProcLoad;
	m_bLoadFontFlag = false;
	m_ScenarioNo	= 0;

	gpSound->mReleasePcSsm();
	gpSound->mReadSsm("seTv.ssm",eBankPlayer);
	m_pEffect = CreateNoiseEffect();
	SetEffectIndeManageFlag( m_pEffect, true );
}

//========================================================================
//					fXgN^
//========================================================================
SK7LoadProcess::~SK7LoadProcess()
{
	gpSound->mReleaseSsm("seTv.ssm");	
	gpSound->mReadPcSsmReq( WK7PLAYERCONTROL->GetMainID() );
}

//========================================================================
//					t[vZX
//========================================================================
bool 	SK7LoadProcess::FrameProcess(void)
{
	if( m_eProc == eProcEnd && m_eProc == eProcLoadEnd )	return true;	

	//----------------	J[h}`FbN	----------------------//
	switch( CheckCardInsert() ){
	case MEMORYCARD_RETCODE_TRUE:	break;
	case MEMORYCARD_RETCODE_FALSE:	if( m_eProc > eProcLoad ) 	m_eProc = eProcMessage;	break;
	case MEMORYCARD_RETCODE_BUSY:	return true;
	case MEMORYCARD_RETCODE_NOCARD:	InitCheck();	return true;
	}
	
	//	`l^C	--------------------------------------------
	if( !m_bChanFlag && m_eProc > eProcLoad ){
		if( ++m_iChanTime >= CHAN_MOVE_TIME ){
			m_iChanTime = 0;
			m_bChanFlag = true;
			SetChannelNomral();
		}
	}

	if( !m_bCtrl && eProcLoad < m_eProc )	return false;
	//----------------	C	----------------------//
	switch( m_eProc ){
	case eProcLoad:
		if( LoadData(LOAD_TV) )	m_eProc = eProcCursolDraw;
		break;
	case eProcCursolDraw:
		if( m_bCtrl ){
			gpSound->mPlaySE( SFX_TV_CONTENTS_CHOICE );
			m_eProc = eProcSlotSelect; 
		}
		return true;
	case eProcSlotSelect:	CheckSlotSelect();		return true;
	case eProcSetMound:		
		SetMessage	( CERROR_MES_CHECK );
		CreateCard();
		SetCardMount( m_chSlotNo );
		m_eProc = eProcCheckMount;
		return true;
	case eProcMessage:
		switch(FrameMessageKey()){
		case MEMORYCARD_RETCODE_BUSY:	
		case MEMORYCARD_RETCODE_TRUE:	return true;
		case MEMORYCARD_RETCODE_FALSE:	InitCheck(); return true;
		}
		return true;
	case eProcCheckMount:	CheckMount();			return true;
	case eProcFileCheck:	CheckFileCheck();		return true;
	case eProcEreaCheck:	CheckFileErea(true);	return true;
	case eProcDammyDelete:	DeleteDammyFile();		return true;
	case eProcSavedDraw:
		CheckCardSavedData();
		if( m_eProc == eProcSaveSelect ){
			m_bLoadFontFlag = true;
			SetMessage( CERROR_MES_NONE );
		}
		return true;
	case eProcSaveSelect:	CheckSavePoint(eBaseLoad);		return true;
	case eProcReadCheck:	CheckSavePlay(eBaseLoad);		return true;
	case eProcRead:			ReadSaveData();					return true;
	case eProcReadEndCheck:	CheckReadEnd();					return true;
	case eProcInit:			CheckInitCard();				return true;	
	case eProcDelete:		FrameDeleteProcess();			return true;
	case eProcLoadEnd:		
	case eProcEnd:			return true;
	}
	return true;
}

//========================================================================
//					_[vZX
//========================================================================
bool 	SK7LoadProcess::RenderProcess(void)
{
	if( !m_bRender )	return false;
	
	//	tHg`
	if( m_bLoadFontFlag ){
		RenderSaveComment();
		RenderMoveComment();
	}
	//	XbgZNĝƂ	
	switch( m_eProc ){
	case eProcEnd:
	case eProcSlotSelect:
	case eProcCursolDraw:
		RenderSlotName( (u8)m_chSlotNo, m_lSlotColor[CARD_SLOT_A], m_lSlotColor[CARD_SLOT_B], true, m_bCtrl );
		break;
	}
	//	`FbNpbZ[W
	RenderCheckFont( (u8)m_chSelect );

	//	`l`	
	m_spChan.Render();
	
	//	tHg`
	RenderFont( (s8)m_chSlotNo, false );
	
	//	bl̃G[`FbN
	RenderCheckFontError( (u8)m_chSelect );

	DrawSousaLine();
	MoveEffect(m_pEffect);
	DrawEffect(m_pEffect);
	_DrawTVScreen();	

	m_spTV.Render();
	return true;
}

//========================================================================
//					Rg
//========================================================================
void 	SK7LoadProcess::RenderMoveComment(void)
{
	for( int i = 0; i < SAVE_DATA_NUM; i++ ){
		m_pYFont->DrawSetup();
		if( i == m_chSaveSelectNo )	m_pYFont->SetColor2(0x000000ff);
		else						m_pYFont->SetColor2(SLOT_FONT_COLOR);
		m_pYFont->DrawFSStr( SAVE_NO_X, SAVE_DATA_Y+i*SAVE_DATA_GAPY, false, true, "%d:", (i+1) ); 
		if( m_chSavedFlag & (1<<i) ){
			char font[32];
			memset( font, '\0', 32 );
			memcpy( font, m_SaveName[i].name, 31 );
			m_pYFont->DrawFSStr( SAVE_DATA_X, SAVE_DATA_Y+i*SAVE_DATA_GAPY, false, true, "%s", font ); 
		}else{
			if( i == m_chSaveSelectNo )	m_pYFont->SetColor2(0x000000ff);
			else						m_pYFont->SetColor2(SLOT_FONT_COLOR);
			m_pYFont->DrawFSStr( SAVE_DATA_X, SAVE_DATA_Y+i*SAVE_DATA_GAPY, false, true, "No Data Saved" ); 
		}	
	}
}

//========================================================================
//					ǂݍ݃`FbN
//========================================================================
bool		SK7LoadProcess::CheckModeFlag()
{
	if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true ){
		if( m_chHopperFlag & (1<<m_chSaveSelectNo) ){
			return true;
		}else{
			SetMessage( CERROR_MES_NODATA_HOPPER7, 0 );			
			m_eProc = eProcMessage;
			return false;
		}
	}else{
		if( m_chHopperFlag & (1<<m_chSaveSelectNo) ){
			SetMessage( CERROR_MES_NODATA_KILLER8, 0 );			
			m_eProc = eProcMessage;
			return false;
		}else{
			return true;
		}
	}
	return false;
}

//========================================================================
//					ǂݍ
//========================================================================
void		SK7LoadProcess::ReadSaveData()
{
	switch( m_chEnumNo ){
	case 0:
		InitCardInfo();	
		char FileName[32];

		ghmGcMemSetAllocName("SAVELOAD");
		if( (m_pSaveData = (SK7SaveData *)ghmGcMemAlloc(SAVEDATA_SIZE))==NULL ){
			return;
		}
		if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true || 
			K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 )==true )
		{
			sprintf( FileName, "%s_%d", SAVE_FILE_NAME8, (int)m_chSaveSelectNo );
		}else{
			sprintf( FileName, "%s_%d", SAVE_FILE_NAME, (int)m_chSaveSelectNo );
		}
		SetCardOpen( m_chSlotNo, FileName );
		m_chEnumNo = 1;
		return;
		
	case 1:
		s32 State = CARDGetResultCode(m_chSlotNo);
		if( State == CARD_RESULT_BUSY )	return;
		if( !ResultCheck( State ) ){
			m_eProc = eProcMessage;
			return;
		}
		SetCardRead( m_pSaveData, SAVEDATA_SIZE, 0 );
		m_chEnumNo	= 0;
		m_eProc 	= eProcReadEndCheck;
		return;
	}	
}

//========================================================================
//					ǂݍݏI
//========================================================================
void			SK7LoadProcess::CheckReadEnd()
{
	s32 State = CARDGetResultCode(m_chSlotNo);
	switch( State ){
	case CARD_RESULT_READY:
		SetCardClose();
		//	f[^Zbg		-------------------------------
		memcpy( &m_SaveData, m_pSaveData, sizeof(m_SaveData) );
		GHMGCMEM_SAFE_FREE( m_pSaveData );		
		
		//	ItZbg`FbN	---------------------------
		CARDStat CardState;
		GetCardStatus( m_chSlotNo, &CardState );
		if( CardState.iconAddr != ICON_OFFSET ){
			goto BREAK_MEMORY;
		}
		//	Rg`FbN	---------------------------		
		char comment[32];
		memset( comment, '\0', 32 );
		memcpy( comment, m_SaveData.Comment, DATACHECK_WORD_NUM );
		if( strcmp( comment, KILLER7_COMMENT )!= 0 ){
			goto BREAK_MEMORY;
		}
		//	bqb`FbN	-----------------------------
		if( CheckCRC( &m_SaveData ) ){
			//	o[W`FbN	---------------------
			if( m_SaveData.Version != SAVELOAD_VERSION ){
				goto BREAK_MEMORY;
			}
			//	Z[uf[^ł̃`FbN	---------------------
			if( !(m_SaveData.SaveData.mClearScenarioID >= -1 && m_SaveData.SaveData.mClearScenarioID <= 7)){
				goto BREAK_MEMORY;
			}
			//	[h`FbN	--------------------------
			if( !CheckModeFlag() )	return;
	
			DeleteCard();
			SetCardUnMount( m_chSlotNo );
			const char* Cm = m_SaveData.Comment;
			if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true ){
				m_ScenarioNo = 0;
			}else{
				m_ScenarioNo = atoi( &Cm[COMMENT_MEM_SCN] );
			}
			m_eProc = eProcLoadEnd;
			return;
		}
		//	폜`FbN	-------------------------------
BREAK_MEMORY:
		GHM_SAFE_DELETE			( m_pCardInfo );	
		SetMessage( CERROR_MES_BREAK_SAVE, 1 );
		m_eProc = eProcMessage;
		return;
		
	case CARD_RESULT_BUSY:	return;
	default:
		ResultCheck( State );
		m_eProc = eProcMessage;
		return;
	}
}

//========================================================================
//					
//========================================================================
bool 		SK7LoadProcess::GetMenuEnd(void)
{
	if( m_eProc == eProcLoadEnd || m_eProc == eProcEnd ){
		return true;
	}
	return false;
}

//========================================================================
//					
//========================================================================
void 		SK7LoadProcess::CheckInitCard(void)
{
	switch( CARDGetResultCode(m_chSlotNo) ){
	case CARD_RESULT_BUSY:			return;
	case CARD_RESULT_READY:			
		InitCheck();
		SetMessage( CERROR_MES_INIT_OK );
		m_eProc 		= eProcMessage;
		return;
	case CARD_RESULT_NOCARD:		InitCheck();	return;
	case CARD_RESULT_FATAL_ERROR:
	case CARD_RESULT_IOERROR:		SetMessage( CERROR_MES_INIT_MISS );	break;
	}
	SetCardUnMount( m_chSlotNo );
	DeleteCard();
	m_eProc = eProcMessage;
}

//========================================================================
//					`FbN
//========================================================================
void 		SK7LoadProcess::InitCheck(void)
{
	InitBase();
	m_eProc 		= eProcSlotSelect;
	m_bLoadFontFlag	= false;
}


//========================================================================
//					폜
//========================================================================
void 		SK7LoadProcess::FrameDeleteProcess(void)
{
	switch( CARDGetResultCode(m_chSlotNo) ){
	case CARD_RESULT_BUSY:	return;
	case CARD_RESULT_READY:
		InitCheck();
		SetMessage( CERROR_MES_DELETE_OK );
		m_eProc = eProcMessage;
		return;
	default:
		ResultCheck( CARDGetResultCode(m_chSlotNo), MESERROR_DELETE );
		m_eProc = eProcMessage;
		return;
	}
}
