////////////////////////////////////////////////////////////////////////////////
//
// Killer7 SaveBase
//
// SK7SaveBase.cpp
//
// Creator       : Sato Shinya (GHM)
// Creation Date : 10/02/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__SK7SAVEBASE_CPP__

#include "SK7SaveBase.h"
#include "SK7FirstCheck.h"
#include "SK7SaveProcess.h"
#include "SK7LoadProcess.h"
#include "../matsu/mK7Sound.h"
#include "../wat/K7GlobalFlag.h"
#include "../Yam/YK7Effect.h"

//	bqbp	------------------------------------------
#define 	CRCPOLY1	(0x04c11db7ul)
typedef unsigned char byte;
u32 Crc(int n, byte c[]);

bool		SK7ResetProcessCallBack( u32 param );	//R[obN

//========================================================================
//					RXgN^
//========================================================================
SK7SaveBase::SK7SaveBase()
{
	m_bRender 		= false;				// `tO
	m_bCtrl 		= false;				// 싖tO
	m_bLoadFontFlag	= false;
	m_bChanFlag		= false;
	m_chSlotInFlag	= CARD_SLOT_EXIT;
	m_chSavedFlag	= 0;					// Z[utO
	m_chSelect		= 0;					// I
	m_chSlotNo		= 0;
	m_chEnumNo		= 0;
	m_chReadNo		= 0;
	m_iChanTime		= 0;
	m_chHopperFlag	= 0;
	m_eLoad			= LOAD_FONT;
	m_pFile			= NULL;
	m_pIconData		= NULL;
	m_pBanner		= NULL;
	m_pSaveData		= NULL;
	m_pEffect		= NULL;
	m_pSavedData	= NULL;
	m_lSlotColor[CARD_SLOT_A]	= m_lSlotColor[CARD_SLOT_B]	= CARD_NOT_COLOR;
	for( int i = 0; i < SAVE_DATA_NUM; i++ ){
		memset( m_SaveName[i].name, '\0', sizeof( char )*32 );
	}
	memset( m_bKiller8Flag, false, sizeof( bool )*SAVE_DATA_NUM );
}


//========================================================================
//					fXgN^
//========================================================================
SK7SaveBase:: ~SK7SaveBase()
{
	GHM_SAFE_DELETE( m_pFile ); 
  	GHM_SAFE_DELETE( m_pBanner );
	GHM_SAFE_DELETE( m_pIconData );
	GHM_SAFE_DELETE( m_pSaveData );
	GHM_SAFE_DELETE_ARRAY( m_pSavedData );
	DeleteEffect( m_pEffect );
	m_spTV.Delete();
}

//========================================================================
//					pl
//========================================================================
SK7SaveBase* SK7SaveBase::Create(u8 id)
{
	switch( id ){
	case	eBaseFirstCheck:	return new SK7FirstCheck(CHECK_STATE_FIRST);
	case 	eBaseKiller8Check:	return new SK7FirstCheck(CHECK_STATE_KILLER8);
	case 	eBaseSave:			return new SK7SaveProcess(SAVE_STATE_TV);
	case 	eBaseLoad:			return new SK7LoadProcess();
	case 	eBaseSaveEnd:		return new SK7SaveProcess(SAVE_STATE_END);
	default:
		break;
	}
	return NULL;
}

//========================================================================
//					Z[uf[^`FbN
//========================================================================
bool		SK7SaveBase::CheckCardExist()
{
	bool DeleteFlag = false;
	CARDStat cardstat;
	m_chSavedFlag = 0;
	m_chHopperFlag= 0;
	char FileName[32];

	for( int j = 0; j < SAVE_DATA_NUM; j++ ){
		if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true || 
			K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 )==true )
		{
			sprintf( FileName, "%s_%d", SAVE_FILE_NAME8, j );
		}else{
			sprintf( FileName, "%s_%d", SAVE_FILE_NAME, j );
		}
		for( int i = 0; i < 127; i++ ){
			memset( &cardstat, 0, sizeof( CARDStat ) );
			CARDGetStatus( m_chSlotNo, i, &cardstat );
			//	폜t@C`FbN	-----------------------------
			if( strcmp( cardstat.fileName, SAVE_DAMMY_NAME )==0 ){
				DeleteFlag = true;
				continue;
			}
			//	t@C`FbN	---------------------------------
			if( strcmp( cardstat.fileName, FileName )==0 ){			//	L[V/8̏㏑f[^
				m_chSavedFlag |= (1<<j);							//	tOĂ
				continue;
			}
		}
	}
	//	폜Ƃ	----------------------------------
	if( DeleteFlag )	return true;
	return false;
}

//========================================================================
//					bZ[WZbg
//========================================================================
void		SK7SaveBase::SetMessageCheck( u8 MesNo, s8 Select )
{
	CreateCheck( MesNo );	
	m_chSelect = Select;
}

//========================================================================
//					bZ[WZbg
//========================================================================
void		SK7SaveBase::SetMessage( u8 MesNo, s8 Select )
{
	CreateMessage( MesNo );	
	m_chSelect = Select;
}

//========================================================================
//					bZ[WZbg
//========================================================================
void		SK7SaveBase::SetMessageUp( u8 MesNo, s8 Select )
{
	CreateCheckUp( MesNo );	
	m_chSelect = Select;
}

//========================================================================
//					Z[uf[^tHg쐬
//========================================================================
void  		SK7SaveBase::RenderSaveComment()
{
	EffectRenderSetup( EF_PROJECT_ORTHO, EF_BLEND_ALPHA, EF_TEV_USETEX ) ;
	
	f32 size;
	if( m_chSavedFlag & (1<<(m_chSaveSelectNo)) ){
		char font[32];
		memset( font, '\0', 32 );
		memcpy( font, m_SaveName[m_chSaveSelectNo].name, 31 );
		size = m_pYFont->DrawFSStr( 0.0f, 0.0f, false, false, "%s", font ); 
	}else{
		size = m_pYFont->DrawFSStr( 0.0f, 0.0f, false, false, "No Data Saved" ); 
	}
	m_spBoxLine.SetSprite	( SAVE_NO_X, SAVE_DATA_Y+m_chSaveSelectNo*SAVE_DATA_GAPY-2, size+SAVE_BOX_PLUSX, SAVE_BOX_SIZEY, SLOT_FONT_COLOR );
	m_spBoxLine.SetVisible	( true );
	m_spBoxLine.Render		();
}

//========================================================================
//					f[^[h
//========================================================================
bool		SK7SaveBase::LoadData( s8 DataNo )
{
	switch( m_eLoad ){
	case LOAD_FONT:
		if( LoadMessage() ){
			if( DataNo == LOAD_FONT )	return true;
			m_eLoad = LOAD_TV;
		}
		break;
		
	case LOAD_TV:
		if( m_spTV.LoadTexture( TV_TEX ) ){
			m_spTV.SetTexSprite( 0, 0, m_spTV.GetTexture() );
			m_spTV.SetVisible( true );
			SetChannel(0);
			if( DataNo == LOAD_TV )		return true;
			ghmGcMemSetAllocName("SFILE");
			m_pFile 	= new SK7File;
			m_eLoad = LOAD_BANNER;
		}
		break;		
				
	case LOAD_BANNER:
		if( m_pFile->OpenDataFile( DVD_BANNER_FILENAME, &((void*)m_pBanner) ) ){
			SetChannel(1);
			m_eLoad = LOAD_ICON;
		}
		break;
	case LOAD_ICON:
		char* IconFile=NULL;
		if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 ) )		IconFile = ICON_TEX_H;
		else if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 ) )	IconFile = ICON_TEX_1;
		else												IconFile = ICON_TEX_7;
		if( m_pFile->OpenDataFile( IconFile, &((void*)m_pIconData) )){
			GHM_SAFE_DELETE( m_pFile );
			return true;
		}
		break;
	}
	return false;
}

//========================================================================
//					Z[uf[^\
//========================================================================
void			SK7SaveBase::CheckCardSavedData()
{
	char FileName[32];
	char DataName2[32];
	char DataName[32];
	char HopperName[32];
	
	switch( m_chEnumNo ){
	case 0:
		for( int i = m_chReadNo; i < SAVE_DATA_NUM; i++ ){
			if( !(m_chSavedFlag & (1<<i)) )		continue;
			InitCardInfo();
			if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true ||
				K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 )==true )
			{
				sprintf( FileName, "%s_%d", SAVE_FILE_NAME8, i );
			}else{
				sprintf( FileName, "%s_%d", SAVE_FILE_NAME, i );
			}
			if( SetCardOpen( m_chSlotNo, FileName ) == CARD_RESULT_READY ){
				GHM_SAFE_DELETE( m_pSaveData );
//				GHM_SAFE_DELETE_ARRAY( m_pSavedData );
				ghmGcMemSetAllocName("SAVELOADRE");
				if( (m_pSaveData = (SK7SaveData *)ghmGcMemAlloc(SAVEDATA_SIZE))==NULL ){
					return;
				}
				m_chReadNo = (s8)i;
				SetCardRead( m_pSaveData, SAVEDATA_SIZE, 0 );
//				SetCardRead( m_pSavedData, CARD_READ_SIZE, 0 );
				m_chEnumNo = 1;
				return;
			}
		}
		m_chEnumNo = 0;
		m_chReadNo= 0;
		m_eProc = eProcSaveSelect;
		return;
		
	case 1:
		switch( CARDGetResultCode(m_chSlotNo) ){
		case CARD_RESULT_READY:
			SetCardClose();
			//	Z[uf[^
			const char* Cm = (const char*)m_pSaveData;
			int No = atoi( &Cm[COMMENT_MEM_SCN] );
			memset( &DataName, '\0', 32 );
			memcpy( DataName,  (byte*)m_pSaveData, 7 );

			//	ItZbg`FbN	---------------------------
			CARDStat CardState;
			GetCardStatus( m_chSlotNo, &CardState );
			if( CardState.iconAddr != ICON_OFFSET ){
				goto BREAK_DATA;
			}
			//	bqb`FbN	-----------------------------
			if( !CheckCRC( m_pSaveData ) ){
				goto BREAK_DATA;
			}				
			//	o[W`FbN	---------------------
			if( m_pSaveData->Version != SAVELOAD_VERSION ){
				goto BREAK_DATA;
			}
			//	Z[uf[^ł̃`FbN	---------------------
			if( !(m_pSaveData->SaveData.mClearScenarioID >= -1 && m_pSaveData->SaveData.mClearScenarioID <= 7)){
				goto BREAK_DATA;
			}
			//	G[`FbN	==========================================
			if( strcmp( DataName, KILLER7_COMMENT )==0 ){
				memset( &HopperName, '\0', 32 );				
				memcpy( HopperName, (byte*)m_pSaveData+9, 9 );
				memset( &DataName2, '\0', 32 );
				memcpy( DataName2, (byte*)m_pSaveData+32, 32 );
				//	zbp[V
				if( strcmp( HopperName, KILLERH_DATANAME )==0 ){
					m_chHopperFlag |= (1<<(int)m_chReadNo);
					sprintf( m_SaveName[m_chReadNo].name, "[%s] %s", KILLERH_FILENAME, DataName2 );
				}else{
					sprintf( m_SaveName[m_chReadNo].name, "[%s] %s", tblSaveScenarioName[No], DataName2 );
				}
			}else{
BREAK_DATA:
				sprintf( m_SaveName[m_chReadNo].name, BREAK_DATA_NAME );				
			}
			GHMGCMEM_SAFE_FREE( m_pSaveData );		
			m_chReadNo++;
			m_chEnumNo=0;
			break;
		case CARD_RESULT_BUSY:	return;
		default:
			ResultCheck( CARDGetResultCode(m_chSlotNo) );
			m_eProc = eProcMessage;
			break;
		}
		GHMGCMEM_SAFE_FREE( m_pSaveData );		
		break;
	}
	return;
}

//========================================================================
//					Z[uʒumF
//========================================================================
void			SK7SaveBase::CheckSavePoint( s8 State )
{
	if( ghmGcFile::IsDiskStandby()==false ){
		return ;
	}
	//		-------------------------------------
	if( CheckKeyR( SK7_KEY_UP ) ){
		if( --m_chSaveSelectNo < 0 ){
			m_chSaveSelectNo = SAVE_DATA_NUM-1;
		}
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		return;
	}
	//		-------------------------------------
	if( CheckKeyR( SK7_KEY_DOWN ) ){
		if( ++m_chSaveSelectNo > SAVE_DATA_NUM-1 ){
			m_chSaveSelectNo = 0;
		}
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		return;
	}
	//	I	-------------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
		gpSound->mPlaySE( SFX_TV_CONTENTS_CHOICE );
		if( m_chSavedFlag & (1<<(m_chSaveSelectNo)) ){
			if( State == eBaseSave ){ 
				SetMessageCheck( CHECK_MES_OVER_WRITE, 1 );
				m_eProc = eProcSaveCheck;
				return;
			}else{	//	ǂݍݎ
				SetMessageCheck( CHECK_MES_LOAD, 1 );
				m_eProc = eProcReadCheck;			
				return;
			}
		}else{
			if( State == eBaseSave ){ 			
				//	Zbg֎~	
				k7app.SetResetProcessCallback( SK7ResetProcessCallBack, false );
				SetMessageCheck( CHECK_MES_SAVECHECK, 1 );
				m_eProc = eProcWrite;
				return;
			}else{
				m_chSaveSelectNo = 0;
			}
		}
	}
	//	LZ	---------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_B ){
		m_bLoadFontFlag			= false;
		SetCardUnMount		 ( m_chSlotNo );	
		DeleteCard();
		for( int i = 0; i < SAVE_DATA_NUM; i++ )	memset( m_SaveName[i].name, '\0', sizeof( char )*32 );
		gpSound->mPlaySE( SFX_TV_CANCEL );
		m_eProc = eProcSlotSelect;
		return;
	}
}

//========================================================================
//				_~[f[^̍폜
//========================================================================
void		SK7SaveBase::DeleteDammyFile()
{
	switch( m_chEnumNo ){
	case 0:
		SetDeleteCard( m_chSlotNo, SAVE_DAMMY_NAME );		//	폜
		m_chEnumNo = 1;		
		return;
		
	case 1:
		switch( CARDGetResultCode(m_chSlotNo) ){
		case CARD_RESULT_READY:	
			m_chEnumNo = 0;	
			m_eProc = eProcSavedDraw;
			return;
		case CARD_RESULT_BUSY:	return;
		default:
			ResultCheck( CARDGetResultCode(m_chSlotNo) );
			m_chEnumNo = 0;
			m_eProc = eProcMessage;
			return;
		}
		break;
	}
	return;
}


//========================================================================
//				RESULTbZ[W`FbN
//========================================================================
bool		SK7SaveBase::ResultCheck( s32 State, s8 Select )
{
	switch( State ){
	case CARD_RESULT_READY:			return true;
	case CARD_RESULT_WRONGDEVICE:	SetMessage( CERROR_MES_NOTCARD );		break;
	case CARD_RESULT_IOERROR:    	
		if( Select == MESERROR_DELETE )	SetMessage( CERROR_MES_DELETE_MISS );
		else							SetMessage( CERROR_MES_BREAK_CARD );
		break;
	case CARD_RESULT_BROKEN:     	SetMessage( CERROR_MES_FORMAT );		break;
	case CARD_RESULT_NOENT:      	SetMessage( CERROR_MES_NO_SPACE );		break;
	case CARD_RESULT_INSSPACE:   	SetMessage( CERROR_MES_NO_SPACE );		break;
	case CARD_RESULT_ENCODING:   	SetMessage( CERROR_MES_FORMAT );		break;
	case CARD_RESULT_FATAL_ERROR:	SetMessage( CERROR_MES_BREAK_CARD );	break;
	default:						SetMessage( CERROR_MES_DELETE_MISS );	break;
	}
	return false;
}

//========================================================================
//					J[h}`FbN
//========================================================================
s8 			SK7SaveBase::CheckCardInsert(void)
{
	s32 retcode[2];
	
	for( int i = 0; i < 2; i++ ){
		retcode[i] = GetCardInsert( i );
		if( retcode[i] == CARD_RESULT_READY || retcode[i] == CARD_RESULT_WRONGDEVICE ){
			m_lSlotColor[i] = SLOT_FONT_COLOR;
		}else{
			m_lSlotColor[i] = CARD_NOT_COLOR;
		}
	}

	if( m_chSlotInFlag == CARD_SLOT_EXIT ){
		return MEMORYCARD_RETCODE_TRUE;
	}else{
		switch( retcode[m_chSlotInFlag] ){
		case CARD_RESULT_READY:
		case CARD_RESULT_BUSY:
			return MEMORYCARD_RETCODE_TRUE;	
		case CARD_RESULT_NOCARD:				
			return MEMORYCARD_RETCODE_NOCARD;		
		case CARD_RESULT_WRONGDEVICE:
			SetMessage( CERROR_MES_NOTCARD );
			return MEMORYCARD_RETCODE_FALSE;
		default:
			SetMessage( CERROR_MES_BREAK_CARD );
			return MEMORYCARD_RETCODE_FALSE;
		}
		return MEMORYCARD_RETCODE_BUSY;
	}
	return MEMORYCARD_RETCODE_BUSY;
}

//========================================================================
//					XbgZNg
//========================================================================
void	  		SK7SaveBase::CheckSlotSelect()
{
	if( CheckKeyT( SK7_KEY_UP ) ){
		if( --m_chSlotNo < 0 ){
			m_chSlotNo = 0;
			return;
		}
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		return;
	}
	if( CheckKeyT( SK7_KEY_DOWN ) ){
		if( ++m_chSlotNo > CARD_SLOT_EXIT ){
			m_chSlotNo = CARD_SLOT_EXIT;
			return;
		}
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		return;
	}
	//	XbgI	-----------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
		gpSound->mPlaySE( SFX_TV_CONTENTS_CHOICE );
		s32 StateCard[2];
		StateCard[CARD_SLOT_A] = GetCardInsert( CARD_SLOT_A );
		StateCard[CARD_SLOT_B] = GetCardInsert( CARD_SLOT_B );
		if( (m_chSlotNo == CARD_SLOT_A && StateCard[CARD_SLOT_A]!=CARD_RESULT_READY ) ||
			(m_chSlotNo == CARD_SLOT_B && StateCard[CARD_SLOT_B]!=CARD_RESULT_READY ) )
		{
			switch( StateCard[m_chSlotNo] ){
			case CARD_RESULT_NOCARD:		
			case CARD_RESULT_BUSY:			SetMessage( CERROR_MES_NOSLOT );		break;
			case CARD_RESULT_WRONGDEVICE:
				m_chSlotInFlag = (s8)m_chSlotNo;
				SetMessage( CERROR_MES_NOTCARD );		
				break;
			default:						SetMessage( CERROR_MES_BREAK_CARD );	break;
			}
			m_eProc = eProcMessage;
			return;
		}
		//	[hȂƂ	
		if( m_chSlotNo == CARD_SLOT_EXIT ){
			m_eProc = eProcEnd;
			return;
		}
		//	ZN^[TCY`FbN	
		if( !GetSectorSize(m_chSlotNo) ){
			SetMessage( CERROR_MES_BREAK_CARD );
			m_eProc = eProcMessage;
			return;
		}
		//	XbgItHg
		m_chSlotInFlag = (s8)m_chSlotNo;
		m_chSaveSelectNo = 0;
		m_eProc = eProcSetMound;
		return;
	}
	//	I	--------------------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_B ){
		gpSound->mPlaySE( SFX_TV_CANCEL );
		if( m_chSlotNo == CARD_SLOT_EXIT ){
			m_eProc = eProcEnd;
			return;
		}else{
			m_chSlotNo = CARD_SLOT_EXIT;
		}
		return;
	}
}

//========================================================================
//				}Egʃ`FbN
//========================================================================
void		SK7SaveBase::CheckMount()
{
	s32 State = CARDGetResultCode( m_chSlotNo );
	switch( State ){
	case CARD_RESULT_BUSY: return;
	case CARD_RESULT_READY:
	case CARD_RESULT_BROKEN:
		SetCardCheck( m_chSlotNo );
		m_eProc = eProcFileCheck;
		return;
	default:
		ResultCheck( State );
		m_eProc = eProcMessage;
		return;
	}
}

//========================================================================
//				t@CCʃ`FbN
//========================================================================
void		SK7SaveBase::CheckFileCheck()
{
	s32 state = CARDGetResultCode( m_chSlotNo );
	if( state == CARD_RESULT_BUSY )	return;
	
	if( ResultCheck( state ) ){
		m_eProc = eProcEreaCheck;
	}else{
		m_eProc = eProcMessage;
	}
}

//========================================================================
//				t@CGA`FbN
//========================================================================
void		SK7SaveBase::CheckFileErea( bool LoadFlag )
{
	//	f[^`FbN
	bool DeleteFlag = CheckCardExist();
	
	//	[hf[^`FbN
	if( LoadFlag && m_chSavedFlag == 0 ){
		SetCardUnMount	( m_chSlotNo );	
		DeleteCard();
		SetMessage( CERROR_MES_NOT_DATA );
		m_eProc = eProcMessage;
		return;
	}

	if( m_chSavedFlag != 0 ){
		if( DeleteFlag ){
			m_eProc = eProcDammyDelete;
		}else{
			m_eProc = eProcSavedDraw;
		}
		return;
	}
	//̈`FbN
	if( GetCardErea( m_chSlotNo, SAVEDATA_SIZE ) ){
		if( DeleteFlag ){
			m_eProc = eProcDammyDelete;
		}else{
			m_eProc = eProcSavedDraw;
		}
		return;
	}else{
		SetCardUnMount		 ( m_chSlotNo );	
		DeleteCard();
		SetMessage( CERROR_MES_NO_SPACE );
		m_eProc = eProcMessage;
	}
}

//========================================================================
//					Z[u[hŏImF
//========================================================================
void			SK7SaveBase::CheckSavePlay( s8 State )
{
	//		-------------------------------------
	if( CheckKeyT( SK7_KEY_LEFT ) ){
		if( m_chSelect == 1 )	gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		m_chSelect = 0;
		return;
	}
	//	E	-------------------------------------
	if( CheckKeyT( SK7_KEY_RIGHT ) ){
		if( m_chSelect == 0 )	gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		m_chSelect = 1;
		return;
	}
	//	I	-------------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
		gpSound->mPlaySE( SFX_TV_CONTENTS_CHOICE );
		if( m_chSelect == 0 ){
			if( State == eBaseSave ){
				//	Zbg֎~	
				k7app.SetResetProcessCallback( SK7ResetProcessCallBack, false );
				SetMessageCheck( CHECK_MES_SAVECHECK, 1 );
				m_eProc = eProcWrite;
			}else{
				SetMessageCheck( CHECK_MES_LOADCHECK, 1 );
//				if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true || 
//					K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 )==true )
//				{
//					m_eProc = eProcReadFlag;					
//				}else{
				m_eProc = eProcRead;
//				}
			}
			return;
		}else{
			SetMessageCheck( CHECK_MES_NONE, 0 );
			m_eProc = eProcSaveSelect;
			return;
		}
	}
	//	LZ	---------------------------------
	if( HSD_PadGetTrigger(0) & HSD_PAD_B ){
		SetMessageCheck( CHECK_MES_NONE, 0 );
		gpSound->mPlaySE( SFX_TV_CANCEL );
		m_eProc = eProcSaveSelect;
		return;
	}
}

//========================================================================
//					bZ[WΉ
//========================================================================
s8 			SK7SaveBase::FrameMessageKey(void)
{
	if( CheckKeyT( SK7_KEY_RIGHT ) ){
		if( GetCheckUpNo() == CHECK_MESUP_NONE || m_chSelect == 1 ) return MEMORYCARD_RETCODE_BUSY;
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		m_chSelect = 1;
		return MEMORYCARD_RETCODE_BUSY;
	}
	if( CheckKeyT( SK7_KEY_LEFT ) ){
		if( GetCheckUpNo() == CHECK_MESUP_NONE || m_chSelect == 0 ) return MEMORYCARD_RETCODE_BUSY;
		gpSound->mPlaySE( SFX_TV_MOVE_CURSOR );
		m_chSelect = 0;
		return MEMORYCARD_RETCODE_BUSY;
	}
	if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
		gpSound->mPlaySE( SFX_TV_CONTENTS_CHOICE );
		u8 MesNo = GetMesNo();
		switch( MesNo ){
		case CERROR_MES_NOTCARD:
		case CERROR_MES_NOSLOT:
		case CERROR_MES_BREAK_CARD:
		case CERROR_MES_BREAK_DATA:
		case CERROR_MES_BREAK_SAVE_NO:
		case CERROR_MES_NODATA_KILLER8:
		case CERROR_MES_NODATA_HOPPER7:
		case CERROR_MES_NOT_DATA:
		case CERROR_MES_INIT_MISS:
		case CERROR_MES_INIT_OK:
		case CERROR_MES_DELETE_OK:
		case CERROR_MES_DELETE_MISS:
		case CERROR_MES_IPL:
			break;
		case CERROR_MES_NO_SPACE:	
			SetMessage( CERROR_MES_FILE_REQ );
			return MEMORYCARD_RETCODE_BUSY;
		case CERROR_MES_FILE_REQ:	
			SetMessage( CERROR_MES_IPL );
			return MEMORYCARD_RETCODE_BUSY;
		case CERROR_MES_FORMAT:
			SetMessage	( CERROR_MES_INIT_CHECK );
			SetMessageUp( CHECK_MESUP_INIT_CHECK, 1 );
			return MEMORYCARD_RETCODE_BUSY;
		case CERROR_MES_BREAK_SAVE:
			SetMessage	( CERROR_MES_BREAK_SAVE_A );
			SetMessageUp( CHECK_MESUP_DELETE_CHECK, 1 );
			return MEMORYCARD_RETCODE_BUSY;			
		}
		switch(m_chSelect){
		case 0:
			switch( MesNo ){
			case CERROR_MES_INIT_CHECK:	
				SetMessage		( CERROR_MES_INITCHECK );
				SetMessageUp	( CHECK_MESUP_NONE, 0 );
				SetCardFormat	( m_chSlotNo );
				m_eProc = eProcInit;
				return MEMORYCARD_RETCODE_TRUE;
			case CERROR_MES_BREAK_SAVE_A:
				SetMessage		( CERROR_MES_DELETECHECK );
				SetMessageUp	( CHECK_MESUP_NONE, 0 );
				char FileName[32];
				if( K7SCR_GET_GLOBAL_FLAG( K7GLFL_HOPPER7 )==true || 
					K7SCR_GET_GLOBAL_FLAG( K7GLFL_KILLER1 )==true )
				{
					sprintf( FileName, "%s_%d", SAVE_FILE_NAME8, (int)m_chSaveSelectNo );
				}else{
					sprintf( FileName, "%s_%d", SAVE_FILE_NAME, (int)m_chSaveSelectNo );
				}
				SetDeleteCard( m_chSlotNo, FileName );
				m_eProc = eProcDelete;
				return MEMORYCARD_RETCODE_TRUE;
			}
			return MEMORYCARD_RETCODE_FALSE;
				
		case 1:
			switch( MesNo ){
			case CERROR_MES_INIT_CHECK:	
//				SetMessageUp	( CHECK_MESUP_NONE, 0 );
//				SetMessage( CERROR_MES_BREAK_DATA );
				return MEMORYCARD_RETCODE_FALSE;
			case CERROR_MES_BREAK_SAVE_A:
				SetMessage		( CERROR_MES_BREAK_SAVE_NO );
				SetMessageUp	( CHECK_MESUP_NONE, 0 );
				return MEMORYCARD_RETCODE_BUSY;			
			}
		}
		return MEMORYCARD_RETCODE_FALSE;
	}
	return MEMORYCARD_RETCODE_BUSY;
}


//========================================================================
//					x[Xh
//========================================================================
void		SK7SaveBase::InitBase()
{
	//	bZ[W
	SetMessage		( CERROR_MES_NONE );
	SetMessageUp	( CHECK_MESUP_NONE, 0 );
	SetMessageCheck	( CHECK_MES_NONE, 0 );
	//	A}Eg
	SetCardUnMount	( m_chSlotNo );
	//	
	DeleteCard();
	//  Z[uf[^
	GHM_SAFE_DELETE	( m_pSaveData );
	GHM_SAFE_DELETE_ARRAY( m_pSavedData );
	//	tO
	memset( m_SaveName, 0, sizeof(SAVENAME)*SAVE_DATA_NUM );
	m_chSlotInFlag 	= CARD_SLOT_EXIT;
	m_chSlotNo 		= CARD_SLOT_A;
	m_chSelect		= 0;
	m_chEnumNo		= 0;
	m_chReadNo		= 0;
}

//========================================================================
//					f[^ǂݍݏI`FbN
//========================================================================
bool  		SK7SaveBase::GetFileReadEnd(void)
{ 
	if(m_eProc > eProcLoad) return true;
	return false;
}	

//========================================================================
//					bqb̎擾
//========================================================================
u32			SK7SaveBase::GetCRC( SK7SaveData* Data )
{
	Data->CrcNo = 0;								//	CRCNA	
	return Crc( sizeof(SK7SaveData), (byte*)Data );	//	CRC`FbN
}

//========================================================================
//					bqb`FbN	
//========================================================================
bool			SK7SaveBase::CheckCRC( SK7SaveData* pSave )
{
	u32 CrcNo,CrcNo2;
	if( pSave != NULL ){
		CrcNo = pSave->CrcNo;
		pSave->CrcNo = 0;
		CrcNo2 = GetCRC( pSave );
		if( CrcNo == CrcNo2 ){
			return true;
		}
	}
	return false;
}

//========================================================================
//					bqb
//========================================================================
u32 Crc( int n, byte c[] )
{
	unsigned int i, j;
	unsigned long r;
	r = 0xFFFFFFFFUL;
	for( i = 0; i < n; i++ ){
		r ^= (unsigned long)c[i] << (32 - CHAR_BIT);
		for( j = 0; j < CHAR_BIT; j++ )
		if( r & 0x80000000UL) 	r = (r<<1) ^ CRCPOLY1;
		else					r <<= 1;
	}
	return ~r & 0xFFFFFFFFUL;
}


//========================================================================
//					݃Zbg֎~
//========================================================================
bool		SK7ResetProcessCallBack( u32 param )
{
	if( param == false )	return false;
	else					return true;
}
