////////////////////////////////////////////////////////////////////////////////
//
// Killer7 AfterImage
//
// WK7AfterImage.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 09/18/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7AFTERIMAGE_CPP__

#include ".\WK7AfterImage.h"
#include ".\WK7Cam.h"
#include ".\WK7MainTask.h"

WK7AfterImage		afterImage;

WK7AfterImage::WK7AfterImage()
{
	mEnable           = false;
	mCapture          = false;
	mCaptureCounter.SetValue(0,0);
	mCaptureTick      = 240;
	mFrameProcess     = false;
	mpPrevImage       = NULL;
	mClearColorUpdate = true;
	mClearAlphaUpdate = true;
	mClearZUpdate     = true;
	mClearColor       = 0x00000000;
	mClearZ           = 1.0f - 0.00001f;
	mRate             = 0xe0;
	mEnableAfterImage = true;
	mBackIntensity.SetValue(0xff);
}

WK7AfterImage::~WK7AfterImage()
{
	GHM_SAFE_DELETE(mpPrevImage);
}

bool WK7AfterImage::SetBackIntensity(u32 inten,u32 tick)
{
	mBackIntensity.SetValue(inten,tick);
	return true;
}

bool WK7AfterImage::Clear(void)
{
	Mtx				view;
	Mtx44			proj;
	
	WK7Cam::Set2DMatrix(view,proj);
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	GXSetScissor     (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetDither      (GX_FALSE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor(GX_COLOR0A0,*((GXColor*)&mClearColor));
	
	// uhȂ
	GXSetBlendMode   (GX_BM_NONE,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,mClearZUpdate==true? GX_TRUE : GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (mClearColorUpdate==true? GX_TRUE : GX_FALSE);
	GXSetAlphaUpdate (mClearAlphaUpdate==true? GX_TRUE : GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	
	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mClearZ);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mClearZ);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mClearZ);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mClearZ);
		
	GXEnd();
	
	return true;
}

bool WK7AfterImage::FrameProcess(void)
{
//	mFrameProcess = true;
	mBackIntensity .FrameProcess();
	mCaptureCounter.FrameProcess();
	return true;
}

bool WK7AfterImage::PreRenderProcess(void)
{
	mFrameProcess = true;
	if( mEnable==false || mpPrevImage==NULL ){
//	if( mFrameProcess==false || mEnable==false || mpPrevImage==NULL ){
		Clear();
		return false;
	}
	Clear();
	
#if 0
	// Ot[_O
	
	Mtx				view;
	Mtx44			proj;
	GXColor			matcolor;
	
	matcolor.r = (u8)mRate;
	matcolor.g = (u8)mRate;
	matcolor.b = (u8)mRate;
	matcolor.a = (u8)(mClearColor & 0x000000ff);
	
	WK7Cam::Set2DMatrix(view,proj);
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	GXSetScissor     (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetDither      (GX_FALSE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor(GX_COLOR0A0,matcolor);
	
	// uhȂ
	GXSetBlendMode   (GX_BM_NONE,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,mClearZUpdate==true? GX_TRUE : GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (mClearColorUpdate==true? GX_TRUE : GX_FALSE);
	GXSetAlphaUpdate (mClearAlphaUpdate==true? GX_TRUE : GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc     (GX_VA_TEX0,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
	
	GXSetNumTexGens  (1);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	mpPrevImage->LoadTexObj(GX_TEXMAP0);
	screenImageFullColor.LoadTexObj(GX_TEXMAP1);
	
	GXSetNumTevStages(3);
	
	GXColor		reg1_color;
#define	REG1_COLOR		(120-1);
	reg1_color.r = REG1_COLOR;
	reg1_color.g = REG1_COLOR;
	reg1_color.b = REG1_COLOR;
	reg1_color.a = 0;
	GXSetTevColor    (GX_TEVREG1,reg1_color);
	
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVREG0);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetTevOrder    (GX_TEVSTAGE1,GX_TEXCOORD0,GX_TEXMAP1,GX_COLOR_NULL);
	GXSetTevColorIn  (GX_TEVSTAGE1,GX_CC_TEXA,GX_CC_C1,GX_CC_C0,GX_CC_ZERO);
//	GXSetTevColorIn  (GX_TEVSTAGE1,GX_CC_TEXA,GX_CC_ZERO,GX_CC_C0,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE1,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE1,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE1,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetTevOrder    (GX_TEVSTAGE2,GX_TEXCOORD0,GX_TEXMAP1,GX_COLOR_NULL);
	GXSetTevColorIn  (GX_TEVSTAGE2,GX_CC_TEXA,GX_CC_ZERO,GX_CC_C0,GX_CC_CPREV);
	GXSetTevColorOp  (GX_TEVSTAGE2,GX_TEV_COMP_RGB8_EQ,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE2,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE2,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mClearZ);
		GXTexCoord2f32(0.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mClearZ);
		GXTexCoord2f32(1.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(1.0f,1.0f);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
#endif
	
	return true;
}

bool WK7AfterImage::_RenderProcess2(void)
{
	
	// Ot[_O
	
	Mtx				view;
	Mtx44			proj;
	GXColor			matcolor;
	
	matcolor.r = (u8)mRate;
	matcolor.g = (u8)mRate;
	matcolor.b = (u8)mRate;
	matcolor.a = (u8)(mClearColor & 0x000000ff);
	
	WK7Cam::Set2DMatrix(view,proj);
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	GXSetScissor     (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetDither      (GX_TRUE);
//	GXSetDither      (GX_FALSE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor(GX_COLOR0A0,matcolor);
	
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,mClearZUpdate==true? GX_TRUE : GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (mClearColorUpdate==true? GX_TRUE : GX_FALSE);
	GXSetAlphaUpdate (GX_FALSE);
//	GXSetAlphaUpdate (mClearAlphaUpdate==true? GX_TRUE : GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc     (GX_VA_TEX0,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
	
	GXSetNumTexGens  (1);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	screenImageFullColor.LoadTexObj(GX_TEXMAP0);
	
	GXSetNumTevStages(1);
	
	// uh
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXSetNumTevStages(1);
	
	screenImageFullColor.LoadTexObj(GX_TEXMAP0);
	
#define	REG0_COLOR		(120-1)
#define	REG0_ALPHA		(120-1)
static	GXColor		reg0_color = { REG0_COLOR,REG0_COLOR,REG0_COLOR,REG0_ALPHA };
static	GXColor		reg1_color = { 0xff,0xff,0xff,0xff };
//		GXColor		reg2_color;
		
	if( mBackIntensity.GetValue() > 0xff ){
		reg1_color.r = 
		reg1_color.g = 
		reg1_color.b = 
		reg1_color.a = 0xff;
	}else{
		reg1_color.r = 
		reg1_color.g = 
		reg1_color.b = 
		reg1_color.a = (u8)mBackIntensity.GetValue();
	}
//	reg2_color.r = 
//	reg2_color.g = 
//	reg2_color.b = 
//	reg2_color.a = (u8)(0xff - reg1_color.a);
	
	GXSetTevColor    (GX_TEVREG0,reg0_color);
	GXSetTevColor    (GX_TEVREG1,reg1_color);
//	GXSetTevColor    (GX_TEVREG2,reg2_color);
	
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_A1,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_TEXC);
//	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_A0,GX_CA_TEXA,GX_CA_ONE,GX_CA_ZERO);
//	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_A0,GX_CA_TEXA,GX_CA_A1,GX_CA_ZERO);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_COMP_A8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mClearZ);
		GXTexCoord2f32(0.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mClearZ);
		GXTexCoord2f32(1.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(1.0f,1.0f);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
	
	return true;
	
	
	return true;
}

bool WK7AfterImage::_RenderProcess(void)
{
	
	// Ot[_O
	
	Mtx				view;
	Mtx44			proj;
	GXColor			matcolor;
	
	matcolor.r = (u8)mRate;
	matcolor.g = (u8)mRate;
	matcolor.b = (u8)mRate;
	matcolor.a = (u8)(mClearColor & 0x000000ff);
	
	WK7Cam::Set2DMatrix(view,proj);
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	GXSetScissor     (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetDither      (GX_TRUE);
//	GXSetDither      (GX_FALSE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor(GX_COLOR0A0,matcolor);
	
	// uhȂ
	GXSetBlendMode   (GX_BM_NONE,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,mClearZUpdate==true? GX_TRUE : GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (mClearColorUpdate==true? GX_TRUE : GX_FALSE);
	GXSetAlphaUpdate (GX_FALSE);
//	GXSetAlphaUpdate (mClearAlphaUpdate==true? GX_TRUE : GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc     (GX_VA_TEX0,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
	
	GXSetNumTexGens  (1);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	if( mEnableAfterImage==true ){
		mpPrevImage->LoadTexObj(GX_TEXMAP0);
	}else{
		screenImageFullColor.LoadTexObj(GX_TEXMAP0);
	}
	
	GXSetNumTevStages(1);
	
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mClearZ);
		GXTexCoord2f32(0.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mClearZ);
		GXTexCoord2f32(1.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(1.0f,1.0f);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
	
	// uh
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXSetNumTevStages(1);
	
	screenImageFullColor.LoadTexObj(GX_TEXMAP0);
	
#define	REG0_COLOR		(120-1)
#define	REG0_ALPHA		(120-1)
static	GXColor		reg0_color = { REG0_COLOR,REG0_COLOR,REG0_COLOR,REG0_ALPHA };
static	GXColor		reg1_color = { 0xff,0xff,0xff,0xff };
		GXColor		reg2_color;
		
	if( mBackIntensity.GetValue() > 0xff ){
		reg1_color.a = 0;
	}else{
		reg1_color.a = (u8)(0xff - mBackIntensity.GetValue());
	}
#if 0
static	bool		flag = true;
static	u8			delta = 0x4;
	
	if( flag==true ){
		if( reg1_color.a < 0xff - delta ){
			reg1_color.a += delta;
		}else{
			flag = false;
		}
	}else{
		if( reg1_color.a > delta ){
			reg1_color.a -= delta;
		}else{
			flag = true;
		}
	}
#endif
	
	reg2_color.r =             reg1_color.r;
	reg2_color.g =             reg1_color.g;
	reg2_color.b =             reg1_color.b;
	reg2_color.a = (u8)(0xff - reg1_color.a);
	
	GXSetTevColor    (GX_TEVREG0,reg0_color);
	GXSetTevColor    (GX_TEVREG1,reg1_color);
	GXSetTevColor    (GX_TEVREG2,reg2_color);
	
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_TEXC);
//	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_TEXA,GX_CA_A0,GX_CA_A1,GX_CA_A2);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_COMP_A8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mClearZ);
		GXTexCoord2f32(0.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mClearZ);
		GXTexCoord2f32(1.0f,0.0f);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(1.0f,1.0f);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mClearZ);
		GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
	
	return true;
}

bool WK7AfterImage::RenderProcess(void)
{
	if( mFrameProcess==false ){
		return false;
	}
	
	mFrameProcess = false;
	
	RenewImageBuffer();
	
	if( mEnable==false ){
		return false;
	}
	
	if( mEnableAfterImage==true ){
		if( mpPrevImage!=NULL && mpPrevImage->GetTexture()!=NULL ){
			if( mCapture==true ){
				_RenderProcess();
			}
			// Lv`
			if( mCaptureCounter.IsAnimating()==false ){
				mCaptureCounter.SetValue(mCaptureTick);
				mCaptureCounter.Play    (0,mCaptureTick);
				mpPrevImage->ScreenCopy(false);
				mCapture = true;
			}
		}
	}else{
		_RenderProcess2();
	}
	
	return true;
}

bool WK7AfterImage::RenewImageBuffer(void)
{
	if( mEnable==true && mEnableAfterImage==true ){
		if( mpPrevImage==NULL ){
			ghmGcMemSetAllocName("ScrImg");
			if( (mpPrevImage = new WK7ScreenImage)==NULL ){
				return false;
			}
			mpPrevImage->SetSize  (WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
			mpPrevImage->SetFormat(GX_TF_RGB5A3);
//			mpPrevImage->SetFormat(GX_TF_RGB565);
			mpPrevImage->SetWrapMode(GX_CLAMP,GX_CLAMP);
			mpPrevImage->SetTexFilter(GX_NEAR,GX_NEAR);
			mpPrevImage->SetClearColor((GXColor){0x00,0x00,0x00,0x00});
			mpPrevImage->SetClearZ(GX_MAX_Z24);
			if( mpPrevImage->AllocBuffer()==false ){
				return false;
			}
			mCapture = false;
		}else{
			if( mpPrevImage->GetTexture()==NULL ){
				if( mpPrevImage->AllocBuffer()==false ){
					return false;
				}
				mCapture = false;
			}
		}
	}else{
		if( mCapture==false ){
			GHM_SAFE_DELETE(mpPrevImage);
		}else{
			mCapture = false;
		}
	}
	return true;
}

bool WK7AfterImage::SetEnable(bool flag)
{
//	if( flag==false ){
//		GHM_SAFE_DELETE(mpPrevImage);
//		mCapture = false;
//	}else{
//		if( mpPrevImage==NULL ){
//			ghmGcMemSetAllocName("ScrImg");
//			if( (mpPrevImage = new WK7ScreenImage)==NULL ){
//				return false;
//			}
//			mpPrevImage->SetSize  (WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
//			mpPrevImage->SetFormat(GX_TF_RGB5A3);
////			mpPrevImage->SetFormat(GX_TF_RGB565);
//			mpPrevImage->SetWrapMode(GX_CLAMP,GX_CLAMP);
//			mpPrevImage->SetTexFilter(GX_NEAR,GX_NEAR);
//			mpPrevImage->SetClearColor((GXColor){0x00,0x00,0x00,0x00});
//			mpPrevImage->SetClearZ(GX_MAX_Z24);
//			if( mpPrevImage->AllocBuffer()==false ){
//				return false;
//			}
//			mCapture = false;
//		}else{
//			if( mpPrevImage->GetTexture()==NULL ){
//				if( mpPrevImage->AllocBuffer()==false ){
//					return false;
//				}
//				mCapture = false;
//			}
//		}
//	}
	mEnable = flag;
	RenewImageBuffer();
	return true;
}

bool WK7AfterImage::SetRate(u32 rate)
{
	if( rate > 0xff ){
		rate = 0xff;
	}
	mRate = rate;
	return true;
}

