////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Archive
//
// WK7Archive.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/08/2003
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__WK7ARCHIVE_H__
#define	__WK7ARCHIVE_H__

//#include "..\Lib\ghmGcDefine.h"
//#include "..\Lib\ghmGcList.h"
//#include "..\Lib\ghmGcUsb.h"
//#include "..\Lib\ghmGcFile.h"
//#include "..\Fujita\C_SceneDesc.h"
//#include ".\WK7SceneDesc.h"
//#include ".\WK7Model.h"
#include ".\WK7Motion.h"
//#include ".\WK7Path.h"

class WK7SceneDesc;
class WK7Model;
class WK7Path;

#define	WK7ARCHIVE_ILLEGAL_FILESIZE		(0x1000000)			// 16Mbyte

inline const char* GetDescName(const HSD_CameraDesc *desc)
{
	return _GetDescName( (char**)desc );
}

struct WK7SdExpInfo{
	const char		*mpVersionStr;
	int				mVersion;
	union{
		struct{
			u32		mBlur           :  1;
			u32		mCreateStartKey :  1;
			u32		mCreateEndKey   :  1;
			u32		mFlagPad        : 25;
			u32		mFogType        :  4;
		};
		u32			mFlag;
	};
	u32				mFogColor;
	f32				mFogStartZ;
	f32				mFogEndZ;
};

struct WK7RenderDesc{
	HSD_TObjDesc	*mpToonTObjDesc;		// gD[C[W
	HSD_TObjDesc	*mpGradTObjDesc;		// Of[VC[W
	int				mTerminator;
};

//
// Sysdolphine Archive
//
class WK7Archive : public WK7Obj{
	friend class WK7ArchiveController;
public:
	enum WK7ArchiveState{
		WK7ARCHIVE_VACANT  = 0,
		WK7ARCHIVE_LOADING,
		WK7ARCHIVE_LOADED,
	};
protected:
	static ghmGcListObj		mAnchor;
#ifdef	_DEBUG
	static f32				mDebugPrintSec;
#endif
	
	HSD_Archive				*mpArchive;
	void					*mpData;
	u32						mRefCount;
	u32						mType;
	u32						mID;
//	WK7Model				*mpModel;
//	WK7ModelMotion			*mpModelMotion;
//	WK7CameraMotion			*mpCameraMotion;
//	WK7LightMotion			*mpLightMotion;
	ghmGcFile				*mpFile;
	WK7ArchiveState			mState;
	
	union{
		struct{
			u32				mArchiveAlloc :  1;
			u32				mDataAlloc    :  1;
			u32				mStatic       :  1;
			u32				mPathAlloc    :  1;
			u32				mFlagPad      : 28;
		};
		u32					mFlag;
	};
	
	static WK7Archive* sysFind(u32 type,u32 id,WK7Archive *pArchive=NULL);
	
	void InitMember(void);
	WK7Archive(ghmGcListObj *pAnchor,int prio=GHMGCLIST_DEFAULT_PRIO);
	HSD_CameraDesc** GetCameraDescList(void);
//	void SetWholeMotionRate(void);
public:
	virtual ~WK7Archive();
	
	inline WK7Archive* GetPrev(void){ return (WK7Archive *)ghmGcListObj::GetPrev(); }
	inline WK7Archive* GetNext(void){ return (WK7Archive *)ghmGcListObj::GetNext(); }
	
	// 
	virtual bool Initialize(void);
	
	// t[
	virtual bool FrameProcess(void);
	
	// I
	virtual bool Terminate(void);
	
	// A[JCum
	static HSD_Archive* AllocArchive(void);
	
	// A[JCu[h
	bool LoadArchiveB(const char *pfilename,HSD_Archive *pArchive=NULL);
	bool LoadArchiveB(ghmGcUsb &serv,const char *pfilename,HSD_Archive *pArchive=NULL);
	bool LoadArchive(const char *pfilename,HSD_Archive *pArchive=NULL);
	bool LoadArchive(void *pData,HSD_Archive *pArchive=NULL);
	
	// A[JCu
	bool FreeArchive(void);
	
	// A[JCuQ
	HSD_Archive* ReferenceArchive(void);
	// A[JCuQƉ
	bool ReleaseReferenceArchive(void);
	
	inline WK7ArchiveState GetState(void){ return mState; }
	
	// QƃJEg擾
	inline u32 GetRefCount(void){ return mRefCount; }
	
	// f[^̈mallocŊmۂĂ邩ǂ`FbN
	inline u32 GetDataAlloc(void){ return mDataAlloc; }
			
	int GetExternNum(void);
	const char* GetExtern(int index);
	
	int GetPublicNum(void);
	const char* GetPublic(int index);
	void* GetPublicAddress(const char *pSymbol);
	
	WK7SceneDesc* GetSceneDesc(void);
	
	int GetModelSetNum(void);
	C_ModelSet* GetModelSet(int index);
	C_ModelSet* GetModelSet(const char *pName);
	bool IncludeModelData(void);
	bool IncludeModelAnim(void);
	bool IncludeModelMatAnim(void);
	bool IncludeModelShapeAnim(void);
	
	C_CameraSet* GetCameraSet(void);
	
	int GetLightSetNum(void);
	C_LightSet* GetLightSet(int index);
	C_LightSet* GetLightSet(const char *pName);
	
	C_FogSet* GetFogSet(void);
	
	inline void SetType(u32 type){ mType = type; }
	inline u32 GetType(void){ return mType; }
	
	inline void SetID(u32 id){ mID = id; }
	inline u32 GetID(void){ return mID; }
	
	inline void SetStatic(bool flag){ mStatic = flag==true? 1:0; }
	inline bool GetStatic(void){ return mStatic? true : false; }
	
	WK7Model* CreateModel(u32 mirrorType=0);
	WK7ModelMotion* CreateModelMotion(void);
	WK7CameraMotion* CreateCameraMotion(void);
	WK7LightMotion* CreateLightMotion(void);
	WK7Path* CreatePath(u32 stgid);
	
	int GetCameraDescNum(void);
	HSD_CameraDesc* GetCameraDesc(int index);
	HSD_CameraDesc* GetCameraDesc(const char *pName);
	HSD_CameraDesc* GetCameraDescPartialName(const char *pPartialName);
	HSD_CameraDesc* GetCameraDescPartialCameraName(const char *pName);
	
	int GetCameraAnimNum(void);
	HSD_CameraAnim* GetCameraAnim(int index);
	HSD_CameraAnim* GetCameraAnim(const char *pName);
	
	WK7ModelMotion* GetModelMotion(u32 type,u32 id,WK7Archive *pArchive=NULL);
	WK7ModelMotion* GetModelMotion(const char *pName,u32 mottype=WK7MODELMOTIONTYPE_ALL,WK7Archive *pArchive=NULL);
	
	WK7CameraMotion* GetCameraMotion(u32 type,u32 id,WK7Archive *pArchive=NULL);
	WK7CameraMotion* GetCameraMotion(const char *pName,WK7Archive *pArchive=NULL);
	
	WK7LightMotion* GetLightMotion(u32 type,u32 id,WK7Archive *pArchive=NULL);
	WK7LightMotion* GetLightMotion(const char *pName,WK7Archive *pArchive=NULL);
	
#ifdef	_DEBUG
private:
	void ReviseFormat(HSD_DObjDesc *pDObjDesc);
	void ReviseFormat(HSD_Joint *pJoint);
	void ReviseFormatTerm(HSD_DObjDesc *pDObjDesc);
	void ReviseFormatTerm(HSD_Joint *pJoint);
public:
#endif
	WK7SdExpInfo* GetSdExpInfo(void);
	
	static WK7Archive* Create(int prio=GHMGCLIST_DEFAULT_PRIO);
	static bool FrameProcessAll(void);
	static bool ReleaseAll(void);
	static bool ReleaseDynamicAll(void);
	static bool ReleaseStaticAll(void);
	
	static WK7Archive* Find(u32 type,u32 id,WK7Archive *pArchive=NULL);
	static bool IsValid(WK7Archive *pArchive);
	
#ifdef	_DEBUG
	static void SetDebugPrintSec(f32 sec=4.0f){ mDebugPrintSec=sec; }
	static f32 GetDebugPrintSec(void){ return mDebugPrintSec; }
#endif
};

#ifdef	_DEBUG
#define	WK7ArchiveSetDebugPrintSec(sec)		WK7Archive::SetDebugPrintSec(sec)
#define	WK7ArchiveSetDebugPrintSecDefault()	WK7Archive::SetDebugPrintSec()
#define	WK7ArchiveGetDebugPrintSec()		WK7Archive::GetDebugPrintSec()
#else
#define	WK7ArchiveSetDebugPrintSec(sec)		
#define	WK7ArchiveSetDebugPrintSecDefault()	
#define	WK7ArchiveGetDebugPrintSec()		
#endif

#endif	//__WK7ARCHIVE_H__

