////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Camera Object
//
// WK7Cam.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 11/09/2002
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7CAM_CPP__

#include ".\WK7Cam.h"
#include ".\WK7MainTask.h"
#include ".\WK7Player.h"
#include ".\WK7Stage.h"
#include ".\WK7Archive.h"
#include ".\WK7SelectIF.h"
//#include "..\Lib\ghmGcFile.h"
#include "..\Fujita\C_Coll.h"
#include "..\Fujita\C_Font.h"
#include ".\WK7MainMenu.h"
#include "..\Matsu\MK7Noise.h"

#include "K7App.h"

#ifdef	K7APP_DEBUG
//#define	DEBUG_PRINT
#endif

#define	WK7CAM_DEFAULT_TARGET_DIST			(12.0f)
#define	WK7CAM_DEFAULT_HEIGHT				(12.0f)
#define	WK7CAM_DEFAULT_Z					(15.0f)

#define	BACKMIRROR_DIST				(20.0f)

static	f32	camdistMargin          =  0.5f;
static	f32	minAWAY_LIMITDIST      = 20.0f;
static	f32	minAPPROACH_LIMITDIST  = 15.0f;
//static	f32	maxAWAY_LIMITDIST      = 30.0f;
//static	f32	maxAPPROACH_LIMITDIST  = 25.0f;
//static	f32	minAWAY_LIMITDIST      = 20.0f;
//static	f32	minAPPROACH_LIMITDIST  = 18.0f;
//static	f32	maxAWAY_LIMITDIST      = 40.0f;
//static	f32	maxAPPROACH_LIMITDIST  = 35.0f;
static	f32	minAWAY_LIMITDIST_br      = 50.0f;
static	f32	minAPPROACH_LIMITDIST_br  = 45.0f;
//static	f32	maxAWAY_LIMITDIST_br      = 60.0f;
//static	f32	maxAPPROACH_LIMITDIST_br  = 50.0f;

HSD_WObjDesc	defpers_pos = {
	NULL,
	{ 0.0f, WK7CAM_DEFAULT_HEIGHT , WK7CAM_DEFAULT_Z },
	NULL,
};

HSD_WObjDesc	defpers_interest = {
	NULL,
	{ 0.0f, WK7CAM_DEFAULT_HEIGHT , WK7CAM_DEFAULT_Z - WK7CAM_DEFAULT_TARGET_DIST  },
	NULL,
};

HSD_CameraDescPerspective	defpers = {
	NULL,
	0,
	COBJ_PROJECTION_PERSPECTIVE,
	{ 0,640,0,480 },
	{ 0,640,0,480 },
	&defpers_pos,
	&defpers_interest,
	0.0f,
	0,
	0.1f,
	100000.0f,
	42.184677f,
	1.18f,
};

WK7Cam				*gpCamera              = NULL;
WK7Archive			*gpWobbleMotionArchive = NULL;
WK7CameraMotion		*gpWobbleMotion        = NULL;

bool WK7Cam::ReviseWobble(Vec &campos,Vec &camtgt)
{
#if 1
#pragma	unused(campos)
#pragma	unused(camtgt)
#else
	if( !mWobbleEnable ){
		return true;
	}
	
	Vec		wobble_campos,wobble_camtgt,wobble_dir,dirv,dirh,dir;
	Mtx		wobble_mtx;
	
	HSD_CObjGetEyePosition(mpCObjWobble,&wobble_campos);
	HSD_CObjGetInterest   (mpCObjWobble,&wobble_camtgt);
	
#ifdef	_DEBUG
	if(0){
		C_Paper	paper(32, 32, 0.75f);
		
		PRINTF("CamPos = %.3f,%.3f,%.3f\n",wobble_campos.x,wobble_campos.y,wobble_campos.z);
		PRINTF("CamTgt = %.3f,%.3f,%.3f\n",wobble_camtgt.x,wobble_camtgt.y,wobble_camtgt.z);
	}
#endif
	
	f32		dirrate = mWobbleDirRate.GetValue();
	VECSubtract        (&camtgt,&campos,&dir);
	VECSubtract        (&wobble_campos,&wobble_camtgt,&wobble_dir );
	wobble_dir.x *= dirrate;
	wobble_dir.y *= dirrate;
	
	VECNormalize       (&dir,&dir);
	ghmMath::Roll2UpVec(dir,0.0f,dirv);
	VECCrossProduct    (&dirv,&dir,&dirh);
	
	wobble_mtx[0][0] = dirh.x;
	wobble_mtx[1][0] = dirh.y;
	wobble_mtx[2][0] = dirh.z;
	wobble_mtx[0][1] = dirv.x;
	wobble_mtx[1][1] = dirv.y;
	wobble_mtx[2][1] = dirv.z;
	wobble_mtx[0][2] = dir .x;
	wobble_mtx[1][2] = dir .y;
	wobble_mtx[2][2] = dir .z;
	wobble_mtx[0][3] = 0.0f;
	wobble_mtx[1][3] = 0.0f;
	wobble_mtx[2][3] = 0.0f;
	
	MTXMultVecSR(wobble_mtx,&wobble_dir,&wobble_dir);
	
	VECScale    (&wobble_campos,&wobble_campos,mWobblePosRate.GetValue());
	VECAdd      (&campos,&wobble_campos,&campos);
	VECAdd      (&campos,&wobble_dir,&camtgt);
	
#endif
	return true;
}

bool WK7CamDelayInterpolateInfo::FrameProcess(WK7Cam *pCamera)
{
	HSD_CObj	*pCObj;
	f32			rate;
	
	if( pCamera==NULL || (pCObj = pCamera->_GetCObj())==NULL || (rate = mInterpolateDelayRate) >= 1.0f ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	Vec		pos,up,interest;
	f32		roll;
	f32		_rate;
	
	_rate = 1.0f - rate;
	
	roll = HSD_CObjGetRoll(pCObj);
	
	HSD_CObjGetEyePosition (pCObj,&pos);
	HSD_CObjGetInterest    (pCObj,&interest);
	
	mPrevPosition.x      = pos     .x * rate + mPrevPosition.x * _rate;
	mPrevPosition.y      = pos     .y * rate + mPrevPosition.y * _rate;
	mPrevPosition.z      = pos     .z * rate + mPrevPosition.z * _rate;
	mPrevInterest.x      = interest.x * rate + mPrevInterest.x * _rate;
	mPrevInterest.y      = interest.y * rate + mPrevInterest.y * _rate;
	mPrevInterest.z      = interest.z * rate + mPrevInterest.z * _rate;
	mPrevRoll            = roll       * rate + mPrevRoll       * _rate;
	mPrevFov             = mFov       * rate + mPrevFov        * _rate;
	
	// I`FbN
#define	POS_FLOAT_ERROR		(0.01f)
#define	FLOAT_ERROR			(0.001f)
	if( fabs(mPrevPosition.x-pos     .x) < POS_FLOAT_ERROR &&
	    fabs(mPrevPosition.y-pos     .y) < POS_FLOAT_ERROR &&
	    fabs(mPrevPosition.z-pos     .z) < POS_FLOAT_ERROR &&
	    fabs(mPrevInterest.x-interest.x) < POS_FLOAT_ERROR &&
	    fabs(mPrevInterest.y-interest.y) < POS_FLOAT_ERROR &&
	    fabs(mPrevInterest.z-interest.z) < FLOAT_ERROR &&
	    fabs(mPrevRoll      -roll      ) < FLOAT_ERROR &&
	    fabs(mPrevFov       -mFov      ) < FLOAT_ERROR ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	pos      = mPrevPosition;
	interest = mPrevInterest;
	pCamera->ReviseWobble(pos,interest);
	
	ghmMath::Roll2UpVec(pos,interest,roll,up);
	
	MTXLookAt(mInterpolateMatrix,&pos,&up,&interest);
	
	HSD_CObjSetRoll(pCObj,roll);
	
	return true;
}

bool WK7CamDelayInterpolateInfo::SetCurrent(WK7Cam *pCamera)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		return false;
	}
	HSD_CObjSetUserDefinedMtx(pCObj,mInterpolateMatrix);
	HSD_CObjSetFov           (pCObj,mPrevFov);
	return true;
}

bool WK7CamDelayInterpolateInfo::Initialize(WK7Cam *pCamera,f32 delayRate)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		mInterpolateDelayRate = 1.0f;
		return false;
	}
	if( delayRate > 1.0f ){
		delayRate = 1.0f;
	}else if( delayRate < 0.0f ){
		delayRate = 0.0f;
	}
	HSD_CObjGetEyePosition(pCObj,&mPrevPosition);
	HSD_CObjGetInterest   (pCObj,&mPrevInterest);
	mPrevRoll                = HSD_CObjGetRoll(pCObj);
	mPrevFov                 = HSD_CObjGetFov (pCObj);
	mFov                     = mPrevFov;
	mInterpolateDelayRate    = delayRate;
	HSD_CObjGetViewingMtx (pCObj,mInterpolateMatrix);
	return true;
}

bool WK7CamCircleInterpolateInfo::FrameProcess(WK7Cam *pCamera)
{
	HSD_CObj	*pCObj;
	
	if( pCamera==NULL || (pCObj = pCamera->_GetCObj())==NULL || mInterpolateTotalFrame<=0.0f ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	f32		animrate = WK7Motion::ConvMotionRate(pCamera->GetMotionRate());
	
	// ĐwbhXV
	mInterpolateCurrentFrame += animrate;
	
	// ԏI`FbN
	if( mInterpolateCurrentFrame > mInterpolateTotalFrame ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	Vec		pos,up,interest;
	f32		roll,croll;
	f32		rate,_rate;
	ghmQuaternion		dstRotation,ipRotation;
	Vec					dstDir;
	f32					dstDist,ipDist;
	Mtx					ipmtx;
static	Vec		zaxis = { 0.0f,0.0f,1.0f };
	
	rate  = mInterpolateCurrentFrame / mInterpolateTotalFrame;
	_rate = 1.0f - rate;
	
	croll = HSD_CObjGetRoll(pCObj);
	
	HSD_CObjGetEyePosition (pCObj,&pos);
	HSD_CObjGetInterest    (pCObj,&interest);
	
	VECSubtract(&pos,&interest,&dstDir);
	dstDist = GHM_VEC_GET_LEN(&dstDir);
	VECNormalize(&dstDir,&dstDir);
	GetDirCoorMatrix(dstRotation,dstDir);
	
	ipRotation.QuatSlerp(&mPrevRotation,&dstRotation,rate);
	MTXQuat     (ipmtx,&ipRotation);
	
	interest.x      = interest.x * rate + mPrevInterest.x * _rate;
	interest.y      = interest.y * rate + mPrevInterest.y * _rate;
	interest.z      = interest.z * rate + mPrevInterest.z * _rate;
	roll            = croll      * rate + mPrevRoll       * _rate;
	mInterpolateFov = mFov       * rate + mPrevFov        * _rate;
	ipDist          = dstDist    * rate + mPrevDist       * _rate;
	
	MTXMultVecSR(ipmtx,&zaxis,&pos);
	VECScale    (&pos,&pos,ipDist);
	VECAdd      (&pos,&interest,&pos);
	
	pCamera->ReviseWobble(pos,interest);
	
	ghmMath::Roll2UpVec(pos,interest,roll,up);
	
	MTXLookAt(mInterpolateMatrix,&pos,&up,&interest);
	mInterpolateInterest = interest;
	mInterpolatePosition = pos;
	
	HSD_CObjSetRoll(pCObj,croll);
	
	return true;
}

bool WK7CamCircleInterpolateInfo::SetCurrent(WK7Cam *pCamera)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		return false;
	}
	HSD_CObjSetUserDefinedMtx(pCObj,mInterpolateMatrix);
	HSD_CObjSetFov           (pCObj,mInterpolateFov);
	return true;
}

bool WK7CamCircleInterpolateInfo::GetDirCoorMatrix(ghmQuaternion &q,const Vec &dir)
{
	Mtx		mtx;
	GetDirCoorMatrix(mtx,dir);
	MatToQuat       (mtx,&q);
	return true;
}

bool WK7CamCircleInterpolateInfo::GetDirCoorMatrix(Mtx mtx,const Vec &dir)
{
	Vec				dirh,dirv;
	
	ghmMath::Roll2UpVec(dir,0.0f,dirv);
	VECCrossProduct(&dirv,&dir,&dirh);
	VECNormalize   (&dirh,&dirh);
	
	mtx[0][0] = dirh.x;
	mtx[1][0] = dirh.y;
	mtx[2][0] = dirh.z;
	mtx[0][1] = dirv.x;
	mtx[1][1] = dirv.y;
	mtx[2][1] = dirv.z;
	mtx[0][2] = dir .x;
	mtx[1][2] = dir .y;
	mtx[2][2] = dir .z;
	mtx[0][3] = 0.0f;
	mtx[1][3] = 0.0f;
	mtx[2][3] = 0.0f;
	
	return true;
}

bool WK7CamCircleInterpolateInfo::Initialize(WK7Cam *pCamera,f32 totalFrame)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		mInterpolateTotalFrame   = 0.0f;
		mInterpolateCurrentFrame = 0.0f;
		return false;
	}
	
	Vec		dir;
	
	HSD_CObjGetEyePosition(pCObj,&mPrevPosition);
	HSD_CObjGetInterest   (pCObj,&mPrevInterest);
	mPrevRoll                = HSD_CObjGetRoll(pCObj);
	mPrevFov                 = HSD_CObjGetFov (pCObj);
	mFov                     = mPrevFov;
	mInterpolateFov          = mPrevFov;
	mInterpolateTotalFrame   = totalFrame;
	mInterpolateCurrentFrame = 0.0f;
	HSD_CObjGetViewingMtx (pCObj,mInterpolateMatrix);
	mInterpolateInterest = mPrevInterest;
	mInterpolatePosition = mPrevPosition;
	
	VECSubtract    (&mPrevPosition,&mPrevInterest,&dir);
	mPrevDist = GHM_VEC_GET_LEN(&dir);
	VECNormalize    (&dir,&dir);
	GetDirCoorMatrix(mPrevRotation,dir);
	
	return true;
}

WK7CamPathFollowReverseInfo::WK7CamPathFollowReverseInfo(WK7PathControl *pCurrPathControl)
{
	mPathControl.SetActive            (true);
	mPathControl.SetInterpolateDisable(true);
	mPathControl.SetControlObj        (this);
	mPathControl.SetPathEnableEx      (true);
	if( (mpTargetPathControl = pCurrPathControl)!=NULL ){
		WK7PathInfo		*pCurrInfo = mpTargetPathControl->GetCurrentPathInfo();
		WK7PathInfo		*pRevInfo  = mPathControl.GetCurrentPathInfo();
		SetControlPath(mpTargetPathControl->GetControlPath());
		*pRevInfo = *pCurrInfo;
	}
	mpBranchList = NULL;
	mPrevDistance = -1.0f;
}

WK7CamPathFollowReverseInfo::~WK7CamPathFollowReverseInfo()
{
	GHM_SAFE_DELETE(mpBranchList);
}

bool WK7CamPathFollowReverseInfo::SetControlPath(WK7Path *pPath)
{
	mPathControl.SetControlPath(pPath);
	return true;
}

void WK7CamPathFollowReverseInfo::GetCurrentPosition(Vec *ppos)
{
	mPathControl.GetCurrentPosition(ppos);
}

bool WK7CamPathFollowReverseInfo::FrameProcess(void)
{
	if( mpTargetPathControl==NULL ){
		return false;
	}
	WK7PathInfo			*pTargetPathInfo  = mpTargetPathControl->GetCurrentPathInfo();
	WK7PathInfo			*pCurrentPathInfo = mPathControl.GetCurrentPathInfo();
	GHM_SAFE_DELETE(mpBranchList);
	mpBranchList = pCurrentPathInfo->GetBranchList(pTargetPathInfo);
	if( mpBranchList==NULL ){
		return false;
	}
#define	REVIP_SPEED		(2.0f)
	f32		speed = REVIP_SPEED;
	if( mPrevDistance >= 0.0f ){
		speed += mpBranchList->GetDistance() - mPrevDistance;
	}
	if( speed < REVIP_SPEED ){
		speed = REVIP_SPEED;
	}
	if( mpBranchList->GetDistance() < speed ){
		return false;
	}
	if( mpBranchList->GetDir() != pCurrentPathInfo->GetDir() ){
		mPathControl.Reverse();
	}
	mPathControl.Move(speed);
	mPathControl.FrameProcess();
	
	mPrevDistance = mpBranchList->GetDistance() - speed;
	
	return true;
}

WK7PathControlObj::CallbackResult WK7CamPathFollowReverseInfo::BranchPointArriveCallback(WK7PathControl *pPathControl,WK7PathInfo *pInfo,WK7PathBranchPoint *pBranchPoint)
{
	if( pBranchPoint==NULL ){
		return WK7PathControlObj::RESULT_CANCEL;
	}
	if( pInfo->GetRestDist() > 0.0f || mpBranchList==NULL ){
		return WK7PathControlObj::RESULT_OK;
	}
	WK7PathBranchList	*pNext = mpBranchList->GetChild();
	if( pNext==NULL ){
		return WK7PathControlObj::RESULT_OK;
	}
	WK7PathBranch		*pNextBranch = pNext->GetBranch();
	
	pPathControl->SetNextBranch(pNextBranch);
	mpBranchList->SetChild(NULL);
	GHM_SAFE_DELETE(mpBranchList);
	mpBranchList = pNext;
	
	return WK7PathControlObj::RESULT_OK;
}

void WK7CamPathFollowInfo::SetBackMirrorMode(bool flag)
{
	if( flag==mBackMirrorMode ){
		return;
	}
	WK7PathInfo		*pInfo = mPathControl.GetCurrentPathInfo();
	if( flag==true ){
		WK7PathBranchList	*pLast;
		if( mpBranchList==NULL ||
		    (pLast=mpBranchList->GetLast())==NULL ){
			return;
		}
		WK7PathNurbs		*pPath;
		f32					param;
		u32					dir;
		WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
		WK7PathInfo			*pTargetPathInfo    = pTargetPathControl->GetCurrentPathInfo();
		pPath = pTargetPathInfo->GetPath(BACKMIRROR_DIST,(int)pLast->GetDir(),&param,&dir,true);
		if( pPath==NULL ){
			return;
		}
		mOldPathInfo = *pInfo;
		mPathControl.SetCurrentPath(pPath,param,!dir);
	}else{
		*pInfo = mOldPathInfo;
	}
	mBackMirrorMode = flag;
	ResetCamPos();
}

bool WK7CamPathFollowInfo::InitEscape(void)
{
	if( mEscapeMode==true ){
		return true;
	}
	
	WK7PathInfo			*pInfo = mPathControl.GetCurrentPathInfo();
	Vec					dir;
	WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
	WK7PathInfo			*pTargetPathInfo    = pTargetPathControl->GetCurrentPathInfo();
	WK7PathNurbs		*pPath;
	
	pPath = pInfo->GetPath();
	pPath->GetDiffVec(dir,pInfo->GetParam());
//	pInfo->GetDeltaPosition(&dir);
//	pInfo->GetEndDeltaPosition(&dir);
	
#define	SLIDE_MAX		(10.0f)
#define	SLIDE_MIN		( 5.0f)
	int		side;
	f32		slide;
	
	slide = SLIDE_MIN + HSD_Randf() * (SLIDE_MAX - SLIDE_MIN);
	switch( mPosHIndex ){
	case	POSHR:
		side  = 1;
		break;
	case	POSHL:
	default:
		side  = 0;
		break;
	}
	
	if( side ){
		slide = -slide;
	}
	
	mEscapeMode = true;
	mEscapeDirDist = 0.0f;
	dir.y =  dir.x;
	dir.x =  dir.z;
	dir.z = -dir.y;
	dir.y =  0.0f;
	VECNormalize(&dir,&dir);
	mEscapeDir.x = slide * dir.x;
	mEscapeDir.y = 0.0f;//sqrtf(mEscapeDist * mEscapeDist - slide * slide);
	mEscapeDir.z = slide * dir.z;
	VECNormalize(&mEscapeDir,&mEscapeDir);
	mEscapePosition.x = 
	mEscapePosition.y = 
	mEscapePosition.z = 0.0f;
	
	return true;
}

WK7PathControlObj::CallbackResult WK7CamPathFollowInfo::PathChangeCallback(WK7PathControl *pPathControl,WK7PathInfo *pInfo,WK7PathBranch *pSelectBranch,bool interpolate,int inv)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
//#pragma unused(pSelectBranch)
#pragma unused(interpolate)
#pragma unused(inv)
	if( pSelectBranch==NULL ){
		mOldDir = -1;
	}else{
		mOldDir = (int)pSelectBranch->GetDir();
	}
	return WK7PathControlObj::RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7CamPathFollowInfo::BranchPointArriveCallback(WK7PathControl *pPathControl,WK7PathInfo *pInfo,WK7PathBranchPoint *pBranchPoint)
{
	if( pBranchPoint==NULL ){
		return WK7PathControlObj::RESULT_CANCEL;
	}
	Vec				dir;
	WK7PathBranch	*pNextBranch;
	
	if( mDir==false ){
		pInfo->GetEndDeltaPosition(&dir);
		VECNormalize(&dir,&dir);
		pNextBranch = pBranchPoint->SelectNextDefaultBranch(&dir,pInfo->GetPath(),true);
		pPathControl->SetNextBranch(pNextBranch);
		if( pInfo->GetRestDist() <= 0.0f ){
			if( pNextBranch==NULL ){
				InitEscape();
			}
		}
	}else{
		if( pInfo->GetRestDist() <= 0.0f && mpBranchList != NULL ){
			WK7PathBranchList	*pNext = mpBranchList->GetChild();
			if( pNext!=NULL ){
				pNextBranch = pNext->GetBranch();
				pPathControl->SetNextBranch(pNextBranch);
				mpBranchList->SetChild(NULL);
				GHM_SAFE_DELETE(mpBranchList);
				mpBranchList = pNext;
			}
		}
	}
	
	return WK7PathControlObj::RESULT_OK;
}

WK7CamPathFollowInfo::WK7CamPathFollowInfo()
{
	mPosition.x = 0.0f;
	mPosition.y = 0.0f;
	mPosition.z = 0.0f;
	mTargetPosition.x = 0.0f;
	mTargetPosition.y = 0.0f;
	mTargetPosition.z = 1.0f;
	mTargetFixPosition.x = 0.0f;
	mTargetFixPosition.y = 0.0f;
	mTargetFixPosition.z = 1.0f;
	mTipReverseProc   = 0;
	mTipReverseCancel = false;
	memset(&mTipReversePosition,0,sizeof(mTipReversePosition));
	mStageStartMode                = false;
	mStageStartCamPosition      .x = 
	mStageStartCamPosition      .y = 
	mStageStartCamPosition      .z = 0.0f;
	mStageStartCamTargetPosition.x = 
	mStageStartCamTargetPosition.y = 0.0f;
	mStageStartCamTargetPosition.z = 1.0f;
	mStageStartTargetInitPosition.x = 
	mStageStartTargetInitPosition.y = 
	mStageStartTargetInitPosition.z = 0.0f;
	mDir           = true;
	mMoveDist      = 0.0f;
	mpBranchList   = NULL;
	mCamPosReset   = 1;
	mEscapeMode    = false;
//	mEscapeDist    = 10.0f;
//	mEscapeDist    = 25.0f;
//	mEscapeDist    = minAPPROACH_LIMITDIST * 0.9f;
	mEscapeDist    = 1.0f;
	mEscapeDirDist = 0.0f;
	mEscapePosition.x = 
	mEscapePosition.y = 
	mEscapePosition.z = 0.0f;
	mPathControl.SetActive            (true);
//	mPathControl.SetInterpolateOperate(true);
	mPathControl.SetInterpolateDisable(true);
	mPathControl.SetControlObj        (this);
	mPathControl.SetPathEnableEx      (true);
	
	mCamHeightCurrent =
	mCamHeightNormal  = 3.0f;
	mCamHeightBp      = 20.0f;
	
	mTargetHeightCurrent = 
	mTargetHeightNormal  = 12.0f;
//	mTargetHeightNormal  = 10.0f;
//	mTargetHeightNormal  = 15.0f;
	mTargetHeightBp      = 5.0f;
	
	mBackMirrorMode = false;
	mpReverseInfo = NULL;
	
	mBranchPointMode = false;
	mApproachLimitDist.SetValue(minAPPROACH_LIMITDIST);
	mAwayLimitDist    .SetValue(minAWAY_LIMITDIST    );
	mAdjustDist       .SetValue(0.0f);
	
	mpDesc = NULL;
	mMode  = FOLLOW_MODE;
	
	mOldDir = -1;
	
	mpBpCamInfo = NULL;
	GHM_VEC_SET_XYZ(&mBpPosition,0.0f,0.0f,0.0f);
	
	SetBpCamInfoPrio();
	
	mRailType = RAILTYPE_TOPLAYER;
	
	int		i;
	for(i=0 ; i<2 ; i++){
		mRailPosition[i].x = 
		mRailPosition[i].y = 
		mRailPosition[i].z = 0.0f;
	}
	SetPos(POSRD);
//	SetPos(POSCD);
}

WK7CamPathFollowInfo::~WK7CamPathFollowInfo()
{
	GHM_SAFE_DELETE(mpBranchList);
	GHM_SAFE_DELETE(mpReverseInfo);
}

void WK7CamPathFollowInfo::SetBpCamInfoPrio(int prio)
{
	if( prio < 0 ){
		prio = 0;
	}
	mBpCamInfoPrio = prio;
}

void WK7CamPathFollowInfo::SetBpCamInfo(WK7StageBpCamInfo *pInfo)
{
	mpBpCamInfo  = pInfo;
	mCamPosReset = 1;
}

bool WK7CamPathFollowInfo::ReversePosLR(void)
{
	switch( mPosHIndex ){
	case	POSHR: mPosHIndex = POSHL; break;
	case	POSHL: 
	default:       mPosHIndex = POSHR; break;
	}
	SetPos(WK7CamPathFollowInfo::RENEW_IM);
	return true;
}

bool WK7CamPathFollowInfo::SetPosV(int pos)
{
	switch( pos ){
	case	POSVD:
	case	POSVC:
	case	POSVU:
		break;
	default:
		return false;
	}
	mPosVIndex = pos;
	RenewPos();
	return true;
}
bool WK7CamPathFollowInfo::SetPosH(int pos)
{
	switch( pos ){
	case	POSHL:
	case	POSHC:
	case	POSHR:
		break;
	default:
		return false;
	}
	mPosHIndex = pos;
	RenewPos();
	return true;
}
#define	POSW			( 3.5f)
#define	POSD			( 0.0f)
#define	POSC			( 8.0f)
#define	POSU			(15.0f)
//#define	POSC			( 3.0f)
//#define	POSU			( 8.0f)
#define	POSID_FD		(0)
#define	POSID_PH		(1)
#define	POSID_PV		(2)
#define	POSID_TF		(3)
#define	POSID_TH		(4)
#define	POSID_TV		(5)

#define	WK7CAMSET_BEGIN()					static WK7CamSet	*camSet[] = {
#define	WK7CAMSET(setid)						camSet##setid,
#define	WK7CAMSET_END()						NULL, };

////////////////////////////////////////////////////////////////////////////////
// JZbg
#include	"CamSet.h"
// JZbg
////////////////////////////////////////////////////////////////////////////////
//WK7CAMSET_DECL_BEGIN(0)
//	WK7CAMSET_DECL_CHR_BEGIN(K7GAL)
//						//      fd        ph        pv               tf         th              tv
//		WK7CAMSET_DECL_CHR( 15.000f , -2.862f ,  3.0f+15.064f ,  10.000f ,   0.000f ,  12.0f+0.000f )	// O
//		WK7CAMSET_DECL_CHR( 15.000f ,  2.862f ,  3.0f+15.064f ,  10.000f ,   0.000f ,  12.0f+0.000f )	// EO
//		WK7CAMSET_DECL_CHR(  7.000f , -2.538f ,  3.0f+15.641f ,  10.000f ,   0.000f ,  12.0f+0.000f )	// 
//		WK7CAMSET_DECL_CHR(  7.000f ,  2.538f ,  3.0f+15.641f ,  10.000f ,   0.000f ,  12.0f+0.000f )	// E
//	WK7CAMSET_DECL_CHR_END()
//WK7CAMSET_DECL_END()
//
//WK7CAMSET_BEGIN()
//	WK7CAMSET(0)
//WK7CAMSET_END()

WK7CamSet* WK7CamPathFollowInfo::GetCamSet(u32 chrid,int setid)
{
	WK7CamSet	*pCamSet;
	int			i;
	for(i=0 ; camSet[i]!=NULL ; i++){
		if( i != setid ){
			continue;
		}
		for(pCamSet=camSet[i] ; pCamSet->mID != 0xffffffff ; pCamSet++){
			if( pCamSet->mID==chrid ){
				return pCamSet;
			}
		}
		return camSet[i];
	}
	return camSet[0];
}

//#define	WK7CAM_RENEW_TICK	(WK7_SEC_TO_TICK(0.125f))
//u32		renewTick = WK7CAM_RENEW_TICK;
bool WK7CamPathFollowInfo::RenewPos(u32 renewTick)
{
#ifdef	_DEBUG
	if( mainMenu.GetFollowCamEdit()==true ){
		return true;
	}
#endif
	if( renewTick==0 ){
		mMoveDist = 0.0f;
	}
	WK7PathInfo			*pTargetPathInfo;
	WK7PathInfo			*pCurrentPathInfo = mPathControl.GetCurrentPathInfo();
	if( WK7PLAYERPATHCONTROL->GetInterpolateState()==WK7PathControl::PCI_ACTIVE ){
		pTargetPathInfo = WK7PLAYERPATHCONTROL->GetNextPathInfo();
	}else{
		pTargetPathInfo = WK7PLAYERPATHCONTROL->GetCurrentPathInfo();
	}
	GHM_SAFE_DELETE(mpBranchList);
	mpBranchList = pCurrentPathInfo->GetBranchList(pTargetPathInfo);
	if( mpBranchList==NULL ){
		return true;
	}
	WK7PathBranchList	*pLast = mpBranchList->GetLast();
	if( pLast!=NULL ){
		int		index;
		u32		tdir = pTargetPathInfo->GetDir();
		switch( mPosHIndex ){
		case	POSHR: index = 1; break;
		case	POSHL: 
		default:       index = 0; break;
//		case	POSHL: index = 0; break;
//		case	POSHR: 
//		default:       index = 1; break;
		}
		if( WK7PLAYERCONTROL->GetState()==PC_STATE_TURN ){
//		if( WK7PLAYERPATHCONTROL->GetReverseReq()==true ){
			tdir ^= 1;
		}
		if( pLast->GetDir()==tdir ){
			// J
			index += 2;
		}
		
		WK7Stage			*pCurrentStage;
		WK7StageInfo		*pStageInfo;
		int					camSetID = -1;
		WK7CamSet			*pCamSet = NULL;
		
		if( (pCurrentStage=WK7Stage::GetCurrentStage())!=NULL && (pCamSet = pCurrentStage->GetCamSet(WK7PLAYERCONTROL->GetMainID()))==NULL ){
			if( (camSetID=pCurrentStage->GetCamSetID()) < 0 ){
				if( (pStageInfo=WK7Stage::GetStageInfo(pCurrentStage->GetID()))!=NULL ){
					camSetID = pStageInfo->mDefaultCamSet;
				}
			}
		}
		if( pCamSet==NULL ){
			pCamSet = GetCamSet(WK7PLAYERCONTROL->GetMainID(),camSetID);
		}
		
		WK7CamSetParam	*pCamSetParam = pCamSet->mParam + index;
		
		if( index & 1 ){
			mPosH.Play(-fabs(pCamSetParam->mPh),renewTick);
			mTgtH.Play(-fabs(pCamSetParam->mTh),renewTick);
		}else{
			mPosH.Play( fabs(pCamSetParam->mPh),renewTick);
			mTgtH.Play( fabs(pCamSetParam->mTh),renewTick);
		}
		mTgtF.Play(pCamSetParam->mTf,renewTick);
		f32		approachDist,awayDist;
		minAPPROACH_LIMITDIST  = pCamSetParam->mFd; 
		minAWAY_LIMITDIST      = minAPPROACH_LIMITDIST + camdistMargin;
		if( mBranchPointMode==true ){
			approachDist = minAPPROACH_LIMITDIST_br;
			awayDist     = minAWAY_LIMITDIST_br;
			mPosV.Play(20.0f,renewTick);
			mTgtV.Play( 5.0f,renewTick);
		}else{
			approachDist = minAPPROACH_LIMITDIST;
			awayDist     = minAWAY_LIMITDIST;
			mPosV.Play(pCamSetParam->mPv,renewTick);
			mTgtV.Play(pCamSetParam->mTv,renewTick);
		}
		mApproachLimitDist.Play(approachDist,renewTick);
		mAwayLimitDist    .Play(awayDist    ,renewTick);
		
//		mOldDir = -1;
	}
	return true;
}
bool WK7CamPathFollowInfo::SetPos(int pos)
{
	switch( pos ){
	case	POSLD: SetPosV(POSVD); SetPosH(POSHL); break;
	case	POSLC: SetPosV(POSVC); SetPosH(POSHL); break;
	case	POSLU: SetPosV(POSVU); SetPosH(POSHL); break;
	case	POSCD: SetPosV(POSVD); SetPosH(POSHC); break;
	case	POSCC: SetPosV(POSVC); SetPosH(POSHC); break;
	case	POSCU: SetPosV(POSVU); SetPosH(POSHC); break;
	case	POSRD: SetPosV(POSVD); SetPosH(POSHR); break;
	case	POSRC: SetPosV(POSVC); SetPosH(POSHR); break;
	case	POSRU: SetPosV(POSVU); SetPosH(POSHR); break;
		break;
	case	RENEW:
		RenewPos();
		break;
	case	RENEW_IM:
		RenewPos(0);
		break;
	case	RENEW_SLOW:
		RenewPos(WK7_SEC_TO_TICK(1.0f));
		break;
	default:
		return false;
	}
	return true;
}

bool WK7CamPathFollowInfo::Reverse(void)
{
	WK7PathBranchList	*pLast;
	if( mpBranchList==NULL || (pLast=mpBranchList->GetLast())==NULL ){
		return false;
	}
	if( mpReverseInfo==NULL ){
		ghmGcMemSetAllocName("WCPFRI");
		if( (mpReverseInfo = new WK7CamPathFollowReverseInfo(&mPathControl))==NULL ){
			return false;
		}
		InitEscape();
	}
	WK7PathNurbs		*pPath;
	f32					param;
	u32					dir;
	WK7PathInfo			*pTargetPathInfo;
	if( WK7PLAYERPATHCONTROL->GetInterpolateState()==WK7PathControl::PCI_ACTIVE ){
		pTargetPathInfo = WK7PLAYERPATHCONTROL->GetNextPathInfo();
	}else{
		pTargetPathInfo = WK7PLAYERPATHCONTROL->GetCurrentPathInfo();
	}
	pPath = pTargetPathInfo->GetPath(mApproachLimitDist.GetValue(),(int)pLast->GetDir(),&param,&dir,true);
	if( pPath==NULL ){
		GHM_SAFE_DELETE(mpReverseInfo);
		return false;
	}
	if( pPath==pTargetPathInfo->GetPath() && GHM_FABS(param - pTargetPathInfo->GetParam()) < 0.0001f ){
		GHM_SAFE_DELETE(mpReverseInfo);
		return false;
	}
	mOldDir = -1;
	mPathControl.SetCurrentPath(pPath,param,!dir);
//	//041122
//	GHM_SAFE_DELETE(mpReverseInfo);
//	SetBranchPointMode(GetBranchPointMode(),true);
//	ResetCamPos();
//	return true;
//	//041122
	RenewPos();
	return true;
}

void WK7CamPathFollowInfo::SetBranchPointMode(bool flag,bool im)
{
	f32	approachdist,awaydist;
	if( (mBranchPointMode = flag)==false ){
		approachdist = minAPPROACH_LIMITDIST;
		awaydist     = minAWAY_LIMITDIST;
	}else{
		approachdist = minAPPROACH_LIMITDIST_br;
		awaydist     = minAWAY_LIMITDIST_br;
	}
	if( im==true ){
		RenewPos(0);
		if( flag==true ){
			mCamHeightCurrent    = mCamHeightBp;
			mTargetHeightCurrent = mTargetHeightBp;
		}else{
			mCamHeightCurrent    = mCamHeightNormal;
			mTargetHeightCurrent = mTargetHeightNormal;
		}
	}else{
		RenewPos(WK7_SEC_TO_TICK(1.0f));
	}
//	if( im==true ){
//		mApproachLimitDist.Play(approachdist,0);
//		mAwayLimitDist    .Play(awaydist    ,0);
//		if( flag==true ){
//			mCamHeightCurrent    = mCamHeightBp;
//			mTargetHeightCurrent = mTargetHeightBp;
//		}else{
//			mCamHeightCurrent    = mCamHeightNormal;
//			mTargetHeightCurrent = mTargetHeightNormal;
//		}
//	}else{
//		mApproachLimitDist.Play(approachdist,0x1200);
//		mAwayLimitDist    .Play(awaydist    ,0x1200);
//	}
}

void WK7CamPathFollowInfo::SetAdjustDist(f32 dist,u32 tick)
{
	if( tick==0xffffffff ){
		tick = WK7_SEC_TO_TICK(1.0f);
	}
	mAdjustDist.SetValue(dist,tick);
}

bool WK7CamPathFollowInfo::Move(f32 dist)
{
	
	if( mBackMirrorMode==true ){
		// obN~[͓Ȃ
		return true;
	}
	
	WK7PathInfo			*pCurrentPathInfo = mPathControl.GetCurrentPathInfo();
	f32					away_limitdist,approach_limitdist,adjust_dist;
	
	adjust_dist        = mAdjustDist       .GetValue();
	away_limitdist     = mAwayLimitDist    .GetValue() + adjust_dist;
	approach_limitdist = mApproachLimitDist.GetValue() + adjust_dist;
	if( dist > away_limitdist ){
		mMoveDist = away_limitdist - dist;
		mDir = true;
	}else if( dist < approach_limitdist ){
		mMoveDist = approach_limitdist - dist;
		mDir = false;
	}else{
		mMoveDist = mMoveDist * (WK7_FRAME_RATE / 60.0f);
		if( dist + mMoveDist > away_limitdist ){
			mMoveDist = 0.0f;
		}else if( dist + mMoveDist < approach_limitdist ){
			mMoveDist = 0.0f;
		}
	}
	
#if 0
static bool dbgprint = true;
	if( dbgprint==true ){
		C_Paper			paper(320,0,0.75f);
		PRINTF("\n\n");
		PRINTF("Velocity = %.3f[u/frame]\n",mMoveDist);
	}
#endif
	
	// ܏H̔]
	if( /*mEscapeMode==true && */dist==0.0f ){
		WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
		WK7PathInfo			*pTargetPathInfo    = pTargetPathControl->GetCurrentPathInfo();
		if( pTargetPathInfo->GetPath ()==pCurrentPathInfo->GetPath () &&
		    GHM_FABS(pTargetPathInfo->GetParam()-pCurrentPathInfo->GetParam()) < 0.00001f ){
			mMoveDist = 1.0f;
			mPathControl.Reverse();
		}
//		mTipReverseProc = 1;
//		mTipReverseCancel = false;
//		mPathControl.GetCurrentPosition(&mTipReversePosition);
	}
	
	if( mMoveDist < 0.0f ){
		if( mpBranchList->GetDir() != pCurrentPathInfo->GetDir() ){
			mPathControl.Reverse();
		}
		mPathControl.Move(-mMoveDist);
	}else{
		if( dist >= 0.0f ){
			if( mpBranchList->GetDir() == pCurrentPathInfo->GetDir() ){
				mPathControl.Reverse();
			}
		}else{
			if( mpBranchList->GetDir() != pCurrentPathInfo->GetDir() ){
				mPathControl.Reverse();
			}
		}
		mPathControl.Move( mMoveDist);
	}
	
	return true;
}

bool WK7CamPathFollowInfo::GetNextPath(void)
{
	
	WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
	WK7PathInfo			*pCurrentPathInfo = mPathControl.GetCurrentPathInfo();
	WK7PathInfo			*pTargetPathInfo;
	WK7PathControl::WK7PathControlInterpolateState	ipstate;
	
	if( (ipstate = pTargetPathControl->GetInterpolateState())==WK7PathControl::PCI_ACTIVE ){
		pTargetPathInfo = pTargetPathControl->GetNextPathInfo();
	}else{
		pTargetPathInfo = pTargetPathControl->GetCurrentPathInfo();
	}
	
	GHM_SAFE_DELETE(mpBranchList);
	
	mpBranchList = pCurrentPathInfo->GetBranchList(pTargetPathInfo);
	
	if( mpBranchList != NULL ){
		
#if 0
		WK7PathBranchList	*pLast = mpBranchList->GetLast();
		
		if( pLast!=NULL ){
			C_Paper			paper(32,32,0.75f);
			if( pLast->GetDir()==pTargetPathInfo->GetDir() ){
				// J
				PRINTF("Cam Backward\n");
			}else{
				// OJ
				PRINTF("Cam Forward\n");
			}
		}
#endif
		
		f32		dist;
		
		dist = mpBranchList->GetDistance();
		
		if( ipstate==WK7PathControl::PCI_ACTIVE ){
			
			WK7PathInfo		*pIpPathInfo;
			
			pIpPathInfo = pTargetPathControl->GetInterpolatePathInfo();
			
			dist -= pIpPathInfo->GetRestDistance();
			
		}
		
//		{
//			C_Paper			paper(0,0,0.75f);
//			
//			PRINTF("\n\n\n\n\n\n\n\n\n\n\n\n");
//			PRINTF(    "   P Distance = %.3f\n",dist);
//			if( ipstate==WK7PathControl::PCI_ACTIVE ){
//				WK7PathInfo		*pIpPathInfo;
//				pIpPathInfo = pTargetPathControl->GetInterpolatePathInfo();
//				PRINTF(    "   I Distance = %.3f\n",pIpPathInfo->GetRestDistance());
//			}
//		}
		
		if( mOldDir >= 0 && mOldDir != mpBranchList->GetDir() ){
			dist = -dist;
		}
		
		Move(dist);
		
//#ifdef	DEBUG_PRINT
#if 0
static bool	dbgprint = true;
		if( dbgprint==true ){
			C_Paper			paper(0,0,0.75f);
			WK7PathNurbs	*pPath     = mpBranchList->GetPath();
			const char		*pPathName = NULL;
			
			if( pPath!=NULL ){
				pPathName = pPath->GetName();
			}
			if( pPathName==NULL ){
				pPathName = "---";
			}
			
			PRINTF("\n\n\n\n\n\n\n\n\n\n\n\n");
			PRINTF(    "   Path       = %s\n",pPathName);
			PRINTF(    "   P Distance = %.3f\n",mpBranchList->GetDistance());
			PRINTF(    "   I Distance = %.3f\n",ipdist);
			PRINTF(    "   T Distance = %.3f\n",dist);
			PRINTF(    "   Move Speed = %.3f\n",mMoveDist);
			PRINTF(    "   relpos     = %d\n",relpos);
		}
#endif
		
	}
	
	return true;
}

bool WK7CamPathFollowInfo::SetControlPath(WK7Path *pPath)
{
	mPathControl.SetControlPath(pPath);
	if( pPath==NULL ){
		GHM_SAFE_DELETE(mpBranchList);
	}
	GHM_SAFE_DELETE(mpReverseInfo);
//	if( mpReverseInfo!=NULL ){
//		mpReverseInfo->SetControlPath(pPath);
//	}
	ResetCamPos();
	mpDesc = NULL;
	return true;
}

// dir =                              0 : pXis
//     =                              1 : pXt
//     = WK7PATHINFO_INITDIR_TGTDIR     : LN^̑O
//     = WK7PATHINFO_INITDIR_TGTDIR_REV : LN^̌
bool WK7CamPathFollowInfo::SetCurrentPath(int initdir)
{
	WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
	
	SetTipReverse(false);
	
	mOldDir = -1;
	
	if( pTargetPathControl==NULL ){
		mPathControl.SetCurrentPath();
		return false;
	}
	WK7PathInfo			*pTargetPathInfo    = pTargetPathControl->GetCurrentPathInfo();
	if( pTargetPathInfo==NULL ){
		mPathControl.SetCurrentPath();
		return false;
	}
	
	f32				param;
	u32				dir;
	WK7PathNurbs	*pPath;
	
	pPath = pTargetPathInfo->GetPath(minAWAY_LIMITDIST,initdir,&param,&dir,true);
	
//	if( pPath==NULL ){
//		mPathControl.SetCurrentPath(pTargetPathInfo->GetPath(),
//		                            pTargetPathInfo->GetParam(),
//		                            pTargetPathInfo->GetDir());
//	}else{
		
		if( pPath!=NULL && pPath->GetEnableEx()==true && pPath->GetStageJump()==false ){
			mPathControl.SetCurrentPath(pPath,param,!dir);
		}else{
			switch( initdir ){
			case	0:
			case	1: initdir ^= 1; break;
			case	WK7PATHINFO_INITDIR_TGTDIR:     initdir=WK7PATHINFO_INITDIR_TGTDIR_REV; break;
			case	WK7PATHINFO_INITDIR_TGTDIR_REV: initdir=WK7PATHINFO_INITDIR_TGTDIR;     break;
			default:
				initdir = 2;
				break;
			}
			
			if( initdir!=2 ){
				pPath = pTargetPathInfo->GetPath(minAWAY_LIMITDIST,initdir,&param,&dir,true);
			}
			
			if( pPath!=NULL ){
				mPathControl.SetCurrentPath(pPath,param,!dir);
			}else{
				mPathControl.SetCurrentPath(pTargetPathInfo->GetPath(),
				                            pTargetPathInfo->GetParam(),
				                            pTargetPathInfo->GetDir());
			}
			
		}
//	}
	
	mPathControl.Move(0.0f);
	
	ResetCamPos();
	
	return true;
}

bool WK7CamPathFollowInfo::SetCurrentPath(WK7PathNurbs *pPath,f32 param,u32 dir)
{
	SetTipReverse(false);
	if( mPathControl.SetCurrentPath(pPath,param,dir)==false ){
		return false;
	}
	ResetCamPos();
	return true;
}

void WK7CamPathFollowInfo::SetStageStartMode(bool flag)
{
	if( mStageStartMode==true ){
		mCamPosReset = 1;
	}
	mStageStartMode = flag;
}

void WK7CamPathFollowInfo::SetStageStartCamPosition(const Vec &pos)
{
	mStageStartCamPosition = pos;
}

void WK7CamPathFollowInfo::SetStageStartCamTargetPosition(const Vec &pos)
{
	mStageStartCamTargetPosition = pos;
}

void WK7CamPathFollowInfo::SetStageStartTargetInitPosition(const Vec &pos)
{
	mStageStartTargetInitPosition = pos;
}
	
static	f32		STAGESTART_CHANEDIST = (15.0f);
static	f32		current_dist = (15.0f);
bool WK7CamPathFollowInfo::SetStageStartDist(f32 dist)
{
	if( dist < 0.0f ){
		current_dist = STAGESTART_CHANEDIST;
		return true;
	}
	current_dist = dist;
	return true;
}

f32 WK7CamPathFollowInfo::GetStageStartDist(void)
{
	return current_dist;
}

bool WK7CamPathFollowInfo::FrameProcess(WK7Cam *pCamera)
{
	
	WK7PathControl		*pTargetPathControl = WK7PLAYERCONTROL->GetPathControl();
	WK7PathInfo			*pCurrentPathInfo   = mPathControl.GetCurrentPathInfo();
	WK7PathInfo			*pTargetPathInfo;
	Vec					pos,tgt,epos,plpos,campos;
	
	pTargetPathControl->GetCurrentPosition(&plpos);
	tgt = plpos;
	
	switch( mTipReverseProc ){
	case	1:
		if( GHM_VEC_DIST(&mTipReversePosition,&plpos) > 20.0f ){
			mTipReverseProc = 0;
			GHM_SAFE_DELETE(mpBranchList);
			pTargetPathInfo = pTargetPathControl->GetCurrentPathInfo();
			if( (mpBranchList = pCurrentPathInfo->GetBranchList(pTargetPathInfo)) != NULL ){
				WK7PathBranchList	*pLast = mpBranchList->GetLast();
				if( pLast!=NULL && pLast->GetDir()!=pTargetPathInfo->GetDir() ){
//					mPathControl.Reverse();
					Reverse();
				}
			}
//			Reverse();
		}
		break;
	default:
		break;
	}
	
	mApproachLimitDist.FrameProcess();
	mAwayLimitDist    .FrameProcess();
	mAdjustDist       .FrameProcess();
	
	GetNextPath();
	
	mPathControl.FrameProcess();
	
	if( pTargetPathControl->GetInterpolateState()==WK7PathControl::PCI_ACTIVE ){
		pTargetPathInfo = pTargetPathControl->GetNextPathInfo();
	}else{
		pTargetPathInfo = pTargetPathControl->GetCurrentPathInfo();
	}
	
	GHM_SAFE_DELETE(mpBranchList);
	if( (mpBranchList = pCurrentPathInfo->GetBranchList(pTargetPathInfo)) != NULL ){
		mOldDir = (int)mpBranchList->GetDir();
	}else{
		mOldDir = -1;
	}
	
static	bool	revinfo = false;
	if( mpReverseInfo!=NULL ){
		revinfo = true;
		if( mpReverseInfo->FrameProcess()==false ){
			GHM_SAFE_DELETE(mpReverseInfo);
			SetBranchPointMode(GetBranchPointMode());
		}else{
			mEscapeMode = true;
		}
	}
	
	mPathControl.GetCurrentPosition(&pos);
	campos = pos;
	
	if( mpReverseInfo!=NULL ){
		mpReverseInfo->GetCurrentPosition(&pos);
	}
	
	if( mStageStartMode==true ){
//		C_Paper	paper(32,400);
//		PRINTF("StageStartCheck!\n");
		if( GHM_VEC_GET_DIST(&tgt,&mStageStartTargetInitPosition) > current_dist ){
			SetCurrentPath(WK7PATHINFO_INITDIR_TGTDIR_REV);
			mPathControl.GetCurrentPosition(&pos);
			mStageStartMode = false;
			current_dist = STAGESTART_CHANEDIST;
			SetPos(RENEW_IM);
		}else{
			mEscapeMode = false;
			mCamPosReset = 1;
		}
	}
	
#ifdef	_DEBUG
	if( mainMenu.GetFollowCamEdit()==true ){
		f32		val;
		if( HSD_PadGetButton(1) & HSD_PAD_Y ){
			
			val  = mTgtH.GetValue();
			val -= HSD_PadGetNormalStickX(1);
			mTgtH.SetValue(val);
			
			val  = mTgtV.GetValue();
			val += HSD_PadGetNormalAnalogR(1);
			val -= HSD_PadGetNormalAnalogL(1);
			if( val < -mTargetHeightNormal ){
				val = -mTargetHeightNormal;
			}
			mTgtV.SetValue(val);
			
			val  = mTgtF.GetValue();
			val += HSD_PadGetNormalStickY(1);
			if( val < 0.0f ){
				val = 0.0f;
			}
			mTgtF.SetValue(val);
			
		}else{
			
			val  = mPosV.GetValue();
			val += HSD_PadGetNormalAnalogR(1);
			val -= HSD_PadGetNormalAnalogL(1);
			if( val < -mCamHeightNormal ){
				val = -mCamHeightNormal;
			}
			mPosV.SetValue(val);
			
			val  = mPosH.GetValue();
			val -= HSD_PadGetNormalStickX(1);
//			val += HSD_PadGetNormalSubStickX(1);
			mPosH.SetValue(val);
			
			val  = mApproachLimitDist.GetValue();
			val -= HSD_PadGetNormalStickY(1);
			if( val < 2.0f ){
				val = 2.0f;
			}
			mApproachLimitDist.SetValue(val);
//			val += camdistMargin;
//			val += minAWAY_LIMITDIST - minAPPROACH_LIMITDIST;
			mAwayLimitDist    .SetValue(val);
			
		}
	}
#endif
	
	mPosH.FrameProcess();
	mPosV.FrameProcess();
	mTgtF.FrameProcess();
	mTgtH.FrameProcess();
	mTgtV.FrameProcess();
	
	if( mStageStartMode==false && pTargetPathControl!=NULL ){
		
		WK7PathInfo		*pPathInfo;
		WK7PathNurbs	*pPath;
		if( (pPathInfo=pTargetPathControl->GetCurrentPathInfo())!=NULL &&
		    (pPath=pPathInfo->GetPath())!=NULL ){
			// XCh
			
			Vec		dir,dirv,dirh,tmpv,tmph,camdir;
			
			pPath->GetDiffVec(dir,pPathInfo->GetParam());
			
			VECSubtract(&tgt,&pos,&camdir);
			if( VECMag(&camdir) > 0.00001f ){
				VECNormalize(&camdir,&camdir);
			}else{
				camdir.x = 0.0f;
				camdir.y = 0.0f;
				camdir.z = 0.0f;
			}
			
			VECNormalize(&dir,&dir);
			dirv.x = 0.0f;
			dirv.y = 1.0f;
			dirv.z = 0.0f;
			VECCrossProduct(&dirv,&dir,&dirh);
			VECNormalize(&dirh,&dirh);
			
			VECScale(&dirv,&tmpv,mPosV.GetValue());
			VECScale(&dirh,&tmph,mPosH.GetValue());
			
			VECAdd(&pos,&tmpv,&pos);
			VECAdd(&pos,&tmph,&pos);
			
			VECScale(&dirv  ,&dirv  ,mTgtV.GetValue());
			VECScale(&dirh  ,&dirh  ,mTgtH.GetValue());
			VECScale(&camdir,&camdir,mTgtF.GetValue());
			
			VECAdd(&tgt,&dirh  ,&tgt);
			VECAdd(&tgt,&dirv  ,&tgt);
			VECAdd(&tgt,&camdir,&tgt);
			
		}
		
	}
	
static f32	rate = 0.25f;
	f32		_rate = 1.0f - rate;
	f32		dch,dth,minh,maxh;
	
#define	HEIGHTMOVE_FRAME		(32.0f)
static f32	heightmove_frame = HEIGHTMOVE_FRAME;
	if( mBranchPointMode==true ){
		dch = (mCamHeightBp    - mCamHeightNormal   ) / heightmove_frame;
		dth = (mTargetHeightBp - mTargetHeightNormal) / heightmove_frame;
	}else{
		dch = (mCamHeightNormal    - mCamHeightBp   ) / heightmove_frame;
		dth = (mTargetHeightNormal - mTargetHeightBp) / heightmove_frame;
	}
	
	mCamHeightCurrent    += dch;
	mTargetHeightCurrent += dth;
	
	if( mCamHeightBp > mCamHeightNormal ){
		minh = mCamHeightNormal;
		maxh = mCamHeightBp;
	}else{
		minh = mCamHeightBp;
		maxh = mCamHeightNormal;
	}
	GHM_LIMIT(mCamHeightCurrent,maxh,minh);
	
	if( mTargetHeightBp > mTargetHeightNormal ){
		minh = mTargetHeightNormal;
		maxh = mTargetHeightBp;
	}else{
		minh = mTargetHeightBp;
		maxh = mTargetHeightNormal;
	}
	GHM_LIMIT(mTargetHeightCurrent,maxh,minh);
	
	if( mEscapeMode==true ){
		
		f32		dist;
		
		dist = GHM_VEC_GET_DIST(&plpos,&pos);
//		dist = GHM_VEC_GET_DIST(&tgt,&pos);
		
		if( revinfo==true && dist > mEscapeDist ){ //minAPPROACH_LIMITDIST*0.9f ){
			
			revinfo = false;
			mEscapeMode = false;
			epos.x = 
			epos.y = 
			epos.z = 0.0f;
			
		}
		
		if( dist > mEscapeDist ){
			
			mEscapeMode = false;
			epos.x = 
			epos.y = 
			epos.z = 0.0f;
			
		}else{
			
			dist = sqrtf(mEscapeDist * mEscapeDist - dist * dist);
			epos.x = mEscapeDir.x * dist;
			epos.y = mEscapeDir.y * dist - mCamHeightCurrent;
			epos.z = mEscapeDir.z * dist;
			
			if( epos.y < 0.0f ){
				epos.y = 0.0f;
			}
			
		}
		
	}else{
		
		epos.x = 
		epos.y = 
		epos.z = 0.0f;
		
	}
	
	if( mStageStartMode==false ){
		if( mpBpCamInfo!=NULL ){
			if( GHM_VEC_GET_DIST(&plpos,&mBpPosition) > WK7SelectIF::GetAppearDist() ){
				mpBpCamInfo = NULL;
			}else{
				pos = mpBpCamInfo->mCamPos;
				tgt = mpBpCamInfo->mCamTgt;
				epos.x = 
				epos.y = 
				epos.z = 0.0f;
			}
			mCamPosReset = 1;
		}
//		pos.y += mCamHeightCurrent;
//		tgt.y += mTargetHeightCurrent;
	}else{
		pos = mStageStartCamPosition;
		tgt = mStageStartCamTargetPosition;
	}
	
	if( !mCamPosReset ){
		
		mPosition      .x = pos.x * rate + mPosition      .x * _rate;
		mPosition      .y = pos.y * rate + mPosition      .y * _rate;
		mPosition      .z = pos.z * rate + mPosition      .z * _rate;
		mTargetPosition.x = tgt.x * rate + mTargetPosition.x * _rate;
		mTargetPosition.y = tgt.y * rate + mTargetPosition.y * _rate;
		mTargetPosition.z = tgt.z * rate + mTargetPosition.z * _rate;
		mEscapePosition.x = epos.x * rate + mEscapePosition.x * _rate;
		mEscapePosition.y = epos.y * rate + mEscapePosition.y * _rate;
		mEscapePosition.z = epos.z * rate + mEscapePosition.z * _rate;
		
	}else{
		
		mPosition      .x = pos.x;
		mPosition      .y = pos.y;
		mPosition      .z = pos.z;
		mTargetPosition.x = tgt.x;
		mTargetPosition.y = tgt.y;
		mTargetPosition.z = tgt.z;
		mEscapePosition.x = epos.x;
		mEscapePosition.y = epos.y;
		mEscapePosition.z = epos.z;
		
		mPosH.SetValue(mPosH.GetDstValue());
		mPosV.SetValue(mPosV.GetDstValue());
		mTgtF.SetValue(mTgtF.GetDstValue());
		mTgtH.SetValue(mTgtH.GetDstValue());
		mTgtV.SetValue(mTgtV.GetDstValue());
		
		mEscapeMode  = false;
		
	}
	
	f32		fov;
	if( mStageStartMode==true ){
		fov = WK7Stage::GetDefaultCameraFov();
	}else{
		fov = pCamera->GetFov();
	}
	pos.x = mPosition.x + mEscapePosition.x;
	pos.y = mPosition.y + mEscapePosition.y;
	pos.z = mPosition.z + mEscapePosition.z;
	tgt.x = mTargetPosition.x;
	tgt.y = mTargetPosition.y;
	tgt.z = mTargetPosition.z;
	
	if( mStageStartMode==false && (mpBpCamInfo==NULL || mBpCamInfoPrio==1) ){
		switch( mMode ){
		case	POSFIXED_MODE:
		case	FIXED_MODE:
			if( mpDesc==NULL || pTargetPathControl==NULL ){
				break;
			}
	//		if( (mBranchPointMode!=false && WK7SELECTIF->GetBranchItemNum(1) > 1) ){
	//			break;
	//		}
			pos = mpDesc->perspective.eyepos->position;
			fov = mpDesc->perspective.fov;
			switch( mMode ){
			case	POSFIXED_MODE:
	static f32		height = 10.0f;
				pTargetPathControl->GetCurrentPosition(&tgt);
				tgt.y += height;
				break;
			case	FIXED_MODE:
				tgt    = mpDesc->perspective.interest->position;
				break;
			}
			break;
		case	RAIL_MODE:{
			Vec		dir;
			f32		len;
			pTargetPathControl->GetCurrentPosition(&tgt);
			tgt.y += mTargetHeightCurrent;
			VECSubtract(&mRailPosition[1],&mRailPosition[0],&dir);
//			fov = pCamera->GetFov();
			if( (len = VECMag(&dir)) < 0.00001f ){
				pos = mRailPosition[0];
			}else{
				f32		ip;
				Vec		dir2,dirzx;
				Vec		descdir;
				VECSubtract(&tgt,&mRailPosition[0],&dir2);
				dir2 .y = 0.0f;
				dirzx.x = dir.x;
				dirzx.y = 0.0f;
				dirzx.z = dir.z;
				VECNormalize(&dirzx,&dirzx);
				VECNormalize(&dir  ,&dir  );
				ip  = VECDotProduct(&dirzx,&dir2);
				if( mpDesc!=NULL ){
					if( mpDesc->common.eyepos!=NULL && mpDesc->common.interest!=NULL ){
						f32		ip2,len2;
						VECScale   (&dirzx,&pos,ip);
						VECAdd     (&pos,&mRailPosition[0],&pos);
						VECSubtract(&mpDesc->common.eyepos->position,&mpDesc->common.interest->position,&descdir);
						VECSubtract(&pos,&tgt,&dir2);
						descdir.y = 0.0f;
						dir2   .y = 0.0f;
						len2 = VECMag(&dir2);
						ip2  = VECDotProduct(&dir2,&descdir);
						if( GHM_FABS(ip2) > 0.000001f ){
							ip2  = VECDotProduct(&descdir,&dirzx) * len2 * len2 / ip2;
							ip  += ip2;
						}
					}
					if( mpDesc->common.projection_type==COBJ_PROJECTION_PERSPECTIVE ){
						fov = mpDesc->perspective.fov;
					}
//				}else{
//					fov = WK7Stage::GetDefaultCameraFov();
				}
				GHM_LIMIT(ip,len,0.0f);
				VECScale(&dir,&pos,ip);
				VECAdd  (&pos,&mRailPosition[0],&pos);
				switch( mRailType ){
				case	RAILTYPE_CAMDIR:
					if( mpDesc==NULL ){
						break;
					}
					VECSubtract(&pos,&descdir,&tgt);
					break;
				case	RAILTYPE_TOPLAYER:
				default:
					break;
				}
			}
			fov = pCamera->GetFov();
			}break;
		case	FOLLOW_MODE:
		case	FREE_MODE:
		case	TARGETFIX_MODE:
		default:
			break;
		}
	}
	
	switch( mMode ){
	case	FREE_MODE:
		break;
	default:
static	f32	float_error	= 0.00001f;
		if( GHM_FABS(pos.x-tgt.x) < float_error &&
		    GHM_FABS(pos.y-tgt.y) < float_error &&
		    GHM_FABS(pos.z-tgt.z) < float_error ){
			pos.z += WK7CAM_DEFAULT_TARGET_DIST;
		}
		pCamera->SetFov           (fov);
		pCamera->SetPosition      (pos);
		if( mMode==TARGETFIX_MODE ){
			pCamera->SetTargetPosition(mTargetFixPosition);
			break;
		}
		pCamera->SetTargetPosition(tgt);
		break;
	}
	
	mCamPosReset = 0;
	
	return true;
}

bool WK7CamPathFollowInfo::SetMode(int mode)
{
	switch( mMode ){
	case	FOLLOW_MODE:
	case	TARGETFIX_MODE:
		//RenewPos();
		break;
	case	RAIL_MODE:
		mRailType = RAILTYPE_TOPLAYER;
	case	POSFIXED_MODE:
	case	FIXED_MODE:
	case	FREE_MODE:
		if( mode==FOLLOW_MODE ){
			SetPos(RENEW_IM);
		}
		break;
	default:
		return false;
	}
	mMode = mode;
	return true;
}

void WK7CamPathFollowInfo::SetPathCamera(HSD_CameraDesc *pDesc)
{
	mpDesc = pDesc;
}

bool WK7CamPathFollowInfo::SetRailType(int type)
{
	switch( type ){
	case	RAILTYPE_TOPLAYER:
	case	RAILTYPE_CAMDIR:
		break;
	default:
		return false;
	}
	mRailType = type;
	return true;
}

bool WK7CamPathFollowInfo::SetRailPosition(const Vec *ppos)
{
	if( ppos==NULL ){
		return false;
	}
	mRailPosition[0] = ppos[0];
	mRailPosition[1] = ppos[1];
	return true;
}

bool WK7CamPathFollowInfo::SetRailPosition(int index,const Vec &pos)
{
	if( index < 0 || index >= 2 ){
		return false;
	}
	mRailPosition[index] = pos;
	return true;
}

bool WK7CamPathFollowInfo::GetRailPosition(int index,Vec &pos)
{
	if( index < 0 || index >= 2 ){
		return false;
	}
	pos = mRailPosition[index];
	return true;
}

bool WK7CamPathFollowInfo::SetTipReverse(bool flag)
{
	if( flag==true ){
		mTipReverseProc   = 1;
		mTipReverseCancel = false;
	}else{
		mTipReverseProc   = 0;
	}
	return true;
}

bool WK7CamPathFollowInfo::TipReverseCancel(void)
{
	if( mTipReverseProc==0 ){
		return false;
	}
	mTipReverseCancel = true;
	return true;
}

bool WK7CamPathFollowInfo::SetTipReversePosition(const Vec &pos)
{
	mTipReversePosition = pos;
	return true;
}

// pXAjp
class WK7CamPathAnimInfo : public WK7SmallMemObj{
	friend class WK7Cam;
protected:
	WK7CamPathAnimInfo();
	~WK7CamPathAnimInfo();
	
	bool FrameProcess(WK7Cam *pCamera);
};


WK7CamPathAnimInfo::WK7CamPathAnimInfo()
{
}

WK7CamPathAnimInfo::~WK7CamPathAnimInfo()
{
}

bool WK7CamPathAnimInfo::FrameProcess(WK7Cam *pCamera)
{
#pragma unused(pCamera)
	return true;
}

WK7CamPathAnimInfo				*mpPathAnimInfo   = NULL;

WK7Cam* WK7Cam::GetInstance(void)
{
	if( gpCamera==NULL ){
//		if( (gpWobbleMotionArchive=WK7Archive::Create())!=NULL ){
//			gpWobbleMotionArchive->LoadArchiveB("Wat/CamWobble_00.dat");
//			gpWobbleMotionArchive->SetStatic(true);
//			gpWobbleMotion = gpWobbleMotionArchive->CreateCameraMotion();
//		}
		ghmGcMemSetAllocName("WK7Cam");
		gpCamera = new WK7Cam;
	}
	return gpCamera;
}

bool WK7Cam::Initialize(void)
{
	GetInstance();
	return true;
}

bool WK7Cam::Terminate(void)
{
	GHM_SAFE_DELETE(gpCamera);
	return true;
}

WK7Cam::WK7Cam()
{
	InitMember();
}

//WK7Cam::WK7Cam(WK7Obj *pAnchor,int prio) : WK7RenderObj(pAnchor,prio)
//{
//	InitMember();
//}

WK7Cam::~WK7Cam()
{
	FreeArchive();
}

#define	WK7CAM_MOV_SPD_MIN			(0.125f)
#define	WK7CAM_MOV_SPD_DEFAULT		(WK7CAM_MOV_SPD_MIN * 4.0f)
#define	WK7CAM_ROT_SPD_MIN			(0.03125f)
#define	WK7CAM_ROT_SPD_DEFAULT		(0.03125f)
void WK7Cam::InitMember(void)
{
	memset(&mRotation,0,sizeof(mRotation));
	mPosition.x = 0.0f;
	mPosition.y = 0.0f;
	mPosition.z = 0.0f;
	mTargetPosition.x = 0.0f;
	mTargetPosition.y = 0.0f;
	mTargetPosition.z = 1.0f;
	mRelativePosition.x = 
	mRelativePosition.y = 
	mRelativePosition.z = 0.0f;
	mRelativeRotation.x = 
	mRelativeRotation.y = 
	mRelativeRotation.z = 0.0f;
	mPrevSystemPosition       = mPosition;
	mPrevSystemTargetPosition = mTargetPosition;
	mFlag                   = 0;
	mMode                   =
	mPrevMode               = WK7CAM_FREE_TARGET;
	mTargetDistance         = WK7CAM_DEFAULT_TARGET_DIST;
	mpTargetArchive         = NULL;
	mpTargetModel           = NULL;
	mCameraMovSpeed         = WK7CAM_MOV_SPD_DEFAULT;
	mCameraRotSpeed         = WK7CAM_ROT_SPD_DEFAULT;
	mMotionRate             = 1.0f;
	mMotionState            = 0;
	mpInterpolateInfo       = NULL;
	mpDelayInterpolateInfo  = NULL;
	mpCircleInterpolateInfo = NULL;
	ghmGcMemSetAllocName("WK7CamPF");
	mpPathFollowInfo        = new WK7CamPathFollowInfo;
	ghmGcMemSetAllocName("WK7CamAI");
	mpPathAnimInfo          = new WK7CamPathAnimInfo;
	mpControlPath           = NULL;
	mpCObj                  = HSD_CObjLoadDesc((HSD_CameraDesc *)&defpers);
	mpCObjCurrent           = HSD_CObjLoadDesc((HSD_CameraDesc *)&defpers);
	mpCObjWobble            = NULL;
	mFov                    = HSD_CObjGetFov(mpCObj);
	mFovDirty               = 1;
	mNear                   = HSD_CObjGetNear(mpCObj);
	mNearDirty              = 1;
	mFar                    = HSD_CObjGetFar(mpCObj);
	mFarDirty               = 1;
	mRoll                   = HSD_CObjGetRoll(mpCObj);
	mRollDirty              = 1;
	mStoreFar               = -1.0f;
	mStoreNear              = -1.0f;
	mStoreFov               = -1.0f;
	mStoreRoll              = TWO_PI;
	mRenderFlag             = true;
//	mCinemaScopeMode        = false;
	mCinemaScopeAdjust      = 1;
	mCinemaScopeColor       = WK7CAM_DEFAULT_CINEMASCOPE_COLOR;
//	mCinemaScopeColor       = 0x781111ff;
	mWobbleMotionRate.SetValue(1.0f);
	mWobblePosRate   .SetValue(0.25f);
	mWobbleDirRate   .SetValue(0.25f);
//	mWobblePosRate   .SetValue(0.5f);
//	mWobbleDirRate   .SetValue(0.5f);
//	mWobblePosRate   .SetValue(1.0f);
//	mWobbleDirRate   .SetValue(1.0f);
	mWobbleEnable           = 1;
	mWobbleForced           = 0;
	HSD_CObjGetEyePosition(mpCObj,&mPosition      );
	HSD_CObjGetInterest   (mpCObj,&mTargetPosition);
	mPrevSystemPosition       = mPosition;
	mPrevSystemTargetPosition = mTargetPosition;
#ifdef	_DEBUG
	mDebugMode        = false;
	mDebugOperate     = true;
#endif
	if( gpWobbleMotionArchive!=NULL && gpWobbleMotion!=NULL ){
		HSD_CameraDesc	*pDesc;
		const char		*pDescName;
		int				i;
		for(i=0 ; i<gpWobbleMotionArchive->GetCameraDescNum() ; i++){
			pDesc     = gpWobbleMotionArchive->GetCameraDesc(i);
			pDescName = GetDescName(pDesc);
			if( pDescName==NULL || strcmp(pDescName,"persp")==0 ){
				continue;
			}
			mpCObjWobble = HSD_CObjLoadDesc(pDesc);
			break;
		}
		if( mpCObjWobble!=NULL ){
			HSD_CameraAnim	*pCameraAnim = gpWobbleMotion->GetAnim(0);
			HSD_CObjAddAnim(mpCObjWobble,pCameraAnim);
			HSD_CObjReqAnim(mpCObjWobble,0.0f);
			if( mpCObjWobble->aobj!=NULL ){
				HSD_AObjSetFlags  (mpCObjWobble->aobj,AOBJ_ANIM_LOOP);
			}
			if( mpCObjWobble->eyepos!=NULL && mpCObjWobble->eyepos->aobj!=NULL ){
				HSD_AObjSetFlags  (mpCObjWobble->eyepos->aobj,AOBJ_ANIM_LOOP);
			}
			if( mpCObjWobble->interest!=NULL && mpCObjWobble->interest->aobj!=NULL ){
				HSD_AObjSetFlags  (mpCObjWobble->interest->aobj,AOBJ_ANIM_LOOP);
			}
		}
	}
}

WK7PathControl* WK7Cam::GetPathControl(void)
{
	if( mpPathFollowInfo==NULL ){
		return NULL;
	}
	return mpPathFollowInfo->GetPathControl();
}

bool WK7Cam::SetCinemaScopeModeFlag(bool flag)
{
	mCinemaScopeMode = (flag==true? 1:0);
	return true;
}

bool WK7Cam::SetCinemaScopeMode(bool flag)
{
	mCinemaScopeMode = (flag==true? 1:0);
	SetScissor ();
	SetViewport();
	return true;
}

bool WK7Cam::SetScissor(void)
{
	HSD_RectU16		sc_rect;
	GetScissor(sc_rect);
	return SetScissor(sc_rect);
}

bool WK7Cam::SetScissor(HSD_RectU16 &sc_rect)
{
	HSD_CObjSetScissor (mpCObjCurrent,&sc_rect);
	HSD_CObjSetScissor (mpCObj       ,&sc_rect);
	return true;
}

//static	HSD_RectU16		cinema_rect = { 0,WK7_SCREEN_WIDTH,64,416 };
//static	HSD_RectU16		cinema_rect = { 0,WK7_SCREEN_WIDTH,44,396 };
static	HSD_RectU16		cinema_rect = { 0,WK7_SCREEN_WIDTH,32,384 };
bool WK7Cam::GetScissor(HSD_RectU16 &sc_rect)
{
	if( mCinemaScopeMode ){
		sc_rect.ymin = cinema_rect.ymin;
		sc_rect.ymax = cinema_rect.ymax;
//		sc_rect.ymin = 44;
//		sc_rect.ymax = 396;
//		sc_rect.ymin = 64;
//		sc_rect.ymax = 416;
	}else{
		sc_rect.ymin = 0;
		sc_rect.ymax = WK7_SCREEN_HEIGHT;
	}
#ifdef	_DEBUG
	if( GETCAPTUREMODE() ){
		sc_rect.ymin = 0;
		sc_rect.ymax = WK7_SCREEN_HEIGHT;
	}
#endif
	sc_rect.xmin = 0;
	sc_rect.xmax = WK7_SCREEN_WIDTH;
	return true;
}

bool WK7Cam::SetViewport(void)
{
	HSD_RectS16		vp_rect;
	GetViewport(vp_rect);
	return SetViewport(vp_rect);
}

bool WK7Cam::SetViewport(HSD_RectS16 &vp_rect)
{
	HSD_CObjSetViewport(mpCObjCurrent,&vp_rect);
	HSD_CObjSetViewport(mpCObj       ,&vp_rect);
	return true;
}

bool WK7Cam::GetViewport(HSD_RectS16 &vp_rect)
{
	vp_rect.xmin = 0;
	vp_rect.xmax = WK7_SCREEN_WIDTH;
	vp_rect.ymin = 0;
	vp_rect.ymax = WK7_SCREEN_HEIGHT;
#ifdef	_DEBUG
	if( GETCAPTUREMODE() ){
		return true;
	}
#endif
	if( mCinemaScopeMode && mCinemaScopeAdjust ){
		s16		delta = (s16)((WK7_SCREEN_HEIGHT - cinema_rect.ymax - (cinema_rect.ymin - 0)) >> 1);
		vp_rect.ymin -= delta;
		vp_rect.ymax -= delta;
	}
	return true;
}

bool WK7Cam::Set2DMatrix(Mtx view,Mtx44 proj,f32 n,f32 f)
{
	Vec				vPos;
	Vec				vUp;
	Vec				vAt;
	
	vUp .x =    0.0f;
	vUp .y = -  1.0f;
	vUp .z =    0.0f;
	vPos.x =    0.0f;
	vPos.y =    0.0f;
	vPos.z =    0.0f;
	vAt .x =  vPos.x;
	vAt .y =  vPos.y;
	vAt .z =    1.0f;
	MTXLookAt(view,&vPos,&vUp,&vAt);
	MTXOrtho (proj,0,-WK7_SCREEN_HEIGHT,0,WK7_SCREEN_WIDTH,n,f);
	
	GXLoadPosMtxImm   (view,GX_PNMTX0);
	GXLoadNrmMtxImm   (view,GX_PNMTX0);
	GXSetCurrentMtx   (GX_PNMTX0);
	GXSetProjection   (proj,GX_ORTHOGRAPHIC);
	
	return true;
	
}

bool WK7Cam::RenderCinemaScopeFrame(bool flag)
{
	if( flag==true && !mCinemaScopeMode ){
		return true;
	}
#ifdef	_DEBUG
	if( GETCAPTUREMODE() ){
		return true;
	}
#endif
	
	Mtx				view;
	Mtx44			proj;
	HSD_RectU16		sc_rect;
	
	Set2DMatrix(view,proj);
	GetScissor (sc_rect);
	
	GXSetTevSwapModeTable(GX_TEV_SWAP0,GX_CH_RED,GX_CH_GREEN,GX_CH_BLUE,GX_CH_ALPHA);
	GXSetTevSwapMode     (GX_TEVSTAGE0,GX_TEV_SWAP0,GX_TEV_SWAP0);
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
//	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
//	GXSetChanMatColor (GX_COLOR0A0,color);
	
	GXSetCullMode     (GX_CULL_NONE);
	
	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetZCompLoc     (GX_TRUE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
//	GXSetBlendMode    (GX_BM_NONE,GX_BL_ONE,GX_BL_ZERO,GX_LO_CLEAR);
//	
//	GXSetNumIndStages(0);
	
	GXSetFog(GX_FOG_NONE,0.0f,1024.0f,0.1f,1024.0f,*((GXColor *)&mCinemaScopeColor));
	
	GXSetScissor      (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport     (0.0f,0.0f,K7APP_SCREEN_WIDTH,K7APP_SCREEN_HEIGHT,0.0f,1.0f);
	
#ifdef	USE_TEXTURE
	
	stTexture		*pNoiseTexture = gpNoise->mGetTexPtrBlue();
	
	if( pNoiseTexture!=NULL ){
		
#define	SAND_NOISE_TEX_W		(160)	//eNX`̕
#define	SAND_NOISE_TEX_H		(120)	//eNX`̍
#define	SAND_NOISE_RATE_MIN		(7)		//g傳ŏ
#define	SAND_NOISE_RATE_MAX		(12)	//g傳ő劄
//			uw = (s16)((f32)SAND_NOISE_TEX_W / mSandScaleRate);
//			uh = (s16)((f32)SAND_NOISE_TEX_H / mSandScaleRate);
//			
//			mSprtNoise.mSetUV0( (s16)(HSD_Randi(SAND_NOISE_TEX_W - uw - 2) + 1),
//								(s16)(HSD_Randi(SAND_NOISE_TEX_H - uh - 2) + 1) );
		
		u32		col = 0xffffffff;
		
		s16		uw,uh;
static	f32		scale = SAND_NOISE_RATE_MAX;
		s16		u0,v0;
		f32		u,v,w,h;
		
		uw = (s16)((f32)SAND_NOISE_TEX_W / scale);
		uh = (s16)((f32)SAND_NOISE_TEX_H / scale);
		u0 = (s16)(HSD_Randi(SAND_NOISE_TEX_W - uw - 2) + 1);
		v0 = (s16)(HSD_Randi(SAND_NOISE_TEX_H - uh - 2) + 1);
		
		GXLoadTexObj      (&pNoiseTexture->obj,GX_TEXMAP0);
		
		GXClearVtxDesc    ();
		GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_CLR0,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_TEX0,GX_DIRECT);
		
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XY,GX_U16,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_CLR0,GX_CLR_RGBA,GX_RGBA8,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
		
		GXSetNumTexGens   (1);
		GXSetTexCoordGen  (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
		
		GXSetNumTevStages (1);
		GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
		GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
		GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4 * 2);
			
			w = (f32)((f32)uw * (sc_rect.xmax - sc_rect.xmin) / (K7APP_SCREEN_WIDTH  * SAND_NOISE_TEX_W));
			h = (f32)((f32)uh * (sc_rect.ymin               ) / (K7APP_SCREEN_HEIGHT * SAND_NOISE_TEX_H));
			u = (f32)((f32)u0 / SAND_NOISE_TEX_W);
			v = (f32)((f32)v0 / SAND_NOISE_TEX_H);
			
			GXPosition2u16(sc_rect.xmin,           0);
			GXColor1u32   (col);
			GXTexCoord2f32(u  ,v);
			GXPosition2u16(sc_rect.xmax,           0);
			GXColor1u32   (col);
			GXTexCoord2f32(u+w,v);
			GXPosition2u16(sc_rect.xmax,sc_rect.ymin);
			GXColor1u32   (col);
			GXTexCoord2f32(u+w,v+h);
			GXPosition2u16(sc_rect.xmin,sc_rect.ymin);
			GXColor1u32   (col);
			GXTexCoord2f32(u  ,v+h);
			
			h = (f32)((f32)uh * (WK7_SCREEN_HEIGHT - sc_rect.ymax) / (K7APP_SCREEN_HEIGHT * SAND_NOISE_TEX_H));
			
			GXPosition2u16(sc_rect.xmin,sc_rect.ymax);
			GXColor1u32   (col);
			GXTexCoord2f32(u  ,v);
			GXPosition2u16(sc_rect.xmax,sc_rect.ymax);
			GXColor1u32   (col);
			GXTexCoord2f32(u+w,v);
			GXPosition2u16(sc_rect.xmax,WK7_SCREEN_HEIGHT);
			GXColor1u32   (col);
			GXTexCoord2f32(u+w,v+h);
			GXPosition2u16(sc_rect.xmin,WK7_SCREEN_HEIGHT);
			GXColor1u32   (col);
			GXTexCoord2f32(u  ,v+h);
		
		GXEnd();
		
		return true;
	}
	
#endif
	
//static	u32		col = 0x781111ff;
//static	u32		col = 0xcc292bff;
//static	u32		col = 0x3c6d8eff;
//static	u32		col = 0x044571ff;
//static	u32		col = 0x3C6DE8FF;
//static	u32		col = 0x629fc4ff;
//static	u32		col = 0x000040ff;
//static	u32		col = 0x800000ff;
	
//	u32		col;
//	if( HSD_PadGetButton(1) & HSD_PAD_X ){
//		col = col0;
//	}else{
//		col = col1;
//	}
	
#if 0
static	u32		coladjust = 0;
	if( coladjust ){
		u32		colr = ((mCinemaScopeColor >> 24) & 0xff);
		u32		colg = ((mCinemaScopeColor >> 16) & 0xff);
		u32		colb = ((mCinemaScopeColor >>  8) & 0xff);
		u32		*pcol = NULL;
		u32		d;
		
		if( HSD_PadGetButton(1) & HSD_PAD_Z ){
			d = 4;
		}else{
			d = 1;
		}
		if( HSD_PadGetButton(1) & HSD_PAD_X ){
			pcol = &colr;
		}else if( HSD_PadGetButton(1) & HSD_PAD_Y ){
			pcol = &colg;
		}else if( HSD_PadGetButton(1) & HSD_PAD_B ){
			pcol = &colb;
		}
		if( pcol!=NULL ){
			if( HSD_PadGetRepeat(1) & HSD_PAD_UP ){
				(*pcol)++;
			}
			if( HSD_PadGetRepeat(1) & HSD_PAD_DOWN ){
				(*pcol)--;
			}
			(*pcol) &= 0xff;
		}
		mCinemaScopeColor = (colr << 24)|(colg << 16)|(colb << 8)|(mCinemaScopeColor & 0xff);
		
		{
			C_Paper		paper(32,32);
			PRINTF("\n RGB=(%02x,%02x,%02x)\n",colr,colg,colb);
		}
		
	}
#endif
	u32		col = mCinemaScopeColor;
	
	GXClearVtxDesc    ();
	GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc      (GX_VA_CLR0,GX_DIRECT);
	
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XY,GX_U16,0);
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_CLR0,GX_CLR_RGBA,GX_RGBA8,0);
	
	GXSetNumTexGens   (0);
	
	GXSetNumTevStages (1);
	GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4 * 2);
	
		GXPosition2u16(sc_rect.xmin,           0);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmax,           0);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmax,sc_rect.ymin);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmin,sc_rect.ymin);
		GXColor1u32   (col);
		
		GXPosition2u16(sc_rect.xmin,sc_rect.ymax);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmax,sc_rect.ymax);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmax,WK7_SCREEN_HEIGHT);
		GXColor1u32   (col);
		GXPosition2u16(sc_rect.xmin,WK7_SCREEN_HEIGHT);
		GXColor1u32   (col);
	
	GXEnd();
	
	return true;
}

bool WK7Cam::StoreState(void)
{
	bool	result = true;
	if( StoreFar()==false ){
		result = false;
	}
	if( StoreNear()==false ){
		result = false;
	}
	if( StoreFov()==false ){
		result = false;
	}
	if( StoreRoll()==false ){
		result = false;
	}
	return result;
}

bool WK7Cam::RestoreState(void)
{
	bool	result = true;
	if( RestoreFar()==false ){
		result = false;
	}
	if( RestoreNear()==false ){
		result = false;
	}
	if( RestoreFov()==false ){
		result = false;
	}
	if( RestoreRoll()==false ){
		result = false;
	}
	return result;
}

bool WK7Cam::StoreFar(void)
{
	mStoreFar = mFar;
	return true;
}

bool WK7Cam::RestoreFar(void)
{
	if( mStoreFar < 0.0f ){
		return false;
	}
	SetFar(mStoreFar);
	mStoreFar = -1.0f;
	return true;
}

bool WK7Cam::StoreNear(void)
{
	mStoreNear = mNear;
	return true;
}

bool WK7Cam::RestoreNear(void)
{
	if( mStoreNear < 0.0f ){
		return false;
	}
	SetNear(mStoreNear);
	mStoreNear = -1.0f;
	return true;
}

bool WK7Cam::StoreFov(void)
{
	mStoreFov = mFov;
	return true;
}

bool WK7Cam::RestoreFov(void)
{
	if( mStoreFov < 0.0f ){
		return false;
	}
	SetFov(mStoreFov);
	mStoreFov = -1.0f;
	return true;
}

bool WK7Cam::StoreRoll(void)
{
	mStoreRoll = mRoll;
	return true;
}

bool WK7Cam::RestoreRoll(void)
{
	if( mStoreRoll >= TWO_PI ){
		return false;
	}
	SetRoll(mStoreRoll);
	mStoreRoll = TWO_PI;
	return true;
}

bool WK7Cam::EnterDebugMode(void)
{
#ifdef	_DEBUG
	mDebugMode = true;
//	StopMotion();
	Vec			dir;
	HSD_CObjGetEyePosition(mpCObj,&mPosition      );
	HSD_CObjGetInterest   (mpCObj,&mTargetPosition);
	VECSubtract(&mTargetPosition,&mPosition,&dir);
//	dir.x = mTargetPosition.x - mPosition.x;
//	dir.y = mTargetPosition.y - mPosition.y;
//	dir.z = mTargetPosition.z - mPosition.z;
//	float		scale;
//	scale = mTargetDistance / sqrtf(dir.x * dir.x + dir.y * dir.y + dir.z * dir.z);
//	mTargetPosition.x = mPosition.x + scale * dir.x;
//	mTargetPosition.y = mPosition.y + scale * dir.y;
//	mTargetPosition.z = mPosition.z + scale * dir.z;
	mTargetDistance = GHM_VEC_GET_LEN(&dir);
//	VECScale(&dir,&mTargetPosition,mTargetDistance);
//	VECAdd  (&mPosition,&mTargetPosition,&mTargetPosition);
#endif
	return true;
}

bool WK7Cam::ExitDebugMode(void)
{
#ifdef	_DEBUG
	mDebugMode = false;
#endif
	return true;
}

bool WK7Cam::IsDebugMode(void)
{
#ifdef	_DEBUG
	return mDebugMode;
#else
	return false;
#endif
}

bool WK7Cam::SetDebugOperate(bool flag)
{
#ifdef	_DEBUG
	mDebugOperate = flag;
#else
#pragma	unused(flag)
#endif
	return true;
}

bool WK7Cam::GetDebugOperate(void)
{
#ifdef	_DEBUG
	return mDebugOperate;
#else
	return true;
#endif
}

HSD_CameraDesc* WK7Cam::GetDescDefault(void)
{
	return (HSD_CameraDesc *)&defpers;
}

void WK7Cam::SetDesc(HSD_CameraDesc *pDesc)
{
	if( pDesc==NULL ){
		pDesc = (HSD_CameraDesc *)&defpers;
	}
	HSD_CObjInit          (mpCObj,pDesc);
	HSD_CObjGetEyePosition(mpCObj,&mPosition      );
	HSD_CObjGetInterest   (mpCObj,&mTargetPosition);
	mFar  = HSD_CObjGetFar (mpCObj);
	mNear = HSD_CObjGetNear(mpCObj);
	mFov  = HSD_CObjGetFov (mpCObj);
	mRoll = HSD_CObjGetRoll(mpCObj);
	mPrevSystemPosition       = mPosition;
	mPrevSystemTargetPosition = mTargetPosition;
	
	if( mpInterpolateInfo!=NULL ){
		mpInterpolateInfo->mFov = mFov;
	}
	if( mpDelayInterpolateInfo!=NULL ){
		mpDelayInterpolateInfo->mFov = mFov;
	}
	if( mpCircleInterpolateInfo!=NULL ){
		mpCircleInterpolateInfo->mFov = mFov;
	}
	
	mPosDirty    = 0;
	mRotDirty    = 0;
	mTgtPosDirty = 0;
	mFovDirty    = 0;
	mNearDirty   = 0;
	mFarDirty    = 0;
	mRollDirty   = 0;
}

void WK7Cam::SetMode(Mode mode)
{
	mPrevMode = mMode;
	mMode     = mode;
	if( mPrevMode!=WK7CAM_PATH_FOLLOW ){
		if( mMode==WK7CAM_PATH_FOLLOW ){
			if( mpPathFollowInfo != NULL ){
	//			mpPathFollowInfo->RenewPos();
				mpPathFollowInfo->RenewPos(0);
			}
		}
	}else{
		if( mMode!=WK7CAM_PATH_FOLLOW ){
			mpPathFollowInfo->mPathControl.ReverseCancel();
			mpPathFollowInfo->ResetOldDir();
		}
	}
	switch( mPrevMode ){
	case	WK7CAM_SYSTEM:
		mPrevSystemPosition       = mPosition;
		mPrevSystemTargetPosition = mTargetPosition;
		break;
	case	WK7CAM_DEBUG:
		break;
	case	WK7CAM_PATH_FOLLOW:
	case	WK7CAM_FREE_ROT:
	case	WK7CAM_FREE_TARGET:
	case	WK7CAM_LINIP_LINPATH:
	case	WK7CAM_DELAYIP_LINPATH:
	case	WK7CAM_LINIP_CIRPATH:
	default:
		if( mMode==WK7CAM_SYSTEM ){
			mPosition       = mPrevSystemPosition;
			mTargetPosition = mPrevSystemTargetPosition;
		}
		break;
	}
	if( mMode==WK7CAM_DEBUG ){
		EnterDebugMode();
	}
#if 0
	if( mMode==WK7CAM_DEBUG ){
		float		scale;
		Vec			dir;
		HSD_CObjGetEyePosition(mpCObj,&mPosition      );
		HSD_CObjGetInterest   (mpCObj,&mTargetPosition);
		dir.x = mTargetPosition.x - mPosition.x;
		dir.y = mTargetPosition.y - mPosition.y;
		dir.z = mTargetPosition.z - mPosition.z;
		scale = mTargetDistance / sqrtf(dir.x * dir.x + dir.y * dir.y + dir.z * dir.z);
		mTargetPosition.x = mPosition.x + scale * dir.x;
		mTargetPosition.y = mPosition.y + scale * dir.y;
		mTargetPosition.z = mPosition.z + scale * dir.z;
	}
#endif
}

void WK7Cam::GetRotationCalc(Vec& rot)
{
	float		scale;
	Vec			dir;
	dir.x = mTargetPosition.x - mPosition.x;
	dir.y = mTargetPosition.y - mPosition.y;
	dir.z = mTargetPosition.z - mPosition.z;
	scale = mTargetDistance / sqrtf(dir.x * dir.x + dir.y * dir.y + dir.z * dir.z);
	rot.x =  scale * dir.x;
	rot.y =  scale * dir.y;
	rot.z =  scale * dir.z;
}

void WK7Cam::RestoreMode(void)
{
	mMode = mPrevMode;
}

static	float	spd    = 0.5f;
static	float	rotspd = 0.0625f / 2.0f;
bool WK7Cam::DebugCameraControl(void)
{
	if( mpCObj==NULL ){
		return true;
	}
	
	switch( mMode ){
	case	WK7CAM_LINIP_LINPATH:
	    if( mpInterpolateInfo!=NULL ){
			return false;
		}
		break;
	case	WK7CAM_DELAYIP_LINPATH:
	    if( mpDelayInterpolateInfo!=NULL ){
			return false;
		}
		break;
	case	WK7CAM_LINIP_CIRPATH:
		if( mpCircleInterpolateInfo!=NULL ){
			return false;
		}
		break;
	default:
		break;
	}
	
	u32			pad;
	u32			trg;
	u32			rpt;
	float		px,py;
	bool		operate;
	
	Vec			dir;
	
	operate = true;
#ifdef	_DEBUG
	if( mDebugOperate==false ){
		operate = false;
	}
#endif
	
	pad = HSD_PadGetButton      (0);
	trg = HSD_PadGetTrigger     (0);
	rpt = HSD_PadGetRepeat      (0);
	px  = HSD_PadGetNormalStickX(0);
	py  = HSD_PadGetNormalStickY(0);
	if( operate==false ){
		pad = 0;
		trg = 0;
		px  = 0.0f;
		py  = 0.0f;
	}
	
	float		rotx,roty;
	float		zx;
	
	if( HSD_PadGetButton(0) & HSD_PAD_Z ){
		
		if( pad & HSD_PAD_A ){
			f32		fov;
			fov = GetFov();
			if( pad & HSD_PAD_STICK_RIGHT ){
				fov += 1.0f;
			}
			if( pad & HSD_PAD_STICK_LEFT ){
				fov -= 1.0f;
			}
			if( fov < 1.0f ){
				fov = 1.0f;
			}else if( fov > 170.0f ){
				fov = 170.0f;
			}
			SetFov (fov);
		}else{
			f32		n,f;
			n   = GetNear();
			f   = GetFar ();
			if( pad & HSD_PAD_STICK_UP ){
				f += mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			}
			if( pad & HSD_PAD_STICK_DOWN ){
				f -= mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			}
			if( pad & HSD_PAD_STICK_RIGHT ){
				n += mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			}
			if( pad & HSD_PAD_STICK_LEFT ){
				n -= mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			}
#define	NF_MARGIN		(1.0f)
			if( n < NF_MARGIN ){
				n = NF_MARGIN;
			}
			if( f < n ){
				f = n + NF_MARGIN;
			}
			SetNear(n);
			SetFar (f);
		}
		
		f32		camspd;
		if( pad & HSD_PAD_A ){
			camspd =  mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
		}else{
			camspd = -mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
		}
		if( pad & HSD_PAD_UP ){
			mTargetDistance += camspd;
		}
		if( pad & HSD_PAD_DOWN ){
			mTargetDistance -= camspd;
		}
		
		if( rpt & HSD_PAD_RIGHT ){
			mCameraMovSpeed *= 2.0f;
		}
		if( rpt & HSD_PAD_LEFT ){
			mCameraMovSpeed *= 0.5f;
		}
		if( mCameraMovSpeed < WK7CAM_MOV_SPD_MIN ){
			mCameraMovSpeed = WK7CAM_MOV_SPD_MIN;
		}
		
		if( mTargetDistance < spd ){
			mTargetDistance = spd;
		}
		
		VECSubtract (&mPosition,&mTargetPosition,&dir);
		VECNormalize(&dir,&dir);
		VECScale    (&dir,&dir,mTargetDistance);
		
		if( pad & HSD_PAD_A ){
			VECSubtract(&mPosition,&dir,&mTargetPosition);
		}else{
			VECAdd     (&mTargetPosition,&dir,&mPosition);
		}
	
	}else{
		
		if( px != 0.0f || py != 0.0f ){
			
			float		len;
			
			dir.x = mTargetPosition.x - mPosition.x;
			dir.y = 0.0f;
			dir.z = mTargetPosition.z - mPosition.z;
			len   = sqrtf(dir.x * dir.x + dir.z * dir.z);
			
			if( len != 0.0f ){
				
				len     = mCameraMovSpeed * (60.0f / WK7_FRAME_RATE) / len;
				dir.x  *= len;
				dir.z  *= len;
				
				Vec		movevec;
				
				movevec.x = py * dir.x - px * dir.z;
				movevec.z = py * dir.z + px * dir.x;
				
				mPosition      .x += movevec.x;
				mPosition      .z += movevec.z;
				mTargetPosition.x += movevec.x;
				mTargetPosition.z += movevec.z;
				
			}
			
		}
		
		float	ana_r,ana_l;
		ana_r = HSD_PadGetNormalAnalogR(0);
		ana_l = HSD_PadGetNormalAnalogL(0);
		if( operate==false ){
			ana_r = 0.0f;
			ana_l = 0.0f;
		}
		if( ana_r > 0.0f ){
			ana_r *= mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			mPosition      .y += ana_r;
			mTargetPosition.y += ana_r;
		}
		if( ana_l > 0.0f ){
			ana_l *= mCameraMovSpeed * (60.0f / WK7_FRAME_RATE);
			mPosition      .y -= ana_l;
			mTargetPosition.y -= ana_l;
		}
		
	}
	
	px = HSD_PadGetNormalSubStickX(0);
	py = HSD_PadGetNormalSubStickY(0);
	if( operate==false ){
		px  = 0.0f;
		py  = 0.0f;
	}
//	if( HSD_PadGetButton(0) & HSD_PAD_Z ){
//		px  = -px;
//		py  = -py;
//	}
	
	dir.x = mPosition.x - mTargetPosition.x;
	dir.y = mPosition.y - mTargetPosition.y;
	dir.z = mPosition.z - mTargetPosition.z;
	zx    = sqrtf(dir.x * dir.x + dir.z * dir.z);
	roty  = atan2f(dir.x,dir.z);
	rotx  = atan2f(dir.y,zx   );
	
	roty += mCameraRotSpeed * px * (60.0f / WK7_FRAME_RATE);
	rotx += mCameraRotSpeed * py * (60.0f / WK7_FRAME_RATE);
	
#define	ROTX_LIMIT		(PI/2.0f - 0.1f)
	GHM_LIMIT      (rotx,ROTX_LIMIT,-ROTX_LIMIT);
	GHM_LIMIT_RANGE(roty,PI,-PI);
	
	if( HSD_PadGetButton(0) & HSD_PAD_Z ){
		mTargetPosition.y = mPosition.y - mTargetDistance * sinf(rotx);
		zx          =                     mTargetDistance * cosf(rotx);
		mTargetPosition.x = mPosition.x - zx              * sinf(roty);
		mTargetPosition.z = mPosition.z - zx              * cosf(roty);
	}else{
		mPosition.y = mTargetPosition.y + mTargetDistance * sinf(rotx);
		zx          =                     mTargetDistance * cosf(rotx);
		mPosition.x = mTargetPosition.x + zx              * sinf(roty);
		mPosition.z = mTargetPosition.z + zx              * cosf(roty);
	}
	
	mRotation.x = rotx;
	mRotation.y = roty;
	
	HSD_CObjSetEyePosition(mpCObj,&mPosition      );
	HSD_CObjSetInterest   (mpCObj,&mTargetPosition);
	
	return true;
}

bool WK7Cam::SetPosRot(void)
{
	Vec		dir,interest;
	Mtx		m,tm;
	
	if( !mPosDirty && !mRotDirty ){
		return false;
	}
	
	dir.x = 0.0f;
	dir.y = 0.0f;
	dir.z = 1.0f;
	
	MTXRotRad   ( m,'y',mRotation.y);
	MTXRotRad   (tm,'x',mRotation.x);
	MTXConcat   (tm,m,m);
	MTXRotRad   (tm,'z',mRotation.z);
	MTXConcat   (tm,m,m);
	MTXMultVec  ( m,&dir,&dir);
	VECNormalize(&dir,&dir);
	VECScale    (&dir,&dir,mTargetDistance);
	VECAdd      (&dir,&mTargetPosition,&interest);
	
	if( mPosDirty ){
		HSD_CObjSetEyePosition(mpCObj,&mPosition);
		mPosDirty = 0;
	}
	
	HSD_CObjSetInterest(mpCObj,&interest);
	mRotDirty = 0;
	
	return true;
}

bool WK7Cam::SetPosTgt(void)
{
	bool	result = false;
	if( mPosDirty ){
		HSD_CObjSetEyePosition(mpCObj,&mPosition      );
		result    = true;
		mPosDirty = 0;
	}
	if( mTgtPosDirty ){
		HSD_CObjSetInterest   (mpCObj,&mTargetPosition);
		result    = true;
		mTgtPosDirty = 0;
	}
	return result;
}

bool WK7Cam::FrameProcess(void)
{
	bool	result = true;
	if( mpCObj==NULL ){
		return false;
	}
	mWobbleMotionRate.FrameProcess();
	mWobblePosRate   .FrameProcess();
	mWobbleDirRate   .FrameProcess();
	if( this==gpCamera && mpCObjWobble!=NULL ){
		HSD_CObjAnim(mpCObjWobble);
	}
	switch( mMode ){
	case	WK7CAM_DEBUG:
		break;
	case	WK7CAM_FREE_ROT:
		SetPosRot();
		break;
	case	WK7CAM_LINIP_LINPATH:
	case	WK7CAM_DELAYIP_LINPATH:
	case	WK7CAM_LINIP_CIRPATH:
	case	WK7CAM_FREE_TARGET:
	case	WK7CAM_SYSTEM:
		SetPosTgt();
		break;
	case	WK7CAM_PATH_FOLLOW:
#ifdef	_DEBUG
		if( mDebugMode==true ){
			break;
		}
#endif
		if( mpPathFollowInfo != NULL ){
			mpPathFollowInfo->FrameProcess(this);
			SetPosTgt();
		}
		break;
	case	WK7CAM_PATH_ANIM:
#ifdef	_DEBUG
		if( mDebugMode==true ){
			break;
		}
#endif
		if( mpPathAnimInfo != NULL ){
			mpPathAnimInfo->FrameProcess(this);
			SetPosTgt();
		}
		break;
	default:
		break;
	}
#ifdef	_DEBUG
	if( mDebugMode==true ){
		result = DebugCameraControl();
	}
#endif
	
	if( mFovDirty ){
		HSD_CObjSetFov (mpCObj,mFov );
		mFovDirty = 0;
	}
	if( mNearDirty ){
		HSD_CObjSetNear(mpCObj,mNear);
		mNearDirty = 0;
	}
	if( mFarDirty ){
		HSD_CObjSetFar (mpCObj,mFar );
		mFarDirty = 0;
	}
	if( mRollDirty ){
		HSD_CObjSetRoll(mpCObj,mRoll);
		mRollDirty = 0;
	}
#ifdef	_DEBUG
	if( mDebugMode == false ){
#endif
	HSD_CObjAnim(mpCObj);
#ifdef	_DEBUG
	}
	if( mDebugMode == true ){
//	if( mMode == WK7CAM_DEBUG ){
		HSD_CObjSetEyePosition(mpCObj,&mPosition);
		HSD_CObjSetInterest   (mpCObj,&mTargetPosition);
	}else{
#endif
		if( mpCObj->aobj!=NULL ){
			mFov  = HSD_CObjGetFov (mpCObj);
			mNear = HSD_CObjGetNear(mpCObj);
			mFar  = HSD_CObjGetFar (mpCObj);
			mRoll = HSD_CObjGetRoll(mpCObj);
		}
		if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
			HSD_CObjGetEyePosition(mpCObj,&mPosition);
		}
		if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
			HSD_CObjGetInterest   (mpCObj,&mTargetPosition);
		}
#ifdef	_DEBUG
	}
#endif
	if( mpCircleInterpolateInfo!=NULL ){
		if( mpCircleInterpolateInfo->FrameProcess(this)==false ){
			GHM_SAFE_DELETE(mpCircleInterpolateInfo);
			SetMode(WK7CAM_FREE_TARGET);
#ifdef	_DEBUG
			if( IsDebugMode()==true ){
				EnterDebugMode();
			}
#endif
		}
	}
	if( mpDelayInterpolateInfo!=NULL ){
		if( mpDelayInterpolateInfo->FrameProcess(this)==false ){
			GHM_SAFE_DELETE(mpDelayInterpolateInfo);
			SetMode(WK7CAM_FREE_TARGET);
#ifdef	_DEBUG
			if( IsDebugMode()==true ){
				EnterDebugMode();
			}
#endif
		}
	}
	if( mpInterpolateInfo!=NULL ){
		if( mpInterpolateInfo->FrameProcess(this)==false ){
			GHM_SAFE_DELETE(mpInterpolateInfo);
			SetMode(WK7CAM_FREE_TARGET);
#ifdef	_DEBUG
			if( IsDebugMode()==true ){
				EnterDebugMode();
			}
#endif
		}
	}
	return result;
}

bool WK7Cam::RenderProcess(void)
{
//	C_Paper	paper(32,32);
//	
//	PRINTF("%d %d\n",mMode,mpPathFollowInfo->mMode);
//	PRINTF("%d %.3f,%.3f,%.3f %.3f\n",
//	                             mpPathFollowInfo->mStageStartMode,
//	                             mpPathFollowInfo->mStageStartTargetInitPosition.x,
//	                             mpPathFollowInfo->mStageStartTargetInitPosition.y,
//	                             mpPathFollowInfo->mStageStartTargetInitPosition.z,
//	                             current_dist);
	
#ifdef	_DEBUG
	if( mRenderFlag==false ){
		return true;
	}
	if( mDebugMode==false ){
		return true;
	}
	if( mpTargetModel!=NULL ){
		WK7Light::DispStart();
		WK7Light::DispAll();
		WK7Light::DispEnd();
		mpTargetModel->RenderProcess();
	}
	HSD_StateSetAlphaUpdate(GX_FALSE);
#endif
	return true;
}

bool WK7Cam::LoadArchive(char *pArchiveFileName)
{
	if( (mpTargetArchive=WK7Archive::Create())==NULL ){
		return false;
	}
	mpTargetArchive->SetStatic(true);
	if( mpTargetArchive->LoadArchive(pArchiveFileName)==false ){
		return false;
	}
//	if( (mpTargetModel = mpTargetArchive->CreateModel())!=NULL ){
//		HSD_PEDesc	*pPEDesc = WK7RenderObj::GetNormalPEDesc();
//		mpTargetModel->SetPE(pPEDesc);
//	}
	return true;
}

bool WK7Cam::LoadProcess(void)
{
	if( mpTargetArchive==NULL ){
		return false;
	}
	if( mpTargetModel!=NULL ){
		return true;
	}
	if( mpTargetArchive->GetState()!=WK7Archive::WK7ARCHIVE_LOADED ){
		return false;
	}
	if( (mpTargetModel = mpTargetArchive->CreateModel())!=NULL ){
		HSD_PEDesc	*pPEDesc = WK7RenderObj::GetNormalPEDesc();
		mpTargetModel->SetPE(pPEDesc);
	}
	return true;
}

bool WK7Cam::IsLoadedArchive(void)
{
#ifdef	_DEBUG
	if( mpTargetModel==NULL ){
		return false;
	}
#endif
	return true;
}

bool WK7Cam::FreeArchive(void)
{
	GHM_SAFE_DELETE(mpCircleInterpolateInfo);
	GHM_SAFE_DELETE(mpDelayInterpolateInfo);
	GHM_SAFE_DELETE(mpInterpolateInfo);
	GHM_SAFE_DELETE(mpPathFollowInfo);
	GHM_SAFE_DELETE(mpPathAnimInfo);
	if( mpCObj!=NULL ){
		HSD_CObjUnref(mpCObj);
		mpCObj = NULL;
	}
	if( mpCObjWobble!=NULL ){
		HSD_CObjUnref(mpCObjWobble);
		mpCObjWobble = NULL;
	}
	if( mpCObjCurrent!=NULL ){
		HSD_CObjUnref(mpCObjCurrent);
		mpCObjCurrent = NULL;
	}
	if( mpTargetModel!=NULL ){
		GHM_SAFE_DELETE(mpTargetModel);
	}
	GHM_SAFE_DELETE(mpTargetArchive);
	return true;
}

bool WK7Cam::SetMotionRate(HSD_WObj *pWObj,f32 rate,bool wholeRateEnable)
{
	if( pWObj==NULL ){
		return false;
	}
	WK7Motion::SetMotionRate(pWObj->aobj,rate,wholeRateEnable);
	return true;
}

bool WK7Cam::SetMotionRate(HSD_CObj *pCObj,f32 rate,bool wholeRateEnable)
{
	if( pCObj==NULL ){
		return false;
	}
	WK7Motion::SetMotionRate(pCObj->aobj,rate,wholeRateEnable);
	SetMotionRate(HSD_CObjGetEyePositionWObj(pCObj),rate,wholeRateEnable);
	SetMotionRate(HSD_CObjGetInterestWObj   (pCObj),rate,wholeRateEnable);
	return true;
}

bool WK7Cam::SetMotionRate(f32 rate)
{
	mMotionRate = rate;
	RenewMotionRate();
	return true;
}

bool WK7Cam::RenewMotionRate(void)
{
	SetMotionRate(mpCObjWobble,mWobbleMotionRate.GetValue(),false);
	if( mMotionState & WK7CAM_MOTION_PLAY ){
		SetMotionRate(mpCObj,mMotionRate,true);
	}else{
		SetMotionRate(mpCObj,0.0f       ,true);
	}
	return true;
}

bool WK7Cam::AttachMotion(HSD_CameraAnim *pCameraAnim)
{
	if( mpCObj==NULL ){
		return false;
	}
	HSD_CObjAddAnim(mpCObj,pCameraAnim);
	return true;
}

bool WK7Cam::AttachMotion(WK7CameraMotion *pCameraMotion,int index)
{
	HSD_CameraAnim	*pCameraAnim = pCameraMotion->GetAnim(index);
	if( pCameraAnim==NULL ){
		return false;
	}
	return AttachMotion(pCameraAnim);
}

bool WK7Cam::AttachMotion(WK7CameraMotion *pCameraMotion,const char *pname)
{
	HSD_CameraAnim	*pCameraAnim = pCameraMotion->GetAnim(pname);
	if( pCameraAnim==NULL ){
		return false;
	}
	return AttachMotion(pCameraAnim);
}

bool WK7Cam::DetachMotion(void)
{
	if( mpCObj==NULL ){
		return false;
	}
	mMotionState &= ~WK7CAM_MOTION_PLAY;
	HSD_CObjRemoveAnim(mpCObj);
	return true;
}

bool WK7Cam::PlayMotion(f32 startframe,bool interpolate,f32 interpolateframe)
{
	if( mpCObj==NULL ){
		return false;
	}
	if( interpolate==true && interpolateframe > 0.0f ){
		if( mpInterpolateInfo==NULL ){
			ghmGcMemSetAllocName("WK7CamIF");
			mpInterpolateInfo = new WK7CamInterpolateInfo;
		}
		if( mpInterpolateInfo!=NULL ){
			mpInterpolateInfo->Initialize(this,interpolateframe);
		}
//	}else{
//		HSD_CObjResetUserDefinedMtx(mpCObj);
//		GHM_SAFE_DELETE(mpInterpolateInfo);
	}
	HSD_CObjReqAnim(mpCObj,startframe);
	mMotionState |= WK7CAM_MOTION_PLAY;
	SetMotionRate(1.0f);
	return true;
}

bool WK7Cam::ReplayMotion(void)
{
	if( mpCObj==NULL ){
		return false;
	}
	mMotionState |= WK7CAM_MOTION_PLAY;
	RenewMotionRate();
	return true;
}

bool WK7Cam::StopMotion(void)
{
	if( mpCObj==NULL ){
		return false;
	}
	mMotionState &= ~WK7CAM_MOTION_PLAY;
	RenewMotionRate();
	return true;
}

bool WK7Cam::SetMotionLoop(bool flag)
{
	if( mpCObj==NULL ){
		return false;
	}
	if( flag==true ){
		if( mpCObj->aobj!=NULL ){
			HSD_AObjSetFlags  (mpCObj->aobj,AOBJ_ANIM_LOOP);
		}
		if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
			HSD_AObjSetFlags  (mpCObj->eyepos->aobj,AOBJ_ANIM_LOOP);
		}
		if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
			HSD_AObjSetFlags  (mpCObj->interest->aobj,AOBJ_ANIM_LOOP);
		}
	}else{
		if( mpCObj->aobj!=NULL ){
			HSD_AObjClearFlags(mpCObj->aobj,AOBJ_ANIM_LOOP);
		}
		if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
			HSD_AObjClearFlags(mpCObj->eyepos->aobj,AOBJ_ANIM_LOOP);
		}
		if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
			HSD_AObjClearFlags(mpCObj->interest->aobj,AOBJ_ANIM_LOOP);
		}
	}
	return true;
}

bool WK7Cam::GetMotionLoop(void)
{
	return (GetMotionState() & AOBJ_ANIM_LOOP)? true : false;
}

f32 WK7Cam::GetCurrentMotionFrame(void)
{
	if( mpCObj!=NULL ){
		if( mpCObj->aobj!=NULL ){
			return mpCObj->aobj->frame;
		}
		if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
			return mpCObj->eyepos->aobj->frame;
		}
		if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
			return mpCObj->interest->aobj->frame;
		}
	}
	return -1.0f;
}

f32 WK7Cam::GetTotalMotionFrame(void)
{
	if( mpCObj!=NULL ){
		if( mpCObj->aobj!=NULL ){
			return mpCObj->aobj->endframe;
		}
		if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
			return mpCObj->eyepos->aobj->endframe;
		}
		if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
			return mpCObj->interest->aobj->endframe;
		}
	}
	return -1.0f;
}

int WK7Cam::GetMotionState(void)
{
	if( mpCObj==NULL ){
		return AOBJ_NO_ANIM;
	}
	if( mpCObj->aobj != NULL ){
		return (int)HSD_AObjGetFlags(mpCObj->aobj);
	}
	if( mpCObj->eyepos!=NULL && mpCObj->eyepos->aobj!=NULL ){
		return (int)HSD_AObjGetFlags(mpCObj->eyepos->aobj);
	}
	if( mpCObj->interest!=NULL && mpCObj->interest->aobj!=NULL ){
		return (int)HSD_AObjGetFlags(mpCObj->interest->aobj);
	}
	return AOBJ_NO_ANIM;
}

bool WK7Cam::SetCurrent(void)
{
	bool		userdef = true;
	Vec			interest;
	HSD_CObj	*pCObj;
	
	if( mpInterpolateInfo!=NULL ){
		mpInterpolateInfo->SetCurrent(this);
		interest = mpInterpolateInfo->mInterpolateInterest;
	}else if( mpDelayInterpolateInfo!=NULL ){
		mpDelayInterpolateInfo->SetCurrent(this);
		interest = mpDelayInterpolateInfo->mPrevInterest;
	}else if( mpCircleInterpolateInfo!=NULL ){
		mpCircleInterpolateInfo->SetCurrent(this);
		interest = mpCircleInterpolateInfo->mInterpolateInterest;
	}else{
		userdef = false;
	}
	
	if( mpTargetModel!=NULL ){
		if( userdef==true ){
			mpTargetModel->SetPosition(interest);
		}else{
			mpTargetModel->SetPosition(mTargetPosition);
		}
	}
	
	HSD_CObjSetupViewingMtx(mpCObj);
	
	if( mpCObjWobble!=NULL ){
		
		if( userdef==true ){
			
			HSD_CObjSetUserDefinedMtx(mpCObjCurrent,HSD_CObjGetViewingMtxPtr(mpCObj));
			
		}else{
			
			Vec		eyepos;
			
			HSD_CObjResetUserDefinedMtx(mpCObjCurrent);
			
			HSD_CObjGetEyePosition(mpCObj,&eyepos);
			HSD_CObjGetInterest   (mpCObj,&interest);
			
			ReviseWobble(eyepos,interest);
			
			HSD_CObjSetEyePosition(mpCObjCurrent,&eyepos);
			HSD_CObjSetInterest   (mpCObjCurrent,&interest);
			
			f32		f,n,aspect,roll;
			f      = HSD_CObjGetFar   (mpCObj);
			n      = HSD_CObjGetNear  (mpCObj);
			aspect = HSD_CObjGetAspect(mpCObj);
			roll   = HSD_CObjGetRoll  (mpCObj);
#ifdef	_DEBUG
			if( GETCAPTUREMODE() ){
				aspect *= 10.0f;
				aspect /=  9.0f;
			}
#endif
			HSD_CObjSetFar   (mpCObjCurrent,f     );
			HSD_CObjSetNear  (mpCObjCurrent,n     );
			HSD_CObjSetAspect(mpCObjCurrent,aspect);
			HSD_CObjSetRoll  (mpCObjCurrent,roll  );
			
		}
		
		HSD_CObjSetFov(mpCObjCurrent,HSD_CObjGetFov(mpCObj));
		
		pCObj = mpCObjCurrent;
		
#ifdef	_DEBUG
		if( mDebugMode==true && !mWobbleForced ){
			HSD_CObjSetUserDefinedMtx(mpCObjCurrent,HSD_CObjGetViewingMtxPtr(mpCObj));
			pCObj = mpCObj;
		}
#endif
		
	}else{
		
		bool	relative;
		Vec		eyepos;
		Mtx		mtx,tm;
		f32		f,n,aspect,roll;
		
		if( mRelativePosition.x==0.0f &&
		    mRelativePosition.y==0.0f &&
		    mRelativePosition.z==0.0f &&
		    mRelativeRotation.x==0.0f &&
		    mRelativeRotation.y==0.0f &&
		    mRelativeRotation.z==0.0f ){
			relative = false;
		}else{
			relative = true;
		}
		if( relative==true ){
			MTXRotRad(mtx,'x',mRelativeRotation.x);
			MTXRotRad( tm,'y',mRelativeRotation.y);
			MTXConcat( tm,mtx,mtx);
			MTXRotRad( tm,'z',mRelativeRotation.z);
			MTXConcat( tm,mtx,mtx);
			MTXTrans ( tm,mRelativePosition.x,mRelativePosition.y,mRelativePosition.z);
			MTXConcat( tm,mtx,mtx);
		}
		
		if( userdef==true ){
			
			if( relative==true ){
				MTXInverse(mtx,mtx);
				MTXConcat (HSD_CObjGetViewingMtxPtr(mpCObj),mtx,mtx);
			}else{
				MTXCopy   (HSD_CObjGetViewingMtxPtr(mpCObj),mtx);
			}
			
			HSD_CObjSetUserDefinedMtx(mpCObjCurrent,mtx);
			
		}else{
			
			HSD_CObjResetUserDefinedMtx(mpCObjCurrent);
			
			HSD_CObjGetEyePosition(mpCObj,&eyepos);
			HSD_CObjGetInterest   (mpCObj,&interest);
			
			ReviseWobble(eyepos,interest);
			
			if( relative==true ){
				MTXMultVec(mtx,&eyepos  ,&eyepos  );
				MTXMultVec(mtx,&interest,&interest);
			}
			
			HSD_CObjSetEyePosition(mpCObjCurrent,&eyepos);
			HSD_CObjSetInterest   (mpCObjCurrent,&interest);
			
		}
		
		f      = HSD_CObjGetFar   (mpCObj);
		n      = HSD_CObjGetNear  (mpCObj);
		aspect = HSD_CObjGetAspect(mpCObj);
		roll   = HSD_CObjGetRoll  (mpCObj);
#ifdef	_DEBUG
		if( GETCAPTUREMODE() ){
			aspect *= 10.0f;
			aspect /=  9.0f;
		}
#endif
		HSD_CObjSetFar   (mpCObjCurrent,f     );
		HSD_CObjSetNear  (mpCObjCurrent,n     );
		HSD_CObjSetAspect(mpCObjCurrent,aspect);
		HSD_CObjSetRoll  (mpCObjCurrent,roll  );
		
		HSD_CObjSetFov(mpCObjCurrent,HSD_CObjGetFov(mpCObj));
		
		pCObj = mpCObjCurrent;
		
	}
	
	if( HSD_CObjSetCurrent(pCObj)!=true ){
		return false;
	}
	
	return true;
}

bool WK7Cam::EndCurrent(void)
{
	HSD_CObjEndCurrent();
	return true;
}

void WK7Cam::SetFov(float fov,bool delay)
{
	mFov      = fov;
	mFovDirty = 1;
	if( mpInterpolateInfo!=NULL ){
		mpInterpolateInfo->mFov = fov;
	}
	if( mpDelayInterpolateInfo!=NULL ){
		mpDelayInterpolateInfo->mFov = fov;
	}
	if( mpCircleInterpolateInfo!=NULL ){
		mpCircleInterpolateInfo->mFov = fov;
	}
	if( delay==false ){
		HSD_CObjSetFov(mpCObj,mFov);
	}
}

bool WK7Cam::SetControlPath(WK7Path *pPath)
{
	mpControlPath = pPath;
	if( mpPathFollowInfo!=NULL ){
		mpPathFollowInfo->SetControlPath(pPath);
	}
	return true;
}

bool WK7Cam::SetCurrentPath(WK7PathNurbs *pPath,f32 param,u32 dir)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	return mpPathFollowInfo->SetCurrentPath(pPath,param,dir);
}

bool WK7Cam::SetCurrentPath(int dir)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	return mpPathFollowInfo->SetCurrentPath(dir);
}

bool WK7Cam::SetBranchPointMode(bool flag,bool im)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	mpPathFollowInfo->SetBranchPointMode(flag,im);
	return true;
}

bool WK7Cam::GetBranchPointMode(void)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	return mpPathFollowInfo->GetBranchPointMode();
}

bool WK7Cam::SetBackMirrorMode(bool flag)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	mpPathFollowInfo->SetBackMirrorMode(flag);
	return true;
}

bool WK7Cam::ReversePathFollow(void)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	mpPathFollowInfo->Reverse();
	return true;
}

bool WK7Cam::GetBackMirrorMode(void)
{
	if( mpPathFollowInfo==NULL ){
		return false;
	}
	return mpPathFollowInfo->GetBackMirrorMode();
}

bool WK7Cam::SetAspect(f32 aspect)
{
	if( mpCObj==NULL ){
		return false;
	}
	HSD_CObjSetAspect(mpCObj,aspect);
	return true;
}

f32 WK7Cam::GetAspect(void)
{
	if( mpCObj==NULL ){
		return 0.0f;
	}
	return HSD_CObjGetAspect(mpCObj);
}

bool WK7CamInterpolateInfo::FrameProcess(WK7Cam *pCamera)
{
	HSD_CObj	*pCObj;
	if( pCamera==NULL || (pCObj=pCamera->_GetCObj())==NULL || mInterpolateTotalFrame<=0.0f ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	f32		animrate = WK7Motion::ConvMotionRate(pCamera->GetMotionRate());
	
	// ĐwbhXV
	mInterpolateCurrentFrame += animrate;
	
	// ԏI`FbN
	if( mInterpolateCurrentFrame > mInterpolateTotalFrame ){
		HSD_CObjResetUserDefinedMtx(pCObj);
		return false;
	}
	
	Vec		pos,up,interest;
	f32		roll,croll;
	f32		rate,_rate;
	
	rate  = mInterpolateCurrentFrame / mInterpolateTotalFrame;
	_rate = 1.0f - rate;
	
	croll = HSD_CObjGetRoll(pCObj);
	
	HSD_CObjGetEyePosition (pCObj,&pos);
	HSD_CObjGetInterest    (pCObj,&interest);
	
	pos     .x      = pos     .x * rate + mPrevPosition.x * _rate;
	pos     .y      = pos     .y * rate + mPrevPosition.y * _rate;
	pos     .z      = pos     .z * rate + mPrevPosition.z * _rate;
	interest.x      = interest.x * rate + mPrevInterest.x * _rate;
	interest.y      = interest.y * rate + mPrevInterest.y * _rate;
	interest.z      = interest.z * rate + mPrevInterest.z * _rate;
	roll            = croll      * rate + mPrevRoll       * _rate;
	mInterpolateFov = mFov       * rate + mPrevFov        * _rate;
	
#if 0
	Vec		dir,crs;
	Mtx		m;
	up      .x = 0.0f;
	up      .y = 1.0f;
	up      .z = 0.0f;
	VECSubtract    (&interest,&pos,&dir);
	VECNormalize   (&dir,&dir);
	VECCrossProduct(&up,&dir,&crs);
	VECNormalize   (&crs,&crs);
	VECCrossProduct(&dir,&crs,&up);
	MTXRotAxisRad  (m,&dir,-roll);
	MTXMultVecSR   (m,&up,&up);
	VECNormalize   (&up,&up);
#endif
	
	pCamera->ReviseWobble(pos,interest);
	
	ghmMath::Roll2UpVec(pos,interest,roll,up);
	
	MTXLookAt(mInterpolateMatrix,&pos,&up,&interest);
	mInterpolateInterest = interest;
	mInterpolatePosition = pos;
	
	HSD_CObjSetRoll(pCObj,croll);
	
	return true;
}

bool WK7CamInterpolateInfo::SetCurrent(WK7Cam *pCamera)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		return false;
	}
	HSD_CObjSetUserDefinedMtx(pCObj,mInterpolateMatrix);
	HSD_CObjSetFov           (pCObj,mInterpolateFov);
	return true;
}

bool WK7CamInterpolateInfo::Initialize(WK7Cam *pCamera,f32 totalFrame)
{
	if( pCamera==NULL ){
		return false;
	}
	HSD_CObj	*pCObj = pCamera->_GetCObj();
	if( pCObj==NULL ){
		mInterpolateTotalFrame   = 0.0f;
		mInterpolateCurrentFrame = 0.0f;
		return false;
	}
	HSD_CObjGetEyePosition(pCObj,&mPrevPosition);
	HSD_CObjGetInterest   (pCObj,&mPrevInterest);
	mPrevRoll                = HSD_CObjGetRoll(pCObj);
	mPrevFov                 = HSD_CObjGetFov (pCObj);
	mFov                     = mPrevFov;
	mInterpolateFov          = mPrevFov;
	mInterpolateTotalFrame   = totalFrame;
	mInterpolateCurrentFrame = 0.0f;
	mInterpolateInterest     = mPrevInterest;
	mInterpolatePosition     = mPrevPosition;
	HSD_CObjGetViewingMtx (pCObj,mInterpolateMatrix);
	return true;
}

bool WK7Cam::StartLinearInterpolateOnLinearPath(const Vec &pos,const Vec &tgt,f32 totalFrame,f32 fov)
{
	if( mpInterpolateInfo==NULL ){
		ghmGcMemSetAllocName("WK7CamIF");
		mpInterpolateInfo = new WK7CamInterpolateInfo;
		if( mpInterpolateInfo==NULL ){
			return false;
		}
	}
	SetMode(WK7CAM_LINIP_LINPATH);
	HSD_CObjSetEyePosition (mpCObj,&mPosition);
	HSD_CObjSetInterest    (mpCObj,&mTargetPosition);
	HSD_CObjSetFov         (mpCObj,mFov);
	mpInterpolateInfo->Initialize(this,totalFrame);
	SetPosition      (pos);
	SetTargetPosition(tgt);
	if( fov > 0.0f ){
		SetFov(fov);
	}
	return true;
}

bool WK7Cam::StartDelayInterpolateOnLinearPath(const Vec &pos,const Vec &tgt,f32 delayRate,f32 fov)
{
	if( mpDelayInterpolateInfo==NULL ){
		ghmGcMemSetAllocName("WK7CamDIF");
		mpDelayInterpolateInfo = new WK7CamDelayInterpolateInfo;
		if( mpDelayInterpolateInfo==NULL ){
			return false;
		}
	}
	SetMode(WK7CAM_DELAYIP_LINPATH);
	HSD_CObjSetEyePosition (mpCObj,&mPosition);
	HSD_CObjSetInterest    (mpCObj,&mTargetPosition);
	HSD_CObjSetFov         (mpCObj,mFov);
	mpDelayInterpolateInfo->Initialize(this,delayRate);
	SetPosition      (pos);
	SetTargetPosition(tgt);
	if( fov > 0.0f ){
		SetFov(fov);
	}
	return true;
}

bool WK7Cam::StartLinearInterpolateOnCirclePath(const Vec &pos,const Vec &tgt,f32 totalFrame,f32 fov)
{
	if( mpCircleInterpolateInfo==NULL ){
		ghmGcMemSetAllocName("WK7CamCIF");
		mpCircleInterpolateInfo = new WK7CamCircleInterpolateInfo;
		if( mpCircleInterpolateInfo==NULL ){
			return false;
		}
	}
	HSD_CObjSetEyePosition (mpCObj,&mPosition);
	HSD_CObjSetInterest    (mpCObj,&mTargetPosition);
	HSD_CObjSetFov         (mpCObj,mFov);
	mpCircleInterpolateInfo->Initialize(this,totalFrame);
	SetMode(WK7CAM_LINIP_CIRPATH);
	SetPosition      (pos);
	SetTargetPosition(tgt);
	if( fov > 0.0f ){
		SetFov(fov);
	}
	return true;
}

bool WK7Cam::StopInterpolate(void)
{
	if( mpInterpolateInfo!=NULL ){
		SetPosition      (mpInterpolateInfo->mInterpolatePosition);
		SetTargetPosition(mpInterpolateInfo->mInterpolateInterest);
		SetFov           (mpInterpolateInfo->mInterpolateFov     );
		delete mpInterpolateInfo;
		mpInterpolateInfo = NULL;
	}
	if( mpDelayInterpolateInfo!=NULL ){
		SetPosition      (mpDelayInterpolateInfo->mPrevPosition);
		SetTargetPosition(mpDelayInterpolateInfo->mPrevInterest);
		SetFov           (mpDelayInterpolateInfo->mPrevFov     );
		delete mpDelayInterpolateInfo;
		mpDelayInterpolateInfo = NULL;
	}
	if( mpCircleInterpolateInfo!=NULL ){
		SetPosition      (mpCircleInterpolateInfo->mInterpolatePosition);
		SetTargetPosition(mpCircleInterpolateInfo->mInterpolateInterest);
		SetFov           (mpCircleInterpolateInfo->mInterpolateFov     );
		delete mpCircleInterpolateInfo;
		mpCircleInterpolateInfo = NULL;
	}
	HSD_CObjSetEyePosition     (mpCObj,&mPosition);
	HSD_CObjSetInterest        (mpCObj,&mTargetPosition);
	HSD_CObjSetFov             (mpCObj,mFov);
	HSD_CObjResetUserDefinedMtx(mpCObj);
	SetMode(WK7CAM_FREE_TARGET);
	return true;
}

MtxPtr WK7Cam::GetViewingMtxPtr(void)
{
	if( mpCObj==NULL ){
		return NULL;
	}
	return HSD_CObjGetViewingMtxPtr(mpCObj);
}

bool WK7Cam::GetViewingMtx(Mtx mtx)
{
	if( mpCObj==NULL ){
		return false;
	}
	HSD_CObjGetViewingMtx(mpCObj,mtx);
	return true;
}

MtxPtr WK7Cam::GetViewingMtxPtrCurrent(void)
{
	if( mpCObjCurrent==NULL ){
		return NULL;
	}
	return HSD_CObjGetViewingMtxPtr(mpCObjCurrent);
}

bool WK7Cam::GetViewingMtxCurrent(Mtx mtx)
{
	if( mpCObjCurrent==NULL ){
		return false;
	}
	HSD_CObjGetViewingMtx(mpCObjCurrent,mtx);
	return true;
}

bool WK7Cam::GetWorldPosition(Vec& pos)
{
	if( mRelativePosition.x==0.0f &&
	    mRelativePosition.y==0.0f &&
	    mRelativePosition.z==0.0f &&
	    mRelativeRotation.x==0.0f &&
	    mRelativeRotation.y==0.0f &&
	    mRelativeRotation.z==0.0f ){
		pos = mPosition;
		return true;
	}
	Mtx		mtx,tm;
	MTXRotRad(mtx,'x',mRelativeRotation.x);
	MTXRotRad( tm,'y',mRelativeRotation.y);
	MTXConcat( tm,mtx,mtx);
	MTXRotRad( tm,'z',mRelativeRotation.z);
	MTXConcat( tm,mtx,mtx);
	MTXTrans ( tm,mRelativePosition.x,mRelativePosition.y,mRelativePosition.z);
	MTXConcat( tm,mtx,mtx);
	MTXMultVec(mtx,&mPosition,&pos);
	return true;
}

bool WK7Cam::GetWorldTargetPosition(Vec& pos)
{
	if( mRelativePosition.x==0.0f &&
	    mRelativePosition.y==0.0f &&
	    mRelativePosition.z==0.0f &&
	    mRelativeRotation.x==0.0f &&
	    mRelativeRotation.y==0.0f &&
	    mRelativeRotation.z==0.0f ){
		pos = mTargetPosition;
		return true;
	}
	Mtx		mtx,tm;
	MTXRotRad(mtx,'x',mRelativeRotation.x);
	MTXRotRad( tm,'y',mRelativeRotation.y);
	MTXConcat( tm,mtx,mtx);
	MTXRotRad( tm,'z',mRelativeRotation.z);
	MTXConcat( tm,mtx,mtx);
	MTXTrans ( tm,mRelativePosition.x,mRelativePosition.y,mRelativePosition.z);
	MTXConcat( tm,mtx,mtx);
	MTXMultVec(mtx,&mTargetPosition,&pos);
	return true;
}

bool WK7Cam::GetWorldPositions(Vec& pos,Vec& tgt)
{
	if( mRelativePosition.x==0.0f &&
	    mRelativePosition.y==0.0f &&
	    mRelativePosition.z==0.0f &&
	    mRelativeRotation.x==0.0f &&
	    mRelativeRotation.y==0.0f &&
	    mRelativeRotation.z==0.0f ){
		pos = mPosition;
		tgt = mTargetPosition;
		return true;
	}
	Mtx		mtx,tm;
	MTXRotRad(mtx,'x',mRelativeRotation.x);
	MTXRotRad( tm,'y',mRelativeRotation.y);
	MTXConcat( tm,mtx,mtx);
	MTXRotRad( tm,'z',mRelativeRotation.z);
	MTXConcat( tm,mtx,mtx);
	MTXTrans ( tm,mRelativePosition.x,mRelativePosition.y,mRelativePosition.z);
	MTXConcat( tm,mtx,mtx);
	MTXMultVec(mtx,&mPosition      ,&pos);
	MTXMultVec(mtx,&mTargetPosition,&tgt);
	return true;
}

