////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Effect
//
// WK7Effect.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 03/10/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7EFFECT_CPP__

//#include <dolphin.h>
#include ".\WK7Effect.h"
#include ".\WK7Define.h"

Mtx				WK7Fade::mView;
Mtx44			WK7Fade::mProj;

WK7Fade			gMask;

WK7Fade::WK7Fade()
{
	mMatColor.r = 0x00;
	mMatColor.g = 0x00;
	mMatColor.b = 0x00;
	mMatColor.a = 0xff;
	mType       = BLEND_TYPE;
}

bool WK7Fade::Initialize(void)
{
	Vec				vPos;
	Vec				vUp;
	Vec				vAt;
	
	vUp .x =    0.0f;
	vUp .y = -  1.0f;
	vUp .z =    0.0f;
	vPos.x =    0.0f;
	vPos.y =    0.0f;
	vPos.z =    0.0f;
	vAt .x =  vPos.x;
	vAt .y =  vPos.y;
	vAt .z =  300.0f;
	
	// }gNXݒ
	MTXLookAt(mView,&vPos,&vUp,&vAt);
	MTXOrtho (mProj,0,-WK7_SCREEN_HEIGHT,0,WK7_SCREEN_WIDTH,0.0f,1024.0f);
	
	gMask.Start(0,0,0);
	
	return true;
}

bool WK7Fade::FrameProcess(void)
{
	mAlpha.FrameProcess();
	return true;
}

bool WK7Fade::RenderProcess(void)
{
	mMatColor.a = (u8)mAlpha.GetValue();
	
	if( mMatColor.a==0 ){
		return true;
	}
	
	GXLoadPosMtxImm (mView,GX_PNMTX0);
	GXLoadNrmMtxImm (mView,GX_PNMTX0);
	GXSetCurrentMtx (GX_PNMTX0);
	GXSetProjection (mProj,GX_ORTHOGRAPHIC);
	
	GXSetScissor    (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport   (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
//	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	switch( mType ){
	case	ADD_TYPE:
		GXSetBlendMode(GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_ONE,GX_LO_CLEAR);
		break;
	case	BLEND_TYPE:
	default:
		GXSetBlendMode(GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
		break;
	}
	
	GXSetCullMode     (GX_CULL_NONE);
	
	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	
	GXClearVtxDesc    ();
	GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc      (GX_VA_NRM ,GX_NONE  );
	GXSetVtxDesc      (GX_VA_CLR0,GX_NONE  );
	GXSetVtxDesc      (GX_VA_TEX0,GX_NONE  );
	
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS,GX_POS_XY,GX_U16,0);
		
	GXSetNumTexGens   (0);
	
	GXSetNumTevStages (1);
	GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	GXSetChanMatColor(GX_COLOR0A0,mMatColor);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition2u16(0               ,0                );
		GXPosition2u16(WK7_SCREEN_WIDTH,0                );
		GXPosition2u16(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
		GXPosition2u16(0               ,WK7_SCREEN_HEIGHT);
		
	GXEnd();
	
	return true;
}

void WK7Fade::Start(u32 alpha,u32 time)
{
	mAlpha.Play(alpha,time);
}

void WK7Fade::Start(u32 alpha,u32 color,u32 time)
{
	mAlpha.Play(alpha,time);
	*((u32 *)&mMatColor) = color;
}

void WK7Fade::SetColor(u32 color)
{
	*((u32 *)&mMatColor) = color;
}

u32 WK7Fade::GetColor(void)
{
	return *((u32 *)&mMatColor);
}

WK7Fade::State WK7Fade::GetState(void)
{
	if( mAlpha.IsAnimating()==true ){
		return WK7Fade::FADE;
	}
	return WK7Fade::IDLE;
}

bool WK7Fade::SetType(Type type)
{
	switch( type ){
	case	BLEND_TYPE:
	case	ADD_TYPE:
		break;
	default:
		return false;
	}
	mType = type;
	return true;
}

