////////////////////////////////////////////////////////////////////////////////
//
// Killer7 EffectBreakObj
//
// WK7EffectBreakObj.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 03/24/2004
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7EFFECTBREAKOBJ_CPP__

#include ".\WK7EffectBreakObj.h"
#include ".\WK7Cam.h"

f32				WK7EffectBreakFace::mLimitSurfaceSize = -1.0f;

ghmGcListObj	WK7EffectBreakObj::mAnchor;
Mtx				WK7EffectBreakObj::mViewMatrix;

WK7EffectBreakObj* WK7EffectBreakObj::Create(WK7JObj *pJObj,bool next,bool child,int level)
{
	if( pJObj==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("WK7EFBO");
	WK7EffectBreakObj	*pObj = new WK7EffectBreakObj(pJObj,next,child,level);
	WK7EffectBreakFace::SetLimitSurfaceSize(-1.0f);
	return pObj;
}

bool WK7EffectBreakObj::Release(WK7EffectBreakObj *pObj)
{
	if( pObj==NULL ){
		return false;
	}
	if( Find(pObj)==false ){
		return false;
	}
	delete pObj;
	return true;
}

bool WK7EffectBreakObj::Find(WK7EffectBreakObj *pObj)
{
	WK7EffectBreakObj	*pFindObj;
	for(pFindObj=(WK7EffectBreakObj *)mAnchor.GetNext() ; pFindObj!=NULL ; pFindObj=pFindObj->GetNext()){
		if( pFindObj==pObj ){
			return true;
		}
	}
	return false;
}

bool WK7EffectBreakObj::IsValid(WK7EffectBreakObj *pObj)
{
	return Find(pObj);
}

WK7EffectBreakObj::WK7EffectBreakObj(WK7JObj *pJObj,bool next,bool child,int level) : ghmGcListObj(&mAnchor)
{
	mMeshCount = 0;
	mState     = WK7EFFECTBREAKOBJ_IDLE;
	mVisible   = false;
	mAlpha.SetValue(1.0f);
	ParseJObj(pJObj,next,child,level);
}

WK7EffectBreakObj::~WK7EffectBreakObj()
{
}

bool WK7EffectBreakObj::ParseDObj(HSD_DObj *pDObj,Mtx mtx,int level)
{
	ghmGcMemSetAllocName("WK7EFBM");
	WK7EffectBreakMesh	*pMesh = new WK7EffectBreakMesh(&mMeshAnchor,this,pDObj,mtx,level);
	if( pMesh==NULL ){
		return false;
	}
	mMeshCount++;
	return true;
}

bool WK7EffectBreakObj::ParseJObj(HSD_JObj *pJObj,int level)
{
	if( pJObj==NULL ){
		return false;
	}
	HSD_DObj		*pDObj;
	Mtx				mtx;
	MTXCopy(HSD_JObjGetMtxPtr(pJObj),mtx);
	
//	mtx[0][3] = 
//	mtx[1][3] = 
//	mtx[2][3] = 0.0f;
	
	for(pDObj=HSD_JObjGetDObj(pJObj) ; pDObj!=NULL ; pDObj=HSD_DObjGetNext(pDObj)){
		ParseDObj(pDObj,mtx,level);
	}
	return true;
}

bool WK7EffectBreakObj::ParseJObj(WK7JObj *pJObj,bool next,bool child,int level)
{
	if( pJObj==NULL ){
		return false;
	}
	ParseJObj(pJObj->GetJObj(),level);
	if( child==true ){
		ParseJObj(pJObj->GetChild(),true,true,level);
	}
	if( next==true ){
		for(pJObj=pJObj->GetNext() ; pJObj!=NULL ; pJObj=pJObj->GetNext()){
			ParseJObj(pJObj,false,child,level);
		}
	}
	return true;
}

void WK7EffectBreakObj::Clear(void)
{
	WK7EffectBreakObj		*pObj,*pNext;
	for(pObj=(WK7EffectBreakObj *)mAnchor.GetNext() ; pObj!=NULL ; pObj=pNext){
		pNext = pObj->GetNext();
		delete pObj;
	}
}

bool WK7EffectBreakObj::FrameProcessAll(void)
{
	WK7EffectBreakObj		*pObj,*pNext;
	for(pObj=(WK7EffectBreakObj *)mAnchor.GetNext() ; pObj!=NULL ; pObj=pNext){
		pNext = pObj->GetNext();
		if( pObj->FrameProcess()==false ){
			delete pObj;
		}
	}
	return true;
}

bool WK7EffectBreakObj::RenderProcessAll(void)
{
	WK7EffectBreakObj		*pObj;
	WK7CAM->GetViewingMtxCurrent(mViewMatrix);
	for(pObj=(WK7EffectBreakObj *)mAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		pObj->RenderProcess();
	}
	HSD_StateInvalidate(HSD_STATE_ALL);
	return true;
}

bool WK7EffectBreakObj::Start(u32 tick)
{
	if( mState == WK7EFFECTBREAKOBJ_TERM ){
		return false;
	}
	mAlpha.Play(0.0f,tick);
	mState = WK7EFFECTBREAKOBJ_EXEC;
	return true;
}

bool WK7EffectBreakObj::Stop(void)
{
	if( mState == WK7EFFECTBREAKOBJ_TERM ){
		return false;
	}
	mState = WK7EFFECTBREAKOBJ_IDLE;
	return true;
}

bool WK7EffectBreakObj::IsBroken(void)
{
	if( mState != WK7EFFECTBREAKOBJ_TERM ){
		return false;
	}
	return true;
}

bool WK7EffectBreakObj::FrameProcess(void)
{
	WK7EffectBreakMesh	*pMesh,*pNext;
	switch( mState ){
	case	WK7EFFECTBREAKOBJ_IDLE:
		return true;
	case	WK7EFFECTBREAKOBJ_EXEC:
		if( mAlpha.GetValue()==0.0f ){
			mState = WK7EFFECTBREAKOBJ_TERM;
			return true;
		}
		break;
	case	WK7EFFECTBREAKOBJ_TERM:
	default:
		return false;
	}
	mAlpha.FrameProcess();
	for(pMesh=(WK7EffectBreakMesh *)mMeshAnchor.GetNext() ; pMesh!=NULL ; pMesh=pNext){
		pNext = pMesh->GetNext();
		if( pMesh->FrameProcess()==false ){
			delete pMesh;
		}
	}
	return true;
}

bool WK7EffectBreakObj::RenderProcess(void)
{
	if( mState == WK7EFFECTBREAKOBJ_TERM ){
		return true;
	}
	if( mVisible==false ){
		return true;
	}
	WK7EffectBreakMesh	*pMesh;
	for(pMesh=(WK7EffectBreakMesh *)mMeshAnchor.GetNext() ; pMesh!=NULL ; pMesh=pMesh->GetNext()){
		pMesh->RenderProcess();
	}
	return true;
}

WK7EffectBreakMesh::WK7EffectBreakMesh(ghmGcListObj *pAnchor,WK7EffectBreakObj *pContainer,HSD_DObj *pDObj,Mtx mtx,int level) : ghmGcListObj(pAnchor)
{
	mpContainer = pContainer;
	mFaceCount  = 0;
//	mState      = 0;
	mpMObj      = NULL;
	mpTObj      = NULL;
	mpRampTObj  = NULL;
	if( (mpDObj=pDObj)!=NULL ){
		if( (mpMObj = mpDObj->mobj)!=NULL ){
			mpTObj     = mpMObj->tobj;
			mpRampTObj = mpMObj->tobj_gradation;
		}
		ParseDObj(pDObj,mtx,level);
	}
}

WK7EffectBreakMesh::~WK7EffectBreakMesh()
{
}

bool WK7EffectBreakMesh::ParseDObj(HSD_DObj *pDObj,Mtx mtx,int level)
{
	HSD_PObj	*pPObj;
	for(pPObj=HSD_DObjGetPObj(pDObj) ; pPObj!=NULL ; pPObj=HSD_PObjGetNext(pPObj)){
		ParsePObj(pPObj,mtx,level);
	}
	return true;
}

bool WK7EffectBreakMesh::CreateFace(Mtx mtx,Vec *pPos,WK7St *pSt,int level)
{
	
	int		i;
	
	if( level > 0 ){
		
		Vec		pos  [3];
		WK7St	st   [3];
		int		index[3];
		int		divindex;
		f32		len,lenmax;
		WK7St	*pst;
		bool	div = true;
		
		level--;
		
		VECSubtract(pPos + 1,pPos + 0,pos + 0);
		VECSubtract(pPos + 2,pPos + 1,pos + 1);
		
		if( WK7EffectBreakFace::mLimitSurfaceSize > 0.0f ){
			
			VECCrossProduct(pos + 0,pos + 1,pos + 2);
			
			if( VECMag(pos + 2) < WK7EffectBreakFace::mLimitSurfaceSize ){
				div = false;
			}
		}
		
		if( div==true ){
			
			VECSubtract(pPos + 0,pPos + 2,pos + 2);
			
			divindex  = 0;
			lenmax = VECMag(pos+0);
			for(i=1 ; i<3 ; i++){
				len = VECMag(pos+i);
				if( lenmax < len ){
					divindex = i;
					lenmax   = len;
				}
			}
			
			if( pSt!= NULL ){
				pst = st;
			}else{
				pst = NULL;
			}
			
			index[0] = (divindex + 1) % 3;
			for(i=0 ; i<2 ; i++){
				switch( i ){
				case	0:
					index[1] = (divindex + 2) % 3;
					index[2] =  divindex;
					break;
				case	1:
				default:
					index[1] = (divindex + 1) % 3;
					index[2] = (divindex + 2) % 3;
					break;
				}
				VECAdd  (pPos + divindex,pPos + index[0],pos);
				VECScale(pos,pos,0.5f);
				pos[1] = pPos[index[1]];
				pos[2] = pPos[index[2]];
				
				if( pst!=NULL ){
					WK7St	*pst0,*pst1;
					pst0    = pSt + divindex;
					pst1    = pSt + index[0];
					st[0].s = (pst0->s + pst1->s) * 0.5f;
					st[0].t = (pst0->t + pst1->t) * 0.5f;
					st[1]   = pSt[index[1]];
					st[2]   = pSt[index[2]];
				}
				
				if( CreateFace(mtx,pos,pst,level)==false ){
					return false;
				}
			}
			
			return true;
			
		}
		
	}
	
	Vec		vertex[3];
	Vec		pos;
	Vec		vel;
	Vec		rot;
	Vec		rotvel;
	Vec		norm,dir[3];
	
	vel   .x =  0.0f;
	vel   .y =  0.0f;
	vel   .z =  0.0f;
	
	rotvel.x =  0.0f;
	rotvel.y =  0.0f;
	rotvel.z =  0.0f;
	
	MTXMultVec(mtx,pPos + 0,vertex + 0);
	pos = vertex[0];
	for(i=1 ; i<3 ; i++){
		MTXMultVec(mtx,pPos + i,vertex + i);
		VECAdd    (&pos,vertex+i,&pos);
	}
	VECScale(&pos,&pos,1.0f / 3.0f);
	
	VECSubtract    (vertex + 1,vertex + 0,dir + 0);
	VECSubtract    (vertex + 2,vertex + 0,dir + 1);
	VECCrossProduct(dir + 1,dir + 0,&norm);
	
#define	FLOAT_ERROR		(0.0001f)
	if( VECMag(&norm) < FLOAT_ERROR ){
		return true;
	}
static	Vec		up = { 0.0f,1.0f,0.0f };
	VECCrossProduct(&up,&norm,dir + 0);
	if( VECMag(dir + 0) < FLOAT_ERROR ){
		dir[0].x = 1.0f;
		dir[0].y = 0.0f;
		dir[0].z = 0.0f;
	}else{
		VECNormalize(dir + 0,dir + 0);
	}
	VECCrossProduct(&norm  ,dir + 0,dir + 1);
	VECNormalize   (dir + 1,dir + 1);
	VECCrossProduct(dir + 0,dir + 1,dir + 2);
	
static	f32	initvelz_max  = 10.0f;
//static	f32	initvelxy_max = 30.0f;
static	f32	initrvel_max  =  3.0f;
//	f32		initvely;
	f32		initvelz;
	f32		initrvelx;
	f32		initrvely;
//	initvely  =  HSD_Randf() * initvely_max;
	initvelz  =  HSD_Randf() * initvelz_max;
	initrvelx = (HSD_Randf() * 2.0f - 1.0f) * initrvel_max;
	initrvely = (HSD_Randf() * 2.0f - 1.0f) * initrvel_max;
	VECScale(dir + 2,&vel,initvelz * WK7_INV_FRAME_RATE);
//	vel   .y = initvely * WK7_INV_FRAME_RATE;
	rotvel.x = initrvelx * WK7_INV_FRAME_RATE;
	rotvel.y = initrvely * WK7_INV_FRAME_RATE;
	
	rot.x = -atan2f(dir[2].y,sqrtf(dir[2].x * dir[2].x + dir[2].z * dir[2].z));
	rot.y =  atan2f(dir[2].x,dir[2].z);
	rot.z =  0.0f;
	
	WK7EffectBreakFace	*pFace;
	if( pSt!=NULL ){
		WK7EffectBreakFaceT		*pFaceT;
		if( mpRampTObj != NULL ){
			ghmGcMemSetAllocName("WKEFBFTR");
			pFaceT = new WK7EffectBreakFaceTR(&mFaceAnchor);
		}else{
			ghmGcMemSetAllocName("WKEFBFT");
			pFaceT = new WK7EffectBreakFaceT(&mFaceAnchor);
		}
		pFace  = pFaceT;
		if( pFaceT!=NULL ){
			for(i=0 ; i<3 ; i++){
				pFaceT->mST[i]	= pSt[i];
			}
		}
	}else{
		if( mpRampTObj != NULL ){
			ghmGcMemSetAllocName("WKEFBFR");
			pFace = new WK7EffectBreakFaceR(&mFaceAnchor);
		}else{
			ghmGcMemSetAllocName("WKEFBF");
			pFace = new WK7EffectBreakFace(&mFaceAnchor);
		}
	}
	if( pFace==NULL ){
		return false;
	}
	
	pFace->mPosition    = pos;
	pFace->mRotation    = rot;
	memset(&pFace->mVelocity   ,0,sizeof(pFace->mVelocity   ));
	memset(&pFace->mRotVelocity,0,sizeof(pFace->mRotVelocity));
	// }gNXݒ
	pFace->FrameProcess();
	
	// ΍Wɕϊ
	Mtx		invm;
	MTXInverse(pFace->mLocalWorldMatrix,invm);
	for(i=0 ; i<3 ; i++){
		MTXMultVec(invm,vertex + i,pFace->mVertex + i);
	}
	pFace->mVelocity    = vel;
	pFace->mRotVelocity = rotvel;
	MTXIdentity(pFace->mLocalWorldMatrix);
	
	mFaceCount++;
	
	return true;
}

bool WK7EffectBreakMesh::ParsePObj(HSD_PObj *pPObj,Mtx mtx,int level)
{
	if( pPObj==NULL ){
		return true;
	}
	
	WK7VtxDescInfo		info;
	u32					stride;
	stride  = WK7JObj::GetStride(pPObj->desc_list,info);
	if( info.mpVertexDesc==NULL ){
		return true;
	}
	
	int					dlsize;
	u8					*pdl;
	int					num,i;
	GXPrimitive			prim;
	WK7GetVertexFunc	*pGetVertex;
	WK7GetStFunc		*pGetSt;
	f32					baseVertex,baseSt;
	Vec					pos[3];
	WK7St				st [3];
	WK7St				*pSt;
	
	pdl     = pPObj->dl;
	dlsize  = (pPObj->dlsize << 5);
	switch( info.mpVertexDesc->comp_type ){
	case	GX_U8:  pGetVertex = WK7JObj::GetVertexU8;  baseVertex = 1.0f / (1<<info.mpVertexDesc->frac); break;
	case	GX_S8:  pGetVertex = WK7JObj::GetVertexS8;  baseVertex = 1.0f / (1<<info.mpVertexDesc->frac); break;
	case	GX_U16: pGetVertex = WK7JObj::GetVertexU16; baseVertex = 1.0f / (1<<info.mpVertexDesc->frac); break;
	case	GX_S16: pGetVertex = WK7JObj::GetVertexS16; baseVertex = 1.0f / (1<<info.mpVertexDesc->frac); break;
	case	GX_F32: pGetVertex = WK7JObj::GetVertexF32; baseVertex = 1.0f;                                break;
	default:
		return true;
	}
	if( info.mpTexDesc!=NULL ){
		switch( info.mpTexDesc->comp_type ){
		case	GX_U8:  pGetSt = WK7JObj::GetStU8;  baseSt = 1.0f / (1<<info.mpTexDesc->frac); break;
		case	GX_S8:  pGetSt = WK7JObj::GetStS8;  baseSt = 1.0f / (1<<info.mpTexDesc->frac); break;
		case	GX_U16: pGetSt = WK7JObj::GetStU16; baseSt = 1.0f / (1<<info.mpTexDesc->frac); break;
		case	GX_S16: pGetSt = WK7JObj::GetStS16; baseSt = 1.0f / (1<<info.mpTexDesc->frac); break;
		case	GX_F32: pGetSt = WK7JObj::GetStF32; baseSt = 1.0f;                             break;
		default:
			return true;
		}
		pSt    = st;
	}else{
		baseSt = 1.0f;
		pGetSt = NULL;
		pSt    = NULL;
	}
	
	while( dlsize > 0 ){
		u8			*pVertexDL;
		u8			*pStDL;
		int			vid,stid;
		void		*pVertexBasePtr;
		void		*pStBasePtr;
		prim    = (GXPrimitive)(pdl[0] & GX_OPCODE_MASK);
		num     = ((int)pdl[1]<<8)|pdl[2];
		i       = 0;
		dlsize -= 3;
		pdl    += 3;
		pVertexDL       = pdl + info.mVertexPos;
		pVertexBasePtr  = info.mpVertexDesc->base_ptr;
		if( pGetSt!=NULL ){
			pStDL      = pdl + info.mTexPos;
			pStBasePtr = info.mpTexDesc->base_ptr;
		}else{
			pStDL      = NULL;
			pStBasePtr = NULL;
		}
		switch( prim ){
		case GX_POINTS:
		case GX_LINES:
		case GX_LINESTRIP:
		case GX_QUADS:
			break;
		case GX_TRIANGLES:
			while( num > 0 ){
				vid = WK7JObj::DlGetIndex(pVertexDL,info.mpVertexDesc);
				pGetVertex(&pos[i],pVertexBasePtr,vid,baseVertex);
				pVertexDL += stride;
				if( pGetSt!=NULL ){
					stid = WK7JObj::DlGetIndex(pStDL,info.mpTexDesc);
					pGetSt(&st[i],pStBasePtr,stid,baseSt);
					pStDL += stride;
				}
				num --;
				if( i==2 ){
					i = 0;
					ghmGcMemSetAllocName("WPFTri");
					if( CreateFace(mtx,pos,pSt,level)==false ){
						return false;
					}
					
				}else{
					i++;
				}
			}
			break;
		case GX_TRIANGLESTRIP:{
			int		pvid[2],pstid[2];
			while( num > 0 ){
				vid = WK7JObj::DlGetIndex(pVertexDL,info.mpVertexDesc);
				pGetVertex(&pos[i],pVertexBasePtr,vid,baseVertex);
				pVertexDL += stride;
				if( pGetSt!=NULL ){
					stid = WK7JObj::DlGetIndex(pStDL,info.mpTexDesc);
					pGetSt(&st[i],pStBasePtr,stid,baseSt);
					pStDL += stride;
				}
				num --;
				if( i==2 ){
					if( pvid[1] != vid && pvid[0] != pvid[1] ){
						ghmGcMemSetAllocName("WPFTrStr");
						if( CreateFace(mtx,pos,pSt,level)==false ){
							return false;
						}
					}
					pos[0] = pos[1];
					pos[1] = pos[2];
					st [0] = st [1];
					st [1] = st [2];
				}else{
					i++;
				}
				pvid [0] = pvid[1];
				pvid [1] = vid;
				pstid[0] = pstid[1];
				pstid[1] = stid;
			}
			}break;
		case GX_TRIANGLEFAN:
			while( num > 0 ){
				vid = WK7JObj::DlGetIndex(pVertexDL,info.mpVertexDesc);
				pGetVertex(&pos[i],pVertexBasePtr,vid,baseVertex);
				pVertexDL += stride;
				if( pGetSt!=NULL ){
					stid = WK7JObj::DlGetIndex(pStDL,info.mpTexDesc);
					pGetSt(&st[i],pStBasePtr,stid,baseSt);
					pStDL += stride;
				}
				num --;
				if( i==2 ){
					
					ghmGcMemSetAllocName("WPFTrFan");
					if( CreateFace(mtx,pos,pSt,level)==false ){
						return false;
					}
					
					pos[1] = pos[2];
					st [1] = st [2];
				}else{
					i++;
				}
			}
			break;
		case GX_NOP:
		default:
			break;
		}
		pdl    += stride * num;
		dlsize -= stride * num;
	}
	
	return true;
}

bool WK7EffectBreakMesh::FrameProcess(void)
{
	WK7EffectBreakFace	*pFace,*pNext;
	for(pFace=(WK7EffectBreakFace *)mFaceAnchor.GetNext() ; pFace!=NULL ; pFace=pNext){
		pNext = pFace->GetNext();
		if( pFace->FrameProcess()==false ){
			delete pFace;
		}
	}
	return true;
}

bool WK7EffectBreakMesh::RenderProcess(void)
{
	if( mpMObj==NULL || mpContainer==NULL ){
		return false;
	}
	
//static		GXColor			matcolor = { 0xff , 0xff , 0xff , 0x80 };
	GXColor			matcolor;
	
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetDither      (GX_TRUE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	
	int		alpha = (int)(mpContainer->GetAlpha() * 0xff);
	GHM_LIMIT(alpha,0xff,0x00);
	matcolor.a = (u8)alpha;
//	matcolor.a = 0xff;
	matcolor.r = mpMObj->mat->diffuse.r;
	matcolor.g = mpMObj->mat->diffuse.g;
	matcolor.b = mpMObj->mat->diffuse.b;
	GXSetChanMatColor(GX_COLOR0A0,matcolor);
	
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_TRUE,GX_LEQUAL,GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	
	u8				texGenNum   = 0;
	u8				tevStageNum = 1;
	GXTexCoordID	texCoord    = GX_TEXCOORD0;
	
	if( mpTObj==NULL ){
		
		GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
		GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
		GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
		GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		
	}else{
	    
	    HSD_ImageDesc	*pImageDesc = mpTObj->imagedesc;
	    
	    if( pImageDesc==NULL ){
			return false;
		}
		
		GXTexObj	texObj;
		
		GXSetVtxDesc     (GX_VA_TEX0,GX_DIRECT);
		GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
		
		GXInitTexObj     (&texObj,pImageDesc->image_ptr,
		                          pImageDesc->width,
		                          pImageDesc->height, 
		                          pImageDesc->format, 
		                          mpTObj->wrap_s,
		                          mpTObj->wrap_t,
		                          GX_FALSE);
		GXInitTexObjLOD  (&texObj,GX_LINEAR,
		                          mpTObj->magFilt,
		                          pImageDesc->minLOD,
		                          pImageDesc->maxLOD,
		                          0.0f,
		                          GX_FALSE,
		                          GX_FALSE,
		                          GX_ANISO_1);
		GXLoadTexObj     (&texObj,GX_TEXMAP0);
		
		GXSetTexCoordGen (texCoord,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
		
		GXSetTevOrder    (GX_TEVSTAGE0,texCoord,GX_TEXMAP0,GX_COLOR0A0);
		GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
		GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_RASA,GX_CA_TEXA,GX_CA_ZERO);
		GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		
		texGenNum++;
		texCoord = (GXTexCoordID)((int)texCoord + 1);
	}
	
    if( mpRampTObj!=NULL ){
		
	    HSD_ImageDesc	*pRampImageDesc = mpRampTObj->imagedesc;
	    
	    if( pRampImageDesc==NULL ){
			return false;
		}
		
		GXTexObj	rampTexObj;
		Mtx			mtx;
		
		GXInitTexObj     (&rampTexObj,pRampImageDesc->image_ptr,
		                              pRampImageDesc->width,
		                              pRampImageDesc->height, 
		                              pRampImageDesc->format, 
		                              mpRampTObj->wrap_s,
		                              mpRampTObj->wrap_t,
		                              GX_FALSE);
		GXInitTexObjLOD  (&rampTexObj,GX_LINEAR,
		                              mpRampTObj->magFilt,
		                              pRampImageDesc->minLOD,
		                              pRampImageDesc->maxLOD,
		                              0.0f,
		                              GX_FALSE,
		                              GX_FALSE,
		                              GX_ANISO_1);
		GXLoadTexObj     (&rampTexObj,GX_TEXMAP1);
		
		GXSetTexCoordGen2(texCoord,
		                  GX_TG_MTX3x4,
		                  GX_TG_POS,
		                  GX_PNMTX0,
		                  GX_DISABLE,
		                  GX_PTTEXMTX0);
		
		//JA≺ĂꍇB
		//eNX`SWɃItZbgB
		//jf̓[hYɍsB
		Vec		eye,y={0.f,1.f,0.f};
		f32		dpro,ofst=0.5f,th=0.33f,w=0.5f;
		Mtx		vmtx;
		eye.x =  0.0f;
		eye.y =  0.0f;
		eye.z = -1.0f;
		MTXTranspose(WK7EffectBreakObj::GetViewMatrix(),vmtx);
		MTXMultVecSR(vmtx,&eye,&eye);
		dpro = VECDotProduct( &eye, &y );
		//30xȏjAɊ蓖ĂB
		if( dpro > th ){
			ofst += (w * (dpro-th) / (1.0-th));
		}else if( dpro < -th ){
			dpro *= -1;
			ofst -= (w * (dpro-th) / (1.0-th));
		}
		MTXLightPerspective(mtx,WK7CAM->GetFov(),WK7CAM->GetAspect(),0.5f,0.5f,0.5f,ofst);
		GXLoadTexMtxImm    (mtx,GX_PTTEXMTX0,GX_MTX3x4);
		
		GXSetTevOrder    (GX_TEVSTAGE1,texCoord,GX_TEXMAP1,GX_COLOR_NULL);
		GXSetTevColorIn  (GX_TEVSTAGE1,GX_CC_CPREV,GX_CC_TEXC,GX_CC_TEXA,GX_CC_ZERO);
		GXSetTevColorOp  (GX_TEVSTAGE1,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn  (GX_TEVSTAGE1,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_APREV);
		GXSetTevAlphaOp  (GX_TEVSTAGE1,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		
		tevStageNum++;
		texGenNum  ++;
		texCoord = (GXTexCoordID)((int)texCoord + 1);
		
	}
	
	GXSetNumTexGens  (texGenNum  );
	GXSetNumTevStages(tevStageNum);
	
	WK7EffectBreakFace	*pFace;
	for(pFace=(WK7EffectBreakFace *)mFaceAnchor.GetNext() ; pFace!=NULL ; pFace=pFace->GetNext()){
		pFace->RenderProcess();
	}
	return true;
}

WK7EffectBreakFace::WK7EffectBreakFace(ghmGcListObj *pAnchor) : ghmGcListObj(pAnchor)
{
//	mState = 0;
}

WK7EffectBreakFace::~WK7EffectBreakFace()
{
}

static	f32		accelY = -98.0f * WK7_INV_FRAME_RATE * WK7_INV_FRAME_RATE;
bool WK7EffectBreakFace::FrameProcess(void)
{
	
	mVelocity.y += accelY;
	
	VECAdd   (&mPosition,&mVelocity   ,&mPosition);
	VECAdd   (&mRotation,&mRotVelocity,&mRotation);
	
	MakeMatrix();
	
	return true;
}

bool WK7EffectBreakFace::MakeMatrix(void)
{
	
	Mtx		tm;
	
	MTXRotRad(mLocalWorldMatrix,'z',mRotation.z);
	MTXRotRad(tm               ,'x',mRotation.x);
	MTXConcat(tm,mLocalWorldMatrix,mLocalWorldMatrix);
	MTXRotRad(tm               ,'y',mRotation.y);
	MTXConcat(tm,mLocalWorldMatrix,mLocalWorldMatrix);
	
	MTXTrans (tm,mPosition.x,mPosition.y,mPosition.z);
	MTXConcat(tm,mLocalWorldMatrix,mLocalWorldMatrix);
	
	return true;
}

bool WK7EffectBreakFace::RenderProcess(void)
{
	
	int				i;
	Mtx				mtx;
	
	MTXConcat        (WK7EffectBreakObj::GetViewMatrix(),mLocalWorldMatrix,mtx);
	GXLoadPosMtxImm  (mtx,GX_PNMTX0);
	GXLoadNrmMtxImm  (mtx,GX_PNMTX0);
	
	GXBegin(GX_TRIANGLES,GX_VTXFMT0,3);
	
	for(i=0 ; i<3 ; i++){
		GXPosition3f32(mVertex[i].x,mVertex[i].y,mVertex[i].z);
	}
	
	GXEnd();
	
	return true;
}

WK7EffectBreakFaceT::WK7EffectBreakFaceT(ghmGcListObj *pAnchor) : WK7EffectBreakFace(pAnchor)
{
}

WK7EffectBreakFaceT::~WK7EffectBreakFaceT()
{
}
	
bool WK7EffectBreakFaceT::FrameProcess(void)
{
	return WK7EffectBreakFace::FrameProcess();
}

bool WK7EffectBreakFaceT::RenderProcess(void)
{
	int				i;
	Mtx				mtx;
	
	MTXConcat        (WK7EffectBreakObj::GetViewMatrix(),mLocalWorldMatrix,mtx);
	GXLoadPosMtxImm  (mtx,GX_PNMTX0);
	GXLoadNrmMtxImm  (mtx,GX_PNMTX0);
	
	GXBegin(GX_TRIANGLES,GX_VTXFMT0,3);
	
	for(i=0 ; i<3 ; i++){
		GXPosition3f32(mVertex[i].x,mVertex[i].y,mVertex[i].z);
		GXTexCoord2f32(mST    [i].s,mST    [i].t);
	}
	
	GXEnd();
	
	return true;
}

