////////////////////////////////////////////////////////////////////////////////
//
// Killer7 EventItem
//
// WK7EventItem.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 09/29/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7EVENTITEM_CPP__

//#include <dolphin.h>
#include ".\WK7EventItem.h"
#include ".\WK7Event.h"
#include ".\WK7GameInfo.h"

WK7EventItemTable*		WK7EventItem::mpTable = NULL;
u32						WK7EventItem::mFlag[WK7EVENTITEM_WORKSIZE];

WK7EventItemTable* WK7EventItem::FindTable(u32 itemno)
{
	WK7EventItemTable	*pTable;
	pTable = mpTable;
	if( pTable==NULL ){
		return NULL;
	}
	for( ; pTable->mItemNo!=WK7EVITEMTBL_TERM_ITEMNO ; pTable++){
		if( pTable->mItemNo==itemno ){
			return pTable;
		}
	}
	return NULL;
}

bool WK7EventItem::Initialize(void)
{
	memset(mFlag,0,sizeof(mFlag));
	return true;
}

int WK7EventItem::FindTableID(u32 itemno)
{
	WK7EventItemTable	*pTable;
	pTable = mpTable;
	if( pTable==NULL ){
		return NULL;
	}
	int					id;
	for(id=0 ; pTable->mItemNo!=WK7EVITEMTBL_TERM_ITEMNO ; pTable++,id++){
		if( pTable->mItemNo==itemno ){
			return id;
		}
	}
	return -1;
}

bool WK7EventItem::HasItem(u32 itemno)
{
	int		id = FindTableID(itemno);
	if( id < 0 ){
		return false;
	}
	u32		workid   = WK7EVENTITEM_WORKID (id);
	u32		workmask = WK7EVENTITEM_WORKBIT(id);
	if( workid >= WK7EVENTITEM_WORKSIZE ){
		return false;
	}
	return (mFlag[workid] & workmask)? true : false;
}

bool WK7EventItem::SetItem(u32 itemno,bool flag)
{
	int		id = FindTableID(itemno);
	if( id < 0 ){
		return false;
	}
	u32		workid   = WK7EVENTITEM_WORKID (id);
	u32		workmask = WK7EVENTITEM_WORKBIT(id);
	if( workid >= WK7EVENTITEM_WORKSIZE ){
		return false;
	}
#ifdef	_DEBUG
	if( WK7Event::GetDebugPrintEnable()==true &&
		((flag==true  && !(mFlag[workid] & workmask)) ||
		 (flag==false &&  (mFlag[workid] & workmask))) ){ 
		char		*pPrefix;
		char		*pType;
		char		str[64];
		if( flag==true ){
			pPrefix = "Get";
		}else{
			pPrefix = "Release";
		}
		switch( itemno & K7EVITEM_TYPE_MASK ){
		case	K7EVITEM_TYPE_KEY:
			pType = "Key";
			break;
		case	K7EVITEM_TYPE_MAP:
			pType = "Map";
			break;
		case	K7EVITEM_TYPE_MEMO:
			pType = "Memo";
			break;
		case	K7EVITEM_TYPE_ZAN:
			pType = "Zan";
			break;
		case	K7EVITEM_TYPE_TAMA:
			pType = "Tama";
			break;
		default:
			pType = "Item";
			break;
		}
		sprintf(str,"%s %s (%06x)",pPrefix,pType,(itemno & 0xffffff));
		WK7Event::DebugPrint(str);
	}
#endif
	if( (itemno & K7EVITEM_TYPE_MASK)==K7EVITEM_TYPE_TAMA ){
		if( mFlag[workid] & workmask ){
			if( flag==false && !(mFlag[workid] & (workmask<<1)) ){
				GameInfo.DeleteBallet();
			}
		}else{
			if( flag==true ){
				GameInfo.CreateBallet();
			}
		}
	}
	if( flag==true ){
		mFlag[workid] |=  workmask;
	}else{
		mFlag[workid] &= ~workmask;
	}
	return true;
}

bool WK7EventItem::IsUsedItem(u32 itemno)
{
	int		id = FindTableID(itemno);
	if( id < 0 ){
		return false;
	}
	u32		workid   = WK7EVENTITEM_WORKID (id);
	u32		workmask = WK7EVENTITEM_WORKBIT(id) << 1;
	if( workid >= WK7EVENTITEM_WORKSIZE ){
		return false;
	}
	return (mFlag[workid] & workmask)? true : false;
}

bool WK7EventItem::UseItem(u32 itemno,bool flag)
{
	int		id = FindTableID(itemno);
	if( id < 0 ){
		return false;
	}
	u32		workid   = WK7EVENTITEM_WORKID (id);
	u32		workmask = WK7EVENTITEM_WORKBIT(id) << 1;
	if( workid >= WK7EVENTITEM_WORKSIZE ){
		return false;
	}
#ifdef	_DEBUG
	if( WK7Event::GetDebugPrintEnable()==true &&
		((flag==true  && !(mFlag[workid] & workmask)) ||
		 (flag==false &&  (mFlag[workid] & workmask))) ){ 
		char		*pPrefix;
		char		*pType;
		char		str[64];
		if( flag==true ){
			pPrefix = "Get";
		}else{
			pPrefix = "Release";
		}
		switch( itemno & K7EVITEM_TYPE_MASK ){
		case	K7EVITEM_TYPE_KEY:
			pType = "Key";
			break;
		case	K7EVITEM_TYPE_MAP:
			pType = "Map";
			break;
		case	K7EVITEM_TYPE_MEMO:
			pType = "Memo";
			break;
		case	K7EVITEM_TYPE_ZAN:
			pType = "Zan";
			break;
		case	K7EVITEM_TYPE_TAMA:
			pType = "Tama";
			break;
		default:
			pType = "Item";
			break;
		}
		sprintf(str,"%s %s (%06x)",pPrefix,pType,(itemno & 0xffffff));
		WK7Event::DebugPrint(str);
	}
#endif
	if( (itemno & K7EVITEM_TYPE_MASK)==K7EVITEM_TYPE_TAMA ){
		if( !(mFlag[workid] & workmask) ){
			if( flag==true ){
				GameInfo.DeleteBallet();
			}
		}
	}
	if( flag==true ){
		mFlag[workid] |=  workmask;
	}else{
		mFlag[workid] &= ~workmask;
	}
	return true;
}

