////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Item Object
//
// WK7Item.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/09/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7ITEM_CPP__

//#include <dolphin.h>
#include ".\WK7Item.h"
#include ".\WK7Define.h"
#include ".\K7ID.h"
#include ".\WK7Event.h"
#include ".\WK7Light.h"

bool	WK7Item::mRenderItem = true;

WK7Item::WK7Item(ghmGcListObj *pAnchor,int prio) : WK7RenderObj(pAnchor,prio)
{
	mStageLight = true;
}

WK7Item::~WK7Item()
{
}

u32 WK7Item::GetMainID(void)
{
	return K7ITEM_MAINID(mID);
}

u32 WK7Item::GetSuffix(void)
{
	return K7ITEM_SUFFIX(mID);
}

bool WK7Item::GetFileName(char *pFileName,u32 id)
{
	u32		mainid = K7ITEM_MAINID(id);
	u32		suffix = K7ITEM_SUFFIX(id);
	OSDebugReportExp(mainid > 1000,"Item Main ID Over 1000 : %d\n" ,mainid);
	OSDebugReportExp(suffix > 100 ,"Item Suffix ID Over 100 : %d\n",suffix);
	sprintf(pFileName,WK7_3DDATA_FOLDER WK7_MODEL_FOLDER WK7_ITEM_FOLDER "%03d/Item%03d_M%02d.dat",mainid,mainid,suffix);
	return true;
}

bool WK7Item::Initialize(void)
{
	if( WK7RenderObj::Initialize()==false ){
		return false;
	}
	return true;
}

bool WK7Item::RenderProcess(void)
{
	if( mType==K7OBJTYPE_ITEM && mRenderItem==false ){
		return false;
	}
	if( WK7RenderObj::RenderProcess()==false ){
		return false;
	}
	return true;
}

bool WK7Item::DispLight(u32 mirrorType,Vec *pPos)
{
	WK7Light	*pLight;
	WK7LightFindData	data;
	data.Initialize();
	while( (pLight=WK7Light::Find(data))!=NULL ){
		if( mObjLight==false ){
			if( pLight->GetObjectType()==mType && pLight->GetObjectID()==mID ){
				continue;
			}
		}
		if( mStageLight==false ){
			if( pLight->GetObjectType()==K7OBJTYPE_STG ){
				continue;
			}
		}
		if( pLight->GetPrivate()==true ){
			DispPrivateLight(pLight,mirrorType,pPos);
		}else{
			if( mDispLight & pLight->GetLightIDBit() ){
				pLight->Disp(mirrorType,pPos);
			}
		}
	}
	return true;
}

//bool WK7Item::ReleaseArchive(u32 id)
//{
//	WK7Archive	*pArchive;
//	if( (pArchive = WK7Archive::Find(K7OBJTYPE_ITEM,id))==NULL ){
//		return false;
//	}
//	delete pArchive;
//	return true;
//}

bool WK7Item::CreateStaticArchive(u32 id)
{
	WK7Archive	*pArchive;
	if( (pArchive = WK7Archive::Find(K7OBJTYPE_ITEM,id))==NULL ){
		if( (pArchive = WK7Archive::Create())==NULL ){
			return false;
		}
		// t@C擾
		char	filename[WK7_FNAME_MAX];
		GetFileName(filename,id);
		// ACe^Cvɐݒ
		pArchive->SetType(K7OBJTYPE_ITEM);
		// hcݒ
		pArchive->SetID  (id);
		// A[JCu[h
		if( pArchive->LoadArchive(filename)==false ){
			delete pArchive;
			return false;
		}
	}
	pArchive->SetStatic(true);
	return true;
}

WK7ItemMotion::WK7ItemMotion(ghmGcListObj *pAnchor,int prio) : WK7MotionObj(pAnchor,prio)
{
}

WK7ItemMotion::~WK7ItemMotion()
{
}

bool WK7ItemMotion::GetFileName(char *pFileName,u32 id)
{
	u32			mainid = K7MOT_MAINID(id);
	u32			suffix = K7MOT_SUFFIX(id);
	
	OSDebugReportExp(mainid > K7ITEMID_MAX,"Item Motion Main ID Over 1000 : %d\n"   ,mainid);
	OSDebugReportExp(suffix > 10000       ,"Item Motion Suffix ID Over 10000 : %d\n",suffix);
	
	sprintf(pFileName,WK7_3DDATA_FOLDER WK7_MOTION_FOLDER WK7_ITEM_FOLDER "%03d/Item%03d_%04d.dat",mainid,mainid,suffix);
	return true;
}

bool WK7ItemMotion::CreateStaticArchive(u32 id)
{
	WK7Archive	*pArchive;
	if( (pArchive = WK7Archive::Find(K7MOT_ATTR|K7OBJTYPE_ITEM,id))==NULL ){
		if( (pArchive = WK7Archive::Create())==NULL ){
			return false;
		}
		// t@C擾
		char	filename[WK7_FNAME_MAX];
		GetFileName(filename,id);
		// ACe^Cvɐݒ
		pArchive->SetType(K7MOT_ATTR|K7OBJTYPE_ITEM);
		// hcݒ
		pArchive->SetID  (id);
		// A[JCu[h
		if( pArchive->LoadArchive(filename)==false ){
			delete pArchive;
			return false;
		}
	}
	pArchive->SetStatic(true);
	return true;
}

