////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Light Object
//
// WK7Light.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/19/2003
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__WK7LIGHT_H__
#define	__WK7LIGHT_H__

//#include <dolphin.h>
//#include <sysdolphin\baselib.h>
#include "..\Fujita\C_LObj.h"
//#include "..\Lib\ghmGcDefine.h"
#include ".\WK7Obj.h"
#include ".\WK7Motion.h"

#define	WK7LIGHT_MOTION_PLAY			(0x00000001)
#define	WK7LIGHT_MOTION_LOOP			(0x00000002)

class WK7LightFindData;

//
// Light Object
//
class WK7Light : public WK7Obj{
public:
protected:
	static ghmGcListObj				mAnchor;
	
	const char						*mpName;
	C_LObj							*mpLObj;
	
	u32								mLightID;
	u32								mObjectType;
	u32								mObjectID;
	WK7RenderObj					*mpRenderObj;
	
	f32								mMotionRate;				// Aj[VĐ[g
	u32								mMotionState;				// Aj[V
	
	Vec								*mpPosition;
	Vec								*mpInterest;
	
	union{
		struct{
			u32						mEnable        : 1;
			u32						mPrivate       : 1;
			u32						mFlagPad       :30;
		};
		u32							mFlag;
	};
	
	void InitMember(void);
	bool SetMotionRate(HSD_WObj *pWObj,f32 rate);
	bool SetMotionRate(HSD_LObj *pLObj,f32 rate);
	
	static u32 GetUniqueID(void);
	void CreateWork(void);
	
	WK7Light();
	WK7Light(ghmGcListObj *pAnchor,int prio=WK7OBJ_DEFAULT_PRIO);
public:
	virtual ~WK7Light();
	
	inline WK7Light* GetPrev(void){ return (WK7Light *)ghmGcListObj::GetPrev(); }
	inline WK7Light* GetNext(void){ return (WK7Light *)ghmGcListObj::GetNext(); }
	
	// LOBJ_AMBIENT  | (S)
	// LOBJ_INFINITE | _̓_(s)
	// LOBJ_POINT    | _
	// LOBJ_SPOT     | t_
	u32 GetType(void);
	
	inline void SetEnable(bool flag){ mEnable = (flag==true)? 1:0; }
	inline bool GetEnable(void){ return mEnable? true : false; }
	
	inline void SetPrivate(bool flag){ mPrivate = (flag==true)? 1:0; }
	inline bool GetPrivate(void){ return mPrivate? true : false; }
	
	static int SetEnableAll(bool flag,u32 objtype=0xffffffff,u32 id=0xffffffff);
	
	void SetDesc(HSD_LightDesc *pDesc);
	void SetDesc(C_LightSet *pDesc);
	
	inline void SetRenderObj(WK7RenderObj *pRenderObj){ mpRenderObj = pRenderObj; }
	inline WK7RenderObj* GetRenderObj(void){ return mpRenderObj; }
	
	HSD_LObj* GetHSDLObj(void);
	inline C_LObj* GetLObj(void){ return mpLObj; }
	inline u32 GetLightID(void){ return mLightID; }
	u32 GetLightIDBit(void);
	
	int GetLObjNum(void);
	bool SetPosition(Vec *pPosition,int index=-1);
	bool SetInterest(Vec *pInterest,int index=-1);
	bool GetPosition(Vec *pPosition,int index);
	bool GetInterest(Vec *pInterest,int index);
	
	// [V֘A
	bool AttachMotion(HSD_LightAnim *pLightAnim);
	bool AttachMotion(WK7LightMotion *pLightMotion,int index);
	bool AttachMotion(WK7LightMotion *pLightMotion,const char *pname=NULL);
	bool DetachMotion(void);
	bool PlayMotion(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f);
	bool ReplayMotion(void);
	bool StopMotion(void);
	bool SetMotionLoop(bool flag);
	bool GetMotionLoop(void);
	int GetMotionState(void);
	bool SetMotionRate(f32 rate=1.0f);
	bool RenewMotionRate(void);
	static bool RenewMotionRateAll(void);
	
	// t[
	virtual bool FrameProcess(void);
	static bool FrameProcessAll(void);
	
	// _O
//	virtual bool RenderProcess(void);
	
	bool Disp(u32 mirrorType=0,Vec *pPos=NULL);
	
	int GetLightType(void);
	
	inline const char* GetName(void){ return mpName; }
	
	static void DispStart(void){ HSD_LObjDeleteCurrentAll(0); }
	static void DispEnd(void){ HSD_LObjSetupInit(HSD_CObjGetCurrent()); }
	static bool DispAll(u32 objtype=0xffffffff,u32 objid=0xffffffff);
	
	inline u32 GetObjectType(void){ return mObjectType; }
	inline u32 GetObjectID(void){ return mObjectID; }
	
	static WK7Light* Create(HSD_LightDesc *pDesc,u32 objtype=0xffffffff,u32 objid=0xffffffff);
	static WK7Light* Create(C_LightSet *pLightSet,u32 objtype=0xffffffff,u32 objid=0xffffffff);
	static bool Release(const char *pName);
	static int Release(u32 objtype,u32 objid,bool privateLight=false);
	static WK7Light* Find(const char *pName);
	static WK7Light* Find(WK7LightFindData &data);
	static WK7Light* Find(u32 lid);
	inline static WK7Light* Top(void){ return (WK7Light *)mAnchor.GetNext(); }
	inline static WK7Light* GetTop(void){ return (WK7Light *)mAnchor.GetNext(); }
	static int GetCount(u32 objtype=0xffffffff,u32 objid=0xffffffff);
};

class WK7LightFindData{
	friend class WK7Light;
protected:
	bool				mInitialized;
	u32					mObjectType;
	u32					mObjectID;
	WK7Light			*mpCurrentLight;
public:
	WK7LightFindData();
	bool Initialize(u32 objtype=0xffffffff,u32 objid=0xffffffff);
};

#endif	//__WK7LIGHT_H__

