////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Mirror Object
//
// WK7Mirror.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 04/12/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7MIRROR_CPP__

#include ".\WK7Mirror.h"
#include ".\WK7Cam.h"

ghmGcListObj		WK7Mirror::mAnchor;

WK7Mirror::WK7Mirror(WK7JObj *pJObj,WK7JObj *pSizeJObj,HSD_DObj *pMat,ghmGcListObj *pAnchor,int prio) : WK7Obj(pAnchor,prio)
{
	mpJObj     = pJObj;
	mpSizeJObj = pSizeJObj;
	mpMat      = pMat;
	ghmGcMemSetAllocName("C_Mirror");
	mpMirror   = new C_Mirror(pJObj->GetJObj(),WK7CAM->GetCObj(),pMat,pSizeJObj->GetJObj());
	mMirrorID  = GetUniqueID();
}

WK7Mirror::~WK7Mirror()
{
	GHM_SAFE_DELETE(mpMirror);
}

bool WK7Mirror::FrameProcessAll(void)
{
	WK7Mirror		*pMirror = (WK7Mirror *)mAnchor.GetNext();
	while( pMirror!=NULL ){
		pMirror->FrameProcess();
		pMirror = pMirror->GetNext();
	}
	return true;
}

bool WK7Mirror::FrameProcess(void)
{
	WK7MirrorReflectObj	*pObj = (WK7MirrorReflectObj *)mRefrectObjAnchor.GetNext();
	while( pObj!=NULL ){
		pObj->FrameProcess();
		pObj = pObj->GetNext();
	}
	return true;
}

bool WK7Mirror::RenderProcessAll(void)
{
	WK7Mirror		*pMirror = (WK7Mirror *)mAnchor.GetNext();
	while( pMirror!=NULL ){
		pMirror->RenderProcess();
		pMirror = pMirror->GetNext();
	}
	return true;
}

bool WK7Mirror::RenderProcess(void)
{
	if( DispStart()==false ){
		return false;
	}
	WK7MirrorReflectObj	*pObj = (WK7MirrorReflectObj *)mRefrectObjAnchor.GetNext();
	while( pObj!=NULL ){
		pObj->RenderProcess();
		pObj = pObj->GetNext();
	}
	DispEnd();
	return true;
}

u32 WK7Mirror::GetUniqueID(void)
{
	u32			uniqueID = 0;
	while( 1 ){
		WK7Mirror	*pMirror = (WK7Mirror *)mAnchor.GetNext();
		while( pMirror!=NULL ){
			if( pMirror->GetMirrorID()==uniqueID ){
				break;
			}
			pMirror = pMirror->GetNext();
		}
		if( pMirror==NULL ){
			break;
		}
		uniqueID++;
	}
	return uniqueID;
}

WK7Mirror* WK7Mirror::Create(WK7JObj *pJObj,WK7JObj *pSizeJObj,HSD_DObj *pMat)
{
	if( pJObj==NULL || pSizeJObj==NULL ){
		return NULL;
	}
	if( pMat==NULL ){
		if( (pMat = pJObj->GetDObj())==NULL ){
			return NULL;
		}
	}
	ghmGcMemSetAllocName("WK7Mir");
	WK7Mirror	*pMirror = new WK7Mirror(pJObj,pSizeJObj,pMat,&mAnchor);
	return pMirror;
}

bool WK7Mirror::Release(const char *pName)
{
	WK7Mirror	*pMirror = Find(pName);
	if( pMirror==NULL ){
		return false;
	}
	delete pMirror;
	return true;
}

const char* WK7Mirror::GetName(void)
{
	if( mpJObj==NULL ){
		return "";
	}
	return mpJObj->GetName();
}

WK7Mirror* WK7Mirror::Find(const char *pName)
{
	WK7Mirror		*pMirror = (WK7Mirror *)mAnchor.GetNext();
	while( pMirror != NULL ){
		if( strcmp(pMirror->GetName(),pName)==0 ){
			return pMirror;
		}
		pMirror = pMirror->GetNext();
	}
	return NULL;
}

WK7Mirror* WK7Mirror::Find(u32 lid)
{
	WK7Mirror		*pMirror = (WK7Mirror *)mAnchor.GetNext();
	while( pMirror != NULL ){
		if( pMirror->GetMirrorID()==lid ){
			return pMirror;
		}
		pMirror = pMirror->GetNext();
	}
	return NULL;
}

bool WK7Mirror::ReleaseJObj(WK7JObj *pJObj)
{
	WK7Mirror		*pMirror = (WK7Mirror *)mAnchor.GetNext();
	WK7Mirror		*pNext;
	while( pMirror != NULL ){
		pNext = pMirror->GetNext();
		if( pMirror->GetJObj()==pJObj ){
			delete pMirror;
		}else{
			pMirror->DeleteReflectJObj(pJObj);
		}
		pMirror = pNext;
	}
	return true;
}

WK7MirrorReflectObj* WK7Mirror::AddReflectJObj(WK7JObj *pJObj)
{
	if( pJObj==NULL || mpJObj==pJObj ){
		return NULL;
	}
	ghmGcMemSetAllocName("WK7MirRO");
	WK7MirrorReflectObj	*pObj = new WK7MirrorReflectObj(pJObj,&mRefrectObjAnchor);
	return pObj;
}

bool WK7Mirror::DeleteReflectJObj(WK7JObj *pJObj)
{
	WK7MirrorReflectObj	*pObj = (WK7MirrorReflectObj *)mRefrectObjAnchor.GetNext();
	WK7MirrorReflectObj	*pNext;
	while( pObj != NULL ){
		pNext = pObj->GetNext();
		if( pObj->GetJObj()==pJObj ){
			delete pObj;
		}
		pObj = pNext;
	}
	return true;
}

WK7MirrorReflectObj* WK7Mirror::FindReflectJObj(WK7JObj *pJObj)
{
	WK7MirrorReflectObj	*pObj = (WK7MirrorReflectObj *)mRefrectObjAnchor.GetNext();
	while( pObj != NULL ){
		if( pObj->GetJObj()==pJObj ){
			return pObj;
		}
		pObj = pObj->GetNext();
	}
	return NULL;
}

bool WK7Mirror::DispStart(void)
{
	if( mpMirror==NULL ){
		return false;
	}
	mpMirror->DispStart();
	return true;
}

bool WK7Mirror::DispEnd(void)
{
	if( mpMirror==NULL ){
		return false;
	}
	mpMirror->DispEnd();
	return true;
}

//------------------------------------------------------------------------------

WK7MirrorReflectObj::WK7MirrorReflectObj(WK7JObj *pJObj,ghmGcListObj *pAnchor,int prio) : WK7Obj(pAnchor,prio)
{
	mpJObj    = pJObj;
	mChildObj = true;
	mVisible  = true;
}

WK7MirrorReflectObj::~WK7MirrorReflectObj()
{
}

bool WK7MirrorReflectObj::FrameProcess(void)
{
	return true;
}

bool WK7MirrorReflectObj::RenderProcess(void)
{
	if( mpJObj==NULL || mVisible==false ){
		return true;
	}
	mpJObj->RenderProcess(false,mChildObj);
	return true;
}

