////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Model
//
// WK7Model.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/08/2003
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__WK7MODEL_H__
#define	__WK7MODEL_H__

//#include "..\Lib\ghmGcDefine.h"
//#include "..\Lib\ghmGcMath.h"
//#include "..\Lib\ghmGcColl.h"
#include ".\WK7Obj.h"
#include ".\WK7Anim.h"
#include ".\WK7SceneDesc.h"
#include ".\WK7Motion.h"
#include "..\Fujita\C_Vec.h"
#include "..\Fujita\C_Edge.h"
#include "..\Fujita\C_EdgeShadow.h"

class WK7Archive;
struct WK7SdExpInfo;
class C_SetAlpha;
class WK7Motion;

class WK7AlphaControl{
protected:
	C_SetAlpha					*mpAlpha;
	WK7AnimLinearF				mAlpha;
	bool						mZUpDate;
public:
	WK7AlphaControl(HSD_JObj *pJObj,f32 alpha,bool zupdate);
	~WK7AlphaControl();
	
	bool FrameProcess(void);
	
	bool SetAlpha(f32 alpha,u32 tick=0);
	inline f32 GetAlpha(void){ return mAlpha.GetValue(); }
	inline f32 GetDstAlpha(void){ return mAlpha.GetDstValue(); }
	inline bool IsAnimatingAlpha(void){ return mAlpha.IsAnimating(); }
	inline bool SetZUpDate(bool zupdate){ mZUpDate = zupdate; return true; }
	inline bool GetZUpDate(void){ return mZUpDate; }
};

class WK7JObjMotionInterpolateInfo : public WK7SmallMemObj{
	friend class WK7JObj;
protected:
	ghmQuaternion					mPrevRotate;
	Vec								mPrevScale;
	Vec								mPrevTranslate;
	ghmQuaternion					mInterpolateRotate;
	Vec								mInterpolateScale;
	Vec								mInterpolateTranslate;
	Mtx								mInterpolateMatrix;
	f32								mInterpolateTotalFrame;
	f32								mInterpolateCurrentFrame;
	bool							mUseCurrRotate;
	bool FrameProcess(WK7JObj *pJObj);
};

#define	WK7JOBJ_MOTION_PLAY			(0x00000001)
#define	WK7JOBJ_MOTION_LOOP			(0x00000002)
#define	WK7JOBJ_MOTION_SHAPEPLAY	(0x00000004)
#define	WK7JOBJ_MOTION_MATPLAY		(0x00000008)

#define	WK7JOBJ_RENDERMODE_NORMAL	(0x00000001)
#define	WK7JOBJ_RENDERMODE_SHADOW	(0x00000002)

class WK7Model;
class WK7JObj;

class WK7JObjAttachModel : ghmGcListObj{
protected:
	WK7Model		*mpModel;
public:
	WK7JObjAttachModel(ghmGcListObj *pAnchor,WK7Model *pModel) : ghmGcListObj(pAnchor){
		mpModel = pModel;
	}
	virtual ~WK7JObjAttachModel(){}
	
	inline WK7JObjAttachModel* GetPrev(void){ return (WK7JObjAttachModel *)ghmGcListObj::GetPrev(); }
	inline WK7JObjAttachModel* GetNext(void){ return (WK7JObjAttachModel *)ghmGcListObj::GetNext(); }
	
	inline WK7Model* GetModel(void){ return mpModel; }
};

class WK7JObjShapeAnim{
protected:
	void							*mpVertexBuffer[2];
	u32								mVertexBufferSize;
	void							*mpNormalBuffer[2];
	u32								mNormalBufferSize;
	int								mBufferIndex;
public:
	WK7JObjShapeAnim(HSD_ShapeSet *pShapeSet,u32 vertexBufferSize,u32 normalBufferSize);
	~WK7JObjShapeAnim();
	
	static void get_shape_vertex_xyz(HSD_ShapeSet *shape_set,int shape_id, int arrayidx, float dst[3]);
	static void get_shape_normal_xyz(HSD_ShapeSet *shape_set,int shape_id, int arrayidx, float dst[3]);
	
	bool FrameProcess(void);
	bool SetShapeBuffer(void);
	bool ResetShapeBuffer(void);
};

class WK7JObjBound{
public:
	enum Type{
		UNKNOWN = -1,
		SPHERE = 0,
		BOX,
	};
protected:
	Vec								mCenter;
	bool							mEnable;
	Type							mType;
	int MakeVertex(vector<C_Vec> &vertlist,HSD_DObj *pDObj);
	int MakeVertex(vector<C_Vec> &vertlist,HSD_PObj *pPObj);
public:
	WK7JObjBound();
	virtual ~WK7JObjBound();
	virtual bool Initialize(HSD_DObj *pDObj);
	
	inline Type GetType(void){ return mType; }
	
	inline void SetEnable(bool flag){ mEnable = flag; }
	inline bool GetEnable(void){ return mEnable; }
	
	inline Vec* GetCenter(void){ return &mCenter; }
	
	virtual int GetCrossState(Mtx lwmat,ghmGcViewVolume *pViewVolume);
	virtual int GetFarClip(f32 dist,Mtx lwmat,ghmGcViewVolume *pViewVolume);
};

class WK7JObjBoundSphere : public WK7JObjBound{
protected:
	ghmGcCollSphere					mBound;
	int CalcCenter(HSD_DObj *pDObj);
	int CalcCenter(HSD_PObj *pPObj);
	void CalcRadius(HSD_DObj *pDObj);
	void CalcRadius(HSD_PObj *pPObj);
public:
	WK7JObjBoundSphere();
	WK7JObjBoundSphere(HSD_DObj *pDObj);
	virtual ~WK7JObjBoundSphere();
	virtual bool Initialize(HSD_DObj *pDObj);
	
	inline ghmGcCollSphere* GetBound(void){ return &mBound; }
	
	virtual int GetCrossState(Mtx lwmat,ghmGcViewVolume *pViewVolume);
	virtual int GetFarClip(f32 dist,Mtx lwmat,ghmGcViewVolume *pViewVolume);
};

class WK7JObjBoundBox : public WK7JObjBound{
protected:
	ghmGcCollBox					mBound;
	int CalcExtent(HSD_DObj *pDObj,Vec &max,Vec &min);
	int CalcExtent(HSD_PObj *pPObj,Vec &max,Vec &min);
public:
	WK7JObjBoundBox();
	WK7JObjBoundBox(HSD_DObj *pDObj);
	virtual ~WK7JObjBoundBox();
	virtual bool Initialize(HSD_DObj *pDObj);
	
	inline ghmGcCollBox* GetBound(void){ return &mBound; }
	
	virtual int GetCrossState(Mtx lwmat,ghmGcViewVolume *pViewVolume);
	virtual int GetFarClip(f32 dist,Mtx lwmat,ghmGcViewVolume *pViewVolume);
};

class WK7JObjAttr{
	friend class WK7Model;
	friend class WK7JObj;
protected:
	union{
		struct{
			u32						mLinkedLights : 1;
			u32						mNoHit        : 1;
			u32						mFogDisable   : 1;
			u32						mFlagPad      :21;
			s8						mPrio;
		};
		u32							mFlag;
	};
	char							*mpName;
	HSD_LightDesc					**mppLinkedLights;
	
public:
	inline const char* GetName(void) const { return mpName; }
	inline const HSD_LightDesc* const * GetLinkedLights(void) const { return mLinkedLights? mppLinkedLights : NULL; }
	inline bool GetNoHit(void) const { return mNoHit? true : false; }
};

struct WK7VtxDescInfo{
	HSD_VtxDescList		*mpVertexDesc;
	HSD_VtxDescList		*mpNormalDesc;
	HSD_VtxDescList		*mpColorDesc;
	HSD_VtxDescList		*mpTexDesc;
	u32					mVertexPos;
	u32					mNormalPos;
	u32					mColorPos;
	u32					mTexPos;
};

typedef struct{
	u16		x,y,z;
}WK7VecU16;
typedef struct{
	s16		x,y,z;
}WK7VecS16;
typedef struct{
	u8		x,y,z;
}WK7VecU8;
typedef struct{
	s8		x,y,z;
}WK7VecS8;

typedef struct{
	f32		s,t;
}WK7St;
typedef struct{
	u16		s,t;
}WK7StU16;
typedef struct{
	s16		s,t;
}WK7StS16;
typedef struct{
	u8		s,t;
}WK7StU8;
typedef struct{
	s8		s,t;
}WK7StS8;

typedef void WK7GetVertexFunc(Vec *pVertex,void *pBasePtr,int index,f32 base);
typedef void WK7GetStFunc(WK7St *pSt,void *pBasePtr,int index,f32 base);

#define	WK7JOBJ_ZSORTPRIO_DEFAULT		(1024)
#define	WK7JOBJ_ZSORTPRIO_SCALE			(8)

#define	WK7JOBJ_CULLMODE_NONE			(0)
#define	WK7JOBJ_CULLMODE_BACK			(1)
#define	WK7JOBJ_CULLMODE_FRONT			(2)
#define	WK7JOBJ_CULLMODE_REV			(3)
class WK7JObj;
typedef	bool WK7JObjCallBackFunc(WK7JObj *pJObj,void *pData);

class WK7Fog;
class WK7EffectBreakObj;
class WK7JObj : public WK7SmallMemObj{
	friend class WK7Model;
protected:
	static WK7SdExpInfo				*mpCurrentSdExpInfo;
	static HSD_TrspMask				mRenderProcessTrspMask;
	static u32						mRenderProcessRenderMode;
	static WK7SdExpInfo				*mpCurrentAttachMotionSdExpInfo;
	
	const char						*mpName;
	WK7JObjAttr						*mpAttr;
	HSD_JObj						*mpJObj;
	HSD_Joint						*mpJoint;
	HSD_AnimJoint					*mpAnimJoint;
	HSD_MatAnimJoint				*mpMatAnimJoint;
	HSD_ShapeAnimJoint				*mpShapeAnimJoint;
	WK7JObj							*mpParent;
	WK7JObj							*mpChild;
	WK7JObj							*mpPrev;
	WK7JObj							*mpNext;
	f32								mMotionRate;
	u32								mMotionState;
	ghmQuaternion					*mpCurrRotate;
	
	WK7Fog							*mpFog;
	WK7Fog							*mpCurrentFog;
	
//	void							*mpVertexBuffer[2];
//	u32								mVertexBufferSize;
//	void							*mpNormalBuffer[2];
//	u32								mNormalBufferSize;
//	int								mBufferIndex;
	
	WK7JObjShapeAnim				*mpShapeAnim;					// VFCvAjp
	WK7JObjBound					*mpBound;						// Nbvp
	
	WK7JObjMotionInterpolateInfo	*mpInterpolateInfo;
	C_EdgeArray						*mpEdgeArray;					// ep
	
	u32								mRenderMode;					// 
	union{
		struct{
			u32						mHide                 : 1;
			u32						mFrameProcess         : 1;
			u32						mReferenceObj         : 1;
			u32						mMotionOpDisable      : 1;
			u32						mCastShadowDisable    : 1;
			u32						mUseUserDefinedMatrix : 1;
			u32						mClothJoint           : 1;
			u32						mFlagPad              :25;
		};
		u32							mFlag;
	};
	
	static C_EdgeShadow				*mpEdgeShadow;
	static int						mEdgeShadowObjNum;
	static u32						mCurrentRenderMode;
	
	ghmGcListObj					mAttachModels;
	
	WK7JObjCallBackFunc				*mpPreFrameProcessCallBack;
	void							*mpPreFrameProcessData;
	WK7JObjCallBackFunc				*mpPostFrameProcessCallBack;
	void							*mpPostFrameProcessData;
	WK7JObjCallBackFunc				*mpPreRenderProcessCallBack;
	void							*mpPreRenderProcessData;
	WK7JObjCallBackFunc				*mpPostRenderProcessCallBack;
	void							*mpPostRenderProcessData;
	
	WK7JObj							*mpReferencedObj;
	u32								mZSortPrio;
	
	WK7EffectBreakObj				*mpEffectBreakObj;
	
	WK7AlphaControl					*mpAlphaControl;
	
	static ghmGcViewVolume			*mpViewVolume;
	
	void SetReferencedObj(WK7JObj *pJObj);
private:
	WK7JObj();
	virtual ~WK7JObj();
	
	bool SetMotionRate(HSD_PObj *pPObj,f32 rate);
	bool SetMotionRate(HSD_MObj *pMObj,f32 rate);
	bool SetMotionRate(HSD_TObj *pTObj,f32 rate);
	bool SetMotionRate(HSD_DObj *pDObj,f32 rate);
	bool SetMotionRate(HSD_JObj *pJObj,f32 rate);
	bool AttachMotion(bool overwrite);
	bool AttachMatMotion(void);
	bool AttachShapeMotion(bool overwrite);
	
	static void JObjUpdateFunc(void *obj,u32 type,FObjData *val);
	void AObjInterpretAnimCustom(void);
	bool SkeltonAnim(void);
	
	bool AttachModel(WK7Model *pModel);
	bool DetachModel(WK7Model *pModel);
	bool ClearAttachModel(void);
	WK7JObjAttachModel* GetAttachModel(WK7Model *pModel);
	inline WK7JObjAttachModel* GetAttachModelTop(void){ return (WK7JObjAttachModel *)mAttachModels.GetNext(); }
	
	void GetBufferSize(HSD_PObj *pPObj,u32 &vertexBufferSize,u32 &normalBufferSize,HSD_ShapeSet *&pShapeSet);
	void GetBufferSize(HSD_DObj *pDObj,u32 &vertexBufferSize,u32 &normalBufferSize,HSD_ShapeSet *&pShapeSet);
	static void get_shape_normal_xyz(HSD_ShapeSet *shape_set,int shape_id, int arrayidx, float dst[3]);
	
	HSD_ShapeSetDesc* GetShapeSetDesc(void);
	const char** GetShapeNameList(HSD_ShapeSetDesc *pShapeSet);
	static const char* GetDescName(const HSD_FObjDesc* desc);
	int SearchName(const char *pShapeAnimName,int shapenum,const char **ppNameList);
	HSD_FObj* FObjLoadDesc_ShapeAnim(HSD_FObjDesc *fd,HSD_FObj *fobj,int shapenum,const char **ppNameList);
	HSD_AObj* AObjLoadDesc_ShapeAnim(HSD_AObjDesc *aobjdesc,HSD_AObj *aobj,int shapenum,const char **ppNameList);
	void PObjAddAnim(HSD_PObj *pobj,HSD_ShapeAnim *shapeanim,bool overwrite);
	void PObjAddAnimAll(HSD_PObj *pobj, HSD_ShapeAnim *shapeanim,bool overwrite);
//	void DObjAddAnim(HSD_DObj *dobj,HSD_MatAnim *matanim,HSD_ShapeAnimDObj *sd);
	void MObjAddAnim(HSD_DObj *dobj,HSD_MatAnim *matanim);
	void PObjAddAnim(HSD_DObj *dobj,HSD_ShapeAnimDObj *sd,bool overwrite);
//	void DObjAddAnimAll(HSD_DObj *dobj,HSD_MatAnim *matanim,HSD_ShapeAnimDObj *shapeanimdobj);
	void MObjAddAnimAll(HSD_DObj *dobj,HSD_DObjDesc *dobjdesc,HSD_MatAnim *matanim);
	void PObjAddAnimAll(HSD_DObj *dobj,HSD_ShapeAnimDObj *shapeanimdobj,bool overwrite);
	HSD_FObj *FObjLoadDesc(HSD_FObjDesc *fd,HSD_FObj *fobj);
	HSD_AObj *AObjLoadDesc(HSD_AObjDesc *aobjdesc,HSD_AObj *aobj);
	void JObjAddAnim(HSD_JObj *jobj,HSD_AnimJoint *ajoint,bool overwrite);
//	void JObjAddAnim(HSD_JObj *jobj,HSD_MatAnimJoint *mjoint,HSD_ShapeAnimJoint *sjoint);
	void JObjAddAnim(HSD_JObj *jobj,HSD_Joint *joint,HSD_MatAnimJoint *mjoint);
	void JObjAddAnim(HSD_JObj *jobj,HSD_ShapeAnimJoint *sjoint,bool overwrite);
	static void JObjSortAnim(HSD_AObj *aobj);
	
	bool _SetRenderMode(u32 rendermode,bool next,bool child);
	
	static void _ZListPreCallbackFunc(void *pData);
	static void _ZListPostCallbackFunc(void *pData);
public:
	
	inline void SetClothJoint(bool flag){ mClothJoint=(flag==true? 1:0); }
	inline bool GetClothJoint(void){ return mClothJoint? true : false; }
	bool ResetClothJointUserDefinedMatrix(bool next=true,bool child=true);
	bool ResetClothJointToBind(bool next=true,bool child=true);
	
	static void SetCurrentAttachMotionSdExpInfo(WK7SdExpInfo *pInfo){ mpCurrentAttachMotionSdExpInfo=pInfo; }
	static void SetCurrentAttachMotionSdExpInfo(WK7Motion *pMotion);
	static void SetCurrentAttachMotionSdExpInfo(void){ mpCurrentAttachMotionSdExpInfo=NULL; }
	static WK7SdExpInfo* GetCurrentAttachMotionSdExpInfo(void){ return mpCurrentAttachMotionSdExpInfo; }
	
	static HSD_TrspMask GetRenderProcessTrpsMask(void){ return mRenderProcessTrspMask; }
	static u32 GetRenderProcessRenderMode(void){ return mRenderProcessRenderMode; }
	
	bool CreateAlphaControl(f32 alpha,bool zupdate);
	bool ReleaseAlphaControl(void);
	WK7AlphaControl* GetAlphaControl(bool next=false,bool child=false);
	bool SetAlpha(f32 alpha,u32 tick=0);
	f32 GetAlpha(void);
	bool IsAnimatingAlpha(void);
	
	bool SetCastShadowDisable(bool flag,bool next=true,bool child=true);
	bool GetCastShadowDisable(bool next=true,bool child=true);
	
	bool LoadFogDesc(HSD_FogDesc *pFogDesc);
	bool ReleaseFog(void);
	bool SetFogEnable(bool flag);
	bool GetFogEnable(void);
	inline WK7Fog* GetFog(void){ return mpFog; }
	
	WK7EffectBreakObj* CreateEffectBreakObj(bool next=false,bool child=true,int level=0);
	bool ReleaseEffectBreakObj(void);
	WK7EffectBreakObj* GetEffectBreakObj(void){ return mpEffectBreakObj; }
	
	static void GetVertexF32(Vec *pVertex,void *pBasePtr,int index,f32 base);
	static void GetVertexU16(Vec *pVertex,void *pBasePtr,int index,f32 base);
	static void GetVertexS16(Vec *pVertex,void *pBasePtr,int index,f32 base);
	static void GetVertexU8 (Vec *pVertex,void *pBasePtr,int index,f32 base);
	static void GetVertexS8 (Vec *pVertex,void *pBasePtr,int index,f32 base);
	static int DlGetIndex(u8 *pdl,HSD_VtxDescList *pDesc);
	static u32 GetStride(HSD_VtxDescList *desc,WK7VtxDescInfo &info);
	static void GetStF32(WK7St *pSt,void *pBasePtr,int index,f32 base);
	static void GetStU16(WK7St *pSt,void *pBasePtr,int index,f32 base);
	static void GetStS16(WK7St *pSt,void *pBasePtr,int index,f32 base);
	static void GetStU8 (WK7St *pSt,void *pBasePtr,int index,f32 base);
	static void GetStS8 (WK7St *pSt,void *pBasePtr,int index,f32 base);
	
	bool SetZSortPrioS(s32 prio,bool next=true,bool child=true);
	bool SetZSortPrio(u32 prio=WK7JOBJ_ZSORTPRIO_DEFAULT,bool next=true,bool child=true);
	inline u32 GetZSortPrio(void){ return mZSortPrio; }
	
	void ZListPreCallbackFunc(void);
	void ZListPostCallbackFunc(void);
	
	inline WK7JObj* GetReferencedObj(void){ return mpReferencedObj; }
	
	bool SetCullMode(u32 mode,bool next,bool child);
	static bool SetCullMode(HSD_JObj *pJObj,u32 mode);
	
	WK7JObj(HSD_JObj *pJObj,HSD_Joint *pJoint,WK7JObj *pParent,WK7JObj *pPrev);
	
	inline WK7JObjBound* GetBound(void){ return mpBound; }
	int GetBoundState(void);		// vViewVolumeݒ
	
	WK7JObjCallBackFunc* SetPreFrameProcessCallBack  (WK7JObjCallBackFunc *pCallBackFunc=NULL,void *pData=NULL);
	inline WK7JObjCallBackFunc* GetPreFrameProcessCallBack  (void){ return mpPreFrameProcessCallBack; }
	inline void* GetPreFrameProcessData(void){ return mpPreFrameProcessData; }
	WK7JObjCallBackFunc* SetPostFrameProcessCallBack (WK7JObjCallBackFunc *pCallBackFunc=NULL,void *pData=NULL);
	inline WK7JObjCallBackFunc* GetPostFrameProcessCallBack (void){ return mpPostFrameProcessCallBack; }
	inline void* GetPostFrameProcessData(void){ return mpPostFrameProcessData; }
	WK7JObjCallBackFunc* SetPreRenderProcessCallBack (WK7JObjCallBackFunc *pCallBackFunc=NULL,void *pData=NULL);
	inline WK7JObjCallBackFunc* GetPreRenderProcessCallBack (void){ return mpPreRenderProcessCallBack; }
	inline void* GetPreRenderProcessData(void){ return mpPreRenderProcessData; }
	WK7JObjCallBackFunc* SetPostRenderProcessCallBack(WK7JObjCallBackFunc *pCallBackFunc=NULL,void *pData=NULL);
	inline WK7JObjCallBackFunc* GetPostRenderProcessCallBack(void){ return mpPostRenderProcessCallBack; }
	inline void* GetPostRenderProcessData(void){ return mpPostRenderProcessData; }
	
	static bool SetMaterialAmbientColor(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialAmbientColor(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialAmbientColor(HSD_JObj *pJObj,GXColor col);
	static bool SetMaterialAmbientColorLE(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialAmbientColorLE(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialAmbientColorLE(HSD_JObj *pJObj,GXColor col);
	
	static bool SetMaterialDiffuseColor(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialDiffuseColor(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialDiffuseColor(HSD_JObj *pJObj,GXColor col);
	static bool SetMaterialDiffuseColorLE(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialDiffuseColorLE(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialDiffuseColorLE(HSD_JObj *pJObj,GXColor col);
	
	static bool SetMaterialSpecularColor(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialSpecularColor(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialSpecularColor(HSD_JObj *pJObj,GXColor col);
	static bool SetMaterialSpecularColorLE(HSD_MObj *pMObj,GXColor col);
	static bool SetMaterialSpecularColorLE(HSD_DObj *pDObj,GXColor col);
	static bool SetMaterialSpecularColorLE(HSD_JObj *pJObj,GXColor col);
	
	bool SetMaterialAmbientColor(GXColor col,bool next=true,bool child=true);
	bool SetMaterialAmbientColorLE(GXColor col,bool next=true,bool child=true);
	bool SetMaterialDiffuseColor(GXColor col,bool next=true,bool child=true);
	bool SetMaterialDiffuseColorLE(GXColor col,bool next=true,bool child=true);
	bool SetMaterialSpecularColor(GXColor col,bool next=true,bool child=true);
	bool SetMaterialSpecularColorLE(GXColor col,bool next=true,bool child=true);
	
	static bool SetViewVolume(HSD_CObj *pCObj);
	inline static int GetEdgeShadowObjNum(void){ return mEdgeShadowObjNum; }
	
	inline static void SetCurrentRenderMode(u32 rendermode){ mCurrentRenderMode = rendermode; }
	inline static u32 GetCurrentRenderMode(void){ return mCurrentRenderMode; }
	
	inline const WK7JObjAttr* GetAttr(void){ return mpAttr; }
	const char* GetNameAttr(void);
	
	bool SetRenderMode(u32 rendermode,bool next=false,bool child=false);
	inline u32 GetRenderMode(void){ return mRenderMode; }
	
	MtxPtr GetLocalWorldMatrix(void);
	
	static	bool SetPE(HSD_JObj *pJObj,HSD_PEDesc *pPEDesc,HSD_Joint *pJoint=NULL);
	bool SetPE(HSD_PEDesc *pPEDesc,bool next=true,bool child=true);
	static bool SetDstAlpha(HSD_JObj *pJObj,u8 alpha);
	bool SetDstAlpha(u8 alpha,bool next=true,bool child=true);
	
	bool HasParent(WK7JObj *pJObj);
	bool HasChild(WK7JObj *pJObj,bool child=true);
	bool HasBrother(WK7JObj *pJObj);
	
	inline WK7JObj* GetParent(void){ return mpParent; }
	inline WK7JObj* GetChild(void){ return mpChild; }
	inline WK7JObj* GetPrev(void){ return mpPrev; }
	inline WK7JObj* GetNext(void){ return mpNext; }
	ghmQuaternion* GetCurrRotate(void){ return mpCurrRotate; }
	inline const char* GetName(void){ return mpName; }
	void ChangeName(const char *pName){ mpName=pName; }
	
	// \\
	void Hide(bool next=true,bool child=true);
	void Disp(bool next=true,bool child=true);
	bool IsHidden(void);
	
	bool HasAlpha(bool next=true,bool child=true);
	
	// `
	void FrameProcess(bool next=true,bool child=true);
	
	// `
#if !FUJITA
	void RenderProcess(bool next=true,bool child=true);
#else
	void RenderProcess(bool next=true,bool child=true, 
					   HSD_TrspMask trsp=HSD_TRSP_ALL, u32 rendermode=0);
#endif
	
	bool MakeBound(bool next=true,bool child=true);
	
	bool ChangeRotInterpolate(const Vec *pRotation=NULL);
	
	// [V֘A
	int AttachMotion(WK7ModelMotion *pModelMotion,bool next=true,bool child=true,bool overwrite=false);
	int AttachFindMotion(WK7ModelMotion *pModelMotion,bool next=true,bool child=true,bool overwrite=false);
	int AttachMotionClassic(WK7ModelMotion *pModelMotion,bool next=true,bool child=true);
	int AttachMotion(HSD_AnimJoint *pAnimJoint,bool next=true,bool child=true,bool overwrite=false);
	int AttachMotionClassic(HSD_AnimJoint *pAnimJoint,bool next=true,bool child=true);
	int AttachMotionAll(WK7ModelMotion *pModelMotion,bool next=true,bool child=true,bool overwrite=false);
	bool DetachMotion(bool next=true,bool child=true);
	bool DetachMotionAll(bool next=true,bool child=true);
	bool ResetMotion(bool next=true,bool child=true);
	bool ResetMotionAll(bool next=true,bool child=true);
	bool PlayMotion(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,bool next=true,bool child=true);
	bool PlayMotionNA(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,bool next=true,bool child=true);
	bool PlayMotionAll(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,bool next=true,bool child=true);
	bool StopMotion(bool next=true,bool child=true);
	bool StopMotionAll(bool next=true,bool child=true);
	bool ReplayMotion(bool next=true,bool child=true);
	bool ReplayMotionAll(bool next=true,bool child=true);
	
	bool SetMotionOperation(bool flag,bool next=true,bool child=true);
	
	f32 GetCurrentMotionFrame(bool next=true,bool child=true);
	f32 GetTotalMotionFrame(bool next=true,bool child=true);
	
	static bool ResetMotionPObj(HSD_PObj *pPObj,HSD_PObjDesc *pPObjDesc);
	static bool ResetMotionTObjAll(HSD_JObj *pJObj,HSD_Joint *pJoint);
	static bool ResetMotionTObj(HSD_DObj *pDObj,HSD_DObjDesc *pDObjDesc);
	static bool ResetMotionTObj(HSD_MObj *pMObj,HSD_MObjDesc *pMObjDesc);
	static bool ResetMotionTObj(HSD_TObj *pTObj,HSD_TObjDesc *pTObjDesc);
	static bool ResetMotionMObj(HSD_MObj *pMObj,HSD_MObjDesc *pMObjDesc,u32 flag);
	static bool ResetMotionDObj(HSD_DObj *pDObj,HSD_DObjDesc *pDObjDesc,u32 flag);
	static bool ResetMotionDObjAll(HSD_JObj *pJObj,HSD_Joint *pJoint,u32 flag);
	
	int AttachMatMotion(WK7ModelMotion *pModelMotion,bool next=true,bool child=true);
	int AttachMatMotionClassic(WK7ModelMotion *pModelMotion,bool next=true,bool child=true);
	int AttachMatMotion(HSD_MatAnimJoint *pMatAnimJoint,bool next=true,bool child=true);
	int AttachMatMotionClassic(HSD_MatAnimJoint *pMatAnimJoint,bool next=true,bool child=true);
	bool DetachMatMotion(bool next=true,bool child=true);
	bool ResetMatMotion(bool next=true,bool child=true);
	bool PlayMatMotion(f32 startframe=0.0f,bool next=true,bool child=true);
	bool StopMatMotion(bool next=true,bool child=true);
	bool ReplayMatMotion(bool next=true,bool child=true);
	
	int AttachShapeMotion(WK7ModelMotion *pModelMotion,bool next=true,bool child=true,bool overwrite=false);
	int AttachShapeMotionClassic(WK7ModelMotion *pModelMotion,bool next=true,bool child=true);
	int AttachShapeMotion(HSD_ShapeAnimJoint *pShapeAnimJoint,bool next=true,bool child=true,bool overwrite=false);
	int AttachShapeMotionClassic(HSD_ShapeAnimJoint *pShapeAnimJoint,bool next=true,bool child=true);
	bool DetachShapeMotion(bool next=true,bool child=true);
	bool ResetShapeMotion(bool next=true,bool child=true);
	bool PlayShapeMotion(f32 startframe=0.0f,bool next=true,bool child=true);
	bool StopShapeMotion(bool next=true,bool child=true);
	bool ReplayShapeMotion(bool next=true,bool child=true);
	
	bool SetMotionRate(f32 rate=1.0f,bool next=true,bool child=true);
	bool RenewMotionRate(bool next=true,bool child=true);
	
	bool SetMotionLoop(bool flag,bool next=true,bool child=true);
	bool GetMotionLoop(bool next=true,bool child=true);
	
	int GetMotionState(bool next=true,bool child=true);
	int GetShapeMotionState(bool next=true,bool child=true);
	
	bool SetFlag(u32 flag,bool next=true,bool child=true);
	u32 GetFlag(void);
	bool ClearFlag(u32 flag,bool next=true,bool child=true);
	
	WK7JObj* GetObj(const char *pName,bool next=true,bool child=true);
	WK7JObj* GetObjLC(const char *pName,bool next=true,bool child=true);
	WK7JObj* GetObjPartialName(const char *pPartialName,bool next=true,bool child=true);
	WK7JObj* GetObj(HSD_Joint *pJoint,bool next=true,bool child=true);
	WK7JObj* GetParent(const char *pName);
	WK7JObj* GetObjPartialName2(const char *pName,bool next=true,bool child=true);
	
	inline HSD_JObj* GetJObj(void){ return mpJObj; }
#if FUJITA
	inline HSD_Joint* GetJoint(void){ return mpJoint; }
#endif	
	HSD_DObj* GetDObj(const char *pName,bool next=true,bool child=true);
	HSD_DObj* GetDObjByPartialName(const char *pName,bool next=true,bool child=true);
	inline HSD_DObj* GetDObj(void){ return mpJObj==NULL? NULL : HSD_JObjGetDObj(mpJObj); }
	const char* GetDObjName(HSD_DObj *pDObj);
	
	bool CreateEdge(void);
	bool ReleaseEdge(void);
	inline C_EdgeArray* GetEdge(void){ return mpEdgeArray; }
	
	bool SetPosition(f32 x,f32 y,f32 z);
	bool SetPosition(const Vec *pPosition);
	inline bool SetPosition(const Vec &position){ return SetPosition(&position); }
	
	bool GetPosition(Vec *pPosition);
	inline bool GetPosition(Vec &position){ return GetPosition(&position); }
	
	bool SetRotation(f32 x,f32 y,f32 z);
	bool SetRotation(const Vec *pRotation);
	inline bool SetRotation(const Vec &rotation){ return SetRotation(&rotation); }
	bool SetRotX(f32 x);
	bool SetRotY(f32 y);
	bool SetRotZ(f32 z);
	f32 GetRotX(void);
	f32 GetRotY(void);
	f32 GetRotZ(void);
	
	bool GetRotation(f32 *px,f32 *py,f32 *pz);
	inline bool GetRotation(f32 &x,f32 &y,f32 &z){ return GetRotation(&x,&y,&z); }
	bool GetRotation(Vec *pRotation);
	inline bool GetRotation(Vec &rotation){ return GetRotation(&rotation); }
	
	bool SetScale(f32 x,f32 y,f32 z);
	bool SetScale(const Vec *pScale);
	inline bool SetScale(const Vec &scale){ return SetScale(&scale); }
	
	bool GetScale(f32 *px,f32 *py,f32 *pz);
	inline bool GetScale(f32 &x,f32 &y,f32 &z){ return GetScale(&x,&y,&z); }
	bool GetScale(Vec *pScale);
	inline bool GetScale(Vec &scale){ return GetScale(&scale); }
	
	MtxPtr GetMtxPtr(void);
	bool GetWorldPosition(Vec *pPosition);
	inline bool GetWorldPosition(Vec &position){ return GetWorldPosition(&position); }
	bool SetWorldPosition(f32 x,f32 y,f32 z);
	bool SetWorldPosition(const Vec *pPosition);
	
	bool GetOpa(void);
	bool GetXlu(void);
	bool GetTexEdge(void);
	bool SetAlphaUpdate(bool flag,bool next=true,bool child=true,u32 trsp=0xffffffff);
	bool SetNoZUpdate(bool flag,bool next=true,bool child=true,u32 trsp=0xffffffff);
	
	bool CopyTRS(WK7JObj *pJObj,bool next=true,bool child=true);
	bool CopyUserDefinedMatrix(WK7JObj *pJObj,bool next=true,bool child=true);
	bool SetUserDefinedMatrix(Mtx mtx);
	bool ResetUserDefinedMatrix(void);
	int SetUserDefinedMatrixFlag(bool flag,bool next=true,bool child=true);
	bool GetUserDefinedMatrixFlag(bool next=true,bool child=true);
};

#define	WK7MODEL_MIRROR_X			(0x00000001)
#define	WK7MODEL_MIRROR_Y			(0x00000002)
#define	WK7MODEL_MIRROR_Z			(0x00000004)
//
// Model
//
class WK7Model : public WK7SmallMemObj{
	friend class WK7Archive;
protected:
	WK7Archive					*mpArchive;
	WK7JObj						*mpTopObj;
	int							mTopObjNum;
	WK7JObj						*mpAttachJObj;
	union{
		struct{
			u32					mAttach               :  1;
			u32					mCameraAlphaControl   :  1;
			u32					mMatAmbientColorFade  :  1;
			u32					mMatAmbientColorFadeLE:  1;
			u32					mMatDiffuseColorFade  :  1;
			u32					mMatDiffuseColorFadeLE:  1;
			u32					mFlagPad              : 26;
		};
		u32						mFlag;
	};
	
	C_SetAlpha					*mpAlpha;
	WK7AnimLinearF				mAlpha;
	
	WK7AnimLinear				mMatAmbientColorR;
	WK7AnimLinear				mMatAmbientColorG;
	WK7AnimLinear				mMatAmbientColorB;
	WK7AnimLinear				mMatDiffuseColorR;
	WK7AnimLinear				mMatDiffuseColorG;
	WK7AnimLinear				mMatDiffuseColorB;
	
	HSD_JObj					*mpMirrorJObj[2];					// ~[p
	u32							mMirrorType;
	Vec							mMirrorPos;
	
	WK7Fog						*mpFog;
	
	static void SetReferencedObj(WK7JObj *pReferJObj,WK7JObj *pBaseJObj);
	
	WK7Model();		// ftHgRXgN^֎~
	WK7Model(WK7Archive *pArchive,u32 mirrorType);
#ifdef	_DEBUG
	int ReviseJointLevel(HSD_Joint *pJoint,bool next,int count);
#endif
	
	bool _SetAlpha(f32 alpha);
public:
	virtual ~WK7Model();
	
	bool CopyUserDefinedMatrix(WK7Model *pModel);
	bool CopyTRS(WK7Model *pModel);
	
	bool SetCastShadowDisable(bool flag,int index=-1);
	bool GetCastShadowDisable(int index=-1);
	
	bool LoadFogDesc(HSD_FogDesc *pFogDesc);
	bool ReleaseFog(void);
	bool SetFogEnable(bool flag);
	bool GetFogEnable(void);
	inline WK7Fog* GetFog(void){ return mpFog; }
	
	inline u32 GetMirrorType(void){ return mMirrorType; }
	inline void GetMirrorPos(Vec &pos){ pos = mMirrorPos; }
	bool ClearMirror(void);
	bool SetMirror(u32 mirrorType,const Vec *pPos=NULL);
	
	WK7SdExpInfo* GetSdExpInfo(void);
	WK7Model* CreateMirrorModel(u32 type);
	
	bool SetAlpha(f32 alpha,u32 tick=0);
	f32 GetAlpha(void);
	inline void SetCameraAlphaControl(bool flag){ mCameraAlphaControl=(flag==true? 1:0); }
	inline bool GetCameraAlphaControl(void){ return mCameraAlphaControl? true:false; }
	
	bool MakeBound(int index=-1);
	
	static bool InitializeAll(void);
	static bool TerminateAll(void);
	static bool FrameProcessEdgeShadow(void);
	static bool RenderProcessEdgeShadow(void);
	static bool RenderProcessEdgeShadowPaint(void);
	
	bool Attach(WK7JObj *mpAttachJObj);
	bool Detach(void);
	
	// \\
	bool Hide(bool next=true);
	bool Disp(bool next=true);
	
	// t[
	virtual bool FrameProcess(void);
	
	// _O
	virtual bool RenderProcess(HSD_TrspMask trsp=HSD_TRSP_ALL,u32 rendermode=0);
	
	WK7JObj* GetObj(const char *pName,bool child=true);
	WK7JObj* GetObjLC(const char *pName,bool child=true);
	WK7JObj* GetObj(int index);
	WK7JObj* GetObj(HSD_Joint *pJoint,bool child=true);
	inline WK7JObj* GetTopObj(void){ return mpTopObj; }
	WK7JObj* GetEffectObj(u32 id);
	WK7JObj* GetObjPartialName(const char *pPartialName,bool child=true);
	WK7JObj* GetObjPartialName2(const char *pName,bool child=true);
	
	bool SetFlag(u32 flag,int index=0);
	bool ClearFlag(u32 flag,int index=0);
	
	bool SetPE(HSD_PEDesc *pPEDesc,int index=-1);
	bool SetDstAlpha(u8 alpha,int index=-1);
	
	bool SetAlphaUpdate(bool flag,int index=-1,u32 trsp=0xffffffff);
	bool SetNoZUpdate(bool flag,int index=-1,u32 trsp=0xffffffff);
	
	f32 GetTotalMotionFrame(bool next=true);
	f32 GetCurrentMotionFrame(bool next=true);
	
	WK7JObj* AttachMotion(WK7ModelMotion *pModelMotion,const char *pModelName,const char *pMotionName,bool next=true,bool child=true,bool overwrite=false);
	int AttachMotion(WK7ModelMotion *pModelMotion,bool overwrite=false);
	int AttachMotion(WK7ModelMotion *pModelMotion,int motindex,int index=0,bool autoattr=true,bool overwrite=false);
	int AttachMotionClassic(WK7ModelMotion *pModelMotion);
	int AttachMotionClassic(WK7ModelMotion *pModelMotion,int motindex,int index=0,bool autoattr=true);
	int AttachMotion(HSD_AnimJoint *pAnimJoint,int index=0,bool overwrite=false);
	int AttachMotionClassic(HSD_AnimJoint *pAnimJoint,int index=0);
	int AttachMotionAll(WK7ModelMotion *pModelMotion,bool overwrite=false);
	bool DetachMotion(int index=-1);
	bool DetachMotionAll(int index=-1);
	bool ResetMotion(int index=-1);
	bool ResetMotionAll(int index=-1);
	bool PlayMotion(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,int index=-1);
	bool PlayMotionNA(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,int index=-1);
	bool PlayMotionAll(f32 startframe=0.0f,bool interpolate=false,f32 interpolateframe=0.0f,int index=-1);
	bool ReplayMotion(int index=-1);
	bool ReplayMotionAll(int index=-1);
	bool StopMotion(int index=-1);
	bool StopMotionAll(int index=-1);
	
	WK7JObj* AttachMatMotion(WK7ModelMotion *pModelMotion,const char *pModelName,const char *pMotionName,bool childOnly=true,bool next=true,bool child=true);
	int AttachMatMotion(WK7ModelMotion *pModelMotion);
	int AttachMatMotion(WK7ModelMotion *pModelMotion,int motindex,int index=0);
	int AttachMatMotionClassic(WK7ModelMotion *pModelMotion);
	int AttachMatMotionClassic(WK7ModelMotion *pModelMotion,int motindex,int index=0);
	int AttachMatMotion(HSD_MatAnimJoint *pMatAnimJoint,int index=0);
	int AttachMatMotionClassic(HSD_MatAnimJoint *pMatAnimJoint,int index=0);
	bool DetachMatMotion(int index=-1);
	bool ResetMatMotion(int index=-1);
	bool PlayMatMotion(f32 startframe=0.0f,int index=-1);
	bool ReplayMatMotion(int index=-1);
	bool StopMatMotion(int index=-1);
	
	WK7JObj* AttachShapeMotion(WK7ModelMotion *pModelMotion,const char *pModelName,const char *pMotionName,bool childOnly=true,bool next=true,bool child=true,bool overwrite=false);
	int AttachShapeMotion(WK7ModelMotion *pModelMotion,bool overwrite=false);
	int AttachShapeMotion(WK7ModelMotion *pModelMotion,int motindex,int index=0,bool overwrite=false);
	int AttachShapeMotionClassic(WK7ModelMotion *pModelMotion);
	int AttachShapeMotionClassic(WK7ModelMotion *pModelMotion,int motindex,int index=0);
	int AttachShapeMotion(HSD_ShapeAnimJoint *pShapeAnimJoint,int index=0,bool overwrite=false);
	int AttachShapeMotionClassic(HSD_ShapeAnimJoint *pShapeAnimJoint,int index=0);
	bool DetachShapeMotion(int index=-1);
	bool ResetShapeMotion(int index=-1);
	bool PlayShapeMotion(f32 startframe=0.0f,int index=-1);
	bool ReplayShapeMotion(int index=-1);
	bool StopShapeMotion(int index=-1);
	
	bool SetMotionRate(f32 rate=1.0f,bool next=true);
	bool RenewMotionRate(bool next=true);
	
	int GetMotionState(void);
	int GetShapeMotionState(void);
	
	bool SetMotionLoop(bool flag,int index=-1);
	bool GetMotionLoop(int index=0);
	
	bool SetPosition(f32 x,f32 y,f32 z,int index=0);
	bool SetPosition(const Vec *pPosition,int index=0);
	inline bool SetPosition(const Vec &position,int index=0){ return SetPosition(&position,index); }
	
	bool GetPosition(Vec *pPosition,int index=0);
	inline bool GetPosition(Vec &position,int index=0){ return GetPosition(&position,index); }
	
	bool SetRotation(f32 x,f32 y,f32 z,int index=0);
	bool SetRotation(const Vec *pRotation,int index=0);
	inline bool SetRotation(const Vec &Rotation,int index=0){ return SetRotation(&Rotation,index); }
	bool SetRotX(f32 x,int index=0);
	bool SetRotY(f32 y,int index=0);
	bool SetRotZ(f32 z,int index=0);
	
	bool GetRotation(f32 *px,f32 *py,f32 *pz,int index=0);
	inline bool GetRotation(f32 &x,f32 &y,f32 &z,int index=0){ return GetRotation(&x,&y,&z,index); }
	bool GetRotation(Vec *pRotation,int index=0);
	inline bool GetRotation(Vec &Rotation,int index=0){ return GetRotation(&Rotation,index); }
	
	bool SetScale(f32 x,f32 y,f32 z,int index=0);
	bool SetScale(const Vec *pScale,int index=0);
	inline bool SetScale(const Vec &Scale,int index=0){ return SetScale(&Scale,index); }
	
	bool GetScale(f32 *px,f32 *py,f32 *pz,int index=0);
	inline bool GetScale(f32 &x,f32 &y,f32 &z,int index=0){ return GetScale(&x,&y,&z,index); }
	bool GetScale(Vec *pScale,int index=0);
	inline bool GetScale(Vec &Scale,int index=0){ return GetScale(&Scale,index); }
	
	MtxPtr GetMtxPtr(int index);
	bool GetWorldPosition(Vec *pPosition,int index=0);
	inline bool GetWorldPosition(Vec &position,int index=0){ return GetWorldPosition(&position,index); }
	
	static bool ClearAlpha(u32 alpha=0);
	
	bool SetMaterialAmbientColor(GXColor col,int index=-1);
	bool SetMaterialAmbientColorLE(GXColor col,int index=-1);
	bool SetMaterialDiffuseColor(GXColor col,int index=-1);
	bool SetMaterialDiffuseColorLE(GXColor col,int index=-1);
	bool SetMaterialSpecularColor(GXColor col,int index=-1);
	bool SetMaterialSpecularColorLE(GXColor col,int index=-1);
	
	bool SetMaterialAmbientColor(GXColor col,u32 tick);
	bool SetMaterialAmbientColorLE(GXColor col,u32 tick);
	bool SetMaterialDiffuseColor(GXColor col,u32 tick);
	bool SetMaterialDiffuseColorLE(GXColor col,u32 tick);
	
};

#define	WK7MODEL_NOANIM			(AOBJ_NO_ANIM)

#endif	//__WK7MODEL_H__

