////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Path
//
// WK7Path.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 01/30/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7PATH_CPP__

//#include <dolphin.h>
//#include <sysdolphin\baselib.h>
//#include "..\Lib\ghmGcDefine.h"
#include ".\WK7Path.h"
#include ".\WK7Event.h"
#include ".\WK7Extra.h"
#include ".\WK7MainMenu.h"
#include "..\Fujita\C_Font.h"
#include "..\Stage\K7Stage121M00.h"
#include "..\Stage\K7Stage578M00.h"
#include ".\WK7SelectIF.h"
#include ".\WK7Archive.h"

//#define	PLAYTEST

f32 WK7Path::mInterpolateDist    = WK7PATH_DEFAULT_INTERPOLATE_DIST;
int WK7Path::mPassPathFlagID     = -1;
int WK7Path::mPassPathFlagIDMax  = -1;
int WK7Path::mSelectObjFlagID    = -1;
int WK7Path::mSelectObjFlagIDMax = -1;
int WK7PathNurbs::mCurrentLevel  = 0;

WK7PathBranch::WK7PathBranch(WK7PathNurbs *pPath,WK7PathBranch *pPrev,WK7PathBranch *pNext,u32 dir)
{
	mpPath = pPath;
	mpPrev = pPrev;
	mpNext = pNext;
	if( pPrev!=NULL ){
		pPrev->mpNext = this;
	}
	if( pNext!=NULL ){
		pNext->mpPrev = this;
	}
	mFlag  = 0;
	mDir   = dir? 1 : 0;
}

WK7PathBranch::~WK7PathBranch()
{
	WK7SELECTIF->ReleaseBranch(this);
	if( mpPrev != NULL ){
		mpPrev->mpNext = mpNext;
	}
	if( mpNext != NULL ){
		mpNext->mpPrev = mpPrev;
	}
}

f32 WK7PathBranch::GetLimitParam(f32 param)
{
	if( param < mpPath->GetStartParam() ){
		param = mpPath->GetStartParam();
	}else if( param > mpPath->GetEndParam() ){
		param = mpPath->GetEndParam();
	}
	return param;
}

bool WK7PathBranch::GetPosition(Vec *pPosition,f32 param,bool limit)
{
	if( mpPath==NULL || pPosition==NULL ){
		return false;
	}
	f32		lparam = GetLimitParam(param);
	if( lparam!=param && limit==false ){
		return false;
	}
	return mpPath->GetPoint((*pPosition),lparam);
}

bool WK7PathBranch::GetDeltaPosition(Vec *pDeltaPosition,f32 param,bool limit)
{
	if( mpPath==NULL || pDeltaPosition==NULL ){
		return false;
	}
	f32		lparam = GetLimitParam(param);
	if( lparam!=param && limit==false ){
		return false;
	}
	bool	result = mpPath->GetDiffVec((*pDeltaPosition),lparam);
	if( mDir ){
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}
	return result;
}

bool WK7PathBranch::GetStartPosition(Vec *pPosition)
{
	if( mpPath==NULL || pPosition==NULL ){
		return false;
	}
	if( mDir ){
		return mpPath->GetPoint((*pPosition),mpPath->GetEndParam());
	}else{
		return mpPath->GetPoint((*pPosition),mpPath->GetStartParam());
	}
}

bool WK7PathBranch::GetStartDeltaPosition(Vec *pDeltaPosition)
{
	if( mpPath==NULL || pDeltaPosition==NULL ){
		return false;
	}
	bool	result;
	if( mDir ){
		result = mpPath->GetDiffVec((*pDeltaPosition),mpPath->GetEndParam());
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}else{
		result = mpPath->GetDiffVec((*pDeltaPosition),mpPath->GetStartParam());
	}
	return result;
}

bool WK7PathBranch::GetEndPosition(Vec *pPosition)
{
	if( mpPath==NULL || pPosition==NULL ){
		return false;
	}
	if( mDir ){
		return mpPath->GetPoint((*pPosition),mpPath->GetStartParam());
	}else{
		return mpPath->GetPoint((*pPosition),mpPath->GetEndParam());
	}
}

bool WK7PathBranch::GetEndDeltaPosition(Vec *pDeltaPosition)
{
	if( mpPath==NULL || pDeltaPosition==NULL ){
		return false;
	}
	bool	result;
	if( mDir ){
		result = mpPath->GetDiffVec((*pDeltaPosition),mpPath->GetStartParam());
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}else{
		result = mpPath->GetDiffVec((*pDeltaPosition),mpPath->GetEndParam());
	}
	return result;
}

f32 WK7PathBranch::GetParamDistance(f32 param)
{
	if( mpPath==NULL ){
		return -1.0f;
	}
	return mpPath->GetParamDistance(param);
}

f32 WK7PathBranch::GetStartParamDistance(f32 param)
{
	if( mpPath==NULL ){
		return -1.0f;
	}
	f32		dist = mpPath->GetParamDistance(param);
	if( mDir ){
		dist = mpPath->GetTotalDistance() - dist;
	}
	return dist;
}

f32 WK7PathBranch::GetEndParamDistance(f32 param)
{
	if( mpPath==NULL ){
		return -1.0f;
	}
	f32		dist = mpPath->GetParamDistance(param);
	if( !mDir ){
		dist = mpPath->GetTotalDistance() - dist;
	}
	return dist;
}

f32 WK7PathBranch::GetTotalDistance(void)
{
	if( mpPath==NULL ){
		return -1.0f;
	}
	return mpPath->GetTotalDistance();
}

bool WK7PathBranch::IsStairs(void)
{
	if( mpPath==NULL ){
		return false;
	}
	return mpPath->GetStairs();
}

int WK7PathBranch::GetStairsState(void)
{
	if( mpPath==NULL ){
		return -1;
	}
	if( mpPath->GetStairs()==false ){
		return WK7PATH_FLAT;
	}
	if( mpPath->GetUpStairs()==true ){
		if( mDir ){
			return WK7PATH_DOWNSTAIRS;
		}else{
			return WK7PATH_UPSTAIRS;
		}
	}else{
		if( mDir ){
			return WK7PATH_UPSTAIRS;
		}else{
			return WK7PATH_DOWNSTAIRS;
		}
	}
}

bool WK7PathBranch::IsCurb(void)
{
	if( mpPath==NULL ){
		return false;
	}
	return mpPath->GetCurb();
}

bool WK7PathBranch::Pass(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->Pass(dir);
}

bool WK7PathBranch::ClearPass(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->ClearPass(dir);
}

bool WK7PathBranch::HasPassed(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->HasPassed(dir);
}

bool WK7PathBranch::Select(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->Select(dir);
}

bool WK7PathBranch::ClearSelect(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->ClearSelect(dir);
}

bool WK7PathBranch::HasSelected(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->HasSelected(dir);
}

WK7PathBranchPointObj::WK7PathBranchPointObj(WK7PathBranchPoint *pContainer,ghmGcListObj *pAnchor) : WK7Obj(pAnchor)
{
	mpContainer   = pContainer;
	mpObjName[0]  = 
	mpObjName[1]  = NULL;
	mEventID      = 0xffffffff;
	mFlag         = 0;
	mPlayEnable   = 1;
	mEnableSelect = 1;
	mType         = TYPE_NORMAL;
	if( WK7Path::GetSelectObjFlagID() >= 0 ){
		mSelectObjFlagID = WK7Path::GetSelectObjFlagID(); WK7Path::IncSelectObjFlagID();
	}else{
		mSelectObjFlagID = -1;
	}
	memset(&mPosition,0,sizeof(mPosition));
}

WK7PathBranchPointObj::~WK7PathBranchPointObj()
{
	WK7Extra::ReleasePathBranchPointObj(this);
	WK7SELECTIF->ReleasePathBranchPointObj(this);
	GHMGCMEM_SAFE_FREE(mpObjName[0]); 
	GHMGCMEM_SAFE_FREE(mpObjName[1]);
}

bool WK7PathBranchPointObj::SetObjName(int index,const char *pName)
{
	if( index < 0 || index > 1 ){
		return false;
	}
	GHMGCMEM_SAFE_FREE(mpObjName[index]);
	if( pName==NULL ){
		return true;
	}
	ghmGcMemSetAllocName("PBPONM");
	if( (mpObjName[index] = (char *)K7AppSmallMemAlloc((int)(strlen(pName) + 1)))==NULL ){
		return false;
	}
	strcpy(mpObjName[index],pName);
	return true;
}

const char* WK7PathBranchPointObj::GetObjName(int index)
{
	if( index < 0 || index > 1 ){
		return NULL;
	}
	return mpObjName[index];
}

bool WK7PathBranchPointObj::Select(int dir)
{
	if( mSelectObjFlagID < 0 || mpContainer==NULL ){
		return false;
	}
	WK7Stage	*pStage = mpContainer->GetStage();
	if( pStage==NULL ){
		return false;
	}
	if( pStage->SetStageSelectObjFlag(mSelectObjFlagID,dir,true) <= 0 ){
		return false;
	}
	return true;
}

bool WK7PathBranchPointObj::ClearSelect(int dir)
{
	if( mSelectObjFlagID < 0 || mpContainer==NULL ){
		return false;
	}
	WK7Stage	*pStage = mpContainer->GetStage();
	if( pStage==NULL ){
		return false;
	}
	if( pStage->SetStageSelectObjFlag(mSelectObjFlagID,dir,false) <= 0 ){
		return false;
	}
	return true;
}

bool WK7PathBranchPointObj::HasSelected(int dir)
{
	if( mSelectObjFlagID < 0 || mpContainer==NULL ){
		return false;
	}
	WK7Stage	*pStage = mpContainer->GetStage();
	if( pStage==NULL ){
		return false;
	}
	if( dir < 0 ){
		for(dir=0 ; dir<2 ; dir++){
			if( pStage->GetStageSelectObjFlag(mSelectObjFlagID,dir)==true ){
				return true;
			}
		}
		return false;
	}
	if( pStage->GetStageSelectObjFlag(mSelectObjFlagID,dir) <= 0 ){
		return false;
	}
	return true;
}

WK7PathBranchPointObjJObj::WK7PathBranchPointObjJObj(WK7PathBranchPoint *pContainer,ghmGcListObj *pAnchor,WK7JObj *pJObj)
                         : WK7PathBranchPointObj(pContainer,pAnchor)
{
	mpJObj = pJObj;
	mType  = TYPE_JOBJ;
	mEventNo = 0xffffffff;
}

void WK7PathBranchPointObjJObj::GetPosition(Vec &pos)
{
	if( mpJObj!=NULL ){
		MtxPtr		mtxptr = mpJObj->GetLocalWorldMatrix();
		mPosition.x = mtxptr[0][3];
		mPosition.y = mtxptr[1][3];
		mPosition.z = mtxptr[2][3];
	}
	pos = mPosition;
}

WK7PathBranchPointObjJObjDoor::WK7PathBranchPointObjJObjDoor(WK7PathBranchPoint *pContainer,ghmGcListObj *pAnchor,WK7JObj *pJObj)
                             : WK7PathBranchPointObjJObj(pContainer,pAnchor,pJObj)
{
	mNextStageID = 0xffffffff;
	mParam       = 0;
	mpNextPoint  = NULL;
	mType        = TYPE_JOBJDOOR;
}

WK7PathBranchPointObjJObjDoor::~WK7PathBranchPointObjJObjDoor()
{
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
}

bool WK7PathBranchPointObjJObjDoor::SetNextPoint(const char *pName)
{
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
	if( pName==NULL ){
		return true;
	}
	u32		len = strlen(pName);
	ghmGcMemSetAllocName("PBNP");
	if( (mpNextPoint = new char[len + 1])==NULL ){
		return false;
	}
	strcpy(mpNextPoint,pName);
	return true;
}

WK7PathBranchPoint::WK7PathBranchPoint(WK7Path *pContainer,const char *pName,WK7PathBranchPoint *pPrev,WK7PathBranchPoint *pNext)
{
	mpContainer    = pContainer;
	mBranchNum     = 0;
	mpBranch       = NULL;
	mFlag          = 0;
	mArriveEventID = WK7PATHBRANCHPOINT_NOARRIVEEVENTID;
	mpPrev         = pPrev;
	mpNext         = pNext;
	mpName         = pName;
	mpShortName    = NULL;
	if( pName!=NULL ){
		ghmGcMemSetAllocName("BpEvent");
		WK7StageBpEvent	*pBpEvent = new WK7StageBpEvent(pName);
		if( pBpEvent!=NULL ){
			const char	*pShortName;
			if( (pShortName=pBpEvent->GetShortBpName())!=NULL ){
				ghmGcMemSetAllocName("PathBpSN");
				mpShortName = new char[strlen(pShortName) + 1];
				if( mpShortName!=NULL ){
					strcpy(mpShortName,pShortName);
				}
			}
			GHM_SAFE_DELETE(pBpEvent);
		}
	}
	if( pPrev!=NULL ){
		pPrev->mpNext = this;
	}
	if( pNext!=NULL ){
		pNext->mpPrev = this;
	}
}

WK7PathBranchPoint::~WK7PathBranchPoint()
{
	if( mpPrev != NULL ){
		mpPrev->mpNext = mpNext;
	}
	if( mpNext != NULL ){
		mpNext->mpPrev = mpPrev;
	}
	WK7PathBranch		*pBranch,*pNext;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pNext){
		pNext = pBranch->GetNext();
		delete pBranch;
	}
	GHM_SAFE_DELETE_ARRAY(mpShortName);
}

WK7Stage* WK7PathBranchPoint::GetStage(void)
{
	if( mpContainer==NULL ){
		return NULL;
	}
	return mpContainer->GetContainer();
}

int WK7PathBranchPoint::AddBranch(WK7PathNurbs *pPath,u32 dir)
{
	if( pPath==NULL ){
		return -1;
	}
	ghmGcMemSetAllocName("PathBr");
	WK7PathBranch	*pBranch = new WK7PathBranch(pPath,NULL,mpBranch,dir);
	mpBranch = pBranch;
	mBranchNum++;
	return 0;
}

bool WK7PathBranchPoint::RemoveBranch(WK7PathBranch* pBranch)
{
	if( pBranch==NULL ){
		return false;
	}
	WK7PathBranch		*pTmpBranch;
	for(pTmpBranch=mpBranch ; pTmpBranch!=NULL ; pTmpBranch=pTmpBranch->GetNext()){
		if( pTmpBranch!=pBranch ){
			continue;
		}
		if( mpBranch==pBranch ){
			mpBranch = pBranch->GetNext();
		}
		delete pBranch;
		mBranchNum--;
		return true;
	}
	return false;
}

bool WK7PathBranchPoint::RemoveBranch(WK7PathNurbs* pPath)
{
	WK7PathBranch	*pBranch = GetBranch(pPath);
	if( pBranch==NULL ){
		return false;
	}
	if( mpBranch==pBranch ){
		mpBranch = pBranch->GetNext();
	}
	delete pBranch;
	mBranchNum--;
	return false;
}

WK7PathBranch* WK7PathBranchPoint::GetBranch(WK7PathNurbs *pPath)
{
	WK7PathBranch		*pBranch;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( pBranch->GetPath()==pPath ){
			return pBranch;
		}
	}
	return NULL;
}

int WK7PathBranchPoint::GetEnableBranchNum(WK7PathBranch *pExceptBranch,bool enableex)
{
	WK7PathNurbs	*pExceptPath;
	if( pExceptBranch==NULL ){
		pExceptPath = NULL;
	}else{
		pExceptPath = pExceptBranch->GetPath();
	}
	return GetEnableBranchNum(pExceptPath,enableex);
}

int WK7PathBranchPoint::GetEnableBranchNum(WK7PathNurbs *pExceptPath,bool enableex)
{
	WK7PathBranch		*pBranch;
	WK7PathNurbs		*pPath;
	int					count;
	for(pBranch=mpBranch,count=0 ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==pExceptPath ){
			continue;
		}
		if( pPath==NULL ){
			continue;
		}
		if( enableex==true ){
			if( pPath->GetEnableEx()==false ){
				continue;
			}
			if( pPath->GetStageJump()==true ){
				continue;
			}
		}else{
			if( pPath->GetEnable()==false ){
				continue;
			}
			if( pPath->GetStageJump()==true ){
				if( pPath->GetNextStageID()==0xffffffff || pPath->GetEnableEvent()==false ){
//				if( pPath->GetEventID()==0xffffffff || pPath->GetEnableEvent()==false ){
					continue;
				}
			}
		}
		count++;
	}
//	count += GetEnableObjNum();
	return count;
}

//int WK7PathBranchPoint::GetEnableBranchMoveNum(WK7PathNurbs *pExceptPath,bool enableex)
//{
//	WK7PathBranch		*pBranch;
//	WK7PathNurbs		*pPath;
//	int					count;
//	for(pBranch=mpBranch,count=0 ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
//		if( (pPath=pBranch->GetPath())==pExceptPath ){
//			continue;
//		}
//		if( pPath==NULL ){
//			continue;
//		}
//		if( pPath->GetEnable()==false ){
//			if( enableex==true ){
//				if( pPath->GetEnableEx()==false ){
//					continue;
//				}
//			}else{
//				continue;
//			}
//		}
//		count++;
//	}
//	return count;
//}

int WK7PathBranchPoint::GetEnableObjNum(void)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetEnable()==false ){
			continue;
		}
		count++;
	}
	return count;
}

int WK7PathBranchPoint::GetObjNum(void)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		count++;
	}
	return count;
}

int WK7PathBranchPoint::GetBranchPathNum(void)
{
	WK7PathBranch		*pBranch;
 	WK7PathNurbs		*pPath;
 	int					count = 0;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==NULL ){
			continue;
		}
		if( pPath->GetEventID()==0xffffffff ){
			continue;
		}
		count++;
	}
	return count;
}

int WK7PathBranchPoint::GetEnableBranchPathNum(WK7PathNurbs *pExceptPath)
{
	WK7PathBranch		*pBranch;
 	WK7PathNurbs		*pPath;
 	int					count = 0;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==NULL ){
			continue;
		}
		if( pPath==pExceptPath ){
			continue;
		}
		if( pPath->GetEnable()==false ){
			continue;
		}
		if( pPath->GetNextStageID()==0xffffffff ){
			continue;
		}
		if( pPath->GetEnableEvent()==false ){
			continue;
		}
		count++;
	}
	return count;
}

WK7PathBranch* WK7PathBranchPoint::SelectNextDefaultBranch(const Vec *pDir,WK7PathNurbs *pExceptPath,bool enableex)
{
 	if( pDir==NULL ){
		return NULL;
	}
 	WK7PathBranch		*pBranch;
 	WK7PathBranch		*pNextBranch = NULL;
 	WK7PathNurbs		*pPath;
	Vec					dir;
	f32					ipmax = -2.0f;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==NULL ){
			continue;
		}
		if( pPath==pExceptPath ){
			continue;
		}
		if( enableex==true ){
			if( pPath->GetEnableEx()==false ){
				continue;
			}
			if( pPath->GetStageJump()==true ){
				continue;
			}
		}else{
			if( pPath->GetEnable()==false ){
				continue;
			}
		}
		pBranch->GetStartDeltaPosition(&dir);
		f32		ip;
		// K
		VECNormalize(&dir,&dir);
		// 
		ip = pDir->x * dir.x + pDir->y * dir.y + pDir->z * dir.z;
		if( ipmax < ip ){
			ipmax = ip;
			pNextBranch = pBranch;
		}
	}
	return pNextBranch;
}

WK7PathBranchPointObj* WK7PathBranchPoint::AddObj(void)
{
	ghmGcMemSetAllocName("PBO");
	WK7PathBranchPointObj		*pObj = new WK7PathBranchPointObj(this,&mObjAnchor);
	return pObj;
}

WK7PathBranchPointObjJObj* WK7PathBranchPoint::AddObjJObj(WK7JObj *pJObj)
{
	ghmGcMemSetAllocName("PBOJ");
	WK7PathBranchPointObjJObj	*pObj = new WK7PathBranchPointObjJObj(this,&mObjAnchor,pJObj);
	return pObj;
}

WK7PathBranchPointObjJObjDoor* WK7PathBranchPoint::AddObjJObjDoor(WK7JObj *pJObj)
{
	ghmGcMemSetAllocName("PBOJD");
	WK7PathBranchPointObjJObjDoor	*pObj = new WK7PathBranchPointObjJObjDoor(this,&mObjAnchor,pJObj);
	return pObj;
}

bool WK7PathBranchPoint::RemoveObj(WK7PathBranchPointObj* pRemoveObj)
{
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj==pRemoveObj ){
			delete pObj;
			return true;
		}
	}
	return false;
}

WK7PathBranchPointObj* WK7PathBranchPoint::GetObj(const char *pObjName0,const char *pObjName1)
{
	if( pObjName0==NULL ){
		return NULL;
	}
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetObjName(0)==NULL ){
			continue;
		}
		if( strcmp(pObjName0,pObj->GetObjName(0))!=0 ){
			continue;
		}
		if( pObjName1==NULL ){
			return pObj;
		}
		if( pObj->GetObjName(1)==NULL ){
			continue;
		}
		if( strcmp(pObjName1,pObj->GetObjName(1))!=0 ){
			continue;
		}
		return pObj;
	}
	return NULL;
}

WK7PathBranchPointObj* WK7PathBranchPoint::GetObj(u32 eventid)
{
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetEventID()==eventid ){
			return pObj;
		}
	}
	return NULL;
}

WK7PathBranchPointObjJObj* WK7PathBranchPoint::GetObj_EventNo(u32 eventno)
{
	WK7PathBranchPointObj		*pObj;
	WK7PathBranchPointObjJObj	*pObjJObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		pObjJObj = (WK7PathBranchPointObjJObj *)pObj;
		if( pObjJObj->GetEventNo()==eventno ){
			return pObjJObj;
		}
	}
	return NULL;
}

WK7PathBranchPointObjJObj* WK7PathBranchPoint::GetObj(WK7JObj *pJObj)
{
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;;
		default:
			continue;
		}
		if( ((WK7PathBranchPointObjJObj *)pObj)->GetJObj()==pJObj ){
			return (WK7PathBranchPointObjJObj *)pObj;
		}
	}
	return NULL;
}

WK7PathBranchPointEvent* WK7PathBranchPoint::FindEvent(WK7PathBranchPointEvent *pEvent,u32 eventid)
{
	if( pEvent==NULL ){
		pEvent = (WK7PathBranchPointEvent *)mEventAnchor.GetNext();
	}else{
		pEvent = pEvent->GetNext();
	}
	if( eventid==0xffffffff ){
		return pEvent;
	}
	for( ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		if( pEvent->GetEventID()==eventid ){
			return pEvent;
		}
	}
	return NULL;
}

WK7PathBranchPointEvent* WK7PathBranchPoint::FindEvent(WK7PathBranchPointEvent *pEvent,WK7PathBranchPointEvent::Type type)
{
	if( pEvent==NULL ){
		pEvent = (WK7PathBranchPointEvent *)mEventAnchor.GetNext();
	}else{
		pEvent = pEvent->GetNext();
	}
	if( type==WK7PathBranchPointEvent::BPEVTYPE_ALL ){
		return pEvent;
	}
	for( ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		if( pEvent->GetType()==type ){
			return pEvent;
		}
	}
	return NULL;
}


WK7PathBranchPointEventArrive* WK7PathBranchPoint::CreateArriveEvent(u32 eventid)
{
	ghmGcMemSetAllocName("PBPEA");
	return new WK7PathBranchPointEventArrive(&mEventAnchor,eventid);
}

WK7PathBranchPointEventArriveEventNo* WK7PathBranchPoint::CreateArriveEventNo(u32 eventid,u32 eventno)
{
	ghmGcMemSetAllocName("PBPEAEN");
	return new WK7PathBranchPointEventArriveEventNo(&mEventAnchor,eventid,eventno);
}

WK7PathBranchPointEventLeave* WK7PathBranchPoint::CreateLeaveEvent(u32 eventid)
{
	ghmGcMemSetAllocName("PBPEL");
	return new WK7PathBranchPointEventLeave(&mEventAnchor,eventid);
}

#define	WK7PATHBRANCHPOINTEVENT_PREFIX		"event_"
char* WK7PathBranchPointEventNo::GetPathName(u32 eventno)
{
	ghmGcMemSetAllocName("PathName");
	char		*pPathName = new char[strlen(WK7PATHBRANCHPOINTEVENT_PREFIX) + 6 + 1];
	if( pPathName==NULL ){
		return NULL;
	}
	sprintf(pPathName,"%s%06x",WK7PATHBRANCHPOINTEVENT_PREFIX,(eventno & 0x00ffffff));
	return pPathName;
}

char* WK7PathBranchPointEventLeaveEventNo::GetPathName(u32 eventno,u32 pathno)
{
	char	*pPrefixPathName = WK7PathBranchPointEventNo::GetPathName(eventno);
	if( pPrefixPathName==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("PathName");
	char	*pPathName = new char[strlen(pPrefixPathName) + 3];
	if( pPathName!=NULL ){
		sprintf(pPathName,"%s_%1d",pPrefixPathName,pathno % 10);
	}
	delete[] pPrefixPathName;
	return pPathName;
}

WK7PathBranchPointEventLeaveEventNo::WK7PathBranchPointEventLeaveEventNo(ghmGcListObj *pAnchor,
                                                                         WK7PathBranch *pBranch,
                                                                         u32 eventid,
                                                                         u32 eventno,
                                                                         u32 pathno) :
                                               WK7PathBranchPointEventNo(pAnchor,eventid,eventno)
{
	const char	*pPathName;
	mType  = BPEVTYPE_LEAVE_EVNO;
	mpPath = NULL;
	if( (pPathName = GetPathName(eventno,pathno))==NULL ){
		return;
	}
	WK7PathNurbs	*pPath;
	for( ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath = pBranch->GetPath())==NULL || pPath->GetName()==NULL ){
			continue;
		}
		if( strstr(pPath->GetName(),pPathName)!=NULL ){
			mpPath = pPath;
			break;
		}
	}
	delete pPathName;
}

WK7PathBranchPointEventLeaveEventNo::WK7PathBranchPointEventLeaveEventNo(ghmGcListObj *pAnchor,
                                                                         WK7PathBranch *pBranch,
                                                                         u32 eventid,
                                                                         u32 eventno,
                                                                         const char* pPathName) :
                                               WK7PathBranchPointEventNo(pAnchor,eventid,eventno)
{
	mType  = BPEVTYPE_LEAVE_EVNO;
	mpPath = NULL;
	WK7PathNurbs	*pPath;
	for( ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath = pBranch->GetPath())==NULL || pPath->GetName()==NULL ){
			continue;
		}
		if( strstr(pPath->GetName(),pPathName)!=NULL ){
			mpPath = pPath;
			break;
		}
	}
}

WK7PathBranchPointEventLeaveEventNo* WK7PathBranchPoint::CreateLeaveEventNo(u32 eventid,u32 eventno,u32 pathno)
{
	if( mpBranch==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("PBPELEN");
	return new WK7PathBranchPointEventLeaveEventNo(&mEventAnchor,mpBranch,eventid,eventno,pathno);
}

WK7PathBranchPointEventLeaveEventNo* WK7PathBranchPoint::CreateLeaveEventNo(u32 eventid,u32 eventno,const char *pPathName)
{
	if( pPathName==NULL || mpBranch==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("PBPELEN");
	return new WK7PathBranchPointEventLeaveEventNo(&mEventAnchor,mpBranch,eventid,eventno,pPathName);
}

int WK7PathBranchPoint::SetPathEnable(bool flag)
{
 	WK7PathBranch		*pBranch;
 	WK7PathNurbs		*pPath;
 	int					count = 0;;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==NULL ){
			continue;
		}
		pPath->SetEnable(flag);
		count++;
	}
	return count;
}

int WK7PathBranchPoint::SetPathEnableEx(bool flag)
{
 	WK7PathBranch		*pBranch;
 	WK7PathNurbs		*pPath;
 	int					count = 0;;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( (pPath=pBranch->GetPath())==NULL ){
			continue;
		}
		pPath->SetEnableEx(flag);
		count++;
	}
	return count;
}

int WK7PathBranchPoint::SetEventEnable(u32 eventid,bool flag)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetEventID()==eventid ){
			pObj->SetEnable(flag);
			count++;
		}
	}
	WK7PathBranchPointEvent	*pEvent;
	for(pEvent=(WK7PathBranchPointEvent *)mEventAnchor.GetNext() ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		if( pEvent->GetEventID()==eventid ){
			pEvent->SetEnable(flag);
			count++;
		}
	}
	return count;
}

int WK7PathBranchPoint::ChangeEventNo(u32 from_evno,u32 to_evno)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		WK7PathBranchPointObjJObj	*pObjJObj = (WK7PathBranchPointObjJObj *)pObj;
		if( pObjJObj->GetEventNo()==from_evno ){
			pObjJObj->SetEventNo(to_evno);
			count++;
		}
	}
	WK7PathBranchPointEvent	*pEvent;
	for(pEvent=(WK7PathBranchPointEvent *)mEventAnchor.GetNext() ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		if( pEvent->HasEventNo()==false ){
			continue;
		}
		WK7PathBranchPointEventNo	*pEventNo = (WK7PathBranchPointEventNo *)pEvent;
		if( pEventNo->GetEventNo()==from_evno ){
			pEventNo->SetEventNo(to_evno);
			count++;
		}
	}
	return count;
}

int WK7PathBranchPoint::SetEventNoEnable(u32 eventno,bool flag)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		WK7PathBranchPointObjJObj	*pObjJObj = (WK7PathBranchPointObjJObj *)pObj;
		if( pObjJObj->GetEventNo()==eventno ){
			pObj->SetEnable(flag);
			count++;
		}
	}
	WK7PathBranchPointEvent	*pEvent;
	for(pEvent=(WK7PathBranchPointEvent *)mEventAnchor.GetNext() ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		switch( pEvent->GetType() ){
		case	WK7PathBranchPointEvent::BPEVTYPE_ARRIVE_EVNO:
		case	WK7PathBranchPointEvent::BPEVTYPE_LEAVE_EVNO:
		case	WK7PathBranchPointEvent::BPEVTYPE_LEAVE_EVNO_PATHNAME:
			break;
		default:
			continue;
		}
		WK7PathBranchPointEventNo	*pEventNo = (WK7PathBranchPointEventNo *)pEvent;
		if( pEventNo->GetEventNo()==eventno ){
			pEvent->SetEnable(flag);
			count++;
		}
	}
	return count;
}

int WK7PathBranchPoint::SetObjVisible(u32 eventid,bool flag)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		if( pObj->GetEventID()!=eventid ){
			continue;
		}
		WK7JObj	*pJObj = ((WK7PathBranchPointObjJObj *)pObj)->GetJObj();
		if( pJObj==NULL ){
			continue;
		}
		if( flag==true ){
			pJObj->Disp(false,true);
		}else{
			pJObj->Hide(false,true);
		}
		count++;
	}
	return count;
}

int WK7PathBranchPoint::SetObjVisibleEventNo(u32 eventno,bool flag)
{
	WK7PathBranchPointObj	*pObj;
	int						count = 0;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		WK7PathBranchPointObjJObj	*pObjJObj = (WK7PathBranchPointObjJObj *)pObj;
		if( pObjJObj->GetEventNo()!=eventno ){
			continue;
		}
		WK7JObj	*pJObj = pObjJObj->GetJObj();
		if( pJObj==NULL ){
			continue;
		}
		if( flag==true ){
			pJObj->Disp(false,true);
		}else{
			pJObj->Hide(false,true);
		}
		count++;
	}
	return count;
}

WK7JObj* WK7PathBranchPoint::GetObjEventNo(u32 eventno)
{
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		switch( pObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			break;
		default:
			continue;
		}
		WK7PathBranchPointObjJObj	*pObjJObj = (WK7PathBranchPointObjJObj *)pObj;
		if( pObjJObj->GetEventNo()!=eventno ){
			continue;
		}
		WK7JObj	*pJObj = pObjJObj->GetJObj();
		if( pJObj==NULL ){
			continue;
		}
		return pJObj;
	}
	return NULL;
}

bool WK7PathBranchPoint::IsValid(WK7PathBranchPointObj *pTgtBranchPointObj)
{
	WK7PathBranchPointObj	*pObj;
	for(pObj=(WK7PathBranchPointObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj==pTgtBranchPointObj ){
			return true;
		}
	}
	return false;
}

bool WK7PathBranchPoint::IsValid(WK7PathBranch *pTgtBranch)
{
	WK7PathBranch	*pBranch;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( pBranch==pTgtBranch ){
			return true;
		}
	}
	return false;
}

WK7PathNurbs::WK7PathNurbs(WK7Path *pContainer,ghmScenePathSet *pPathSet,WK7PathNurbs *pPrev,WK7PathNurbs *pNext,u32 stgid)
{
	InitMember(pContainer,pPathSet,pPrev,pNext,stgid);
}

WK7PathNurbs::WK7PathNurbs(WK7Path *pContainer,ghmScenePathSet *pPathSet)
{
	InitMember(pContainer,pPathSet,NULL,NULL,0xffffffff);
}

WK7PathNurbs::~WK7PathNurbs()
{
	SetRouteName(NULL);
	if( mpPrev != NULL ){
		mpPrev->mpNext = mpNext;
	}
	if( mpNext != NULL ){
		mpNext->mpPrev = mpPrev;
	}
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
	GHM_SAFE_DELETE_ARRAY(mpSelectObjName);
	RemoveStartBranchPoint();
	RemoveEndBranchPoint  ();
#ifdef	_DEBUG
	ReleaseRenderPosition ();
#endif
}

WK7Stage* WK7PathNurbs::GetStage(void)
{
	if( mpContainer==NULL ){
		return NULL;
	}
	return mpContainer->GetContainer();
}

int WK7PathNurbs::SetEventEnable(u32 eventid,bool flag)
{
	if( mEventID==0xffffffff ){
		return 0;
	}
	if( mEventID!=eventid ){
		return 0;
	}
	if( flag==true ){
		mEnableEvent = 1;
	}else{
		mEnableEvent = 0;
	}
	return 1;
}

int WK7PathNurbs::ChangeEventNo(u32 from_evno,u32 to_evno)
{
	if( mEventNo!=from_evno ){
		return 0;
	}
	mEventNo = to_evno;
	return 1;
}

int WK7PathNurbs::SetEventNoEnable(u32 eventno,bool flag)
{
	if( mEventNo!=eventno ){
		return 0;
	}
	if( flag==true ){
		mEnableEvent = 1;
	}else{
		mEnableEvent = 0;
	}
	return 1;
}

void WK7PathNurbs::ClearSearchPathInfo(void)
{
	mpSearchBranch         = NULL;
	mpSearchPath           = NULL;
	mSearchDist            = 0.0f;
	mSearchLevel           = -1;
	mNoSearchPath          = 0;
}

void WK7PathNurbs::ClearSearchPathInfoAll(void)
{
	WK7PathNurbs		*pPath = this;
	while( pPath->mpPrev!=NULL ){
		pPath = pPath->mpPrev;
	}
	while( pPath!=NULL ){
		pPath->ClearSearchPathInfo();
		pPath = pPath->mpNext;
	}
}

void WK7PathNurbs::InitMember(WK7Path *pContainer,ghmScenePathSet *pPathSet,WK7PathNurbs *pPrev,WK7PathNurbs *pNext,u32 stgid)
{
	WK7PathDataDesc		*pPathDataDesc = (WK7PathDataDesc *)pPathSet->mpData;
	mpContainer            = pContainer;
	mpSearchBranch         = NULL;
	mpSearchPath           = NULL;
	mSearchDist            = 0.0f;
	mSearchLevel           = -1;
	if( pPrev!=NULL ){
		pPrev->mpNext = this;
	}
	if( pNext!=NULL ){
		pNext->mpPrev = this;
	}
	mpPathSet              = pPathSet;
	mpPrev                 = pPrev;
	mpNext                 = pNext;
	mpStartBranchPoint     = NULL;
	mpEndBranchPoint       = NULL;
	mpStartGate            = NULL;
	mpEndGate              = NULL;
	mFlag                  = 0;
	mEnableEvent           = 1;
	mEnableSelect          = 1;
	if( pPathDataDesc!=NULL ){
		if( pPathDataDesc->mPass ){
			mEnable   = 1;
			mEnableEx = 1;
		}
		if( pContainer!=NULL && pContainer->GetVersion() >= 162 ){
			if( pPathDataDesc->mCamPass ){
				mEnableEx = 1;
			}else{
				mEnableEx = 0;
			}
		}
	}
	mNurbsCurve.SetDesc(pPathSet->mpDesc);
	mStartBranchLimitParam = mNurbsCurve.GetStartParam();
	mEndBranchLimitParam   = mNurbsCurve.GetEndParam  ();
	SetStartBranchLimitDist(WK7Path::GetInterpolateDist());
	SetEndBranchLimitDist  (WK7Path::GetInterpolateDist());
	if( mStartBranchLimitParam > mEndBranchLimitParam ){
		f32		av = (mStartBranchLimitParam + mEndBranchLimitParam) * 0.5f;
		mStartBranchLimitParam = av;
		mEndBranchLimitParam   = av;
	}
	const char	*pPathName;
	if( (pPathName=GetName())!=NULL ){
		Vec		s,e;
		GetStartPoint(s);
		GetEndPoint  (e);
		if( s.y <= e.y ){
			mUpStairs = 1;
		}
		if( strstr(pPathName,"stairs")==pPathName ){
			mStairs = 1;
		}else if( strstr(pPathName,"curb")==pPathName ){
			mCurb = 1;
		}
	}
#ifdef	_DEBUG
	mpRenderPosition   = NULL;
	mRenderPositionNum = 0;
	CreateRenderPosition();
#endif
	mpSelectObjName = NULL;
	mpNextPoint = NULL;
	SetEvent(stgid);
	mPassPathFlagID = -1;
}

bool WK7PathNurbs::SetPassFlag(void)
{
	if( mpStartBranchPoint==NULL && mpEndBranchPoint==NULL ){
		return false;
	}
	if( WK7Path::GetPassPathFlagID() < 0 ){
		return false;
	}
	mPassPathFlagID = WK7Path::GetPassPathFlagID(); WK7Path::IncPassPathFlagID();
	return true;
}

u16 WK7PathNurbs::GetFootSE(void)
{
	WK7PathDataDesc	*pDesc = GetPathDataDesc();
	if( pDesc==NULL ){
		return WK7PATH_FOOTSE_DEFAULT;
	}
	return pDesc->mFootSE;
}

WK7PathDataDesc* WK7PathNurbs::GetPathDataDesc(void)
{
	if( mpPathSet==NULL ){
		return NULL;
	}
	return (WK7PathDataDesc *)mpPathSet->mpData;
}

bool WK7PathNurbs::SetRouteName(const char *pRouteName)
{
	WK7PathDataDesc	*pDesc = GetPathDataDesc();
	if( pDesc==NULL ){
		return false;
	}
	if( pDesc->mRouteNameAlloc ){
		GHM_SAFE_DELETE_ARRAY(pDesc->mpRouteName);
		pDesc->mRouteNameAlloc = 0;
	}
	if( pRouteName==NULL ){
		return true;
	}
	u32		len = strlen(pRouteName) + 1;
	ghmGcMemSetAllocName("PATHRN");
	if( (pDesc->mpRouteName = new char[len])==NULL ){
		return false;
	}
	strcpy(pDesc->mpRouteName,pRouteName);
	pDesc->mRouteNameAlloc = 1;
	return true;
}

bool WK7PathNurbs::SearchName(const char *pName)
{
	if( pName==NULL ){
		return false;
	}
	const char	*pPathName = GetName();
	if( strcmp(pName,pPathName)==0 ){
		return true;
	}
	if( mpNextPoint!=NULL ){
		if( strstr(pPathName,WK7PATH_DOOR_PREFIX)==pPathName ){
			char	doorname[32];
			sprintf(doorname,WK7PATH_DOOR_PREFIX "_%06x",mEventNo & 0xffffff);
			if( strcmp(doorname,pName)==0 ){
				return true;
			}
		}else if( strstr(pPathName,WK7PATH_BRANCH_PREFIX)==pPathName ){
			char	*pbranchname;
			GetPartialStr(pbranchname,pPathName,'_');
			if( pbranchname!=NULL ){
				if( strcmp(pName,pbranchname)==0 ){
					GHM_SAFE_DELETE_ARRAY(pbranchname);
					return true;
				}
				GHM_SAFE_DELETE_ARRAY(pbranchname);
			}
		}
	}
	return false;
}

char	WK7PathNurbs::mShortPathNameWork[WK7PATH_SHORTPATHNAME_MAX] = { 0,};
const char* WK7PathNurbs::GetShortName(const char *pPathName,char *pName)
{
	if( pPathName==NULL ){
		return NULL;
	}
	char		doorname[32];
	const char	*pShortName;
	char		*pAllocName;
	bool		alloc = false;
	int			i;
	if( strstr(pPathName,WK7PATH_DOOR_PREFIX)==pPathName ){
//		sprintf(doorname,WK7PATH_DOOR_PREFIX "_%06x",mEventNo & 0xffffff);
		i = (int)strlen(WK7PATH_DOOR_PREFIX)+6+1;
		doorname[i] = '\0';
		for(i-- ; i>=0 ; i--){
			doorname[i] = pPathName[i];
		}
		pShortName = doorname;
	}else if( strstr(pPathName,WK7PATH_BRANCH_PREFIX)==pPathName ){
		GetPartialStr(pAllocName,pPathName,'_');
		if( pAllocName==NULL ){
			return NULL;
		}
		pShortName = pAllocName;
		alloc = true;
	}else{
		pShortName = pPathName;
	}
	
	if( pName==NULL ){
		strncpy(mShortPathNameWork,pShortName,WK7PATH_SHORTPATHNAME_MAX-1);
		if( alloc==true ){
			GHM_SAFE_DELETE_ARRAY(pAllocName);
		}
		return mShortPathNameWork;
	}
	
	strcpy(pName,pShortName);
	if( alloc==true ){
		GHM_SAFE_DELETE_ARRAY(pAllocName);
	}
	return pName;
}

const char* WK7PathNurbs::GetShortName(char *pName)
{
	if( mpNextPoint==NULL ){
		return NULL;
	}
	return WK7PathNurbs::GetShortName(GetName(),pName);
}

bool WK7PathNurbs::Pass(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->SetStagePassPathFlag(mPassPathFlagID,dir,true)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::ClearPass(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->SetStagePassPathFlag(mPassPathFlagID,dir,false)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::HasPassed(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	if( dir < 0 ){
		for(dir=0 ; dir<2 ; dir++){
			if( pStage->GetStagePassPathFlag(mPassPathFlagID,dir)==true ){
				return true;
			}
		}
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->GetStagePassPathFlag(mPassPathFlagID,dir)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::Select(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->SetStagePassPathFlag(mPassPathFlagID,2+dir,true)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::ClearSelect(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->SetStagePassPathFlag(mPassPathFlagID,2+dir,false)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::HasSelected(int dir)
{
	if( mPassPathFlagID < 0 ){
		return false;
	}
	WK7Stage	*pStage = GetStage();
	if( pStage==NULL ){
		return false;
	}
	if( dir < 0 ){
		for(dir=2 ; dir<4 ; dir++){
			if( pStage->GetStagePassPathFlag(mPassPathFlagID,dir)==true ){
				return true;
			}
		}
		return false;
	}
	switch( dir ){
	case	0:
	case	1:
		break;
	default:
		return false;
	}
	if( pStage->GetStagePassPathFlag(mPassPathFlagID,2+dir)==false ){
		return false;
	}
	return true;
}

bool WK7PathNurbs::ParseBranchEvent(u32 stgid)
{
#pragma	unused(stgid)
	
	const char	*pPathName = GetName();
	
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
	if( (pPathName=GetPartialStr(mpNextPoint,pPathName,'_'))==NULL ){
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	pPathName++;
	
	if( *pPathName < '0' || *pPathName > '9' ){
		GHM_SAFE_DELETE_ARRAY(mpSelectObjName);
		if( (pPathName=GetPartialStr(mpSelectObjName,pPathName,'_'))==NULL ){
			return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
		}
	}
	
	pPathName++;
	if( (pPathName = WK7Stage::GetStageID(pPathName,mNextStageID))==NULL ){
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	switch( mNextStageID ){
	case	WK7STAGE_TVSTAGEID:
	case	WK7STAGE_GMSTAGEID:
		if( pPathName[0] >= '1' && pPathName[0] <= '9' ){
			mParam = (u32)(pPathName[0] - '0');
			pPathName++;
			if( mNextStageID==WK7STAGE_TVSTAGEID ){
				if( pPathName[0]==WK7STAGE_TVSTAGE_SAVELETTER ){
					mParam |= WK7STAGE_TVSTAGE_SAVEPARAM;
					pPathName++;
				}
			}
		}
		break;
	default:
		break;
	}
	
	switch( *pPathName ){
	case	'\0':
	case	'_':
		break;
	default:
		GHM_SAFE_DELETE(mpNextPoint);
		if( (pPathName=GetPartialStr(mpNextPoint,pPathName,'_'))!=NULL ){
			pPathName++;
			if( strcmp(mpNextPoint,WK7PATH_DOOR_PREFIX)==0 ){
				const char	*pPrevNextPoint = mpNextPoint;
				char		*pEvNo;
				if( (pPathName=GetPartialStr(pEvNo,pPathName,'_'))!=NULL ){
					pPathName++;
				}
				u32		len = strlen(pEvNo) + strlen(pPrevNextPoint) + 2;
				ghmGcMemSetAllocName("PATHNPB0");
				if( (mpNextPoint = new char[len])==NULL ){
					GHM_SAFE_DELETE_ARRAY(pEvNo         );
					return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
				}
				strcpy(mpNextPoint,pPrevNextPoint);
				strcat(mpNextPoint,"_");
				strcat(mpNextPoint,pEvNo);
				GHM_SAFE_DELETE_ARRAY(pEvNo         );
				GHM_SAFE_DELETE_ARRAY(pPrevNextPoint);
			}else{
				break;
			}
		}
		break;
	}
	
	switch( mNextStageID ){
	case	WK7STAGE_TVSTAGEID:
		GHM_SAFE_DELETE_ARRAY(mpNextPoint);
		ghmGcMemSetAllocName("PATHNPB1");
		if( (mpNextPoint = new char[strlen(WK7STAGE_TVSTAGE_BPNAME) + 1])!=NULL ){
			strcpy(mpNextPoint,WK7STAGE_TVSTAGE_BPNAME);
		}
		break;
	case	WK7STAGE_GMSTAGEID:
		switch( mParam ){
		case	1:
		case	2:
		case	3:
			GHM_SAFE_DELETE_ARRAY(mpNextPoint);
			ghmGcMemSetAllocName("PATHNPB2");
			K7Stage578M00_PointInfo		*pPointInfo = K7Stage578M00::GetKihanShortCut(GameInfo.GetScenarioFlow(),(int)mParam);
			if( pPointInfo!=NULL ){
				if( (mpNextPoint = new char[strlen(pPointInfo->mpBpName) + 1])!=NULL ){
					strcpy(mpNextPoint,pPointInfo->mpBpName);
					mNextStageID = pPointInfo->mStageID;
				}
			}else{
				if( (mpNextPoint = new char[strlen(WK7STAGE_GMSTAGE_BPNAME(0)) + 1])!=NULL ){
					sprintf(mpNextPoint,WK7STAGE_GMSTAGE_BPNAME_BASE "%d",mParam % 10);
				}
			}
			break;
		default:
			if( strstr(mpNextPoint,WK7STAGE_GMSTAGE_BPNAME_BASE)==mpNextPoint ){
				const char	*pParam = mpNextPoint + strlen(WK7STAGE_GMSTAGE_BPNAME_BASE);
				if( *pParam >= '1' && *pParam <= '9' ){
					mParam = (u32)(*pParam - '0');
				}
			}
			break;
		}
		break;
	}
	
	mStageJump = 1;
//	mEnable    = 0;
	mEnableEx  = 0;
	
	return true;
}

bool WK7PathNurbs::ParseDoorEvent(u32 stgid)
{
#pragma	unused(stgid)
	
	const char	*pPathName = GetName();
	char		*pThisPointName = NULL;
	
	if( (pPathName=GetPartialStr(pThisPointName,pPathName,'_'))==NULL ){
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	pPathName++;
	
	// ÛPUi
	char		*pEventNo = NULL;
	int			no;
	
	if( (pPathName=GetPartialStr(pEventNo,pPathName,'_'))==NULL || pEventNo==NULL ){
		GHM_SAFE_DELETE_ARRAY(pThisPointName);
		GHM_SAFE_DELETE_ARRAY(pEventNo);
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	
	switch( sscanf(pEventNo,"%x",&no) ){
	case	EOF:
	case	0:
		GHM_SAFE_DELETE_ARRAY(pThisPointName);
		GHM_SAFE_DELETE_ARRAY(pEventNo);
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	default:
		mEventNo = (u32)(no | K7EV_TYPE_DOOR);
		break;
	}
	
	u32			len;
	len = strlen(pThisPointName) + strlen(pEventNo) + 2;
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
	ghmGcMemSetAllocName("PATHNPD0");
	if( (mpNextPoint = new char[len])==NULL ){
		GHM_SAFE_DELETE_ARRAY(pThisPointName);
		GHM_SAFE_DELETE_ARRAY(pEventNo);
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	
	strcpy(mpNextPoint,pThisPointName);
	strcat(mpNextPoint,"_");
	strcat(mpNextPoint,pEventNo);
	GHM_SAFE_DELETE_ARRAY(pThisPointName);
	GHM_SAFE_DELETE_ARRAY(pEventNo);
	
	pPathName++;
	
	if( *pPathName < '0' || *pPathName > '9' ){
		GHM_SAFE_DELETE_ARRAY(mpSelectObjName);
		if( (pPathName=GetPartialStr(mpSelectObjName,pPathName,'_'))==NULL ){
			return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
		}
		pPathName++;
	}
	
	if( (pPathName = WK7Stage::GetStageID(pPathName,mNextStageID))==NULL ){
		return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
	}
	switch( mNextStageID ){
	case	WK7STAGE_TVSTAGEID:
	case	WK7STAGE_GMSTAGEID:
		if( pPathName[0] >= '1' && pPathName[0] <= '9' ){
			mParam = (u32)(pPathName[0] - '0');
			pPathName++;
			if( mNextStageID==WK7STAGE_TVSTAGEID ){
				if( pPathName[0]==WK7STAGE_TVSTAGE_SAVELETTER ){
					mParam |= WK7STAGE_TVSTAGE_SAVEPARAM;
					pPathName++;
				}
			}
		}
		break;
	default:
		break;
	}
	
	switch( *pPathName ){
	case	'\0':
	case	'_':
		break;
	default:
		GHM_SAFE_DELETE(mpNextPoint);
		if( (pPathName=GetPartialStr(mpNextPoint,pPathName,'_'))!=NULL ){
			pPathName++;
			if( strcmp(mpNextPoint,WK7PATH_DOOR_PREFIX)==0 ){
				const char	*pPrevNextPoint = mpNextPoint;
				char		*pEvNo;
				if( (pPathName=GetPartialStr(pEvNo,pPathName,'_'))!=NULL ){
					pPathName++;
				}
				u32		len = strlen(pEvNo) + strlen(pPrevNextPoint) + 2;
				ghmGcMemSetAllocName("PATHNPD1");
				if( (mpNextPoint = new char[len])==NULL ){
					GHM_SAFE_DELETE_ARRAY(pEvNo         );
					return WK7STAGEBPARRIVEEVENT_PARSE_ERROR;
				}
				strcpy(mpNextPoint,pPrevNextPoint);
				strcat(mpNextPoint,"_");
				strcat(mpNextPoint,pEvNo);
				GHM_SAFE_DELETE_ARRAY(pEvNo         );
				GHM_SAFE_DELETE_ARRAY(pPrevNextPoint);
			}else{
				break;
			}
		}
		break;
	}
	
	WK7EventIDTable			*pEventIDTable = WK7Event::FindIDTable(mEventNo);
	if( pEventIDTable!=NULL ){
		mEventID = pEventIDTable->mEventID;
	}
	
	switch( mNextStageID ){
	case	WK7STAGE_TVSTAGEID:
		GHM_SAFE_DELETE_ARRAY(mpNextPoint);
		ghmGcMemSetAllocName("PATHNPD2");
		if( (mpNextPoint = new char[strlen(WK7STAGE_TVSTAGE_BPNAME) + 1])!=NULL ){
			strcpy(mpNextPoint,WK7STAGE_TVSTAGE_BPNAME);
		}
		break;
	case	WK7STAGE_GMSTAGEID:
		switch( mParam ){
		case	1:
		case	2:
		case	3:
			GHM_SAFE_DELETE_ARRAY(mpNextPoint);
			ghmGcMemSetAllocName("PATHNPD3");
			K7Stage578M00_PointInfo		*pPointInfo = K7Stage578M00::GetKihanShortCut(GameInfo.GetScenarioFlow(),(int)mParam);
			if( pPointInfo!=NULL ){
				if( (mpNextPoint = new char[strlen(pPointInfo->mpBpName) + 1])!=NULL ){
					strcpy(mpNextPoint,pPointInfo->mpBpName);
					mNextStageID = pPointInfo->mStageID;
				}
			}else{
				if( (mpNextPoint = new char[strlen(WK7STAGE_GMSTAGE_BPNAME(0)) + 1])!=NULL ){
					sprintf(mpNextPoint,WK7STAGE_GMSTAGE_BPNAME_BASE "%d",mParam % 10);
				}
			}
			break;
		default:
			if( strstr(mpNextPoint,WK7STAGE_GMSTAGE_BPNAME_BASE)==mpNextPoint ){
				const char	*pParam = mpNextPoint + strlen(WK7STAGE_GMSTAGE_BPNAME_BASE);
				if( *pParam >= '1' && *pParam <= '9' ){
					mParam = (u32)(*pParam - '0');
				}
			}
			break;
		}
		break;
	default:
		break;
	}
	
	mStageJump = 1;
//	mEnable    = 0;
	mEnableEx  = 0;
	
	return true;
}

bool WK7PathNurbs::SetEvent(u32 stgid)
{
	const char	*pPathName;
	
	GHM_SAFE_DELETE_ARRAY(mpNextPoint);
	GHM_SAFE_DELETE_ARRAY(mpSelectObjName);
	mNextStageID    = 0xffffffff;
	mEventNo        = 0xffffffff;
	mEventID        = 0xffffffff;
	mParam          = 0;
	
	if( (pPathName=GetName())==NULL ){
		return false;
	}
	
	if( strstr(pPathName,WK7PATH_BRANCH_PREFIX)==pPathName ){
		return ParseBranchEvent(stgid);
	}else if( strstr(pPathName,WK7PATH_DOOR_PREFIX)==pPathName ){
		return ParseDoorEvent(stgid);
	}
	
	return false;
}

#ifdef	_DEBUG
#define	WK7PATH_DIV_NUM			(32)
bool WK7PathNurbs::CreateRenderPosition(void)
{
	if( ReleaseRenderPosition()==false || mpPathSet==NULL ){
		return false;
	}
	ghmGcMemSetAllocName("Vec[]");
	mpRenderPosition = new Vec[WK7PATH_DIV_NUM];
	if( mpRenderPosition==NULL ){
		return false;
	}
	f32		startparam = mNurbsCurve.GetStartParam();
	f32		deltaparam = mNurbsCurve.GetEndParam  () - startparam;
	f32		param;
	int		i;
	Vec		*pRenderPosition = mpRenderPosition;
	for(i=0 ; i<WK7PATH_DIV_NUM ; i++,pRenderPosition++){
		param = startparam + i * deltaparam / (WK7PATH_DIV_NUM-1);
		mNurbsCurve.GetPoint((*pRenderPosition),param);
	}
	mRenderPositionNum = WK7PATH_DIV_NUM;
	return true;
}

bool WK7PathNurbs::ReleaseRenderPosition(void)
{
	GHM_SAFE_DELETE(mpRenderPosition);
	mRenderPositionNum = 0;
	return true;
}

bool WK7PathNurbs::RenderPosition(void)
{
	if( mpRenderPosition==NULL ){
		return false;
	}
	
	Mtx			view;
	HSD_CObj	*pCObj;
	u32			color;
	
	if( mEnable ){
		if( mStageJump ){
			color = 0x008000ff;
		}else{
			color = 0x000080ff;
		}
	}else{
		color = 0x800000ff;
	}
	if( mEnableEx ){
		if( mStageJump ){
			color |= 0x007f0000;
		}else{
			color |= 0x00007f00;
		}
	}else{
		color |= 0x7f0000ff;
	}
	
	if( (pCObj = HSD_CObjGetCurrent())!=NULL ){
		HSD_CObjGetViewingMtx(pCObj,view);
	}else{
		MTXIdentity(view);
	}
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_NONE   );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT0,GX_DF_NONE,GX_AF_NONE);
	
	GXSetNumTexGens  (0);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	
	GXSetCullMode    (GX_CULL_NONE);
	
	GXBegin(GX_LINESTRIP,GX_VTXFMT0,(u16)mRenderPositionNum);
		
		Vec		*pRenderPosition = mpRenderPosition;
		int		i;
		
		for(i=0 ; i<mRenderPositionNum ; i++,pRenderPosition++){
			GXPosition3f32(pRenderPosition->x,pRenderPosition->y + 1.0f,pRenderPosition->z);
			GXColor1u32   (color);
		}
		
	GXEnd();
	
	return true;
}
#endif

bool WK7PathNurbs::RemoveStartBranchPoint(void)
{
	if( mpStartBranchPoint==NULL ){
		return false;
	}
	mpStartBranchPoint->RemoveBranch(mpStartBranchPoint->GetBranch(this));
	mpStartBranchPoint = NULL;
	return true;
}

bool WK7PathNurbs::RemoveEndBranchPoint(void)
{
	if( mpEndBranchPoint==NULL ){
		return false;
	}
	mpEndBranchPoint->RemoveBranch(mpEndBranchPoint->GetBranch(this));
	mpEndBranchPoint = NULL;
	return true;
}

bool WK7PathNurbs::SetStartBranchPoint(WK7PathBranchPoint *pBranchPoint)
{
	if( pBranchPoint==NULL ){
		return false;
	}
	mpStartBranchPoint = pBranchPoint;
	mpStartBranchPoint->AddBranch(this,0);
	return true;
}

bool WK7PathNurbs::SetEndBranchPoint(WK7PathBranchPoint *pBranchPoint)
{
	if( pBranchPoint==NULL ){
		return false;
	}
	mpEndBranchPoint = pBranchPoint;
	mpEndBranchPoint->AddBranch(this,1);
	return true;
}

#define	DELTA_FINE	(16)
bool WK7PathNurbs::SetStartBranchLimitDist(f32 dist)
{
	f32			param = mNurbsCurve.GetStartParam();
	f32			end   = mNurbsCurve.GetEndParam();
	f32			delta = (end - param) * dist / mNurbsCurve.GetTotalDistance();
	Vec			diff;
	
	for( ; param<end && dist>0.0f ; param+=delta){
		mNurbsCurve.GetDiffVec(diff,param);
		dist -= GHM_VEC_GET_LEN(&diff) * delta;
	}
	
	return SetStartBranchLimitParam(param);
}

bool WK7PathNurbs::SetStartBranchLimitParam(f32 param)
{
	if( param < mNurbsCurve.GetStartParam() ){
		param = mNurbsCurve.GetStartParam();
	}else if( param > mNurbsCurve.GetEndParam() ){
		param = mNurbsCurve.GetEndParam();
	}
	mStartBranchLimitParam = param;
	return true;
}

bool WK7PathNurbs::SetEndBranchLimitDist(f32 dist)
{
	f32			start = mNurbsCurve.GetStartParam();
	f32			param = mNurbsCurve.GetEndParam();
	f32			delta = (param - start) * dist / mNurbsCurve.GetTotalDistance();
	Vec			diff;
	
	for( ; param>start && dist>0.0f ; param-=delta){
		mNurbsCurve.GetDiffVec(diff,param);
		dist -= GHM_VEC_GET_LEN(&diff) * delta;
	}
	
	return SetEndBranchLimitParam(param);
}

bool WK7PathNurbs::SetEndBranchLimitParam(f32 param)
{
	if( param < mNurbsCurve.GetStartParam() ){
		param = mNurbsCurve.GetStartParam();
	}else if( param > mNurbsCurve.GetEndParam() ){
		param = mNurbsCurve.GetEndParam();
	}
	mEndBranchLimitParam = param;
	return true;
}

const char* WK7PathNurbs::GetName(void)
{
	if( mpPathSet==NULL ){
		return NULL;
	}
	return mpPathSet->mpName;
}

WK7PathBranchList::WK7PathBranchList(WK7PathBranch *pBranch,WK7PathBranchList *pParent)
{
	WK7PathNurbs		*pPath;
	mpBranch  = pBranch;
	mpParent  = pParent;
	mpChild   = NULL;
	if( pBranch!=NULL ){
		mpPath = pBranch->GetPath();
		mDir   = pBranch->GetDir ();
	}else{
		mpPath = NULL;
		mDir   = 0;
	}
	if( pBranch!=NULL && (pPath=pBranch->GetPath())!=NULL ){
		mDistance = pPath->GetTotalDistance();
	}else{
		mDistance = 0.0f;
	}
}

WK7PathBranchList::~WK7PathBranchList()
{
	if( mpParent!=NULL && mpParent->GetChild()==this ){
		mpParent->SetChild(NULL);
	}
	GHM_SAFE_DELETE(mpChild);
}

bool WK7PathBranchList::SetPath(WK7PathNurbs *pPath)
{
	if( mpBranch!=NULL ){
		return false;
	}
	mpPath = pPath;
	return true;
}

bool WK7PathBranchList::SetDir(u32 dir)
{
	if( mpBranch!=NULL ){
		return false;
	}
	mDir = dir;
	return true;
}

bool WK7PathBranchList::SetDistance(f32 dist)
{
//	if( mpBranch!=NULL ){
//		return false;
//	}
	mDistance = dist;
	return true;
}

void WK7PathBranchList::SetChild(WK7PathBranchList *pChild)
{
#ifdef	_DEBUG
	if( mpChild!=NULL && pChild!=NULL ){
		OSReport("PathBranchList Invalid SetChild !! (old:%s,new:%s)\n",mpChild->mpPath->GetName(),pChild->mpPath->GetName());
	}
#endif
	if( (mpChild = pChild)!=NULL ){
		mDistance += mpChild->GetDistance();
	}
}

void WK7PathBranchList::SetParent(WK7PathBranchList *pParent)
{
	if( mpParent!=NULL ){
		if( mpParent->GetChild()==this ){
			mpParent->SetChild(NULL);
		}
	}
	if( (mpParent = pParent)!=NULL ){
		mpParent->SetChild(this);
	}
}

WK7PathBranchPoint* WK7PathBranch::GetEndBranchPoint(void)
{
	if( mpPath==NULL ){
		return NULL;
	}
	return mDir? mpPath->GetStartBranchPoint() : mpPath->GetEndBranchPoint();
}

WK7PathBranchPoint* WK7PathBranch::GetStartBranchPoint(void)
{
	if( mpPath==NULL ){
		return NULL;
	}
	return mDir? mpPath->GetEndBranchPoint() : mpPath->GetStartBranchPoint();
}

#if 0
f32 WK7PathBranch::GetPathDistance(WK7PathNurbs *pSearchPath,WK7PathBranchPointList *pList)
{
	if( pSearchPath==mpPath ){
		return 0.0f;
	}
	
	WK7PathBranchPoint		*pNextBP;
	WK7PathBranchPointList	*pBpList;
	
	pNextBP = GetNextBranchPoint();
	
	for(pBpList=pList ; pBpList!=NULL ; pBpList=pBpList->GetNext()){
		if( pBpList->GetBranchPoint()==pNextBP ){
			return -1.0f;
		}
	}
	
	f32		dist;
	
	ghmGcMemSetAllocName("PBPL1");
	pBpList = new WK7PathBranchPointList(pNextBP,pList);
	dist    = pNextBP->GetPathDistance(pSearchPath,pBpList);
	GHM_SAFE_DELETE(pBpList);
	
	if( dist < 0.0f ){
		return -1.0f;
	}
	
	return dist + mpPath->GetTotalDistance();
}
#endif

bool WK7PathBranchList::IsIncluded(WK7PathBranch *pBranch,int flag)
{
	if( pBranch==NULL ){
		return false;
	}
	if( mpPath==pBranch->GetPath() ){
		return true;
	}
//	if( mpBranch==NULL ){
//		if( mpPath==pBranch->GetPath() &&
//		    mDir  ==pBranch->GetDir () ){
//			return true;
//		}
//	}else{
//		if( mpBranch==pBranch ){
//			return true;
//		}
//	}
	if( (flag & 1) && mpParent!=NULL ){
		if( mpParent->IsIncluded(pBranch,1)==true ){
			return true;
		}
	}
	if( (flag & 2) && mpChild!=NULL ){
		if( mpChild->IsIncluded(pBranch,2)==true ){
			return true;
		}
	}
	return false;
}

bool WK7PathBranch::SearchPath(WK7PathNurbs *pSearchPath,f32 param,WK7PathNurbs *pRootPath,int level)
{
	if( mpPath==pRootPath ){
		return true;
	}
	if( level > 0 ){
		level--;
		WK7PathNurbs		*pCurrSearchPath;
		pCurrSearchPath = mpPath->GetSearchPath();
		if( pCurrSearchPath!=pRootPath ){
			return true;
		}
		WK7PathBranchPoint	*pNextBP = GetEndBranchPoint();
		if( pNextBP==NULL ){
			return true;
		}
		return pNextBP->SearchPath(pSearchPath,param,mpPath,level);
	}
#ifdef	_DEBUG
	if( level < 0 ){
		OSReport("SearchPath level < 0 !!\n");
		return true;
	}
#endif
	
	if( mpPath->GetSearchPath()==pRootPath ){
		return true;
	}
	
	f32		dist;
	bool	result;
	
	if( mpPath==pSearchPath ){
		dist = mpPath->GetParamDistance(param);
		if( mDir ){
			dist = mpPath->GetTotalDistance() - dist;
		}
		dist += pRootPath->GetSearchDist();
		result = true;
	}else{
		dist   = mpPath->GetTotalDistance() + pRootPath->GetSearchDist();
		result = false;
	}
	
	if( mpPath->GetSearchPath()==NULL || mpPath->GetSearchDist() > dist ){
		mpPath->SetSearchPath(pRootPath);
		mpPath->SetSearchDist(dist);
		mpPath->SetSearchLevel(WK7PathNurbs::GetCurrentLevel());
		mpPath->SetSearchBranch(this);
	}
	
	return result;
}

bool WK7PathBranchPoint::SearchPath(WK7PathNurbs *pSearchPath,f32 param,WK7PathNurbs *pRootPath,int level)
{
	WK7PathBranch		*pBranch;
	bool				result = true;
	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
		if( pBranch->SearchPath(pSearchPath,param,pRootPath,level)==false ){
			result = false;
		}
	}
	return result;
}

//WK7PathBranchList* WK7PathBranch::GetBranchList(WK7PathNurbs *pSearchPath,WK7PathBranchList *pParent)
//{
//	
//	if( pParent!=NULL && pParent->IsIncluded(this)==true ){
//		return NULL;
//	}
//	
//	WK7PathBranchList		*pBranchList;
//	
//	ghmGcMemSetAllocName("PBPL2");
//	pBranchList = new WK7PathBranchList(this,pParent);
//#ifdef _DEBUG
//	if( pBranchList==NULL ){
//		OSDebugReport("BranchList Creation Failed !!\n");
//		return NULL;
//	}
//#endif
//	
//	if( pSearchPath==mpPath ){
//		pBranchList->SetDistance(0.0f);
//		return pBranchList;
//	}
//	
//	WK7PathBranchPoint		*pNextBP;
//	
//	if( (pNextBP = GetNextBranchPoint())==NULL ){
//		delete pBranchList;
//		return NULL;
//	}
//	
//	WK7PathBranchList		*pChild;
//	
//	if( (pChild=pNextBP->GetBranchList(pSearchPath,pBranchList))==NULL ){
//		delete pBranchList;
//		return NULL;
//	}
//	
//	pBranchList->SetChild(pChild);
//	
//	return pBranchList;
//}

//#if 0
//f32 WK7PathBranchPoint::GetPathDistance(WK7PathNurbs *pSearchPath,WK7PathBranchPointList *pList)
//{
//	WK7PathBranch		*pBranch;
//	f32					searchDist = -1.0f;
//	
//	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
//		f32		dist;
//		dist = pBranch->GetPathDistance(pSearchPath,pList);
//		if( searchDist < 0.0f || searchDist > dist ){
//			searchDist = dist;
//		}
//	}
//	
//	return searchDist;
//}
//
//WK7PathBranch* WK7PathBranchPoint::GetBranchSearchPath(WK7PathNurbs *pSearchPath,f32 *pDist)
//{
//	WK7PathBranch		*pBranch;
//	WK7PathBranch		*pSearchBranch = NULL;
//	f32					searchDist = 1.0f;
//	
//	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
//		f32		dist;
//		dist = pBranch->GetPathDistance(pSearchPath,NULL);
//		if( searchDist < 0.0f || searchDist > dist ){
//			searchDist = dist;
//			pSearchBranch = pBranch;
//		}
//	}
//	if( pSearchBranch!=NULL && pDist!=NULL ){
//		*pDist = searchDist;
//	}
//	return pSearchBranch;
//}
//#endif

//WK7PathBranchList* WK7PathBranchPoint::GetBranchList(WK7PathNurbs *pSearchPath,WK7PathBranchList *pParent)
//{
//	WK7PathBranch		*pBranch;
//	WK7PathBranchList	*pSearchList = NULL;
//	
//#if 0
//	OSReport("BP:%s ",GetName());
//	if( pParent!=NULL ){
//		WK7PathBranchList	*pList = pParent;
//		while( pList!=NULL ){
//			OSReport("->%s",pList->GetPath()->GetName());
//			pList = pList->GetParent();
//		}
//		
//	}
//	OSReport("\n");
//#endif
//	
//	for(pBranch=mpBranch ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
//		WK7PathBranchList	*pList;
//		if( (pList=pBranch->GetBranchList(pSearchPath,pParent))==NULL ){
//			continue;
//		}
//		int		select = -1;
//		if( pSearchList==NULL ){
//			select = 0;
//			pSearchList = pList;
//		}else if( pSearchList->GetDistance() > pList->GetDistance() ){
//			select = 1;
//			delete pSearchList;
//			pSearchList = pList;
//		}else{
//			select = 2;
//			delete pList;
//		}
//	}
//	return pSearchList;
//}

//#if 0
//WK7PathBranchPoint* WK7PathNurbs::GetBranchPoint(WK7PathNurbs *pSearchPath,f32 *pDist)
//{
//	WK7PathBranchPoint	*pBranchPoint = NULL;
//	f32					bpdist,dist;
//	
//	if( mpStartBranchPoint != NULL ){
//		if( (bpdist=mpStartBranchPoint->GetPathDistance(pSearchPath)) >= 0.0f ){
//			pBranchPoint = mpStartBranchPoint;
//		}
//	}
//	
//	if( mpEndBranchPoint   != NULL ){
//		if( (dist=mpEndBranchPoint->GetPathDistance(pSearchPath)) >= 0.0f ){
//			if( pBranchPoint==NULL || dist < bpdist ){
//				pBranchPoint = mpStartBranchPoint;
//				bpdist       = dist;
//			}
//		}
//	}
//	
//	if( pBranchPoint!=NULL && pDist!=NULL ){
//		*pDist = bpdist;
//	}
//	return pBranchPoint;
//}
//#endif

WK7PathBranchList* WK7PathBranchList::GetLast(void)
{
	if( mpChild!=NULL ){
		return mpChild->GetLast();
	}
	return this;
}

WK7PathBranchList* WK7PathInfo::GetBranchList(WK7PathInfo *pSearchPathInfo)
{
	if( mpPath==NULL || pSearchPathInfo==NULL ){
		return NULL;
	}
	WK7PathNurbs		*pSearchPath = pSearchPathInfo->GetPath();
	if( pSearchPath==NULL ){
		return NULL;
	}
	
//#if 0
//	WK7PathBranchList	*pBranchList = mpPath->GetBranchList(pSearchPath);
//	
//	if( pBranchList==NULL ){
//		return NULL;
//	}
//	
//	f32		sp = mpPath->GetStartParam();
//	f32		ep = mpPath->GetEndParam();
//	f32		dp = ep - sp;
//	
//	if( mpPath==pSearchPath ){
//		sp = mParam - pSearchPathInfo->GetParam();
//		if( sp > 0.0f ){
//			pBranchList->SetDir(1);
//		}else{
//			pBranchList->SetDir(0);
//		}
//	}else{
//		
//		WK7PathBranchList	*pLastBranchList = pBranchList->GetLast();
//		
//		OSDebugReportExp(pLastBranchList==NULL || pLastBranchList==pBranchList,"Invalid BranchList !!\n");
//		
//		if( pLastBranchList != NULL ){
//			f32		lsp = pSearchPath->GetStartParam();
//			f32		lep = pSearchPath->GetEndParam();
//			f32		ldp = lep - lsp;
//			if( pLastBranchList->GetDir() ){
//				lsp = lep - pSearchPathInfo->GetParam();
//			}else{
//				lsp = pSearchPathInfo->GetParam() - lsp;
//			}
//			ldp = pSearchPath->GetTotalDistance() / ldp;
//			if( lsp < 0.0f ){
//				lsp = -lsp;
//			}
//			lsp = lsp * ldp;
//			pLastBranchList->SetDistance(lsp);
//			
//			pBranchList->SetDistance(pBranchList->GetDistance() + lsp);
//		}
//		
//		if( pBranchList->GetDir() ){
//			sp = mParam - sp;
//		}else{
//			sp = ep - mParam;
//		}
//	}
//	
//	dp = mpPath->GetTotalDistance() / dp;
//	if( sp < 0.0f ){
//		sp = -sp;
//	}
//	sp = pBranchList->GetDistance() + sp * dp;
//	pBranchList->SetDistance(sp);
//#else
	WK7PathBranchList	*pBranchList = mpPath->GetBranchList(mParam,pSearchPath,pSearchPathInfo->GetParam());
//#endif
	
	return pBranchList;
}

WK7PathBranchList* WK7PathNurbs::GetBranchList(f32 srcparam,WK7PathNurbs *pSearchPath,f32 dstparam)
{
	if( pSearchPath==NULL ){
		return NULL;
	}
	
	ClearSearchPathInfoAll();
	
	SetSearchPath (this);
	SetSearchLevel(-1);
	
	f32		dist = GetParamDistance(srcparam);
	int		level;
	int		result = 2;
	
	mCurrentLevel = 0;
	for(level=0 ; result!=0 ; level++,mCurrentLevel=level){
		result = 2;
		if( mpStartBranchPoint != NULL ){
			SetSearchDist(dist);
			if( mpStartBranchPoint->SearchPath(pSearchPath,dstparam,this, level)==true ){
				result--;
			}
		}else{
			result--;
		}
		if( mpEndBranchPoint   != NULL ){
			SetSearchDist(GetTotalDistance() - dist);
			if( mpEndBranchPoint  ->SearchPath(pSearchPath,dstparam,this, level)==true ){
				result--;
			}
		}else{
			result--;
		}
	}
	
//#if 1
	WK7PathNurbs		*pPath;
	WK7PathBranch		*pBranch;
	WK7PathBranchList	*pBranchList,*pBranchListChild = NULL;
	
	for(pPath=pSearchPath ; pPath!=this ; pPath=pPath->GetSearchPath()){
		
		if( (pBranch = pPath->GetSearchBranch())==NULL ){
			// BȂ
			return NULL;
		}
		
		ghmGcMemSetAllocName("PBPL3");
		pBranchList = new WK7PathBranchList(pBranch,NULL);
		
#ifdef	_DEBUG
		if( pBranchList==NULL ){
			OSDebugReport("BranchList Creation Failed!!\n");
			GHM_SAFE_DELETE(pBranchListChild);
			continue;
		}
#endif
		
		if( pPath==pSearchPath ){
			dist = pBranch->GetStartParamDistance(dstparam);
			pBranchList->SetDistance(dist);
		}
		
		if( pBranchListChild!=NULL ){
			pBranchListChild->SetParent(pBranchList);
		}
//		pBranchList->SetChild(pBranchListChild);
		pBranchListChild = pBranchList;
		
	}
	
	WK7PathBranchList	*pTopList;
	f32					sp  = GetStartParam();
	f32					ep  = GetEndParam();
	u32					dir;
	
	if( pBranchListChild==NULL ){
		if( dstparam > srcparam ){
			dir = 0;
			dist = GetParamDistance(dstparam) - GetParamDistance(srcparam);
		}else{
			dir = 1;
			dist = GetParamDistance(srcparam) - GetParamDistance(dstparam);
		}
	}else{
		pBranch = pBranchListChild->GetBranch();
		WK7PathBranchPoint	*pBranchPoint = pBranch->GetStartBranchPoint();
		if( pBranchPoint==mpStartBranchPoint ){
			dir = 1;
			dist = GetParamDistance(srcparam);
		}else{
#ifdef	_DEBUG
			if( pBranchPoint!=mpEndBranchPoint ){
				OSDebugReport("BranchList Creation Failed!!\n");
				return NULL;
			}
#endif
			dir = 0;
			dist = GetTotalDistance() - GetParamDistance(srcparam);
		}
	}
	
	ghmGcMemSetAllocName("PBPL4");
	pTopList = new WK7PathBranchList(NULL,NULL);
#ifdef	_DEBUG
	if( pTopList==NULL ){
		OSDebugReport("BranchList Creation Failed!!\n");
		GHM_SAFE_DELETE(pBranchListChild);
		return NULL;
	}
#endif
	pTopList->SetPath    (this);
	pTopList->SetDistance(dist);
	pTopList->SetDir     (dir);
	if( pBranchListChild!=NULL ){
		pBranchListChild->SetParent(pTopList);
	}
//	pTopList->SetChild   (pBranchListChild);

	return pTopList;
//#else
//	
//	return NULL;
//#endif
}

//WK7PathBranchList* WK7PathNurbs::GetBranchList(WK7PathNurbs *pSearchPath)
//{
//	if( pSearchPath==NULL ){
//		return NULL;
//	}
//	
//	WK7PathBranchList	*pSearchList,*pList,*pTopList;
//	
//	pSearchList = NULL;
//	ghmGcMemSetAllocName("PBPL");
//	pTopList = new WK7PathBranchList(NULL,NULL);
//#ifdef	_DEBUG
//	if( pTopList==NULL ){
//		OSDebugReport("BranchList Creation Failed!!\n");
//		return NULL;
//	}
//#endif
//	pTopList->SetPath(this);
//	pTopList->SetDir (0);
//	pTopList->SetDistance(0.0f);
//	
//	if( this!=pSearchPath ){
//		
//		
//		if( mpStartBranchPoint != NULL ){
//#if 0
//			OSReport("StartBP %s\n",mpStartBranchPoint->GetName());
//#endif
//			if( (pList=mpStartBranchPoint->GetBranchList(pSearchPath,pTopList)) != NULL ){
//				pSearchList = pList;
//				pTopList->SetDir(1);
//			}
//		}
//		
//		if( mpEndBranchPoint   != NULL ){
//#if 0
//			OSReport("End  BP %s\n",mpEndBranchPoint->GetName());
//#endif
//			if( (pList=mpEndBranchPoint->GetBranchList(pSearchPath,pTopList)) != NULL ){
//				if( pSearchList==NULL ){
//					pSearchList = pList;
//					pTopList->SetDir(0);
//				}else if( pSearchList->GetDistance() > pList->GetDistance() ){
//					delete pSearchList;
//					pSearchList = pList;
//					pTopList->SetDir(0);
//				}else{
//					delete pList;
//				}
//			}
//		}
//		
//		pTopList->SetChild(pSearchList);
//		
//	}
//	
//	return pTopList;
//}

bool WK7PathInfo::GetPosition(Vec *pPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	return mpPath->GetPoint((*pPosition),mParam);
}

bool WK7PathInfo::GetDeltaPosition(Vec *pDeltaPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	if( mpPath->GetDiffVec((*pDeltaPosition),mParam)==false ){
		return false;
	}
	if( mDir ){
		// ]
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}
	return true;
}

bool WK7PathInfo::GetStartPosition(Vec *pPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	if( mDir ){
		return mpPath->GetEndPoint(*pPosition);
	}else{
		return mpPath->GetStartPoint(*pPosition);
	}
}

bool WK7PathInfo::GetStartDeltaPosition(Vec *pDeltaPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	if( mDir ){
		if( mpPath->GetEndDiffVec(*pDeltaPosition)==false ){
			return false;
		}
		// ]
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}else{
		if( mpPath->GetStartDiffVec(*pDeltaPosition)==false ){
			return false;
		}
	}
	return true;
}

bool WK7PathInfo::GetEndPosition(Vec *pPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	if( mDir ){
		return mpPath->GetStartPoint(*pPosition);
	}else{
		return mpPath->GetEndPoint(*pPosition);
	}
}

bool WK7PathInfo::GetEndDeltaPosition(Vec *pDeltaPosition)
{
	if( mpPath==NULL || mpPath->GetDesc()==NULL ){
		return false;
	}
	if( mDir ){
		if( mpPath->GetStartDiffVec(*pDeltaPosition)==false ){
			return false;
		}
		// ]
		pDeltaPosition->x = -pDeltaPosition->x;
		pDeltaPosition->y = -pDeltaPosition->y;
		pDeltaPosition->z = -pDeltaPosition->z;
	}else{
		if( mpPath->GetEndDiffVec(*pDeltaPosition)==false ){
			return false;
		}
	}
	return true;
}

WK7PathBranchPoint* WK7PathInfo::GetNextBranchPoint(void)
{
	if( mpPath==NULL ){
		return NULL;
	}
	if( mDir ){
		return mpPath->GetStartBranchPoint();
	}
	return mpPath->GetEndBranchPoint();
}

WK7PathBranchPoint* WK7PathInfo::GetPrevBranchPoint(void)
{
	if( mpPath==NULL ){
		return NULL;
	}
	if( !mDir ){
		return mpPath->GetStartBranchPoint();
	}
	return mpPath->GetEndBranchPoint();
}

WK7PathNurbs* WK7PathInfo::GetPath(f32 dist,int initdir,f32 *pParam,u32 *pDir,bool enableex)
{
	if( mpPath==NULL ){
		return NULL;
	}
	u32		dir;
	switch( initdir ){
	case	0:
	case	1:
		dir = (u32)initdir;
		break;
	case	-2:
		dir = (u32)(mDir ^ 1);
		break;
	case	-1:
	default:
		dir = (u32)mDir;
		break;
	}
	return mpPath->GetPath(dist,mParam,dir,pParam,pDir,enableex);
}

WK7PathNurbs* WK7PathNurbs::GetPath(f32 dist,f32 param,u32 dir,f32 *pParam,u32 *pDir,bool enableex)
{
	
	WK7PathNurbs	*pPath = this;
	f32				cdist;
	
	cdist = GetParamDistance(param);
	if( !dir ){
		cdist = GetTotalDistance() - cdist;
	}
	
	while( dist > cdist ){
		
		dist -= cdist;
		
		WK7PathBranchPoint	*pBP;
		WK7PathBranch		*pBranch;
		Vec					delta;
		
		if( dir ){
			pPath->GetStartDiffVec(delta);
			delta.x = -delta.x;
			delta.y = -delta.y;
			delta.z = -delta.z;
		}else{
			pPath->GetEndDiffVec(delta);
		}
		
		VECNormalize(&delta,&delta);
		
		if( (pBP = pPath->GetBranchPoint(!dir))==NULL ||
		    (pBranch=pBP->SelectNextDefaultBranch(&delta,pPath,enableex))==NULL ||
		    pBranch->GetPath()==NULL ){
			if( pPath==this ){
				if( enableex==true ){
					if( pPath->GetEnableEx()==false ){
						return NULL;
					}
					if( pPath->GetStageJump()==true ){
						return NULL;
					}
				}else{
					if( pPath->GetEnable()==false ){
						return NULL;
					}
				}
			}
			if( pParam!=NULL ){
				if( dir ){
					param = pPath->GetStartParam();
				}else{
					param = pPath->GetEndParam();
				}
				*pParam = param;
			}
			if( pDir!=NULL ){
				*pDir = dir;
			}
			return pPath;
		}
		
		pPath = pBranch->GetPath();
		dir   = pBranch->GetDir();
		cdist = pPath->GetTotalDistance();
		if( dir ){
			param = pPath->GetEndParam();
		}else{
			param = pPath->GetStartParam();
		}
		
	}
	
	if( pParam!=NULL ){
		f32		sdist;
		sdist = pPath->CalcDistance(param);
		if( dir ){
			sdist -= dist;
		}else{
			sdist += dist;
		}
		*pParam = pPath->CalcParam(sdist);
	}
	
	if( pDir!=NULL ){
		*pDir = dir;
	}
	
	return pPath;
}

f32 WK7PathNurbs::GetParamDistance(f32 param)
{
	f32		ep = mNurbsCurve.GetEndParam();
	f32		sp = mNurbsCurve.GetStartParam();
	
	if( param > ep ){
		param = ep;
	}else if( param < sp ){
		param = sp;
	}
	
	return CalcDistance(param);
}

f32 WK7PathNurbs::GetParam(f32 dist)
{
	return mNurbsCurve.CalcParam(dist);
}

bool WK7PathInfo::SetBranchPath(WK7PathBranch *pBranch)
{
	WK7PathNurbs		*pPath;
	if( pBranch==NULL || (pPath=pBranch->GetPath())==NULL ){
		return false;
	}
	u32		dir;
	SetPath(pPath);
	if( (dir=pBranch->GetDir())!=0 ){
		SetParam(pPath->GetEndParam());
	}else{
		SetParam(pPath->GetStartParam());
	}
	SetDir(dir);
	return true;
}

f32 WK7PathInfo::GetLimitParam(f32 param)
{
	if( param < mpPath->GetStartParam() ){
		param = mpPath->GetStartParam();
	}else if( param > mpPath->GetEndParam() ){
		param = mpPath->GetEndParam();
	}
	return param;
}

bool WK7PathInfo::SetPath(WK7PathNurbs *pPath,bool limit)
{
	mpPath = pPath;
	if( mpPath!=NULL && limit==true ){
		mParam = GetLimitParam(mParam);
	}
	GetPosition     (&mPosition     );
	GetDeltaPosition(&mDeltaPosition);
	return true;
}

bool WK7PathInfo::SetParam(f32 param,bool limit)
{
	if( mpPath!=NULL ){
		f32		lparam;
		lparam = GetLimitParam(param);
		if( lparam!=param && limit==false ){
			return false;
		}
		param = lparam;
	}
	mParam = param;
	GetPosition     (&mPosition     );
	GetDeltaPosition(&mDeltaPosition);
	return true;
}

bool WK7PathInfo::SetDir(u32 dir)
{
	dir = dir? 1UL : 0UL;
	if( mDir != dir ){
		mDeltaPosition.x = -mDeltaPosition.x;
		mDeltaPosition.y = -mDeltaPosition.y;
		mDeltaPosition.z = -mDeltaPosition.z;
		mDir  = dir;
	}
	return true;
}

f32 WK7PathInfo::Move(f32 dist)
{
	if( mpPath==NULL ){
		return 0.0f;
	}
	f32		paramdist = mpPath->CalcDistance(mParam);
	if( mDir ){
		// t
		if( paramdist > dist && mParam > mpPath->GetStartParam() ){
			mParam = mpPath->CalcParam(paramdist - dist);
		}else{
			mParam = mpPath->GetStartParam();
			dist   = paramdist;
//			dist  -= paramdist;
		}
	}else{
		// 
		if( paramdist + dist < mpPath->GetTotalDistance() && mParam < mpPath->GetEndParam() ){
			mParam = mpPath->CalcParam(paramdist + dist);
		}else{
			mParam = mpPath->GetEndParam();
			dist   = mpPath->GetTotalDistance() - paramdist;
//			dist  -= mpPath->GetTotalDistance() - paramdist;
		}
	}
	if( dist > 0.0f ){
		GetPosition     (&mPosition     );
		GetDeltaPosition(&mDeltaPosition);
	}
	return dist;
}

bool WK7PathInfo::CheckInterpolateDist(void)
{
	if( mpPath==NULL ){
		return false;
	}
	if( mDir ){
		if( mParam < mpPath->GetStartBranchLimitParam() ){
			return true;
		}
	}else{
		if( mParam > mpPath->GetEndBranchLimitParam() ){
			return true;
		}
	}
	return false;
}

f32 WK7PathInfo::GetRestLinearDistance(void)
{
	if( mpPath==NULL ){
		return 0.0f;
	}
	Vec		cpos,tpos;
	f32		dist;
	
	mpPath->GetPoint(cpos,mParam);
	if( mDir ){
		// t
		mpPath->GetStartPoint(tpos);
	}else{
		// 
		mpPath->GetEndPoint  (tpos);
	}
	
	dist = GHM_VEC_GET_DIST(&cpos,&tpos);
	
	return dist;
}

int WK7PathInfo::GetRelativePosition(WK7PathInfo *pFromPathInfo,WK7PathInfo *pToPathInfo)
{
	if( pFromPathInfo==NULL ){
		return -1;
	}
	WK7PathBranchPoint		*pNextBP = pFromPathInfo->GetNextBranchPoint();
	if( pNextBP==NULL ){
		return -2;
	}
	WK7PathNurbs			*pToPath   = pToPathInfo->GetPath();
	WK7PathNurbs			*pFromPath = pFromPathInfo->GetPath();
	WK7PathBranch			*pBranch   = pNextBP->GetBranch(pToPath);
	if( pBranch==NULL ){
		return -3;
	}
	u32						dir;
	if( (dir=pBranch->GetDir())!=pToPathInfo->GetDir() ){
		return -4;
	}
	if( pToPath==pFromPath ){
		// pX
		if( dir ){
			if( pFromPathInfo->GetParam() < pToPathInfo->GetParam() ){
				return -5;
			}
			if( mpPath!=pToPath ){
				return 0;
			}
			if( mParam > pFromPathInfo->GetParam() ){
				return 0;
			}
			if( mParam > pToPathInfo->GetParam() ){
				return 1;
			}
		}else{
			if( pFromPathInfo->GetParam() > pToPathInfo->GetParam() ){
				return -6;
			}
			if( mpPath!=pToPath ){
				return 0;
			}
			if( mParam < pFromPathInfo->GetParam() ){
				return 0;
			}
			if( mParam < pToPathInfo->GetParam() ){
				return 2;
			}
		}
	}else{
		if( mpPath==pToPath ){
			if( dir ){
				if( mParam > pToPathInfo->GetParam() ){
					return 3;
				}
			}else{
				if( mParam < pToPathInfo->GetParam() ){
					return 4;
				}
			}
		}else if( mpPath==pFromPath ){
			if( pFromPathInfo->GetDir() ){
				if( mParam < pFromPathInfo->GetParam() ){
					return 5;
				}
			}else{
				if( mParam > pFromPathInfo->GetParam() ){
					return 6;
				}
			}
		}
	}
	return 0;
}

f32 WK7PathInfo::GetDistance(void)
{
	if( mpPath==NULL ){
		return 0.0f;
	}
	f32		dist;
	if( mParam < mpPath->GetStartParam() ){
		dist = 0.0f;
	}else if( mParam > mpPath->GetEndParam() ){
		dist = mpPath->GetTotalDistance();
	}else{
		dist = mpPath->GetParamDistance(mParam);
	}
	if( mDir ){
		// t
		dist = mpPath->GetTotalDistance() - dist;
	}
	return dist;
}

f32 WK7PathInfo::GetRestDistance(void)
{
	if( mpPath==NULL ){
		return 0.0f;
	}
	f32		dist;
	if( mParam < mpPath->GetStartParam() ){
		dist = 0.0f;
	}else if( mParam > mpPath->GetEndParam() ){
		dist = mpPath->GetTotalDistance();
	}else{
		dist = mpPath->GetParamDistance(mParam);
	}
	if( !mDir ){
		// 
		dist = mpPath->GetTotalDistance() - dist;
	}
	return dist;
}

f32 WK7PathInfo::GetTotalDistance(void)
{
	if( mpPath==NULL ){
		return 0.0f;
	}
	return mpPath->GetTotalDistance();
}

bool WK7PathInfo::IsStairs(void)
{
	if( mpPath==NULL ){
		return false;
	}
	return mpPath->GetStairs();
}

int WK7PathInfo::GetStairsState(void)
{
	if( mpPath==NULL ){
		return -1;
	}
	if( mpPath->GetStairs()==false ){
		return WK7PATH_FLAT;
	}
	if( mpPath->GetUpStairs()==true ){
		if( mDir ){
			return WK7PATH_DOWNSTAIRS;
		}else{
			return WK7PATH_UPSTAIRS;
		}
	}else{
		if( mDir ){
			return WK7PATH_UPSTAIRS;
		}else{
			return WK7PATH_DOWNSTAIRS;
		}
	}
}

bool WK7PathInfo::IsCurb(void)
{
	if( mpPath==NULL ){
		return false;
	}
	return mpPath->GetCurb();
}

bool WK7PathInfo::Pass(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->Pass(dir);
}

bool WK7PathInfo::ClearPass(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->ClearPass(dir);
}

bool WK7PathInfo::HasPassed(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->HasPassed(dir);
}

bool WK7PathInfo::Select(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->Select(dir);
}

bool WK7PathInfo::ClearSelect(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->ClearSelect(dir);
}

bool WK7PathInfo::HasSelected(bool rev)
{
	if( mpPath==NULL ){
		return false;
	}
	int	dir = (int)(rev==true? !mDir : mDir);
	return mpPath->HasSelected(dir);
}

WK7Path::WK7Path(WK7Archive *pArchive,u32 stgid)
{
	InitMember();
	if( (mpArchive = pArchive)==NULL ){
		return;
	}
	ghmScenePathSet		**ppPathSet;
	WK7BranchPointDesc	**ppBranchPointDesc;
	ppPathSet         = (ghmScenePathSet    **)mpArchive->GetPublicAddress("scene_path");
	ppBranchPointDesc = (WK7BranchPointDesc **)mpArchive->GetPublicAddress("scene_bp"  );
	mppCameraPathDesc = (WK7CameraPathDesc  **)mpArchive->GetPublicAddress("scene_cp"  );
	ParseData(ppPathSet,ppBranchPointDesc,stgid);
	mpArchive->ReferenceArchive();
}

WK7Path::WK7Path()
{
	InitMember();
}

WK7Path::~WK7Path()
{
	FreeArchive();
	if( mpArchive!=NULL ){
		mpArchive->ReleaseReferenceArchive();
	}
}

void WK7Path::InitMember(void)
{
	mpContainer          = NULL;
	mpArchive            = NULL;
	mpPathTop            = NULL;
	mpBranchPointTop     = NULL;
	mPathNum             = 0;
	mBranchPointNum      = 0;
	mFlag                = 0;
}

int WK7Path::GetVersion(void)
{
	if( mpArchive==NULL ){
		return -1;
	}
	WK7SdExpInfo	*pSdExpInfo = mpArchive->GetSdExpInfo();
	if( pSdExpInfo==NULL ){
		return -1;
	}
	return pSdExpInfo->mVersion;
}

bool WK7Path::FreeArchive(void)
{
	WK7PathNurbs		*pPath = mpPathTop;
	WK7PathBranchPoint	*pBranchPoint = mpBranchPointTop;
	WK7PathNurbs		*pNextPath;
	WK7PathBranchPoint	*pNextBranchPoint;
	
	while( pPath!=NULL ){
		pNextPath = pPath->GetNext();
		delete pPath;
		pPath     = pNextPath;
	}
	while( pBranchPoint!=NULL ){
		pNextBranchPoint = pBranchPoint->GetNext();
		delete pBranchPoint;
		pBranchPoint     = pNextBranchPoint;
	}
	mpPathTop         = NULL;
	mpBranchPointTop  = NULL;
	mPathNum          = 0;
	mBranchPointNum   = 0;
	return true;
}

bool WK7Path::ClearFlagID(void)
{
	mPassPathFlagID     = 0;
	mSelectObjFlagID    = 0;
	mPassPathFlagIDMax  = 0;
	mSelectObjFlagIDMax = 0;
	return true;
}

//bool WK7Path::SetFlagID(int startPassPathFlagID,int endPassPathFlagID,int startSelectObjFlagID,int endSelectObjFlagID)
//{
//	mPassPathFlagID     = startPassPathFlagID;
//	mSelectObjFlagID    = startSelectObjFlagID;
//	mPassPathFlagIDMax  = endPassPathFlagID;
//	mSelectObjFlagIDMax = endSelectObjFlagID;
//	return true;
//}

bool WK7Path::InvalidFlagID(void)
{
	mPassPathFlagID     = -1;
	mSelectObjFlagID    = -1;
	mPassPathFlagIDMax  = -1;
	mSelectObjFlagIDMax = -1;
	return true;
}

bool WK7Path::ParseData(ghmScenePathSet **ppPathSet,WK7BranchPointDesc **ppBranchPointDesc,u32 stgid)
{
	WK7PathNurbs	*pPath;
	// pXݒ
	if( ppPathSet!=NULL ){
		while( (*ppPathSet)!=NULL ){
			AddPath((*ppPathSet),stgid);
			ppPathSet++;
		}
	}
	// _ݒ
	if( ppBranchPointDesc!=NULL ){
		while( (*ppBranchPointDesc)!=NULL ){
			WK7PathBranchPoint		*pBranchPoint = AddBranchPoint((*ppBranchPointDesc)->mpName);
			WK7BranchDesc			**ppBranchDesc;
			int						i,branchNum;
			
			branchNum    = (int)(*ppBranchPointDesc)->mBranchNum;
			ppBranchDesc =      (*ppBranchPointDesc)->mppBranch;
			for(i=0 ; i<branchNum && *ppBranchDesc!=NULL; i++,ppBranchDesc++){
				if( (*ppBranchDesc)->mpPathSet==NULL ){
					continue;
				}
				if( (pPath = GetPath((*ppBranchDesc)->mpPathSet->mpDesc))==NULL ){
					continue;
				}
				if( (*ppBranchDesc)->mDir ){
					pPath->SetEndBranchPoint  (pBranchPoint);
					pPath->GetEndPoint(pBranchPoint->mPosition);
				}else{
					pPath->SetStartBranchPoint(pBranchPoint);
					pPath->GetStartPoint(pBranchPoint->mPosition);
				}
			}
			
			ppBranchPointDesc++;
		}
	}
	SetPassFlag();
	return true;
}

WK7PathNurbs* WK7Path::SearchPath(const char *pName)
{
	WK7PathNurbs	*pPath;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( pPath->SearchName(pName)==true ){
			return pPath;
		}
	}
	return NULL;
}

bool WK7Path::SetPassFlag(void)
{
	WK7PathNurbs	*pPath;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		pPath->SetPassFlag();
	}
	return true;
}

WK7PathNurbs* WK7Path::AddPath(ghmScenePathSet *pPathSet,u32 stgid)
{
	if( pPathSet==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("Nurbs");
	WK7PathNurbs	*pPath = new WK7PathNurbs(this,pPathSet,NULL,mpPathTop,stgid);
	mpPathTop = pPath;
	mPathNum++;
	return pPath;
}

bool WK7Path::DeletePath(ghmNurbsPathDesc *pDesc)
{
	if( pDesc==NULL ){
		return false;
	}
	WK7PathNurbs	*pPath = GetPath(pDesc);
	if( pPath==NULL ){
		return false;
	}
	if( mpPathTop==pPath ){
		mpPathTop = pPath->GetNext();
	}
	delete pPath;
	mPathNum--;
	return true;
}

bool WK7Path::DeletePath(ghmScenePathSet *pPathSet)
{
	if( pPathSet==NULL ){
		return false;
	}
	WK7PathNurbs	*pPath = GetPath(pPathSet);
	if( pPath==NULL ){
		return false;
	}
	if( mpPathTop==pPath ){
		mpPathTop = pPath->GetNext();
	}
	delete pPath;
	mPathNum--;
	return true;
}

WK7PathNurbs* WK7Path::GetPath(ghmNurbsPathDesc *pDesc)
{
	if( pDesc==NULL ){
		return NULL;
	}
	WK7PathNurbs	*pPath;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( pPath->GetDesc()==pDesc ){
			return pPath;
		}
	}
	return NULL;
}

WK7PathNurbs* WK7Path::GetPath(ghmScenePathSet *pPathSet)
{
	if( pPathSet==NULL ){
		return NULL;
	}
	WK7PathNurbs	*pPath;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( pPath->GetPathSet()==pPathSet ){
			return pPath;
		}
	}
	return NULL;
}

WK7PathNurbs* WK7Path::GetPath(const char *pName)
{
	if( pName==NULL ){
		return NULL;
	}
	WK7PathNurbs		*pPath;
	ghmScenePathSet		*pPathSet;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( (pPathSet=pPath->GetPathSet())==NULL ){
			continue;
		}
		if( pPathSet->mpName==NULL ){
			continue;
		}
		if( strcmp(pPathSet->mpName,pName)==0 ){
			return pPath;
		}
	}
	return NULL;
}

WK7PathNurbs* WK7Path::GetPath_EventNo(u32 evno)
{
	if( evno==0xffffffff ){
		return NULL;
	}
	WK7PathNurbs		*pPath;
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( pPath->GetEventNo()==evno ){
			return pPath;
		}
	}
	return NULL;
}

bool WK7Path::FrameProcess(void)
{
	return true;
}

bool WK7Path::RenderPath(void)
{
#ifdef	_DEBUG
	WK7PathNurbs		*pPath;
	if( mainMenu.IsDispPathInfo()==false ){
		return true;
	}
	bool				result = true;
	GXSetLineWidth(12,GX_TO_ZERO);
	for(pPath=mpPathTop ; pPath!=NULL ; pPath=pPath->GetNext()){
		if( pPath->RenderPosition()==false ){
			result = false;
		}
	}
	HSD_StateInvalidate(HSD_STATE_ALL);
	return result;
#else
	return true;
#endif
}

WK7PathBranchPoint* WK7Path::AddBranchPoint(const char *pName)
{
	if( pName==NULL ){
		return NULL;
	}
	ghmGcMemSetAllocName("PBP");
	WK7PathBranchPoint	*pBranchPoint = new WK7PathBranchPoint(this,pName,NULL,mpBranchPointTop);
	mpBranchPointTop = pBranchPoint;
	mBranchPointNum++;
	return pBranchPoint;
}

bool WK7Path::DeleteBranchPoint(const char *pName)
{
	WK7PathBranchPoint	*pBranchPoint = GetBranchPoint(pName);
	if( pBranchPoint==NULL ){
		return false;
	}
	if( mpBranchPointTop==pBranchPoint ){
		mpBranchPointTop = pBranchPoint->GetNext();
	}
	delete pBranchPoint;
	mBranchPointNum--;
	return true;
}

WK7PathBranchPoint* WK7Path::GetBranchPoint(const char *pName)
{
	if( pName==NULL ){
		return NULL;
	}
	WK7PathBranchPoint	*pBranchPoint;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( strcmp(pBranchPoint->GetName(),pName)==0 ){
			return pBranchPoint;
		}
		ghmGcMemSetAllocName("BPEVENT0");
		WK7StageBpEvent		*pBpEvent = new WK7StageBpEvent(pBranchPoint->GetName());
		if( pBpEvent==NULL ){
			continue;
		}
		if( pBpEvent->GetShortBpName()!=NULL ){
			if( strcmp(pBpEvent->GetShortBpName(),pName)==0 ){
				GHM_SAFE_DELETE(pBpEvent);
				return pBranchPoint;
			}
		}
		GHM_SAFE_DELETE(pBpEvent);
	}
	return NULL;
}

WK7PathBranchPoint* WK7Path::GetBranchPoint_EventNo(u32 eventno,WK7PathBranchPoint *pBranchPoint)
{
	char	*pEventNo = WK7PathBranchPointEventNo::GetPathName(eventno);
	if( pEventNo==NULL ){
		return NULL;
	}
	if( pBranchPoint==NULL ){
		pBranchPoint = mpBranchPointTop;
	}else{
		pBranchPoint = pBranchPoint->GetNext();
	}
	for( ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( strstr(pBranchPoint->GetName(),pEventNo)!=NULL ){
			break;
		}
	}
	delete[] pEventNo;
	return pBranchPoint;
}

WK7PathBranchPointObjJObj* WK7Path::GetObj_EventNo(u32 eventno)
{
	WK7PathBranchPoint			*pBranchPoint;
	WK7PathBranchPointObjJObj	*pObj;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( (pObj=pBranchPoint->GetObj_EventNo(eventno))!=NULL ){
			return pObj;
		}
	}
	return NULL;
}

WK7CameraPathDesc* WK7Path::GetCameraPath(const char *pName)
{
	if( mppCameraPathDesc==NULL || pName==NULL ){
		return NULL;
	}
	WK7CameraPathDesc	**ppCameraPathDesc;
	for(ppCameraPathDesc=mppCameraPathDesc ; *ppCameraPathDesc!=NULL ; ppCameraPathDesc++){
		if( (*ppCameraPathDesc)->mpName==NULL ){
			continue;
		}
		if( strcmp((*ppCameraPathDesc)->mpName,pName)==0 ){
			return *ppCameraPathDesc;
		}
	}
	return NULL;
}

WK7CameraPathDesc* WK7Path::GetCameraPath(WK7PathNurbs *pPath)
{
	if( mpArchive==NULL || pPath==NULL || pPath->mpPathSet==NULL ){
		return NULL;
	}
	char		name[256];
	
	const char	*pName;
	
	if( (pName = GetTokenPointer(pPath->mpPathSet->mpName,'_'))==NULL ){
		return NULL;
	}
	pName++;
	GetPartialStr(name,sizeof(name),pName,'_');
	return GetCameraPath(name);
}

HSD_CameraDesc* WK7Path::GetPathCamera(WK7PathNurbs *pPath)
{
	if( mpArchive==NULL || pPath==NULL || pPath->mpPathSet==NULL ){
		return NULL;
	}
	char		name[256];
	int			i;
	u32			size;
	const char	*ptop;
	for(i=0 ; i<sizeof(name) ; i++){
		switch( pPath->mpPathSet->mpName[i] ){
		case	'\0':
			break;
		case	'_':
			i++;
			break;
		default:
			continue;
		}
		break;
	}
	ptop = pPath->mpPathSet->mpName + i;
	for(size=0 ; i<sizeof(name) ; i++,size++){
		switch( pPath->mpPathSet->mpName[i] ){
		case	'\0':
			break;
		case	'_':
			i++;
			break;
		default:
			continue;
		}
		break;
	}
	if( size > sizeof(name)-1 ){
		size = sizeof(name)-1;
	}
	memcpy(name,ptop,size);
	name[size] = '\0';
//	strncpy(name,pPath->mpPathSet->mpName+i,sizeof(name));
	return mpArchive->GetCameraDesc(name);
//	return mpArchive->GetCameraDescPartialCameraName(name);
//	return mpArchive->GetCameraDescPartialCameraName(pPath->mpPathSet->mpName);
}

HSD_CameraAnim* WK7Path::GetPathCameraAnim(WK7PathNurbs *pPath)
{
	if( mpArchive==NULL || pPath==NULL || pPath->mpPathSet==NULL ){
		return NULL;
	}
	char		name[256];
	int			i;
	for(i=0 ; i<sizeof(name) ; i++){
		switch( pPath->mpPathSet->mpName[i] ){
		case	'\0':
			break;
		case	'_':
			i++;
			break;
		default:
			continue;
		}
		break;
	}
	strncpy(name,pPath->mpPathSet->mpName+i,sizeof(name));
	return mpArchive->GetCameraAnim(name);
}

int WK7Path::SetEventEnable(u32 eventid,bool flag)
{
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		count += pBranchPoint->SetEventEnable(eventid,flag);
	}
	WK7PathNurbs		*pPathNurbs;
	for(pPathNurbs=mpPathTop ; pPathNurbs!=NULL ; pPathNurbs=pPathNurbs->GetNext()){
		count += pPathNurbs->SetEventEnable(eventid,flag);
	}
	return count;
}

int WK7Path::ChangeEventNo(u32 from_evno,u32 to_evno)
{
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		count += pBranchPoint->ChangeEventNo(from_evno,to_evno);
	}
	WK7PathNurbs		*pPathNurbs;
	for(pPathNurbs=mpPathTop ; pPathNurbs!=NULL ; pPathNurbs=pPathNurbs->GetNext()){
		count += pPathNurbs->ChangeEventNo(from_evno,to_evno);
	}
	return count;
}

int WK7Path::SetEventNoEnable(u32 eventno,bool flag)
{
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		count += pBranchPoint->SetEventNoEnable(eventno,flag);
	}
	WK7PathNurbs		*pPathNurbs;
	for(pPathNurbs=mpPathTop ; pPathNurbs!=NULL ; pPathNurbs=pPathNurbs->GetNext()){
		count += pPathNurbs->SetEventNoEnable(eventno,flag);
	}
	return count;
}

int WK7Path::SetObjVisible(u32 eventid,bool flag)
{
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		count += pBranchPoint->SetObjVisible(eventid,flag);
	}
	return count;
}

int WK7Path::SetObjVisibleEventNo(u32 eventno,bool flag)
{
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		count += pBranchPoint->SetObjVisibleEventNo(eventno,flag);
	}
	return count;
}

WK7JObj* WK7Path::GetObjEventNo(u32 eventno)
{
	WK7PathBranchPoint	*pBranchPoint;
	WK7JObj				*pJObj;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( (pJObj=pBranchPoint->GetObjEventNo(eventno))==NULL ){
			continue;
		}
		return pJObj;
	}
	return NULL;
}

int WK7Path::SetBpCamDisable(const char *pName,bool flag)
{
	if( pName==NULL ){
		return 0;
	}
	WK7PathBranchPoint	*pBranchPoint;
	int					count = 0;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( pBranchPoint->GetName()!=NULL ){
			if( strcmp(pBranchPoint->GetName(),pName)==0 ){
				pBranchPoint->SetBpCamDisable(flag);
				count++;
				continue;
			}
		}
		if( pBranchPoint->GetShortName()!=NULL ){
			if( strcmp(pBranchPoint->GetShortName(),pName)==0 ){
				pBranchPoint->SetBpCamDisable(flag);
				count++;
				continue;
			}
		}
	}
	return count;
}

void WK7Path::BranchEventTask(void)
{
	DoorEventTask();
}

void WK7Path::DoorEventTask(void)
{
	
	WK7PathNurbs				*pPath = (WK7PathNurbs *)K7EV_GETEVENTPARAM(1);
	WK7Stage					*pStage = pPath->GetStage();
	WK7DoorEventInfo			doorEventInfo;
	
	doorEventInfo.Initialize();
	if( pStage!=NULL ){
		doorEventInfo.mCurrStageID = pStage->GetID();
		WK7Stage::SetPrevStageID(doorEventInfo.mCurrStageID);
	}
	doorEventInfo.mNextStageID = pPath->GetNextStageID();
	doorEventInfo.SetNextBpName(pPath->GetNextPoint());
	
	WK7Stage::SetCurrentStageBpName(pPath->GetNextPoint());
	WK7Stage::SetPrevStagePathName(pPath->GetShortName());
	WK7Stage::SetPrevStageBpName(NULL);
	WK7Stage::SetCurrentStageParam(pPath->GetParam());
	
	WK7Stage::DoorEventTask(doorEventInfo);
}

bool WK7Path::IsValid(WK7PathBranchPointObj *pTgtBranchPointObj)
{
	WK7PathBranchPoint		*pBranchPoint;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( pBranchPoint->IsValid(pTgtBranchPointObj)==true ){
			return true;
		}
	}
	return false;
}

bool WK7Path::IsValid(WK7PathBranch *pTgtBranch)
{
	WK7PathBranchPoint		*pBranchPoint;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( pBranchPoint->IsValid(pTgtBranch)==true ){
			return true;
		}
	}
	return false;
}

bool WK7Path::IsValid(WK7PathBranchPoint *pTgtBranchPoint)
{
	WK7PathBranchPoint		*pBranchPoint;
	for(pBranchPoint=mpBranchPointTop ; pBranchPoint!=NULL ; pBranchPoint=pBranchPoint->GetNext()){
		if( pBranchPoint==pTgtBranchPoint ){
			return true;
		}
	}
	return false;
}

bool WK7Path::IsValid(WK7PathNurbs *pTgtPath)
{
	WK7PathNurbs		*pPathNurbs;
	for(pPathNurbs=mpPathTop ; pPathNurbs!=NULL ; pPathNurbs=pPathNurbs->GetNext()){
		if( pPathNurbs==pTgtPath ){
			return true;
		}
	}
	return false;
}

WK7PathControl::WK7PathControl()
{
	mpControlObj                 = NULL;
	mpControlPath                = NULL;
	mActiveCounter               = 0;
	mState                       = PC_IDLE;
	mInterpolateState            = PCI_IDLE;
	mReqMoveDistance             = 0.0f;
	mPrevMoveDistance            = 0.0f;
	mIpMoveDistance.SetValue(0.0f);
	mDefaultMoveDistance         = 1.0f;
	mBranchSelectAppearDistance  = 100.0f;
	mFlag                        = 0;
	mEnable                      = 1;
	mpCurrentCameraAnim          = NULL;
	mCurrentCameraAnimTotalFrame = 0.0f;
	mpNextBranch                 = NULL;
	mCurrentPosition.x           = 0.0f;
	mCurrentPosition.y           = 0.0f;
	mCurrentPosition.z           = 0.0f;
	mCurrentDir     .x           = 0.0f;
	mCurrentDir     .y           = 0.0f;
	mCurrentDir     .z           = 1.0f;
}

WK7PathControl::~WK7PathControl()
{
	ReleaseInterpolatePath();
}

bool WK7PathControl::SetNextBranch(WK7PathBranch *pBranch)
{
	mpNextBranch = pBranch;
	return true;
}

bool WK7PathControl::SetControlPath(WK7Path *pPath)
{
	if( (mpControlPath = pPath)==NULL ){
		SetCurrentPath((WK7PathNurbs *)NULL);
	}else{
		SetCurrentPath(pPath->GetPathTop());
	}
	return true;
}

bool WK7PathControl::SetCurrentPath(const char *pBpName,const char *pPathName,f32 dparam)
{
	if( mpControlPath==NULL ){
		return false;
	}
	WK7PathBranchPoint	*pBranchPoint;
	if( (pBranchPoint=mpControlPath->GetBranchPoint(pBpName))==NULL ){
		ghmGcMemSetAllocName("BPEVENT1");
		WK7StageBpEvent		*pBpEvent = new WK7StageBpEvent(pBpName);
		if( pBpEvent==NULL || pBpEvent->GetShortBpName()==NULL ){
			GHM_SAFE_DELETE(pBpEvent);
			return NULL;
		}
		if( (pBranchPoint=mpControlPath->GetBranchPoint(pBpEvent->GetShortBpName()))==NULL ){
			GHM_SAFE_DELETE(pBpEvent);
			return NULL;
		}
		GHM_SAFE_DELETE(pBpEvent);
	}
	WK7PathNurbs		*pPathNurbs;
	if( pPathName!=NULL ){
		pPathNurbs = mpControlPath->GetPath(pPathName);
	}else{
		WK7PathBranch	*pBranch;
		for(pBranch=pBranchPoint->GetTopBranch() ; pBranch!=NULL ; pBranch=pBranch->GetNext()){
			if( (pPathNurbs = pBranch->GetPath())==NULL ){
				continue;
			}
			if( mPathEnableEx ){
				if( pPathNurbs->GetEnableEx()==false ){
					continue;
				}
				if( pPathNurbs->GetStageJump()==false ){
					continue;
				}
			}else{
				if( pPathNurbs->GetEnable()==false ){
					continue;
				}
			}
			break;
		}
		if( pBranch==NULL ){
			return false;
		}
	}
	if( pPathNurbs==NULL ){
		return false;
	}
	WK7PathBranch		*pBranch;
	if( (pBranch=pBranchPoint->GetBranch(pPathNurbs))==NULL ){
		return false;
	}
	f32		param;
	u32		dir;
	if( dparam < 0.0f ){
		dparam = 0.0f;
	}
	if( (dir=pBranch->GetDir()) ){
		param = pPathNurbs->GetEndParam() - dparam;
	}else{
		param = pPathNurbs->GetStartParam() + dparam;
	}
	return SetCurrentPath(pPathNurbs,param,dir);
}

bool WK7PathControl::SetCurrentPathToBp(const char *pBpName,const char *pPathName,f32 dparam)
{
	if( mpControlPath==NULL ){
		return false;
	}
	WK7PathBranchPoint	*pBranchPoint;
	if( (pBranchPoint=mpControlPath->GetBranchPoint(pBpName))==NULL ){
		ghmGcMemSetAllocName("BPEVENT2");
		WK7StageBpEvent		*pBpEvent = new WK7StageBpEvent(pBpName);
		if( pBpEvent==NULL || pBpEvent->GetShortBpName()==NULL ){
			GHM_SAFE_DELETE(pBpEvent);
			return NULL;
		}
		if( (pBranchPoint=mpControlPath->GetBranchPoint(pBpEvent->GetShortBpName()))==NULL ){
			GHM_SAFE_DELETE(pBpEvent);
			return NULL;
		}
		GHM_SAFE_DELETE(pBpEvent);
	}
	WK7PathNurbs		*pPathNurbs;
	if( pPathName!=NULL ){
		pPathNurbs = mpControlPath->SearchPath(pPathName);
	}else{
		WK7PathBranch	*pBranch = pBranchPoint->GetTopBranch();
		if( pBranch==NULL ){
			return false;
		}
		pPathNurbs = pBranch->GetPath();
	}
	if( pPathNurbs==NULL ){
		return false;
	}
	WK7PathBranch		*pBranch;
	if( (pBranch=pBranchPoint->GetBranch(pPathNurbs))==NULL ){
		return false;
	}
	f32		param;
	u32		dir;
	if( dparam < 0.0f ){
		dparam = 0.0f;
	}
	if( (dir=pBranch->GetDir()) ){
		param = pPathNurbs->GetEndParam() - dparam;
	}else{
		param = pPathNurbs->GetStartParam() + dparam;
	}
	return SetCurrentPath(pPathNurbs,param,dir ^ 1);
}

bool WK7PathControl::SetCurrentPath(const char *pName,f32 param,u32 dir)
{
	if( mpControlPath==NULL ){
		return false;
	}
	WK7PathNurbs		*pPathNurbs;
	if( (pPathNurbs=mpControlPath->SearchPath(pName))==NULL ){
//	if( (pPathNurbs=mpControlPath->GetPath(pName))==NULL ){
		return false;
	}
	return SetCurrentPath(pPathNurbs,param,dir);
}

bool WK7PathControl::CameraRenew(void)
{
	mRenew   = 1;
	mCamJump = 1;
	return true;
}

bool WK7PathControl::CallPathChagneCallback(void)
{
	return CallPathChagneCallback(mCurrentPath.GetPath(),0);
}

bool WK7PathControl::CallPathChagneCallback(WK7PathNurbs *pPath,int inv)
{
	if( pPath==NULL ){
		return false;
	}
	WK7Stage	*pStage = pPath->GetStage();
	if( pStage==NULL ){
		return false;
	}
	WK7PathChangeInfo		info;
	info.Initialize();
	info.mpPathName         = pPath->GetName();
	info.SetPath(pPath);
	info.mInvPath           = inv;
	pStage->CallPathChangeCallBackTask(info);
	return true;
}

bool WK7PathControl::SetCurrentBranchPath(WK7PathBranch *pBranch)
{
	WK7PathNurbs		*pPath;
	int					inv;
	if( pBranch!=NULL ){
		if( pBranch->GetDir()==mCurrentPath.GetDir() ){
			inv = 0;
		}else{
			inv = 1;
		}
	}
	mCurrentPath.SetBranchPath(pBranch);
	if( pBranch==NULL ){
		return false;
	}
	pPath = pBranch->GetPath();
	if( mEventCall && GameInfo.GetMode()!=WK7GameInfo::BATTLE ){
		CallPathChagneCallback(pPath,inv);
	}
	return true;
}

bool WK7PathControl::SetCurrentPath(WK7PathNurbs *pPath,f32 param,u32 dir)
{
	int			inv;
	if( mCurrentPath.GetDir()==dir ){
		inv = 0;
	}else{
		inv = 1;
	}
	mCurrentPath.SetPath         (pPath);
	mCurrentPath.SetParam        (param);
	mCurrentPath.SetDir          (dir);
	mCurrentPath.GetPosition     (&mCurrentPosition);
	mCurrentPath.GetDeltaPosition(&mCurrentDir     );
	mRenew   = 1;
	mCamJump = 1;
	ReleaseInterpolatePath();
	if( mEventCall && GameInfo.GetMode()!=WK7GameInfo::BATTLE ){
		CallPathChagneCallback(pPath,inv);
	}
	if( !mPathEnableEx && pPath!=NULL ){
		mPassPathRequest = 1;
//		pPath->Pass((int)dir);
	}
	mSetCurrentPath = 1;
	mRenew = 1;
	if( mpControlObj!=NULL ){
		if( mpControlObj->RenewCallback(this,&mCurrentPath)==WK7PathControlObj::RESULT_OK ){
			mRenew = 0;
		}
	}else{
		mRenew = 0;
	}
	return true;
}

HSD_CameraDesc* WK7PathControl::GetCurrentPathCamera(void)
{
	if( mpControlPath==NULL ){
		return NULL;
	}
	WK7PathNurbs		*pPath;
	if( (pPath = mCurrentPath.GetPath())==NULL ){
		return NULL;
	}
	return mpControlPath->GetPathCamera(pPath);
}

WK7CameraPathDesc* WK7PathControl::GetCurrentCameraPath(void)
{
	if( mpControlPath==NULL ){
		return NULL;
	}
	WK7PathNurbs		*pPath;
	if( (pPath = mCurrentPath.GetPath())==NULL ){
		return NULL;
	}
	return mpControlPath->GetCameraPath(pPath);
}

bool WK7PathControl::InterpolateSkip(void)
{
	if( mInterpolateState != PCI_ACTIVE ){
		return false;
	}
	// ̃pXJgpX
	SetCurrentPath(mNextPath.GetPath(),mNextPath.GetParam(),mNextPath.GetDir());
	return true;
}

bool WK7PathControl::FrameProcess(void)
{
	if( mpControlPath==NULL || !mEnable ){
		return true;
	}
	
	if( mActiveCounter < 0xffffffff - WK7_TICK_PER_FRAME ){
		mActiveCounter += WK7_TICK_PER_FRAME;
	}else{
		mActiveCounter  = 0xffffffff;
	}
	
	WK7PathInfo		*pMovePathInfo;
	f32				restdist;
	
	switch( mInterpolateState ){
	case	PCI_ACTIVE:
		// ԃpXړ
		pMovePathInfo = &mInterpolatePath;
		if( mInterpolateStop ){
			restdist = 0.0f;
			break;
		}
		if( !mInterpolateOperate ){
			mReverseReq = 0;
			if( mIpMoveDistance.GetValue()==0.0f ){
//				mIpMoveDistance.SetValue(mPrevMoveDistance);
//				mIpMoveDistance.Play    (mDefaultMoveDistance,WK7_SEC_TO_TICK(1));
				mIpMoveDistance.SetValue(mDefaultMoveDistance);
				if( mDefaultMoveDistance <= 0.0f ){
					mIpMoveDistance.SetValue(1.0f);
				}
			}
			mIpMoveDistance.FrameProcess();
			restdist = mIpMoveDistance.GetValue();
			break;
		}
		restdist          = mReqMoveDistance;
		mPrevMoveDistance = mReqMoveDistance;
		mReqMoveDistance  = 0.0f;
		break;
	case	PCI_IDLE:
		// JgpXړ
		pMovePathInfo = &mCurrentPath;
		restdist = mReqMoveDistance;
		mPrevMoveDistance = mReqMoveDistance;
		mReqMoveDistance  = 0.0f;
		break;
	}
	
	if( pMovePathInfo==NULL || pMovePathInfo->GetPath()==NULL ){
		return false;
	}
	
	if( mReverseReq ){
		// ]
		mReverseReq = 0;
		pMovePathInfo->SetDir(pMovePathInfo->GetDir()? 0UL:1UL);
		pMovePathInfo->GetDeltaPosition(&mCurrentDir);
		pMovePathInfo->GetPosition(&mCurrentPosition);
		// R[obNĂяo
		if( mpControlObj!=NULL ){
			mpControlObj->ReverseCallback(this,pMovePathInfo);
		}
		SetActive(true);
	}
	
	if( mRenew && mpControlObj!=NULL ){
		switch( mInterpolateState ){
		case	PCI_IDLE:
			// JgpXړ
			pMovePathInfo = &mCurrentPath;
			break;
		case	PCI_ACTIVE:
			pMovePathInfo = &mInterpolatePath;
			break;
		default:
			pMovePathInfo = NULL;
			break;
		}
		if( pMovePathInfo != NULL ){
			if( mpControlObj->RenewCallback(this,pMovePathInfo)==WK7PathControlObj::RESULT_OK ){
				mRenew = 0;
			}
		}
	}
	
	if( mState==PC_PATHSELECT ){
		if( mpControlObj!=NULL ){
			switch( mpControlObj->PathSelectCallback(this,pMovePathInfo,mCurrentPath.GetNextBranchPoint()) ){
			case	WK7PathControlObj::RESULT_CANCEL:
				restdist = 0.0f;
				break;
			case	WK7PathControlObj::RESULT_OK:
			default:
				break;
			}
		}
	}
	
	while( restdist > 0.0f ){
		
		mSetCurrentPath = 0;
		
		switch( mState ){
		case	PC_IDLE:
			restdist = 0.0f;
			continue;
		case	PC_PATHSELECT:
		case	PC_ACTIVE:
			break;
		default:
			OSDebugReport("Illegal Path Control State : %x\n",mState);
			return false;
		}
		
		switch( mInterpolateState ){
		case	PCI_IDLE:
			// JgpXړ
			pMovePathInfo = &mCurrentPath;
			break;
		case	PCI_ACTIVE:
			// ԃpXړ
			pMovePathInfo = &mInterpolatePath;
			break;
		default:
			OSDebugReport("Illegal Path Control Interpolate State : %x\n",mInterpolateState);
			return false;
		}
		
		if( pMovePathInfo->GetPath()==NULL ){
			break;
		}
		
		f32		param;
		f32		dist = restdist;
		param = pMovePathInfo->GetParam();
		
		if( mpControlObj!=NULL ){
			switch( mpControlObj->PreMoveCallback(this,pMovePathInfo,dist) ){
			case	WK7PathControlObj::RESULT_OK:
				break;
			case	WK7PathControlObj::RESULT_CANCEL:
				restdist = 0.0f;
				continue;
			default:
				break;
			}
		}
		
		if( mInterpolateState==PCI_IDLE ){
			WK7PathNurbs	*pPathNurbs;
			if( !mPathEnableEx && (pPathNurbs=pMovePathInfo->GetPath())!=NULL && pPathNurbs->GetStageJump()==false ){
				pMovePathInfo->Pass();
			}
		}
		
		f32		movedist;
		pMovePathInfo->GetDeltaPosition(&mCurrentDir);
		restdist -= (movedist=pMovePathInfo->Move(restdist));
		pMovePathInfo->GetPosition(&mCurrentPosition);
		pMovePathInfo->GetDeltaPosition(&mCurrentDir);
			
		if( mpControlObj!=NULL ){
			switch( mpControlObj->PostMoveCallback(this,pMovePathInfo,restdist,movedist) ){
			case	WK7PathControlObj::RESULT_OK:
				break;
			case	WK7PathControlObj::RESULT_CANCEL:
				pMovePathInfo->SetParam(param);
				pMovePathInfo->GetPosition(&mCurrentPosition);
				pMovePathInfo->GetDeltaPosition(&mCurrentDir);
				restdist = 0.0f;
				continue;
			default:
				break;
			}
		}
			
		// pX؂ւ
		
		switch( mInterpolateState ){
		case	PCI_IDLE:
			if( mCurrentPath.CheckInterpolateDist()==true ){
				
				// ʏړ
				WK7PathBranchPoint		*pNextBranchPoint= mCurrentPath.GetNextBranchPoint();
				f32						bpdist = mCurrentPath.GetRestDist();
				
				if( pNextBranchPoint==NULL ){
					
					if( mCurrentPath.GetRestDist() > 0.0f ){
						continue;
					}
					
					// ̂܂܃pXړǁAɐi߂Ȃc苗O
					
					restdist = 0.0f;
					
					// [_Cxg邩H
					
					//CallPathTermEvent(mCurrentPath.GetPath());
					
					continue;
				}
				
				if( mInterpolateDisable && mCurrentPath.GetRestDist() > 0.0f ){
					continue;
				}
				
				if( mpControlObj!=NULL ){
					switch( mpControlObj->BranchPointArriveCallback(this,pMovePathInfo,pNextBranchPoint) ){
					case	WK7PathControlObj::RESULT_OK:
						break;
					default:
						restdist = 0.0f;
						if( pNextBranchPoint!=NULL ){
							CallArriveEvent(pNextBranchPoint,NULL);
						}
						continue;
					}
				}
				
				int				nextnum = (int)pNextBranchPoint->GetEnableBranchNum(mCurrentPath.GetPath(),mPathEnableEx? true : false);
				bool			interpolate;
				WK7PathBranch	*pNextBranch;
				WK7PathNurbs	*pPathNurbs;
				
				if( nextnum <= 0 ){
					
					if( mCurrentPath.GetRestDist() <= 0.0f ){
						CallArriveEvent(pNextBranchPoint,NULL);
					}
					
					// Ȃ
					restdist = 0.0f;
					continue;
					
				}else if( nextnum==1 ){
					
					// 򂪈{Ȃ
					
					if( mpNextBranch==NULL && mCurrentPath.GetRestDist() > 0.0f ){
						restdist = 0.0f;
						continue;
					}
					
					for(pNextBranch=pNextBranchPoint->GetTopBranch() ; pNextBranch!=NULL ; pNextBranch=pNextBranch->GetNext()){
						if( (pPathNurbs=pNextBranch->GetPath())==mCurrentPath.GetPath() ){
							continue;
						}
						if( mPathEnableEx ){
							if( pPathNurbs->GetEnableEx()==false ){
								continue;
							}
						}else{
							if( pPathNurbs->GetEnable()==false ){
								continue;
							}
						}
						if( pPathNurbs->GetEnableEvent()==false ){
							continue;
						}
						break;
					}
					WK7PathBranchPointObj	*pObj;
					int						ocnt;
					for(pObj=pNextBranchPoint->GetTopObj(),ocnt=0 ; pObj!=NULL ; pObj=pObj->GetNext()){
						if( pObj->GetEnable()==false ){
							continue;
						}
						ocnt++;
					}
					
					if( pNextBranch==NULL ){
						pPathNurbs = NULL;
					}
					
					if( !mPathEnableEx && ocnt > 0 ){
						interpolate = true;
					}else{
						if( CallArriveEvent(pNextBranchPoint,pPathNurbs)==0 ){
							restdist = 0.0f;
							continue;
						}
						interpolate = false;
					}
					
					if( pNextBranch==NULL ){
						restdist = 0.0f;
						continue;
					}
					
				}else{
					
					if( (pNextBranch = mpNextBranch)!=NULL ){
						pPathNurbs = pNextBranch->GetPath();
					}else{
						pPathNurbs = NULL;
					}
					
					if( CallArriveEvent(pNextBranchPoint,pPathNurbs)==0 ){
						restdist = 0.0f;
						continue;
					}
					
					if( pNextBranch==NULL ){
						restdist = 0.0f;
						continue;
					}
					
					if( mCurrentPath.GetRestDist() > 0.0f ){
						interpolate = true;
					}else{
						interpolate = false;
					}
					
				}
				
				if( mpControlObj!=NULL ){
					int		inv;
					if( mCurrentPath.GetDir()==pNextBranch->GetDir() ){
						inv = 0;
					}else{
						inv = 1;
					}
					switch( mpControlObj->PathChangeCallback(this,pMovePathInfo,pNextBranch,interpolate,inv) ){
					case	WK7PathControlObj::RESULT_OK:
						break;
					case	WK7PathControlObj::RESULT_CANCEL:
					default:
						restdist = 0.0f;
						continue;
					}
				}
				
				mPathChangeCancel = 0;
				
				if( PrePathChangeProcess(pNextBranch)==0 ){
					restdist = 0.0f;
					continue;
				}

				if( mPathChangeCancel ){
					// pXύXLZ
					mPathChangeCancel = 0;
					restdist = 0.0f;
					continue;
				}
				
				SetBranchPointFix(false);
				SetActive(true);
				
				if( interpolate==false ){
					
					// pX؂ւ
					
					CallLeaveEvent(pNextBranchPoint,pNextBranch->GetPath());
					
					if( mSetCurrentPath ){
						if( mpControlObj!=NULL ){
							mpControlObj->PostMoveCallback(this,pMovePathInfo,restdist,0.0f);
						}
						break;
					}
					
					SetCurrentBranchPath(pNextBranch);
					
				}else{
					
					// ԏ
					
					pPathNurbs = pNextBranch->GetPath();
					
					CallLeaveEvent(pNextBranchPoint,pPathNurbs);
					
					if( mSetCurrentPath ){
						if( mpControlObj!=NULL ){
							mpControlObj->PostMoveCallback(this,pMovePathInfo,restdist,0.0f);
						}
						break;
					}
					
					// ԊJn
					u32		nextDir = pNextBranch->GetDir();
					f32		nextParam;
					if( nextDir ){
						nextParam = pPathNurbs->GetEndBranchLimitParam();
					}else{
						nextParam = pPathNurbs->GetStartBranchLimitParam();
					}
					if( CreateInterpolatePath(&mCurrentPosition,&mCurrentDir,pPathNurbs,nextParam,nextDir)==false ){
						restdist = 0.0f;
						continue;
					}
					
				}
				
				mpNextBranch = NULL;
				
			}
			break;
		case	PCI_ACTIVE:
			// ԃpXړ
			
			if( mInterpolatePath.GetRestDist() <= 0.0f ){
				
				// ԃpX
				ReleaseInterpolatePath();
				
				// ̃pXJgpX
				SetCurrentPath(mNextPath.GetPath(),mNextPath.GetParam(),mNextPath.GetDir());
				
			}
			break;
		}
		
	}
	
	mpNextBranch = NULL;
	
	return true;
	
}

bool WK7PathControl::RenderProcess(void)
{
	if( mpControlPath==NULL ){
		return true;
	}
#ifdef	_DEBUG
	WK7PathNurbs	*pPath;
	WK7PathNurbs		*pPathNurbs;
	if( mainMenu.IsDispPathInfo()==true ){
		if( (pPath=mInterpolatePath.GetPath())!=NULL ){
			pPath->RenderPosition();
		}
		if( mainMenu.IsActive()==false ){
			ghmScenePathSet		*pPathSet;
			WK7PathBranchPoint	*pNextBP;
			const char			*pPathName  = "NULL";
			const char			*pBPName    = "NULL";
			WK7Stage			*pStage     = WK7Stage::GetCurrentStage();
			char				stageID[16];
			C_Paper				paper(0,0,0.75f);
			WK7PathBranch		*pNextBranch;
			Vec					pos;
			f32					param;
			const char			*pAreaName;
			const char			*pScenarioFlowName;
			
			WK7PathControl			*pCamPathControl = WK7CAM->GetPathControl();
			WK7CamPathFollowInfo	*pPathFollowInfo = WK7CAM->GetPathFollowInfo();
			WK7PathInfo				*pCamPathInfo;
			const char				*pCamPathName = NULL;
			f32						camparam;
			u32						camdir;
			const char				*pFootSe;
			u16						footse;
			
			switch( (footse=WK7PLAYERCONTROL->GetCurrentFootSE()) ){
			case	WK7PATH_FOOTSE_WOOD:	pFootSe="Wood";     break;
			case	WK7PATH_FOOTSE_STONE:	pFootSe="Stone";    break;
			case	WK7PATH_FOOTSE_WOOD1:	pFootSe="Carpet";   break;
			case	WK7PATH_FOOTSE_ROOF:	pFootSe="Roof";     break;
			case	WK7PATH_FOOTSE_STONEW:	pFootSe="Stone(W)"; break;
			case	WK7PATH_FOOTSE_STONEC:	pFootSe="Stone(C)"; break;
			case	WK7PATH_FOOTSE_GROUND:	pFootSe="Ground";   break;
			case	WK7PATH_FOOTSE_GRASS:	pFootSe="Grass";    break;
			case	WK7PATH_FOOTSE_MUD:		pFootSe="Mud";      break;
			case	WK7PATH_FOOTSE_IRON:	pFootSe="Iron";     break;
			case	WK7PATH_FOOTSE_DEFAULT:	
			default:						pFootSe="Default";  break;
			}
			
			if( pCamPathControl!=NULL && (pCamPathInfo=pCamPathControl->GetCurrentPathInfo())!=NULL ){
				pCamPathName = pCamPathInfo->GetPathName();
				camparam     = pCamPathInfo->GetParam();
				camdir       = pCamPathInfo->GetDir  ();
			}
			if( pCamPathName==NULL ){
				pCamPathName = "NULL";
				camparam     = 0.0f;
				camdir       = 0;
			}
			
		    if( (pPathNurbs = mCurrentPath.GetPath())!=NULL ){
				pPathSet = pPathNurbs->GetPathSet();
			}
			pNextBP = mCurrentPath.GetNextBranchPoint();
			
			if( pPathSet!=NULL && pPathSet->mpName!=NULL ){
				pPathName = pPathSet->mpName;
			}
			if( pNextBP!=NULL && pNextBP->GetName()!=NULL ){
				pBPName = pNextBP->GetName();
			}
			switch( mInterpolateState ){
			case	PCI_ACTIVE:
				mInterpolatePath.GetPosition(&pos);
				param = mInterpolatePath.GetParam();
				break;
			case	PCI_IDLE:
			default:
				mCurrentPath    .GetPosition(&pos);
				param = mCurrentPath.GetParam();
				break;
			}
			
			s16		hn,on,pn,mn;
			
			if( pStage==NULL ){
				strcpy(stageID,"---");
				hn = 0;
				on = 0;
				pn = 0;
				mn = 0;
			}else{
				u32		mainid = K7STG_MAINID(pStage->GetID());
				u32		suffix = K7STG_SUFFIX(pStage->GetID());
				sprintf(stageID,"Stage%03dM%02d",mainid,suffix);
				hn = pStage->GetHarmanRoomNum();
				on = pStage->GetSelectObjNum();
				pn = pStage->GetSelectPathNum();
				mn = pStage->GetMapNum();
			}
			
			switch( GameInfo.GetAreaID() ){
			case	K7AREA_CEL: pAreaName="CEL"; break;
			case	K7AREA_FUK: pAreaName="FUK"; break;
			case	K7AREA_KAK: pAreaName="KAK"; break;
			case	K7AREA_TXS: pAreaName="TXS"; break;
			case	K7AREA_ISZ: pAreaName="ISZ"; break;
			case	K7AREA_CUR: pAreaName="CUR"; break;
			case	K7AREA_DOM: pAreaName="DOM"; break;
			case	K7AREA_HTL: pAreaName="HTL"; break;
			case	K7AREA_SCL: pAreaName="SCL"; break;
			case	K7AREA_GNK: pAreaName="GNK"; break;
			default:            pAreaName="---"; break;
			}
			
			switch( GameInfo.GetScenarioFlow() ){
			case	K7SCNF_CEL: pScenarioFlowName="CEL"; break;
			case	K7SCNF_FUK: pScenarioFlowName="FUK"; break;
			case	K7SCNF_KAK: pScenarioFlowName="KAK"; break;
			case	K7SCNF_TXS: pScenarioFlowName="TXS"; break;
			case	K7SCNF_ISZ: pScenarioFlowName="ISZ"; break;
			case	K7SCNF_CUR: pScenarioFlowName="CUR"; break;
			case	K7SCNF_DM1: pScenarioFlowName="DM1"; break;
			case	K7SCNF_DM2: pScenarioFlowName="DM2"; break;
			case	K7SCNF_HT1: pScenarioFlowName="HT1"; break;
			case	K7SCNF_SCL: pScenarioFlowName="SCL"; break;
			case	K7SCNF_HT2: pScenarioFlowName="HT2"; break;
			case	K7SCNF_GNK: pScenarioFlowName="GNK"; break;
			default:            pScenarioFlowName="---"; break;
			}
			
			char	str[128];
			
			PRINTF("\n\n  Stage  - %s , Area : %s , Flow : %s\n",stageID,pAreaName,pScenarioFlowName);
			PRINTF(    "         - H %d, O %d, P %d, M %d\n",hn,on,pn,mn);
			PRINTF(    "  FootSE - %s(%02d)\n",pFootSe,footse);
			PRINTF(    "  Path   - %.3f,%s(%c)\n",param,pPathName,mCurrentPath.GetDir()==0? 'F':'B');
			PRINTF(    "  CamPath- %.3f,%s(%c)\n",camparam,pCamPathName,camdir==0? 'F':'B');
			PRINTF(    "  Pos    - %.1f,%.1f,%.1f\n",pos.x,pos.y,pos.z);
			
			bool	first = true;
			
			do{
#define	LINE_MAXLEN		(54)
				if( strlen(pBPName) > LINE_MAXLEN ){
					memcpy(str,pBPName,LINE_MAXLEN);
					str[LINE_MAXLEN] = '\0';
					pBPName += LINE_MAXLEN;
				}else{
					strcpy(str,pBPName);
					pBPName = NULL;
				}
				if( first==true ){
					PRINTF("  NextBP - %s\n",str);
					first = false;
				}else{
					PRINTF("           %s\n",str);
				}
			}while( pBPName!=NULL );
			
			if( pNextBP!=NULL ){
				for(pNextBranch = pNextBP->GetTopBranch() ; pNextBranch!=NULL ; pNextBranch=pNextBranch->GetNext()){
					const char		*pBName = "NULL";
					WK7PathNurbs	*pNextPathNurbs;
					if( (pNextPathNurbs = pNextBranch->GetPath())!=NULL ){
						if( pNextPathNurbs==pPathNurbs ){
							continue;
						}
						if( (pPathSet       = pNextPathNurbs ->GetPathSet())!=NULL &&
						     pPathSet->mpName!=NULL ){
							pBName = pPathSet->mpName;
						}
					}
					PRINTF("   -> %s\n",pBName);
				}
			}
			
			str[0] = '\0';
			if( mCurrentPath.IsStairs()==true ){
				strcat(str," Stairs");
				switch( mCurrentPath.GetStairsState() ){
				case	WK7PATH_UPSTAIRS:
					strcat(str,"-Up");
					break;
				case	WK7PATH_DOWNSTAIRS:
					strcat(str,"-Down");
					break;
				}
			}
			if( mCurrentPath.IsCurb()==true ){
				strcat(str," Curb");
			}
			PRINTF("  Attr   -%s\n",str);
			
		}
	}
#endif
	return true;
}

int WK7PathBranchPoint::CallArriveEvent(void)
{
	int		count = 0;
	
	// CxgR[
	if( mArriveEventID!=WK7PATHBRANCHPOINT_NOARRIVEEVENTID ){
		WK7EVENTCONTROLLER->Call(GetArriveEventID(),(u32)this,0);
		count++;
	}
	
	WK7PathBranchPointEvent	*pEvent;
	for(pEvent=(WK7PathBranchPointEvent *)mEventAnchor.GetNext() ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		switch( pEvent->GetType() ){
		case	WK7PathBranchPointEvent::BPEVTYPE_ARRIVE:
		case	WK7PathBranchPointEvent::BPEVTYPE_ARRIVE_EVNO:
			break;
		default:
			continue;
		}
		WK7EVENTCONTROLLER->Call(pEvent->GetEventID(),(u32)this,(u32)pEvent);
		count++;
	}
	
	return count;
}

int WK7PathBranchPoint::CallLeaveEvent(WK7PathNurbs *pNextPath)
{
	int				count = 0;
	
	// oCxgR[
	WK7PathBranchPointEvent	*pEvent;
	for(pEvent=(WK7PathBranchPointEvent *)mEventAnchor.GetNext() ; pEvent!=NULL ; pEvent=pEvent->GetNext()){
		switch( pEvent->GetType() ){
		case	WK7PathBranchPointEvent::BPEVTYPE_LEAVE:
			break;
		case	WK7PathBranchPointEvent::BPEVTYPE_LEAVE_EVNO:
			if( pNextPath!=NULL ){
				WK7PathBranchPointEventLeaveEventNo	*pEventNo = (WK7PathBranchPointEventLeaveEventNo *)pEvent;
				if( pEventNo->GetPath()==pNextPath ){
					break;
				}
			}
			continue;
		default:
			continue;
		}
		WK7EVENTCONTROLLER->Call(pEvent->GetEventID(),(u32)this,(u32)pEvent);
		count++;
	}
	
	if( pNextPath->GetEnableEvent()==true ){
		// pXXe[WWvCxg
		if( pNextPath->GetEventID()!=0xffffffff ){
			WK7EVENTCONTROLLER->Call(pNextPath->GetEventID(),0,(u32)pNextPath);
			count++;
		}else if( pNextPath->GetNextStageID()!=0xffffffff ){
			WK7EVENTCONTROLLER->CallIM(WK7Stage::DoorEventTask,NULL,0,(u32)pNextPath);
			count++;
		}
	}
	
	return count;
}

int WK7PathControl::PrePathChangeProcess(WK7PathBranch *pBranch)
{
	if( !mEventCall || WK7EVENTCONTROLLER==NULL ){
		return -1;
	}
	WK7Stage			*pStage;
	WK7PathNurbs		*pCurrPath;
	WK7PathNurbs		*pNextPath;
	int					inv;
	if( pBranch==NULL ){
		return 0;
	}
	if( (pNextPath = pBranch->GetPath())==NULL ){
		return 0;
	}
	if( (pCurrPath = mCurrentPath.GetPath())==NULL ){
		return 0;
	}
	if( pBranch->GetDir()==mCurrentPath.GetDir() ){
		inv = 0;
	}else{
		inv = 1;
	}
	if( mpControlPath!=NULL && (pStage = mpControlPath->GetContainer())!=NULL ){
		WK7PrePathChangeInfo	info;
		info.Initialize();
		info.mpCurrPathName     = pCurrPath->GetName();
		info.mpNextPathName     = pNextPath->GetName();
		info.mInvPath           = inv;
		info.SetCurrPath(pCurrPath);
		info.SetNextPath(pNextPath);
		pStage->CallPrePathChangeCallBackTask(info);
	}
	return 1;
}

int WK7PathControl::CallArriveEvent(WK7PathBranchPoint *pBranchPoint,WK7PathNurbs *pNextPath)
{
	if( !mEventCall || WK7EVENTCONTROLLER==NULL ){
		return -1;
	}
	
//#ifdef	_DEBUG
//	if( mainMenu.IsAutoPlay()==false && pBranchPoint->GetAutoPlay()==true ){
//		return -1;
///	}
//#endif
	
#ifdef	_DEBUG
	if( mainMenu.IsAutoPlay()==true ){
#endif
	WK7Stage	*pStage;
	if( mpControlPath!=NULL && (pStage = mpControlPath->GetContainer())!=NULL ){
		
		WK7BpInfo		info;
		const char		*pBranchPointName;
		
		info.Initialize();
		info.SetBranchPoint(pBranchPoint);
		info.SetCurrentPath(mCurrentPath.GetPath());
		info.SetNextPath   (pNextPath);
		pStage->CallBpArriveCallBackTask(info);
		
		// L
		if( (pBranchPointName=pBranchPoint->GetName())!=NULL ){
			ghmGcMemSetAllocName("BPEVENT3");
			WK7StageBpEvent				*pBpEvent = new WK7StageBpEvent(pBranchPointName);
			if( pBpEvent!=NULL ){
				WK7StageBpEventPlayPoint	*pPlayPoint;
				WK7StageBpEventZanPoint		*pZanPoint;
				if( (pPlayPoint = pBpEvent->GetPlayPoint())!=NULL ){
					WK7StageBpEventPlay		*pPlay;
					for(pPlay=pPlayPoint->GetPlayTop() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
						if( pPlay->GetSelectObjName()!=NULL ){
							continue;
						}
						WK7PlayEventInfo	playeventinfo;
						playeventinfo.Initialize();
						playeventinfo.mEventNo      = pPlay->GetEventNo();
						playeventinfo.SetBranchPoint(pBranchPoint);
						pStage->CallPlayEventCallBackTask(playeventinfo);
					}
				}
				if( (pZanPoint = pBpEvent->GetZanPoint())!=NULL ){
					WK7StageBpEventZan		*pZan;
					for(pZan=pZanPoint->GetZanTop() ; pZan!=NULL ; pZan=pZan->GetNext()){
						if( pZan->GetSelectObjName()!=NULL ){
							continue;
						}
						WK7ZanEventInfo		zaneventinfo;
						zaneventinfo.Initialize();
						zaneventinfo.mEventNo = pZan->GetEventNo();
						zaneventinfo.SetBranchPoint(pBranchPoint);
						zaneventinfo.mpStage       = pStage;
						pStage->CallZanEventCallBackTask(zaneventinfo);
					}
				}
			}
			GHM_SAFE_DELETE(pBpEvent);
		}
	}
#ifdef	_DEBUG
	}
#endif
	
	// CxgR[
	if( pBranchPoint->CallArriveEvent() <= 0 ){
		return -1;
	}
	
	// CxgR[Ɉړ֎~ǂH
	if( pBranchPoint->GetMoveDisableAfterEventCall()==true ){
		mState = PC_IDLE;
		return 0;
	}
	
	return 1;
}

bool WK7PathControl::CallPathTermEvent(WK7PathNurbs *pPath)
{
	if( !mEventCall || WK7EVENTCONTROLLER==NULL ){
		return false;
	}
	
	if( pPath->GetEnableEvent()==false ){
		return false;
	}
	
	if( pPath->GetEventID()==0xffffffff ){
		if( pPath->GetEventNo()==0xffffffff ){
			if( pPath->GetNextStageID()==0xffffffff ){
				return false;
			}
			WK7EVENTCONTROLLER->CallIM(WK7Stage::DoorEventTask,NULL,0,(u32)pPath);
		}else{
			WK7Stage		*pStage = pPath->GetStage();
			if( pStage==NULL ){
				return NULL;
			}
			WK7DoorEventInfo		info;
			info.Initialize();
			info.mEventNo     = pPath->GetEventNo();
			info.mCurrStageID = pStage->GetID();
			info.mNextStageID = pPath->GetNextStageID();
			info.SetNextBpName(pPath->GetNextPoint());
			WK7Stage::SetPrevStageID(info.mCurrStageID);
			WK7Stage::SetCurrentStageBpName(pPath->GetNextPoint());
			WK7Stage::SetPrevStagePathName(pPath->GetShortName());
			WK7Stage::SetPrevStageBpName(NULL);
			WK7Stage::SetCurrentStageParam(pPath->GetParam());
			pStage->CallDoorEventCallBackTask(info);
		}
		return true;
	}
	
	WK7EVENTCONTROLLER->Call(pPath->GetEventID(),0,(u32)pPath);
	
	return true;
}

int WK7PathControl::CallLeaveEvent(WK7PathBranchPoint *pBranchPoint,WK7PathNurbs *pNextPath)
{
	if( !mEventCall || WK7EVENTCONTROLLER==NULL ){
		return -1;
	}
	
	WK7Stage	*pStage;
	if( mpControlPath!=NULL && (pStage = mpControlPath->GetContainer())!=NULL ){
		WK7BpInfo		info;
		info.Initialize();
		info.SetBranchPoint(pBranchPoint);
		info.SetCurrentPath(mCurrentPath.GetPath());
		info.SetNextPath   (pNextPath);
		pStage->CallBpLeaveCallBackTask(info);
	}
	
	// oCxgR[
	if( pBranchPoint->CallLeaveEvent(pNextPath) <= 0 ){
		return 0;
	}
	
	return 1;
}

bool WK7PathControl::CreateInterpolatePath(const Vec *pStartPos,const Vec *pStartDir,WK7PathNurbs *pNextPath,f32 nextParam,u32 nextDir)
{
	Vec		endPos,endDir;
	mNextPath.SetPath         (pNextPath);
	mNextPath.SetParam        (nextParam);
	mNextPath.SetDir          (nextDir  );
	mNextPath.GetPosition     (&endPos  );
	mNextPath.GetDeltaPosition(&endDir  );
	return CreateInterpolatePath(pStartPos,pStartDir,&endPos,&endDir);
}

#define	INTERPOLATE_PATH_DISTCALC_PATHDIVNUM		(32)
bool WK7PathControl::CreateInterpolatePath(const Vec *pStartPos,const Vec *pStartDir,const Vec *pEndPos,const Vec *pEndDir)
{
	ghmNurbsPathDesc	*pPathDesc;
	ghmScenePathSet		*pPathSet;
	WK7PathNurbs		*pPathNurbs;
	Vec					cur,pre;
	f32					dist;
	
	ghmGcMemSetAllocName("PACTIP");
	pPathDesc = (ghmNurbsPathDesc *)K7AppSmallMemAlloc(sizeof(ghmNurbsPathDesc));
	pPathDesc->mDegree        = 2;
	pPathDesc->mSpanCount     = 1;
	pPathDesc->mCVCount       = 4;
	pPathDesc->mKnotCount     = 7;
	pPathDesc->mStartParam    = 0.0f;
	pPathDesc->mEndParam      = 2.0f;
	pPathDesc->mTotalDistance = 0.0f;
	ghmGcMemSetAllocName("PACTIPCV");
	pPathDesc->mpCV           = (Vec4 *)K7AppSmallMemAlloc((int)(sizeof(Vec4) * pPathDesc->mCVCount  ));
	ghmGcMemSetAllocName("PACTIPKN");
	pPathDesc->mpKnot         = (f32  *)K7AppSmallMemAlloc((int)(sizeof(f32 ) * pPathDesc->mKnotCount));
	pPathDesc->mpKnot[0]      = 
	pPathDesc->mpKnot[1]      = 
	pPathDesc->mpKnot[2]      = 0.0f;
	pPathDesc->mpKnot[3]      = 1.0f;
	pPathDesc->mpKnot[4]      = 
	pPathDesc->mpKnot[5]      = 
	pPathDesc->mpKnot[6]      = 2.0f;
	pPathDesc->mpCV  [0].x    = pStartPos->x;
	pPathDesc->mpCV  [0].y    = pStartPos->y;
	pPathDesc->mpCV  [0].z    = pStartPos->z;
	pPathDesc->mpCV  [0].w    = 1.0f;
	pPathDesc->mpCV  [3].x    = pEndPos  ->x;
	pPathDesc->mpCV  [3].y    = pEndPos  ->y;
	pPathDesc->mpCV  [3].z    = pEndPos  ->z;
	pPathDesc->mpCV  [3].w    = 1.0f;
	
	pre.x = pEndPos->x - pStartPos->x;
	pre.y = pEndPos->y - pStartPos->y;
	pre.z = pEndPos->z - pStartPos->z;
	
	VECNormalize(pStartDir,&cur);
//	dist  = 1.0f / GHM_VEC_GET_LEN(pStartDir);
//	cur.x = dist * pStartDir->x;
//	cur.y = dist * pStartDir->y;
//	cur.z = dist * pStartDir->z;
	dist  = cur.x * pre.x + cur.y * pre.y + cur.z * pre.z;
static f32	startrate = 0.3f;
	dist *= startrate;
//	dist *= 0.5f;
	
	pPathDesc->mpCV  [1].x    = pStartPos->x + dist * cur.x;
	pPathDesc->mpCV  [1].y    = pStartPos->y + dist * cur.y;
	pPathDesc->mpCV  [1].z    = pStartPos->z + dist * cur.z;
	pPathDesc->mpCV  [1].w    = 1.0f;
	
	VECNormalize(pEndDir,&cur);
//	dist  = 1.0f / GHM_VEC_GET_LEN(pEndDir);
//	cur.x = dist * pEndDir->x;
//	cur.y = dist * pEndDir->y;
//	cur.z = dist * pEndDir->z;
	dist  = cur.x * pre.x + cur.y * pre.y + cur.z * pre.z;
static f32	endrate = 0.3f;
	dist *= endrate;
//	dist *= 0.5f;
	
	pPathDesc->mpCV  [2].x    = pEndPos->x - dist * cur.x;
	pPathDesc->mpCV  [2].y    = pEndPos->y - dist * cur.y;
	pPathDesc->mpCV  [2].z    = pEndPos->z - dist * cur.z;
	pPathDesc->mpCV  [2].w    = 1.0f;
	
	ghmGcMemSetAllocName("PACTIPPS");
	pPathSet  = (ghmScenePathSet *)K7AppSmallMemAlloc(sizeof(ghmScenePathSet));
	pPathSet->mpDesc = pPathDesc;
	pPathSet->mpName = "InterpolatePath";
	pPathSet->mpData = NULL;
	
	ghmGcMemSetAllocName("Nurbs");
	pPathNurbs = new WK7PathNurbs(NULL,pPathSet);
	
	mInterpolatePath.SetPath (pPathNurbs);
	mInterpolatePath.SetParam(pPathDesc->mStartParam);
	mInterpolatePath.SetDir  (0);
	
	pPathDesc->mTotalDistance = pPathNurbs->CalcDistance(pPathDesc->mEndParam);
	
	mInterpolateState = PCI_ACTIVE;
	mInterpolateStop  = 0;
	
	mIpMoveDistance.SetValue(0.0f);
	
	return true;
}

bool WK7PathControl::ReleaseInterpolatePath(void)
{
	if( mInterpolateState!=PCI_ACTIVE ){
		return false;
	}
	
	ghmNurbsPathDesc	*pPathDesc;
	ghmScenePathSet		*pPathSet;
	WK7PathNurbs		*pPathNurbs;
	
	pPathNurbs = mInterpolatePath.GetPath();
	pPathSet   = pPathNurbs->GetPathSet();
	pPathDesc  = pPathSet->mpDesc;
	
	GHM_SAFE_DELETE   (pPathNurbs       );
	GHMGCMEM_SAFE_FREE(pPathDesc->mpCV  );
	GHMGCMEM_SAFE_FREE(pPathDesc->mpKnot);
	GHMGCMEM_SAFE_FREE(pPathDesc        );
	GHMGCMEM_SAFE_FREE(pPathSet         );
	
	mInterpolatePath.SetPath((WK7PathNurbs *)NULL);
	
	mInterpolateState = PCI_IDLE;
	
	return true;
}

bool WK7PathControl::Move(f32 dist)
{
	mReqMoveDistance = dist;
	return true;
}

bool WK7PathControl::Reverse(void)
{
	mReverseReq = 1;
	return true;
}

bool WK7PathControl::ReverseCancel(void)
{
	mReverseReq = 0;
	return true;
}

bool WK7PathControl::SetActive(bool flag)
{
	if( mBranchPointFix ){
		return false;
	}
	switch( mState ){
	case	PC_IDLE:
		if( flag==false ){
			return true;
		}
		mState = PC_ACTIVE;
		mActiveCounter = 0;
		break;
	case	PC_ACTIVE:
		if( flag==true ){
			return true;
		}
		mState = PC_IDLE;
		break;
	case	PC_PATHSELECT:{
//		WK7PathNurbs	*pPathNurbs;
//		if( (pPathNurbs=mCurrentPath.GetPath())==NULL ){
//			return false;
//		}
//		if( pPathNurbs->GetEnable()==false ){
//			return false;
//		}
		if( flag==true ){
			mState = PC_ACTIVE;
		}else{
			mState = PC_IDLE;
		}
		}break;
	default:
		return false;
	}
	return true;
}

bool WK7PathControl::SetPathSelectMode(void)
{
	if( mState!=PC_ACTIVE ){
		return false;
	}
	mState = PC_PATHSELECT;
	return true;
}

bool WK7PathControl::SetControlObj(WK7PathControlObj *pObj)
{
	mpControlObj = pObj;
	return true;
}

WK7PathInfo* WK7PathControl::GetControlPathInfo(void)
{
	switch( mInterpolateState ){
	case	PCI_ACTIVE:
		return &mInterpolatePath;
	case	PCI_IDLE:
	default:
		break;
	}
	return &mCurrentPath;
}

WK7CameraPathDesc* WK7PathControl::GetCameraPath(const char *pName)
{
	if( mpControlPath==NULL ){
		return NULL;
	}
	return mpControlPath->GetCameraPath(pName);
}

WK7CameraPathDesc* WK7PathControl::GetCameraPath(WK7PathNurbs *pPath)
{
	if( mpControlPath==NULL ){
		return NULL;
	}
	return mpControlPath->GetCameraPath(pPath);
}

u16 WK7PathControl::GetCurrentFootSE(void)
{
	WK7PathNurbs	*pPath = mCurrentPath.GetPath();
	if( pPath==NULL ){
		return WK7PATH_FOOTSE_DEFAULT;
	}
	return pPath->GetFootSE();
}

WK7PathControlObj::WK7PathControlObj()
{
}

WK7PathControlObj::~WK7PathControlObj()
{
}

WK7PathControlObj::CallbackResult WK7PathControlObj::PreMoveCallback(WK7PathControl *pPathControl,
                                                                     WK7PathInfo *pInfo,
                                                                     f32 restdist)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
#pragma unused(restdist)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::PostMoveCallback(WK7PathControl *pPathControl,
                                                                      WK7PathInfo *pInfo,
                                                                      f32 restdist,
                                                                      f32 movedist)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
#pragma unused(restdist)
#pragma unused(movedist)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::ReverseCallback(WK7PathControl *pPathControl,
                                                                     WK7PathInfo *pInfo)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::RenewCallback(WK7PathControl *pPathControl,
                                                                   WK7PathInfo *pInfo)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::BranchPointArriveCallback(WK7PathControl *pPathControl,
                                                                               WK7PathInfo *pInfo,
                                                                               WK7PathBranchPoint *pBranchPoint)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
#pragma unused(pBranchPoint)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::PathSelectCallback(WK7PathControl *pPathControl,
                                                                        WK7PathInfo *pInfo,
                                                                        WK7PathBranchPoint *pBranchPoint)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
#pragma unused(pBranchPoint)
	return RESULT_OK;
}

WK7PathControlObj::CallbackResult WK7PathControlObj::PathChangeCallback(WK7PathControl *pPathControl,
                                                                        WK7PathInfo *pInfo,
                                                                        WK7PathBranch *pSelectBranch,
                                                                        bool interpolate,
                                                                        int inv)
{
#pragma unused(pPathControl)
#pragma unused(pInfo)
#pragma unused(pSelectBranch)
#pragma unused(interpolate)
#pragma unused(inv)
	return RESULT_OK;
}

