////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Play  : L
//
// WK7Play.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 04/03/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7PLAY_CPP__

//#include <dolphin.h>
#include ".\WK7Play.h"
#include ".\WK7MainTask.h"
#include ".\WK7Npc.h"
#include ".\WK7Shadow.h"
#include ".\WK7CharaPresent.h"
#include "..\Play\K7PlayEfCharaPresent.h"
#include ".\WK7Event.h"
#include ".\WK7EventScript.h"
#include ".\WK7Archive.h"
#include ".\WK7Item.h"
#include ".\WK7SceneBlur.h"
#include "..\Matsu\MK7EngTelop.h"
#include "..\Matsu\MK7Noise.h"

#include "..\Yam\YK7Effect.h"
#include "..\Yam\YK7EffectHeliExp.h"
#include "..\Yam\YK7EffectCharGeki.h"
#include "..\Yam\YK7EffectMuzzle.h"
#include "..\Yam\YK7EffectHSDead.h"
#include "..\Yam\YK7EffectHSExplode.h"
#include "..\Yam\YK7EffectGore.h"
#include "..\Yam\YK7EffectFlash.h"

#include "..\Matsu\MK7CharaDisp.h"

#include "..\Fujita\C_Font.h"

#include ".\PlayTable.h"

#define	WK7PLAY_HOPPER7

#define	DEBUGPRINT
#if	!defined(_DEBUG) || defined(K7APP_MASTER) 
#undef	DEBUGPRINT
#endif

//#define	WK7PLAY_VOLUME_MUSIC		(VOLUME_MUSIC * 13 / 10)
//#define	WK7PLAY_VOLUME_MUSIC		(255)
#define	WK7PLAY_VOLUME_MUSIC		(VOLUME_MUSIC * 14 / 10)

ghmGcListObj			WK7Play::mAnchor;
ghmGcTaskController		WK7Play::mTaskController;
WK7Play*				WK7Play::mpCurrentPlay = NULL;
u32						WK7Play::mPlayedFlag[WK7PLAYTBL_WORKSIZE] = { 0, };
bool					WK7Play::mCamWobble = false;
bool					WK7Play::mPlayTBStored = false;
s16						WK7Play::mStoredBgmID  = -1;
bool					WK7Play::mPlayerRelease = true;
f32						WK7Play::mStoredSceneBlurDist[WK7PLAY_STORE_SCENEBLUR_NUM] = { 0.0f, };
bool					WK7Play::mStoredSceneBlurVisible = true;
bool					WK7Play::mTermCaptureRenderOn    = false;
bool					WK7Play::mTermCaptureRenderMono  = false;
bool					WK7Play::mTermCapturePreRender   = false;
bool					WK7Play::mInitCaptureRenderOff   = false;
bool					WK7Play::mSkipCancel             = false;
bool					WK7Play::mDontStopStream         = false;
bool					WK7Play::mDontReleasePlayerSsm   = false;
bool					WK7Play::mHopper7Mode            = false;
static		f32	zdist[WK7PLAY_STORE_SCENEBLUR_NUM] = {
	5000.0f,
	5000.0f,
	2500.0f,
	2500.0f,
};

WK7PlayMotion::WK7PlayMotion(WK7Play *pPlay,ghmGcListObj *pAnchor,int prio) : WK7PlayMotionBase(pPlay),ghmGcListObj(pAnchor,prio)
{
	mID = (u32)prio;
}

WK7PlayMotion::~WK7PlayMotion()
{
}

WK7PlayMotionBase::WK7PlayMotionBase(WK7Play *pPlay)
{
	mpArchive      = NULL;
	mpModelMotion  = NULL;
	mpCameraMotion = NULL;
	mpLightMotion  = NULL;
	mpFileName     = NULL;
	mState         = IDLE;
	mpPlay         = pPlay;
	mID            = 0;
}

WK7PlayMotionBase::~WK7PlayMotionBase()
{
	Release();
}

bool WK7PlayMotionBase::Release(void)
{
	if( mpModelMotion==NULL && mpCameraMotion==NULL && mpLightMotion==NULL ){
		GHM_SAFE_DELETE(mpArchive);
	}else{
		GHM_SAFE_DELETE(mpModelMotion );
		GHM_SAFE_DELETE(mpCameraMotion);
		GHM_SAFE_DELETE(mpLightMotion );
	}
	GHM_SAFE_DELETE_ARRAY(mpFileName    );
	return true;
}

bool WK7PlayMotionBase::LoadStart(void)
{
	if( mState!=IDLE ){
		return false;
	}
	if( mpFileName==NULL ){
		return false;
	}
	if( (mpArchive = WK7Archive::Create())==NULL ){
		return false;
	}
	if( mpArchive->LoadArchive(mpFileName)==false ){
		GHM_SAFE_DELETE(mpArchive);
		return false;
	}
	u32		playid;
	if( mpPlay != NULL ){
		playid = mpPlay->GetPlayID();
	}else{
		playid = 0;
	}
	u32		type = WK7PLAY_TYPE_MODEL(playid);
	mpArchive->SetType(type);
	mpArchive->SetID  (mID);
	mState = LOADING;
	return true;
}

bool WK7PlayMotionBase::SetArchive(void *pArchiveData)
{
	if( mState!=IDLE ){
		return false;
	}
	if( pArchiveData==NULL ){
		mState = LOADED;
		return true;
	}
	if( (mpArchive = WK7Archive::Create())==NULL ){
		return false;
	}
	if( mpArchive->LoadArchive(pArchiveData)==false ){
		GHM_SAFE_DELETE(mpArchive);
		return false;
	}
	u32		playid;
	if( mpPlay != NULL ){
		playid = mpPlay->GetPlayID();
	}else{
		playid = 0;
	}
	u32		type = WK7PLAY_TYPE_MODEL(playid);
	mpArchive->SetType(type);
	mpArchive->SetID  (mID);
	mState = LOADED;
	Initialize();
	return true;
}

bool WK7PlayMotionBase::LoadWait(void)
{
	while( mState==IDLE ){
		if( LoadStart()==true ){
			break;
		}
		GHMGCTASK_RETSYS();
	}
	while( mState!=LOADED ){
		GHMGCTASK_RETSYS();
	}
	return true;
}

bool WK7PlayMotionBase::Initialize(void)
{
	if( mpArchive==NULL ){
		return true;
	}
	mpModelMotion  = mpArchive->CreateModelMotion ();
	mpCameraMotion = mpArchive->CreateCameraMotion();
	mpLightMotion  = mpArchive->CreateLightMotion ();
	return true;
}

bool WK7PlayMotionBase::FrameProcess(void)
{
	switch( mState ){
	case	IDLE:
		break;
	case	LOADING:
		if( mpArchive->GetState()!=WK7Archive::WK7ARCHIVE_LOADED ){
			break;
		}
		mState = LOADED;
		Initialize();
	case	LOADED:
	default:
		break;
	}
	return true;
}

bool WK7PlayMotionBase::SetFileName(const char *pfname)
{
	GHM_SAFE_DELETE_ARRAY(mpFileName);
	if( pfname==NULL ){
		return true;
	}
	ghmGcMemSetAllocName("FileName");
	if( (mpFileName = new char[strlen(pfname) + 1])==NULL ){
		return false;
	}
	strcpy(mpFileName,pfname);
	return true;
}

WK7PlaySepMotion::WK7PlaySepMotion(WK7Play *pPlay) : mModelCameraMotion(pPlay),
                                                     mMatMotion        (pPlay),
                                                     mShapeMotion      (pPlay),
                                                     mEffectMotion     (pPlay),
                                                     mLightMotion      (pPlay)
{
	mState    = IDLE;
	mpSepName = NULL;
	mpPlay    = pPlay;
	mID       = 0;
}

WK7PlaySepMotion::~WK7PlaySepMotion()
{
	GHM_SAFE_DELETE_ARRAY(mpSepName);
}

bool WK7PlaySepMotion::SetID(u32 id)
{
	mID   = id;
	id  <<= WK7PLAYSEPMOTION_IDSHIFT;
	id   |= WK7PLAYSEPMOTION_IDTYPE;
	mModelCameraMotion.SetID(id|WK7PLAYSEPMOTION_MODELCAMERA);
	mMatMotion        .SetID(id|WK7PLAYSEPMOTION_MAT        );
	mShapeMotion      .SetID(id|WK7PLAYSEPMOTION_SHAPE      );
	mEffectMotion     .SetID(id|WK7PLAYSEPMOTION_EFFECT     );
	mLightMotion      .SetID(id|WK7PLAYSEPMOTION_EFFECT     );
	return true;
}

bool WK7PlaySepMotion::FrameProcess(void)
{
	mModelCameraMotion.FrameProcess();
	mMatMotion        .FrameProcess();
	mShapeMotion      .FrameProcess();
	mEffectMotion     .FrameProcess();
	mLightMotion      .FrameProcess();
	switch( mState ){
	case	IDLE:
		break;
	case	LOADING:
		if( mModelCameraMotion.GetState()==WK7PlayMotionBase::LOADING ){
			break;
		}
		if( mMatMotion.GetState()==WK7PlayMotionBase::LOADING ){
			break;
		}
		if( mShapeMotion.GetState()==WK7PlayMotionBase::LOADING ){
			break;
		}
		if( mEffectMotion.GetState()==WK7PlayMotionBase::LOADING ){
			break;
		}
		if( mLightMotion.GetState()==WK7PlayMotionBase::LOADING ){
			break;
		}
		mState = LOADED;
	case	LOADED:
	default:
		break;
	}
	return true;
}

bool WK7PlaySepMotion::SetArchive(void *pModelCameraMotionArchiveData,
                                  void *pShapeMotionArchiveData,
                                  void *pMatMotionArchiveData,
                                  void *pEffectMotionArchiveData,
                                  void *pLightMotionArchiveData)
{
	mModelCameraMotion.SetArchive(pModelCameraMotionArchiveData);
	mShapeMotion      .SetArchive(pShapeMotionArchiveData      );
	mMatMotion        .SetArchive(pMatMotionArchiveData        );
	mEffectMotion     .SetArchive(pEffectMotionArchiveData     );
	mLightMotion      .SetArchive(pLightMotionArchiveData      );
	mState = LOADED;
	return true;
}

bool WK7PlaySepMotion::LoadStart(void)
{
	if( mpSepName==NULL || mpPlay==NULL ){
		return true;
	}
	char		resourceName   [WK7PLAY_RESOURCE_NAME_MAX];
	char		motResourceName[WK7PLAY_RESOURCE_NAME_MAX];
	
	mpPlay->GetFileName(resourceName,true);
	
	sprintf(motResourceName,"%smt%s.dat",resourceName,mpSepName);
	mModelCameraMotion.SetFileName(motResourceName);
	
	sprintf(motResourceName,"%smm%s.dat",resourceName,mpSepName);
	mMatMotion        .SetFileName(motResourceName);
	
	sprintf(motResourceName,"%ssm%s.dat",resourceName,mpSepName);
	mShapeMotion      .SetFileName(motResourceName);
	
	sprintf(motResourceName,"%sef%s.dat",resourceName,mpSepName);
	mEffectMotion     .SetFileName(motResourceName);
	
	sprintf(motResourceName,"%slm%s.dat",resourceName,mpSepName);
	mLightMotion      .SetFileName(motResourceName);
	
	if( mModelCameraMotion.LoadStart()==false ){
		return false;
	}
	if( mMatMotion.LoadStart()==false ){
		return false;
	}
	if( mShapeMotion.LoadStart()==false ){
		return false;
	}
	if( mEffectMotion.LoadStart()==false ){
		return false;
	}
	if( mLightMotion.LoadStart()==false ){
		return false;
	}
	mState = LOADING;
	return true;
}

bool WK7PlaySepMotion::Release(void)
{
	mModelCameraMotion.Release();
	mMatMotion        .Release();
	mShapeMotion      .Release();
	mEffectMotion     .Release();
	mLightMotion      .Release();
	SetSepName(NULL);
	return true;
}

bool WK7PlaySepMotion::SetSepName(const char *pname)
{
	GHM_SAFE_DELETE_ARRAY(mpSepName);
	if( pname==NULL ){
		return true;
	}
	ghmGcMemSetAllocName("SepName");
	if( (mpSepName = new char[strlen(pname) + 1])==NULL ){
		return false;
	}
	strcpy(mpSepName,pname);
	return true;
}

WK7ModelMotion* WK7PlaySepMotion::GetModelMotion(void)
{
	return mModelCameraMotion.GetModelMotion();
}

WK7CameraMotion* WK7PlaySepMotion::GetCameraMotion(void)
{
	return mModelCameraMotion.GetCameraMotion();
}

WK7ModelMotion* WK7PlaySepMotion::GetMatMotion(void)
{
	return mMatMotion.GetModelMotion();
}

WK7ModelMotion* WK7PlaySepMotion::GetShapeMotion(void)
{
	return mShapeMotion.GetModelMotion();
}

WK7ModelMotion* WK7PlaySepMotion::GetEffectMotion(void)
{
	return mEffectMotion.GetModelMotion();
}

WK7LightMotion* WK7PlaySepMotion::GetLightMotion(void)
{
	WK7LightMotion	*pLightMotion;
	if( (pLightMotion=mLightMotion.GetLightMotion())!=NULL ){
		return pLightMotion;
	}
	return mEffectMotion.GetLightMotion();
}

WK7PlaySepMotionPack::WK7PlaySepMotionPack(WK7Play *pPlay)
{
	mState         = IDLE;
	mpPlay         = pPlay;
	mpPackName     = NULL;
	mppSepMotion   = NULL;
	mSepMotionNum  = 0;
	mPlaySepMotion = -1;
	mID            = 0;
	mpPackData     = NULL;
	mpPackResLink  = NULL;
	mpFile         = NULL;
	mAllocBuffer   = false;
}

WK7PlaySepMotionPack::~WK7PlaySepMotionPack()
{
	Release();
}

bool WK7PlaySepMotionPack::FrameProcess(void)
{
	switch( mState ){
	case	IDLE:
		break;
	case	LOADING:
		if( mpFile==NULL ){
			mState = LOADED;
			break;
		}
		if( mpFile->GetState()!=ghmGcFile::READFINISH ){
			break;
		}
		mpFile->Close();
		delete mpFile;
		mpFile = NULL;
		mState = LOADED;
		Initialize();
	case	LOADED:
	default:
		break;
	}
	return true;
}

bool WK7PlaySepMotionPack::GetResourceName(char *pResourceName)
{
	if( mpPackName==NULL || mpPlay==NULL || pResourceName==NULL ){
		return false;
	}
	
	char		resourceName[WK7PLAY_RESOURCE_NAME_MAX];
	
	mpPlay->GetFileName(resourceName,true);
	
	sprintf(pResourceName,"%s_%s.pak",resourceName,mpPackName);
	
	return true;
}

bool WK7PlaySepMotionPack::LoadStart(void *pBuff)
{
	if( mpPackName==NULL || mpPlay==NULL ){
		return true;
	}
	char		packName[WK7PLAY_RESOURCE_NAME_MAX];
	GetResourceName(packName);
	ghmGcMemSetAllocName("ghmGcFile");
	if( (mpFile = new ghmGcFile)==NULL ){
		return false;
	}
	if( mpFile->GetState()==ghmGcFile::CLOSED ){
		if( (mpFile->Open(packName))==false ){
			return false;
		}
	}
	if( pBuff==NULL ){
		mAllocBuffer = true;
	}
	if( (mpPackData = (ghmGcResLink *)mpFile->Read(pBuff))==NULL ){
//		Release();
		return false;
	}
	mState = LOADING;
	return true;
}

bool WK7PlaySepMotionPack::Initialize(void)
{
	if( (mpPackResLink = new(mpPackData) ghmGcResLink)==NULL ){
		return false;
	}
	ghmGcResGroup	*pGroup            [5];
	void			*pMotionArchiveData[5];
	u32				i,j;
	// fJCg[V
	for(i=0 ; i<5 ; i++){
		pGroup[i] = mpPackResLink->GetGroup((int)i);
	}
	if( pGroup[0]!=NULL && (mSepMotionNum = pGroup[0]->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLSMA");
		mppSepMotion  = (WK7PlaySepMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7PlaySepMotion*) * mSepMotionNum));
		for(i=0 ; i<mSepMotionNum ; i++){
			mppSepMotion[i] = NULL;
		}
		for(i=0 ; i<mSepMotionNum ; i++){
			ghmGcMemSetAllocName("PlSepMot");
			if( (mppSepMotion[i]=new WK7PlaySepMotion(mpPlay))==NULL ){
				return false;
			}
			for(j=0 ; j<5 ; j++){
				if( pGroup[j]!=NULL ){
					pMotionArchiveData[j] = pGroup[j]->GetResource((int)i);
				}else{
					pMotionArchiveData[j] = NULL;
				}
			}
			mppSepMotion[i]->SetArchive(pMotionArchiveData[0],
			                            pMotionArchiveData[1],
			                            pMotionArchiveData[2],
			                            pMotionArchiveData[3],
			                            pMotionArchiveData[4]);
		}
	}
	return true;
}

bool WK7PlaySepMotionPack::Release(void)
{
	int			i;
	if( mppSepMotion != NULL ){
		for(i=0 ; i<mSepMotionNum ; i++){
			GHM_SAFE_DELETE(mppSepMotion[i]);
		}
		GHM_SAFE_DELETE_ARRAY(mppSepMotion);
	}
	GHM_SAFE_DELETE_ARRAY(mpPackName   );
	GHM_SAFE_DELETE      (mpFile       );
	if( mAllocBuffer==true ){
		if( mpPackResLink != NULL ){
			GHM_SAFE_DELETE(mpPackResLink);
		}else{
			GHMGCMEM_SAFE_FREE(mpPackData);
		}
		mAllocBuffer = false;
	}
	mpPackData     = NULL;
	mpPackResLink  = NULL;
	mState         = IDLE;
	mpPlay         = NULL;
	mSepMotionNum  = 0;
	mPlaySepMotion = -1;
	mID            = 0;
	return true;
}

bool WK7PlaySepMotionPack::IncPlayCount(void)
{
	if( mPlaySepMotion >= (int)mSepMotionNum ){
		return false;
	}
	mPlaySepMotion++;
	return true;
}

bool WK7PlaySepMotionPack::SetID(u32 id)
{
	u32					i;
	WK7PlaySepMotion	**ppSepMotion;
	mID   = id;
	id  <<= WK7PLAYSEPMOTIONPACK_IDSHIFT;
	for(i=0,ppSepMotion=mppSepMotion ; i<mSepMotionNum ; i++,ppSepMotion++){
		if( *ppSepMotion==NULL ){
			continue;
		}
		(*ppSepMotion)->SetID(i|id);
	}
	return true;
}

bool WK7PlaySepMotionPack::SetPackName(const char *pname)
{
	GHM_SAFE_DELETE_ARRAY(mpPackName);
	if( pname==NULL ){
		return true;
	}
	ghmGcMemSetAllocName("PackName");
	if( (mpPackName = new char[strlen(pname) + 1])==NULL ){
		OSDebugReport("Warning : SetPackName Alloc Error !!\n");
		K7AppDebugPrint("Warning : SetPackName Alloc Error !!");
		return false;
	}
	strcpy(mpPackName,pname);
	return true;
}

WK7PlaySepMotion* WK7PlaySepMotionPack::GetSepMotion(int index)
{
	if( index < 0 || index >= mSepMotionNum || mppSepMotion==NULL ){
		return NULL;
	}
	return mppSepMotion[index];
}

WK7PlaySepMotion* WK7PlaySepMotionPack::GetPlaySepMotion(void)
{
	return GetSepMotion(mPlaySepMotion);
}

WK7PlaySepMotion* WK7PlaySepMotionPack::GetNextSepMotion(void)
{
	return GetSepMotion(mPlaySepMotion + 1);
}

WK7PlayRenderObj::WK7PlayRenderObj(ghmGcListObj *pAnchor,int prio) : WK7Stage(pAnchor,prio)
{
	mClothEnable         = false;
	mpAlpha              = NULL;
	mpCurrentMotion      = NULL;
	mpCurrentMatMotion   = NULL;
	mpCurrentShapeMotion = NULL;
}

WK7PlayRenderObj::~WK7PlayRenderObj()
{
	GHM_SAFE_DELETE(mpAlpha             );
	GHM_SAFE_DELETE(mpCurrentMotion     );
	GHM_SAFE_DELETE(mpCurrentMatMotion  );
	GHM_SAFE_DELETE(mpCurrentShapeMotion);
}

bool WK7PlayRenderObj::Initialize(void)
{
	InitializeLight();
	if( WK7RenderObj::Initialize()==false ){
		return false;
	}
	// epCg擾
	FindShadowLight();
	WK7JObj		*pJObj,*pChild,*pFindJObj;
	if( mpModel!=NULL ){
		for(pJObj=mpModel->GetTopObj() ; pJObj!=NULL ; pJObj=pJObj->GetNext()){
			if( (pFindJObj=pJObj->GetObj(K7CHRBONE_WAIST,false,true))!=NULL ){
				WK7CastShadowObj	*pCastShadowObj;
				if( (pCastShadowObj = WK7SHADOW->AddCastShadowObj(pFindJObj))!=NULL ){
					pCastShadowObj->SetRenderObj(this);
				}
#ifdef	LAS_VEGAS
			// XxKXŌ聫
			}else if( (pFindJObj=pJObj->GetObj("Trunk",false,true))!=NULL ){
				WK7SHADOW->AddCastShadowObj   (pJObj);
			// XxKXŌ聪
#endif
			}else if( (pFindJObj=pJObj->GetObj("floor",false,true))!=NULL && (pChild=pFindJObj->GetChild())!=NULL ){
				WK7SHADOW->AddReceiveShadowObj(pChild);
			}
		}
	}
	return true;
}

bool WK7PlayRenderObj::DispLight(u32 mirrorType,Vec *pPos)
{
	WK7Light			*pLight;
	WK7LightFindData	data;
//	WK7Light			*pPlayerLight = WK7PLAYERCONTROL->GetLight();
	data.Initialize();
	while( (pLight=WK7Light::Find(data))!=NULL ){
		if( pLight->GetObjectType()!=mType ){
			continue;
		}
		if( mObjLight==false ){
			if( pLight->GetObjectID()==mID ){
				continue;
			}
		}
		if( pLight->GetPrivate()==true ){
			DispPrivateLight(pLight,mirrorType,pPos);
		}else{
//			if( mPlayerLight==false && pLight==pPlayerLight ){
//				continue;
//			}
			if( mDispLight & pLight->GetLightIDBit() ){
				pLight->Disp(mirrorType,pPos);
			}
		}
	}
	return false;
}

bool WK7PlayRenderObj::SetAlpha(f32 alpha)
{
	if( alpha > 1.0f ){
		alpha = 1.0f;
	}else if( alpha < 0.0f ){
		alpha = 0.0f;
	}
	if( alpha>=1.0f ){
		GHM_SAFE_DELETE(mpAlpha);
		return true;
	}
	if( mpAlpha==NULL ){
		if( mpModel==NULL ){
			return false;
		}
		WK7JObj		*pTopObj = mpModel->GetTopObj();
		if( pTopObj==NULL ){
			return false;
		}
		HSD_JObj	*pJObj = pTopObj->GetJObj();
		ghmGcMemSetAllocName("C_SetAl");
		if( (mpAlpha = new C_SetAlpha(pJObj,alpha))==NULL ){
			return false;
		}
	}else{
		mpAlpha->SetVal(alpha);
	}
	return true;
}

bool WK7PlayRenderObj::PlayMotion(WK7PlayMotion *pMotion,f32 sec)
{
	if( StopMotion()==false ){
		return false;
	}
	if( mpModel==NULL || pMotion==NULL ){
		return false;
	}
	WK7ModelMotion		*pModelMotion = pMotion->GetModelMotion();
	if( pModelMotion==NULL ){
		return false;
	}
	bool	ip;
	if( sec==0.0f ){
		ip = false;
	}else{
		ip = true;
	}
	mpModel->AttachMotion(pModelMotion,-1,0,false);
	mpModel->PlayMotion  (0.0f,ip,60.0f * sec);
	return true;
}

bool WK7PlayRenderObj::PlayMatMotion(WK7PlayMotion *pMotion)
{
	if( StopMatMotion()==false ){
		return false;
	}
	if( mpModel==NULL || pMotion==NULL ){
		return false;
	}
	WK7ModelMotion		*pModelMotion = pMotion->GetModelMotion();
	if( pModelMotion==NULL ){
		return false;
	}
	mpModel->AttachMatMotion(pModelMotion);
	mpModel->PlayMatMotion  ();
	return true;
}

bool WK7PlayRenderObj::PlayShapeMotion(WK7PlayMotion *pMotion)
{
	if( StopShapeMotion()==false ){
		return false;
	}
	if( mpModel==NULL || pMotion==NULL ){
		return false;
	}
	WK7ModelMotion		*pModelMotion = pMotion->GetModelMotion();
	if( pModelMotion==NULL ){
		return false;
	}
	mpModel->AttachShapeMotion(pModelMotion);
	mpModel->PlayShapeMotion  ();
	return true;
}

bool WK7PlayRenderObj::StopMotion(void)
{
	if( mpModel==NULL || mpCurrentMatMotion==NULL ){
		return true;
	}
	mpModel->StopMotion  ();
	mpModel->DetachMotion();
	delete mpCurrentMotion;
	mpCurrentMotion = NULL;
	return true;
}

bool WK7PlayRenderObj::StopMatMotion(void)
{
	if( mpModel==NULL || mpCurrentMatMotion==NULL ){
		return true;
	}
	mpModel->StopMatMotion  ();
	mpModel->DetachMatMotion();
	delete mpCurrentMatMotion;
	mpCurrentMatMotion = NULL;
	return true;
}

bool WK7PlayRenderObj::StopShapeMotion(void)
{
	if( mpModel==NULL || mpCurrentShapeMotion==NULL ){
		return true;
	}
	mpModel->StopShapeMotion  ();
	mpModel->DetachShapeMotion();
	delete mpCurrentShapeMotion;
	mpCurrentShapeMotion = NULL;
	return true;
}

bool WK7PlayRenderObj::StopMotionAll(void)
{
	bool	result = true;
//	if( mpModel->StopMotion()==false ){
//		result = false;
//	}
//	if( mpModel->StopMatMotion()==false ){
//		result = false;
//	}
//	if( mpModel->StopShapeMotion()==false ){
//		result = false;
//	}
	if( mpModel->StopMotionAll()==false ){
		result = false;
	}
	return result;
}

WK7PlayLight::WK7PlayLight()
{
	mpLight         = NULL;
	mpCurrentMotion = NULL;
}

WK7PlayLight::~WK7PlayLight()
{
	GHM_SAFE_DELETE(mpCurrentMotion);
}

bool WK7PlayLight::SetLight(WK7Light *pLight)
{
	if( mpLight!=NULL && StopMotion()==false ){
		return false;
	}
	mpLight = pLight;
	return true;
}

bool WK7PlayLight::PlayMotion(WK7PlayMotion *pMotion)
{
	if( StopMotion()==false ){
		return false;
	}
	if( mpLight==NULL || pMotion==NULL ){
		return false;
	}
	WK7LightMotion		*pLightMotion = pMotion->GetLightMotion();
	if( pLightMotion==NULL ){
		return false;
	}
	mpLight->AttachMotion(pLightMotion);
	mpLight->PlayMotion  ();
	return true;
}

bool WK7PlayLight::StopMotion(void)
{
	if( mpLight==NULL || mpCurrentMotion==NULL ){
		return true;
	}
	mpLight->StopMotion  ();
	mpLight->DetachMotion();
	delete mpCurrentMotion;
	mpCurrentMotion = NULL;
	return true;
}

bool WK7PlayLight::WaitMotion(void)
{
	if( mpLight==NULL ){
		return true;
	}
	while( mpLight->GetMotionState() != AOBJ_NO_ANIM ){
		GHMGCTASK_RETSYS();
	}
	return true;
}

bool WK7PlayLight::SetMotionLoop(bool flag)
{
	if( mpLight==NULL ){
		return false;
	}
	mpLight->SetMotionLoop(flag);
	return true;
}

WK7Script::ObjectType	objectTypeTable[] = {
	{ WK7Play::ScriptObjectCallMethod  , 0 , 0 , 0 , 0 , 0 },
	WK7SCRPIT_OBJECTTYPE_TABLE_END,
};

bool WK7Play::SetExceptPlayRenderObj(bool flag)
{
	WK7Chara::SetRenderChara(flag);
	WK7Stage::SetRenderStage(flag);
	WK7Item::SetRenderItem(flag);
	return true;
}

WK7Play::WK7Play() : ghmGcListObj(&mAnchor,GHMGCLIST_DEFAULT_PRIO)
{
	mSsmCharaNo              = 0xffffffff;
	mPlayID                  = GetUniquePlayID();
	mPlayNo                  = 0xffffffff;
	mState                   = IDLE;
	mpFile                   = NULL;
	mpData                   = NULL;
	mpResLink                = NULL;
	mTaskID                  = -1;
	mpScript                 = NULL;
	mppModel                 = NULL;
	mModelNum                = 0;
	mppModelMotion           = NULL;
	mModelMotionNum          = 0;
	mppModelShapeMotion      = NULL;
	mModelShapeMotionNum     = 0;
	mppCameraMotion          = NULL;
	mCameraMotionNum         = 0;
	mppLightMotion           = NULL;
	mLightMotionNum          = 0;
	mppRipShapeMotion        = NULL;
	mRipShapeMotionNum       = 0;
	mppSepMotionPack         = NULL;
	mSepMotionPackNum        = 0;
	mPlaySepMotionPack       = -1;
	mpSepMotionPackBuffer[0] = 
	mpSepMotionPackBuffer[1] = NULL;
	mFlag                    = 0;
	mpJimakuData             = NULL;
	mpTelopData              = NULL;
	mpCharaNameData          = NULL;
	mpNextPlayMotion         = NULL;
	mpNextLoadMotion         = NULL;
	mLoadRequestMotionNum    = 5;
	mpCurrentCameraMotion    = NULL;
	mpPlayEffect             = NULL;
	mPlayEffectID            = 0xffffffff;
	memset(mResourceName,0,WK7PLAY_RESOURCE_NAME_MAX);
	memset(mResourceDir ,0,WK7PLAY_RESOURCE_NAME_MAX);
	memset(&mCamStartPosition      ,0,sizeof(mCamStartPosition      ));
	memset(&mCamEndPosition        ,0,sizeof(mCamEndPosition        ));
	memset(&mCamStartTargetPosition,0,sizeof(mCamStartTargetPosition));
	memset(&mCamEndTargetPosition  ,0,sizeof(mCamEndTargetPosition  ));
	mEffectInfo.Initialize();
	mPrepareSec = 0.0f;
	mPlayCount = 0;
	mPlayFrame = 0;
	mCinemaScopeColor = 0;
	mKeepPlayerSsm = 0;
	if( mDontReleasePlayerSsm==true ){
		mKeepPlayerSsm = 1;
		mDontReleasePlayerSsm = false;
	}
}

WK7Play::~WK7Play()
{
	Release();
	if( WK7Play::mpCurrentPlay==this ){
		WK7Play::mpCurrentPlay = NULL;
	}
}

WK7PlayTable* WK7Play::GetTableTop(void)
{
	return playTable;
}

WK7PlayTable* WK7Play::GetTable(u32 no)
{
	WK7PlayTable	*pTable;
	no &= K7EV_ID_MASK;
	for(pTable=playTable ; pTable->mPlayNo!=WK7PLAYTBL_TERM_NO ; pTable++){
		if( pTable->mPlayNo==no ){
			return pTable;
		}
	}
	return NULL;
}

int WK7Play::GetTableIndex(u32 no)
{
	WK7PlayTable	*pTable;
	int				i;
	no &= K7EV_ID_MASK;
	for(pTable=playTable,i=0 ; pTable->mPlayNo!=WK7PLAYTBL_TERM_NO ; pTable++,i++){
		if( pTable->mPlayNo==no ){
			return i;
		}
	}
	return -1;
}

bool WK7Play::SetPlayed(u32 no,bool flag)
{
	int		id = GetTableIndex(no);
	if( id < 0 || id >=WK7PLAYTBL_MAX ){
		return false;
	}
	u32		bit = WK7PLAYTBL_WORKBIT(id);
	u32		fid = WK7PLAYTBL_WORKID (id);
	if( flag==true ){
		mPlayedFlag[fid] |=  bit;
	}else{
		mPlayedFlag[fid] &= ~bit;
	}
	return true;
}

bool WK7Play::HasPlayed(u32 no)
{
	int		id = GetTableIndex(no);
	if( id < 0 || id >=WK7PLAYTBL_MAX ){
		return false;
	}
	u32		bit = WK7PLAYTBL_WORKBIT(id);
	u32		fid  = WK7PLAYTBL_WORKID (id);
	if( mPlayedFlag[fid] & bit ){
		return true;
	}
	return false;
}

void WK7Play::ClearCaptureRenderProc(void)
{
	screenImageFullColor.SetPreRender (false);
	screenImageFullColor.SetRender    (false);
	screenImageFullColor.SetCopyEnable(true );
	screenImageMonoColor.SetPreRender (false);
	screenImageMonoColor.SetRender    (false);
	screenImageMonoColor.SetCopyEnable(true );
}

WK7PlayMotion* WK7Play::GetNextPlayMotion(void)
{
	if( mpNextPlayMotion==NULL ){
		return NULL;
	}
	WK7PlayMotion	*pCurrentMotion = mpNextPlayMotion;
	mpNextPlayMotion = mpNextPlayMotion->GetNext();
	return pCurrentMotion;
}

bool WK7Play::PlayMotionProcess(void)
{
//	if( !mInitialized ){
//		return true;
//	}
//	
//	WK7PlayMotion	*pMotion;
//	int				loadMotionNum = mLoadRequestMotionNum - (GetPlayMotionNum() - GetIdlePlayMotionNum());
//	
//	if( loadMotionNum > 0 ){
//		pMotion = mpNextLoadMotion;
//		while( pMotion!=NULL ){
//			if( pMotion->LoadStart()==false ){
//				break;
//			}
//			if( --loadMotionNum <= 0 ){
//				break;
//			}
//			pMotion = pMotion->GetNext();
//			mpNextLoadMotion = pMotion;
//		}
//	}
//	
//	for(pMotion=(WK7PlayMotion *)mMotionAnchor.GetNext() ; pMotion!=NULL ; pMotion=pMotion->GetNext()){
//		pMotion->FrameProcess();
//	}
	return true;
}

int WK7Play::GetPlayMotionNum(void)
{
	WK7PlayMotion	*pPlayMotion;
	int				i;
	for(i=0,pPlayMotion=(WK7PlayMotion *)mMotionAnchor.GetNext() ; pPlayMotion!=NULL ; i++,pPlayMotion = pPlayMotion->GetNext()){}
	return i;
}

int WK7Play::GetIdlePlayMotionNum(void)
{
	WK7PlayMotion	*pPlayMotion;
	int				i;
	for(i=0,pPlayMotion=(WK7PlayMotion *)mMotionAnchor.GetNext() ; pPlayMotion!=NULL ; pPlayMotion=pPlayMotion->GetNext()){
		if( pPlayMotion->GetState()==WK7PlayMotion::IDLE ){
			i++;
		}
	}
	return i;
}

bool WK7Play::SetPause(void)
{
	WK7Light			*pLight;
	WK7PlayRenderObj	*pRenderObj;
	WK7Model			*pModel;
	
	WK7CAM->StopMotion();
	
	if( (pRenderObj=GetRenderObj(0))!=NULL && (pModel=pRenderObj->GetModel())!=NULL ){
		pModel->StopMotionAll();
	}
	
	gpJimaku  ->mSetPause();
	gpEngTelop->mSetPause();
	
	for(pLight=WK7Light::GetTop() ; pLight!=NULL ; pLight=pLight->GetNext()){
		pLight->StopMotion();
	}
	
	// Xg[|[Y
//	gpSound   ->mSetPauseStream();
	return true;
}

bool WK7Play::ReleasePause(void)
{
	WK7Light			*pLight;
	WK7PlayRenderObj	*pRenderObj;
	WK7Model			*pModel;
	
	WK7CAM->ReplayMotion();
	
	if( (pRenderObj=GetRenderObj(0))!=NULL && (pModel=pRenderObj->GetModel())!=NULL ){
		pModel->ReplayMotionAll();
	}
	
	gpJimaku  ->mReleasePause();
	gpEngTelop->mReleasePause();
	
	for(pLight=WK7Light::GetTop() ; pLight!=NULL ; pLight=pLight->GetNext()){
		pLight->ReplayMotion();
	}
	
	// Xg[|[Y
//	gpSound   ->mReleasePauseStream();
	gpSound->mJustPlayStream(STREAM_PAUSE );
	
	return true;
}

//#define	FIRSTFRAME_CAPTURE
int WK7Play::PlaySepMotionPack(void)
{
	WK7PlaySepMotionPack		*pPlaySepMotionPack = GetPlaySepMotionPack();
	
	// ebvꎞ~
	gpJimaku  ->mSetPause();
	gpEngTelop->mSetPause();
	
	if( pPlaySepMotionPack==NULL ){
		// ŏ
		if( mPlaySepMotionPack >= (int)mSepMotionPackNum ){
			return -1;
		}
		mPlaySepMotionPack++;
		if( (pPlaySepMotionPack = GetPlaySepMotionPack())==NULL ){
			return -1;
		}
	}
	
	if( pPlaySepMotionPack->GetState()!=WK7PlaySepMotionPack::LOADED ){
		return 0;
	}
	
	WK7PlaySepMotion			*pPlaySepMotion;
	while( (pPlaySepMotion=pPlaySepMotionPack->GetNextSepMotion())==NULL ){
		WK7PlaySepMotionPack		*pPrevPlaySepMotionPack = pPlaySepMotionPack;
		pPlaySepMotionPack = GetNextSepMotionPack();
		if( pPlaySepMotionPack==NULL ){
			if( pPrevPlaySepMotionPack != NULL ){
				pPrevPlaySepMotionPack->Release();
			}
			return -1;
		}
		if( pPlaySepMotionPack->GetState()!=WK7PlaySepMotionPack::LOADED ){
			return 0;
		}
		if( pPrevPlaySepMotionPack != NULL ){
			pPrevPlaySepMotionPack->Release();
		}
		mPlaySepMotionPack++;
	}
	
	WK7PlayRenderObj	*pRenderObj = GetRenderObj(0);
	WK7Model			*pModel;
	
	if( pRenderObj==NULL || (pModel=pRenderObj->GetModel())==NULL ){
		return 1;
	}
	
	f32		startframe;
//	if( pPlaySepMotionPack->GetPlayCount() < 0 ){
		startframe = 0.0f;
//	}else{
//		startframe = 2.0f;
//	}
	
	// [VA^b`
	WK7ModelMotion	*pModelMotion;
	WK7CameraMotion	*pCameraMotion;
	WK7LightMotion	*pLightMotion;
	pModel->DetachMotionAll();
	if( (pModelMotion = pPlaySepMotion->GetModelMotion())!=NULL ){
		pModel->AttachMotion(pModelMotion);
	}
	mEffectInfo.mpMotionModelCamera = pModelMotion;
	if( (pModelMotion = pPlaySepMotion->GetMatMotion())!=NULL ){
		pModel->AttachMatMotion(pModelMotion);
	}
	mEffectInfo.mpMotionMat = pModelMotion;
	if( (pModelMotion = pPlaySepMotion->GetShapeMotion())!=NULL ){
		pModel->AttachMotion     (pModelMotion,true);
		pModel->AttachShapeMotion(pModelMotion);
	}
	mEffectInfo.mpMotionShape = pModelMotion;
	if( (pModelMotion = pPlaySepMotion->GetEffectMotion())!=NULL ){
		pModel->AttachMotion     (pModelMotion,true);
		pModel->AttachShapeMotion(pModelMotion,true);
	}
	if( (pLightMotion = pPlaySepMotion->GetLightMotion())!=NULL ){
		int				i,num = pLightMotion->GetAnimNum();
		const char		*pLightName;
		WK7Light		*pLight;
		HSD_LightAnim	*pLightAnim;
		for(i=0 ; i<num ; i++){
			pLightName = pLightMotion->GetAnimName(i);
			if( (pLightAnim = pLightMotion->GetAnim(i))==NULL ){
				continue;
			}
			if( (pLight=WK7Light::Find(pLightName))==NULL ){
				continue;
			}
			pLight->AttachMotion (pLightAnim);
			pLight->PlayMotion   (startframe);
			pLight->SetMotionLoop(false);
#ifdef	FIRSTFRAME_CAPTURE
			pLight->StopMotion   ();
#endif
		}
	}
	mEffectInfo.mpMotionEffect = pModelMotion;
	mEffectInfo.mpMotionLight  = pLightMotion;
	WK7CAM->DetachMotion();
static	Vec		zero = { 0.0f,0.0f,0.0f };
	WK7CAM->SetRelativePosition(zero);
	WK7CAM->SetRelativeRotation(zero);
	if( (pCameraMotion = pPlaySepMotion->GetCameraMotion())!=NULL ){
		WK7CAM->AttachMotion (pCameraMotion);
		WK7CAM->PlayMotion   (startframe);
		WK7CAM->SetMotionLoop(false);
	}
	mEffectInfo.mpMotionCamera = pCameraMotion;
	
	// [VĐ
	pModel->PlayMotion     (startframe);
	pModel->PlayMatMotion  (startframe);
	pModel->PlayShapeMotion(startframe);
	pModel->SetMotionLoop  (false);
	
#ifdef	_DEBUG
	{
		f32		totalframe;
		char	str[256];
		totalframe = pModel->GetTotalMotionFrame();
		sprintf(str,"PlayMotion -> %.0f\n",totalframe);
		OSDebugReport(str);
	}
#endif
	
	if( pPlaySepMotionPack->GetPlayCount() < 0 ){
		// [VpbN̍ŏ̃[VĐ
		// Xg[Đ
//		gpSound->mJustPlayStream(STREAM_PAUSE );
		if( mPlayCountFlag ){
			gpSound->mJustPlayStream(STREAM_PAUSE );
		}
	}
	
#ifdef	FIRSTFRAME_CAPTURE
	WK7CAM->StopMotion();
	pModel->StopMotionAll();
#endif
	
	// ebv|[Y
#ifndef	FIRSTFRAME_CAPTURE
	gpJimaku  ->mReleasePause();
	gpEngTelop->mReleasePause();
	
	// [VpbN̍ĐJE^CNg
	pPlaySepMotionPack->IncPlayCount();
#endif
	
	if( !mPlayCountFlag ){
		SetPause();
	}
	
	return 1;
}

bool WK7Play::PrepareStream(void)
{
	WK7PlaySepMotionPack	*pNextSepMotionPack = GetNextSepMotionPack();
	mPrepareSec = 0.0f;
	mPrepareStream = 1;
	if( pNextSepMotionPack==NULL || pNextSepMotionPack->GetPackName()==NULL ){
		return false;
	}
	char					hpsname[WK7PLAY_RESOURCE_NAME_MAX];
	char					tmpname[WK7PLAY_RESOURCE_NAME_MAX];
	u8						vol;
	GetFileName(tmpname,false);
	strcpy(hpsname,"/sound/CharaGeki/");
	strcat(hpsname,tmpname);
	strcat(hpsname,pNextSepMotionPack->GetPackName());
	strcat(hpsname,"00.hps");
	switch( mPlayNo ){
	case	0x05500101:
		vol = 255;
		break;
	default:
		vol = WK7PLAY_VOLUME_MUSIC;
		break;
	}
	gpSound->mPrepareStream(hpsname,vol);
	gpSound->mSetPitchStream(-2);
	return true;
}

bool WK7Play::PrepareNextSepMotionPack(void)
{
	WK7PlaySepMotionPack	*pNextSepMotionPack = GetNextSepMotionPack();
	if( pNextSepMotionPack!=NULL && pNextSepMotionPack->GetState()==WK7PlaySepMotionPack::IDLE ){
		// Xg[\
		void		*pBuff;
		if( (pBuff=mpSepMotionPackBuffer[(mPlaySepMotionPack + 1) & 1])==NULL ){
			return false;
		}
		PrepareStream();
		if( pNextSepMotionPack->LoadStart(pBuff)==false ){
			return false;
		}
	}
	return true;
}

bool WK7Play::IsPreparedNextSepMotionPack(void)
{
	if( mPrepareStream ){
		if( mPrepareSec < 1.0f ){
			return false;
		}
		mPrepareStream = 0;
	}
	WK7PlaySepMotionPack		*pNextSepMotionPack = GetNextSepMotionPack();
	if( pNextSepMotionPack==NULL ){
		return false;
	}
	if( pNextSepMotionPack->GetState()!=WK7PlaySepMotionPack::LOADED ){
		return false;
	}
	return true;
}

bool WK7Play::SepMotionPackProcess(void)
{
	
	mPrepareSec += (f32)WK7_FRAME_TO_SEC(1.0f);
	
	WK7PlaySepMotionPack	**ppPlayMotionPack;
	int						i;
	
	for(i=0,ppPlayMotionPack=mppSepMotionPack ; i<mSepMotionPackNum ; i++,ppPlayMotionPack++){
		if( (*ppPlayMotionPack)==NULL ){
			continue;
		}
		(*ppPlayMotionPack)->FrameProcess();
	}
	
	WK7PlayRenderObj	*pRenderObj = GetRenderObj(0);
	WK7Model			*pModel;
	if( pRenderObj!=NULL && (pModel=pRenderObj->GetModel())==NULL ){
		mSepPlayEnd = 1;
		return true;
	}
	
	if( mState==PLAY ){
		if( mPlayCountFlag ){
			mPlayCount += WK7_FRAME_COUNT;
		}
		mPlayFrame++;
	}
	
	if( mState==PLAY && !mSepPlayEnd ){
		
		// ̃[VpbN\
		PrepareNextSepMotionPack();
		
		if( pModel->GetMotionState()==WK7MODEL_NOANIM ){
			// [Vǂ݊Ă玟̃[VĐ
			switch( PlaySepMotionPack() ){
			case	0:
				break;
			case	1:
				pRenderObj->SetRenderFlag(true);
				if( mpPlayEffect!=NULL && CheckEffectValidState(mpPlayEffect,mPlayEffectID)==TRUE ){
					::SetEffectDrawFlagAll(mpPlayEffect,true);
					WK7RenderObjEffectInfo::UnParent(mpPlayEffect);
					::DeleteEffect(mpPlayEffect);
					mpPlayEffect = NULL;
					mPlayEffectID = 0xffffffff;
				}
//				::SetEffectAllDrawFlag(true);
				mFirstPlay = 1;
//				mPlayCountFlag = 1;
//				mPlayCount = 0;
				//e𗎂Ƃ̐ݒ
				switch( mPlayNo ){
				case	0x00020711:
				case	0x05120101:
					WK7SHADOW->SetViewDistManual( true );
					WK7SHADOW->SetViewDist( WK7SHADOW_VIEWDIST_DEF*4 );
					break;
				default:
					break;
				}
				break;
			default:
				// ĐI
//				pRenderObj->SetRenderFlag(false);
				mSepPlayEnd = 1;
				mEffectInfo.mpMotionModelCamera = NULL;
				mEffectInfo.mpMotionMat         = NULL;
				mEffectInfo.mpMotionShape       = NULL;
				mEffectInfo.mpMotionEffect      = NULL;
				mEffectInfo.mpMotionCamera      = NULL;
				mEffectInfo.mpMotionLight       = NULL;
				break;
			}
		}
		
	}
	
	if( mState==PLAY ){
		FrameProcessPlayEffect();
	}
	
	return true;
}

WK7PlaySepMotionPack* WK7Play::GetSepMotionPack(int index)
{
	if( mppSepMotionPack==NULL ){
		return NULL;
	}
	if( index >= mSepMotionPackNum ){
		return NULL;
	}
	return mppSepMotionPack[index];
}

WK7PlaySepMotionPack* WK7Play::GetPlaySepMotionPack(void)
{
	return GetSepMotionPack(mPlaySepMotionPack);
}

WK7PlaySepMotionPack* WK7Play::GetNextSepMotionPack(void)
{
	return GetSepMotionPack(mPlaySepMotionPack + 1);
}

bool WK7Play::Initialize(void)
{
	int		i;
	mTaskController.Initialize();
	if( GameInfo.GetContinue()==false ){
		for(i=0 ; i<WK7PLAYTBL_WORKSIZE ; i++){
			mPlayedFlag[i] = 0;
		}
	}
	return true;
}

bool WK7Play::Terminate(void)
{
	ReleaseAll();
	mTaskController.Terminate();
	return true;
}

bool WK7Play::ReleaseAll(void)
{
	WK7Play		*pPlay = (WK7Play *)mAnchor.GetNext();
	WK7Play		*pNext;
	while( pPlay!=NULL ){
		pNext = pPlay->GetNext();
		delete pPlay;
		pPlay = pNext;
	}
	return true;
}

bool WK7Play::Prepare(void)
{
	mPrepareRequest = 1;
	return true;
}

bool WK7Play::Start(void)
{
	mStartRequest   = 1;
	mPrepareRequest = 1;
	if( mPlayNo!=0xffffffff ){
		SetPlayed(mPlayNo,true);
	}
	return true;
}

bool WK7Play::SetupProcess(void)
{
	void	*pScriptData;
	
	// Cxgpg`t@Cǂݍ݊҂
	//while( gpSound->mGetProcess()!=eSndStandby ){
	//while( gpSound->mGetReadEventSeProc()!=eSndReadReqWaiting ){
	//	if( RetSys()==false ){
	//		return true;
	//	}
	//}
	
//	// epCg擾
//	WK7Light			*pLight;
//	WK7LightFindData	data;
//	HSD_LObj			*pLObj;
//	data.Initialize(WK7PLAY_TYPE_RES_MODEL(mPlayID));
//	while( (pLight=WK7Light::Find(data))!=NULL ){
//		pLObj = pLight->GetHSDLObj();
//		switch( HSD_LObjGetType(pLObj) ){
//		case	LOBJ_INFINITE:
//			break;
//		case	LOBJ_POINT:
//		case	LOBJ_SPOT:
//		case	LOBJ_AMBIENT:
//		default:
//			continue;
//		}
//		break;
//	}
//	if( pLight!=NULL ){
//		WK7SHADOW->SetLight(pLight->GetHSDLObj());
//	}
	
	// GtFNgZbgAbv
	WK7PlayEffect	*pPlayEffect;
	for(pPlayEffect=(WK7PlayEffect *)mPlayEffectAnchor.GetNext() ; pPlayEffect!=NULL ; pPlayEffect=pPlayEffect->GetNext()){
		while( pPlayEffect->CheckSetup()==false ){
#ifdef	DEBUGPRINT
			C_Paper		paper(32,240);
			PRINTF("new Effect Setup !!");
#endif
			if( RetSys()==false ){
				return true;
			}
		}
	}
	
	if( mpScript==NULL ||
	    mpResLink->IsGroup(0)==true ||
	    (pScriptData=mpResLink->GetResource(0))==NULL ){
		return false;
	}
	
	mpScript->Reset();
	mpScript->LoadData(pScriptData);
	
	mInitialized = 1;
	
	return true;
}

bool WK7Play::RetSys(void)
{
	if( mTermRequest ){
		return false;
	}
	GHMGCTASK_RETSYS();
	return true;
}

bool WK7Play::ScriptParse(void)
{
	SetupProcess();
	
	while( !mTermRequest ){
		if( mState!=PLAY ){
			GHMGCTASK_RETSYS();
			continue;
		}
		if( mReleasePause ){
			if( !mpScript->Parse() ){
				break;
			}
		}else{
			GHMGCTASK_RETSYS();
		}
	}
	
	while( !mSepPlayEnd && !mTermRequest ){
		GHMGCTASK_RETSYS();
	}
	GHMGCTASK_RETSYS();
	
	// mNȂ
	switch( mPlayNo ){
	case	0x00010101:
	case	0x00020101:
	case	0x00020103:
	case	0x00020203:
	case	0x00020301:
	case	0x00020502:
	case	0x00020702:
	case	0x00020703:
	case	0x00020707:
	case	0x00020709:
	case	0x00020711:
	case	0x01020101:
	case	0x01020201:
	case	0x01020203:
	case	0x01050101:
	case	0x01050102:
	case	0x01050203:
	case	0x01050204:
	case	0x01050206:
	case	0x01070210:
	case	0x01100101:
	case	0x01500101:
	case	0x01500201:
	case	0x02010101:
	case	0x02020101:
	case	0x02020201:
	case	0x02020203:
	case	0x02020102:
	case	0x03010101:
	case	0x03020101:
	case	0x03020201:
	case	0x03020203:
	case	0x03040103:
	case	0x03040105:
	case	0x03050107:
	case	0x03080402:
	case	0x03080403:
	case	0x05020101:
	case	0x05020201:
	case	0x05020302:
	case	0x05040101:
	case	0x05040201:
	case	0x05080102:
	case	0x05080202:
	case	0x05090101:
	case	0x05090102:
	case	0x05090202:
	case	0x05090302:
	case	0x05090402:
	case	0x05090502:
	case	0x05090702:
	case	0x05090902:
	case	0x05100101:
	case	0x05100103:
	case	0x05100105:
	case	0x05100106:
	case	0x05120101:
	case	0x05500101:
	case	0x05500201:
	case	0x05500301:
	case	0x99010101:
		break;
	
	// mN
	default:
		if( !mTermRequest ){
			
			int		time = (int)WK7_SEC_TO_TICK(1);
			bool	full = screenImageMonoColor.GetFullScreen();
			
			screenImageMonoColor.SetRender    (true );
			screenImageMonoColor.SetCopyEnable(false);
			screenImageMonoColor.SetTexFilter (GX_NEAR,GX_NEAR);
			screenImageMonoColor.SetFullScreen(true);
			
			while( !mTermRequest && time > 0 ){
				time -= WK7_TICK_PER_FRAME;
				GHMGCTASK_RETSYS();
			}
			
			u32		fadetick = WK7_SEC_TO_TICK(2.5f);
#if 0
			k7app.SetFadeColor(0x00000000);
			k7app.FadeStart(0xff,fadetick);
#else
			screenImageMonoColor.SetR(0x00,fadetick);
			screenImageMonoColor.SetG(0x00,fadetick);
			screenImageMonoColor.SetB(0x00,fadetick);
#endif
			
			while( screenImageMonoColor.IsAnimatingR()==true ){
				GHMGCTASK_RETSYS();
			}
			while( k7app.IsFading()==true ){
				GHMGCTASK_RETSYS();
			}
			
			SetVisibleAll(false);
			GHMGCTASK_RETSYS();
#if 0
			k7app.FadeStart(0x00,0);
#else
			screenImageMonoColor.SetR(0xff);
			screenImageMonoColor.SetG(0xff);
			screenImageMonoColor.SetB(0xff);
#endif
			WK7CAM->SetCinemaScopeMode(false);
			screenImageMonoColor.SetRender    (false);
			screenImageMonoColor.SetCopyEnable(true );
			screenImageMonoColor.SetFullScreen(full);
			
			if( mPlayNo==0x03050109 ){
				// pb`
				
				TerminatePlayEffect();
				// LGtFNgpݒ
				::SetCharGekiPlayEffectInfo(NULL);
			}
		}
		break;
	}
	
	// ҂
	switch( mPlayNo ){
	case	0x00020709:
		if( !mTermRequest ){
			while( gpSound->mStreamCheckPlay()==true ){
#ifdef	_DEBUG
				if( gpSound->mGetSoundDebug()==true ){
					break;
				}
#endif
				GHMGCTASK_RETSYS();
			}
		}
		break;
	default:
		break;
	}
	
	//e𗎂Ƃ̃}jA
	WK7SHADOW->SetViewDistManual( false );
	
	// J̃[lftHg
	WK7CAM->SetRoll(0.0f);
	
	return true;
}

bool WK7Play::Release(void)
{
	if( mTaskID >= 0 ){
		mTaskController.Stop(mTaskID);
		mTaskID  = -1;
	}
	
	if( mpPlayEffect!=NULL && CheckEffectValidState(mpPlayEffect,mPlayEffectID)==TRUE ){
		WK7RenderObjEffectInfo::UnParent(mpPlayEffect);
		::DeleteEffect(mpPlayEffect);
		mpPlayEffect  = NULL;
		mPlayEffectID = 0xffffffff;
	}
	
	ReleaseSsm();
	RestorePlayerSsm();
	
	TerminatePlayEffect();
	::SetCharGekiPlayEffectInfo(NULL);
	TermResource();
	CameraStopSeqMotion();
	
	WK7SHADOW->SetInitMinRadius();
	
	GHM_SAFE_DELETE(mpFile);
	
	u32		hopper7 = mHopper7;
	mState               = IDLE;
	mFlag                = 0;
	mHopper7             = hopper7;
	memset(mResourceName,0,WK7PLAY_RESOURCE_NAME_MAX);
//	WK7SHADOW->SetLight(NULL);
	mEffectInfo.Initialize();
	mPrepareSec = 0.0f;
	
	return true;
}

bool WK7Play::InitScript(void)
{
	ghmGcMemSetAllocName("WK7Script");
	if( (mpScript = new WK7Script(objectTypeTable,K7SCRIPT_LOCAL_FLAG_NUM))==NULL ){
		return false;
	}
	mpScript->SetParam(0,(u32)this);
	return true;
}

bool WK7Play::GetDirName(const char *pfname,char *pReturnDirName)
{
	if( pfname==NULL || pReturnDirName==NULL ){
		return false;
	}
	int		len = (int)strlen(pfname);
	int		count;
	if( len <= 0 ){
		*pReturnDirName = '\0';
		return true;
	}
	
	for(count=len-1 ;count>=0 ; count--){
		
		if( pfname[count]=='/' ){
			
			if( count > WK7PLAY_RESOURCE_NAME_MAX-1 ){
				*pReturnDirName = '\0';
				return false;
			}
			
			count++;
			memcpy(pReturnDirName,pfname,(u32)count);
			pReturnDirName[count] = '\0';
			
			return true;
		}
		
	}
	
	*pReturnDirName = '\0';
	return true;
}

bool WK7Play::GetFileName(char *pReturnFileName,bool dir)
{
	if( pReturnFileName==NULL ){
		return false;
	}
	const char	*pPeriod    = NULL;
	const char	*pLastSlash = NULL;
	const char	*pName;
	u32			len = strlen(mResourceName);
	int			i;
	for(i=(int)len,pName=mResourceName+(i-1) ; i>=0 ; i--,pName--){
		switch( *pName ){
		case	'/':
			if( pLastSlash==NULL ){
				pLastSlash = pName;
			}
			break;
		case	'.':
			if( pPeriod==NULL ){
				pPeriod = pName;
			}
			break;
		}
	}
	u32		size = 0;
	if( pPeriod!=NULL ){
		size = (u32)pPeriod - (u32)mResourceName;
	}else{
		size = len;
	}
	if( dir==true ){
		i = 0;
	}else{
		if( pLastSlash!=NULL ){
			i = (int)((u32)pLastSlash - (u32)mResourceName) + 1;
		}else{
			i = 0;
		}
	}
	size -= i;
	pReturnFileName[size] = '\0';
	memcpy(pReturnFileName,mResourceName + i,size);
	return true;
}

bool WK7Play::GetFileName(const char *pfname,char *pReturnFileName,bool hopper7)
{
	if( pfname==NULL || pReturnFileName==NULL ){
		return false;
	}
	if( strchr(pfname,'/')!=NULL ){
		strncpy(pReturnFileName,pfname,WK7PLAY_RESOURCE_NAME_MAX);
		return true;
	}
	char		tmp[WK7PLAY_RESOURCE_NAME_MAX];
	char		*pPeriod;
	u32			len;
	
	pPeriod = strchr(pfname,'.');
	
	strcpy(pReturnFileName,"/CharaGeki/");
	
	if( pPeriod!=NULL ){
		len = (u32)pPeriod - (u32)pfname;
	}else{
		len = strlen(pfname);
	}
	memcpy(tmp,pfname,len);
	tmp[len] = '\0';
	
	strcat(pReturnFileName,tmp);
	strcat(pReturnFileName,"/");
	strcat(pReturnFileName,tmp);
	if( mHopper7 && hopper7==true ){
		strcat(pReturnFileName,"h");
	}
	
	if( pPeriod==NULL ){
		len = WK7PLAY_RESOURCE_NAME_MAX - strlen(pReturnFileName);
		strncat(pReturnFileName,".rsl",len);
	}
	
	return true;
}

WK7Play* WK7Play::Create(u32 no)
{
	char		fname[16];
	WK7Play		*pPlay;
	sprintf(fname,"%08x",no & K7EV_ID_MASK);
#ifdef	WK7PLAY_HOPPER7
	if( K7SCR_GET_GLOBAL_FLAG(K7GLFL_HOPPER7)==true ){
		mHopper7Mode = true;
		if( (pPlay = Create(fname))!=NULL ){
			goto create_end;
		}
		mHopper7Mode = false;
	}
#endif
	if( (pPlay = Create(fname))==NULL ){
		return NULL;
	}
create_end:
	pPlay->SetPlayNo(no & K7EV_ID_MASK);
	return pPlay;
}

WK7Play* WK7Play::Create(const char *pfname)
{
	ghmGcMemSetAllocName("WK7Play");
	WK7Play	*pPlay = new WK7Play;
	if( pPlay==NULL ){
		return NULL;
	}
		
#ifdef	WK7PLAY_HOPPER7
	if( mHopper7Mode==true ){
		pPlay->mHopper7 = 1;
	}else{
		pPlay->mHopper7 = 0;
	}
#endif
	if( pPlay->Load(pfname)==false ){
		GHM_SAFE_DELETE(pPlay);
		return NULL;
	}
	return pPlay;
}

WK7Play* WK7Play::CreateB(const char *pfname)
{
	ghmGcMemSetAllocName("WK7Play");
	WK7Play	*pPlay = new WK7Play;
	if( pPlay==NULL ){
		return NULL;
	}
	if( pPlay->LoadB(pfname)==false ){
		GHM_SAFE_DELETE(pPlay);
		return NULL;
	}
	return pPlay;
}

static bool		mStoredShadowEnableAdjust = true;
bool WK7Play::StorePlayTB(bool render)
{
	mStoredBgmID  = gpSound->mGetPlayStreamNo();
	mPlayTBStored = true;
	mCamWobble = WK7CAM->GetWobbleEnable();
	mStoredShadowEnableAdjust = WK7SHADOW->GetEnableAdjust();
	WK7CAM->StoreState();
	WK7CAM->SetNear   (0.1f);
//	WK7CAM->SetFar    (30000.0f);
	WK7CAM->SetFar    (100000.0f);
	if( render==true ){
		WK7Chara::SetRenderChara(false);
		WK7Stage::SetRenderStage(false);
		WK7Item::SetRenderItem(false);
	}
	return true;
}

WK7Play* WK7Play::Create(void *pResource,const char *pResourceName)
{
	ghmGcMemSetAllocName("WK7Play");
	WK7Play	*pPlay = new WK7Play;
	if( pPlay==NULL ){
		return NULL;
	}
	if( pPlay->Load(pResource,pResourceName)==false ){
		GHM_SAFE_DELETE(pPlay);
		return NULL;
	}
	return pPlay;
}

bool WK7Play::PlayTB(u32 evno,bool render)
{
	if( WK7Play::HasPlayed(evno)==true ){
		return false;
	}
	char	fname[16];
	sprintf(fname,"%08x",evno & K7EV_ID_MASK);
#ifdef	WK7PLAY_HOPPER7
	if( K7SCR_GET_GLOBAL_FLAG(K7GLFL_HOPPER7)==true ){
		mHopper7Mode = true;
		if( PlayTB(fname,render)==true ){
			goto playtb_end;
		}
		mHopper7Mode = false;
	}
#endif
	PlayTB(fname,render);
playtb_end:
	SetPlayed(evno,true);
	return true;
}

bool WK7Play::PlayTB(const char *pfname,bool render)
{
	ghmGcMemSetAllocName("WK7Play");
	WK7Play	*pPlay = new WK7Play;
	if( pPlay==NULL ){
		return false;
	}
	
#ifdef	WK7PLAY_HOPPER7
	if( mHopper7Mode==true ){
		pPlay->mHopper7 = 1;
	}else{
		pPlay->mHopper7 = 0;
	}
#endif
	if( pPlay->Load(pfname)==false ){
		GHM_SAFE_DELETE(pPlay);
		return false;
	}
	pPlay->Prepare();
	while( pPlay->GetState()!=WK7Play::PREPARED ){
		GHMGCTASK_RETSYS();
	}
	if( render==true ){
		WK7Chara::SetRenderChara(false);
		WK7Stage::SetRenderStage(false);
		WK7Item::SetRenderItem(false);
	}
	
	pPlay->Start();
	GHMGCTASK_RETSYS();
	
	while( pPlay->GetState()!=WK7Play::PLAYED ){
		GHMGCTASK_RETSYS();
	}
	
	pPlay->SetVisibleAll(false);
	GHMGCTASK_RETSYS();
	GHMGCTASK_RETSYS();
	
	GHM_SAFE_DELETE(pPlay);
	return true;
}

bool WK7Play::RestorePlayTB(bool render)
{
	if( mPlayTBStored==false ){
		return false;
	}
	mPlayTBStored = false;
	WK7CAM->RestoreState();
	WK7CAM->SetCinemaScopeMode(false);
	WK7CAM->SetWobbleEnable(mCamWobble);
	if( render==true ){
		WK7Chara::SetRenderChara(true);
		WK7Stage::SetRenderStage(true);
		WK7Item::SetRenderItem(true);
	}
	WK7Stage	*pStage;
	if( (pStage=WK7Stage::GetCurrentStage())!=NULL ){
		WK7Light	*pLight = pStage->GetShadowLight();
		if( pLight!=NULL ){
			WK7SHADOW->SetLight(pLight->GetHSDLObj());
		}else{
			WK7SHADOW->SetLight(NULL);
		}
	}else{
		WK7SHADOW->SetLight(NULL);
	}
	WK7SHADOW->SetEnableAdjust(mStoredShadowEnableAdjust);
	if( mStoredBgmID >= 0 ){
//		gpSound->mPlayStreamNo(mStoredBgmID);
		gpSound->mResumeStream();
		mStoredBgmID  = -1;
	}
	return true;
}

bool WK7Play::PlayInit(bool player,bool release)
{
	StorePlayTB();
	if( gpNoise->mGetSandFrameProc()==eSandStandby ){
		while( gpNoise->mSandRequest(eSandLongNoise,eNoiseColorBlue,NOISE_SE_NORMAL)==false ){
			GHMGCTASK_RETSYS();
		}
	}
	mPlayerRelease = player;
	if( release==true && WK7Stage::IsReleasedStageArchive()==false ){
		while( WK7Stage::ReleaseStageArchive(player)==false ){
			GHMGCTASK_RETSYS();
		}
		while( WK7Stage::IsReleasedStageArchive()==false ){
			K7SCR_RETSYS();
		}
		GHMGCTASK_RETSYS();
		GHMGCTASK_RETSYS();
	}
	return true;
}

bool WK7Play::LoadWait(u32 no)
{
	WK7Play		*pPlay = Find(no);
	if( pPlay==NULL ){
		return false;
	}
	while( pPlay->GetState()!=WK7Play::LOADED ){
		GHMGCTASK_RETSYS();
	}
	return true;
}

bool WK7Play::Prepare(u32 no)
{
	WK7Play		*pPlay = Find(no);
	if( pPlay==NULL ){
		if( (pPlay = Create(no))==NULL ){
			return false;
		}
	}
	pPlay->Prepare();
	return true;
}

bool WK7Play::PrepareWait(u32 no,bool release)
{
	WK7Play		*pPlay = Find(no);
	if( pPlay==NULL ){
		return false;
	}
	while( pPlay->GetState()!=WK7Play::PREPARED ){
		GHMGCTASK_RETSYS();
	}
	if( release==true && WK7Stage::IsReleasedStageArchive()==false ){
		while( WK7Stage::ReleaseStageArchive(mPlayerRelease)==false ){
			K7SCR_RETSYS();
		}
		while( WK7Stage::IsReleasedStageArchive()==false ){
			K7SCR_RETSYS();
		}
		GHMGCTASK_RETSYS();
		GHMGCTASK_RETSYS();
		while( gpNoise->mGetSandFrameProc()!=eSandExec ){
			GHMGCTASK_RETSYS();
		}
	}
	return true;
}

bool WK7Play::Start(u32 no,bool release)
{
	WK7Play		*pPlay = Find(no);
	if( release==true && WK7Stage::IsReleasedStageArchive()==false ){
		while( WK7Stage::ReleaseStageArchive(mPlayerRelease)==false ){
			K7SCR_RETSYS();
		}
		if( pPlay==NULL ){
			if( Prepare(no)==false || (pPlay = Find(no))==NULL ){
				return false;
			}
		}
		while( WK7Stage::IsReleasedStageArchive()==false ){
			K7SCR_RETSYS();
		}
		GHMGCTASK_RETSYS();
		GHMGCTASK_RETSYS();
	}
	if( pPlay==NULL ){
		if( Prepare(no)==false || (pPlay = Find(no))==NULL ){
			return false;
		}
	}
	if( gpNoise->mGetSandFrameProc()!=eSandStandby ){
		pPlay->Prepare();
		while( gpNoise->mGetSandFrameProc()!=eSandExec ){
			GHMGCTASK_RETSYS();
		}
		while( pPlay->GetState()!=PREPARED ){
			GHMGCTASK_RETSYS();
		}
	}
	pPlay->Start();
	if( gpNoise->mGetSandFrameProc()==eSandExec ){
		while( gpNoise->mSandRelease()==false ){
			GHMGCTASK_RETSYS();
		}
	}
	return true;
}

bool WK7Play::PlayWait(u32 no)
{
	WK7Play		*pPlay = Find(no);
	if( pPlay==NULL ){
		return false;
	}
	while( pPlay->GetState()!=WK7Play::PLAYED ){
		GHMGCTASK_RETSYS();
	}
	return true;
}

bool WK7Play::StartB(u32 no,bool release)
{
	if( Start(no,release)==false ){
		return false;
	}
	if( gpNoise->mGetSandFrameProc()==eSandExec ){
		while( gpNoise->mSandRelease()==false ){
			GHMGCTASK_RETSYS();
		}
	}
	PlayWait(no);
	return true;
}

bool WK7Play::PlayReleaseAll(void)
{
	WK7Play		*pPlay;
	for(pPlay=(WK7Play *)mAnchor.GetNext() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
		pPlay->ReleaseRequest();
	}
	return true;
}

bool WK7Play::Release(u32 no)
{
	WK7Play		*pPlay = Find(no);
	if( pPlay==NULL ){
		return false;
	}
	pPlay->ReleaseRequest();
	return true;
}

bool WK7Play::ReleaseRequest(void)
{
	mReleaseRequest = 1;
	SetState(RELEASE);
	return true;
}

bool WK7Play::PlayTerm(bool release)
{
	while( gpNoise->mSandRequest(eSandLongNoise,eNoiseColorBlue,NOISE_SE_NORMAL)==false ){
		GHMGCTASK_RETSYS();
	}
	
	PlayReleaseAll();
	
	GHMGCTASK_RETSYS();
	GHMGCTASK_RETSYS();
	
	if( release==true ){
		while( WK7Stage::RestoreStageArchive()==false ){
			GHMGCTASK_RETSYS();
		}
		
		while( WK7Stage::IsRestoredStageArchive()==false ){
			GHMGCTASK_RETSYS();
		}
	}
	
	while( gpNoise->mGetSandFrameProc()!=eSandExec ){
		GHMGCTASK_RETSYS();
	}
	
	GHMGCTASK_RETSYS();
	RestorePlayTB();
	WK7CAM->SetMode(WK7Cam::WK7CAM_PATH_FOLLOW);
	GameInfo.InitOptionWindow();
	
	while( gpNoise->mSandRelease()==false ){
		GHMGCTASK_RETSYS();
	}
	while( gpNoise->mGetSandFrameProc()!=eSandStandby ){
		GHMGCTASK_RETSYS();
	}
	
	return true;
}

WK7Play* WK7Play::Find(u32 no)
{
	WK7Play		*pPlay;
	no &= K7EV_ID_MASK;
	for(pPlay=(WK7Play *)mAnchor.GetNext() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
		if( pPlay->GetPlayNo()==no ){
			return pPlay;
		}
	}
	return NULL;
}

u32 WK7Play::GetUniquePlayID(void)
{
	WK7Play	*pPlay;
	u32		id = 0;
	do{
		for(pPlay=(WK7Play *)mAnchor.GetNext() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
			if( pPlay==this ){
				continue;
			}
			if( pPlay->mPlayID==id ){
				id++;
				break;
			}
		}
	}while( pPlay!=NULL );
	return id;
}

u32 WK7Play::GetCurrentPlayID(void)
{
	WK7Play	*pPlay;
	if( (pPlay=(WK7Play *)mAnchor.GetNext())==NULL ){
		return 0xffffffff;
	}
	return pPlay->GetPlayID();
}

bool WK7Play::SetDefaultPlayNo(void)
{
	if( mPlayNo!=0xffffffff ){
		return true;
	}
	int		len = (int)strlen(mResourceName);
	int		i;
	char	*pStr;
	u32		no = 0;
//	char	*pStr = strrchr(mResourceName,'.');
//	if( pStr==NULL ){
//		return false;
//	}
//	pStr -= 8;
	for(i=len-2 ; i>=0 ; i--){
		switch( mResourceName[i] ){
		case	'/':
		case	'\\':
			break;
		default:
			continue;
		}
		break;
	}
	if( i<0 ){
		pStr = mResourceName;
	}else{
		pStr = mResourceName + i + 1;
	}
	for(i=0 ; i<8 ; i++,pStr++){
		if( *pStr < '0' || *pStr > '9' ){
			no = 0xffffffff;
			break;
		}
		no <<= 4;
		no  += (*pStr - '0');
	}
	mPlayNo = no;
	return true;
}

bool WK7Play::GetSsmName(char *pfname)
{
	if( pfname==NULL ){
		return false;
	}
	if( mPlayNo==0xffffffff ){
		return false;
	}
	sprintf(pfname,"se%08x.ssm",mPlayNo & 0x0fffffff);
//	sprintf(pfname,"/Sound/SE/se%08x.ssm",mPlayNo & 0x0fffffff);
	return true;
}

bool WK7Play::ReadSsm(void)
{
	if( mKeepPlayerSsm ){
		return true;
	}
	char	fname[WK7PLAY_RESOURCE_NAME_MAX];
	if( GetSsmName(fname)==false ){
		return false;
	}
	gpSound->mReadSsm(fname,eBankPlayer);
	return true;
}

bool WK7Play::ReleaseSsm(void)
{
	if( mKeepPlayerSsm ){
		return true;
	}
	char	fname[WK7PLAY_RESOURCE_NAME_MAX];
	if( GetSsmName(fname)==false ){
		return false;
	}
	gpSound->mReleaseSsm(fname);
	return true;
}

bool WK7Play::ReleasePlayerSsm(void)
{
	if( mKeepPlayerSsm ){
		return true;
	}
	if( mSsmCharaNo==0xffffffff ){
		mSsmCharaNo = gpSound->mGetPcSsmNo();
		gpSound->mReleasePcSsm();
	}
	return true;
}

bool WK7Play::RestorePlayerSsm(void)
{
	if( mSsmCharaNo != 0xffffffff ){
		gpSound->mReadPcSsmReq(mSsmCharaNo);
		mSsmCharaNo  = 0xffffffff;
	}
	return true;
}

bool WK7Play::Load(void *pResource,const char *pfname)
{
	if( pResource==NULL ){
		return false;
	}
	switch( mState ){
	case	IDLE:
		break;
	default:
		return false;
	}
	Release();
	GetFileName(pfname,mResourceName,false);
	GetDirName (mResourceName,mResourceDir);
	SetDefaultPlayNo();
	mpData = pResource;
	if( InitResource(mpData)==false ){
		Release();
		return false;
	}
	ReleasePlayerSsm();
	ReadSsm();
	mState = PREPARERESOURCE;
//	mState = SSMLOADING;
//	mState = LOADED;
	return true;
}

bool WK7Play::Load(const char *pfname)
{
	switch( mState ){
	case	IDLE:
		break;
	default:
		return false;
	}
	char	fname[WK7PLAY_RESOURCE_NAME_MAX];
	Release();
	GetFileName(pfname,mResourceName,false);
	GetFileName(pfname,fname        ,true );
	GetDirName (mResourceName,mResourceDir);
	SetDefaultPlayNo();
	ghmGcMemSetAllocName("ghmGcFile");
	if( (mpFile = new ghmGcFile)==NULL ){
		return false;
	}
	if( (mpFile->Open((char *)fname))==false ){
		Release();
		return false;
	}
	if( (mpData = (ghmGcResLink *)mpFile->Read())==NULL ){
		Release();
		return false;
	}
	mResAlloc = 1;
	mState = LOADING;
	return true;
}

bool WK7Play::LoadB(const char *pfname)
{
	switch( mState ){
	case	IDLE:
		break;
	default:
		return false;
	}
	char	fname[WK7PLAY_RESOURCE_NAME_MAX];
	Release();
	GetFileName(pfname,mResourceName,false);
	GetFileName(pfname,fname        ,true );
	GetDirName (mResourceName,mResourceDir);
	SetDefaultPlayNo();
	ghmGcMemSetAllocName("ghmGcFile");
	if( (mpFile = new ghmGcFile)==NULL ){
		return false;
	}
	if( (mpFile->Open((char *)fname))==false ){
		Release();
		return false;
	}
	if( (mpData = mpFile->ReadB())==NULL ){
		Release();
		return false;
	}
	mpFile->Close();
	delete mpFile;
	mpFile = NULL;
	mResAlloc = 1;
	if( InitResource(mpData)==false ){
		Release();
		return false;
	}
	ReleasePlayerSsm();
	ReadSsm();
	mState = PREPARERESOURCE;
//	mState = SSMLOADING;
//	mState = LOADED;
	return true;
}

void WK7Play::ScriptParseHandler(ghmGcTask *pTask)
{
	WK7Play		*pPlay = (WK7Play *)pTask->GetParam(0);
	pPlay->ScriptParse();
}

void WK7Play::ScriptParseHandlerTerm(ghmGcTask *pTask)
{
	WK7Play		*pPlay = (WK7Play *)pTask->GetParam(0);
	if( mTermCaptureRenderOn==true ){
		if( mTermCaptureRenderMono==true ){
			if( mTermCapturePreRender==true ){
				screenImageMonoColor.SetPreRender(true );
			}else{
				screenImageMonoColor.SetRender   (true );
			}
			screenImageMonoColor.SetCopyEnable(false);
			screenImageMonoColor.SetTexFilter(GX_NEAR,GX_NEAR);
		}else{
			if( mTermCapturePreRender==true ){
				screenImageFullColor.SetPreRender(true );
			}else{
				screenImageFullColor.SetRender   (true );
			}
			screenImageFullColor.SetCopyEnable(false);
			screenImageFullColor.SetTexFilter(GX_NEAR,GX_NEAR);
		}
	}
	WK7CAM->DetachMotion();
	pPlay->ScriptTerm();
	pPlay->StopMotionAll();
	if( pPlay->mReleaseRequest ) {
		pPlay->SetState(RELEASE);
	}else{
		pPlay->SetState(PLAYED);
	}
}

bool WK7Play::InitCastShadowPlayRenderObj(WK7PlayRenderObj *pObj,const char **ppObjList)
{
	if( pObj==NULL || ppObjList==NULL ){
		return false;
	}
	for( ; *ppObjList!=NULL ; ppObjList++){
		InitCastShadowPlayRenderObj(pObj,*ppObjList);
	}
	return true;
}

bool WK7Play::InitCastShadowPlayRenderObj(WK7PlayRenderObj *pObj,const char *pObjName)
{
	if( pObj==NULL || pObjName==NULL ){
		return false;
	}
	WK7JObj				*pJObj;
	WK7CastShadowObj	*pCastShadowObj;
	if( (pJObj = pObj->GetJObj(pObjName))==NULL ){
		return false;
	}
	if( (pCastShadowObj = WK7SHADOW->AddCastShadowObj(pJObj))==NULL ){
		return false;
	}
	pCastShadowObj->SetRenderObj(pObj);
	return true;
}

bool WK7Play::InitReceiveShadowPlayRenderObj(WK7PlayRenderObj *pObj,const char **ppObjList)
{
	if( pObj==NULL || ppObjList==NULL ){
		return false;
	}
	for( ; *ppObjList!=NULL ; ppObjList++){
		InitReceiveShadowPlayRenderObj(pObj,ppObjList);
	}
	return true;
}

bool WK7Play::InitReceiveShadowPlayRenderObj(WK7PlayRenderObj *pObj,const char *pObjName)
{
	if( pObj==NULL || pObjName==NULL ){
		return false;
	}
	WK7JObj		*pJObj;
	if( (pJObj = pObj->GetJObj(pObjName))==NULL ){
		return false;
	}
	WK7SHADOW->AddReceiveShadowObj(pJObj);
	return true;
}

bool WK7Play::InitShadowPlayRenderObj(WK7PlayRenderObj *pObj)
{
	switch( mPlayNo ){
	case	0x00010101:
		InitCastShadowPlayRenderObj(pObj,"Trunk");
		break;
	case	0x01070202:
		InitReceiveShadowPlayRenderObj(pObj,"MJ_Root");
		break;
	case	0x00020711:
	case	0x05120101:
		WK7SHADOW->SetMinRadius( 2.0f );
		InitCastShadowPlayRenderObj(pObj,"Bullet");
		InitCastShadowPlayRenderObj(pObj,"Kun_Asi");
		InitCastShadowPlayRenderObj(pObj,"WheelChair_Body");
		break;
	case	0x03060101:
		InitCastShadowPlayRenderObj(pObj,"S_BALL");
		break;
	case	0x05500101:
		InitCastShadowPlayRenderObj(pObj,"TrunkGun");
		break;
	default:
		break;
	}
	return true;
}

bool WK7Play::InitResource(void *pData)
{
	if( (mpResLink = new(pData) ghmGcResLink)==NULL ){
		return false;
	}
	if( mpResLink->IsGroup(WK7PLAY_RES_SCRIPT)==true ){
		return false;
	}
	u32		size = mpResLink->GetResourceSize(WK7PLAY_RES_SCRIPT);
	if( size & 1 ){
		return false;
	}
	u16		*pScriptData;
	if( (pScriptData = (u16 *)mpResLink->GetResource(WK7PLAY_RES_SCRIPT))==NULL ){
		return false;
	}
	// GfBAϊ
	while( size ){
		*pScriptData = CONV_U16_ENDIAN(*pScriptData);
		size -= 2;
		pScriptData++;
	}
	WK7Archive		*pArchive;
	ghmGcResGroup	*pGroup;
	u32				i;
	
	// f\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_MODEL))==NULL ){
		return false;
	}
	if( (mModelNum = pGroup->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLMDL");
		mppModel  = (WK7PlayRenderObj **)K7AppSmallMemAlloc((int)(sizeof(WK7PlayRenderObj*) * mModelNum));
		for(i=0 ; i<mModelNum ; i++){
			mppModel[i] = NULL;
		}
		for(i=0 ; i<mModelNum ; i++){
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			u32		type = WK7PLAY_TYPE_RES_MODEL(mPlayID);
			pArchive->SetType(type);
			pArchive->SetID  (i);
			pArchive->LoadArchive(pGroup->GetResource((int)i));
 			mppModel[i] = WK7RenderObj::CreatePlayRenderObj(type,i);
 			InitShadowPlayRenderObj(mppModel[i]);
		}
		mEffectInfo.mpModel = GetModel(0);
//		WK7PlayRenderObj	*pObj = GetRenderObj(0);
//		WK7Light			*pLight;
//		if( pObj!=NULL && (pLight = pObj->GetShadowLight())!=NULL ){
//			WK7SHADOW->SetLight(pLight->GetHSDLObj());
//		}else{
//			WK7SHADOW->SetLight(NULL);
//		}
	}
	
	// f[V\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_MODELMOTION))==NULL ){
		return false;
	}
	if( (mModelMotionNum = pGroup->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLMOT");
		mppModelMotion  = (WK7ModelMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7ModelMotion*) * mModelMotionNum));
		for(i=0 ; i<mModelMotionNum ; i++){
			mppModelMotion[i] = NULL;
		}
		for(i=0 ; i<mModelMotionNum ; i++){
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			pArchive->SetType(WK7PLAY_TYPE_RES_MODELMOTION(mPlayID));
			pArchive->SetID  (i);
			pArchive->LoadArchive(pGroup->GetResource((int)i));
			mppModelMotion[i] = pArchive->CreateModelMotion();
		}
	}
	
	// fVFCv[V\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_MODELSHAPEMOTION))==NULL ){
		return false;
	}
	if( (mModelShapeMotionNum = pGroup->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLSMOT");
		mppModelShapeMotion  = (WK7ModelMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7ModelMotion*) * mModelShapeMotionNum));
		for(i=0 ; i<mModelShapeMotionNum ; i++){
			mppModelShapeMotion[i] = NULL;
		}
		for(i=0 ; i<mModelShapeMotionNum ; i++){
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			pArchive->SetType(WK7PLAY_TYPE_RES_MODELSHAPEMOTION(mPlayID));
			pArchive->SetID  (i);
			pArchive->LoadArchive(pGroup->GetResource((int)i));
			mppModelShapeMotion[i] = pArchive->CreateModelMotion();
		}
	}
	
	// J[V\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_CAMERAMOTION))==NULL ){
		return false;
	}
	if( (mCameraMotionNum = pGroup->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLCMOT");
		mppCameraMotion  = (WK7CameraMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7CameraMotion*) * mCameraMotionNum));
		for(i=0 ; i<mCameraMotionNum ; i++){
			mppCameraMotion[i] = NULL;
		}
		for(i=0 ; i<mCameraMotionNum ; i++){
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			pArchive->SetType(WK7PLAY_TYPE_RES_CAMERAMOTION(mPlayID));
			pArchive->SetID  (i);
			pArchive->LoadArchive(pGroup->GetResource((int)i));
			mppCameraMotion[i] = pArchive->CreateCameraMotion();
		}
	}
	
	// Cg[V\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_LIGHTMOTION))==NULL ){
		return false;
	}
	if( (mLightMotionNum = pGroup->GetResourceNum()) > 0 ){
		ghmGcMemSetAllocName("PLLMOT");
		mppLightMotion  = (WK7LightMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7LightMotion*) * mLightMotionNum));
		for(i=0 ; i<mLightMotionNum ; i++){
			mppLightMotion[i] = NULL;
		}
		for(i=0 ; i<mLightMotionNum ; i++){
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			pArchive->SetType(WK7PLAY_TYPE_RES_LIGHTMOTION(mPlayID));
			pArchive->SetID  (i);
			pArchive->LoadArchive(pGroup->GetResource((int)i));
			mppLightMotion[i] = pArchive->CreateLightMotion();
		}
	}
	
	// 
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_JIMAKU))!=NULL ){
		switch( pGroup->GetResourceNum() ){
		case	2:
#ifdef	_DEBUG
			if( GETCAPTUREMODE() ){
				break;
			}
#endif
			if( (mpTelopData = pGroup->GetResource(1))!=NULL ){
				gpEngTelop->mDataSet(mpTelopData);
			}
		case	1:
#ifdef	_DEBUG
			if( GETCAPTUREMODE() ){
				break;
			}
#endif
			if( (mpJimakuData = pGroup->GetResource(0))!=NULL ){
				gpJimaku->mDataSet(mpJimakuData);
			}
			break;
		default:
			return false;
		}
	}else{
		if( (mpJimakuData = mpResLink->GetResource(WK7PLAY_RES_JIMAKU))!=NULL ){
#ifdef	_DEBUG
			if( !GETCAPTUREMODE() ){
#endif
			gpJimaku->mDataSet(mpJimakuData);
#ifdef	_DEBUG
			}
#endif
		}
	}
	
	// Cxgpg`t@Cǂݍ݃NGXg
//	gpSound->mReadEventSsmReq(mResourceName);
	
#if	0	//03/11/12 pN[V폜̂߃Jbg
	// bvVFCv[V\[X
	if( (mRipShapeMotionNum = gpJimaku->mGetSMotionNum()) > 0 ){
		void		**pMotionPtrTable = gpJimaku->mGetSMotionPtr();
		mppRipShapeMotion  = (WK7ModelMotion **)K7AppSmallMemAlloc((int)(sizeof(WK7ModelMotion*) * mRipShapeMotionNum));
		for(i=0 ; i<mRipShapeMotionNum ; i++){
			mppRipShapeMotion[i] = NULL;
		}
		for(i=0 ; i<mRipShapeMotionNum ; i++){
			if( pMotionPtrTable[i]==NULL ){
				continue;
			}
			if( (pArchive = WK7Archive::Create())==NULL ){
				return false;
			}
			if( pGroup->IsGroup((int)i)==true ){
				return false;
			}
			pArchive->SetType(WK7PLAY_TYPE_RES_RIPSHAPEMOTION(mPlayID));
			pArchive->SetID  (i);
			pArchive->LoadArchive(pMotionPtrTable[i]);
			mppRipShapeMotion[i] = pArchive->CreateModelMotion();
		}
	}
#endif
	
	// [VXg\[X
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_MOTIONLIST))!=NULL ){
		u32			motionListNum = pGroup->GetResourceNum();
		for(i=0 ; i<motionListNum ; i++){
			const char	*pResourceName = pGroup->GetResourceName((int)i);
			if( pResourceName!=NULL ){
				ghmGcMemSetAllocName("PlayMt");
				WK7PlayMotion	*pPlayMotion = new WK7PlayMotion(this,&mMotionAnchor,(int)i);
				if( pPlayMotion!=NULL ){
					char		fname[WK7PLAY_RESOURCE_NAME_MAX];
					strcpy(fname,mResourceDir);
					strcat(fname,pResourceName);
					pPlayMotion->SetFileName(fname);
				}
			}
		}
		mpNextPlayMotion = 
		mpNextLoadMotion = (WK7PlayMotion *)mMotionAnchor.GetNext();
	}
	
	// [VpbNXg
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_SEPPACKLIST))!=NULL ){
		if( (mSepMotionPackNum = pGroup->GetResourceNum()) > 0 ){
			ghmGcMemSetAllocName("PLSMPA");
			if( (mppSepMotionPack  = (WK7PlaySepMotionPack **)K7AppSmallMemAlloc((int)(sizeof(WK7PlaySepMotionPack*) * mSepMotionPackNum)))!=NULL ){
				char					packName[WK7PLAY_RESOURCE_NAME_MAX];
				u32						packsize[2] = { 0,0 };
				WK7PlaySepMotionPack	**ppSepMotionPack;
				ghmGcFile				file;
				for(i=0,ppSepMotionPack=mppSepMotionPack ; i<mSepMotionPackNum ; i++,ppSepMotionPack++){
					*ppSepMotionPack = NULL;
				}
				for(i=0,ppSepMotionPack=mppSepMotionPack ; i<mSepMotionPackNum ; i++,ppSepMotionPack++){
					ghmGcMemSetAllocName("PlSepMtP");
					if( (*ppSepMotionPack = new WK7PlaySepMotionPack(this))==NULL ){
						continue;
					}
					(*ppSepMotionPack)->SetID      (i);
					(*ppSepMotionPack)->SetPackName(pGroup->GetResourceName((int)i));
					(*ppSepMotionPack)->GetResourceName(packName);
					if( file.Open(packName)==true ){
						u32		size = file.GetLength();
						if( packsize[i & 1] < size ){
							packsize[i & 1] = size;
						}
						file.Close();
					}
				}
				for(i=0 ; i<2 ; i++){
					if( packsize[i] <= 0 ){
						continue;
					}
					mpSepMotionPackBuffer[i] = ghmGcMemAlloc((int)packsize[i]);
				}
			}
		}
	}
	
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_CHARANAME))!=NULL ){
		return false;
	}
	if( (mpCharaNameData = mpResLink->GetResource(WK7PLAY_RES_CHARANAME))!=NULL ){
		gpCharaDispMng->mDataSet(mpCharaNameData);
	}
	
	return true;
}

bool WK7Play::IsLoadedResource(void)
{
	if( mppModel==NULL ){
		return true;
	}
	
	WK7PlayRenderObj	*pObj;
	int					i;
	
	for(i=0 ; i<mModelNum ; i++){
		if( (pObj = mppModel[i])==NULL ){
			continue;
		}
		if( pObj->GetState()!=WK7RenderObj::LOADED ){
			return false;
		}
	}
	return true;
}

bool WK7Play::PrepareResource(void)
{
	InitializeShadowLight();
	InitScript();
	InitializePlayEffect();
	return true;
}

bool WK7Play::InitializeShadowLight(void)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(0);
	WK7Light			*pLight;
	if( pObj!=NULL && (pLight = pObj->GetShadowLight())!=NULL ){
		WK7SHADOW->SetLight(pLight->GetHSDLObj());
	}else{
		WK7SHADOW->SetLight(NULL);
	}
	return true;
}

bool WK7Play::TermResource(void)
{
	int		i;
	
	// Cxgpg`t@C
//	gpSound ->mReleaseEventSsm();
	
	GHM_SAFE_DELETE(mpScript);
	
	for(i=0 ; i<mModelNum ; i++){
		GHM_SAFE_DELETE(mppModel           [i]);
	}
	for(i=0 ; i<mModelMotionNum ; i++){
		GHM_SAFE_DELETE(mppModelMotion     [i]);
	}
	for(i=0 ; i<mModelShapeMotionNum ; i++){
		GHM_SAFE_DELETE(mppModelShapeMotion[i]);
	}
	for(i=0 ; i<mCameraMotionNum ; i++){
		GHM_SAFE_DELETE(mppCameraMotion    [i]);
	}
	for(i=0 ; i<mLightMotionNum ; i++){
		GHM_SAFE_DELETE(mppLightMotion     [i]);
	}
	for(i=0 ; i<mRipShapeMotionNum ; i++){
		GHM_SAFE_DELETE(mppRipShapeMotion  [i]);
	}
	for(i=0 ; i<mSepMotionPackNum ; i++){
		GHM_SAFE_DELETE(mppSepMotionPack   [i]);
	}
	GHMGCMEM_SAFE_FREE(mppModel           );
	GHMGCMEM_SAFE_FREE(mppModelMotion     );
	GHMGCMEM_SAFE_FREE(mppModelShapeMotion);
	GHMGCMEM_SAFE_FREE(mppCameraMotion    );
	GHMGCMEM_SAFE_FREE(mppLightMotion     );
	GHMGCMEM_SAFE_FREE(mppRipShapeMotion  );
	GHMGCMEM_SAFE_FREE(mppSepMotionPack   );
	GHMGCMEM_SAFE_FREE(mpSepMotionPackBuffer[0]);
	GHMGCMEM_SAFE_FREE(mpSepMotionPackBuffer[1]);
	mModelNum            = 0;
	mModelMotionNum      = 0;
	mModelShapeMotionNum = 0;
	mCameraMotionNum     = 0;
	mLightMotionNum      = 0;
	mRipShapeMotionNum   = 0;
	mSepMotionPackNum    = 0;
	mPlaySepMotionPack   = -1;
	
	if( mResAlloc ){
		GHM_SAFE_DELETE(mpResLink);
	}else{
		mpResLink = NULL;
	}
	
	return true;
}

bool WK7Play::ScriptTerm(void)
{
	// ^XNhc
	mTaskID = -1;
	
	mPlaySepMotionPack  = -1;
	// LN^Љf[^
	ClearCharaName();
	gpCharaDispMng->mDataSet(NULL);
	WK7CHARAPRESENT->DeleteMaskWindowAll();
	WK7CHARAPRESENT->SetEnable(false);
	// ebv~
	gpEngTelop->mDataSet(NULL);
	gpEngTelop->mDispStop();
	// ~
	gpJimaku->mDataSet(NULL);
	gpJimaku->mDispStop();
	// Xg[~
	if( mDontStopStream == false )
	{
		gpSound->mStopStream();
		gpSound->mSetPitchStream(0);
		gpSound->_mResetPauseFlagStream();
	}
	mDontStopStream = false;
	// GtFNgI
//	TerminatePlayEffect();
	// LGtFNgpݒ
//	::SetCharGekiPlayEffectInfo(NULL);
	return true;
}

bool WK7Play::StopMotionAll(void)
{
	if( mppModel==NULL ){
		return false;
	}
	int		i;
	for(i=0 ; i<mModelNum ; i++){
		if( mppModel[i]==NULL ){
			continue;
		}
		mppModel[i]->StopMotionAll();
	}
	return true;
}

bool WK7Play::SetVisibleAll(bool flag)
{
	if( mppModel==NULL ){
		return false;
	}
	int		i;
	for(i=0 ; i<mModelNum ; i++){
		if( mppModel[i]==NULL ){
			continue;
		}
		mppModel[i]->SetVisible(flag);
	}
	return true;
}

WK7PlayRenderObj* WK7Play::GetRenderObj(int index)
{
	if( mppModel==NULL || index < 0 || index >= mModelNum ){
		return NULL;
	}
	return mppModel[index];
}

WK7Model* WK7Play::GetModel(int index)
{
	WK7RenderObj	*pObj;
	if( (pObj=GetRenderObj(index))==NULL ){
		return NULL;
	}
	return pObj->GetModel();
}

WK7ModelMotion* WK7Play::GetModelMotion(int index)
{
	if( mppModelMotion==NULL || index < 0 || index >= mModelMotionNum ){
		return NULL;
	}
	return mppModelMotion[index];
}

WK7ModelMotion* WK7Play::GetModelShapeMotion(int index)
{
	if( mppModelShapeMotion==NULL || index < 0 || index >= mModelShapeMotionNum ){
		return NULL;
	}
	return mppModelShapeMotion[index];
}

WK7CameraMotion* WK7Play::GetCameraMotion(int index)
{
	if( mppCameraMotion==NULL || index < 0 || index >= mCameraMotionNum ){
		return NULL;
	}
	return mppCameraMotion[index];
}

WK7LightMotion* WK7Play::GetLightMotion(int index)
{
	if( mppLightMotion==NULL || index < 0 || index >= mLightMotionNum ){
		return NULL;
	}
	return mppLightMotion[index];
}

WK7ModelMotion* WK7Play::GetRipShapeMotion(int index)
{
	if( mppRipShapeMotion==NULL || index < 0 || index >= mRipShapeMotionNum ){
		return NULL;
	}
	return mppRipShapeMotion[index];
}

WK7PlayRenderObj* WK7Play::GetRenderObjByCharaID(u32 chrid)
{
	if( mppModel==NULL ){
		return NULL;
	}
	ghmGcResGroup	*pGroup;
	if( (pGroup=mpResLink->GetGroup(WK7PLAY_RES_MODEL))==NULL ){
		return NULL;
	}
	char		chrname[32];
	if( WK7Chara::GetInitial(chrid,chrname)==false ){
		return NULL;
	}
	u32					i;
	ghmGcResAttr		*pAttr;
	const char			*pName;
	char				resourceName[64];
	for(i=0 ; i<pGroup->GetResourceNum() ; i++){
		pAttr = (ghmGcResAttr *)pGroup->GetFileAttr((int)i);
		if( (pName=pAttr->GetResourceName())==NULL ){
			continue;
		}
		strncpy(resourceName,pName,sizeof(resourceName)-1);
		strupr (resourceName);
		if( strstr(resourceName,chrname)==NULL ){
			continue;
		}
		return mppModel[i];
	}
	return NULL;
}

WK7PlayCharaName::WK7PlayCharaName(ghmGcListObj *pAnchor) : ghmGcListObj(pAnchor)
{
	ghmGcMemSetAllocName("MK7CHDSP");
	if( (mpCharaDisp = new MK7CharaDisp)==NULL ){
		mReleaseRequest = true;
	}else{
		mReleaseRequest = false;
	}
}

WK7PlayCharaName::~WK7PlayCharaName()
{
	GHM_SAFE_DELETE(mpCharaDisp);
}

bool WK7PlayCharaName::FrameProcess(void)
{
	if( mpCharaDisp==NULL ){
		return false;
	}
	if( mpCharaDisp->mGetFrameProcess()==eCharaDispStandby ){
		if( mReleaseRequest==true ){
			return false;
		}
	}
	return true;
}

bool WK7PlayCharaName::Disp(s16 inNo,s16 inX,s16 inY,s32 inTick)
{
	if( mpCharaDisp==NULL ){
		return false;
	}
	mpCharaDisp->mRequest(inNo,inX,inY,inTick);
	return true;
}

bool WK7PlayCharaName::Hide(void)
{
	if( mpCharaDisp==NULL ){
		return false;
	}
	mpCharaDisp->mRelease();
	return true;
}

bool WK7Play::DispCharaName(s16 inNo,s16 inX,s16 inY,s32 inTick)
{
	if( mpCharaNameData==NULL ){
		return false;
	}
	ghmGcMemSetAllocName("WKPCHNM");
	WK7PlayCharaName		*pCharaName = new WK7PlayCharaName(&mCharaNameAnchor);
	if( pCharaName==NULL ){
		return false;
	}
	pCharaName->Disp(inNo,inX,inY,inTick);
	return true;
}

bool WK7Play::HideCharaName(void)
{
	WK7PlayCharaName	*pCharaName;
	for(pCharaName=(WK7PlayCharaName *)mCharaNameAnchor.GetNext() ; pCharaName!=NULL ; pCharaName=pCharaName->GetNext()){
		pCharaName->Release();
	}
	return true;
}

bool WK7Play::ClearCharaName(void)
{
	WK7PlayCharaName	*pCharaName,*pNext;
	for(pCharaName=(WK7PlayCharaName *)mCharaNameAnchor.GetNext() ; pCharaName!=NULL ; pCharaName=pNext){
		pNext = pCharaName->GetNext();
		delete pCharaName;
	}
	return true;
}

bool WK7Play::FrameProcessCharaName(void)
{
	WK7PlayCharaName	*pCharaName;
	WK7PlayCharaName	*pNext;
	for(pCharaName=(WK7PlayCharaName *)mCharaNameAnchor.GetNext() ; pCharaName!=NULL ; pCharaName=pNext){
		pNext = pCharaName->GetNext();
		if( pCharaName->FrameProcess()==false ){
			delete pCharaName;
		}
	}
	return true;
}

bool WK7Play::StopProcess(void)
{
	u32					i;
	WK7PlayRenderObj	*pRenderObj;
	for(i=0 ; i<mModelNum ; i++){
		if( (pRenderObj = mppModel[i])==NULL ){
			continue;
		}
		pRenderObj->StopMotionAll();
	}
	WK7CAM->StopMotion();
	WK7PlaySepMotionPack	*pPlaySepMotionPack = GetPlaySepMotionPack();
	WK7PlaySepMotion		*pPlaySepMotion;
	if( pPlaySepMotionPack != NULL && (pPlaySepMotion = pPlaySepMotionPack->GetPlaySepMotion())!=NULL ){
		WK7LightMotion	*pLightMotion;
		if( (pLightMotion = pPlaySepMotion->GetLightMotion())!=NULL ){
			int				i,num = pLightMotion->GetAnimNum();
			const char		*pLightName;
			WK7Light		*pLight;
			HSD_LightAnim	*pLightAnim;
			for(i=0 ; i<num ; i++){
				pLightName = pLightMotion->GetAnimName(i);
				if( (pLightAnim = pLightMotion->GetAnim(i))==NULL ){
					continue;
				}
				if( (pLight=WK7Light::Find(pLightName))==NULL ){
					continue;
				}
				pLight->StopMotion();
			}
		}
	}
	// ebv|[Y
	gpJimaku  ->mSetPause();
	gpEngTelop->mSetPause();
	mPlayCountFlag = 0;
	// Xg[|[Y
	gpSound   ->mSetPauseStream();
	return true;
}

bool WK7Play::RestartProcess(void)
{
	u32					i;
	WK7PlayRenderObj	*pRenderObj;
	for(i=0 ; i<mModelNum ; i++){
		if( (pRenderObj = mppModel[i])==NULL ){
			continue;
		}
		pRenderObj->ReplayMotionAll();
	}
	WK7CAM->ReplayMotion();
	WK7PlaySepMotionPack	*pPlaySepMotionPack = GetPlaySepMotionPack();
	WK7PlaySepMotion		*pPlaySepMotion;
	if( pPlaySepMotionPack != NULL && (pPlaySepMotion = pPlaySepMotionPack->GetPlaySepMotion())!=NULL ){
		WK7LightMotion	*pLightMotion;
		if( (pLightMotion = pPlaySepMotion->GetLightMotion())!=NULL ){
			int				i,num = pLightMotion->GetAnimNum();
			const char		*pLightName;
			WK7Light		*pLight;
			HSD_LightAnim	*pLightAnim;
			for(i=0 ; i<num ; i++){
				pLightName = pLightMotion->GetAnimName(i);
				if( (pLightAnim = pLightMotion->GetAnim(i))==NULL ){
					continue;
				}
				if( (pLight=WK7Light::Find(pLightName))==NULL ){
					continue;
				}
				pLight->ReplayMotion();
			}
		}
	}
	// ebv|[Y
	gpJimaku  ->mReleasePause();
	gpEngTelop->mReleasePause();
	mPlayCountFlag = 1;
	// Xg[|[Y
	gpSound   ->mReleasePauseStream();
	
	return true;
}

bool WK7Play::FrameProcess(void)
{
	
//	if( HSD_PadGetTrigger(1) & HSD_PAD_RIGHT ){
//static	f32	x = 320.0f - 128.0f * 0.5f,y = 160.0f - 128.0f * 0.5f,w = 128.0f,h= 128.0f;
//		WK7CHARAPRESENT->AddMaskWindow(x,y,w,h);
//		WK7CHARAPRESENT->SetEnable(true);
//		StopProcess();
//	}
//	if( HSD_PadGetTrigger(1) & HSD_PAD_LEFT ){
//		WK7CHARAPRESENT->DeleteMaskWindowAll();
//		WK7CHARAPRESENT->SetEnable(false);
//		RestartProcess();
//	}
	int		i;
	
	if( mSkipCancel==false && HSD_PadGetTrigger(0)==HSD_PAD_START ){
		switch( mState ){
//		case	PREPARED:
		case	PLAY:
			if( !mFirstPlay ){
				break;
			}
			Stop();
			break;
		}
	}
	switch( mState ){
	case	IDLE:
		break;
	case	LOADING:
		if( mpFile==NULL ){
			mState = IDLE;
			break;
		}
		if( mpFile->GetState()!=ghmGcFile::READFINISH ){
			break;
		}
		mpFile->Close();
		delete mpFile;
		mpFile = NULL;
		if( InitResource(mpData)==false ){
			Release();
			break;
		}
		ReleasePlayerSsm();
		ReadSsm();
//		mState = SSMLOADING;
		mState = PREPARERESOURCE;
	case	PREPARERESOURCE:
		if( IsLoadedResource()==false ){
			break;
		}
//		::SetEffectAllDrawFlag(false);
		PrepareResource();
		mState = SSMLOADING;
	case	SSMLOADING:
		if( eSndStandby != gpSound->mGetProcess() ){
			break;
		}
		mState = LOADED;
	case	LOADED:
	case	PLAYED:
		// ǂݍ݊
		if( !mPrepareRequest ){
			break;
		}
		if( mTaskID >= 0 ){
			mTaskController.Stop(mTaskID);
			mTaskID  = -1;
		}
		if( (mTaskID=mTaskController.Start(ScriptParseHandler,
		                                   ScriptParseHandlerTerm,
		                                   (u32)this,
		                                   0,
		                                   NULL,
		                                   WK7PLAY_TASK_STACK_SIZE,
		                                   GHMGCLIST_DEFAULT_PRIO)) < 0 ){
			break;
		}
		mState = PREPARE;
	case	PREPARE:
		if( PrepareNextSepMotionPack()==false ){
//			Release();
			break;
		}
		mpScript->Reset();
		mState = PREPARING;
		mPrepareRequest = 0;
	case	PREPARING:
		// 
		if( !mInitialized ){
			break;
		}
		if( IsPreparedNextSepMotionPack()==false ){
			break;
		}
		mState = PREPARED;
	case	PREPARED:
		// 
		if( !mStartRequest ){
			break;
		}else{
			WK7PlayTable	*pTable = GetTable(mPlayNo);
			mStoredSceneBlurVisible = SceneBlur.GetVisible();
			if( pTable!=NULL && pTable->mBlurDist0 < 0.0f && pTable->mBlurDist1==0.0f ){
				SceneBlur.SetVisible(false);
			}else{
				SceneBlur.SetVisible(true);
			}
			for(i=0 ; i<WK7PLAY_STORE_SCENEBLUR_NUM ; i++){
				WK7SceneBlurScreen	*pScreen = SceneBlur.GetScreen(i);
				if( pScreen==NULL ){
					continue;
				}
				mStoredSceneBlurDist[i] = pScreen->GetZDist();
				if( pTable!=NULL && pTable->mBlurDist0 >= 0.0f ){
					switch( i ){
					case	2:
					case	3:
						pScreen->SetZDist(pTable->mBlurDist1);
						break;
					default:
						pScreen->SetZDist(pTable->mBlurDist0);
						break;
					}
				}else{
					pScreen->SetZDist(zdist[i]);
				}
			}
			::SetEffectSoundOffFlag(TRUE);
//			if( mCinemaScopeColor==0 ){
//				if( pTable!=NULL && pTable->mCinemaScopeColor!=0 ){
//					WK7CAM->SetCinemaScopeColor(pTable->mCinemaScopeColor);
//				}else{
//					WK7CAM->SetCinemaScopeColor(0x781111ff);
//				}
//			}else{
//				WK7CAM->SetCinemaScopeColor(mCinemaScopeColor);
//			}
			WK7CAM->SetCinemaScopeColor(WK7CAM_DEFAULT_CINEMASCOPE_COLOR);
			WK7CAM->SetWobbleEnable(false);
			WK7CAM->SetCinemaScopeMode(true);
			WK7CAM->SetFov(42.165f);
			WK7CAM->SetRoll(0.0f);
			WK7Play::mpCurrentPlay = this;
			mState = PLAY;
			mStartRequest = 0;
			mPrepareRequest = 0;
			if( mInitCaptureRenderOff==true ){
				screenImageFullColor.SetRender    (false);
				screenImageFullColor.SetPreRender (false);
				screenImageFullColor.SetCopyEnable(true );
				screenImageFullColor.SetTexFilter(GX_NEAR,GX_NEAR);
				screenImageMonoColor.SetRender    (false);
				screenImageMonoColor.SetPreRender (false);
				screenImageMonoColor.SetCopyEnable(true );
				screenImageMonoColor.SetTexFilter(GX_NEAR,GX_NEAR);
			}
		}
	case	PLAY:
		// LĐ
		if( !mReleasePause && mFirstPlay ){
			if( gpNoise->mGetSandFrameProc()!=eSandStandby ){
				break;
			}
			mReleasePause  = 1;
			mPlayCountFlag = 1;
			ReleasePause();
		}
		break;
	case	RELEASE:
		if( mReleaseRequest ){
			TerminatePlayEffect();
			// LGtFNgpݒ
			::SetCharGekiPlayEffectInfo(NULL);
			for(i=0 ; i<WK7PLAY_STORE_SCENEBLUR_NUM ; i++){
				WK7SceneBlurScreen	*pScreen = SceneBlur.GetScreen(i);
				if( pScreen==NULL ){
					continue;
				}
				pScreen->SetZDist(mStoredSceneBlurDist[i]);
			}
			SceneBlur.SetVisible(mStoredSceneBlurVisible);
			SetVisibleAll(false);
			WK7CAM->SetRoll(0.0f);
			mReleaseRequest = 0;
			break;
		}
		return false;
	default:
		OSDebugReport("WK7Play Invalid State : %x\n",mState);
		break;
	}
	
//	PlayMotionProcess();
	SepMotionPackProcess();
	FrameProcessCharaName();
	
	return true;
}

//static	f32		backWholeMotionRate =  1.0f;
//static	f32		skipMotionRate      = 10.0f;
bool WK7Play::FrameProcessAll(void)
{
	WK7Play		*pPlay;
	WK7Play		*pNext;
	
	// ^XN
	mTaskController.FrameProcess();
	
	pPlay = (WK7Play *)mAnchor.GetNext();
#ifdef	_DEBUG
//	if( pPlay!=NULL ){
//		if( HSD_PadGetButton(1) & HSD_PAD_Z ){
//			if( skipMotionRate!=WK7Motion::GetWholeMotionRate() ){
//				if( backWholeMotionRate!=WK7Motion::GetWholeMotionRate() ){
//					backWholeMotionRate = WK7Motion::GetWholeMotionRate();
//				}
//				WK7Motion::SetWholeMotionRate(skipMotionRate);
//			}
//		}else{
//			if( backWholeMotionRate!=WK7Motion::GetWholeMotionRate() ){
//				WK7Motion::SetWholeMotionRate(backWholeMotionRate);
//			}
//		}
//	}
#endif
	while( pPlay!=NULL ){
		pNext = pPlay->GetNext();
		if( pPlay->FrameProcess()==false ){
			delete pPlay;
		}
		pPlay = pNext;
	}
	
	return true;
}

bool WK7Play::RenderProcessPlayEffect(void)
{
	WK7PlayEffect	*pPlayEffect;
	for(pPlayEffect=(WK7PlayEffect *)mPlayEffectAnchor.GetNext() ; pPlayEffect!=NULL ; pPlayEffect=pPlayEffect->GetNext()){
		pPlayEffect->RenderProcess();
	}
	return true;
}

bool WK7Play::RenderProcess(void)
{
	RenderProcessPlayEffect();
	return true;
}

bool WK7Play::RenderProcessAll(void)
{
	WK7Play		*pPlay;
	
	for(pPlay=(WK7Play *)mAnchor.GetNext() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
		pPlay->RenderProcess();
	}
	
	return true;
}

bool WK7Play::IsPlaying(void)
{
	WK7Play	*pPlay;
	for(pPlay=(WK7Play *)mAnchor.GetNext() ; pPlay!=NULL ; pPlay=pPlay->GetNext()){
		if( pPlay->GetState()==PLAY ){
			return true;
		}
	}
	return false;
}

bool WK7Play::SetStopScript(bool flag)
{
	if( mpScript==NULL ){
		return false;
	}
	if( flag==true ){
		mpScript->Stop();
	}else{
		mpScript->Restart();
	}
	return true;
}

bool WK7Play::GetStopScript(void)
{
	if( mpScript==NULL ){
		return false;
	}
	return mpScript->IsStopped();
}

bool WK7Play::ScriptObjectCallMethod(WK7Script *pScript,u16 methodID,u16 objID)
{
	WK7Play	*pPlay = (WK7Play *)pScript->GetParam(0);
	if( pPlay==NULL ){
		return false;
	}
	return pPlay->_ScriptObjectCallMethod(pScript,methodID,objID);
}

bool WK7Play::_ScriptObjectCallMethod(WK7Script *pScript,u16 methodID,u16 /*objID*/)
{
	// objID͓IuWFNgߊ֌WȂ
	s32		iarg[7];
	f32		farg[7];
	switch( methodID ){
	case	0:	// 
		pScript->PushExpInt(pScript->RetSys());
		break;
	case	1:	// ԑ҂
		pScript->PushExpInt(pScript->TimeWaitTick(pScript->PopInt()));
		break;
	case	2:	// ԑ҂
		pScript->PushExpInt(pScript->TimeWaitSec(pScript->PopFloat()));
		break;
	case	3:	// ٕ\ݒ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetVisible(iarg[0],iarg[1]));
		break;
	case	4:	// ӰݍĐ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlayMotion(iarg[0],iarg[1]));
		break;
	case	5:	// ӰݍĐiԂj
		farg[0] = pScript->PopFloat();
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlayMotionI(iarg[0],iarg[1],farg[0]));
		break;
	case	6:	// ټӰݍĐ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlayShapeMotion(iarg[0],iarg[1]));
		break;
	case	7:	// Ӱݒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopMotion(iarg[0]));
		break;
	case	8:	// ټӰݒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopShapeMotion(iarg[0]));
		break;
	case	9:	// ّSӰݒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopMotionAll(iarg[0]));
		break;
	case	10:	// ӰݍĐI҂
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelWaitMotion(iarg[0]));
		break;
	case	11:	// ټӰݍĐI҂
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelWaitShapeMotion(iarg[0]));
		break;
	case	12:	// ّSӰݍĐI҂
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelWaitMotionAll(iarg[0]));
		break;
	case	13:	// Ӱٰߐݒ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetMotionLoop(iarg[0],iarg[1]));
		break;
	case	14:	// ӰݍĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(CameraPlayMotion(iarg[0]));
		break;
	case	15:	// ӰݍĐiԂj
		farg[0] = pScript->PopFloat();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(CameraPlayMotionI(iarg[0],farg[0]));
		break;
	case	16:	// Ӱݒ~
		pScript->PushExpInt(CameraStopMotion());
		break;
	case	17:	// ӰݍĐI҂
		pScript->PushExpInt(CameraWaitMotion());
		break;
	case	18:	// Ӱٰߐݒ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(CameraSetMotionLoop(iarg[0]));
		break;
	case	19:	// ײӰݍĐ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightPlayMotion(iarg[0],iarg[1]));
		break;
	case	20:	// ײӰݒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightStopMotion(iarg[0]));
		break;
	case	21:	// ײӰݍĐI҂
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightWaitMotion(iarg[0]));
		break;
	case	22:	// ײӰٰߐݒ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightSetMotionLoop(iarg[0],iarg[1]));
		break;
	case	23:	// Jn
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(JimakuDisp(iarg[0]));
		break;
	case	24:	// I҂
		pScript->PushExpInt(JimakuWait());
		break;
	case	25:	// rdĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(SePlay(iarg[0]));
		break;
	case	26:	// Xg[Đ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(StreamPlay(iarg[0]));
		break;
	case	27:	// Xg[~
		pScript->PushExpInt(StreamStop());
		break;
	case	28:	// Xg[ĐI҂
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(StreamWait(iarg[0]));
		break;
	case	29:	// حѼ޳
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelCreateShadow(iarg[0]));
		break;
	case	30:	// حѼ޳\ݒ
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetShadowVisible(iarg[0],iarg[1]));
		break;
	case	31:	// ̪ފJn(tick)
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		k7app.FadeStart((u32)iarg[0],(u32)iarg[1]);
		pScript->PushExpInt(1);
		break;
	case	32:	// ̪ފJn(sec)
		farg[0] = pScript->PopFloat();
		iarg[0] = pScript->PopInt();
		k7app.FadeStartSec((u32)iarg[0],farg[0]);
		pScript->PushExpInt(1);
		break;
	case	33:	// ̪޶װݒ
		iarg[2] = pScript->PopInt();
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		k7app.SetFadeColor( (((u32)iarg[0]<<24)|((u32)iarg[0]<<16)|((u32)iarg[0]<<8)) );
		pScript->PushExpInt(1);
		break;
	case	34:	// ̪ޏI҂
		pScript->PushExpInt(FadeWait());
		break;
	case	35:	// rd~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(SeStop(iarg[0]));
		break;
	case	36:	// fCgݒ
		iarg[2] = pScript->PopInt();
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetLight(iarg[0],iarg[1],iarg[2]));
		break;
	case	37:	// JړJnWݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetStartPosition(farg[0],farg[1],farg[2]));
		break;
	case	38:	// JړIWݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetEndPosition(farg[0],farg[1],farg[2]));
		break;
	case	39:	// J^[QbgړJnWݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetStartTargetPosition(farg[0],farg[1],farg[2]));
		break;
	case	40:	// J^[QbgړIWݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetEndTargetPosition(farg[0],farg[1],farg[2]));
		break;
	case	41:	// J`ԈړipXj
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraLinearIpLinearPath(farg[0]));
		break;
	case	42:	// JfBCԈړipXj
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraDelayIpLinearPath(farg[0]));
		break;
	case	43:	// J`Ԉړi~ʃpXj
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraLinearIpCirclePath(farg[0]));
		break;
	case	44:	// Jʒuݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetPosition(farg[0],farg[1],farg[2]));
		break;
	case	45:	// J^[Qbgʒuݒ
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetTargetPosition(farg[0],farg[1],farg[2]));
		break;
	case	46:	// Jpݒ
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetFov(farg[0]));
		break;
	case	47:	// JړJnpݒ
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetStartFov(farg[0]));
		break;
	case	48:	// JړIpݒ
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetEndFov(farg[0]));
		break;
	case	49:	// Xg[tF[h{[
		farg[0] = pScript->PopFloat();
		iarg[0] = pScript->PopInt  ();
		pScript->PushExpInt(StreamFadeVolume(iarg[0],farg[0]));
		break;
	case	50:	// JjAʐݒ
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetNear(farg[0]));
		break;
	case	51:	// Jt@[ʐݒ
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraSetFar(farg[0]));
		break;
	case	52:	// JԈړ~
		pScript->PushExpInt(CameraStopIp());
		break;
	case	53:	// GtFNg
		farg[3] = pScript->PopFloat();
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CreateExplodeEffect(farg[0],farg[1],farg[2],farg[3]));
		break;
	case	54:	// }YGtFNgZbg
		iarg[4] = pScript->PopInt();
		iarg[3] = pScript->PopInt();
		iarg[2] = pScript->PopInt();
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(SetMuzzleEffect(iarg[0],iarg[1],iarg[2],iarg[3],iarg[4]));
		break;
	case	55:	// }YGtFNg
		farg[0] = pScript->PopFloat();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(CreateMuzzleEffect(iarg[0],farg[0]));
		break;
	case	56:	// }YGtFNgZbg
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ResetMuzzleEffect(iarg[0]));
		break;
	case	57:	// wuX}CGtFNg
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CreateHSExplodeEffect(farg[0],farg[1],farg[2]));
		break;
	case	58:	// 凋COGtFNgZbg
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetVfxEffect(iarg[0]));
		break;
	case	59:	// 凋COGtFNgZbg
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelResetVfxEffect(iarg[0]));
		break;
	case	60:	// flݒ
		farg[0] = pScript->PopFloat();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelSetAlpha(iarg[0],farg[0]));
		break;
	case	61:	// ЃGtFNg
		farg[3] = pScript->PopFloat();
		farg[2] = pScript->PopFloat();
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CreateGoreEffect(farg[0],farg[1],farg[2],farg[3]));
		break;
	case	62:	// tbV
		farg[1] = pScript->PopFloat();
		farg[0] = pScript->PopFloat();
		iarg[3] = pScript->PopInt();
		iarg[2] = pScript->PopInt();
		iarg[1] = pScript->PopInt();
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(CreateFlashEffect(iarg[0],iarg[1],iarg[2],iarg[3],farg[0],farg[1]));
		break;
	case	63:	// ӰؽčĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlaySeqMotion(iarg[0]));
		break;
	case	64:	// ӰؽčĐiԂj
		iarg[0] = pScript->PopInt();
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(ModelPlaySeqMotionI(iarg[0],farg[0]));
		break;
	case	65:	// رӰؽčĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlaySeqMatMotion(iarg[0]));
		break;
	case	66:	// ټӰؽčĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelPlaySeqShapeMotion(iarg[0]));
		break;
	case	67:	// ӰؽĒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopSeqMotion(iarg[0]));
		break;
	case	68:	// رӰؽĒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopSeqMatMotion(iarg[0]));
		break;
	case	69:	// ټӰؽĒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopSeqShapeMotion(iarg[0]));
		break;
	case	70:	// ّSӰؽĒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(ModelStopSeqMotionAll(iarg[0]));
		break;
	case	71:	// ӰؽčĐ
		pScript->PushExpInt(CameraPlaySeqMotion());
		break;
	case	72:	// ӰؽčĐiԂj
		farg[0] = pScript->PopFloat();
		pScript->PushExpInt(CameraPlaySeqMotionI(farg[0]));
		break;
	case	73:	// ӰؽĒ~
		pScript->PushExpInt(CameraStopSeqMotion());
		break;
	case	74:	// ײӰؽčĐ
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightPlaySeqMotion(iarg[0]));
		break;
	case	75:	// ײӰؽĒ~
		iarg[0] = pScript->PopInt();
		pScript->PushExpInt(LightStopSeqMotion(iarg[0]));
		break;
	default:
		OSDebugReport("WK7Play ScriptObjectCallMethod : Invalid Method ID , %x\n",methodID);
		return false;
	}
	return true;
}

int WK7Play::ModelSetVisible(int mdlid,int flag)
{
	WK7RenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->SetRenderFlag(flag!=0);
	WK7Model	*pModel = GetModel(mdlid);
	if( pModel!=NULL ){
		if( flag ){
			pModel->Disp();
		}else{
			pModel->Hide();
		}
	}
	return 1;
}

int WK7Play::ModelPlayMotion(int mdlid,int motid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	WK7ModelMotion	*pModelMotion = GetModelMotion(motid);
	if( pModelMotion==NULL ){
		return 0;
	}
	pModel->AttachMotion(pModelMotion,-1,0,false);
	pModel->PlayMotion  ();
	return 1;
}

int WK7Play::ModelPlayMotionI(int mdlid,int motid,f32 sec)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	WK7ModelMotion	*pModelMotion = GetModelMotion(motid);
	if( pModelMotion==NULL ){
		return 0;
	}
	pModel->AttachMotion(pModelMotion,-1,0,false);
	pModel->PlayMotion  (0.0f,true,60.0f * sec);
	return 1;
}

int WK7Play::ModelPlayShapeMotion(int mdlid,int motid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	WK7ModelMotion	*pModelShapeMotion = GetModelShapeMotion(motid);
	if( pModelShapeMotion==NULL ){
		return 0;
	}
	pModel->AttachShapeMotion(pModelShapeMotion,-1,0);
	pModel->PlayMotion       ();
	return 1;
}

int WK7Play::ModelStopMotion(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	pModel->StopMotion  ();
	pModel->DetachMotion();
	return 1;
}

int WK7Play::ModelStopShapeMotion(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	pModel->StopShapeMotion  ();
	pModel->DetachShapeMotion();
	return 1;
}

int WK7Play::ModelStopMotionAll(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	pModel->StopMotion       ();
	pModel->StopShapeMotion  ();
	pModel->DetachMotion     ();
	pModel->DetachShapeMotion();
	return 1;
}

int WK7Play::ModelWaitMotion(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	while( pModel->GetMotionState()!=AOBJ_NO_ANIM ){
		RetSys();
	}
	return 1;
}

int WK7Play::ModelWaitShapeMotion(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	while( pModel->GetShapeMotionState()!=AOBJ_NO_ANIM ){
		RetSys();
	}
	return 1;
}

int WK7Play::ModelWaitMotionAll(int mdlid)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	while( pModel->GetMotionState     ()!=AOBJ_NO_ANIM ||
	       pModel->GetShapeMotionState()!=AOBJ_NO_ANIM ){
		RetSys();
	}
	return 1;
}

int WK7Play::ModelSetMotionLoop(int mdlid,int flag)
{
	WK7Model		*pModel = GetModel(mdlid);
	if( pModel==NULL ){
		return 0;
	}
	pModel->SetMotionLoop(flag!=0);
	return 1;
}

int WK7Play::CameraPlayMotion(int motid)
{
	WK7CameraMotion		*pCameraMotion = GetCameraMotion(motid);
	if( pCameraMotion==NULL ){
		return 0;
	}
	WK7CAM->AttachMotion(pCameraMotion);
	WK7CAM->PlayMotion  ();
	return 1;
}

int WK7Play::CameraPlayMotionI(int motid,f32 sec)
{
	WK7CameraMotion		*pCameraMotion = GetCameraMotion(motid);
	if( pCameraMotion==NULL ){
		return 0;
	}
	WK7CAM->AttachMotion(pCameraMotion);
	WK7CAM->PlayMotion  (0.0f,true,60.0f * sec);
	return 1;
}

int WK7Play::CameraStopMotion(void)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	return 1;
}

int WK7Play::CameraWaitMotion(void)
{
	while( WK7CAM->GetMotionState()!=AOBJ_NO_ANIM ){
		RetSys();
	}
	return 1;
}

int WK7Play::CameraSetMotionLoop(int flag)
{
	WK7CAM->SetMotionLoop(flag!=0);
	return 1;
}

int WK7Play::LightPlayMotion(int lid,int motid)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	WK7LightMotion		*pLightMotion = GetLightMotion(motid);
	if( pLightMotion==NULL ){
		return 0;
	}
	const char	*pLightMotionName = pLightMotion->GetAnimName(0);
	if( pLightMotionName==NULL ){
		return 0;
	}
	WK7Light			*pLight = WK7Light::Find(pLightMotionName);
	if( pLight==NULL ){
		return 0;
	}
	mPlayLights[lid].SetLight(pLight);
	pLight->AttachMotion(pLightMotion);
	pLight->PlayMotion  ();
	return 1;
}

int WK7Play::LightStopMotion(int lid)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	WK7Light	*pLight = mPlayLights[lid].GetLight();
	if( pLight==NULL ){
		return 0;
	}
	pLight->StopMotion  ();
	pLight->DetachMotion();
	return 1;
}

int WK7Play::LightWaitMotion(int lid)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	mPlayLights[lid].WaitMotion();
	return 1;
}

int WK7Play::LightSetMotionLoop(int lid,int flag)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	if( mPlayLights[lid].SetMotionLoop(flag!=0)==false ){
		return 0;
	}
	return 1;
}

int WK7Play::JimakuDisp(int id)
{
#pragma unused(id)
	return 1;
	//03/11/12 vOύX̂ߏC
	//return gpJimaku->mDisp((s16)id)==true? 1:0;
}

int WK7Play::JimakuWait(void)
{
	while( gpJimaku->mGetStatus()!=eJimakuStandby ){
		RetSys();
	}
	return 1;
}

int WK7Play::SePlay(int seid)
{
	// Cxgse̍Đ
	//return gpSound->mPlayEventSE(seid);
	seid;
	return 1;
}

int WK7Play::SeStop(int seid)
{
	// Cxgse̒~
	//return gpSound->mStopEventSE(seid);
	seid;
	return 1;
}

int WK7Play::StreamPlay(int stid)
{
	// Xg[Đ
	return gpSound->mPlayStreamNo(stid);
}

int WK7Play::StreamStop(void)
{
	// Xg[~
	return gpSound->mStopStream();
}

int WK7Play::StreamWait(int /*stid*/)
{
	// @\
	return 1;
}

int WK7Play::ModelCreateShadow(int mdlid)
{
	WK7PlayRenderObj	*pObj;
	if( mdlid >= 0 ){
		pObj = GetRenderObj(mdlid);
		if( pObj==NULL ){
			return 0;
		}
		pObj->CreateEdge();
		return 1;
	}else{
		int		i;
		for(i=0 ; i<mModelNum ; i++){
			pObj = GetRenderObj(i);
			pObj->CreateEdge();
		}
	}
	return 1;
}

int WK7Play::ModelSetShadowVisible(int mdlid,int flag)
{
	WK7PlayRenderObj	*pObj;
	if( mdlid >= 0 ){
		pObj = GetRenderObj(mdlid);
		if( pObj==NULL ){
			return 0;
		}
		pObj->SetEdgeShadowVisible(flag!=0);
		return 1;
	}else{
		int		i;
		for(i=0 ; i<mModelNum ; i++){
			pObj = GetRenderObj(i);
			pObj->SetEdgeShadowVisible(flag!=0);
		}
	}
	return 1;
}

int WK7Play::FadeWait(void)
{
	while( k7app.IsFade()==true ){
		RetSys();
	}
	return 1;
}

int WK7Play::ModelSetLight(int mdlid,int lightmdlid,int flag)
{
	WK7PlayRenderObj	*pObj,*pLightObj;
	pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pLightObj = GetRenderObj(lightmdlid);
	if( pLightObj==NULL ){
		return 0;
	}
	u32					lightobjid = pLightObj->GetID();
	WK7LightFindData	finddata;
	WK7Light			*pLight;
	bool				dispflag = flag? true : false;
	
	finddata.Initialize(WK7PLAY_TYPE_RES_MODEL(mPlayID),lightobjid);
	while( (pLight=WK7Light::Find(finddata))!=NULL ){
		pObj->SetDispLight(pLight,dispflag);
	}
	
	return 1;
}

int WK7Play::CameraSetStartPosition(f32 x,f32 y,f32 z)
{
	mCamStartPosition.x = x;
	mCamStartPosition.y = y;
	mCamStartPosition.z = z;
	mCamStartPosSet      = 1;
	return 1;
}

int WK7Play::CameraSetEndPosition(f32 x,f32 y,f32 z)
{
	mCamEndPosition.x = x;
	mCamEndPosition.y = y;
	mCamEndPosition.z = z;
	return 1;
}

int WK7Play::CameraSetStartTargetPosition(f32 x,f32 y,f32 z)
{
	mCamStartTargetPosition.x = x;
	mCamStartTargetPosition.y = y;
	mCamStartTargetPosition.z = z;
	mCamStartTargetPosSet     = 1;
	return 1;
}

int WK7Play::CameraSetEndTargetPosition(f32 x,f32 y,f32 z)
{
	mCamEndTargetPosition.x = x;
	mCamEndTargetPosition.y = y;
	mCamEndTargetPosition.z = z;
	return 1;
}

int WK7Play::CameraLinearIpLinearPath(f32 sec)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	if( mCamStartPosSet ){
		mCamStartPosSet = 0;
		WK7CAM->SetPosition(mCamStartPosition);
	}
	if( mCamStartTargetPosSet ){
		mCamStartTargetPosSet = 0;
		WK7CAM->SetTargetPosition(mCamStartTargetPosition);
	}
	if( !mCamEndFovSet ){
		mCamEndFov = WK7CAM->GetFov();
	}else{
		mCamEndFovSet = 0;
	}
	if( mCamStartFovSet ){
		mCamStartFovSet = 0;
		WK7CAM->SetFov(mCamStartFov);
	}
	WK7CAM->StartLinearInterpolateOnLinearPath(mCamEndPosition,
	                                           mCamEndTargetPosition,
	                                           60.0f * sec,
	                                           mCamEndFov);
	return 1;
}

int WK7Play::CameraDelayIpLinearPath(f32 delay)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	if( mCamStartPosSet ){
		mCamStartPosSet = 0;
		WK7CAM->SetPosition(mCamStartPosition);
	}
	if( mCamStartTargetPosSet ){
		mCamStartTargetPosSet = 0;
		WK7CAM->SetTargetPosition(mCamStartTargetPosition);
	}
	if( !mCamEndFovSet ){
		mCamEndFov = WK7CAM->GetFov();
	}else{
		mCamEndFovSet = 0;
	}
	if( mCamStartFovSet ){
		mCamStartFovSet = 0;
		WK7CAM->SetFov(mCamStartFov);
	}
	WK7CAM->StartDelayInterpolateOnLinearPath (mCamEndPosition,
	                                           mCamEndTargetPosition,
	                                           delay,
	                                           mCamEndFov);
	mCamStartPosSet       = 0;
	mCamStartTargetPosSet = 0;
	return 1;
}

int WK7Play::CameraLinearIpCirclePath(f32 sec)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	if( mCamStartPosSet ){
		mCamStartPosSet = 0;
		WK7CAM->SetPosition(mCamStartPosition);
	}
	if( mCamStartTargetPosSet ){
		mCamStartTargetPosSet = 0;
		WK7CAM->SetTargetPosition(mCamStartTargetPosition);
	}
	if( !mCamEndFovSet ){
		mCamEndFov = WK7CAM->GetFov();
	}else{
		mCamEndFovSet = 0;
	}
	if( mCamStartFovSet ){
		mCamStartFovSet = 0;
		WK7CAM->SetFov(mCamStartFov);
	}
	WK7CAM->StartLinearInterpolateOnCirclePath(mCamEndPosition,
	                                           mCamEndTargetPosition,
	                                           60.0f * sec,
	                                           mCamEndFov);
	return 1;
}

int WK7Play::CameraSetPosition(f32 x,f32 y,f32 z)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	WK7CAM->SetPosition(x,y,z);
	return 1;
}

int WK7Play::CameraSetTargetPosition(f32 x,f32 y,f32 z)
{
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	WK7CAM->SetTargetPosition(x,y,z);
	return 1;
}

int WK7Play::CameraSetFov(f32 fov)
{
	WK7CAM->SetFov(fov);
	return 1;
}

int WK7Play::CameraSetStartFov(f32 fov)
{
	mCamStartFov    = fov;
	mCamStartFovSet = 1;
	return 1;
}

int WK7Play::CameraSetEndFov(f32 fov)
{
	mCamEndFov    = fov;
	mCamEndFovSet = 1;
	return 1;
}

int WK7Play::StreamFadeVolume(int vol,f32 sec)
{
	u16		time = (u16)(sec * 200.0f);
	return gpSound->mFadeVolumeStream((u8)vol,time);
}

int WK7Play::CameraSetNear(f32 n)
{
	WK7CAM->SetNear(n);
	return 1;
}

int WK7Play::CameraSetFar(f32 f)
{
	WK7CAM->SetFar(f);
	return 1;
}

int WK7Play::CameraStopIp(void)
{
	WK7CAM->StopInterpolate();
	return 1;
}

int WK7Play::CreateExplodeEffect(f32 x,f32 y,f32 z,f32 size)
{
	Vec		point;
	point.x = x;
	point.y = y;
	point.z = z;
	if( size < 0.0f ){
		size = VParam[ED_EXP][0] / 10.0F;
	}
	return ::CreateHeliExpEffect(point)!=NULL;
}

enum{
	PLAYPARTS_HANDR = 0,
	PLAYPARTS_HANDL,
	PLAYPARTS_NUM,
};

const char *gpPartsName[] = {
	K7CHRBONE_GUNHANDR,
	K7CHRBONE_GUNHANDL,
};

int WK7Play::SetMuzzleEffect(int gunid,int gunmdlid,int gunmotid,int chrmdlid,int partsid)
{
	WK7Model		*pGunModel       = GetModel(gunmdlid);
	WK7ModelMotion	*pGunModelMotion = GetModelMotion(gunmotid);
	WK7Model		*pChrModel       = GetModel(chrmdlid);
	if( pGunModel==NULL || pGunModelMotion==NULL || pChrModel==NULL ){
		return 0;
	}
	WK7JObj			*pJObj;
	if( partsid < 0 ){
		pJObj = NULL;
	}else{
		if( partsid >=0 && partsid < PLAYPARTS_NUM ){
			pJObj = pChrModel->GetObj(gpPartsName[partsid]);
		}else{
			pJObj = NULL;
		}
		if( pJObj!=NULL ){
			pChrModel = NULL;
		}
	}
	SetUserGunModel((s8)gunid,pGunModel,pGunModelMotion);
	AttachGunModel ((s8)gunid,pJObj,pChrModel,0);
	return 1;
}

int WK7Play::CreateMuzzleEffect(int gunid,f32 height)
{
	::CreateMuzzleEffect((s8)gunid,height);
	return 1;
}

int WK7Play::ResetMuzzleEffect(int gunid)
{
	DetachGunModel   ((s8)gunid);
	ResetUserGunModel((s8)gunid);
	return 1;
}

int WK7Play::CreateHSExplodeEffect(f32 x,f32 y,f32 z)
{
	Vec		point;
	point.x = x;
	point.y = y;
	point.z = z;
	::CreateHSExplodeEffect(VParam[ED_HSEXP2][5],point,2.0F,100);
	return 1;
}

int WK7Play::ModelSetVfxEffect(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->AllocRenderID(true,WK7RENDEROBJ_RENDERID_HALF);
	pObj->EnableVfxEffect();
	return 1;
}

int WK7Play::ModelResetVfxEffect(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->AllocRenderID();
	pObj->ClearEffect();
	return 1;
}

int WK7Play::ModelSetAlpha(int mdlid,f32 alpha)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->SetAlpha(alpha);
	return 1;
}

int WK7Play::CreateGoreEffect(f32 x,f32 y,f32 w,f32 h)
{
	Vec		point;
	point.x = x;
	point.y = y;
	if( ::CreateGoreEffect(point,w,h,3)==NULL ){
		return 0;
	}
	return 1;
}

int WK7Play::CreateFlashEffect(int r,int g,int b,int a,f32 upsec,f32 downsec)
{
	Vec		point;
	GXColor	color;
	point.x = 
	point.y = 
	point.z = 0.0f;
	color.r = (u8)r;
	color.g = (u8)g;
	color.b = (u8)b;
	color.a = (u8)a;
	if( ::CreateFlashEffect(TRUE,point,0.0f,color,
	                       (u8)WK7_SEC_TO_FRAME(upsec),
	                       (u8)WK7_SEC_TO_FRAME(downsec),
	                       FALSE)==NULL ){
		return 0;
	}
	return 1;

}

int WK7Play::ModelPlaySeqMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	WK7PlayMotion	*pPlayMotion = GetNextPlayMotion();
	if( pPlayMotion==NULL ){
		return 0;
	}
	pPlayMotion->LoadWait();
	pObj->PlayMotion(pPlayMotion);
	ModelSetMotionLoop(mdlid,0);
	return 1;
}

int WK7Play::ModelPlaySeqMotionI(int mdlid,f32 sec)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	WK7PlayMotion	*pPlayMotion = GetNextPlayMotion();
	if( pPlayMotion==NULL ){
		return 0;
	}
	pPlayMotion->LoadWait();
	pObj->PlayMotion(pPlayMotion,60.0f * sec);
	ModelSetMotionLoop(mdlid,0);
	return 1;
}

int WK7Play::ModelPlaySeqMatMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	WK7PlayMotion	*pPlayMotion = GetNextPlayMotion();
	if( pPlayMotion==NULL ){
		return 0;
	}
	pPlayMotion->LoadWait();
	pObj->PlayMatMotion(pPlayMotion);
	ModelSetMotionLoop(mdlid,0);
	return 1;
}

int WK7Play::ModelPlaySeqShapeMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	WK7PlayMotion	*pPlayMotion = GetNextPlayMotion();
	if( pPlayMotion==NULL ){
		return 0;
	}
	pPlayMotion->LoadWait();
	pObj->PlayShapeMotion(pPlayMotion);
	ModelSetMotionLoop(mdlid,0);
	return 1;
}

int WK7Play::ModelStopSeqMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->StopMotion();
	return 1;
}

int WK7Play::ModelStopSeqMatMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->StopMatMotion();
	return 1;
}

int WK7Play::ModelStopSeqShapeMotion(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->StopShapeMotion();
	return 1;
}

int WK7Play::ModelStopSeqMotionAll(int mdlid)
{
	WK7PlayRenderObj	*pObj = GetRenderObj(mdlid);
	if( pObj==NULL ){
		return 0;
	}
	pObj->StopMotionAll();
	return 1;
}

int WK7Play::CameraPlaySeqMotion(void)
{
	if( CameraStopSeqMotion()==0 ){
		return 0;
	}
	mpCurrentCameraMotion = GetNextPlayMotion();
	if( mpCurrentCameraMotion==NULL ){
		return 0;
	}
	mpCurrentCameraMotion->LoadWait();
	WK7CameraMotion	*pCameraMotion = mpCurrentCameraMotion->GetCameraMotion();
	if( pCameraMotion==NULL ){
		return 0;
	}
	WK7CAM->AttachMotion(pCameraMotion);
	WK7CAM->PlayMotion  ();
	WK7CAM->SetMotionLoop(false);
	return 1;
}

int WK7Play::CameraPlaySeqMotionI(f32 sec)
{
	if( CameraStopSeqMotion()==0 ){
		return 0;
	}
	mpCurrentCameraMotion = GetNextPlayMotion();
	if( mpCurrentCameraMotion==NULL ){
		return 0;
	}
	mpCurrentCameraMotion->LoadWait();
	WK7CameraMotion	*pCameraMotion = mpCurrentCameraMotion->GetCameraMotion();
	if( pCameraMotion==NULL ){
		return 0;
	}
	WK7CAM->AttachMotion(pCameraMotion);
	WK7CAM->PlayMotion  (0.0f,true,60.0f * sec);
	WK7CAM->SetMotionLoop(false);
	return 1;
}

int WK7Play::CameraStopSeqMotion(void)
{
	if( mpCurrentCameraMotion==NULL ){
		return 1;
	}
	WK7CAM->StopMotion  ();
	WK7CAM->DetachMotion();
	delete mpCurrentCameraMotion;
	mpCurrentCameraMotion = NULL;
	return 1;
}

int WK7Play::LightPlaySeqMotion(int lid)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	WK7PlayMotion	*pPlayMotion = GetNextPlayMotion();
	if( pPlayMotion==NULL ){
		return 0;
	}
	pPlayMotion->LoadWait();
	WK7LightMotion	*pLightMotion = pPlayMotion->GetLightMotion();
	if( pLightMotion==NULL ){
		return 0;
	}
	const char		*pLightMotionName = pLightMotion->GetAnimName(0);
	if( pLightMotionName==NULL ){
		return 0;
	}
	WK7Light		*pLight = WK7Light::Find(pLightMotionName);
	if( pLight==NULL ){
		return 0;
	}
	mPlayLights[lid].SetLight(pLight);
	mPlayLights[lid].PlayMotion(pPlayMotion);
	mPlayLights[lid].SetMotionLoop(false);
	return 1;
}

int WK7Play::LightStopSeqMotion(int lid)
{
	if( lid < 0 || lid >= WK7PLAY_PLAYLIGHT_NUM ){
		return 0;
	}
	mPlayLights[lid].StopMotion();
	return 1;
}

bool WK7Play::InitializePlayEffect(void)
{
//	return true;
	WK7Model	*pModel = GetModel(0);
	// LGtFNgpݒ
	::SetCharGekiPlayEffectInfo(&mEffectInfo);
	if( pModel==NULL ){
		return true;
	}
	WK7PlayEffect		*pEffect;
	ghmGcMemSetAllocName("K7PLEFCP");
	// LN^ЉpGtFNg쐬
#ifdef	_DEBUG
	if( !GETCAPTUREMODE() ){
#endif
	if( (pEffect = new K7PlayEfCharaPresent(this,&mPlayEffectAnchor,NULL))!=NULL ){
		
	}
#ifdef	_DEBUG
	}
#endif
	
	WK7JObj		*pEffectGroup = pModel->GetObj("effect");
	if( pEffectGroup==NULL ){
		return true;
	}
	WK7RenderObjEffectInfo		playEffectInfo;
	
	if( (mpPlayEffect = ::CreateCharGekiEffect(11111,0))!=NULL ){
		mPlayEffectID = ::GetEffectID(mpPlayEffect);
	}
	playEffectInfo.Create();
	
	WK7JObj				*pEffectLocator;
	for(pEffectLocator=pEffectGroup->GetChild() ; pEffectLocator!=NULL ; pEffectLocator=pEffectLocator->GetNext()){
		if( pEffectLocator->GetName()==NULL ){
			continue;
		}
		if( (pEffect=CreatePlayEffect(pEffectLocator))==NULL ){
			continue;
		}
	}
	
	if( mpPlayEffect!=NULL && CheckEffectValidState(mpPlayEffect,mPlayEffectID)==TRUE ){
		playEffectInfo.Parent(mpPlayEffect);
		::SetEffectDrawFlagAll(mpPlayEffect,false);
	}
	
	return true;
}

bool WK7Play::FrameProcessPlayEffect(void)
{
	WK7PlayEffect	*pPlayEffect;
	WK7PlayEffect	*pNext;
	for(pPlayEffect=(WK7PlayEffect *)mPlayEffectAnchor.GetNext() ; pPlayEffect!=NULL ; pPlayEffect=pNext){
		pNext = pPlayEffect->GetNext();
		if( pPlayEffect->FrameProcess()==false ){
			delete pPlayEffect;
		}
	}
	return true;
}

bool WK7Play::TerminatePlayEffect(void)
{
	WK7PlayEffect	*pPlayEffect;
	WK7PlayEffect	*pNext;
	for(pPlayEffect=(WK7PlayEffect *)mPlayEffectAnchor.GetNext() ; pPlayEffect!=NULL ; pPlayEffect=pNext){
		pNext = pPlayEffect->GetNext();
		delete pPlayEffect;
	}
	return true;
}

WK7PlayEffectInfo* WK7PlayEffect::GetEffectInfo(void)
{
	if( mpContainer==NULL ){
		return NULL;
	}
	return mpContainer->GetEffectInfo();
}

u32 WK7PlayEffect::GetEffectID(WK7JObj *pJObj)
{
	if( pJObj==NULL ){
		return 0xffffffff;
	}
	return GetEffectID(pJObj->GetName());
}

u32 WK7PlayEffect::GetEffectID(const char *pName)
{
	if( pName==NULL ){
		return 0xffffffff;
	}
	// ID擾
	if( strstr(pName,"ef")!=pName ){
		return 0xffffffff;
	}
	int		i;
	u32		id = 0;
	pName += 2;
	for(i=3 ; i>=0 ; i--,pName++){
		if( *pName < '0' || *pName > '9' ){
			break;
		}
		id *= 10;
		id += (*pName-'0');
	}
	if( i>=0 || *pName!='\0' ){
		return 0xffffffff;
	}
	return id;
}

WK7PlayEffect::WK7PlayEffect(WK7Play *pPlay,ghmGcListObj *pAnchor,WK7JObj *pJObj) : ghmGcListObj(pAnchor)
{
	u32		id = GetEffectID(pJObj);
	mpContainer = pPlay;
	mID         = id;
	mTrigger    = false;
	if( (mpJObj=pJObj)!=NULL && id!=0xffffffff ){
		// ID擾
		pJObj->Hide(false,false);
	}
}

WK7PlayEffect::~WK7PlayEffect()
{
	FreeEffectAll();
}

WK7PlayEffectInstance* WK7PlayEffect::CreateInstance(WK7JObj */*pJObj*/)
{
	return NULL;
}

bool WK7PlayEffect::CheckSetup(void)
{
	if( mID==0xffffffff ){
		return true;
	}
	return true;
}

bool WK7PlayEffect::FrameProcess(void)
{
	if( mpJObj==NULL || mID==0xffffffff ){
		return false;
	}
	
	bool					visible = (mpJObj->GetFlag() & JOBJ_HIDDEN)? false : true;
	WK7PlayEffectInstance	*pInstance;
	WK7PlayEffectInstance	*pNext;
	bool					stopReq = false;
	
	// 
	if( mTrigger==false ){
		if( visible==true ){
			// GtFNg
			ghmGcMemSetAllocName("PlEfInst");
			pInstance = CreateInstance(mpJObj);
#ifdef	DEBUGPRINT
			if( pInstance!=NULL ){
				char		str[256];
				if( mpJObj==NULL ){
					sprintf(str,"Start Effect %04d!!",mID);
				}else{
					Vec		pos;
					mpJObj->GetPosition(&pos);
					sprintf(str,"Start Effect %04d (%.3f,%.3f,%.3f)!!",mID,pos.x,pos.y,pos.z);
				}
//				OSReport(str);
//				K7AppDebugPrint(str);
			}
#endif
			mTrigger = true;
		}
	}else{
		if( visible==false ){
			stopReq = true;
#ifdef	DEBUGPRINT
			char		str[256];
			sprintf(str,"Stop Effect %04d!!",mID);
//			OSReport(str);
//			K7AppDebugPrint(str);
#endif
			mTrigger = visible;
		}
	}
	
	for(pInstance=(WK7PlayEffectInstance *)mInstanceAnchor.GetNext() ; pInstance!=NULL ; pInstance=pNext){
		pNext = pInstance->GetNext();
		if( pInstance->FrameProcess(stopReq)==false ){
			delete pInstance;
		}
	}
	
	return true;
}

bool WK7PlayEffect::RenderProcess(void)
{
	WK7PlayEffectInstance	*pInstance;
	
	for(pInstance=(WK7PlayEffectInstance *)mInstanceAnchor.GetNext() ; pInstance!=NULL ; pInstance=pInstance->GetNext()){
		pInstance->RenderProcess();
	}
	
	return true;
}

bool WK7PlayEffect::FreeEffectAll(void)
{
	WK7PlayEffectInstance	*pInstance;
	WK7PlayEffectInstance	*pNext;
	for(pInstance=(WK7PlayEffectInstance *)mInstanceAnchor.GetNext() ; pInstance!=NULL ; pInstance=pNext){
		pNext = pInstance->GetNext();
		delete pInstance;
	}
	return true;
}

bool WK7PlayEffect::DispCharaName(s16 inNo,s16 inX,s16 inY,s32 inTick)
{
	if( mpContainer==NULL ){
		return false;
	}
	return mpContainer->DispCharaName(inNo,inX,inY,inTick);
}

bool WK7PlayEffect::HideCharaName(void)
{
	if( mpContainer==NULL ){
		return false;
	}
	return mpContainer->HideCharaName();
}

bool WK7PlayEffect::StopProcess(void)
{
	if( mpContainer==NULL ){
		return false;
	}
	return mpContainer->StopProcess();
}

bool WK7PlayEffect::RestartProcess(void)
{
	if( mpContainer==NULL ){
		return false;
	}
	return mpContainer->RestartProcess();
}

WK7Play* WK7PlayEffectInstance::GetPlay(void)
{
	if( mpContainer==NULL ){
		return NULL;
	}
	return mpContainer->GetContainer();
}

bool WK7PlayEffectInstance::DispCharaName(s16 inNo,s16 inX,s16 inY,s32 inTick)
{
	WK7Play		*pPlay = GetPlay();
	if( pPlay==NULL ){
		return false;
	}
	return pPlay->DispCharaName(inNo,inX,inY,inTick);
}

bool WK7PlayEffectInstance::HideCharaName(void)
{
	WK7Play		*pPlay = GetPlay();
	if( pPlay==NULL ){
		return false;
	}
	return pPlay->HideCharaName();
}

bool WK7PlayEffectInstance::StopProcess(void)
{
	WK7Play		*pPlay = GetPlay();
	if( pPlay==NULL ){
		return false;
	}
	return pPlay->StopProcess();
}

bool WK7PlayEffectInstance::RestartProcess(void)
{
	WK7Play		*pPlay = GetPlay();
	if( pPlay==NULL ){
		return false;
	}
	return pPlay->RestartProcess();
}

