////////////////////////////////////////////////////////////////////////////////
//
// Killer7 ScreenImage
//
// WK7ScreenImage.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/18/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7SCREENIMAGE_CPP__

//#include <dolphin.h>
//#include "..\Lib\ghmGcMem.h"
//#include ".\WK7Define.h"
#include ".\WK7ScreenImage.h"
#include ".\WK7Cam.h"

WK7ScreenImage::WK7ScreenImage()
{
	mpTexture     = NULL;
	mWidth        = WK7_SCREEN_WIDTH;
	mHeight       = WK7_SCREEN_HEIGHT;
	mFormat       = GX_TF_RGBA8;
	mWrapS        = GX_CLAMP;
	mWrapT        = GX_CLAMP;
	mMinFilt      = GX_NEAR;
	mMagFilt      = GX_NEAR;
	mTexSize      = 0;
	mClearColor   = (GXColor)K7APP_DEFAULT_CLEARCOLOR;
	mClearZ       = GX_MAX_Z24;
	mFlag         = 0;
	mCopyEnable   = 1;
	mZUpdate      = 1;
	mR.SetValue(0xff);
	mG.SetValue(0xff);
	mB.SetValue(0xff);
	mA.SetValue(0xff);
}

WK7ScreenImage::~WK7ScreenImage()
{
	FreeBuffer();
}

bool WK7ScreenImage::ScreenCopy(bool clear)
{
	if( mpTexture==NULL ){
		return false;
	}
	if( !mCopyEnable ){
		return true;
	}
//	DCInvalidateRange(mpTexture,mTexSize);
//	DCStoreRange(mpTexture,mTexSize);
//	DCFlushRange(mpTexture,mTexSize);
	GXSetTexCopySrc   (0,0,mWidth,mHeight);
	GXSetTexCopyDst   (mWidth,mHeight,mFormat,GX_FALSE);
	if( clear==true ){
		GXSetCopyClear(mClearColor,mClearZ);
		GXCopyTex     (mpTexture,GX_TRUE);
	}else{
		GXCopyTex     (mpTexture,GX_FALSE);
	}
	GXPixModeSync     ();
	GXInvalidateTexAll();
	InitTexObj();
	return true;
}

bool WK7ScreenImage::InitTexObj(void)
{
	GXTexFmt	fmt;
	switch( mFormat ){
	case	GX_CTF_R4:
	case	GX_CTF_Z4:
		fmt = GX_TF_I4;
		break;
	case	GX_CTF_RA4:
		fmt = GX_TF_IA4;
		break;
	case	GX_CTF_RA8:
	case	GX_CTF_RG8:
	case	GX_CTF_GB8:
		fmt = GX_TF_IA8;
		break;
	case	GX_CTF_A8:
	case	GX_CTF_R8:
	case	GX_CTF_G8:
	case	GX_CTF_B8:
	case	GX_CTF_Z8M:
	case	GX_CTF_Z8L:
		fmt = GX_TF_I8;
		break;
	case	GX_CTF_Z16L:
		fmt = GX_TF_Z16;
		break;
	default:
		fmt = mFormat;
		break;
	}
	GXInitTexObj   (&mTexObj,mpTexture,mWidth,mHeight,fmt,mWrapS,mWrapT,GX_FALSE);
	GXInitTexObjLOD(&mTexObj,mMinFilt,mMagFilt,0.0f,0.0f,0.0f,GX_FALSE,GX_FALSE,GX_ANISO_1);
	return true;
}

bool WK7ScreenImage::SetSize(u16 w,u16 h)
{
	mWidth  = w;
	mHeight = h;
	return true;
}

bool WK7ScreenImage::GetSize(u16 *w,u16 *h)
{
	if( w!=NULL ){ *w = mWidth;  }
	if( h!=NULL ){ *h = mHeight; }
	return true;
}

bool WK7ScreenImage::SetFormat(GXTexFmt fmt)
{
	mFormat = fmt;
	return true;
}

bool WK7ScreenImage::SetWrapMode(GXTexWrapMode s,GXTexWrapMode t)
{
	mWrapS = s;
	mWrapT = t;
	return true;
}

bool WK7ScreenImage::SetTexFilter(GXTexFilter min,GXTexFilter mag)
{
	mMinFilt = min;
	mMagFilt = mag;
	return true;
}

bool WK7ScreenImage::AllocBuffer(void *pBuff)
{
	if( FreeBuffer()==false ){
		return false;
	}
	mpTexture = pBuff;
	return true;
}

bool WK7ScreenImage::AllocBuffer(void)
{
	u32		texsize;
	if( (texsize = GXGetTexBufferSize(mWidth,mHeight,mFormat,GX_FALSE,0))==0 ){
		return false;
	}
	mTexSize  = texsize;
	GHMGCMEM_SAFE_FREE(mpTexture);
	if( (mpTexture = ghmGcMemAlloc((int)texsize))==NULL ){
		return false;
	}
//#ifdef	_DEBUG
	switch( mFormat ){
	case	GX_TF_Z24X8:
	case	GX_TF_Z16:
	case	GX_TF_Z8:
//		memset(mpTexture,0xff,mTexSize);
		memset(mpTexture,0x00,mTexSize);
		break;
	default:
		memset(mpTexture,0xff,mTexSize);
		break;
	}
	DCFlushRange(mpTexture,mTexSize);
	mAllocBuffer = 1;
//#endif
	return true;
}

bool WK7ScreenImage::FreeBuffer(void)
{
	if( mAllocBuffer ){
		GHMGCMEM_SAFE_FREE(mpTexture);
		mAllocBuffer = 0;
	}else{
		mpTexture = NULL;
	}
	return true;
}

bool WK7ScreenImage::FrameProcess(void)
{
	mR.FrameProcess();
	mG.FrameProcess();
	mB.FrameProcess();
	mA.FrameProcess();
	return true;
}

#define	CAM_FARZ		(1024.0f)
#define	CAM_NEARZ		(0.0f)
bool WK7ScreenImage::RenderProcess(bool flag)
{
	if( flag==true ){
		if( !mRender ){
			return true;
		}
	}else{
		if( !mPreRender ){
			return true;
		}
	}
	
	Mtx				view;
	Mtx44			proj;
	GXColor			color;
	
	color.r = (u8)mR.GetValue();
	color.g = (u8)mG.GetValue();
	color.b = (u8)mB.GetValue();
	color.a = (u8)mA.GetValue();
	
	WK7Cam::Set2DMatrix(view,proj,CAM_NEARZ,CAM_FARZ);
	GXLoadPosMtxImm  (view,GX_PNMTX0);
	GXLoadNrmMtxImm  (view,GX_PNMTX0);
	GXSetCurrentMtx  (GX_PNMTX0);
	GXSetProjection  (proj,GX_ORTHOGRAPHIC);
	
	if( mFullScreen ){
		GXSetScissor(0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	}else{
		HSD_RectU16		sc_rect;
		WK7CAM->GetScissor (sc_rect);
		GXSetScissor(sc_rect.xmin,sc_rect.ymin,(u32)(sc_rect.xmax-sc_rect.xmin),(u32)(sc_rect.ymax-sc_rect.ymin));
	}
	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
//	GXSetScissor     (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
//	GXSetViewport    (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetTevSwapModeTable(GX_TEV_SWAP0,GX_CH_RED,GX_CH_GREEN,GX_CH_BLUE,GX_CH_ALPHA);
	GXSetTevSwapMode     (GX_TEVSTAGE0,GX_TEV_SWAP0,GX_TEV_SWAP0);
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor (GX_COLOR0A0,color);
	
	GXSetCullMode     (GX_CULL_NONE);
	
	GXBool	zupdate = mZUpdate? GX_TRUE : GX_FALSE;
	GXSetZMode        (GX_TRUE,GX_ALWAYS,zupdate);
	GXSetZCompLoc     (GX_TRUE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXSetNumIndStages(0);
	
//	GXSetScissor      (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
//	GXSetViewport     (0.0f,0.0f,K7APP_SCREEN_WIDTH,K7APP_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXClearVtxDesc    ();
	GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
	GXSetVtxDesc      (GX_VA_TEX0,GX_DIRECT);
	
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ,GX_F32,0);
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST ,GX_F32,0);
	
	GXSetNumTexGens   (1);
	GXSetTexCoordGen  (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	
	GXSetDither       (GX_FALSE);
	
	LoadTexObj(GX_TEXMAP0);
	
	GXSetNumTevStages (1);
	GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(                 0.0f,                  0.0f,CAM_NEARZ);
		GXTexCoord2f32(0.0f,0.0f);
		GXPosition3f32((f32)WK7_SCREEN_WIDTH,                  0.0f,CAM_NEARZ);
		GXTexCoord2f32(1.0f,0.0f);
		GXPosition3f32((f32)WK7_SCREEN_WIDTH,(f32)WK7_SCREEN_HEIGHT,CAM_NEARZ);
		GXTexCoord2f32(1.0f,1.0f);
		GXPosition3f32(                 0.0f,(f32)WK7_SCREEN_HEIGHT,CAM_NEARZ);
		GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
	
	return true;
}

bool WK7ScreenImage::SetA(u32 a,u32 tick)
{
	mA.SetValue(a,tick);
	return true;
}

bool WK7ScreenImage::SetR(u32 r,u32 tick)
{
	mR.SetValue(r,tick);
	return true;
}

bool WK7ScreenImage::SetG(u32 g,u32 tick)
{
	mG.SetValue(g,tick);
	return true;
}

bool WK7ScreenImage::SetB(u32 b,u32 tick)
{
	mB.SetValue(b,tick);
	return true;
}

