////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Script
//
// WK7Script.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 03/28/2003
//
////////////////////////////////////////////////////////////////////////////////

#ifndef __WK7SCRIPT_H__
#define __WK7SCRIPT_H__

//#include "..\Lib\GhmGcTypes.h"
//#include "..\Lib\GhmGcTask.h"
#include ".\WSlSynt.def"

#define	WK7SCRIPT_STACK_DATA_SIZE		(5)
struct WK7ScriptStackData{
	u_short				mData[WK7SCRIPT_STACK_DATA_SIZE];
	u_char				mSize;
	u_char				mType;
	union{
		s64				mLong;
		int				mInt;
		float			mFloat;
	}mValue;
};

#define	WK7SCRIPTSTACK_WORKSIZE	(512)
class WK7ScriptStack{
private:
	u_short				*mpSpTop;			// X^bN|C^gbv
	u_short				*mpSp;				// X^bN|C^
	int					mSpCnt;				// X^bN|C^JE^
	u_short				mWork[WK7SCRIPTSTACK_WORKSIZE];
	static s16 GetShortVal(void *pdata);
	static s32 GetIntVal(void *pdata);
	static s64 GetLongVal(void *pdata);
public:
	bool Initialize(void);
	bool Terminate(void);
	bool Push(u_short *data,u_short size);
	bool Pop(WK7ScriptStackData* pdata);
	void Clear(void);
	WK7ScriptStack();
	virtual ~WK7ScriptStack();
};

class WK7ScriptValue{
protected:
	s8					*mpChar;
	int					mCharNum;
	
	s16					*mpShort;
	int					mShortNum;
	
	s32					*mpInt;
	int					mIntNum;
	
	s64					*mpLong;
	int					mLongNum;
	
	f32					*mpFloat;
	int					mFloatNum;
	
public:
	WK7ScriptValue();
	~WK7ScriptValue();
	
	bool Initialize(int charnum,int shortnum,int intnum,int longnum,int floatnum);
	bool Terminate(void);
	
	bool SetChar(int index,s8 val);
	bool GetChar(int index,s8 &val);
	s8 GetChar(int index);
	
	bool SetShort(int index,s16 val);
	bool GetShort(int index,s16 &val);
	s16 GetShort(int index);
	
	bool SetInt(int index,s32 val);
	bool GetInt(int index,s32 &val);
	s32 GetInt(int index);
	
	bool SetLong(int index,s64 val);
	bool GetLong(int index,s64 &val);
	s64 GetLong(int index);
	
	bool SetFloat(int index,f32 val);
	bool GetFloat(int index,f32 &val);
	f32 GetFloat(int index);
	
};

#define	WK7SCRIPT_PARAM_NUM		(2)
class WK7Script : public ghmGcListObj{
public:
	typedef bool ObjectCallMethod(WK7Script *pScript,u16 methodID,u16 objID);
	typedef struct{
		ObjectCallMethod				*mpMethod;
		int								mPropCharNum;
		int								mPropShortNum;
		int								mPropIntNum;
		int								mPropLongNum;
		int								mPropFloatNum;
	}ObjectType;
protected:
	
	static ghmGcListObj					mAnchor;
	
	ObjectType							*mpObjectType;		// IuWFNg^Cv
	int									mObjectTypeNum;		// IuWFNg^Cv
	WK7ScriptValue						*mpObjectProperty;	// IuWFNgvpeB
	WK7ScriptValue						*mpLocalValue;		// [Jϐ
	static WK7ScriptValue				*mpGlobalValue;		// O[oϐ
	
	u32									*mpLocalFlag;		// [JtO
	u32									mLocalFlagNum;		// [JtO
	static u32							*mpGlobalFlag;		// O[otO
	static u32							mGlobalFlagNum;		// O[otO
	
	u16									*mpEpTop;			// Cxg|C^gbv
	u16									*mpEp;				// Cxg|C^
	int									mEpCnt;				// Cxg|C^JE^
	WK7ScriptStack						mStack;				// CxgX^bN
	
	u32									mParam[WK7SCRIPT_PARAM_NUM];
															// p[^
	
	union{
		struct{
			u32							mStop    :  1;		// XNvg~
			u32							mFlagPad : 31;
		};
		u32								mFlag;				// tO
	};
	
	void ExecuteObjectMethod(void);
	void ExecutePlusOpe(void);
	void ExecuteMinusOpe(void);
	void ExecuteLogNotOpe(void);
	void ExecuteBitRevOpe(void);
	void ExecuteMulOpe(void);
	void ExecuteDivOpe(void);
	void ExecuteAddOpe(void);
	void ExecuteSubOpe(void);
	void ExecuteLeOpe(void);
	void ExecuteLeEquOpe(void);
	void ExecuteGrOpe(void);
	void ExecuteGrEquOpe(void);
	void ExecuteEquOpe(void);
	void ExecuteNotEquOpe(void);
	void ExecuteBitAndOpe(void);
	void ExecuteBitXorOpe(void);
	void ExecuteBitOrOpe(void);
	void ExecuteLogAndOpe(void);
	void ExecuteLogOrOpe(void);
	void ExecuteSubstOpe(void);
	void ExecuteIfSentence(void);
	void ExecuteGotoSentence(void);
	
	bool _ObjectCallMethod(u16 objType,u16 methodID,u16 objID);
	static u32* AllocFlag(int flagnum);
	static int SetFlag(u32 *pFlagTop,u32 flagnum,u32 id,bool flag);
	static int GetFlag(u32 *pFlagTop,u32 flagnum,u32 id);
public:
	
	inline WK7Script* GetPrev(void){ return (WK7Script *)ghmGcListObj::GetPrev(); }
	inline WK7Script* GetNext(void){ return (WK7Script *)ghmGcListObj::GetNext(); }
	
	void PushExpInt(s32 val);
	void PushExpLong(s64 val);
	void PushExpFloat(f32 val);
	bool Pop(WK7ScriptStackData* pdata);
	s32 PopInt(void);
	s64 PopLong(void);
	f32 PopFloat(void);
	
	bool LoadData(void *pData);
	int Parse(void);
	bool RetSys(void);
	bool Term(void);
	bool TimeWaitTick(int tick);
	bool TimeWaitSec(f32 sec);
	bool Reset(void);
	
	WK7Script(ObjectType *pObjectType,int lflagnum=0,int lcharnum=0,int lshortnum=0,int lintnum=0,int llongnum=0,int lfloatnum=0);
	virtual ~WK7Script();
	
	static bool Initialize(int gflagnum=0,int gcharnum=0,int gshortnum=0,int gintnum=0,int glongnum=0,int gfloatnum=0);
	static bool ReleaseAll(void);
	static bool Terminate(void);
	
	bool SetFlag(u32 id,bool flag);
	bool GetFlag(u32 id);
	bool SetLocalFlag(u32 id,bool flag);
	bool GetLocalFlag(u32 id);
	static bool SetGlobalFlag(u32 id,bool flag);
	static bool GetGlobalFlag(u32 id);
	
	bool SetParam(int index,u32 param);
	bool GetParam(int index,u32 &param);
	u32 GetParam(int index);
	
	inline bool IsStopped(void){ return mStop? true : false; }
	inline void Stop(void){ mStop=1; }
	inline void Restart(void){ mStop=0; }
};

#define	WK7SCRPIT_OBJECTTYPE_TABLE_END	{ ((WK7Script::ObjectCallMethod *)0xffffffff), }

#endif	//__WK7SCRIPT_H__

