////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Tutorial
//
// WK7Tutorial.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 07/24/2004
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7TUTORIAL_CPP__
	
#include "WK7Tutorial.h"
#include "..\Fujita\C_Font.h"
#include "..\Matsu\MK7Sound.h"

#define	WK7TUTORIAL_BASE_PRIO		(100)
#define	WK7TUTORIAL_BACK_PRIO		(WK7TUTORIAL_BASE_PRIO)
#define	WK7TUTORIAL_IWAZARU_PRIO	(WK7TUTORIAL_BASE_PRIO+16)
#define	WK7TUTORIAL_SELECT_PRIO		(WK7TUTORIAL_BASE_PRIO+32)
#define	WK7TUTORIAL_MESSAGE_PRIO	(WK7TUTORIAL_BASE_PRIO+48)

#define	WK7TUTORIAL_FADE_TICK		(2400)
#define	WK7TUTORIAL_FADE_TICK_QUICK	(600)
#define	WK7TUTORIAL_MOVE_TICK		(600)

#define	WK7TUTORIAL_SELECTEDCOLOR		(0xff)
#define	WK7TUTORIAL_NOTSELECTEDCOLOR	(0x80)

#define	WK7TUTORIAL_SE_VOLUME			(SOUND_SYSTEM_SE_VOLUME)

#define	WK7TUTORIAL_BUTTON_A			(0)
#define	WK7TUTORIAL_BUTTON_B			(1)
#define	WK7TUTORIAL_BUTTON_L			(2)
#define	WK7TUTORIAL_BUTTON_R			(3)
#define	WK7TUTORIAL_BUTTON_KEY			(4)
#define	WK7TUTORIAL_BUTTON_JS			(5)

WK7Tutorial	*gpInstance = NULL;

WK7Tutorial* WK7Tutorial::GetInstance(void)
{
//	if( gpInstance==NULL ){
//		gpInstance = new WK7Tutorial;
//	}
	return gpInstance;
}

bool WK7Tutorial::IsTutorialMode(void)
{
	return (gpInstance!=NULL);
}

bool WK7Tutorial::Initialize(void)
{
	if( gpInstance==NULL ){
		gpInstance = new WK7Tutorial;
	}
	return (gpInstance!=NULL);
}

bool WK7Tutorial::Terminate(void)
{
	GHM_SAFE_DELETE(gpInstance);
	return true;
}

WK7Tutorial::WK7Tutorial()
{
	mK7Sprite	*pPrim;
	mProc   = 0;
	mLoaded = false;
	mpFile  = NULL;
	mpData     = NULL;
	mpResource = NULL;
	mpIwazaruTexture   = NULL;
	mpMessageTexture   = NULL;
	mpMessageExTexture = NULL;
	mppButtonTexture   = NULL;
	mIwazaruTextureNum = -1;
	mMessageTextureNum = -1;
	mButtonGroupNum    = -1;
	mpButtonTextureNum = NULL;
	mCurrentButtonIndex = -1;
	mCurrentButtonIndexEx = -1;
	if( (mpBackPrim = new WK7TutorialPrim((stTexture *)NULL,&mPrimAnchor))!=NULL &&
	    (pPrim=mpBackPrim->GetPrim())!=NULL ){
#define	BACKPRIM_Y			(350)
		mpBackPrim->SetXY(0,BACKPRIM_Y);
		pPrim->mSetDispFlag(true);
		mpBackPrim->SetWH(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT-BACKPRIM_Y);
		mpBackPrim->SetRGB(0x5a0a0a00);
		mpBackPrim->SetAlpha(0);
		mpBackPrim->SetPriority(WK7TUTORIAL_BACK_PRIO);
	}
	mBackAlpha.SetValue(0);
	mpIwazaruPrim = NULL;
	mpMessagePrim = NULL;
	mpMessageExPrim = NULL;
	mpButtonPrim = NULL;
	mpButtonPrimEx = NULL;
	mCurrentIwazaruIndex = -1;
	mCurrentMessageIndex = -1;
	mMessageSE = -1;
	mRender = false;
	mSelectMode  = false;
	mSelectIndex = -1;
	mCurrentSelectIndex = -1;
	mMessageProc = 0;
	mMessageExX.SetValue(0.0f);
	mMessageExY.SetValue(0.0f);
	mButtonCounter = 0;
	mButtonTextureID = 0;
	mButtonTextureIDEx = 0;
}

WK7Tutorial::~WK7Tutorial()
{
	int		i;
	if( mppButtonTexture!=NULL && mButtonGroupNum>0 ){
		for(i=0 ; i<mButtonGroupNum ; i++){
			GHMGCMEM_SAFE_FREE(mppButtonTexture[i]);
		}
	}
	GHMGCMEM_SAFE_FREE(mpIwazaruTexture  );
	GHMGCMEM_SAFE_FREE(mpMessageTexture  );
	GHMGCMEM_SAFE_FREE(mpMessageExTexture);
	GHMGCMEM_SAFE_FREE(mpButtonTextureNum);
	GHMGCMEM_SAFE_FREE(mppButtonTexture  );
	GHMGCMEM_SAFE_FREE(mpData            );
	GHM_SAFE_DELETE   (mpFile            );
}

bool WK7Tutorial::SetIwazaru(int index)
{
	if( index>=mIwazaruTextureNum || mpIwazaruPrim==NULL ){
		return false;
	}
	if( index==mCurrentIwazaruIndex ){
		return true;
	}
	if( mCurrentIwazaruIndex >= 0 ){
		WK7TutorialPrim		*pBlur = new WK7TutorialPrim(mpIwazaruPrim,&mPrimAnchor);
		if( pBlur!=NULL ){
			pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
			pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
			pBlur->SetPriority(WK7TUTORIAL_IWAZARU_PRIO-1);
			pBlur->SetAutoRelease(true);
		}
	}
	mpIwazaruPrim->SetAlpha(0);
	if( (mCurrentIwazaruIndex = index) >= 0 ){
		mK7Sprite	*pPrim = mpIwazaruPrim->GetPrim();
		mpIwazaruPrim->SetTexture(mpIwazaruTexture + index);
		mpIwazaruPrim->SetWH((s16)mpIwazaruTexture[index].tex->mWidth,
		                     (s16)mpIwazaruTexture[index].tex->mHeight);
		switch( index ){
		case	3:
			mpIwazaruPrim->SetXY(470,0);
			break;
		default:
			mpIwazaruPrim->SetXY(387,0);
			break;
		}
		pPrim->mSetUV0 (0,0);
//		pPrim->mSetRGBA(0xffffffcc);
		mpIwazaruPrim->SetRGB  (0xffffffcc);
		mpIwazaruPrim->SetAlpha(0xcc,WK7TUTORIAL_FADE_TICK);
	}
	return true;
}

bool WK7Tutorial::SetMessage(int index)
{
	if( index>=mMessageTextureNum ||
		mpMessagePrim  ==NULL ||
		mpMessageExPrim==NULL ||
		mpButtonPrim   ==NULL ){
		return false;
	}
	if( index==mCurrentMessageIndex ){
		return true;
	}
	WK7TutorialPrim		*pBlur;
	if( mCurrentMessageIndex >= 0 ){
		if( (pBlur = new WK7TutorialPrim(mpMessagePrim,&mPrimAnchor))!=NULL ){
			pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
			pBlur->SetPriority(WK7TUTORIAL_MESSAGE_PRIO-3);
			pBlur->SetAutoRelease(true);
		}
		if( mCurrentMessageIndex==TUTOMSG_JUNCTION ){
			if( (pBlur = new WK7TutorialPrim(mpMessageExPrim,&mPrimAnchor))!=NULL ){
				pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
				pBlur->SetPriority(WK7TUTORIAL_MESSAGE_PRIO-1);
				pBlur->SetAutoRelease(true);
			}
		}
	}
	if( mCurrentButtonIndex >= 0 ){
		if( (pBlur = new WK7TutorialPrim(mpButtonPrim,&mPrimAnchor))!=NULL ){
			pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
			pBlur->SetPriority(WK7TUTORIAL_MESSAGE_PRIO-2);
			pBlur->SetAutoRelease(true);
		}
	}
	if( mCurrentButtonIndexEx >= 0 ){
		if( (pBlur = new WK7TutorialPrim(mpButtonPrimEx,&mPrimAnchor))!=NULL ){
			pBlur->SetAlpha(0,WK7TUTORIAL_FADE_TICK);
			pBlur->SetPriority(WK7TUTORIAL_MESSAGE_PRIO-2);
			pBlur->SetAutoRelease(true);
		}
	}
	if( mMessageSE >= 0 ){
		gpSound->mSetVolumeSENo(mMessageSE,0,100);
		mMessageSE = -1;
	}
	mpMessagePrim->SetAlpha(0);
	
	int		buttonID = -1;
	f32		x,y;
	
	if( (mCurrentMessageIndex = index) >= 0 ){
		
		mK7Sprite	*pPrim = mpMessagePrim->GetPrim();
		
		switch( index ){
		case	TUTOMSG_TUTO_QUERY:   x=151.0f; y=374.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL01,WK7TUTORIAL_SE_VOLUME); break;
		case	TUTOMSG_RUN:          x=160.0f; y=363.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL02,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_A;   break;
		case	TUTOMSG_TURN:         x=163.0f; y=369.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL03,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_B;   break;
		case	TUTOMSG_KAMAE:        x=189.0f; y=367.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL04,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_R;   break;
		case	TUTOMSG_SCAN:         x=185.0f; y=366.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL05,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_L;   break;
		case	TUTOMSG_ATTACK:       x=162.0f; y=367.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL06,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_A;   break;
		case	TUTOMSG_LOCKON:       x=163.0f; y=367.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL07,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_B;   break;
		case	TUTOMSG_TARGETCHANGE: x=162.0f; y=362.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL08,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_KEY; break;
		case	TUTOMSG_JUNCTION:     x=114.0f; y=364.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL09,WK7TUTORIAL_SE_VOLUME); buttonID = WK7TUTORIAL_BUTTON_JS;  break;
		case	TUTOMSG_TRYAGAIN:     x=110.0f; y=372.0f-10.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL10,WK7TUTORIAL_SE_VOLUME); break;
		default:
			index = -1;
//		case	TUTOMSG_GO_QUERY:     x=111.0f; y=372.0f; mMessageSE = gpSound->mPlaySE(SFX_TUTORIAL11,WK7TUTORIAL_SE_VOLUME); break;
		}
		
		if( index >= 0 ){
			mpMessagePrim->SetTexture(mpMessageTexture + index);
			mpMessagePrim->SetWH((s16)mpMessageTexture[index].tex->mWidth,
			                     (s16)mpMessageTexture[index].tex->mHeight);
			mpMessagePrim->SetXY(x,y);
			pPrim->mSetUV0 (0,0);
			mpMessagePrim->SetAlpha(0xff,WK7TUTORIAL_FADE_TICK);
		}else{
			mpMessagePrim->SetAlpha(0x00,WK7TUTORIAL_FADE_TICK);
		}
		
	}
	
	mMessageProc = 0;
	if( index==TUTOMSG_JUNCTION ){
		mpMessageExPrim->SetAlpha(0xff,WK7TUTORIAL_FADE_TICK);
	}else{
		mpMessageExPrim->SetAlpha(0x00,WK7TUTORIAL_FADE_TICK);
	}
	
	mButtonCounter = 0;
	mButtonTextureID = 0;
	mButtonTextureIDEx = 0;
	
	switch( index ){
//	case	TUTOMSG_TUTO_QUERY:
	case	TUTOMSG_RUN:          x=158.0f; y=371.0f-10.0f; break;
	case	TUTOMSG_TURN:         x=158.0f; y=371.0f-10.0f; break;
	case	TUTOMSG_KAMAE:        x=106.0f; y=372.0f-10.0f; break;
	case	TUTOMSG_SCAN:         x=106.0f; y=372.0f-10.0f; break;
	case	TUTOMSG_ATTACK:       x=158.0f; y=371.0f-10.0f; break;
	case	TUTOMSG_LOCKON:       x=158.0f; y=371.0f-10.0f; break;
	case	TUTOMSG_TARGETCHANGE: x=158.0f; y=371.0f-10.0f; break;
	case	TUTOMSG_JUNCTION:     x= 52.0f; y=364.0f-10.0f; break;
//	case	TUTOMSG_TRYAGAIN:
	default:
		x=0.0f; y=0.0f;
		break;
	}
	switch( buttonID ){
	case	WK7TUTORIAL_BUTTON_A:
	case	WK7TUTORIAL_BUTTON_B:
	case	WK7TUTORIAL_BUTTON_KEY:
	case	WK7TUTORIAL_BUTTON_L:
	case	WK7TUTORIAL_BUTTON_R:
	case	WK7TUTORIAL_BUTTON_JS:
		break;
	default:
		buttonID = -1;
		mpButtonPrim->SetAlpha(0x00);
		break;
	}
	if( buttonID >= 0 ){
		mpButtonPrim->SetTexture(&mppButtonTexture[buttonID][0]);
		mpButtonPrim->SetWH((s16)mppButtonTexture[buttonID][0].tex->mWidth,
		                    (s16)mppButtonTexture[buttonID][0].tex->mHeight);
		mpButtonPrim->SetXY(x,y);
		mpButtonPrim->SetAlpha(0xff,WK7TUTORIAL_FADE_TICK);
	}
	if( mpButtonPrimEx != NULL ){
		if( index==TUTOMSG_JUNCTION ){
			mpButtonPrimEx->SetAlpha(0xff,WK7TUTORIAL_FADE_TICK);
			mCurrentButtonIndexEx = WK7TUTORIAL_BUTTON_A;
		}else{
			mpButtonPrimEx->SetAlpha(0x00,WK7TUTORIAL_FADE_TICK);
			mCurrentButtonIndexEx = -1;
		}
	}
	mCurrentButtonIndex = buttonID;
	
	return true;
}

bool WK7Tutorial::DispSelect(bool disp)
{
	if( mRender==false ){
		disp = false;
	}
	int		i;
	for(i=0 ; i<2 ; i++){
		mSelect[i].Disp(disp);
	}
	return true;
}

bool WK7Tutorial::InitPrim(void)
{
	int			i,j;
	if( (mpResource = new(mpData) ghmGcResLink)!=NULL ){
		ghmGcResGroup	*pGroup;
		ghmGcResGroup	*pSubGroup;
		if( (pGroup=mpResource->GetGroup(0))!=NULL ){
			mIwazaruTextureNum = (int)pGroup->GetResourceNum();
			if( mIwazaruTextureNum > 0 && (mpIwazaruTexture = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture) * mIwazaruTextureNum)))!=NULL ){
				for(i=0 ; i<mIwazaruTextureNum ; i++){
					mpIwazaruTexture[i].tex = (GhmGcTexHeader *)pGroup->GetResource(i);
					mK7InitTexture(mpIwazaruTexture + i);
				}
				if( (mpIwazaruPrim = new WK7TutorialPrim(mpIwazaruTexture,&mPrimAnchor))!=NULL ){
					mpIwazaruPrim->SetPriority(WK7TUTORIAL_IWAZARU_PRIO);
				}
				SetIwazaru(-1);
			}
		}
		if( mpResource->GetResource(3)!=NULL ){
			if( (mpMessageExTexture = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture))))!=NULL ){
				mpMessageExTexture->tex = (GhmGcTexHeader *)mpResource->GetResource(3);
				mK7InitTexture(mpMessageExTexture);
				if( (mpMessageExPrim = new WK7TutorialPrim(mpMessageExTexture,&mPrimAnchor))!=NULL ){
					mpMessageExPrim->SetPriority(WK7TUTORIAL_MESSAGE_PRIO+2);
					mpMessageExPrim->SetAlpha(0x00);
				}
			}
		}
		if( (pGroup=mpResource->GetGroup(4))!=NULL ){
			mButtonGroupNum = (int)pGroup->GetResourceNum();
			if( mButtonGroupNum > 0 ){
				if( (mppButtonTexture   = (stTexture**)K7AppSmallMemAlloc((int)(sizeof(stTexture*) * mButtonGroupNum)))!=NULL ){
					for(j=0 ; j<mButtonGroupNum ; j++){
						mppButtonTexture[j] = NULL;
					}
				}
				if( (mpButtonTextureNum = (int*)K7AppSmallMemAlloc((int)(sizeof(int) * mButtonGroupNum)))!=NULL ){
					for(j=0 ; j<mButtonGroupNum ; j++){
						mpButtonTextureNum[j] = 0;
					}
				}
				if( mpButtonTextureNum!=NULL && mppButtonTexture!=NULL ){
					for(j=0 ; j<mButtonGroupNum ; j++){
						if( (pSubGroup=pGroup->GetGroup(j))==NULL ){
							continue;
						}
						mpButtonTextureNum[j] = (int)pSubGroup->GetResourceNum();
						if( mpButtonTextureNum[j] > 0 && (mppButtonTexture[j] = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture) * mpButtonTextureNum[j])))!=NULL ){
							for(i=0 ; i<mpButtonTextureNum[j] ; i++){
								mppButtonTexture[j][i].tex = (GhmGcTexHeader *)pSubGroup->GetResource(i);
								mK7InitTexture(mppButtonTexture[j] + i);
							}
						}
					}
					if( (mpButtonPrim = new WK7TutorialPrim(mppButtonTexture[0],&mPrimAnchor))!=NULL ){
						mpButtonPrim->SetPriority(WK7TUTORIAL_MESSAGE_PRIO+1);
						mpButtonPrim->SetAlpha(0x00);
					}
					if( (mpButtonPrimEx = new WK7TutorialPrim(mppButtonTexture[0] + WK7TUTORIAL_BUTTON_A,&mPrimAnchor))!=NULL ){
						mpButtonPrimEx->SetPriority(WK7TUTORIAL_MESSAGE_PRIO+1);
						mpButtonPrimEx->SetAlpha(0x00);
						mpButtonPrimEx->SetXY(206,424-10);
//						mpButtonPrimEx->SetXY(206,424);
//						mpButtonPrimEx->SetXY(136,414);
					}
				}
			}
		}
		if( (pGroup=mpResource->GetGroup(1))!=NULL ){
			mMessageTextureNum = (int)pGroup->GetResourceNum();
			if( mMessageTextureNum > 0 && (mpMessageTexture = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture) * mMessageTextureNum)))!=NULL ){
				for(i=0 ; i<mMessageTextureNum ; i++){
					mpMessageTexture[i].tex = (GhmGcTexHeader *)pGroup->GetResource(i);
					mK7InitTexture(mpMessageTexture + i);
				}
				if( (mpMessagePrim = new WK7TutorialPrim(mpMessageTexture,&mPrimAnchor))!=NULL ){
					mpMessagePrim->SetPriority(WK7TUTORIAL_MESSAGE_PRIO);
				}
				SetMessage(-1);
			}
		}
		if( (pGroup=mpResource->GetGroup(2))!=NULL ){
			ghmGcResGroup	*pSatuGroup = pGroup->GetGroup(0);
			f32				x[2],y[2];
			for(i=0 ; i<2 ; i++){
				if( (pSubGroup=pGroup->GetGroup(i+1))==NULL ){
					continue;
				}
				switch( i ){
				case	0:
					x[0] =  27.0f; y[0] =  67.0f;
					x[1] = 189.0f; y[1] = 214.0f;
//					x[0] = 122.0f; y[0] = 140.0f;
//					x[1] = 207.0f; y[1] = 196.0f;
//					x[0] = 115.0f; y[0] = 114.0f;
//					x[1] = 220.0f; y[1] = 154.0f;
					break;
				default:
					x[0] = 239.0f; y[0] =  36.0f;
					x[1] = 392.0f; y[1] = 123.0f;
//					x[0] = 298.0f; y[0] =  86.0f;
//					x[1] = 346.0f; y[1] = 133.0f;
//					x[0] = 322.0f; y[0] = 114.0f;
//					x[1] = 387.0f; y[1] = 154.0f;
					break;
				}
				mSelect[i].InitPrim(mPrimAnchor,pSatuGroup,x[0],y[0],pSubGroup,x[1],y[1]);
				mSelect[i].SetRGB(0,0xff0000ff);
				mSelect[i].SetRGB(1,0xffffffff);
			}
			HideSelect();
		}
	}
	return true;
}

bool WK7Tutorial::FrameProcess(void)
{
	switch( mProc ){
	case	0:
		if( (mpFile=new ghmGcFile)==NULL ){
			break;
		}
		mProc++;
	case	1:
		if( mpFile->Open("/Wat/Tutorial.rsl")==false ){
			break;
		}
		mProc++;
	case	2:
		if( (mpData=mpFile->Read())==NULL ){
			break;
		}
		mProc++;
	case	3:
		if( mpFile->GetState()!=ghmGcFile::READFINISH ){
			break;
		}
		mProc++;
		mLoaded = true;
		InitPrim();
	case	4:
		if( mSelectMode==true ){
			if( mSelectIndex < 0 ){
				bool	sound = false;
				if( HSD_PadGetTrigger(0) & (HSD_PAD_STICK_RIGHT|HSD_PAD_RIGHT) ){
					mCurrentSelectIndex = 1;
					mSelect[0].Select(false);
					mSelect[1].Select(true );
					sound = true;
				}
				if( HSD_PadGetTrigger(0) & (HSD_PAD_STICK_LEFT|HSD_PAD_LEFT) ){
					mCurrentSelectIndex = 0;
					mSelect[0].Select(true );
					mSelect[1].Select(false);
					sound = true;
				}
				if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
					if( mCurrentSelectIndex >= 0 ){
						mSelectIndex = mCurrentSelectIndex;
						sound = false;
						gpSound->mPlaySE(SFX_SYS_DECIDE);
					}
				}
				if( sound==true ){
					gpSound->mPlaySE(SFX_SYS_CURSOR);
	//				gpSound->mPlaySE(SFX_SYS_HOUKOU);
				}
			}
		}else{
			switch( mCurrentIwazaruIndex ){
			case	0:
			case	1:
			case	2:
static	int		tick = 0;
				if( tick >= WK7_TICK_PER_FRAME ){
					tick -= WK7_TICK_PER_FRAME;
					break;
				}
				int		id = HSD_Randi(3);
				if( mCurrentIwazaruIndex!=id ){
					DispIwazaru(id);
				}
				tick = WK7_SEC_TO_TICK(5.0f);
				break;
			}
		}
//		if( HSD_PadGetTrigger(1) & HSD_PAD_X ){
//			if( mpSelectPrim[0]!=NULL ){
//				if( mpSelectPrim[0]->GetAlpha().GetDstValue()==0 ){
//					DispSelect(true);
//				}else{
//					DispSelect(false);
//				}
//			}
//		}
//		if( HSD_PadGetTrigger(1) & HSD_PAD_A ){
//			if( mCurrentIwazaruIndex+1>=mIwazaruTextureNum ){
//				HideIwazaru();
//			}else{
//				DispIwazaru(mCurrentIwazaruIndex+1);
//			}
//		}
//		if( HSD_PadGetTrigger(1) & HSD_PAD_Y ){
//			if( mCurrentMessageIndex+1>=mMessageTextureNum ){
//				HideMessage();
//			}else{
//				DispMessage(mCurrentMessageIndex+1);
//			}
//		}
		break;
	default:
		break;
	}
	int			i;
	for(i=0 ; i<2 ; i++){
		mSelect[i].FrameProcess();
	}
	mBackAlpha.FrameProcess();
	if( mpBackPrim!=NULL ){
		mpBackPrim->SetAlpha(mBackAlpha.GetValue());
	}
	mMessageExX.FrameProcess();
	mMessageExY.FrameProcess();
#define	EX_OFSX		(14.0f)
#define	EX_OFSY		(14.0f-10.0f)
	if( mpMessageExPrim!=NULL && mCurrentMessageIndex==TUTOMSG_JUNCTION ){
		switch( mMessageProc ){
		case	0:
			if( mMessageExX.IsAnimating()==true ){
				break;
			}
			mMessageExX.SetValue( 47.0f + EX_OFSX,WK7TUTORIAL_MOVE_TICK);
			mMessageExY.SetValue(365.0f + EX_OFSY,WK7TUTORIAL_MOVE_TICK);
			mMessageProc++;
			break;
		case	1:
			if( mMessageExX.IsAnimating()==true ){
				break;
			}
			mMessageExX.SetValue( 47.0f + EX_OFSX,WK7TUTORIAL_MOVE_TICK);
			mMessageExY.SetValue(373.0f + EX_OFSY,WK7TUTORIAL_MOVE_TICK);
			mMessageProc++;
			break;
		case	2:
			if( mMessageExX.IsAnimating()==true ){
				break;
			}
			mMessageExX.SetValue( 61.0f + EX_OFSX,WK7TUTORIAL_MOVE_TICK);
			mMessageExY.SetValue(365.0f + EX_OFSY,WK7TUTORIAL_MOVE_TICK);
			mMessageProc++;
			break;
		case	3:
			if( mMessageExX.IsAnimating()==true ){
				break;
			}
			mMessageExX.SetValue( 61.0f + EX_OFSX,WK7TUTORIAL_MOVE_TICK);
			mMessageExY.SetValue(373.0f + EX_OFSY,WK7TUTORIAL_MOVE_TICK);
			mMessageProc = 0;
			break;
		}
		mpMessageExPrim->SetXY(mMessageExX.GetValue(),mMessageExY.GetValue());
	}
	if( mpButtonTextureNum!=NULL ){
		if( mButtonCounter  < WK7TUTORIAL_MOVE_TICK - WK7_TICK_PER_FRAME ){
			mButtonCounter += WK7_TICK_PER_FRAME;
		}else{
			if( mCurrentButtonIndex >= 0 && mpButtonTextureNum[mCurrentButtonIndex]!=NULL ){
				mButtonTextureID++;
				if( mButtonTextureID >= mpButtonTextureNum[mCurrentButtonIndex] ){
					mButtonTextureID  = 0;
				}
				mpButtonPrim->SetTexture(&mppButtonTexture[mCurrentButtonIndex][mButtonTextureID]);
			}
			if( mCurrentButtonIndexEx >= 0 && mpButtonTextureNum[mCurrentButtonIndexEx]!=NULL ){
				mButtonTextureIDEx++;
				if( mButtonTextureIDEx >= mpButtonTextureNum[mCurrentButtonIndexEx] ){
					mButtonTextureIDEx  = 0;
				}
				mpButtonPrimEx->SetTexture(&mppButtonTexture[mCurrentButtonIndexEx][mButtonTextureIDEx]);
			}
			mButtonCounter -= WK7TUTORIAL_MOVE_TICK;
		}
	}
	WK7TutorialPrim		*pPrim,*pNext;
	for(pPrim=(WK7TutorialPrim *)mPrimAnchor.GetNext() ; pPrim!=NULL ; pPrim=pNext){
		pNext = pPrim->GetNext();
		if( pPrim->FrameProcess()==false ){
			delete pPrim;
		}
	}
	return true;
}

void WK7Tutorial::SetRender(bool flag)
{
	mRender = flag;
}

bool WK7Tutorial::RenderProcess(void)
{
	WK7TutorialPrim		*pPrim;
	for(pPrim=(WK7TutorialPrim *)mPrimAnchor.GetNext() ; pPrim!=NULL ; pPrim=pPrim->GetNext()){
		pPrim->RenderProcess();
	}
	if( mRender==false ){
		return true;
	}
	
//	C_Paper		paper(32,32);
//	PRINTF("Panel Tutorial Test\n");
	if( mLoaded==false ){
		return true;
	}
	
	return true;
}

bool WK7Tutorial::IsLoading(void)
{
	return (mLoaded==false);
}

bool WK7Tutorial::IsHiddenIwazaru(void)
{
	if( mCurrentIwazaruIndex >= 0 ){
		return false;
	}
	if( mpIwazaruPrim==NULL ){
		return true;
	}
	if( mpIwazaruPrim->GetAlpha().GetValue()!=0 ){
		return false;
	}
	return true;
}

bool WK7Tutorial::IsHiddenMessage(void)
{
	if( mCurrentMessageIndex >= 0 ){
		return false;
	}
	if( mpMessagePrim==NULL ){
		return true;
	}
	if( mpMessagePrim->GetAlpha().GetValue()!=0 ){
		return false;
	}
	return true;
}

bool WK7Tutorial::IsHiddenSelect(void)
{
//	if( mpSelectPrim[0]==NULL || mpSelectPrim[1]==NULL ){
//		return true;
//	}
//	if( mpSelectPrim[0]->GetAlpha().GetValue()!=0 ){
//		return false;
//	}
//	return true;
	int		i;
	for(i=0 ; i<2 ; i++){
		if( mSelect[i].IsHidden()==false ){
			return false;
		}
	}
	return true;
}

void WK7Tutorial::SetSelectMode(bool flag)
{
	mSelectMode  = flag;
	mSelectIndex = -1;
	mCurrentSelectIndex = -1;
	mSelect[0].Select(false);
	mSelect[1].Select(false);
	DispSelect(flag);
}

//bool WK7Tutorial::SetSelectColor(int index,u32 color,u32 tick)
//{
//	if( index < 0 || index >= 2 ){
//		return false;
//	}
//	mSelectColor[index][0].SetValue((color >> 24) & 0xff,tick);
//	mSelectColor[index][1].SetValue((color >> 16) & 0xff,tick);
//	mSelectColor[index][2].SetValue((color >>  8) & 0xff,tick);
//	return true;
//}

//u32 WK7Tutorial::GetSelectColor(int index)
//{
//	if( index < 0 || index >= 2 ){
//		return 0;
//	}
//	return (mSelectColor[index][0].GetValue()<<24)|
//	       (mSelectColor[index][1].GetValue()<<16)|
//	       (mSelectColor[index][2].GetValue()<< 8);
//}

bool WK7Tutorial::DispBack(void)
{
	mBackAlpha.SetValue(0xc0,WK7TUTORIAL_FADE_TICK);
//	mBackAlpha.SetValue(0x80,WK7TUTORIAL_FADE_TICK);
	return true;
}

bool WK7Tutorial::HideBack(void)
{
	mBackAlpha.SetValue(0x00,WK7TUTORIAL_FADE_TICK);
	return true;
}

WK7TutorialPrim::WK7TutorialPrim(WK7TutorialPrim *pPrim,ghmGcListObj *pAnchor) : ghmGcListObj(pAnchor)
{
	mK7Sprite		*pSprite;
	if( pPrim==NULL || pPrim->GetTexture()==NULL ){
		mpTexture = NULL;
		mPrim.mSetBoxPrim();
	}else{
		mpTexture = pPrim->GetTexture();
		mPrim.mSetSpritePrim();
		mPrim.mRelateTex(mpTexture);
	}
	mX = mY = 0.0f;
	mAlpha.SetValue(0xff);
	mRGB = 0xffffff00|(mAlpha.GetValue() & 0xff);
	if( pPrim!=NULL && (pSprite=pPrim->GetPrim())!=NULL ){
		mX   = pSprite->mGetX0();
		mY   = pSprite->mGetY0();
		mW   = pSprite->mGetW();
		mH   = pSprite->mGetH();
		mAlpha.SetValue(pPrim->mAlpha.GetValue());
		mRGB = (pSprite->mGetRGBA0() & 0xffffff00)|(mAlpha.GetValue() & 0xff);
		mPrim.mSetWH  (mW,mH);
		mPrim.mSetXY0 (mX,mY);
		mPrim.mSetUV0 (pSprite->mGetU0(),pSprite->mGetV0());
		mPrim.mSetRGBA(mRGB);
	}
	mAutoRelease = false;
}

WK7TutorialPrim::WK7TutorialPrim(stTexture *pTexture,ghmGcListObj *pAnchor) : ghmGcListObj(pAnchor)
{
	mX = mY = 0.0f;
	mW = mH = 0;
	mRGB = 0xffffffff;
	if( (mpTexture = pTexture)==NULL ){
		mPrim.mSetBoxPrim();
	}else{
		mW = (s16)pTexture->tex->mWidth;
		mH = (s16)pTexture->tex->mHeight;
		mPrim.mSetSpritePrim();
		mPrim.mRelateTex(pTexture);
		mPrim.mSetWH  (mW,mH);
		mPrim.mSetUV0 (0,0);
	}
	mPrim.mSetXY0(mX,mY);
	mPrim.mSetRGBA(mRGB);
	mAlpha.SetValue(0xff);
	mAutoRelease = false;
}

WK7TutorialPrim::~WK7TutorialPrim(void)
{
}

bool WK7TutorialPrim::SetRGB(u32 rgb)
{
	mRGB = (rgb & 0xffffff00)|(mAlpha.GetValue() & 0xff);
//	mPrim.mSetRGBA(mRGB);
	return true;
}

bool WK7TutorialPrim::SetAlpha(u32 alpha,u32 tick)
{
	mAlpha.SetValue(alpha,tick);
//	mPrim.mSetA((u8)mAlpha.GetValue());
	return true;
}

bool WK7TutorialPrim::FrameProcess(void)
{
	mAlpha.FrameProcess();
	if( mpTexture!=NULL ){
		mPrim.mRelateTex(mpTexture);
	}
	mPrim.mSetRGBA((mRGB & 0xffffff00)|(mAlpha.GetValue() & 0xff));
	mPrim.mSetWH  (mW,mH);
	mPrim.mSetXY0 (mX,mY);
	WK7Tutorial	*pTutorial = WK7Tutorial::GetInstance();
	if( pTutorial!=NULL ){
		if( pTutorial->GetRender()==false || mAlpha.GetValue()==0 ){
			mPrim.mSetDispFlag(false);
		}else{
			mPrim.mSetDispFlag(true);
		}
	}
	if( mAutoRelease==true ){
		return mAlpha.GetValue()!=0;
	}
	return true;
}

bool WK7TutorialPrim::RenderProcess(void)
{
	return true;
}

bool WK7TutorialPrim::SetPriority(int prio)
{
	mPrim.mSetPriority(prio);
	return true;
}

bool WK7TutorialPrim::SetWH(s16 w,s16 h)
{
	mW = w;
	mH = h;
	return true;
}

bool WK7TutorialPrim::SetXY(f32 x,f32 y)
{
	mX = x;
	mY = y;
//	mPrim.mSetXY0(x,y);
	return true;
}

bool WK7TutorialPrim::SetTexture(stTexture* pTexture)
{
	if( mpTexture==NULL || pTexture==NULL ){
		return false;
	}
	mpTexture = pTexture;
//	mPrim.mRelateTex(pTexture);
	return true;
}

WK7TutorialSelect::WK7TutorialSelect()
{
	int		i;
	for(i=0 ; i<2 ; i++){
		mpPrim     [i] = NULL;
		mTextureNum[i] = -1;
		mpTexture  [i] = NULL;
	}
	mProc       = 0;
	mCounter    = 0;
	mSelect     = false;
	mColor.SetValue(0x80);
	mRGB[0] = 0xffffffff;
	mRGB[1] = 0xffffffff;
}

WK7TutorialSelect::~WK7TutorialSelect()
{
	int		i;
	for(i=0 ; i<2 ; i++){
		GHMGCMEM_SAFE_FREE(mpTexture[i]);
	}
}

bool WK7TutorialSelect::InitPrim(ghmGcListObj &anchor,ghmGcResGroup *pGroup,f32 x,f32 y,ghmGcResGroup *pGroup2,f32 x2,f32 y2)
{
	int		i;
	if( pGroup!=NULL ){
		mTextureNum[0] = (int)pGroup->GetResourceNum();
		if( mTextureNum[0] > 0 && (mpTexture[0] = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture) * mTextureNum[0])))!=NULL ){
			for(i=0 ; i<mTextureNum[0] ; i++){
				mpTexture[0][i].tex = (GhmGcTexHeader *)pGroup->GetResource(i);
				mK7InitTexture(mpTexture[0] + i);
			}
			mTextureID = 0;
			if( (mpPrim[0] = new WK7TutorialPrim(mpTexture[0],&anchor))!=NULL ){
				mpPrim[0]->SetPriority(WK7TUTORIAL_SELECT_PRIO);
				mpPrim[0]->SetXY(x,y);
			}
		}
	}
	if( pGroup2!=NULL ){
		mTextureNum[1] = (int)pGroup2->GetResourceNum();
		if( mTextureNum[1] > 0 && (mpTexture[1] = (stTexture*)K7AppSmallMemAlloc((int)(sizeof(stTexture) * mTextureNum[1])))!=NULL ){
			for(i=0 ; i<mTextureNum[1] ; i++){
				mpTexture[1][i].tex = (GhmGcTexHeader *)pGroup2->GetResource(i);
				mK7InitTexture(mpTexture[1] + i);
			}
			if( (mpPrim[1] = new WK7TutorialPrim(mpTexture[1],&anchor))!=NULL ){
				mpPrim[1]->SetPriority(WK7TUTORIAL_SELECT_PRIO);
				mpPrim[1]->SetXY(x2,y2);
			}
		}
	}
	return true;
}

bool WK7TutorialSelect::SetRGB(int index,u32 rgb)
{
	if( index < 0 || index > 1 ){
		return false;
	}
	mRGB[index] = rgb;
	return true;
}

bool WK7TutorialSelect::GetRGB(int index,u32 &rgb)
{
	if( index < 0 || index > 1 ){
		return false;
	}
	rgb = mRGB[index];
	return true;
}

bool WK7TutorialSelect::Select(bool flag)
{
	u32		color,tick;
	if( (mSelect = flag)==true ){
		color = WK7TUTORIAL_SELECTEDCOLOR;
		if( !mProc ){
			mProc++;
		}
	}else{
		color = WK7TUTORIAL_NOTSELECTEDCOLOR;
	}
	tick  = WK7TUTORIAL_FADE_TICK_QUICK;
	mColor.SetValue(color & 0xff,tick);
	return true;
}

bool WK7TutorialSelect::FrameProcess(void)
{
	int		i;
	u32		color,r,g,b;
	mColor.FrameProcess();
	color = mColor.GetValue();
	for(i=0 ; i<2 ; i++){
		r     = (color * ((mRGB[i] >> 24) & 0xff)) / 0xff;
		g     = (color * ((mRGB[i] >> 16) & 0xff)) / 0xff;
		b     = (color * ((mRGB[i] >>  8) & 0xff)) / 0xff;
		if( mpPrim[i]!=NULL ){
			mpPrim[i]->SetRGB((r<<24)|(g<<16)|(b<< 8));
		}
	}
	switch( mProc ){
	case	0:
		mTextureID = 0;
		mCounter = 0;
		break;
	case	1:
		if( mCounter < 2 * (WK7_TICK_PER_SEC/30) ){
			mCounter += WK7_TICK_PER_FRAME;
			break;
		}
		if( mSelect==false ){
			mProc = 0;
			mTextureID = 0;
		}else{
			mTextureID = HSD_Randi(mTextureNum[0]);
		}
		if( mpPrim[0]!=NULL ){
			mpPrim[0]->SetTexture(mpTexture[0] + mTextureID);
		}
		break;
	}
	return true;
}

bool WK7TutorialSelect::Disp(bool disp)
{
	int		i;
	for(i=0 ; i<2 ; i++){
		if( mpPrim[i]==NULL ){
			continue;
		}
		if( disp==true ){
			mpPrim[i]->SetAlpha(0xff,WK7TUTORIAL_FADE_TICK);
		}else{
			mpPrim[i]->SetAlpha(0x00,WK7TUTORIAL_FADE_TICK);
		}
	}
	return true;
}

bool WK7TutorialSelect::IsHidden(void)
{
	int		i;
	for(i=0 ; i<2 ; i++){
		if( mpPrim[i]==NULL ){
			continue;
		}
		if( mpPrim[i]->GetAlpha().GetValue()!=0 ){
			return false;
		}
	}
	return true;
}

