////////////////////////////////////////////////////////////////////////////////
//
// Killer7 WK7Tutorial
//
// WK7Tutorial.h
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 07/24/2004
//
////////////////////////////////////////////////////////////////////////////////

#ifndef	__WK7TUTORIAL_H__
#define	__WK7TUTORIAL_H__

#include "..\Matsu\MK7Sprite.h"

#define	WK7TUTORIAL_SELECT_YES		(0)
#define	WK7TUTORIAL_SELECT_NO		(1)

#define	WK7TUTORIAL_SSM_NAME		"seTutorial.ssm"

class WK7TutorialPrim : public ghmGcListObj
{
protected:
	mK7Sprite				mPrim;
	f32						mX,mY;
	s16						mW,mH;
	u32						mRGB;
	stTexture				*mpTexture;
	WK7AnimLinear			mAlpha;
	bool					mAutoRelease;
public:
	WK7TutorialPrim(WK7TutorialPrim *pPrim,ghmGcListObj *pAnchor);
	WK7TutorialPrim(stTexture *pTexture,ghmGcListObj *pAnchor);
	virtual ~WK7TutorialPrim(void);
	
	inline mK7Sprite* GetPrim(void){ return &mPrim; }
	bool SetTexture(stTexture* pTexture);
	inline stTexture* GetTexture(void){ return mpTexture; }
	WK7AnimLinear& GetAlpha(void){ return mAlpha; }
	
	inline WK7TutorialPrim* GetNext(void){ return (WK7TutorialPrim*)ghmGcListObj::GetNext(); }
	inline WK7TutorialPrim* GetPrev(void){ return (WK7TutorialPrim*)ghmGcListObj::GetPrev(); }
	
	inline void SetAutoRelease(bool flag){ mAutoRelease=flag; }
	inline bool GetAutoRelease(void){ return mAutoRelease; }
	
	bool SetRGB(u32 rgb);
	bool SetAlpha(u32 alpha,u32 tick=0);
	bool SetPriority(int prio);
	bool SetXY(f32 x,f32 y);
	bool SetWH(s16 w,s16 h);
	
	bool FrameProcess(void);
	bool RenderProcess(void);
};

class WK7TutorialSelect
{
protected:
	WK7TutorialPrim			*mpPrim    [2];
	int						mTextureNum[2];
	stTexture				*mpTexture [2];
	WK7AnimLinear			mColor;
	u32						mRGB[2];
	int						mProc;
	int						mTextureID;
	u32						mCounter;
	bool					mSelect;
public:
	WK7TutorialSelect();
	~WK7TutorialSelect();
	
	bool InitPrim(ghmGcListObj &anchor,ghmGcResGroup *pGroup,f32 x,f32 y,ghmGcResGroup *pGroup2,f32 x2,f32 y2);
	bool Select(bool flag);
	bool FrameProcess(void);
	
	bool SetRGB(int index,u32 rgb);
	bool GetRGB(int index,u32 &rgb);
	bool Disp(bool disp=true);
	inline bool Hide(void){ return Disp(false); }
	bool IsHidden(void);
};

class WK7Tutorial
{
protected:
	bool					mRender;
	int						mProc;
	
	bool					mLoaded;
	ghmGcFile				*mpFile;
	void					*mpData;
	ghmGcResLink			*mpResource;
	
	ghmGcListObj			mPrimAnchor;
	
	WK7TutorialPrim			*mpBackPrim;
	
	WK7TutorialPrim			*mpIwazaruPrim;
	int						mIwazaruTextureNum;
	stTexture				*mpIwazaruTexture;
	int						mCurrentIwazaruIndex;
	
	WK7TutorialPrim			*mpMessagePrim;
	int						mMessageTextureNum;
	stTexture				*mpMessageTexture;
	int						mCurrentMessageIndex;
	int						mMessageSE;
	stTexture				*mpMessageExTexture;
	WK7TutorialPrim			*mpMessageExPrim;
	int						mMessageProc;
	WK7AnimLinearF			mMessageExX;
	WK7AnimLinearF			mMessageExY;
	
	int						mButtonGroupNum;
	int						*mpButtonTextureNum;
	stTexture				**mppButtonTexture;
	WK7TutorialPrim			*mpButtonPrim;
	WK7TutorialPrim			*mpButtonPrimEx;
	int						mCurrentButtonIndex;
	int						mCurrentButtonIndexEx;
	int						mButtonTextureID;
	int						mButtonTextureIDEx;
	int						mButtonCounter;
	
	WK7TutorialSelect		mSelect[2];
//	WK7TutorialPrim			*mpSelectPrim[2];
//	int						mSelectTextureNum[2];
//	stTexture				*mpSelectTexture[2];
//	WK7AnimLinear			mSelectColor[2][3];
	bool					mSelectMode;
	int						mCurrentSelectIndex;
	int						mSelectIndex;
//	int						mSelectProc;
//	int						mSelectCounter;
	
	WK7AnimLinear			mBackAlpha;
	
	bool InitPrim(void);
	
	WK7Tutorial();
public:
	static WK7Tutorial* GetInstance(void);
	static bool Initialize(void);
	static bool Terminate(void);
	static bool IsTutorialMode(void);
	
	virtual ~WK7Tutorial();
	
	bool SetIwazaru(int index);
	bool SetMessage(int index);
	bool DispSelect(bool disp);
	
	void SetSelectMode(bool flag);
	inline bool GetSelectMode(void){ return mSelectMode; }
	inline int GetSelectIndex(void){ return mSelectIndex; }
	
	inline bool DispIwazaru(int index){ return SetIwazaru(index); }
	inline bool HideIwazaru(void){ return SetIwazaru(-1); }
	bool IsHiddenIwazaru(void);
	inline bool DispMessage(int index){ return SetMessage(index); }
	inline bool HideMessage(void){ return SetMessage(-1); }
	bool IsHiddenMessage(void);
	inline bool HideSelect(void){ return DispSelect(false); }
	bool IsHiddenSelect(void);
	
	bool DispBack(void);
	bool HideBack(void);
	
	void SetRender(bool flag);
	inline bool GetRender(void){ return mRender; }
	
	bool FrameProcess(void);
	
	bool RenderProcess(void);
	
	bool IsLoading(void);

};

enum{
	TUTOMSG_TUTO_QUERY = 0,
	TUTOMSG_RUN,
	TUTOMSG_TURN,
	TUTOMSG_KAMAE,
	TUTOMSG_SCAN,
	TUTOMSG_ATTACK,
	TUTOMSG_LOCKON,
	TUTOMSG_TARGETCHANGE,
	TUTOMSG_JUNCTION,
	TUTOMSG_TRYAGAIN,
	TUTOMSG_GO_QUERY,
};

enum{
	
};

#endif	//__WK7TUTORIAL_H__
