#include "common.h"
#include "c_impulse.h"
#pragma warn_implicitconv off
#include "smoke/scene_data.c"
#include "bullet/_null1_dolphin.c"
#pragma warn_implicitconv on

C_Impulse* C_Impulse::instance = 0;

#define WARP 0
C_ShockWave::C_ShockWave( C_Vec &_pos, C_Vec &_dir, f32 _speed, s32 _life )
{
  counter = life = _life;
  C_Vec z_axis = _dir, x_axis;
  C_Vec y_axis(0.f,1.f,0.f);
  x_axis = y_axis & z_axis;
  y_axis = x_axis & z_axis;
  y_axis.Nrm();
  s32 ofst = HSD_Rand()%60;
  for( int i=0; i<kShockWaveNum; i++ ){
	f32 rand = (HSD_Rand()%100)/100.f;
	Mtx m;
	MTXRotAxisDeg( m, &z_axis, ofst + 360 * i / kShockWaveNum );
	MTXMultVec( m, &y_axis, &dir[i] );
	dir[i] *= _speed;
	jobjs[i] = HSD_JObjLoadJoint(scene_data.models[0]->joint);
	HSD_JObjSetTranslate( jobjs[i],  &_pos );
	HSD_JObjSetRz( jobjs[i], MTXDegToRad(HSD_Rand()%360) );
	HSD_JObjSetSx( jobjs[i], 0.1f * rand );
	HSD_JObjSetSy( jobjs[i], 0.1f * rand );
  }
}
C_ShockWave::~C_ShockWave()
{
  for( int i=0; i<kShockWaveNum; i++ ){
	if( jobjs[i] ) HSD_JObjUnref( jobjs[i] );
  }
}
s32 C_ShockWave::Proc( void )
{
  if( counter ){
	f32 val = (f32)(counter/(f32)life);
	f32 scl = (1.f - val) * (1.f - val);
	for( int i=0; i<kShockWaveNum; i++ ){
	  f32 rand = (HSD_Rand()%100)/100.f;
	  //]B
	  HSD_JObjAddRz( jobjs[i], MTXDegToRad(360*val));
	  //B
	  HSD_JObjAddTx( jobjs[i], dir[i].x );
	  HSD_JObjAddTy( jobjs[i], dir[i].y );
	  HSD_JObjAddTz( jobjs[i], dir[i].z );
	  dir[i] *= 0.8f + 0.2f*rand;
	  //gB
	  HSD_JObjAddSx( jobjs[i], 0.01f * (1.f-val) * rand );
	  HSD_JObjAddSy( jobjs[i], 0.01f * (1.f-val) * rand );
	}
	counter--;
  }
  return counter;
}
void C_ShockWave::Disp( void )
{
  if( counter ){
	for( int i=0; i<kShockWaveNum; i++ ){
	  //HSD_JObjDispAll(jobjs[i], 0, HSD_TRSP_ALL, 0);
	  HSD_JObjDispAll(jobjs[i], 0, HSD_TRSP_OPA, RENDER_SHADOW|RENDER_EFFECT);
	}
  }
}

C_ShockWaveSet::C_ShockWaveSet( Vec *_start, Vec *_end, f32 _speed )
{
  pos = *_start;
  end = *_end;
  dir = end - pos;
  hit = false;
  if( dir.IsValid() && !dir.IsZero() ){
	dir.Nrm();
	dir *= _speed;
  }else{
	dir.Set(0.f);
  }
  waves.clear();
  bullet_jobj = HSD_JObjLoadJoint( &_null1_joint );
#if WARP
  jobj = HSD_JObjLoadJoint(scene_data.models[0]->joint);
  HSD_JObjSetSx( jobj, 0.5f );
  HSD_JObjSetSy( jobj, 0.5f );
#endif
}
C_ShockWaveSet::~C_ShockWaveSet()
{
  vector<C_ShockWave*>::iterator	w;
  for( w = waves.begin(); w!=waves.end(); w++ ) delete *w;
  HSD_JObjUnref( bullet_jobj );
#if WARP
  HSD_JObjUnref( jobj );
#endif
}
s32 C_ShockWaveSet::Proc( void )
{
  C_Vec pos2end = end - pos;
  if( (pos2end | dir) > 0.f ){
	ghmGcMemSetAllocName("CShckW");
	C_ShockWave *sw = new C_ShockWave( pos, dir, 0.01f, 90 );
	waves.push_back( sw );
  }else{
	hit = true;
  }
  for( u32 i=0; i<waves.size(); i++ ) waves[i]->Proc();
  while( 1 ){
	vector<C_ShockWave*>::iterator	w = waves.begin();
	for( u32 i=0; i<waves.size(); i++,w++ ){
	  if( waves[i]->GetCounter()==0 ){ delete w[i]; waves.erase( w ); break;}
	}
	if( w==waves.end() ) break;
  }
  pos += dir;

  if( !hit ){
	//ěƈʒuݒB
	Mtx rot,trs,scl,mtx;
	//B
	Vec y={0.f,1.f,0.f},x,z=dir;
	VECNormalize( &z, &z );
	VECCrossProduct( &z, &y, &x );
	VECCrossProduct( &x, &z, &y );
	rot[0][0] = x.x; rot[0][1] = x.y; rot[0][2] = x.z;
	rot[1][0] = y.x; rot[1][1] = y.y; rot[1][2] = y.z;
	rot[2][0] = z.x; rot[2][1] = z.y; rot[2][2] = z.z;
	MTXTranspose( rot, rot );
	//ʒuB
	MTXTrans( trs, pos.x, pos.y, pos.z );
	//XP[B
	MTXScale( scl, 0.2f, 0.2f, 0.2f );
	MTXConcat( rot, scl, mtx );
	MTXConcat( trs, mtx, mtx );
  
	HSD_JObjSetUserDefinedMtx( bullet_jobj, mtx );
  }
#if WARP
  HSD_JObjSetTranslate( jobj,  &pos );
#endif
  return (s32)waves.size();
}
void C_ShockWaveSet::DispBullet( void )
{
  HSD_StateSetCullMode(GX_CULL_BACK);
  GXSetCullMode( GX_CULL_FRONT );
  HSD_StateInvalidate( HSD_STATE_ALL );
  if( !hit ) HSD_JObjDispAll(bullet_jobj, 0, HSD_TRSP_ALL, 0);
}
void C_ShockWaveSet::Disp( void )
{
  vector<C_ShockWave*>::iterator	w;

  //
  for( w = waves.begin(); w!=waves.end(); w++ ){
	f32 val = (f32)((*w)->GetCounter()) / (f32)((*w)->GetLife());

	GXColor col;
	col.r = col.g = (u8)(16*val);
	col.b = (u8)(24*val);
	col.a = (u8)(255 * val);
	GXSetChanMatColor( GX_COLOR0A0, col );

	f32 intensity = 5.0f * val * val;
	f32 indMtx[2][3];
	Mtx tmpMtx,r,s,t;

	MTXRotDeg(r, 'z', 360*val );
	MTXScale(s, intensity, intensity, intensity );
	MTXTrans(t, VIGetRetraceCount()*0.01f,0.f,0.f);
	MTXConcat( r,s,tmpMtx );
	indMtx[0][0] = tmpMtx[0][0]*0.5f;
	indMtx[0][1] = tmpMtx[0][1]*0.5f;
	indMtx[0][2] = 0.0f;
	indMtx[1][0] = tmpMtx[1][0]*0.5f;
	indMtx[1][1] = tmpMtx[1][1]*0.5f;
	indMtx[1][2] = 0.0f;
	GXSetIndTexMtx(GX_ITM_0, indMtx, 1);
	GXSetNumIndStages(1);

	(*w)->Disp();
  }
#if WARP
  //擪̂䂪݁B
  //iAt@gj
  GXSetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
  GXSetTevOp( GX_TEVSTAGE0, GX_REPLACE );
  //ʃRs[
  GXSetTevOrder(GX_TEVSTAGE1, GX_TEXCOORD1, GX_TEXMAP1, GX_COLOR0A0);
  GXSetTevColorIn( GX_TEVSTAGE1, GX_CC_ZERO, GX_CC_ZERO, GX_CC_ZERO, GX_CC_TEXC );
  GXSetTevColorOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_TRUE, GX_TEVPREV );
  GXSetTevAlphaIn( GX_TEVSTAGE1, GX_CA_ZERO, GX_CA_ZERO, GX_CA_ZERO, GX_CA_APREV );
  GXSetTevAlphaOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_TRUE, GX_TEVPREV );
  GXSetNumTevStages(2);

  //PE
  GXSetColorUpdate( GX_ENABLE );
  GXSetAlphaUpdate( GX_DISABLE );
  GXSetBlendMode(GX_BM_NONE, GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
  GXSetZMode( GX_TRUE, GX_LEQUAL, GX_FALSE );

  HSD_JObjDispAll(jobj, 0, HSD_TRSP_OPA, RENDER_SHADOW|RENDER_EFFECT);

  GXSetTevDirect(GX_TEVSTAGE0);
  GXSetTevDirect(GX_TEVSTAGE1);
#endif
}

C_Impulse& C_Impulse::GetInstance( void ){
  if( instance ){
	return *instance;
  }else{
	ghmGcMemSetAllocName("C_Impl");
	instance = new C_Impulse();
	return *instance;
  }
}

#define kTexSize 64
C_Impulse::C_Impulse( void ) :
  indirect_texture_rand(kTexSize,kTexSize,GX_TF_IA8,GX_REPEAT,GX_REPEAT)
{
  wavesets.clear();
  //OςႾ߁B
  HSD_ImageDesc *img = &smoke_cube1_mat1_smoke03_ia4_0_image_desc;
  GXInitTexObj(&texobj, img->image_ptr,
			   img->width, img->height, img->format, 
			   GX_CLAMP, GX_CLAMP,
			   img->mipmap? GX_TRUE : GX_FALSE);
  GXInitTexObjLOD(&texobj, GX_NEAR, GX_NEAR, 0, 0, 0, 0, 0, GX_ANISO_1);
  {
	//C_CNgB_B
	s32 val = 6;
	for( int i=0; i<indirect_texture_rand.GetSize(); i++ ){
	  indirect_texture_rand.GetImage()[i] = (u8)(127 + ( HSD_Rand()%(val*2) - val));
	}
	DCFlushRange((void *) indirect_texture_rand.GetImage(), 
				 indirect_texture_rand.GetSize());
  }
  GXInvalidateTexAll();
}


s32 C_Impulse::Proc( void )
{
  for( u32 i=0; i<wavesets.size(); i++ ) wavesets[i]->Proc();
  while( 1 ){
	vector<C_ShockWaveSet*>::iterator	ws = wavesets.begin();
	for( u32 i=0; i<wavesets.size(); i++,ws++ ){
	  if( wavesets[i]->GetNum()==0 ){ delete wavesets[i]; wavesets.erase( ws ); break;}
	}
	if( ws==wavesets.end() ) break;
  }
  return (s32)wavesets.size();
}

void C_Impulse::Disp( GXTexObj *_tobj )
{
  //ŁAʂ̐ݒxsB
  //XLUƌł܂Ƃ߂Ă܂̂Ńf[^XLUƂ邱ƁB

  //`l
  GXSetChanCtrl(GX_COLOR0A0,
				GX_DISABLE,		// enable channel
				GX_SRC_REG,		// amb source
				GX_SRC_REG,		// mat source
				GX_LIGHT_NULL,	// light mask
				GX_DF_CLAMP,	// diffuse function
				GX_AF_NONE);
  GXSetNumChans(1);

  //eNX`
  //
  GXLoadTexObj(&texobj, GX_TEXMAP0);
  //ʃRs[
  GXLoadTexObj(_tobj, GX_TEXMAP1);
  //C_CNg@_
  GXLoadTexObj(indirect_texture_rand.GetTexObj(), GX_TEXMAP2);

  //eNX`W
  //
  GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
  //ʃRs[
  Mtx m,tm;
  HSD_CObj *cobj = HSD_CObjGetCurrent();
  MTXLightPerspective(m,
					  cobj->projection_param.perspective.fov,
					  cobj->projection_param.perspective.aspect,
					  0.5f, -0.5f, 0.5f, 0.5f );
  MTXConcat(m, HSD_CObjGetViewingMtxPtrDirect(cobj), tm);
  MTXConcat(tm, HSD_CObjGetInvViewingMtxPtrDirect(cobj), m);
  GXLoadTexMtxImm(m, GX_PTTEXMTX0, GX_MTX3x4);
  GXSetTexCoordGen2(GX_TEXCOORD1, GX_TG_MTX3x4, GX_TG_POS, GX_PNMTX0, GX_DISABLE, GX_PTTEXMTX0);
  GXSetNumTexGens(2);

  //TEV
  //out_reg = (d (op) ((1.0 - c)*a + c*b) + bias) * scale;
  //  void GXSetTevColorIn(
  //      GXTevStageID   stage,
  //      GXTevColorArg  a,
  //      GXTevColorArg  b,
  //      GXTevColorArg  c,
  //      GXTevColorArg  d );
  //  void GXSetTevColorOp(
  //      GXTevStageID    stage,
  //      GXTevOp         op,
  //      GXTevBias       bias,
  //      GXTevScale      scale,
  //      GXBool          clamp,
  //      GXTevRegID      out_reg );
  //
  GXSetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
  GXSetTevOp(GX_TEVSTAGE0, GX_MODULATE);
  //ʃRs[
  GXSetTevOrder(GX_TEVSTAGE1, GX_TEXCOORD1, GX_TEXMAP1, GX_COLOR0A0);
  GXSetTevColorIn( GX_TEVSTAGE1, GX_CC_TEXC, GX_CC_ZERO, GX_CC_CPREV, GX_CC_RASC );
  GXSetTevColorOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_TRUE, GX_TEVPREV );
  GXSetTevAlphaIn( GX_TEVSTAGE1, GX_CA_ZERO, GX_CA_ZERO, GX_CA_ZERO, GX_CA_APREV );
  GXSetTevAlphaOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_TRUE, GX_TEVPREV );
  GXSetNumTevStages(2);

  //C_CNg _
  GXSetIndTexOrder(GX_INDTEXSTAGE0, GX_TEXCOORD1, GX_TEXMAP2);
  GXSetIndTexCoordScale(GX_INDTEXSTAGE0, GX_ITS_8, GX_ITS_8);
  GXSetTevIndWarp(GX_TEVSTAGE1,       // tev stage
				  GX_INDTEXSTAGE0,    // indirect stage
				  GX_TRUE,            // signed offsets?
				  GX_FALSE,           // replace mode?
				  GX_ITM_0);          // ind matrix select

  //PE
  GXSetColorUpdate( GX_ENABLE );
  GXSetAlphaUpdate( GX_DISABLE );
  GXSetBlendMode(GX_BM_BLEND, GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
  GXSetZMode( GX_TRUE, GX_LEQUAL, GX_FALSE );

  //Cull
  GXSetCullMode( GX_CULL_NONE );

  vector<C_ShockWaveSet*>::iterator	ws;
  for( ws = wavesets.begin(); ws!=wavesets.end(); ws++ ) (*ws)->Disp();

  GXSetTevDirect(GX_TEVSTAGE0);
  GXSetTevDirect(GX_TEVSTAGE1);
  GXSetNumIndStages(0);

  GXSetCullMode( GX_CULL_BACK );
  HSD_StateInvalidate( HSD_STATE_ALL );

  //ŷ߂ɁAK̕`Ƃ͕B
  for( ws = wavesets.begin(); ws!=wavesets.end(); ws++ ) (*ws)->DispBullet();
}

C_ShockWaveSet *C_Impulse::Request( Vec *_start, Vec *_end, f32 _speed )
{
  ghmGcMemSetAllocName("CShckWS");
  C_ShockWaveSet *w = new C_ShockWaveSet( _start, _end, _speed );
  if( w ){
	wavesets.push_back( w );
	return w;
  }else{
	return 0;
  }
}
C_Vec *C_Impulse::GetPos( C_ShockWaveSet *wave )
{
  vector<C_ShockWaveSet*>::iterator	ws;
  for( ws = wavesets.begin(); ws!=wavesets.end(); ws++ ){
	if( *ws == wave ) return &(wave->GetPos());
  }
  return 0;
}

bool C_Impulse::TerminateAll( void )
{
  vector<C_ShockWaveSet*>::iterator	ws;
  for( ws = wavesets.begin(); ws!=wavesets.end(); ws++ ) delete (*ws);
  wavesets.clear();
  return true;
}
