#include "common.h"
#include "c_tumor.h"

#define INDEX_CHECK( I ) ( (I < 0) || (I >= NPCWEAKOBJID_MAX) )
C_TumorSet::C_TumorSet( WK7JObj *bone0, WK7JObj *bone1, WK7JObj *bone2,
						WK7JObj *bone3, WK7JObj *bone4, WK7JObj *bone5 ){
  tumor[0].Init( bone0 );	
  tumor[1].Init( bone1 );	
  tumor[2].Init( bone2 );	
  tumor[3].Init( bone3 );	
  tumor[4].Init( bone4 );	
  tumor[5].Init( bone5 );	
}
void C_TumorSet::Proc( const Vec &_eye, const f32 _dist ){
  for( int i=0; i<NPCWEAKOBJID_MAX; i++ )  tumor[ i ].Proc( _eye, _dist );
}
void C_TumorSet::Start( s32 _idx, TUMOR_LEVEL _level ){
  if( INDEX_CHECK( _idx ) ) return;
  tumor[_idx].Start( _level );
}
void C_TumorSet::Stop( s32 _idx ){
  if( INDEX_CHECK( _idx ) ) return;
  tumor[_idx].Stop();
}
TUMOR_LEVEL C_TumorSet::AddLevel( s32 _idx ){
  if( INDEX_CHECK( _idx ) ) return TUMOR_NONE;
  return tumor[_idx].AddLevel();
}
TUMOR_LEVEL C_TumorSet::SubLevel( s32 _idx ){
  if( INDEX_CHECK( _idx ) ) return TUMOR_NONE;
  return tumor[_idx].SubLevel();
}
TUMOR_LEVEL C_TumorSet::SetLevel( s32 _idx, TUMOR_LEVEL _level ){
  if( INDEX_CHECK( _idx ) ) return TUMOR_NONE;
  return tumor[_idx].SetLevel( _level );
}
C_TumorSet::~C_TumorSet(){
}
C_TumorSet::C_Tumor::C_Tumor( void ){
  jobj = 0;
  parent_jobj = 0;
  gen = 0;
  parent_bone = 0;
  fire = false;
}
C_TumorSet::C_Tumor::~C_Tumor(){
  if( jobj ) HSD_JObjUnref( jobj );
  if( parent_jobj ) HSD_JObjUnref( parent_jobj );
  if( gen ) delete gen;
}
void C_TumorSet::C_Tumor::Init( WK7JObj *_bone ){
  if( _bone ){
	parent_bone = _bone;
	SetupJObj();
	SetupGen();
  }
}
void C_TumorSet::C_Tumor::Proc( const Vec &_eye, const f32 _dist ){
  if( gen && fire ){
	Vec _trs,_dir;
	HSD_JObjSetupMatrix( parent_jobj );
	HSD_MtxGetTranslate( parent_jobj->mtx, &_trs );
	VECSubtract( &_eye, &_trs, &_dir );
	VECNormalize( &_dir, &_dir );
	VECScale( &_dir, &_dir, _dist );
	VECAdd( &_trs, &_dir, &_trs );
	HSD_JObjSetTranslate( jobj, &_trs );
	HSD_JObjSetupMatrix( jobj );
  }
}
void C_TumorSet::C_Tumor::Start( TUMOR_LEVEL _level ){
  Stop();
  SetupGen( _level );
  gen->Fire();
  fire = true;
}
void C_TumorSet::C_Tumor::Stop( void ){
  if( gen )	delete gen;
  gen = 0;
  fire = false;
}
TUMOR_LEVEL C_TumorSet::C_Tumor::AddLevel( void ){
  Start( (TUMOR_LEVEL)(++level) ); return (TUMOR_LEVEL)level;
}
TUMOR_LEVEL C_TumorSet::C_Tumor::SubLevel( void ){
  Start( (TUMOR_LEVEL)(--level) ); return (TUMOR_LEVEL)level;
}
TUMOR_LEVEL C_TumorSet::C_Tumor::SetLevel( TUMOR_LEVEL _level ){
  Start( (TUMOR_LEVEL)(level = _level) );  return (TUMOR_LEVEL)level;
}
C_TumorSet::C_Tumor::C_Level::C_Level( TUMOR_LEVEL l_ )
{
#if 1
  pno[ TUMOR_LOW ] = 15;
  pno[ TUMOR_MID ] = 15;
  pno[ TUMOR_HIGH ] = 15;
#else
  pno[ TUMOR_LOW ] = 17;
  pno[ TUMOR_MID ] = 17;
  pno[ TUMOR_HIGH ] = 17;
#endif
  level = (s32)l_;
}
C_TumorSet::C_Tumor::C_Level& C_TumorSet::C_Tumor::C_Level:: operator ++ () {	level ++; Regulate(); return *this;}
C_TumorSet::C_Tumor::C_Level& C_TumorSet::C_Tumor::C_Level:: operator -- () {	level --; Regulate(); return *this;}
C_TumorSet::C_Tumor::C_Level& C_TumorSet::C_Tumor::C_Level:: operator = ( TUMOR_LEVEL _level ) {	level = (s32)_level; Regulate(); return *this;}
C_TumorSet::C_Tumor::C_Level:: operator TUMOR_LEVEL () {	return (TUMOR_LEVEL)level; }
s32 C_TumorSet::C_Tumor::C_Level::GetPtclNo( void ){return pno[ level ];}
C_TumorSet::C_Tumor::C_Level::~C_Level(){}
s32 C_TumorSet::C_Tumor::C_Level::Regulate( void ){
  if( level >= TUMOR_MAX ) level = TUMOR_MAX - 1;
  if( level <= TUMOR_NONE ) level = TUMOR_NONE + 1;
  return level;
}
void C_TumorSet::C_Tumor::SetupJObj( void ){
  if( parent_bone ){
	parent_jobj = parent_bone->GetJObj();		
	assert( parent_jobj );
	HSD_JObjRef( parent_jobj );
	jobj = HSD_JObjAlloc();
	assert(jobj );
  }
}
void C_TumorSet::C_Tumor::SetupGen( TUMOR_LEVEL _level ){
  if( jobj ){
	level = _level;
	gen = new C_Generator( jobj, level.GetPtclNo() );
	assert( gen );
  }
}



