/*
 * Copyright 1999-2002 HAL Laboratory, Inc. All rights reserved.
 *
 * $Id: state.c,v 1.6 2004/08/06 09:26:03 TypeR Exp $
 */

#ifdef FUJITA
#pragma cplusplus			off
#pragma cpp_extensions		off
#pragma warn_implicitconv	off
#endif

#include "sysdolphin/baselib/state.h"
#include "sysdolphin/baselib/debug.h"
#include "sysdolphin/baselib/tev.h"
#include "sysdolphin/baselib/displayfunc.h"
#include "sysdolphin/baselib/lobj.h"
#include "private.h"

/** @package State */

/* FIXME! check default setting */

/* HSD_STATE_PRIMITIVE Group */
static u8		state_line_width;
static GXTexOffset	state_line_tex_offsets;
static u8		state_point_size;
static GXTexOffset	state_point_tex_offsets;
static GXCullMode	state_cull_mode;
static GXBool		state_coplanar;

/* HSD_STATE_RENDER_MODE Group */
static GXBlendMode	state_blend_type;
static GXBlendFactor	state_src_factor;
static GXBlendFactor	state_dst_factor;
static GXLogicOp	state_logic_op;
static GXBool		state_z_enable;
static GXCompare	state_z_func;
static GXBool		state_z_update;
static GXCompare	state_alpha_comp0;
static u8		state_alpha_ref0;
static GXAlphaOp	state_alpha_op;
static GXCompare	state_alpha_comp1;
static u8		state_alpha_ref1;
static GXBool		state_color_update;
static GXBool		state_alpha_update;
static GXBool		state_enable_dst_alpha;
static u8		state_dst_alpha;
static GXBool		state_before_tex;
static GXBool		state_dither;


static struct {
  GXColor ambient;
  GXColor diffuse;
  GXColor specular;
  u8   alpha;
  float shininess;
} matstate;

/**
 * J[`lݒ肷B
 * 
 * rendermodëȉ̃rbg`lݒ̃^Cv߂B
 * 
 * <b>DIFFUSECeBO</b>:
 * 
 * RENDER_DIFFUSErbgĂꍇAv~eBu̓fBt[YCg
 * ʂ󂯂BCeBǑvZɂ̓`lOgpB
 * 
 * }eADIFFUSEF̃^CvɂA`l͈ȉ̂悤ɐݒ肳B
 * <pre>
 * ---------------------+-----------------------------------------------------
 *  RENDER_DIFFUSE_BITS | ݒ
 * ---------------------+-----------------------------------------------------
 *  RENDER_DIFFUSE_MAT  | AMB = La.diff * M.ambi
 *                      | CH0 = 1.0 * (AMB + L0.diff + L1.diff + ...)
 * ---------------------+-----------------------------------------------------
 *  RENDER_DIFFUSE_VTX  | AMB = La.diff * M.ambi
 *  RENDER_DIFFUSE_BOTH | CH0 = VTX * (AMB + L0.diff + L1.diff + ...)
 * ---------------------+-----------------------------------------------------
 * </pre>
 * 
 * }eAEArGglɂHSD_SetMaterialColor()Őݒ肳ꂽambient
 * gpB
 * 
 * <b>ALPHACeBO</b>:
 * 
 * RENDER_DIFFUSErbgĂAAt@Cg݂ꍇɂ
 * v~eBu̓At@Cg̉e󂯂B
 * 
 * At@CeBOɎgp`l̓}eA_At@
 * gp邩ۂňقȂB
 * 
 * _At@gpꍇA_At@ʉ߂̂Ƀ`lO
 * gpB]āACeBOɂ̓`lPgpB
 * 
 * <pre>
 * -------------------+-------------------------------------------------------
 *  RENDER_ALPHA_BITS | ݒ
 * -------------------+-------------------------------------------------------
 *  RENDER_ALPHA_MAT  | CH0  = 1.0 * (La.alph + L0.alph + L1.alph + ...)
 * -------------------+-------------------------------------------------------
 *  RENDER_ALPHA_VTX  | CH0  = VTX
 *  RENDER_ALPHA_BOTH | CH1  = 1.0 * (La.alph + L0.alph + L1.alph + ...)
 * -------------------+-------------------------------------------------------
 * </pre>
 *
 * A
 * <pre>
 *       M: }eA
 *      La: ArGgCg
 *      Li: iԖڂInfinite/Point/SpotCg
 *  
 *  X.diff: IuWFNgX̃fBt[YJ[ (ς݁j
 *  X.alph: IuWFNgX̃At@l         (ς݁j
 *  IuWFNg݂Ȃꍇ0.0B
 * </pre>
 *
 * <b>SPECULARCeBO</b>:
 * 
 * RENDER_SPECUARrbgĂꍇAXyLCeBOsȂB
 * vZɂ̓`lPgpB
 * <pre>
 * ----------------+----------------------------------------------------------
 *  rendermode     | output (RGB)
 * ----------------+----------------------------------------------------------
 * RENDER_SPECULAR | CH1 = 1.0 * ( L0.spec + L1.spec + ... )
 * ----------------+----------------------------------------------------------
 * A
 * Li.spec: iԖڂlobj̃XyLJ[ (ς)
 * </pre>
 * 
 * ݃ANeBuȃCgshininesslHSD_SetMaterialShininess()Ŏw肵
 * lɐݒ肷B
 * 
 * @see HSD_SetMaterialColor
 * @see HSD_SetMaterialShininess
 */
void
HSD_SetupChannelMode(u32 rendermode)
{
  u32 diffuse_mode;
  u32 alpha_mode;
  int use_color1 = 0;
  int use_alpha1 = 0;

  /*
   * ʌ݊̂߂̃tO
   */

  diffuse_mode = rendermode & RENDER_DIFFUSE_BITS;
  if (diffuse_mode == RENDER_DIFFUSE_MAT0) {
    diffuse_mode = RENDER_DIFFUSE_MAT;
  }

  alpha_mode = rendermode & RENDER_ALPHA_BITS;
  if (alpha_mode == RENDER_ALPHA_COMPAT) {
    alpha_mode = diffuse_mode << (RENDER_ALPHA_SHIFT - RENDER_DIFFUSE_SHIFT);
  }

  if (rendermode & RENDER_SPECULAR) {
    static HSD_Chan ch = {
      NULL, GX_COLOR1, 0, 
      { 0, 0, 0, 0 },			/* amb */
      { 255, 255, 255, 255 },		/* mat */
      GX_TRUE, GX_SRC_REG, GX_SRC_REG, 
      (GXLightID)0, GX_DF_NONE, GX_AF_SPEC,
    };
    int i, num;
    
    ch.light_mask = HSD_LObjGetLightMaskSpecular();
#if 0
    HSD_Report("SPECULAR light_mask (%x)\n", ch.light_mask);
#endif
    //ch.enable = ch.light_mask ? GX_TRUE : GX_FALSE;
    HSD_SetupChannel(&ch);
    use_color1 = 1;

    num = HSD_LObjGetNbActive();
    for (i = 0; i < num; i++) {
      HSD_LObj *lobj = HSD_LObjGetActiveByIndex(i);

      if (lobj != NULL) {
	HSD_LObjSetup(lobj, lobj->color, matstate.shininess);
      }
    }
  }

  if (rendermode & RENDER_DIFFUSE) {
    HSD_LObj *amb;
    static HSD_Chan ch = {
      NULL, GX_COLOR0, 0, 
      { 0, 0, 0, 0 },			/* amb_src black */
      { 255, 255, 255, 255 },		/* mat_src white */
      GX_ENABLE, GX_SRC_REG, GX_SRC_REG, 
      (GXLightID)0, GX_DF_CLAMP, GX_AF_SPOT,
    };
    static HSD_Chan ch_a = {
      NULL, GX_ALPHA0, 0, 
      { 0, 0, 0, 0 },		/* amb */
      { 0, 0, 0, 0 },		/* mat */
      GX_ENABLE, GX_SRC_REG, GX_SRC_REG, 
      (GXLightID)0, GX_DF_CLAMP, GX_AF_SPOT,
    };
    static HSD_Chan ch_vtx_a = {
      NULL, GX_ALPHA0, 0,
      { 0, 0, 0, 0 },		/* amb */
      { 0, 0, 0, 0 },		/* mat */
      GX_DISABLE, GX_SRC_REG, GX_SRC_VTX,
      (GXLightID)0, GX_DF_CLAMP, GX_AF_SPOT
    };
    int amb_alpha;

    static GXColor dark_matter = {0, 0, 0, 255};

    amb = HSD_LObjGetActiveByID(GX_LIGHT_AMBIENT);

    if (amb && (amb->flags & LOBJ_DIFFUSE)) {
      HSD_MulColor(&matstate.ambient, &amb->color, &ch.amb_color);
    } else {
      ch.amb_color = dark_matter;
    }

    ch.mat_src = (diffuse_mode & RENDER_DIFFUSE_VTX)? GX_SRC_VTX : GX_SRC_REG;
    ch.light_mask = HSD_LObjGetLightMaskDiffuse();
    HSD_SetupChannel(&ch);

    /*
     * ݃ANeBuLObjLOBJ_ALPHArbgɉ
     * At@`l̐ݒsB
     */
    
    if (alpha_mode & RENDER_ALPHA_VTX) {
      /*
       * At@Cgƒ_At@Ɏgꍇɂ́AAt@Cg
       * vZCHANNEL 1ōsȂB
       */
      ch_a.chan = GX_ALPHA1;
      use_alpha1 = 1;

      /*
       * _At@CHANNEL 0Œʉ߂
       */
      HSD_SetupChannel(&ch_vtx_a);

    } else {
      ch_a.chan = GX_ALPHA0;
    }

    ch_a.light_mask = HSD_LObjGetLightMaskAlpha();

    if (amb && (amb->flags & LOBJ_ALPHA)) {
      amb_alpha = amb->color.a;
    } else {
      amb_alpha = 0;
    }

    if (ch_a.light_mask) {
      ch_a.enable = GX_TRUE;
      ch_a.mat_color.a = 255;
      ch_a.amb_color.a = amb_alpha;
    } else {
      ch_a.enable = GX_FALSE;
      ch_a.mat_color.a = amb_alpha;
    }

    HSD_SetupChannel(&ch_a);

  // !(rendermode & RENDER_DIFFUSE)
  } else {		
    static HSD_Chan ch = {
      NULL, GX_COLOR0, 0, 
      { 0, 0, 0, 0 },			/* amb */
      { 255, 255, 255, 255 },		/* mat */
      GX_FALSE, GX_SRC_REG, GX_SRC_VTX, 
      (GXLightID)0, GX_DF_NONE, GX_AF_NONE,
    };
    static HSD_Chan ch_a = {
      NULL, GX_ALPHA0, 0, 
      { 0, 0, 0, 0 },			/* amb */
      { 0, 0, 0, 255 },			/* mat */
      GX_FALSE, GX_SRC_REG, GX_SRC_VTX, 
      (GXLightID)0, GX_DF_NONE, GX_AF_NONE,
    };

    ch.mat_src = (diffuse_mode & RENDER_DIFFUSE_VTX)? GX_SRC_VTX : GX_SRC_REG;
    HSD_SetupChannel(&ch);

    ch_a.mat_src = (alpha_mode & RENDER_ALPHA_VTX)? GX_SRC_VTX : GX_SRC_REG;
    HSD_SetupChannel(&ch_a);
  }

  /*
   * ݒ̃J[EAt@`l͖gpɐݒ肷B
   */

  if (use_color1) {
    if (!use_alpha1) {
      HSD_DisableChannelLighting(GX_ALPHA1);
    }
    HSD_StateSetNumChans(2);
  } else { // !use_color1
    if (use_alpha1) {
      HSD_DisableChannelLighting(GX_COLOR1);
      HSD_StateSetNumChans(2);
    } else {
      HSD_DisableChannelLighting(GX_COLOR1A1);
      HSD_StateSetNumChans(1);
    }
  }
}

/**
 * Pixel Engine ̃[hݒ肷B
 * 
 *  pe ŃJX^ PE ݒ肪w肳ꂽꍇÂƂɐݒ肷B
 * JX^ PE ݒ肪w肳ȂꍇrendermodeɊÂă[h
 * ݒ肷B
 * 
 * rendermode ƑΉݒ͈ȉ̂Ƃ
 * <pre>
 * o ɐݒ肳鍀
 * 
 *    HSD_StateSetColorUpdate (GX_ENABLE);
 *    HSD_StateSetAlphaUpdate (GX_DISABLE);
 *    HSD_StateSetDstAlpha    (GX_DISABLE, 0);
 *    HSD_StateSetDither      (GX_DISABLE);
 * 
 *  o uh[hݒ (HSD_StateSetBlendMode)
 * 
 *   RENDER_XLUw肳ꂽꍇ,
 * 
 *     dst = (1-src.alpha) * dst + src.alpha * src
 * 
 *   RENDER_XLUw肳Ȃꍇ
 * 
 *     dst = src
 * 
 * o Z[hݒ (HSD_StateSetZMode)
 * 
 *    RENDER_ZMODE_ALWAYSw肳ꂽꍇAZr𖳌ɐݒ肷B
 *    RENDER_NO_ZUPDATEw肳ꂽꍇAZl̍XV͍sȂB
 * 
 * o Zr̎{^C~OAAt@rݒ
 *   (HSD_StateSetZCompLoc, HSD_StateSetAlphaCompare)
 * 
 *   o !RENDER_NO_ZUPDATE && RENDER_XLU
 *       TEXEDGE\BZrTEVXe[WɍsȂAAt@lO
 *       傫ꍇɂ̂EFB֏ށB
 * 
 *   o ̑
 *       ZrTEVXe[WOłȂBAt@r͍sȂB
 * </pre> 
 * 
 * @see HSD_StateSetColorUpdate
 * @see HSD_StateSetAlphaUpdate
 * @see HSD_StateSetDstAlpha
 * @see HSD_StateSetBlendMode
 * @see HSD_StateSetZMode
 * @see HSD_StateSetZCompLoc
 * @see HSD_StateSetAlphaCompare
 * @see HSD_StateSetDither
 */
void
HSD_SetupPEMode(u32 rendermode, HSD_PEDesc *pe)
{
  if (pe) {
    HSD_StateSetColorUpdate  (pe->flags & ENABLE_COLOR_UPDATE);
    HSD_StateSetAlphaUpdate  (pe->flags & ENABLE_ALPHA_UPDATE);
    HSD_StateSetDstAlpha     (pe->flags & ENABLE_DST_ALPHA,
			      pe->dst_alpha);
    HSD_StateSetBlendMode    ((GXBlendMode) pe->type,
			      (GXBlendFactor) pe->src_factor,
			      (GXBlendFactor) pe->dst_factor,
			      (GXLogicOp) pe->logic_op);
    HSD_StateSetZMode        (pe->flags & ENABLE_COMPARE,
			      (GXCompare) pe->z_comp,
			      pe->flags & ENABLE_ZUPDATE);
    HSD_StateSetZCompLoc     (pe->flags & BEFORE_TEX);
    HSD_StateSetAlphaCompare ((GXCompare) pe->alpha_comp0, pe->ref0,
			      (GXAlphaOp) pe->alpha_op,
			      (GXCompare) pe->alpha_comp1, pe->ref1);
    HSD_StateSetDither       (pe->flags & ENABLE_DITHER);
  } else {
    HSD_StateSetColorUpdate  (GX_ENABLE);
//#ifndef	WATANABE
    HSD_StateSetAlphaUpdate  (GX_DISABLE);
    HSD_StateSetDstAlpha     (GX_DISABLE, 0);
//#endif
    HSD_StateSetBlendMode((rendermode&RENDER_XLU)?GX_BM_BLEND:GX_BM_NONE,
			  GX_BL_SRCALPHA,
			  GX_BL_INVSRCALPHA,
			  GX_LO_SET);
    HSD_StateSetZMode(GX_TRUE,
		      (rendermode&RENDER_ZMODE_ALWAYS) ? GX_ALWAYS : GX_LEQUAL, 
		      (rendermode&RENDER_NO_ZUPDATE) ? GX_FALSE : GX_TRUE);
    if (!(rendermode&RENDER_NO_ZUPDATE) && (rendermode&RENDER_XLU)) {
      HSD_StateSetZCompLoc(GX_FALSE);
      HSD_StateSetAlphaCompare(GX_GREATER, 0,
			       GX_AOP_AND,      /*  e2.0 gp */
			       GX_GREATER, 0);  /*  e2.0 gp */
    } else {
      HSD_StateSetZCompLoc(GX_TRUE);
      HSD_StateSetAlphaCompare(GX_ALWAYS, 0,
			       GX_AOP_AND,      /*  e2.0 gp */
			       GX_ALWAYS, 0);  /*  e2.0 gp */
    }
    HSD_StateSetDither(GX_DISABLE);
  }
}

static void
setupTevMode_last(u32 rendermode)
{
#pragma unused(rendermode)

  if (!HSD_StateGetNumTevStages()) {
    HSD_TevDesc tevdesc;

    /*  TEV ݒ肳ĂȂꍇ GX_PASSCLR ݒ */
    tevdesc.flags = TEVOP_MODE;
    tevdesc.stage = HSD_StateAssignTev();
    tevdesc.coord = GX_TEXCOORD_NULL;
    tevdesc.map = GX_TEXMAP_NULL;
    tevdesc.color = GX_COLOR0A0;
    tevdesc.u.tevop.tevmode = GX_PASSCLR;
    HSD_SetupTevStage(&tevdesc);
  }
}

/**
 * _O[hݒ肷B
 * 
 * Pixel Engine ̐ݒɃJX^PEgpȊOAHSD_SetupRenderMode()
 * lB
 * 
 * @see HSD_SetupRenderMode
 */
void
HSD_SetupRenderModeWithCustomPE(u32 rendermode, HSD_PEDesc *pe)
{
#ifdef PRINT_DEBUG
  HSD_Report("setupRenderMode in rendermode = 0x%x\n", rendermode);
#endif
  
  setupTevMode_last(rendermode);
  HSD_SetupPEMode(rendermode, pe);
  HSD_SetTevRegAll();
  HSD_StateSetNumTevStages();
  HSD_StateSetNumTexGens();
  /* n[h̃bNhׂɃ`lݒ TEV ̐ݒɍs */
  HSD_SetupChannelMode(rendermode);

#ifdef PRINT_DEBUG
  HSD_Report("setupRenderMode out\n");
#endif
}

/**
 * _O[hݒ肷B
 * 
 * @heading ݒ荀
 * <ul>
 * <li> TEVXe[WiATexGenݒ肷B
 * <li> TEVXe[Wݒ肪ĂȂꍇAJ[`lȌo͂
 *      ̂܂TEVXe[Wo͂ƂB
 * <li> J[`l̃[hݒ肷B
 * <li> JX^ PE ݒpPixel Engine ݒ肷B
 * </ul>
 * 
 * @see HSD_SetupPEMode
 * @see Tev#HSD_StateSetNumTevStages
 * @see Tev#HSD_StateSetNumTexGens
 * @see HSD_SetupChannelMode
 */
void
HSD_SetupRenderMode(u32 rendermode)
{
  HSD_SetupRenderModeWithCustomPE(rendermode, NULL);
}

/**
 * }eA state ɃJ[ݒ
 * 
 * HSD_SetupChannelMode()Őݒɗpp[^ݒ肷B
 * {֐HSD_SetupChannelMode()ɐ旧ČĂяoB
 *
 * @see HSD_SetupChannelMode
 * @see HSD_SetMaterialShininess
 */
void
HSD_SetMaterialColor(GXColor ambient,
		     GXColor diffuse, 
		     GXColor specular, 
		     float alpha)
{
  matstate.ambient = ambient;
  matstate.diffuse = diffuse;
  matstate.specular = specular;
  matstate.alpha = (u8)(255.0F * HSD_ClampFloat(alpha, 0, 1));
}

/**
 * }eA state  shininess ݒ
 * 
 * HSD_SetupChannelMode()Őݒɗpp[^ݒ肷B
 * {֐HSD_SetupChannelMode()ɐ旧ČĂяoB
 *
 * @see HSD_SetupChannelMode
 * @see HSD_SetMaterialColor
 */
void
HSD_SetMaterialShininess(float shininess)
{
  matstate.shininess = shininess;
}

/**
 * LineWidth̐ݒ
 * 
 * Xe[gQƂAKvɉGXSetLineWidth()ĂяoB
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetLineWidth(u8 width, GXTexOffset tex_offsets)
{
  if (state_line_width != width ||
      state_line_tex_offsets != tex_offsets)
  {
    GXSetLineWidth(width, tex_offsets);
    state_line_width = width;
    state_line_tex_offsets = tex_offsets;
  }
}

/**
 * JO[h̐ݒ
 * 
 * Xe[gQƂAKvɉGXSetCullMode()ĂяoB
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetCullMode(GXCullMode mode)
{
  if (state_cull_mode != mode) {
    GXSetCullMode(mode);
    state_cull_mode = mode;
  }
}

/**
 * CoPlanar ݒ
 * 
 * Xe[gQƂAKvɉGXSetCoPlanar()ĂяoB
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetCoPlanar(int enable)
{
  GXBool _enable = (enable)? GX_TRUE : GX_FALSE;

  if (_enable != state_coplanar) {
    GXSetCoPlanar(_enable);
    state_coplanar = _enable;
  }
}

/**
 * BlendMode̐ݒ
 * 
 * Xe[gQƂAKvɉGXSetBlendMode()ĂяoB
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetBlendMode(GXBlendMode type,
		      GXBlendFactor src_factor, 
		      GXBlendFactor dst_factor, 
		      GXLogicOp op)
{
  if (state_blend_type != type
      || state_src_factor != src_factor
      || state_dst_factor != dst_factor
      || state_logic_op != op) {
    GXSetBlendMode(type, src_factor, dst_factor, op);
    state_blend_type = type;
    state_src_factor = src_factor;
    state_dst_factor = dst_factor;
    state_logic_op = op;
  }
}

/**
 * ZMode ̐ݒ
 * 
 * Xe[gQƂAKvɉGXSetZMode()ĂяoB
 * 
 * @param enable	ZrLɂi^lj
 * @param func		Zr[h
 * @param update	ZlZobt@ɏށi^lj
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetZMode(int enable, GXCompare func, int update)
{
  GXBool _enable = (enable)? GX_ENABLE : GX_DISABLE;
  GXBool _update = (update)? GX_ENABLE : GX_DISABLE;

  if (state_z_enable != _enable ||
      state_z_func != func ||
      state_z_update != _update)
  {
    GXSetZMode(_enable, func, _update);
    state_z_enable = _enable;
    state_z_func = func;
    state_z_update = _update;
  }
}

/**
 * PointSize ̐ݒ
 * 
 * Xe[gQƂAKvɉGXSetPointSize()ĂяoB
 * 
 * @see HSD_StateInvalidate
 */
void
HSD_StateSetPointSize(u8 size, GXTexOffset tex_offsets)
{
  if (state_point_size != size ||
      state_point_tex_offsets != tex_offsets)
  {
    GXSetPointSize(size, tex_offsets);
    state_point_size = size;
    state_point_tex_offsets = tex_offsets;
  }
}

/**
 * At@r[h̐ݒ
 * 
 * {֐Őݒ肵ɓKꍇɂ̂EFBւ݂̏
 * sB
 * 
 * @warning
 * ZrTEVXe[WOōsꍇɂ́AAt@r
 * ӖȂB
 * 
 * @see HSD_StateSetZCompLoc
 */
void
HSD_StateSetAlphaCompare(GXCompare comp0, u8 ref0, GXAlphaOp op,
			 GXCompare comp1, u8 ref1)
{
  if (state_alpha_comp0 != comp0 ||
      state_alpha_ref0 != ref0 ||
      state_alpha_op != op ||
      state_alpha_comp1 != comp1 ||
      state_alpha_ref1 != ref1)
  {
    GXSetAlphaCompare(comp0, ref0, op, comp1, ref1);
    state_alpha_comp0 = comp0;
    state_alpha_ref0 = ref0;
    state_alpha_op = op;
    state_alpha_comp1 = comp1;
    state_alpha_ref1 = ref1;
  }
}

/**
 * J[EFBւ݂̏ݒ肷
 * 
 * @param enable	^̂ƂAEFBւ̃J[݂̏L
 */
void
HSD_StateSetColorUpdate(int enable)
{
  u8 _enable = (enable)? GX_ENABLE : GX_DISABLE;

  if (state_color_update != _enable) {
    GXSetColorUpdate(_enable);
    state_color_update = _enable;
  }
}

/**
 * At@lEFBւ݂̏ݒ肷
 * 
 * @param enable	^̂ƂAEFBւ̃J[݂̏L
 */
void
HSD_StateSetAlphaUpdate(int enable)
{
  u8 _enable = (enable)? GX_ENABLE : GX_DISABLE;

  if (state_alpha_update != _enable) {
    GXSetAlphaUpdate(_enable);
    state_alpha_update = _enable;
  }
}

/**
 * TEV̏o͂At@l̑ɁAw̃At@lEFBɏ
 * 
 * @warning
 * {֐Ŏw肷At@lEFBւ݂̏ɂ̂ݎgpB
 * uhɂ͏TEV̏o͂At@lgpB
 *
 * @param enable	^̂ƂAAt@l̒uL
 * @param alpha		uAt@l
 */
void
HSD_StateSetDstAlpha(int enable, u8 alpha)
{
  u8 _enable = (enable)? GX_ENABLE : GX_DISABLE;

  if (state_enable_dst_alpha != _enable ||
      state_dst_alpha != alpha)
  {
    GXSetDstAlpha(_enable, alpha);
    state_enable_dst_alpha = _enable;
    state_dst_alpha = alpha;
  }
}

/**
 * Zr̎{ʒuݒ肷
 * 
 * before_texɐ^lw肵ꍇAZrTEVXe[W̎sOɍsB
 * ̏ꍇAAt@r͎{ȂȂB
 * 
 * @param before_tex	^̂ƂAZrTEV̑Oōs
 */
void
HSD_StateSetZCompLoc(int before_tex)
{
  u8 _before_tex = (before_tex)? GX_TRUE : GX_FALSE;

  if (state_before_tex != _before_tex) {
    GXSetZCompLoc(_before_tex);
    state_before_tex = _before_tex;
  }
}

/**
 * fBUݒ肷
 * 
 * @param dither	^̂ƂfBULɂȂ
 */
void
HSD_StateSetDither(int dither)
{
  u8 _dither = (dither)? GX_ENABLE : GX_DISABLE;

  if (state_dither != _dither) {
    GXSetDither(_dither);
    state_dither = _dither;
  }
}

static void
_HSD_StateInvalidatePrimitive(void)
{
  state_line_width	= 0;
  state_point_size	= 0;
  state_cull_mode	= (GXCullMode) -1;
  state_line_width	= (GXBool) -1;
}

static void
_HSD_StateInvalidateVtxAttr(void)
{
  HSD_ClearVtxDesc();
}

static void
_HSD_StateInvalidateRenderMode(void)
{
  state_blend_type	 = (GXBlendMode) -1;
  state_src_factor	 = (GXBlendFactor) -1;
  state_dst_factor	 = (GXBlendFactor) -1;
  state_logic_op	 = (GXLogicOp) -1;
  state_z_enable	 = (GXBool) -1;
  state_z_func		 = (GXCompare) -1;
  state_z_update	 = (GXBool) -1;
  state_alpha_comp0	 = (GXCompare) -1;
  state_alpha_ref0	 = 0;
  state_alpha_op	 = (GXAlphaOp) -1;
  state_alpha_comp1	 = (GXCompare) -1;
  state_alpha_ref1	 = 0;
  state_color_update	 = (GXBool) -1;
  state_alpha_update	 = (GXBool) -1;
  state_enable_dst_alpha = (GXBool) -1;
  state_dst_alpha	 = 0;
  state_before_tex	 = (GXBool) -1;
  state_dither		 = (GXBool) -1;
}

/**
 * Xe[g𖳌B
 * 
 * baselib GP̏Ԃ̃Rs[iXe[gjɎ
 * ԂJԂݒ肷邱Ƃ̂Ȃ悤ɂĂB
 * 
 * {֐̓Xe[g𖳌邱Ƃ baselib ɁAKvɉ
 * n[hEFA̍ĐݒȂƂwB
 * 
 * GX߂pĒڃn[hEFȀԂݒ肵ꍇɂ́A
 * {֐pđΉXe[g𖳌Ȃ΂ȂȂB
 *
 * <pre>
 *          state           | 
 * -------------------------+-------------------------------------------
 *  HSD_STATE_PRIMITIVE     | `摮
 *                          | GXSetLineWidth()
 *                          | GXSetPointSize()
 *                          | GXSetCullMode()
 *  HSD_STATE_VTX_ATTR      | _
 *                          | GXSetVtxDesc()
 *  HSD_STATE_COLOR_CHANNEL | J[`l
 *                          | GXSetChan*()
 *                          | GXSetNumChans()
 *  HSD_STATE_TEV_STAGE     | TEVXe[W
 *                          | GXSetTev*()
 *                          | GXSetNumTevStages()
 *  HSD_STATE_TEV_REGISTER  | TEVJ[WX^^TEVRX^gWX^
 *                          | GXSetTevColor*()
 *  HSD_STATE_TEX_COORD_GEN |
 *                          | GXSetTexCoordGen()
 *                          | GXSetNumTexGens()
 *  HSD_STATE_RENDER_MODE   |
 *                          | GXSetBlendMode()
 *                          | GXSetColorUpdate()
 *                          | GXSetAlphaCompare()
 *                          | GXSetAlphaUpdate()
 *                          | GXSetDstAlpha()
 *                          | GXSetZMode()
 *                          | GXSetZCompLoc()
 *  [group]                 |
 *  HSD_STATE_TEV_GROUP     | TEV_STAGE | TEV_REGISTER | TEX_COORD_GEN
 *  
 */
void
HSD_StateInvalidate(HSD_StateMask state)
{
  extern void _HSD_StateInvalidateColorChannel(void);
  extern void _HSD_StateInvalidateTevStage(void);
  extern void _HSD_StateInvalidateTevRegister(void);
  extern void _HSD_StateInvalidateTexCoordGen(void);

  static struct {
    HSD_StateMask mask;
    void (*func)(void);
  } invalidate_funcs[] = {
    { HSD_STATE_PRIMITIVE,	_HSD_StateInvalidatePrimitive },
    { HSD_STATE_VTX_ATTR,	_HSD_StateInvalidateVtxAttr },
    { HSD_STATE_COLOR_CHANNEL,	_HSD_StateInvalidateColorChannel },
    { HSD_STATE_TEV_STAGE,	_HSD_StateInvalidateTevStage },
    { HSD_STATE_TEV_REGISTER,	_HSD_StateInvalidateTevRegister },
    { HSD_STATE_TEX_COORD_GEN,	_HSD_StateInvalidateTexCoordGen },
    { HSD_STATE_RENDER_MODE,	_HSD_StateInvalidateRenderMode },
    { HSD_STATE_NONE, NULL }
  };
  int i;

  for (i = 0; invalidate_funcs[i].mask != HSD_STATE_NONE; i++) {
    if (state & invalidate_funcs[i].mask) {
      invalidate_funcs[i].func();
    }
  }
}
