////////////////////////////////////////////////////////////////////////////////
//
// Killer7 Select InterFace
//
// WK7SelectIF.cpp
//
// Creator       : Watanabe Kazuhisa (GHM)
// Creation Date : 02/18/2003
//
////////////////////////////////////////////////////////////////////////////////

#define	__WK7SELECTIF_CPP__

//#include <dolphin.h>
#include ".\WK7SelectIF.h"
#include ".\WK7Define.h"
#include ".\WK7MainTask.h"
#include "..\Fujita\C_Font.h"
#include "..\Stage\K7Stage121M00.h"
#include ".\WK7Event.h"
#include ".\WK7GameInfo.h"
#include ".\WK7Player.h"
#include ".\WK7MainMenu.h"

#define	FLOAT_ERROR			(0.00001f)

static	f32		size_min = 2.5f,size_max = 8.0f;
static	u32	COLOR_WHITE			=	(0xffffff00);
static	u32	COLOR_RED			=	(0xe8100800);
//static	u32	COLOR_RED			=	(0xff000000);
//static	u32	COLOR_RED			=	(0x991f3600);
//static	u32	COLOR_RED			=	(0xff006c00);
static	u32	COLOR_BLUE			=	(0x0020ff00);
static	u32	HIGHLIGHTTED_COLOR	=	(0xff006c00);
//static	u32	HIGHLIGHTTED_COLOR	=	(0xd0006e00);
//#define	HIGHLIGHTTED_COLOR		(0xf5e61700)

f32		WK7SelectIF::mAppearDist = 40.0f;

//#define	DBGPAD

#define	WK7SELECTIF_LINESIZE		(5.0f)
//#define	WK7SELECTIF_LINESIZE		(2.0f)

WK7SelectObj::WK7SelectObj() : WK7Obj(&(WK7SELECTIF->mObjAnchor))
{
	mpObjName[0]     = 
	mpObjName[1]     = NULL;
	mFlag            = 0;
	mPlayEnable      = 1;
	mpSelectCallback = NULL;
	mParam   [0]     = 
	mParam   [1]     = 0;
	mSelectEventID   = 0xffffffff;
}

WK7SelectObj::~WK7SelectObj()
{
	GHMGCMEM_SAFE_FREE(mpObjName[0]);
	GHMGCMEM_SAFE_FREE(mpObjName[1]);
	WK7SELECTIF->ReleaseObj(this);
}

bool WK7SelectObj::SetParam(int index,u32 param)
{
	if( index < 0 || index > 1 ){
		return false;
	}
	mParam[index] = param;
	return true;
}

u32 WK7SelectObj::GetParam(int index)
{
	if( index < 0 || index > 1 ){
		return 0;
	}
	return mParam[index];
}

bool WK7SelectObj::SetObjName(int index,const char *pName)
{
	if( index < 0 || index > 1 ){
		return false;
	}
	GHMGCMEM_SAFE_FREE(mpObjName[index]);
	if( pName==NULL ){
		return true;
	}
	int		len = (int)strlen(pName);
	ghmGcMemSetAllocName("SLONM");
	mpObjName[index] = (char *)K7AppSmallMemAlloc(len+1);
	strcpy(mpObjName[index],pName);
	return true;
}

const char* WK7SelectObj::GetObjName(int index)
{
	if( index < 0 || index > 1 ){
		return NULL;
	}
	return mpObjName[index];
}

WK7SelectObjCallback* WK7SelectObj::SetSelectCallback(WK7SelectObjCallback *pSelectCallback)
{
	WK7SelectObjCallback	 *pOldCallback = mpSelectCallback;
	mpSelectCallback = pSelectCallback;
	return pOldCallback;
}

bool WK7SelectObj::CallSelectCallback(void)
{
	if( mpSelectCallback==NULL ){
		return true;
	}
	if( mPlayEnable ){
		GameInfo.SetMode(WK7GameInfo::PLAY);
	}
	switch( mpSelectCallback(this) ){
	case	SELECT_CANCEL:
		return false;
	case	SELECT_EVENT_CANCEL:
		return true;
	case	SELECT_OK:
		if( mSelectEventID != 0xffffffff ){
			WK7EVENTCONTROLLER->Call(mSelectEventID,(u32)this);
		}else{
			GameInfo.SetMode(WK7GameInfo::MOVE);
		}
		return true;
	default:
		break;
	}
	return false;
}

WK7SelectObjSphere::WK7SelectObjSphere() : WK7SelectObj()
{
	mSphere.mRadius = 1.0f;
}

WK7SelectObjSphere::~WK7SelectObjSphere()
{
}

bool WK7SelectObjSphere::SetRadius(f32 radius)
{
	if( radius < 0.0f ){
		return false;
	}
	mSphere.mRadius = radius;
	return true;
}

bool WK7SelectObjSphere::SetPosition(const Vec &pos)
{
	mSphere.mPosition = pos;
	return true;
}

bool WK7SelectObjSphere::GetPosition(Vec &pos)
{
	pos = mSphere.mPosition;
	return true;
}

WK7SelectObjSphereJObj::WK7SelectObjSphereJObj(WK7JObj *pJObj)
{
	mpJObj = pJObj;
}

WK7SelectObjSphereJObj::~WK7SelectObjSphereJObj()
{
}

bool WK7SelectObjSphereJObj::SetPosition(const Vec &pos)
{
	if( mpJObj==NULL ){
		return false;
	}
	mpJObj->SetPosition(pos);
	mSphere.mPosition = pos;
	return true;
}

bool WK7SelectObjSphereJObj::GetPosition(Vec &pos)
{
	if( mpJObj==NULL ){
		return false;
	}
	pos = mSphere.mPosition;
	return true;
}

bool WK7SelectObjSphereJObj::FrameProcess(void)
{
	if( mpJObj==NULL ){
		return true;
	}
	HSD_JObj	*pJObj = mpJObj->GetJObj();
	if( pJObj==NULL ){
		return true;
	}
	mpJObj->GetPosition(mSphere.mPosition);
	mSphere.mRadius = HSD_JObjGetSx(pJObj);
	return true;
}

int WK7SelectObjSphere::GetCrossState(ghmGcViewVolume &vv)
{
	return vv.GetCrossState(mSphere);
}


WK7SelectItemFrameLineSize::WK7SelectItemFrameLineSize()
{
	mMin = 1.0f;
	mMax = 10.0f;
	mSize.SetValue(mMin);
static	f32	COLORRATE_MAX	=	(1.0f);
static	f32	COLORRATE_MIN	=	(0.5f);
//#define	COLORRATE_MAX		(1.0f)
//#define	COLORRATE_MIN		(0.2f)
	mColorRate = (COLORRATE_MAX - COLORRATE_MIN) * HSD_Randf() + COLORRATE_MIN;
}

bool WK7SelectItemFrameLineSize::SetRange(f32 max,f32 min)
{
	if( min > max ){
		return false;
	}
	mMin = min;
	mMax = max;
	if( mSize.GetValue() < mMin ){
		mSize.SetValue(mMin);
	}else if( mSize.GetValue() > mMax ){
		mSize.SetValue(mMax);
	}
	return true;
}

bool WK7SelectItemFrameLineSize::SetSize(f32 size)
{
	if( size < mMin || size > mMax ){
		return false;
	}
	mSize.SetValue(size);
	return true;
}

WK7SelectItemFrameLineSize::~WK7SelectItemFrameLineSize()
{
}

bool WK7SelectItemFrameLineSize::FrameProcess(void)
{
	mSize.FrameProcess();
	if( mSize.IsAnimating()==true ){
		return true;
	}
	f32		rn = HSD_Randf();
	rn = rn * rn * 0xa00;
	mSize.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(rn + 0x800));
//	mSize.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(rn + 0x200));
//	mSize.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(HSD_Rand() % 0x600 + 0x600));
	return true;
}

WK7SelectItemFrameLinePoint::WK7SelectItemFrameLinePoint()
{
	mMin = 0.0f;
	mMax = 1.0f;
	mRate.SetValue(mMin);
}

WK7SelectItemFrameLinePoint::~WK7SelectItemFrameLinePoint()
{
}

bool WK7SelectItemFrameLinePoint::SetRange(f32 max,f32 min)
{
	if( min > max ){
		return false;
	}
	mMin = min;
	mMax = max;
	if( mRate.GetValue() < mMin ){
		mRate.SetValue(mMin);
	}else if( mRate.GetValue() > mMax ){
		mRate.SetValue(mMax);
	}
	return true;
}

bool WK7SelectItemFrameLinePoint::SetRate(f32 rate)
{
	if( rate < mMin || rate > mMax ){
		return false;
	}
	mRate.SetValue(rate);
	return true;
}

bool WK7SelectItemFrameLinePoint::FrameProcess(void)
{
	mRate.FrameProcess();
	if( mRate.IsAnimating()==true ){
		return true;
	}
	f32		rn = HSD_Randf();
	rn = rn * rn * 0xa00;
	mRate.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(rn + 0x800));
//	mRate.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(rn + 0x200));
//	mRate.Play(HSD_Randf() * (mMax - mMin) + mMin,(u32)(HSD_Rand() % 0x600 + 0x600));
	return true;
}

f32	WK7SelectItemFrameLine::mRenderZ = 0.0f;
WK7SelectItemFrameLine::WK7SelectItemFrameLine()
{
	mpPosition[0] = 
	mpPosition[1] = NULL;
	mColor    [0] = 
	mColor    [1] = 0xffffffff;
	mpPoint       = NULL;
	mPointCount   = 0;
	mpSize        = NULL;
}

WK7SelectItemFrameLine::~WK7SelectItemFrameLine()
{
	GHM_SAFE_DELETE_ARRAY(mpPoint);
	GHM_SAFE_DELETE_ARRAY(mpSize );
}

bool WK7SelectItemFrameLine::SetDivCount(int count)
{
	int			i;
	mPointCount = count;
	if( count > 0 ){
		GHM_SAFE_DELETE_ARRAY(mpPoint);
		ghmGcMemSetAllocName("SIFIFLP");
		if( (mpPoint = new WK7SelectItemFrameLinePoint[count])!=NULL ){
			f32		rate  = 0.0f;
			f32		drate = 1.0f / count;
			for(i=0 ; i<count ; i++,rate+=drate){
				mpPoint[i].SetRange(rate + drate,rate);
			}
		}
	}
	if( count >= 0 ){
		GHM_SAFE_DELETE_ARRAY(mpSize);
		ghmGcMemSetAllocName("SIFIFLS");
		mpSize = new WK7SelectItemFrameLineSize[count + 1];
	}
	return true;
}

void WK7SelectItemFrameLine::SetPosition(Vec *pStartPos,Vec *pEndPos)
{
	mpPosition[0] = pStartPos;
	mpPosition[1] = pEndPos;
}

void WK7SelectItemFrameLine::SetPositionP(Vec *pStartPos,Vec *pEndPos)
{
	mpPositionP[0] = pStartPos;
	mpPositionP[1] = pEndPos;
}

void WK7SelectItemFrameLine::SetColor(u32 scol,u32 ecol)
{
	mColor[0] = scol;
	mColor[1] = ecol;
}

bool WK7SelectItemFrameLine::FrameProcess(void)
{
	int		i;
	if( mPointCount > 0 ){
		for(i=0 ; i<mPointCount ; i++){
			mpPoint[i].FrameProcess();
		}
	}
	if( mpSize!=NULL && mPointCount >= 0 ){
		for(i=0 ; i<=mPointCount ; i++){
			mpSize[i].FrameProcess();
		}
	}
	return true;
}

class InterpolateColor{
public:
	u32				mRGB;
	u32				mR;
	u32				mG;
	u32				mB;
	u32				mA;
	InterpolateColor();
	InterpolateColor(u32 rgb0,u32 rgb1,f32 rate);
	void Interpolate(u32 rgb0,u32 rgb1,f32 rate);
};

InterpolateColor::InterpolateColor()
{
	mRGB = mR = mG = mB = mA = 0;
}

InterpolateColor::InterpolateColor(u32 rgb0,u32 rgb1,f32 rate)
{
	Interpolate(rgb0,rgb1,rate);
}

void InterpolateColor::Interpolate(u32 rgb0,u32 rgb1,f32 rate)
{
	u32			r0,g0,b0;
	u32			r1,g1,b1;
	f32			rf,gf,bf;
	f32			invrate = 1.0f - rate;
	
	mA =  rgb0        & 0xff;
	r0 =  rgb0 >> 24;
	g0 = (rgb0 >> 16) & 0xff;
	b0 = (rgb0 >>  8) & 0xff;
	r1 =  rgb1 >> 24;
	g1 = (rgb1 >> 16) & 0xff;
	b1 = (rgb1 >>  8) & 0xff;
	
	rf = r0 * invrate + r1 * rate;
	gf = g0 * invrate + g1 * rate;
	bf = b0 * invrate + b1 * rate;
	GHM_LIMIT(rf,255.0f,0.0f);
	GHM_LIMIT(gf,255.0f,0.0f);
	GHM_LIMIT(bf,255.0f,0.0f);
	
	mR = (u32)rf;
	mG = (u32)gf;
	mB = (u32)bf;
	
	mRGB = (mR<<24)|(mG<<16)|(mB<<8)|mA;
}

bool WK7SelectItemFrameLine::RenderProcess(f32 linesize)
{
	if( mpPosition [0]==NULL || mpPosition [1]==NULL ||
	    mpPositionP[0]==NULL || mpPositionP[1]==NULL ||
	    mPointCount<0 || mpSize==NULL ){
		return false;
	}
	
	int								i;
	Vec								vdir,hdir,hdir0;
	WK7SelectItemFrameLineSize		*pSize  = mpSize;
	WK7SelectItemFrameLinePoint		*pPoint = mpPoint;
	f32								rate,prev_rate = 0.0f;
	f32								size;
	Vec								pos[2][2];
//	InterpolateColor				col,prevcol;
	InterpolateColor				col;
	
	VECSubtract (mpPosition[1],mpPosition[0],&vdir);
	vdir .z = 0.0f;
	
	if( VECMag(&vdir) < FLOAT_ERROR ){
		return true;
	}
	
	hdir0.x =  vdir.y;
	hdir0.y = -vdir.x;
	hdir0.z = 0.0f;
	VECNormalize(&hdir0,&hdir0);
	VECScale    (&hdir0,&hdir0,linesize);
	
	pos[0][0].z = 0.0f;
	pos[0][1].z = 0.0f;
	pos[1][0].z = 0.0f;
	pos[1][1].z = 0.0f;
	
//	prevcol.Interpolate(mColor[0],mColor[0],0.0f);
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,(u16)(4 + mPointCount * 4));
	
	for(i=0 ; i<mPointCount ; i++,prev_rate=rate,pPoint++,pSize++){
		
		rate = pPoint->mRate.GetValue();
		size = pSize ->mSize.GetValue() / linesize;
		
//		col.Interpolate(mColor[0],mColor[1],rate);
//		col.Interpolate(0xffffff00|(mColor[1] & 0xff),mColor[1],pSize->GetColorRate());
		col.Interpolate(0xffffff00|(mColor[1] & 0xff),mColor[1],1.0f);
		
		if( prev_rate==0.0f ){
			pos[0][0].x = mpPosition [0]->x;
			pos[0][0].y = mpPosition [0]->y;
			pos[0][1].x = mpPositionP[0]->x;
			pos[0][1].y = mpPositionP[0]->y;
			VECSubtract (&pos[0][1],&pos[0][0],&hdir);
			VECScale    (&hdir,&hdir,size);
		}else{
			VECScale    (&vdir         ,&pos[0][0],prev_rate);
			VECAdd      (mpPosition [0],&pos[0][0],&pos[0][0]);
			VECScale    (&vdir         ,&pos[0][1],prev_rate);
			VECAdd      (mpPositionP[0],&pos[0][1],&pos[0][1]);
			VECScale    (&hdir0,&hdir,size);
		}
		VECAdd      (&pos[0][0],&hdir,&pos[0][1]);
		VECSubtract (&pos[0][0],&hdir,&pos[0][0]);
		
		VECScale    (&vdir         ,&pos[1][0],rate);
		VECAdd      (mpPosition [0],&pos[1][0],&pos[1][0]);
		VECScale    (&vdir         ,&pos[1][1],rate);
		VECAdd      (mpPositionP[0],&pos[1][1],&pos[1][1]);
		VECScale    (&hdir0,&hdir,size);
		VECAdd      (&pos[1][0],&hdir,&pos[1][1]);
		VECSubtract (&pos[1][0],&hdir,&pos[1][0]);
		
		GXPosition3f32(pos[0][0].x,pos[0][0].y,mRenderZ);
//		GXColor1u32   (prevcol.mRGB);
		GXColor1u32   (col.mRGB);
		GXPosition3f32(pos[0][1].x,pos[0][1].y,mRenderZ);
//		GXColor1u32   (prevcol.mRGB);
		GXColor1u32   (col.mRGB);
		
		GXPosition3f32(pos[1][0].x,pos[1][0].y,mRenderZ);
		GXColor1u32   (col.mRGB);
		GXPosition3f32(pos[1][1].x,pos[1][1].y,mRenderZ);
		GXColor1u32   (col.mRGB);
		
//		prevcol = col;
	}
	
	size = pSize ->mSize.GetValue() / linesize;
//	col.Interpolate(mColor[1],mColor[1],1.0f);
	col.Interpolate(0xffffff00|(mColor[1] & 0xff),mColor[1],pSize->GetColorRate());
	
	if( prev_rate==0.0f ){
		pos[0][0].x = mpPosition [0]->x;
		pos[0][0].y = mpPosition [0]->y;
		pos[0][1].x = mpPositionP[0]->x;
		pos[0][1].y = mpPositionP[0]->y;
		VECSubtract (&pos[0][1],&pos[0][0],&hdir);
		VECScale    (&hdir,&hdir,size);
	}else{
		VECScale    (&vdir         ,&pos[0][0],prev_rate);
		VECAdd      (mpPosition [0],&pos[0][0],&pos[0][0]);
		VECScale    (&vdir         ,&pos[0][1],prev_rate);
		VECAdd      (mpPositionP[0],&pos[0][1],&pos[0][1]);
		VECScale    (&hdir0,&hdir,size);
	}
	VECAdd      (&pos[0][0],&hdir,&pos[0][1]);
	VECSubtract (&pos[0][0],&hdir,&pos[0][0]);
	
	pos[1][0].x = mpPosition [1]->x;
	pos[1][0].y = mpPosition [1]->y;
	pos[1][1].x = mpPositionP[1]->x;
	pos[1][1].y = mpPositionP[1]->y;
	VECSubtract (&pos[1][1],&pos[1][0],&hdir);
	VECScale    (&hdir,&hdir,size);
	VECAdd      (&pos[1][0],&hdir,&pos[1][1]);
	VECSubtract (&pos[1][0],&hdir,&pos[1][0]);
	
	GXPosition3f32(pos[0][0].x,pos[0][0].y,mRenderZ);
	GXColor1u32   (col.mRGB);
//	GXColor1u32   (prevcol.mRGB);
	GXPosition3f32(pos[0][1].x,pos[0][1].y,mRenderZ);
	GXColor1u32   (col.mRGB);
//	GXColor1u32   (prevcol.mRGB);
	
	GXPosition3f32(pos[1][0].x,pos[1][0].y,mRenderZ);
	GXColor1u32   (col.mRGB);
	GXPosition3f32(pos[1][1].x,pos[1][1].y,mRenderZ);
	GXColor1u32   (col.mRGB);
	
	GXEnd();
	
	return true;
}

WK7SelectItemLockOn::WK7SelectItemLockOn()
{
	mState = IDLE;
	mPosX  = 
	mPosY  = 
	mZ     = 0.0f;
	mColor = 0xffffffff;
	mCount.SetValue(0);
}

WK7SelectItemLockOn::~WK7SelectItemLockOn()
{
}

bool WK7SelectItemLockOn::SetPosition(f32 x,f32 y)
{
	mPosX = x;
	mPosY = y;
	return true;
}

#define	WK7SELECTITEMLOCKON_APPEAR		(WK7_TICK_PER_SEC / 2)
#define	WK7SELECTITEMLOCKON_BLINK		(WK7_TICK_PER_SEC / 2)
bool WK7SelectItemLockOn::FrameProcess(void)
{
	mCount.FrameProcess();
	switch( mState ){
	case	IDLE:
//		mCount.SetValue(WK7SELECTITEMLOCKON_APPEAR);
//		mCount.Play    (0,mCount.GetValue());
//		mState = LOCKING;
		mCount.SetValue(WK7SELECTITEMLOCKON_BLINK);
		mCount.Play    (0,mCount.GetValue());
		mState = LOCKED;
		break;
	case	LOCKING:
		if( mCount.IsAnimating()==true ){
			break;
		}
		mState = LOCKED;
	case	LOCKED:
		if( mCount.IsAnimating()==false ){
			mCount.SetValue(WK7SELECTITEMLOCKON_BLINK);
			mCount.Play    (0,mCount.GetValue());
		}
		break;
	}
	return true;
}

struct Vec2D{
	f32		x;
	f32		y;
};
#define	LOCKONCURSOR_SIZE		(4.0f)
#define	LOCKONCURSOR_POS(x,y)	{ ((x) * LOCKONCURSOR_SIZE) , ((y) * LOCKONCURSOR_SIZE) }
static	Vec2D	lockOnCursorPos[4][6] = {
	{
		LOCKONCURSOR_POS(-1.0f,-0.0f),
		LOCKONCURSOR_POS(-2.0f,-0.0f),
		LOCKONCURSOR_POS(-1.0f,-1.0f),
		LOCKONCURSOR_POS(-2.0f,-2.0f),
		LOCKONCURSOR_POS(-0.0f,-1.0f),
		LOCKONCURSOR_POS(-0.0f,-2.0f),
	},
	{
		LOCKONCURSOR_POS( 1.0f,-0.0f),
		LOCKONCURSOR_POS( 2.0f,-0.0f),
		LOCKONCURSOR_POS( 1.0f,-1.0f),
		LOCKONCURSOR_POS( 2.0f,-2.0f),
		LOCKONCURSOR_POS( 0.0f,-1.0f),
		LOCKONCURSOR_POS( 0.0f,-2.0f),
	},
	{
		LOCKONCURSOR_POS(-1.0f, 0.0f),
		LOCKONCURSOR_POS(-2.0f, 0.0f),
		LOCKONCURSOR_POS(-1.0f, 1.0f),
		LOCKONCURSOR_POS(-2.0f, 2.0f),
		LOCKONCURSOR_POS(-0.0f, 1.0f),
		LOCKONCURSOR_POS(-0.0f, 2.0f),
	},
	{
		LOCKONCURSOR_POS( 1.0f, 0.0f),
		LOCKONCURSOR_POS( 2.0f, 0.0f),
		LOCKONCURSOR_POS( 1.0f, 1.0f),
		LOCKONCURSOR_POS( 2.0f, 2.0f),
		LOCKONCURSOR_POS( 0.0f, 1.0f),
		LOCKONCURSOR_POS( 0.0f, 2.0f),
	},
};
bool RenderLockOnCursor(Vec &pos,Vec2D *pVec);
bool RenderLockOnCursor(Vec &pos,Vec2D *pVec)
{
	int		i;
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,6);
		
	for(i=0 ; i<6 ; i++,pVec++){
		GXPosition3f32(pos.x + pVec->x,pos.y + pVec->y,pos.z);
	}
	
	GXEnd();
	
	return true;
}
static	Vec2D	lockOnCursorInPos[4] = {
	{ 0.0f             , 0.0f              },
	{ WK7_SCREEN_WIDTH , 0.0f              },
	{ 0.0f             , WK7_SCREEN_HEIGHT },
	{ WK7_SCREEN_WIDTH , WK7_SCREEN_HEIGHT },
};
static f32	LOCKONCURSOR_BLINK_MAXSIZE	=	(20.0f);
static f32	LOCKONCURSOR_BLINK_MINSIZE	=	( 5.0f);
//#define	LOCKONCURSOR_BLINK_MAXSIZE		(10.0f)
//#define	LOCKONCURSOR_BLINK_MINSIZE		( 2.0f)
static	Vec2D	lockOnCursorBlinkDeltaDir[4] = {
	{ -1.0f,-1.0f },
	{  1.0f,-1.0f },
	{ -1.0f, 1.0f },
	{  1.0f, 1.0f },
};
bool WK7SelectItemLockOn::RenderProcess(void)
{
	
	Vec				pos[4];
	int				i;
	f32				rate;
	u32				color;
	
	switch( mState ){
	case	LOCKING:
		rate = (f32)mCount.GetValue() / WK7SELECTITEMLOCKON_APPEAR;
//		color = (mColor & 0xffffff00)|(u32)((mColor & 0xff) * (1.0f - rate));
		for(i=0 ; i<4 ; i++){
			pos[i].x  = LOCKONCURSOR_BLINK_MINSIZE * lockOnCursorBlinkDeltaDir[i].x + mPosX;
			pos[i].x += (lockOnCursorInPos[i].x - pos[i].x) * rate;
			pos[i].y  = LOCKONCURSOR_BLINK_MINSIZE * lockOnCursorBlinkDeltaDir[i].y + mPosY;
			pos[i].y += (lockOnCursorInPos[i].y - pos[i].y) * rate;
			pos[i].z  = mZ;
		}
		break;
	case	LOCKED:
		rate = (f32)mCount.GetValue() / WK7SELECTITEMLOCKON_BLINK;
//		color = mColor;
		for(i=0 ; i<4 ; i++){
			pos[i].x  = LOCKONCURSOR_BLINK_MINSIZE * lockOnCursorBlinkDeltaDir[i].x + mPosX;
			pos[i].x += rate * (LOCKONCURSOR_BLINK_MAXSIZE - LOCKONCURSOR_BLINK_MINSIZE) * lockOnCursorBlinkDeltaDir[i].x;
			pos[i].y  = LOCKONCURSOR_BLINK_MINSIZE * lockOnCursorBlinkDeltaDir[i].y + mPosY;
			pos[i].y += rate * (LOCKONCURSOR_BLINK_MAXSIZE - LOCKONCURSOR_BLINK_MINSIZE) * lockOnCursorBlinkDeltaDir[i].y;
			pos[i].z  = mZ;
		}
		break;
	case	IDLE:
	default:
		return true;
	}
	
	color =  mColor & 0xff;
//	color = (mColor & 0xff) * WK7SELECTIF->GetBackAlphaValue() / 0xff;
	color = (mColor & 0xffffff00)|(u32)(color * (1.0f - rate));
	
	GXSetDither      (GX_FALSE);
	GXSetDstAlpha    (0,0);
	GXSetAlphaCompare(GX_ALWAYS,0,GX_AOP_AND,GX_ALWAYS,0);
	
	GXSetNumChans    (1);
	GXSetChanCtrl    (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetChanMatColor(GX_COLOR0A0,*((GXColor*)&color));
	
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode    (GX_CULL_NONE);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetZCompLoc    (GX_TRUE);
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS ,GX_DIRECT);
	GXSetVtxAttrFmt  (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	
	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_TEX0 , GX_TEX_ST   , GX_F32   , 0);
	
	for(i=0 ; i<4 ; i++){
		RenderLockOnCursor(pos[i],lockOnCursorPos[i]);
	}
	
	return true;
}

bool	WK7SelectItem::mTargetCircleVisible = false;
int		WK7SelectItem::mPadDirection        = -1;
GXColor	WK7SelectItem::mNormalColor         = { 0xff,0xff,0xff,0x00 };
//GXColor	WK7SelectItem::mNormalColor         = { 0x80,0x80,0x80,0x00 };
GXColor	WK7SelectItem::mSelectedColor       = { 0xff,0x20,0x00,0x00 };
//GXColor	WK7SelectItem::mSelectedColor       = { 0xff,0x40,0x00,0x00 };
//GXColor	WK7SelectItem::mSelectedColor       = { 0xff,0x00,0x00,0x00 };
Vec		WK7SelectItem::mWorldItemCenter;
Vec		WK7SelectItem::mScreenItemCenter;
f32		WK7SelectItem::mProjection[GX_PROJECTION_SZ];
f32		WK7SelectItem::mViewport  [GX_VIEWPORT_SZ  ];
Mtx		WK7SelectItem::mViewMatrix;
u32		WK7SelectItem::mIDSeed = 0x12345678;	// K

static	f32		tgth = 5.0f;

static	f32		cutratemin = 0.2f;
static	f32		cutratemax = 0.5f;
static	int		wK7SelectItemRouteID = 0;

#define	COLOR_RANGE			(100)
#define	RATEPRIM_NUM		(4)
#define	RATEPRIM_SCALE		(0.5f)

#define	SELECTITEM_LETTER		"KILL"
#define	SELECTITEM_LETTER_NUM	(sizeof(SELECTITEM_LETTER) - 1)
//#define	SELECTITEM_LETTER	"GO TO KILL"
static	const char	*pSelectItemLetter  = SELECTITEM_LETTER;
static	int			selectItemLetterNum = sizeof(SELECTITEM_LETTER) - 1;

WK7Letter::WK7Letter()
{
	mCurPosX = 
	mCurPosY = 
	mDstPosX = 
	mDstPosY = 0.0f;
//	mRGBA    = 0xffffffff;
//	mRGBA    = COLOR_WHITE|0x000000ff;
	mRGBA    = COLOR_RED|0x000000ff;
	mType    = FONTTYPE_ARIAL;
	mLetter  = ' ';
	mDisp    = 0;
	SetTexture();
	mPrim.mSetXY0 (0,0);
	mPrim.mSetWH  (0,0);
	mPrim.mSetUV0 (0,0);
	mPrim.mSetRGBA(mRGBA);
	mPrim.mSetDispFlag(false);
}

#define	LETTER_SCALE		(0.75f)
//#define	LETTER_SCALE		(0.5f)
bool WK7Letter::SetPrim(void)
{
	stFontParam	*pFontParam = GetFontParam();
	if( pFontParam==NULL || mLetter==' ' ){
		s16		w,h;
		w = 6;
		if( (pFontParam = ::GetFontParam('0',mType))!=NULL ){
			h = (s16)pFontParam->h;
		}else if( (pFontParam = gpTypoMng->mGetFontParamPtr())!=NULL ){
			h = (s16)pFontParam->h;
		}else{
			h = 40;
		}
		mDisp = 0;
		mPrim.mSetWH (w,h);
		mPrim.mSetScale(eScaleWH,LETTER_SCALE,1);
		mPrim.mScaleProc();
		mPrim.mSetDispFlag(false);
		return false;
	}
	mPrim.mSetWH ((s16)pFontParam->w,(s16)pFontParam->h);
	mPrim.mSetUV0((s16)pFontParam->u,(s16)pFontParam->v);
	mPrim.mSetScale(eScaleWH,LETTER_SCALE,1);
	mPrim.mScaleProc();
	return true;
}

bool WK7Letter::SetTexture(void)
{
	stTexture		*pTexture = gpTypoMng->mGetTexStrPtr();
	if( pTexture==NULL ){
		return false;
	}
	mPrim.mRelateTex(pTexture);
	return true;
}

stFontParam* WK7Letter::GetFontParam(void)
{
	stFontParam		*pFontParam = ::GetFontParam((char)mLetter,mType);
	if( pFontParam==NULL ){
		return false;
	}
	return pFontParam;
}

void WK7Letter::SetLetter(s8 letter)
{
	mLetter = letter;
	SetPrim();
}

bool WK7Letter::SetType(int type)
{
	switch( type ){
	case	FONTTYPE_ARIAL:
	case	FONTTYPE_ARIALBLACK:
		break;
	default:
		return false;
	}
	mType = type;
	SetPrim();
	return true;
}

void WK7Letter::SetDisp(bool flag)
{
	mDisp = (u8)(flag==true? 1:0);
	mPrim.mSetDispFlag(flag);
	SetPrim();
}

void WK7Letter::SetXY(f32 x,f32 y,bool im)
{
	mDstPosX = x;
	mDstPosY = y;
	if( im==true ){
		mCurPosX = x;
		mCurPosY = y;
	}
}

void WK7Letter::SetRGBA(u32 rgba)
{
	mRGBA = rgba;
	mPrim.mSetRGBA(mRGBA);
}

void WK7Letter::SetRGB(u32 rgb)
{
	mRGBA = (rgb & 0xffffff00)|(mRGBA & 0xff);
	mPrim.mSetRGBA(mRGBA);
}

void WK7Letter::SetR(u32 r)
{
	mRGBA = ((r << 24) & 0xff000000)|(mRGBA & 0x00ffffff);
	mPrim.mSetRGBA(mRGBA);
}

void WK7Letter::SetG(u32 g)
{
	mRGBA = ((g << 16) & 0x00ff0000)|(mRGBA & 0xff00ffff);
	mPrim.mSetRGBA(mRGBA);
}

void WK7Letter::SetB(u32 b)
{
	mRGBA = ((b <<  8) & 0x0000ff00)|(mRGBA & 0xffff00ff);
	mPrim.mSetRGBA(mRGBA);
}

void WK7Letter::SetA(u32 a)
{
	mRGBA = (a & 0xff)|(mRGBA & 0xffffff00);
	mPrim.mSetRGBA(mRGBA);
}

bool WK7Letter::FrameProcess(void)
{
	mCurPosX += (mDstPosX - mCurPosX) * 0.125f;
	mCurPosY += (mDstPosY - mCurPosY) * 0.125f;
	mPrim.mSetXY0(mCurPosX,mCurPosY);
	return true;
}

WK7SelectItemFrameBlur::WK7SelectItemFrameBlur(ghmGcListObj *pAnchor,WK7SelectItemFrame &frame,u32 color,f32 z) : WK7Obj(pAnchor)
{
	mColor     = color & 0xffffff00;
	mRenderZ   = z;
	mFrame     = frame;
	mTerminate = false;
	mAlpha.SetValue(0xff);
	mAlpha.Play    (0x00,WK7_TICK_PER_SEC / 2);
//	mAlpha.Play    (0x00,WK7_TICK_PER_SEC);
}

bool WK7SelectItemFrameBlur::FrameProcess(void)
{
	if( mTerminate==true ){
		return false;
	}
	if( mAlpha.GetValue()==0 ){
		mTerminate = true;
	}
	mAlpha.FrameProcess();
	return true;
}

bool WK7SelectItemFrameBlur::RenderProcess(void)
{
	if( mTerminate==true ){
		return true;
	}
	
	u32		color = mColor | mAlpha.GetValue();
	int		i;
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,8);
		
		for(i=0 ; i<4 ; i++){
			GXPosition3f32(mFrame.mCurrentFramePosition [i].x,mFrame.mCurrentFramePosition [i].y,mRenderZ);
			GXColor1u32   (color);
			GXPosition3f32(mFrame.mParallelFramePosition[i].x,mFrame.mParallelFramePosition[i].y,mRenderZ);
			GXColor1u32   (color);
		}
		
	GXEnd();
	
	return true;
}

#define	IS_EQUAL(a,b)		(fabs((a) - (b)) < FLOAT_ERROR)
bool WK7SelectItemFrameBlur::IsEqual(WK7SelectItemFrame &frame)
{
	int			i;
	for(i=0 ; i<4 ; i++){
		if( IS_EQUAL(mFrame.mCurrentFramePosition [i].x,frame.mCurrentFramePosition [i].x)==false ){
			return false;
		}
		if( IS_EQUAL(mFrame.mCurrentFramePosition [i].y,frame.mCurrentFramePosition [i].y)==false ){
			return false;
		}
		if( IS_EQUAL(mFrame.mParallelFramePosition[i].x,frame.mParallelFramePosition[i].x)==false ){
			return false;
		}
		if( IS_EQUAL(mFrame.mParallelFramePosition[i].y,frame.mParallelFramePosition[i].y)==false ){
			return false;
		}
	}
	return true;
}

WK7SelectItem::WK7SelectItem(ghmGcListObj *pAnchor,int prio) : WK7Obj(pAnchor,prio)
{
	mIcon             = (u32)TF_MAX;
	mpRatePrim        = NULL;
	mpLetter          = NULL;
	mID               = mIDSeed++;
//	mpLetter          = new WK7Letter[selectItemLetterNum];
//	for(int i=0 ; i<selectItemLetterNum ; i++){
//		mpLetter[i].SetType  (FONTTYPE_ARIALBLACK);
//		mpLetter[i].SetLetter(pSelectItemLetter[i]);
//	}
//	mpRatePrim        = new mK7Sprite[PRIMNUM];
//	stTexture	*pTexture   = gpTypoMng->mGetTexStrPtr();
//	for(int i=0 ; i<PRIMNUM ; i++){
//		mpRatePrim[i].mSetDispFlag(false);
//		mpRatePrim[i].mRelateTex(pTexture);
//		mpRatePrim[i].mSetRGBA(0xffffffff);
//	}
	mType             = TYPE_CUT;
	mpLockOn          = NULL;
	mpTargetPoints    = NULL;
	mTargetPointNum   = NULL;
	mCutInPosition .x = 0.0f;
	mCutInPosition .y = WK7_SCREEN_HEIGHT / 2.0f;
	mCutInPosition .z = 0.0f;
	mCutInDirection.x = 1.0f;
	mCutInDirection.y = 0.0f;
	mCutInDirection.z = 0.0f;
	memset(mFrame.mCutFramePosition,0,sizeof(mFrame.mCutFramePosition));
	mCutRate          = cutratemin + HSD_Randf() * (cutratemax - cutratemin);
	mFlag             = 0;
	mEnable           = 1;
	mEnableSelect     = 1;
	mResetTypoPos     = 1;
	mLineWidth        = 24;
	mRenderZ          = 0.0f;
	mState            = CUTFRAME;
	mAlpha.Play(0xff,WK7SELECTITEM_DEFAULT_FADETICK);
	
	mColorR.SetValue(0);
//	mColorR.SetValue(mNormalColor.r);
//	mColorG.SetValue(mNormalColor.g);
//	mColorB.SetValue(mNormalColor.b);
	
	mPrevScreenTargetCenter.x = 0.0f;
	mPrevScreenTargetCenter.y = 0.0f;
	mPrevScreenTargetCenter.z = 0.0f;
	mScreenTargetCenter.x = 0.0f;
	mScreenTargetCenter.y = 0.0f;
	mScreenTargetCenter.z = 0.0f;
	mTypoCenterCurrent .x = 0.0f;
	mTypoCenterCurrent .y = 0.0f;
	mTypoCenterCurrent .z = 0.0f;
	mTypoCenter        .x = 0.0f;
	mTypoCenter        .y = 0.0f;
	mTypoCenter        .z = 0.0f;
	mTypoCenterVelocity.x = 0.0f;
	mTypoCenterVelocity.y = 0.0f;
	mTypoCenterVelocity.z = 0.0f;
	mTypoCenterAccel   .x = 0.0f;
	mTypoCenterAccel   .y = 0.0f;
	mTypoCenterAccel   .z = 0.0f;
	mDirection       = -1;
	mSelected        = false;
	mParam0          = 0;
	mParam1          = 0;
	mpBranch         = NULL;
	mpBranchPointObj = NULL;
	mpObj            = NULL;
	mCallEventID     = 0xffffffff;
	mDeformRate.SetValue(0.0f);
	mLetterDirRate.SetValue(0.0f);
	mBlurCounter.SetValue(0);
	ghmGcMemSetAllocName("SIFIFL");
	if( (mpFrameLine = new WK7SelectItemFrameLine[3])!=NULL ){
		mpFrameLine[0].SetPosition (&mFrame.mCurrentFramePosition [0],&mFrame.mCurrentFramePosition [1]);
		mpFrameLine[0].SetPositionP(&mFrame.mParallelFramePosition[0],&mFrame.mParallelFramePosition[1]);
		mpFrameLine[0].SetDivCount(3);
		mpFrameLine[1].SetPosition (&mFrame.mCurrentFramePosition [1],&mFrame.mCurrentFramePosition [2]);
		mpFrameLine[1].SetPositionP(&mFrame.mParallelFramePosition[1],&mFrame.mParallelFramePosition[2]);
		mpFrameLine[1].SetDivCount(1);
		mpFrameLine[2].SetPosition (&mFrame.mCurrentFramePosition [2],&mFrame.mCurrentFramePosition [3]);
		mpFrameLine[2].SetPositionP(&mFrame.mParallelFramePosition[2],&mFrame.mParallelFramePosition[3]);
		mpFrameLine[2].SetDivCount(3);
	}
}

WK7SelectItem::~WK7SelectItem()
{
	Terminate();
}

u32 WK7SelectItem::GetID(void)
{
	return mID;
}

GXColor WK7SelectItem::GetSelectedColor(void)
{
	if( mHighLighted ){
		static GXColor	highLightedSelectedColor = { 0xcc,0xff,0x66,0x00 };
//		static GXColor	highLightedSelectedColor = { 0xff,0x00,0xff,0x00 };
		return highLightedSelectedColor;
	}
	return mSelectedColor;
}

GXColor WK7SelectItem::GetNormalColor(void)
{
	if( mHighLighted ){
		static GXColor	highLightedNormalColor = { 0x99,0xff,0x00,0x00 };
//		static GXColor	highLightedNormalColor = { 0x80,0x00,0x80,0x00 };
		return highLightedNormalColor;
	}
	return mNormalColor;
}

bool WK7SelectItem::SetTypo(const char *pName,const char *pDir,u32 icon)
{
	if( pName==NULL ){
		pName = "Unknown Route";
	}
	if( pDir==NULL ){
		pDir = "Unknown Direction";
	}
	if( icon!=0xffffffff ){
		mIcon = icon;
	}
	mTypos.mCreate((char *)pDir,(char *)pName,eTypoDisp00,mIcon);
//	mTypos.mSetRGBA(0xffffff00);
	return true;
}

bool WK7SelectItem::SetTargetPoints(WK7JObj *pObj)
{
	if( pObj==NULL ){
		return false;
	}
	return SetTargetPoints(pObj->GetJObj());
}

bool WK7SelectItem::SetTargetPoints(HSD_JObj *pJObj)
{
	if( pJObj==NULL || pJObj->u.dobj==NULL ){
		return false;
	}
	HSD_PObj	*pPObj;
	if( (pPObj=pJObj->u.dobj->pobj)==NULL ){
		return false;
	}
	HSD_VtxDescList		*pCurrVtxDesc = pPObj->desc_list;
	HSD_VtxDescList		*pVtxDesc = NULL;
	u32					stride = 0;
	while( pCurrVtxDesc->attr!=GX_VA_NULL ){
		if( pCurrVtxDesc->attr==GX_VA_POS ){
			pVtxDesc = pCurrVtxDesc;
		}
		switch( pCurrVtxDesc->attr_type ){
		case	GX_INDEX8 : stride++; break;
		case	GX_INDEX16: stride++; break;
		}
		pCurrVtxDesc++;
	}
	if( pVtxDesc == NULL ){
		return false;
	}
	Vec		*pPosition = (Vec *)pVtxDesc->base_ptr;
	int		vertnum    = (((u16)pPObj->dl[1])<<8)|pPObj->dl[2];
	int		num = 0;
	u8		*pVertID = pPObj->dl + 3;
	if( pVtxDesc->attr_type==GX_INDEX8 ){
		while( vertnum>0 ){
			if( num < *pVertID ){
				num = *pVertID;
			}
			pVertID += stride;
			vertnum--;
		}
		num++;
	}else if( pVtxDesc->attr_type==GX_INDEX16 ){
		int		id;
		while( vertnum>0 ){
			id = (((u16)pVertID[0]) << 8)|pVertID[1];
			if( num < id ){
				num = id;
			}
			pVertID += stride;
			vertnum--;
		}
		num++;
	}
	if( !num ){
		return false;
	}
	HSD_JObjSetupMatrix(pJObj);
	SetTargetPoints(pPosition,num,pJObj->mtx);
	return true;
}

bool WK7SelectItem::SetTargetPoints(const Vec *pPoints,int pointNum,MtxPtr pLocalWorldMatrix)
{
	GHMGCMEM_SAFE_FREE(mpTargetPoints);
	mTargetPointNum = pointNum;
	if( pointNum <= 0 ){
		return false;
	}
	int			i;
	Vec 		*pPoint;
	ghmGcMemSetAllocName("SLITTP");
	mpTargetPoints       = (Vec *)K7AppSmallMemAlloc((int)(sizeof(Vec) * 2 * pointNum));
	mpScreenTargetPoints = mpTargetPoints + pointNum;
	pPoint               = mpTargetPoints;
	if( pLocalWorldMatrix != NULL ){
		for(i=0 ; i<pointNum ; i++,pPoint++,pPoints++){
			MTXMultVec(pLocalWorldMatrix,pPoints,pPoint);
			GHM_VEC_SET_XYZ0(mpScreenTargetPoints);
		}
	}else{
		for(i=0 ; i<pointNum ; i++,pPoint++,pPoints++){
			GHM_VEC_COPY_XYZ(pPoint,pPoints);
			GHM_VEC_SET_XYZ0(mpScreenTargetPoints);
		}
	}
	return true;
}

void WK7SelectItem::SetCutInParam(const Vec *pCutInPosition,const Vec *pCutInDirection)
{
	float		invlen;
	invlen = 1.0f / GHM_VEC_GET_LEN(pCutInDirection);
	mCutInDirection.x = pCutInDirection->x * invlen;
	mCutInDirection.y = pCutInDirection->y * invlen;
	mCutInDirection.z = pCutInDirection->z * invlen;
	GHM_VEC_COPY_XYZ(&mCutInPosition,pCutInPosition);
}

bool WK7SelectItem::GetTargetPointAverage(Vec &ave)
{
	if( mTargetPointNum <= 0 || mpTargetPoints==NULL ){
		return false;
	}
	int				i;
	Vec				*pTargetPoint = mpTargetPoints;
	f32				invnum = 1.0f / mTargetPointNum;
	ave.x = 0.0f;
	ave.y = 0.0f;
	ave.z = 0.0f;
	for(i=0 ; i<mTargetPointNum ; i++,pTargetPoint++){
		ave.x += pTargetPoint->x;
		ave.y += pTargetPoint->y;
		ave.z += pTargetPoint->z;
	}
	ave.x *= invnum;
	ave.y *= invnum;
	ave.z *= invnum;
	return true;
}

bool WK7SelectItem::Initialize(void)
{
	return true;
}

bool WK7SelectItem::Terminate(void)
{
	GHM_SAFE_DELETE_ARRAY(mpLetter      );
	GHM_SAFE_DELETE_ARRAY(mpRatePrim    );
	GHM_SAFE_DELETE      (mpLockOn      );
	GHMGCMEM_SAFE_FREE   (mpTargetPoints);
	GHM_SAFE_DELETE_ARRAY(mpFrameLine   );
	return true;
}

bool WK7SelectItem::CalcPadDirection(void)
{
	// L[
	
	if( !WK7SELECTIF->GetEnable() ){
		
		mPadDirection = -1;
		return false;
	}
	
	f32		x = HSD_PadGetNormalStickX(0);
	f32		y = HSD_PadGetNormalStickY(0);
	f32		len = sqrtf(x * x + y * y);
	
	if( len > 0.8f ){
		
		// AiO|ꂽ
		f32		ang = atan2f(-y,x) + (PI * 9.0f / 8.0f);
		
		mPadDirection  = (int)(ang * 4.0f / PI);
		mPadDirection &= 7;
		
	}else{
		
		mPadDirection = -1;
		
	}
	
	return true;
}

bool WK7SelectItem::SetDirection(int dir)
{
	if( dir < -1 || dir > 7 ){
		return false;
	}
	mDirection = dir;
	return true;
}

#define	WK7SELECTIF_ASPECT			(1.1f)		// 1.1FL[ũhbgc1:0.9l
f32 WK7SelectItem::GetSelectRate(const Vec &dir)
{
	if( !mRender ){
		return -1.0f;
	}
	Vec		cdir;
	cdir.x = mCutInDirection.x;
	cdir.y = mCutInDirection.y * WK7SELECTIF_ASPECT;		// L[ũhbgcl
	cdir.z = mCutInDirection.z;
	VECNormalize(&cdir,&cdir);
	return -VECDotProduct(&dir,&cdir);
}

#define	REVDIST			(10.0f)
#define	SELECT_FADETICK	(600)
bool WK7SelectItem::SetSelected(bool flag)
{
	switch( mState ){
	case	EXPANDED:
	case	EXPAND:
		break;
	default:
		if( flag==true ){
			if( mSelected==false ){
				mColorR.Play(COLOR_RANGE,SELECT_FADETICK);
//				mColorR.Play(mSelectedColor.r,SELECT_FADETICK);
//				mColorG.Play(mSelectedColor.g,SELECT_FADETICK);
//				mColorB.Play(mSelectedColor.b,SELECT_FADETICK);
			}
			mSelected = true;
		}else{
			if( mSelected==true ){
				mColorR.Play(0,SELECT_FADETICK);
//				mColorR.Play(mNormalColor.r,SELECT_FADETICK);
//				mColorG.Play(mNormalColor.g,SELECT_FADETICK);
//				mColorB.Play(mNormalColor.b,SELECT_FADETICK);
			}
			mSelected = false;
		}
		break;
	}
	return true;
}

u32 WK7SelectItem::GetBackAlpha(void)
{
	u32		alpha = WK7SELECTIF->GetBackAlphaValue();
	if( alpha > mAlpha.GetValue() ){
		return mAlpha.GetValue();
	}
	return alpha;
}

bool WK7SelectItem::FrameProcess(void)
{
//	switch( mState ){
//	case	EXPANDED:
//	case	EXPAND:
//		break;
//	default:
//		if( mDirection >= 0 && mDirection==mPadDirection ){
//			if( mSelected==false ){
//				mColorR.Play(mSelectedColor.r,SELECT_FADETICK);
//				mColorG.Play(mSelectedColor.g,SELECT_FADETICK);
//				mColorB.Play(mSelectedColor.b,SELECT_FADETICK);
//			}
//			mSelected = true;
//		}else{
//			if( mSelected==true ){
//				mColorR.Play(mNormalColor.r,SELECT_FADETICK);
//				mColorG.Play(mNormalColor.g,SELECT_FADETICK);
//				mColorB.Play(mNormalColor.b,SELECT_FADETICK);
//			}
//			mSelected = false;
//		}
//		break;
//	}
	
	WK7SelectItemFrameBlur		*pBlur,*pNextBlur;
	for(pBlur=(WK7SelectItemFrameBlur *)mBlurAnchor.GetNext() ; pBlur!=NULL ; pBlur=pNextBlur){
		pNextBlur = pBlur->GetNext();
		if( pBlur->FrameProcess()==false ){
			delete pBlur;
		}
	}
	
	// l
	mAlpha.FrameProcess();
	// g劄
	mExpandRate.FrameProcess();
	// J[l
	mColorR.FrameProcess();
	mColorG.FrameProcess();
	mColorB.FrameProcess();
	// ό`
	mDeformRate.FrameProcess();
	// ̌̕ω
	mLetterDirRate.FrameProcess();
	// u[JE^[
	mBlurCounter.FrameProcess();
	// ^C|ʒu
//	mTypoPosition.FrameProcess();
	
//	u32	col = (mColorR.GetValue()<<24)|(mColorG.GetValue()<<16)|(mColorB.GetValue()<<8);
//	f32		rate,_rate;
	u32		col;
	
//	GXColor		selectedColor = GetSelectedColor();
	GXColor		normalColor   = GetNormalColor();
	
//	 rate = (f32)mColorR.GetValue() / COLOR_RANGE;
//	_rate = 1.0f - rate;
//	col   = ((u32)(normalColor.r * _rate + selectedColor.r * rate) << 24)|
//	        ((u32)(normalColor.g * _rate + selectedColor.g * rate) << 16)|
//	        ((u32)(normalColor.b * _rate + selectedColor.b * rate) <<  8);
//	col   = ((u32)normalColor.r << 24)| ((u32)normalColor.g << 16)| ((u32)normalColor.b << 8);
//	col   = 0xffffffff;
	
	if( GetSelected()==true ){
		col = WK7SELECTIF->GetSelectedFrameColor();
//		col = COLOR_RED;
	}else{
		col = COLOR_WHITE;
	}
//	col = COLOR_WHITE;//0xffffff00;
//	if( GetHighLighted()==true ){
//		col = HIGHLIGHTTED_COLOR;
//	}else{
//		col = 0xffffff00;
//	}
	col |= GetBackAlpha();
	mTypos.mSetRGB(col);
//	mTypos.mSetRGB(0x80ff8000);
//	mTypos.mSetRGB(mCurrentBackColor);
//	mTypos.mSetRGB(col);
//	mTypos.mSetRGBA(col);
	
	switch( mState ){
	case	FADEIN:
		if( mAlpha.GetValue()!=0xff ){
			break;
		}
		mState = CUTFRAME;
		break;
	case	FADEOUT:
		if( mAlpha.GetValue()!=0x00 ){
			break;
		}
//		if( mTypos.mGetFadeEnd()==false && mVisible==false ){
//			break;
//		}
		if( mTypos.mGetFadeEnd()==false ){
			break;
		}
		mState = VANISHED;
	case	VANISHED:
		if( mBlurAnchor.GetNext()!=NULL ){
			break;
		}
		return false;
	case	EXPAND:
		if( mExpandRate.GetValue()!=WK7SELECTITEM_EXPAND_RATE_MAX ){
			break;
		}
		if( mTypos.mGetFadeEnd()==false ){
//		if( mTypos.mGetFadeEnd()==false && mVisible==false ){
			break;
		}
		mState = EXPANDED;
	case	EXPANDED:
		if( mBlurAnchor.GetNext()!=NULL ){
			break;
		}
		return false;
	case	CUTFRAME:
	default:
		break;
	}
	
	bool	result = CalcFramePosition();
	
	switch( mState ){
	case	FADEIN:
	case	CUTFRAME:
		if( !mRender ){
			break;
		}
		if( mBlurCounter.IsAnimating()==true ){
			break;
		}
		if( IS_EQUAL(mPrevScreenTargetCenter.x,mScreenTargetCenter.x)==false ||
		    IS_EQUAL(mPrevScreenTargetCenter.y,mScreenTargetCenter.y)==false ){
			u32		color;
			if( GetSelected()==true ){
//			if( GetSelected()==false ){
//				if( mType==TYPE_LINE ){
//					color = COLOR_BLUE;
//				}else{
					color = WK7SELECTIF->GetSelectedFrameColor();
//					color = COLOR_RED;
//				}
			}else{
				color = COLOR_WHITE;
			}
			ghmGcMemSetAllocName("SIFRBLUR");
			pBlur = new WK7SelectItemFrameBlur(&mBlurAnchor,mFrame,color,mRenderZ+10.0f);
			mPrevScreenTargetCenter = mScreenTargetCenter;
			mBlurCounter.SetValue(0xff);
			mBlurCounter.Play(0x00,WK7_TICK_PER_SEC / 10);
		}
		break;
	}
	
//	if( mpLetter!=NULL ){
//		WK7Letter		*pLetter;
//		int				i;
//		for(pLetter=mpLetter,i=0 ; i<selectItemLetterNum ; i++,pLetter++){
//			pLetter->FrameProcess();
//		}
//	}
	
	return result;
}

bool WK7SelectItem::CalcCenterPosition(void)
{
	
	HSD_CObj	*pCObj;
	
	WK7CAM->SetCurrent();
	
	if( (pCObj = HSD_CObjGetCurrent())!=NULL ){
		HSD_CObjGetViewingMtx(pCObj,mViewMatrix);
	}
	GXGetProjectionv(mProjection);
	GXGetViewportv  (mViewport);
	
	WK7PLAYERCONTROL->GetPosition(mWorldItemCenter);
	WK7SelectItem::mWorldItemCenter.y += tgth;
	
	GXProject(mWorldItemCenter.x,
	          mWorldItemCenter.y,
	          mWorldItemCenter.z,
	          mViewMatrix,
	          mProjection,
	          mViewport,
	          &mScreenItemCenter.x,
	          &mScreenItemCenter.y,
	          &mScreenItemCenter.z);
	
	return true;
};

bool WK7SelectItem::CalcScreenTargetPoint(void)
{
	
static f32	scrr = 30.0f;
	
	Vec		dir;
	
	if( mpBranch!=NULL ){
		
		Vec					targetPoint;
		WK7PathNurbs		*pPath = mpBranch->GetPath();
		f32					param;
		f32					sp,ep;
//		f32					sp,ep,dp,mp;
		f32					dist,totaldist;
		HSD_CObj			*pCObj;
		pCObj = HSD_CObjGetCurrent();
		
static f32	itemdist = 20.0f;
#if 1
static f32	distmax = 20.0f;
static f32	distmin = 10.0f;
static f32	fovmax  = 75.0f;
static f32	fovmin  = 42.0f;
		if( pCObj!=NULL ){
			f32		fov = HSD_CObjGetFov(pCObj);
			if( fov >= fovmax ){
				dist = distmax;
			}else if( fov <= fovmin ){
				dist = distmin;
			}else{
				dist = (fov - fovmin) * (distmax - distmin) / (fovmax - fovmin) + distmin;
			}
		}else{
			dist = distmin;
		}
		sp = pPath->GetStartParam();
		ep = pPath->GetEndParam  ();
		totaldist = pPath->GetTotalDistance();
		
		if( dist <= totaldist ){
			if( mpBranch->GetDir() ){
				param = pPath->CalcParam(totaldist - dist);
			}else{
				param = pPath->CalcParam(dist);
			}
			mpBranch->GetPosition(&targetPoint,param);
		}else{
			mpBranch->GetEndPosition     (&targetPoint);
			mpBranch->GetEndDeltaPosition(&dir);
			if( VECMag(&dir) >= 0.00001f ){
				VECNormalize(&dir,&dir);
			}
			dist -= totaldist;
			VECScale(&dir,&dir,dist);
			VECAdd  (&targetPoint,&dir,&targetPoint);
		}
//		dp = ep - sp;
//		mp = sp + dp * 0.5f;
		
//		if( mpBranch->GetDir() ){
//			param = pPath->CalcParam(pPath->GetTotalDistance() - dist);
//		}else{
//			param = pPath->CalcParam(dist);
//		}
//		
//		if( mpBranch->GetDir() ){
//			if( param < mp ){
//				param = mp;
//			}
//		}else{
//			if( param > mp ){
//				param = mp;
//			}
//		}
//		mpBranch->GetPosition(&targetPoint,param);
#else
		f32					totaldist;
		totaldist = pPath->GetTotalDistance();
		sp = pPath->GetStartParam();
		ep = pPath->GetEndParam  ();
		dp = ep - sp;
		mp = sp + dp * 0.5f;
		
		param = itemdist / totaldist;
		if( mpBranch->GetDir() ){
			param = ep - param * dp;
			if( param < mp ){
				param = mp;
			}
		}else{
			param = sp + param * dp;
			if( param > mp ){
				param = mp;
			}
		}
		
		mpBranch->GetPosition(&targetPoint,param);
		
		dist = GHM_VEC_GET_DIST(&targetPoint,&mWorldItemCenter);
		
		if( dist > itemdist ){
			
			if( mpBranch->GetDir() ){
				param  = ep - param;
				param *= itemdist / dist;
				param  = ep - param;
			}else{
				param *= itemdist / dist;
			}
			
			mpBranch->GetPosition(&targetPoint,param);
			
		}
#endif
		
		targetPoint.y += tgth;
		
		if( pCObj!=NULL ){
			
			MtxPtr		mtxptr = HSD_CObjGetViewingMtxPtr(pCObj);
			Mtx			invview;
			Mtx			trans;
			Vec			viewcenter,viewtgtpos;
			
			MTXTranspose(mtxptr,invview);
			MTXTrans    (trans ,-mtxptr[0][3],-mtxptr[1][3],-mtxptr[2][3]);
			MTXConcat   (invview,trans,invview);
			
//			viewtgtpos.x =  0.0f;
//			viewtgtpos.y =  0.0f;
//			viewtgtpos.z = -1.0f;
//			
//			MTXMultVecSR(invview,&viewtgtpos,&viewtgtpos);
//			
//			if( viewtgtpos.y > 0.0f ){
				
				MTXMultVec  (mtxptr ,&targetPoint     ,&viewtgtpos);
				MTXMultVec  (mtxptr ,&mWorldItemCenter,&viewcenter);
				
	static		f32		scale  = 0.1f;
				f32		deltaz = viewcenter.z - viewtgtpos.z;
				
				viewtgtpos.y = viewcenter.y + deltaz * scale;
				
				MTXMultVec  (invview,&viewtgtpos ,&targetPoint);
				
//			}
			
		}
		
		GXProject(targetPoint.x,
		          targetPoint.y,
		          targetPoint.z,
		          mViewMatrix,
		          mProjection,
		          mViewport,
		          &mScreenTargetCenter.x,
		          &mScreenTargetCenter.y,
		          &mScreenTargetCenter.z);
		
		mScreenTargetRadius = scrr;
		
		mpBranch->GetStartDeltaPosition(&dir);
		
	}else if( mpBranchPointObj!=NULL ){
		
		Vec		pos;
		
		mpBranchPointObj->GetPosition(pos);
		VECSubtract(&pos,&mWorldItemCenter,&dir);
		
		GXProject(pos.x,
		          pos.y,
		          pos.z,
		          mViewMatrix,
		          mProjection,
		          mViewport,
		          &mScreenTargetCenter.x,
		          &mScreenTargetCenter.y,
		          &mScreenTargetCenter.z);
		
		mScreenTargetRadius = scrr;
		
	}else if( mpObj!=NULL ){
		
		Vec		pos;
		
		mpObj->GetPosition(pos);
		
		GXProject(pos.x,
		          pos.y,
		          pos.z,
		          mViewMatrix,
		          mProjection,
		          mViewport,
		          &mScreenTargetCenter.x,
		          &mScreenTargetCenter.y,
		          &mScreenTargetCenter.z);
		
		mScreenTargetRadius = scrr;
		
	}else{
		
		if( mpTargetPoints==NULL ){
			return false;
		}
		
		int					i;
		const Vec			*pTargetPoint;
		Vec					*pScreenTargetPoint;
		f32					invPointNum = 1.0f / mTargetPointNum;
		GHM_VEC_SET_XYZ0(&mScreenTargetCenter);
		pTargetPoint       = mpTargetPoints;
		pScreenTargetPoint = mpScreenTargetPoints;
		for(i=0 ; i<mTargetPointNum ; i++,pTargetPoint++,pScreenTargetPoint++){
			GXProject(pTargetPoint->x,
			          pTargetPoint->y,
			          pTargetPoint->z,
			          mViewMatrix,
			          mProjection,
			          mViewport,
			          &pScreenTargetPoint->x,
			          &pScreenTargetPoint->y,
			          &pScreenTargetPoint->z);
			mScreenTargetCenter.x += pScreenTargetPoint->x;
			mScreenTargetCenter.y += pScreenTargetPoint->y;
			mScreenTargetCenter.z += pScreenTargetPoint->z;
		}
		mScreenTargetCenter.x *= invPointNum;
		mScreenTargetCenter.y *= invPointNum;
		mScreenTargetCenter.z *= invPointNum;
		mScreenTargetRadius    = 0.0f;
		pScreenTargetPoint = mpScreenTargetPoints;
		for(i=0 ; i<mTargetPointNum ; i++,pScreenTargetPoint++){
			f32		dx   = pScreenTargetPoint->x - mScreenTargetCenter.x;
			f32		dy   = pScreenTargetPoint->y - mScreenTargetCenter.y;
			f32		dist = sqrtf(dx * dx + dy * dy);
			if( mScreenTargetRadius < dist ){
				mScreenTargetRadius = dist;
			}
		}
		
	static	f32		radius_margin = 2.0f;
		mScreenTargetRadius += radius_margin;
		
		dir.x = 0.0f;
		dir.y = 0.0f;
		dir.z = 1.0f;
		
	}
	
	// ~bg
	if( mScreenTargetCenter.x < mScreenTargetRadius ){
		mScreenTargetCenter.x = mScreenTargetRadius;
	}else if( mScreenTargetCenter.x > WK7_SCREEN_WIDTH - mScreenTargetRadius ){
		mScreenTargetCenter.x = WK7_SCREEN_WIDTH - mScreenTargetRadius;
	}
	if( mScreenTargetCenter.y < mScreenTargetRadius ){
		mScreenTargetCenter.y = mScreenTargetRadius;
	}else if( mScreenTargetCenter.y > WK7_SCREEN_HEIGHT - mScreenTargetRadius ){
		mScreenTargetCenter.y = WK7_SCREEN_HEIGHT - mScreenTargetRadius;
	}
	
#if 0
#define	DELTAANG		(TWO_PI / 3.0f)
	f32		ang,angmin;
	Vec		rgbmin,rgbmax,rgb;
	ang = atan2f(dir.x,dir.z);
	
	if( ang < -TWO_PI / 3.0f ){
		// ΂
		angmin = -TWO_PI * 2.0f / 3.0f;
		rgbmin.x = 0.0f;
		rgbmin.y = 1.0f;
		rgbmin.z = 0.0f;
		rgbmax.x = 1.0f;
		rgbmax.y = 0.0f;
		rgbmax.z = 0.0f;
	}else if( ang < 0.0f ){
		// Ԃ
		angmin = -TWO_PI / 3.0f;
		rgbmin.x = 1.0f;
		rgbmin.y = 0.0f;
		rgbmin.z = 0.0f;
		rgbmax.x = 0.0f;
		rgbmax.y = 0.0f;
		rgbmax.z = 1.0f;
	}else if( ang < TWO_PI / 3.0f ){
		// 
		angmin =  0.0f;
		rgbmin.x = 0.0f;
		rgbmin.y = 0.0f;
		rgbmin.z = 1.0f;
		rgbmax.x = 0.0f;
		rgbmax.y = 1.0f;
		rgbmax.z = 0.0f;
	}else{
		// ΂
		angmin =  TWO_PI / 3.0f;
		rgbmin.x = 0.0f;
		rgbmin.y = 1.0f;
		rgbmin.z = 0.0f;
		rgbmax.x = 1.0f;
		rgbmax.y = 0.0f;
		rgbmax.z = 0.0f;
	}
	
	ang = (ang - angmin) / DELTAANG;
	VECScale(&rgbmin,&rgbmin,1.0f - ang);
	VECScale(&rgbmax,&rgbmax,       ang);
	VECAdd  (&rgbmax,&rgbmin,&rgb      );
//	if( mHighLighted ){
//		VECScale(&rgb,&rgb,0.5f);
//	}
#endif
	if( mSelected==true ){
//		mCurrentBackColor = ((u32)(rgb.x * 0xff) << 24)|((u32)(rgb.y * 0xff) << 16)|((u32)(rgb.z * 0xff) << 8);
		if( mHighLighted ){
			mCurrentBackColor = 0x0000ff00;
		}else{
			mCurrentBackColor = 0xffffff00;
		}
	}else{
		if( mHighLighted ){
			mCurrentBackColor = 0x00008000;
		}else{
			mCurrentBackColor = 0x80808000;
		}
//		if( mHighLighted ){
//			mCurrentBackColor = 0x80808000;
//		}else{
//			mCurrentBackColor = 0xffffff00;
//		}
	}
	
	return true;
}

static	Vec		ScreenFramePos[4] = {
	{ 0.0f             , 0.0f              , 0.0f },
	{ WK7_SCREEN_WIDTH , 0.0f              , 0.0f },
	{ WK7_SCREEN_WIDTH , WK7_SCREEN_HEIGHT , 0.0f },
	{ 0.0f             , WK7_SCREEN_HEIGHT , 0.0f },
};
bool WK7SelectItem::CalcCutInPosition(void)
{
	
	if( mScreenTargetCenter.z < 0.0f || mScreenTargetCenter.z > 1.0f ){
		// Nbv
		return false;
	}
	
	int			i;
	int			posid;
	f32			ip,ipmin;
	Vec			dir;
	Vec			center;
	
#if 1
	center = mScreenItemCenter;
#else
	if( mpObj!=NULL ){
static	Vec	screen_center = { WK7_SCREEN_WIDTH * 0.5f ,WK7_SCREEN_HEIGHT * 0.5f , 0.0f };
		center = screen_center;
	}else{
		center = mScreenItemCenter;
	}
#endif
	if( center.x!=mScreenTargetCenter.x || center.y!=mScreenTargetCenter.y ){
		VECSubtract (&center,&mScreenTargetCenter,&mCutInDirection);
		mCutInDirection.z = 0.0f;
		VECNormalize(&mCutInDirection,&mCutInDirection);
	}else{
		mCutInDirection.x = 1.0f;
		mCutInDirection.y = 0.0f;
		mCutInDirection.z = 0.0f;
	}
	
	VECSubtract (&ScreenFramePos[0],&center,&dir);
	ipmin = VECDotProduct(&mCutInDirection,&dir);
	posid = 0;
	
	for(i=1 ; i<4 ; i++){
		VECSubtract(&ScreenFramePos[i],&center,&dir);
		ip = VECDotProduct(&mCutInDirection,&dir);
		if( ipmin > ip ){
			ipmin = ip;
			posid = i;
		}
	}
	
	ipmin -= mScreenTargetRadius;
	VECScale(&mCutInDirection,&dir,ipmin);
	VECAdd  (&dir,&center,&mCutInPosition);
	
	VECSubtract(&mScreenTargetCenter,&mCutInPosition,&dir);
	ip = VECDotProduct(&dir,&mCutInDirection);
	
	if( ip < 0.0f ){
		// ʊO
		return false;
	}
	
static f32	HMARGIN = 32.0f;
static u32	PosMoveTime = 2400;
	f32		h   = mTypos.mGetHeight();
	h = h * 0.5f + HMARGIN;
	
//	if( mCutInDirection.y < 0.0f ){
//		if( mTypoPosition.GetDstValue()!=WK7_SCREEN_HEIGHT - h ){
//			mTypoPosition.Play((u32)(WK7_SCREEN_HEIGHT - h),PosMoveTime);
////			mTypos.mSetReverse(
//		}
//	}else{
//		if( mTypoPosition.GetDstValue()!=h ){
//			mTypoPosition.Play((u32)h,PosMoveTime);
////			mTypos.mSetReverse(
//		}
//	}
	
	return true;
}

bool WK7SelectItem::CalcCutFramePosition(void)
{
	
#if 0
	if( mType==TYPE_LINE ){
		switch( mState ){
		case	CUTFRAME:
		case	FADEIN:
			break;
		default:
			return true;
		}
		Vec				dir;
		f32				len;
		VECSubtract(&mScreenTargetCenter,&mCutInPosition,&dir);
		mFrame.mCutFramePosition[0] = mCutInPosition;
		if( (len=VECMag(&dir)) < 0.0001f ){
			mFrame.mCutFramePosition[1] = mScreenTargetCenter;
			mFrame.mCutFramePosition[2] = mScreenTargetCenter;
			mRate = 0.0f;
		}else{
			
			WK7PathInfo	*pPathInfo = WK7PLAYERPATHCONTROL->GetCurrentPathInfo();
//			static	f32	marginDist = 20.0f;
			static	f32	marginDist =  5.0f;
			if( pPathInfo!=NULL ){
				mRate = 1.0f - (pPathInfo->GetRestDistance()-marginDist) / (WK7SELECTIF->GetAppearDist()-marginDist);
				GHM_LIMIT(mRate,1.0f,0.0f);
			}else{
				mRate = 0.0f;
			}
			VECNormalize(&dir,&dir);
			VECScale    (&dir,&mFrame.mCutFramePosition[2],len * mRate);
			VECAdd      (&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[2],&mFrame.mCutFramePosition[2]);
			
			len -= LOCKONCURSOR_BLINK_MAXSIZE + 2.0f;
			if( len < 0.0f ){
				len = 0.0f;
			}
			
			VECScale    (&dir,&mFrame.mCutFramePosition[1],len * mRate);
			VECAdd      (&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[1],&mFrame.mCutFramePosition[1]);
			
		}
		if( len * mRate > 0.0001f ){
			f32		size;
			if( GetSelected()==true ){
				size = 2.50f;
			}else{
				size = 1.25f;
			}
			::CalcParallelFramePosition2(mFrame.mParallelFramePosition,mFrame.mCutFramePosition,2,size);
//			::CalcParallelFramePosition2(mFrame.mParallelFramePosition,mFrame.mCutFramePosition,2,2.5f * 0.5f);
		}
		return true;
	}
#endif
	
	f32		rate,_rate;
	
	// Kill`
	
#if 0
	if( mpLetter!=NULL && GetSelected()==true && (mType!=TYPE_LINE || mRate==1.0f) && mVisible ){
		
		int				i;
		WK7Letter		*pLetter;
		Vec				dir;
		Vec				tdir;
		Vec				mdir;
		Vec				tpos;
	static	f32			margin   =  4.0f;
		f32				rx,ry,sc;
		f32				w,h;
		u32				alpha = GetBackAlpha();
		Vec				pos[2][SELECTITEM_LETTER_NUM];
		u32				tick;
		
		VECSubtract (&mCutInPosition,&mScreenTargetCenter,&dir);
		dir.z = 0.0f;
		VECNormalize(&dir,&dir);
		rx = fabs(dir.x);
		ry = fabs(dir.y);
		VECScale(&dir,&mdir,margin);
		
	static	f32	hsc = 0.5f;
		
		if( mLetterDisp ){
			tick = WK7_TICK_PER_SEC / 2;
		}else{
			tick = 0;
		}
		
		if( dir.x >= 0.0f ){
			if( mLetterDirRate.GetDstValue()!=1.0f ){
				mLetterDirRate.Play(1.0f,tick);
			}
		}else{
			if( mLetterDirRate.GetDstValue()!=0.0f ){
				mLetterDirRate.Play(0.0f,tick);
			}
		}
		
		tpos = mScreenTargetCenter;
		for(i=0,pLetter=mpLetter ; i<SELECTITEM_LETTER_NUM ; i++,pLetter++){
			w = (f32)pLetter->GetW();
			h = (f32)pLetter->GetH();
			sc = h * hsc;
			sc = 0.5f * w * sc / sqrtf(sc * sc * rx * rx + w * w * ry * ry);
			VECScale(&dir,&tdir,sc);
			VECAdd(&tpos,&tdir,&tpos);
			pos[0][i].x = tpos.x - w * 0.5f;
			pos[0][i].y = tpos.y - h * (1.0f - hsc);
			VECAdd(&tpos,&tdir,&tpos);
			VECAdd(&tpos,&mdir,&tpos);
		}
		
		tpos = mScreenTargetCenter;
		for(i=SELECTITEM_LETTER_NUM-1,pLetter=mpLetter+(SELECTITEM_LETTER_NUM-1) ; i>=0 ; i--,pLetter--){
			w = (f32)pLetter->GetW();
			h = (f32)pLetter->GetH();
			sc = h * hsc;
			sc = 0.5f * w * sc / sqrtf(sc * sc * rx * rx + w * w * ry * ry);
			VECScale(&dir,&tdir,sc);
			VECAdd(&tpos,&tdir,&tpos);
			pos[1][i].x = tpos.x - w * 0.5f;
			pos[1][i].y = tpos.y - h * (1.0f - hsc);
			VECAdd(&tpos,&tdir,&tpos);
			VECAdd(&tpos,&mdir,&tpos);
		}
		
		for(i=0,pLetter=mpLetter ; i<SELECTITEM_LETTER_NUM ; i++,pLetter++){
			f32		rate = mLetterDirRate.GetValue();
			VECScale(&pos[0][i],&pos[0][i],       rate);
			VECScale(&pos[1][i],&pos[1][i],1.0f - rate);
			VECAdd  (&pos[1][i],&pos[0][i],&tpos);
			pLetter->SetXY  (tpos.x,tpos.y,true);
			pLetter->SetDisp(true);
			pLetter->SetA   (alpha);
		}
		
		mLetterDisp = 1;
		
	}else{
		
		mLetterDisp = 0;
		
	}
#endif
	
#if 0
	if( mType==TYPE_LINE ){
		
		WK7PathInfo	*pPathInfo = WK7PLAYERPATHCONTROL->GetCurrentPathInfo();
		
		if( pPathInfo!=NULL ){
			mRate = 1.0f - pPathInfo->GetRestDistance() / WK7SELECTIF->GetAppearDist();
			GHM_LIMIT(mRate,1.0f,0.0f);
		}else{
			mRate = 0.0f;
		}
		
		Vec				dir,vdir;
		f32				len;
static	f32				size = 20.0f;
static	f32				dlen = 20.0f;
		
		dir.x = mScreenTargetCenter.x - mCutInPosition.x;
		dir.y = mScreenTargetCenter.y - mCutInPosition.y;
		dir.z = 0.0f;
		len = VECMag(&dir);
		VECNormalize(&dir,&dir);
		vdir.x =  dir.y;
		vdir.y = -dir.x;
		vdir.z = 0.0f;
		VECScale    (&dir,&dir,dlen + len);
		
		VECScale    (&vdir,&vdir,size);
		
		mFocusPos.x = dir.x + mCutInPosition.x;
		mFocusPos.y = dir.y + mCutInPosition.y;
		
		rate = mDeformRate.GetValue() * mRate;
		
//		mFrame.mCutFramePosition[0].x = mFocusPos.x - dv.x * rate;
//		mFrame.mCutFramePosition[0].y = mFocusPos.y - dv.y * rate;
//		
//		mFrame.mCutFramePosition[3].x = mFocusPos.x - dv.x * rate;
//		mFrame.mCutFramePosition[3].y = mFocusPos.y - dv.y * rate;
//		
//		mFrame.mCutFramePosition[1].x = (b1 * ax_by0 - b0 * ax_by1) / a0b1_a1b0;
//		mFrame.mCutFramePosition[1].y = (a0 * ax_by1 - a1 * ax_by0) / a0b1_a1b0;
//		
//		mFrame.mCutFramePosition[2].x = ov.x;
//		mFrame.mCutFramePosition[2].y = ov.y;
		
		VECScale   (&vdir,&mFrame.mCutFramePosition[0],rate);
		VECSubtract(&mCutInPosition,&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[3]);
		VECSubtract(&mFocusPos     ,&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[2]);
		VECAdd     (&mFocusPos     ,&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[1]);
		VECAdd     (&mCutInPosition,&mFrame.mCutFramePosition[0],&mFrame.mCutFramePosition[0]);
		
		return true;
	}
#endif
	
	int				i;
	Vec				tp[2];
	f32				x0,y0;
	f32				x02_y02,x02,y02;
	f32				r2,r4,dx,dy;
	Vec				ov;
	Vec				dv;
	f32				op;
	Vec				cp;
	f32				ip,invlen;
	
	dv.x = mScreenTargetCenter.x - mCutInPosition.x;
	dv.y = mScreenTargetCenter.y - mCutInPosition.y;
	dv.z = sqrtf(dv.x * dv.x + dv.y * dv.y);
	invlen = 1.0f / dv.z;
	dv.x *= invlen;
	dv.y *= invlen;
	dv.z *= 1.2f;
	dv.z += mScreenTargetRadius;
	mFocusPos.x = dv.z * dv.x + mCutInPosition.x;
	mFocusPos.y = dv.z * dv.y + mCutInPosition.y;
	
	x0      = mFocusPos.x - mScreenTargetCenter.x;
	y0      = mFocusPos.y - mScreenTargetCenter.y;
	x02     = x0 * x0;
	y02     = y0 * y0;
	x02_y02 = x02 + y02;
	r2      = mScreenTargetRadius * mScreenTargetRadius;
	r4      = r2 * r2;
	dx      = x02 * r4 - x02_y02 * (r4 - y02 * r2);
	dy      = y02 * r4 - x02_y02 * (r4 - x02 * r2);
	if( dx < 0.0f ){ dx = 0.0f; }
	if( dy < 0.0f ){ dy = 0.0f; }
	dx      = sqrtf(dx);
	dy      = sqrtf(dy);
//	dx      = sqrtf(x02 * r4 - x02_y02 * (r4 - y02 * r2));
//	dy      = sqrtf(y02 * r4 - x02_y02 * (r4 - x02 * r2));
	
	tp[0].x = (x0 * r2 + dx) / x02_y02;
	tp[0].y = (y0 * r2 + dy) / x02_y02;
	
	tp[1].x = (x0 * r2 - dx) / x02_y02;
	tp[1].y = (y0 * r2 - dy) / x02_y02;
	
	for(i=0 ; i<2 ; i++){
		tp[i].x += mScreenTargetCenter.x;
		tp[i].y += mScreenTargetCenter.y;
	}
	
	dv.x = tp[0].x - mScreenTargetCenter.x;
	dv.y = tp[0].y - mScreenTargetCenter.y;
	op   = sqrtf(dv.x * dv.x + dv.y * dv.y);
	if( fabs(op - mScreenTargetRadius) > 0.001f ){
		dv.y    = tp[0].y;
		tp[0].y = tp[1].y;
		tp[1].y = dv.y;
	}
	
	// ~ƃ^[Qbg|CgƂ̑Έʒuɓɂ
	
	dv.x = tp[0].x - mScreenTargetCenter.x;
	dv.y = tp[0].y - mScreenTargetCenter.y;
	op   = dv.x * y0 - dv.y * x0;
	if( op < 0.0f ){
		dv   .x = tp[0].x;
		dv   .y = tp[0].y;
		tp[0].x = tp[1].x;
		tp[0].y = tp[1].y;
		tp[1].x = dv   .x;
		tp[1].y = dv   .y;
	}
	
	dv.x = mFocusPos.x - tp[0].x;
	dv.y = mFocusPos.y - tp[0].y;
	ov.x = mFocusPos.x - mCutInPosition.x;
	ov.y = mFocusPos.y - mCutInPosition.y;
	
	rate = (mCutInDirection.x * ov.x + mCutInDirection.y * ov.y)
	     / (mCutInDirection.x * dv.x + mCutInDirection.y * dv.y);
//	rate *= 1.01f;
	
	mFrame.mCutFramePosition[0].x = mFocusPos.x - dv.x * rate;
	mFrame.mCutFramePosition[0].y = mFocusPos.y - dv.y * rate;
	
	dv.x = mFocusPos.x - tp[1].x;
	dv.y = mFocusPos.y - tp[1].y;
	ov.x = mFocusPos.x - mCutInPosition.x;
	ov.y = mFocusPos.y - mCutInPosition.y;
	
	rate = (mCutInDirection.x * ov.x + mCutInDirection.y * ov.y)
	     / (mCutInDirection.x * dv.x + mCutInDirection.y * dv.y);
	
	mFrame.mCutFramePosition[3].x = mFocusPos.x - dv.x * rate;
	mFrame.mCutFramePosition[3].y = mFocusPos.y - dv.y * rate;
	
	ov.x    = tp[1].x + dv.x * mCutRate;
	ov.y    = tp[1].y + dv.y * mCutRate;
	
	dv.x    = mScreenTargetCenter.x - ov.x;
	dv.y    = mScreenTargetCenter.y - ov.y;
	invlen  = 1.0f / sqrtf(dv.x * dv.x + dv.y * dv.y);
	dv.x   *= invlen;
	dv.y   *= invlen;
	cp.x    = tp[1].x - ov.x;
	cp.y    = tp[1].y - ov.y;
	ip      = 2.0f * (cp.x * dv.x + cp.y * dv.y);
	dv.x    = dv.x * ip - cp.x;
	dv.y    = dv.y * ip - cp.y;
	cp.x    = ov.x + dv.x;
	cp.y    = ov.y + dv.y;
	
	f32			a0,b0;
	f32			a1,b1;
	f32			ax_by0,ax_by1;
	f32			a0b1_a1b0;
	
	a0 =   cp.y - ov.y;
	b0 = -(cp.x - ov.x);
	a1 =  (mFocusPos.y - tp[0].y);
	b1 = -(mFocusPos.x - tp[0].x);
	ax_by0 = a0 * cp       .x + b0 * cp       .y;
	ax_by1 = a1 * mFocusPos.x + b1 * mFocusPos.y;
	a0b1_a1b0 = a0 * b1 - a1 * b0;
	
	mFrame.mCutFramePosition[4].x = (b1 * ax_by0 - b0 * ax_by1) / a0b1_a1b0;
	mFrame.mCutFramePosition[4].y = (a0 * ax_by1 - a1 * ax_by0) / a0b1_a1b0;
	
	mFrame.mCutFramePosition[5].x = ov.x;
	mFrame.mCutFramePosition[5].y = ov.y;
	
	rate  = mDeformRate.GetValue();
	_rate = 1.0f - rate;
	
	for(i=0 ; i<2 ; i++){
		mFrame.mCutFramePosition[i+1].x = mFocusPos.x * _rate + mFrame.mCutFramePosition[i+4].x * rate;
		mFrame.mCutFramePosition[i+1].y = mFocusPos.y * _rate + mFrame.mCutFramePosition[i+4].y * rate;
	}
	
//	mFrame.mCutFramePosition[1]   = mFrame.mCutFramePosition[4];
//	mFrame.mCutFramePosition[2]   = mFrame.mCutFramePosition[5];
	
	return true;
}

bool WK7SelectItem::CalcExpandPosition(void)
{
	
	switch( mState ){
	case	EXPAND:
	case	EXPANDED:
		break;
	default:
		memcpy(mFrame.mCurrentFramePosition,
		       mFrame.mCutFramePosition,
		       sizeof(mFrame.mCutFramePosition));
		return true;
	}
	
	// XN[̊Oډ~擾
	f32		radius = sqrtf(WK7_SCREEN_WIDTH * WK7_SCREEN_WIDTH + WK7_SCREEN_HEIGHT * WK7_SCREEN_HEIGHT);
	f32		rate,invrate;
	Vec		dir;
	int		i;
	
	dir.x = radius * mCutInDirection.x;
	dir.y = radius * mCutInDirection.y;
	
	mFrame.mExpandFramePosition[0].x = WK7_SCREEN_CENTER_X - dir.x + dir.y;
	mFrame.mExpandFramePosition[0].y = WK7_SCREEN_CENTER_Y - dir.y - dir.x;
	mFrame.mExpandFramePosition[1].x = WK7_SCREEN_CENTER_X + dir.x + dir.y;
	mFrame.mExpandFramePosition[1].y = WK7_SCREEN_CENTER_Y + dir.y - dir.x;
	mFrame.mExpandFramePosition[2].x = WK7_SCREEN_CENTER_X + dir.x - dir.y;
	mFrame.mExpandFramePosition[2].y = WK7_SCREEN_CENTER_Y + dir.y + dir.x;
	mFrame.mExpandFramePosition[3].x = WK7_SCREEN_CENTER_X - dir.x - dir.y;
	mFrame.mExpandFramePosition[3].y = WK7_SCREEN_CENTER_Y - dir.y + dir.x;
	
	rate    = (f32)mExpandRate.GetValue() / WK7SELECTITEM_EXPAND_RATE_MAX;
	invrate = 1.0f - rate;
	for(i=0 ; i<4 ; i++){
		mFrame.mCurrentFramePosition[i].x = mFrame.mCutFramePosition[i].x * invrate + mFrame.mExpandFramePosition[i].x * rate;
		mFrame.mCurrentFramePosition[i].y = mFrame.mCutFramePosition[i].y * invrate + mFrame.mExpandFramePosition[i].y * rate;
	}
	
	return true;
}

#define	RANDF_VALUE(max,min)		(HSD_Randf() * ((max)-(min)) + (min))
#define	RAND_VALUE(max,min)			(HSD_Rand() % ((max)-(min)) + (min))
static	f32	DELTA_MAX			=	( 10.0f);
static	f32	DELTA_MIN			=	(-10.0f);
#define	TICK_MAX					(4800)
#define	TICK_MIN					(2400)

WK7SelectItemFrameEx::WK7SelectItemFrameEx()
{
	int		i;
	for(i=0 ; i<4 ; i++){
		mDeltaX[i].SetValue(RANDF_VALUE(DELTA_MAX,DELTA_MIN));
		mDeltaY[i].SetValue(RANDF_VALUE(DELTA_MAX,DELTA_MIN));
	}
}

bool WK7SelectItem::CalcFrameExPosition(WK7SelectItemFrameEx *pFrameEx)
{
	// ʏ펞
	int			i;
	
	for(i=0 ; i<4 ; i++){
		pFrameEx->mDeltaX[i].FrameProcess();
		pFrameEx->mDeltaY[i].FrameProcess();
//		if( pFrameEx->mDeltaX[i].IsAnimating()==false ){
//			pFrameEx->mDeltaX[i].Play(RANDF_VALUE(DELTA_MAX,DELTA_MIN),(u32)RAND_VALUE(TICK_MAX,TICK_MIN));
//		}
//		if( pFrameEx->mDeltaY[i].IsAnimating()==false ){
//			pFrameEx->mDeltaY[i].Play(RANDF_VALUE(DELTA_MAX,DELTA_MIN),(u32)RAND_VALUE(TICK_MAX,TICK_MIN));
//		}
		pFrameEx->mCutFramePosition[i].x = mFrame.mCutFramePosition[i].x + pFrameEx->mDeltaX[i].GetValue();
		pFrameEx->mCutFramePosition[i].y = mFrame.mCutFramePosition[i].y + pFrameEx->mDeltaY[i].GetValue();
	}
	
	switch( mState ){
	case	EXPAND:
	case	EXPANDED:
		break;
	default:
		memcpy(pFrameEx->mCurrentFramePosition,
		       pFrameEx->mCutFramePosition,
		       sizeof(pFrameEx->mCutFramePosition));
		return true;
	}
	
	// gWvZ
	
	f32		rate,invrate;
	rate    = (f32)mExpandRate.GetValue() / WK7SELECTITEM_EXPAND_RATE_MAX;
	invrate = 1.0f - rate;
	
	for(i=0 ; i<4 ; i++){
		pFrameEx->mExpandFramePosition [i].x = mFrame.mExpandFramePosition[i].x + pFrameEx->mDeltaX[i].GetValue();
		pFrameEx->mExpandFramePosition [i].y = mFrame.mExpandFramePosition[i].y + pFrameEx->mDeltaY[i].GetValue();
		pFrameEx->mCurrentFramePosition[i].x = pFrameEx->mCutFramePosition[i].x * invrate + pFrameEx->mExpandFramePosition[i].x * rate;
		pFrameEx->mCurrentFramePosition[i].y = pFrameEx->mCutFramePosition[i].y * invrate + pFrameEx->mExpandFramePosition[i].y * rate;
	}
	
	return true;
}

bool WK7SelectItem::CalcFramePosition(void)
{
	
	mRender = 0;
	
//	if( mpLetter!=NULL ){
//		WK7Letter		*pLetter;
//		int				i;
//		for(i=0,pLetter=mpLetter ; i<selectItemLetterNum ; i++,pLetter++){
//			pLetter->SetDisp(false);
//		}
//	}
	
	// ACẽXN[^[QbgWZo
	if( CalcScreenTargetPoint()==false ){
		mTargetCenterSet = 0;
		return true;
	}
	
	// JbgC|WVZoiNbv肠j
	if( CalcCutInPosition()==false ){
		mTargetCenterSet = 0;
		return true;
	}
	
	// ^C|\Wݒ
	if( !mTargetCenterSet ){
		mTargetCenterSet      = 1;
		mTypoCenterCurrent    = 
		mTypoCenter           = mScreenTargetCenter;
//		mTypoCenter.y         = mTypoPosition.SetValue(mTypoPosition.GetValue());
//		mTypos.mSetReverse(
		mTypoCenterVelocity.x = 0.0f;
		mTypoCenterVelocity.y = 0.0f;
		mTypoCenterVelocity.z = 0.0f;
		mTypoCenterAccel   .x = 0.0f;
		mTypoCenterAccel   .y = 0.0f;
		mTypoCenterAccel   .z = 0.0f;
	}
	
	// ^C|\Wݒ
//	mTypos.mSetXYCenter((s16)mScreenTargetCenter.x,(s16)mScreenTargetCenter.y);
	
	mRender = 1;
	
	// t[|WVZo
	CalcCutFramePosition();
	
//	if( mpFrameLine!=NULL ){
//		int		i;
//		for(i=0 ; i<3 ; i++){
//			mpFrameLine[i].FrameProcess();
//		}
//	}
	
	// g|WVZo
	CalcExpandPosition();
	
	f32				size;
	
//	if( GetHighLighted()==false ){
	if( GetSelected()==false ){
		size = size_min;
	}else{
		size = size_max;
	}
	CalcParallelFramePosition(size);
	
	if( mpFrameLine!=NULL ){
		int		i;
		for(i=0 ; i<3 ; i++){
			mpFrameLine[i].FrameProcess();
		}
	}
	
//	CalcParallelFramePosition(WK7SELECTIF->GetLineSize());
	
//	CalcFrameExPosition(mFrameEx + 0);
//	CalcFrameExPosition(mFrameEx + 1);
	
#if 0
	switch( mType ){
	case	TYPE_LINE:{
//		if( mpLockOn == NULL ){
//			if( mRate >= 1.0f ){
//				mpLockOn = new WK7SelectItemLockOn;
//			}
//		}
//		if( mpLockOn != NULL ){
//			mpLockOn->SetPosition(mScreenTargetCenter.x,mScreenTargetCenter.y);
//			mpLockOn->SetZ       (1.0f);
//			if( mpLockOn->FrameProcess()==false ){
//				delete mpLockOn;
//				mpLockOn = NULL;
//			}
//		}
//		if( mpRatePrim != NULL ){
//			int				i;
//			f32				x,y;
//			stFontParam		*pFontParamTop = GetFontParam('0',FONTTYPE_ARIALBLACK);
//			x = mFrame.mCutFramePosition[2].x;
//			y = mFrame.mCutFramePosition[2].y;
//			mpRatePrim[3].mSetDispFlag(true);
//			if( pFontParamTop!=NULL ){
//				u32				percent;
//				u32				num;
//				f32				w;
//				f32				h;
//				stFontParam		*pFontParam[4];
//				percent = (u32)(mRate * 100);
//				w       = 0.0f;
//				h       = 0.0f;
//				for(i=2 ; i>=0 ; i--){
//					num           = percent % 10;
//					percent       = (percent - num) / 10;
//					pFontParam[i] = pFontParamTop + num;
//					if( !percent && !num ){
//						mpRatePrim[i].mSetDispFlag(false);
//					}else{
//						mpRatePrim[i].mSetWH ((s16)pFontParam[i]->w,(s16)pFontParam[i]->h);
//						mpRatePrim[i].mSetUV0((s16)pFontParam[i]->u,(s16)pFontParam[i]->v);
//						mpRatePrim[i].mSetDispFlag(true);
//						mpRatePrim[i].mSetScale(eScaleWH,RATEPRIM_SCALE,1);
//						mpRatePrim[i].mScaleProc();
//					}
//					w += (f32)pFontParam[i]->w * RATEPRIM_SCALE;
//					if( h < pFontParam[i]->h * RATEPRIM_SCALE ){
//						h = pFontParam[i]->h * RATEPRIM_SCALE;
//					}
//				}
//				if( (pFontParam[3] = GetFontParam('%',FONTTYPE_ARIALBLACK))!=NULL ){
//					w += (f32)pFontParam[3]->w * RATEPRIM_SCALE;
//					if( h < pFontParam[3]->h * RATEPRIM_SCALE ){
//						h = pFontParam[3]->h * RATEPRIM_SCALE;
//					}
//				}
//				x -= 0.5f * w;
//				y -= 0.5f * h;
//				for(i=0 ; i<RATEPRIM_NUM ; i++){
//					mpRatePrim[i].mSetXY0(x,y);
//					x += (f32)pFontParam[i]->w * RATEPRIM_SCALE;
//				}
//			}
//		}
		}break;
	default:
		break;
	}
#endif
	
	return true;
}

bool WK7SelectItem::CalcTypoOverlap(WK7SelectItem *pItem)
{
	int			x0,y0,w0,h0;
	int			x1,y1,w1,h1;
	int			dx,dy,w,h;
	Vec			dpos;
	
static int	WMARGIN = 16;
static int	HMARGIN = 8;
	x0 = (int)       mTypoCenter.x;
	y0 = (int)       mTypoCenter.y;
	x1 = (int)pItem->mTypoCenter.x;
	y1 = (int)pItem->mTypoCenter.y;
	w0 =             mTypos.mGetWidth () + WMARGIN;
	h0 =             mTypos.mGetHeight() + HMARGIN;
	w1 =      pItem->mTypos.mGetWidth () + WMARGIN;
	h1 =      pItem->mTypos.mGetHeight() + HMARGIN;
	
	dx = x0 - x1; dx = GHM_ABS(dx);
	dy = y0 - y1; dy = GHM_ABS(dy);
	w  = (w0 + w1) >> 1;
	h  = (h0 + h1) >> 1;
	
	w -= dx;
	h -= dy;
	
	if( w < 0 ){
		return false;
	}
	if( h < 0 ){
		return false;
	}
	
static f32		rate = 0.005f;
//static f32		rate = 0.01f;
//static f32		rate = 0.5f;
	f32			delta = w * h * rate;
	
	VECSubtract(&mTypoCenter,&pItem->mTypoCenter,&dpos);
	if( dpos.x!=0.0f || dpos.y!=0.0f || dpos.z!=0.0f ){
		VECNormalize(&dpos,&dpos);
	}
	VECScale   (&dpos,&dpos,delta);
	VECAdd     (&       mTypoCenterVelocity,&dpos,&       mTypoCenterVelocity);
	VECSubtract(&pItem->mTypoCenterVelocity,&dpos,&pItem->mTypoCenterVelocity);
//	VECAdd     (&       mTypoCenterAccel,&dpos,&       mTypoCenterAccel);
//	VECSubtract(&pItem->mTypoCenterAccel,&dpos,&pItem->mTypoCenterAccel);
	
	return true;
}

bool WK7SelectItem::CalcTypoOverlap(void)
{
	WK7SelectItem	*pItem;
	
	for(pItem=GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem==this ){
			continue;
		}
		CalcTypoOverlap(pItem);
	}
	return true;
}

bool WK7SelectItem::CalcTypoPosition(void)
{
	if( !mRender ){
//		mResetTypoPos = 1;
		return true;
	}
	
static f32		rate = 0.2f;
//static f32		rate = 0.5f;
	Vec		dpos;
	Vec		pos;
	
static	Vec		screen_dir[4] = {
	{ -1.0f,  0.0f, 0.0f },
	{  1.0f,  0.0f, 0.0f },
	{  0.0f, -1.0f, 0.0f },
	{  0.0f,  1.0f, 0.0f },
};
static f32	WMARGIN = 32.0f;
static f32	HMARGIN = 32.0f;
static f32	ZEROERR = 0.0000001f;
	int		i,id = -1;
	f32		min = 10000.0f,ip,l;
	f32		w   = mTypos.mGetWidth ();
	f32		h   = mTypos.mGetHeight();
	w = w * 0.5f + WMARGIN;
	h = h * 0.5f + HMARGIN;
	VECScale(&mCutInDirection,&dpos,-1.0f);
	dpos.z = 0.0f;
	VECNormalize(&dpos,&dpos);
	for(i=0 ; i<4 ; i++){
		ip = VECDotProduct(&dpos,screen_dir+i);
		if( ip < ZEROERR ){
			continue;
		}
		switch( i ){
		case	0: l = mScreenTargetCenter.x - w; break;
		case	1: l = WK7_SCREEN_WIDTH  - w - mScreenTargetCenter.x; break;
		case	2: l = mScreenTargetCenter.y - h; break;
		case	3: l = WK7_SCREEN_HEIGHT - h - mScreenTargetCenter.y; break;
		}
		ip = l / ip;
		if( min > ip ){
			min = ip;
			id  = i;
		}
	}
	
	pos = mScreenTargetCenter;
	if( id >= 0 ){
		// 
		VECScale(&dpos,&dpos,min);
		VECAdd  (&pos,&dpos,&pos);
	}
	
	if( pos.x < w ){
		pos.x = w;
	}else if( pos.x > WK7_SCREEN_WIDTH  - w ){
		pos.x = WK7_SCREEN_WIDTH  - w;
	}
	if( pos.y < h ){
		pos.y = h;
	}else if( pos.y > WK7_SCREEN_HEIGHT - h ){
		pos.y = WK7_SCREEN_HEIGHT - h;
	}
	
	VECSubtract(&pos,&mTypoCenter,&dpos);
//	VECSubtract(&mScreenTargetCenter,&mTypoCenter,&dpos);
	VECScale   (&dpos,&dpos,rate);
	VECAdd     (&mTypoCenterVelocity,&dpos,&mTypoCenterVelocity);
	
	mTypoCenterVelocity.x += mTypoCenterAccel   .x;
	mTypoCenterVelocity.y += mTypoCenterAccel   .y;
	mTypoCenterVelocity.z += mTypoCenterAccel   .z;
	mTypoCenter        .x += mTypoCenterVelocity.x;
	mTypoCenter        .y += mTypoCenterVelocity.y;
	mTypoCenter        .z += mTypoCenterVelocity.z;
	
//	mTypoCenter.y          = mTypoPosition.GetValue();
	
	mTypoCenterVelocity.x  = 0.0f;
	mTypoCenterVelocity.y  = 0.0f;
	mTypoCenterVelocity.z  = 0.0f;
	
//	if( !mResetTypoPos ){
//		if( GHM_VEC_GET_DIST(&mTypoCenterCurrent,&mTypoCenter) > 1.5f ){
//	static	f32		delay = 0.25f;
//			f32		_delay = 1.0f - delay;
//			mTypoCenterCurrent.x = mTypoCenter.x * delay + mTypoCenterCurrent.x * _delay;
//			mTypoCenterCurrent.y = mTypoCenter.y * delay + mTypoCenterCurrent.y * _delay;
//			mTypoCenterCurrent.z = mTypoCenter.z * delay + mTypoCenterCurrent.z * _delay;
//		}
//	}else{
////		mResetTypoPos = 0;
		mTypoCenter        = pos;
		mTypoCenterCurrent = pos;
//	}
	
	switch( mState ){
	case	CUTFRAME:
	case	FADEIN:
		mTypos.mSetXYCenter((s16)mTypoCenterCurrent.x,(s16)mTypoCenterCurrent.y);
		break;
	case	FADEOUT:
	case	VANISHED:
	case	EXPAND:
	case	EXPANDED:
	default:
		break;
	}
//	mTypos.mSetXYCenter((s16)mTypoCenterCurrent.x,(s16)mTypoCenterCurrent.y);
//	mTypos.mSetXYCenter((s16)mTypoCenter.x,(s16)mTypoCenter.y);
	
	return true;
}

void RenderRect(Vec &cur,Vec &curp,Vec &next,Vec &nextp,f32 rate0,f32 rate1,u32 ccol,u32 ncol,f32 z,f32 size);
void RenderRect(Vec &cur,Vec &curp,Vec &next,Vec &nextp,f32 rate0,f32 rate1,u32 ccol,u32 ncol,f32 z,f32 size)
{
	Vec		pos0,posp0;
	Vec		pos1,posp1;
	Vec		hdir0,hdir1;
	
	VECSubtract (&next ,&cur  ,&pos0 );
	VECScale    (&pos0 ,&pos1 ,rate1 );
	VECAdd      (&pos1 ,&cur  ,&pos1 );
	VECScale    (&pos0 ,&pos0 ,rate0 );
	VECAdd      (&pos0 ,&cur  ,&pos0 );
	
	VECSubtract (&nextp,&curp ,&posp0);
	VECScale    (&posp0,&posp1,rate1 );
	VECAdd      (&posp1,&curp ,&posp1);
	VECScale    (&posp0,&posp0,rate0 );
	VECAdd      (&posp0,&curp ,&posp0);
	
	VECSubtract (&posp0,&pos0 ,&hdir0);
	VECNormalize(&hdir0,&hdir0);
	VECScale    (&hdir0,&hdir0,size  );
	
	VECSubtract (&posp1,&pos1 ,&hdir1);
	VECNormalize(&hdir1,&hdir1);
	VECScale    (&hdir1,&hdir1,size  );
	
	VECAdd      (&pos0 ,&hdir0,&posp0);
	VECSubtract (&pos0 ,&hdir0,&pos0 );
	VECAdd      (&pos1 ,&hdir1,&posp1);
	VECSubtract (&pos1 ,&hdir1,&pos1 );
	
	GXPosition3f32(pos0 .x,pos0 .y,z);
	GXColor1u32   (ccol);
	GXPosition3f32(posp0.x,posp0.y,z);
	GXColor1u32   (ncol);
	
	GXPosition3f32(pos1 .x,pos1 .y,z);
	GXColor1u32   (ccol);
	GXPosition3f32(posp1.x,posp1.y,z);
	GXColor1u32   (ncol);
	
}

bool WK7SelectItem::RenderTargetCircle(void)
{
	if( mTargetCircleVisible==false ){
		return true;
	}
#define	CIRCLE_DIVNUM		(32)
#if 0
	
	GXBegin(GX_LINESTRIP,GX_VTXFMT0,CIRCLE_DIVNUM+1);
		
		u32		col = 0x00ff0040;
		
		for(i=0 ; i<CIRCLE_DIVNUM ; i++){
			
			f32		ang = -PI + (TWO_PI / CIRCLE_DIVNUM) * i;
			f32		x = mScreenTargetCenter.x + mScreenTargetRadius * cosf(ang);
			f32		y = mScreenTargetCenter.y + mScreenTargetRadius * sinf(ang);
			
			GXPosition3f32(x,y,mRenderZ);
			GXColor1u32   (col);
			
		}
		
		GXPosition3f32(mScreenTargetCenter.x - mScreenTargetRadius,mScreenTargetCenter.y,mRenderZ + 20.0f);
		GXColor1u32   (col);
		
	GXEnd();
	
	GXBegin(GX_LINESTRIP,GX_VTXFMT0,5);
		
		f32		x0,y0,x1,y1;
		
		col = 0xffff00ff;
		x0 = mTypoCenter.x - mTypos.mGetWidth () / 2.0f;
		y0 = mTypoCenter.y - mTypos.mGetHeight() / 2.0f;
//			x0 = (u32)mScreenTargetCenter.x - mTypos.mGetWidth () / 2;
//			y0 = (u32)mScreenTargetCenter.y - mTypos.mGetHeight() / 2;
		x1 = x0 + mTypos.mGetWidth ();
		y1 = y0 + mTypos.mGetHeight();
		
		GXPosition3f32(x0,y0,mRenderZ);
		GXColor1u32   (col);
		GXPosition3f32(x0,y1,mRenderZ);
		GXColor1u32   (col);
		GXPosition3f32(x1,y1,mRenderZ);
		GXColor1u32   (col);
		GXPosition3f32(x1,y0,mRenderZ);
		GXColor1u32   (col);
		GXPosition3f32(x0,y0,mRenderZ);
		GXColor1u32   (col);
		
	GXEnd();
	
#endif
	return true;
}

bool WK7SelectItem::RenderBack(void)
{
	int			i;
	u32			color;
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_DIRECT );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_TEX0 , GX_TEX_ST   , GX_F32   , 0);
	
	GXSetNumTexGens  (1);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	
	GXTexMapID		texmapid;
//static	GXColor			deltacolor = { 0x40,0x40,0x40,0x00 };
	GXColor			dcolor;
	texmapid = GX_TEXMAP1;
	if( GetSelected()==true ){
//		texmapid = GX_TEXMAP1;
		*((u32 *)&dcolor) = WK7SELECTIF->GetSelectBackDeltaColor();
//		dcolor = deltacolor;
	}else{
//		texmapid = GX_TEXMAP0;
		if( GetHighLighted()==true ){
			if( GetSemiHighLighted()==true ){
				*((u32 *)&dcolor) = WK7SELECTIF->GetSemiHighLightedBackDeltaColor();
			}else{
				*((u32 *)&dcolor) = WK7SELECTIF->GetHighLightedBackDeltaColor();
			}
		}else{
			texmapid = GX_TEXMAP0;
			dcolor.r = 
			dcolor.g = 
			dcolor.b = 
			dcolor.a = 0;
		}
//		dcolor.r = 
//		dcolor.g = 
//		dcolor.b = 
//		dcolor.a = 0;
	}
	GXSetTevColor    (GX_TEVREG0,dcolor);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,texmapid,GX_COLOR0A0);
//	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_C0);
//	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
//	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetZMode        (GX_TRUE,GX_GREATER,GX_TRUE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXSetDither       (GX_TRUE);
	
	GXBegin(GX_TRIANGLEFAN,GX_VTXFMT0,4);
	
	if( GetSelected()==true ){
		color = WK7SELECTIF->GetSelectBackColor();
	}else{
		if( GetHighLighted()==true ){
			if( GetSemiHighLighted()==true ){
				color = WK7SELECTIF->GetSemiHighLightedBackColor();
			}else{
				color = WK7SELECTIF->GetHighLightedBackColor();
			}
		}else{
			color = 0xffffff00;
		}
	}
//	if( GetHighLighted()==true && GetSelected()==false ){
//		if( GetSemiHighLighted()==true ){
//			color = WK7SELECTIF->GetSemiHighLightedBackColor();
//		}else{
//			color = WK7SELECTIF->GetHighLightedBackColor();
//		}
////		color = 0xff000000;
////		color = 0x80808000;
//	}else{
////		color = 0xffffff00;
//		color = WK7SELECTIF->GetSelectBackColor();
//	}
	color |= GetBackAlpha();
	
	for(i=0 ; i<4 ; i++){
		GXPosition3f32(mFrame.mCurrentFramePosition[i].x,mFrame.mCurrentFramePosition[i].y,mRenderZ);
		GXColor1u32   (color);
//		GXColor1u32   (mCurrentMaskColor);
		GXTexCoord2f32(mFrame.mCurrentFramePosition[i].x / 640.0f,mFrame.mCurrentFramePosition[i].y / 480.0f);
	}
	
	GXEnd();
	
	if( GetSelected()==true ){
		
static	GXColor			cmpcolor = { 0x00,0x00,0x00,0x00 };
	
		cmpcolor.a = WK7RenderObj::ConvRenderIDToAlpha(WK7RENDEROBJ_RENDERID_PLAYER);
		GXSetTevColor    (GX_TEVREG0,cmpcolor);
		
		GXSetNumTevStages(1);
		GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
		
		GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_RASC,GX_CC_TEXC,GX_CC_ZERO);
		GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_TEXA,GX_CA_A0,GX_CA_RASA,GX_CA_ZERO);
		GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_COMP_A8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
		GXBegin(GX_TRIANGLEFAN,GX_VTXFMT0,4);
			
		for(i=0 ; i<4 ; i++){
			GXPosition3f32(mFrame.mCurrentFramePosition[i].x,mFrame.mCurrentFramePosition[i].y,mRenderZ);
			GXColor1u32   (color);
			GXTexCoord2f32(mFrame.mCurrentFramePosition[i].x / 640.0f,mFrame.mCurrentFramePosition[i].y / 480.0f);
		}
		
		GXEnd();
	}
	
	return true;
}

GXTexMapID     SetupContrastTev_texmapID = GX_TEXMAP1;
bool WK7SelectItem::RenderBackColor(void)
{
	
	u32			backcolor;
	f32			scale = WK7SELECTIF->GetBackScale();
	f32			u,v;
	int			i;
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	
	GXSetDstAlpha     (GX_TRUE,(u8)(0xff-GetBackAlpha()));
	
	GXSetZMode        (GX_TRUE,GX_GREATER,GX_TRUE);
//	GXSetZMode        (GX_TRUE,GX_ALWAYS,GX_TRUE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_TRUE);
	
	GXSetNumTexGens   (1);
	GXSetTexCoordGen  (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	if( GetSelected()==true ){
		
		GXClearVtxDesc    ();
		GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_CLR0,GX_DIRECT);
		
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_CLR0,GX_CLR_RGBA,GX_RGBA8,0);
		
		GXSetNumTevStages (1);
		
		GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
		GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
		GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
		
		backcolor = COLOR_WHITE;
//		if( mType==TYPE_LINE ){
//			backcolor = COLOR_BLUE;
//		}else{
//			backcolor = COLOR_RED;
//		}
		backcolor |= GetBackAlpha();
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4);
			
			for(i=0 ; i<4 ; i++){
				GXPosition3f32(mFrame.mCurrentFramePosition[i].x,mFrame.mCurrentFramePosition[i].y,mRenderZ);
				GXColor1u32   (backcolor);
			}
			
		GXEnd();
		
	}else{
		
		GXClearVtxDesc    ();
		GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_NRM ,GX_NONE  );
		GXSetVtxDesc      (GX_VA_CLR0,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_TEX0,GX_DIRECT);
		
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_CLR0,GX_CLR_RGBA,GX_RGBA8,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
		
//		if( mHighLighted ){
//			SetupContrastTev_texmapID = GX_TEXMAP0;
//		}else{
			SetupContrastTev_texmapID = GX_TEXMAP1;
//		}
		
		WK7SelectIF::SetupContrastTev();
		SetupContrastTev_texmapID = GX_TEXMAP1;
//		backcolor = (mCurrentBackColor|GetBackAlpha());
//		backcolor = (COLOR_WHITE|GetBackAlpha());
//		backcolor = (0xff000000|GetBackAlpha());
		backcolor = (COLOR_RED|GetBackAlpha());
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4);
			
			for(i=0 ; i<4 ; i++){
				
				u = 0.5f + (mFrame.mCurrentFramePosition[i].x / (f32)WK7_SCREEN_WIDTH  - 0.5f) * scale;
				v = 0.5f + (mFrame.mCurrentFramePosition[i].y / (f32)WK7_SCREEN_HEIGHT - 0.5f) * scale;
				
				GXPosition3f32(mFrame.mCurrentFramePosition[i].x,mFrame.mCurrentFramePosition[i].y,mRenderZ);
				GXColor1u32   (backcolor);
				GXTexCoord2f32(u,v);
			}
		
		GXEnd();
		
	}
#if 0
	// v[[ƓGtJ[\
static	GXColor		col = { 0x00,0x00,0x00,0x00 };
	
	col.a = WK7RenderObj::ConvRenderIDToAlpha(WK7RENDEROBJ_RENDERID_PLAYER);
//	GXColor		col = { 0x00,0x00,0x00,0x08 };
	GXSetTevKColor   (GX_KCOLOR0,col);
	GXSetTevKAlphaSel(GX_TEVSTAGE0,GX_TEV_KASEL_K0_A);
	
	GXSetNumTevStages (1);
	
	GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_TEXC);
	GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_TEXA,GX_CA_KONST,GX_CA_RASA,GX_CA_ZERO);
	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_COMP_A8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_COMP_A8_EQ,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetAlphaCompare (GX_GREATER,0x00,GX_AOP_AND,GX_ALWAYS,0xff);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mBackZ);
		GXColor1u32   (backcolor);
		GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f - mBackScale));
		
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mBackZ);
		GXColor1u32   (backcolor);
		GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f - mBackScale));
		
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mBackZ);
		GXColor1u32   (backcolor);
		GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f + mBackScale));
		
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mBackZ);
		GXColor1u32   (backcolor);
		GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f + mBackScale));
		
	GXEnd();
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
#endif
	
	return true;
}

bool WK7SelectItem::RenderFrame(void)
{
	int				i;
//	int				i,j;
		f32				size;
		size = size_max;
//		
//		if( GetHighLighted()==false ){
//	//	if( GetSelected()==false ){
//			size = size_min;
//		}else{
//			size = size_max;
//		}
//		CalcParallelFramePosition(size);
//	//	CalcParallelFramePosition(WK7SELECTIF->GetLineSize());
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_NONE   );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);

	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	
//	GXSetZMode        (GX_TRUE,GX_LEQUAL,GX_TRUE);
//	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetZMode        (GX_TRUE,GX_GREATER,GX_TRUE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	
	GXSetLineWidth((u8)(mLineWidth>>1),GX_TO_ZERO);
	
	GXSetDither      (GX_FALSE);
	
//	WK7SelectItemFrameBlur		*pBlur,*pNextBlur;
//	for(pBlur=(WK7SelectItemFrameBlur *)mBlurAnchor.GetNext() ; pBlur!=NULL ; pBlur=pNextBlur){
//		pNextBlur = pBlur->GetNext();
//		pBlur->RenderProcess();
//	}
	
	u32		col;
	
	if( GetSelected()==false ){
//	if( GetSelected()==true ){
		col = COLOR_WHITE;
	}else{
//		if( mType==TYPE_LINE ){
//			col = COLOR_BLUE;
//		}else{
			col = WK7SELECTIF->GetSelectedFrameColor();
//			col = COLOR_RED;
//		}
	}
//	if( GetHighLighted()==true ){
//		col = HIGHLIGHTTED_COLOR;
//	}else{
//		col = 0xffffff00;
//	}
	col |= GetBackAlpha();
	
	if( GetSelected()==false ){
//	if( GetSelected()==false || GetHighLighted()==true ){
		
//		col = 0xffffff00|GetBackAlpha();
//		col = mCurrentNormalColor;
		
		GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,8);
			
			for(i=0 ; i<4 ; i++){
//				u32		col;
//				switch( i ){
//				case	1:
//				case	2:
//					col = mCurrentSelectedColor;
//					break;
//				default:
//					col = mCurrentNormalColor;
//					break;
//				}
				GXPosition3f32(mFrame.mCurrentFramePosition [i].x,mFrame.mCurrentFramePosition [i].y,mRenderZ + 10.0f);
				GXColor1u32   (col);
				GXPosition3f32(mFrame.mParallelFramePosition[i].x,mFrame.mParallelFramePosition[i].y,mRenderZ + 10.0f);
				GXColor1u32   (col);
			}
			
		GXEnd();
		
//		::CalcParallelFramePosition(mFrameEx[0].mParallelFramePosition,
//		                            mFrameEx[0].mCurrentFramePosition,sizeof(mFrameEx[0].mCurrentFramePosition) / sizeof(Vec),size);
//		
//		GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,8);
//			
//			for(i=0 ; i<4 ; i++){
//				GXPosition3f32(mFrameEx[0].mCurrentFramePosition [i].x,mFrameEx[0].mCurrentFramePosition [i].y,mRenderZ + 10.0f);
//				GXColor1u32   (col);
//				GXPosition3f32(mFrameEx[0].mParallelFramePosition[i].x,mFrameEx[0].mParallelFramePosition[i].y,mRenderZ + 10.0f);
//				GXColor1u32   (col);
//			}
//			
//		GXEnd();
		
	}
		
//	if( GetHighLighted()==true ){
//		
//		for(j=0 ; j<2 ; j++){
//			
//			if( j ){
//				col = 0xff000000|WK7SELECTIF->GetBackAlphaValue();
//			}else{
//				col = 0x0000ff00|WK7SELECTIF->GetBackAlphaValue();
//			}
//			
//			::CalcParallelFramePosition(mFrameEx[j].mParallelFramePosition,
//			                            mFrameEx[j].mCurrentFramePosition,sizeof(mFrameEx[j].mCurrentFramePosition) / sizeof(Vec),size);
//			
//			GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,8);
//				
//				for(i=0 ; i<4 ; i++){
//					GXPosition3f32(mFrameEx[j].mCurrentFramePosition [i].x,mFrameEx[j].mCurrentFramePosition [i].y,mRenderZ + 10.0f);
//					GXColor1u32   (col);
//					GXPosition3f32(mFrameEx[j].mParallelFramePosition[i].x,mFrameEx[j].mParallelFramePosition[i].y,mRenderZ + 10.0f);
//					GXColor1u32   (col);
//				}
//				
//			GXEnd();
//			
//		}
//		
//	}else{
	
	if( GetSelected()==true ){
		WK7SelectItemFrameLine::SetRenderZ(mRenderZ + 10.0f);
		if( mpFrameLine!=NULL ){
			u32		line_color = col;
//			u32		line_color = 0xff000000|GetBackAlpha();
//			u32		line_color = (mCurrentBackColor & 0xffffff00)|GetBackAlpha();
//			u32		line_color = (mCurrentSelectedColor & 0xffffff00)|GetBackAlpha();
			mpFrameLine[0].SetColor(line_color,line_color);
			mpFrameLine[1].SetColor(line_color,line_color);
			mpFrameLine[2].SetColor(line_color,line_color);
//static	bool	oneline = true;
//			if( oneline==true ){
//			mpFrameLine[1].RenderProcess(size);
//			}else
			for(i=0 ; i<3 ; i++){
				mpFrameLine[i].RenderProcess(size);
			}
		}
	}
//	}
	
	WK7SelectItemFrameBlur		*pBlur,*pNextBlur;
	for(pBlur=(WK7SelectItemFrameBlur *)mBlurAnchor.GetNext() ; pBlur!=NULL ; pBlur=pNextBlur){
		pNextBlur = pBlur->GetNext();
		pBlur->mRenderZ = mRenderZ + 20.0f;
		pBlur->RenderProcess();
	}
	
	return true;
}

bool WK7SelectItem::RenderItemLine(void)
{
	int				i;
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_NONE   );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);

	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode     (GX_CULL_NONE);
	GXSetDither       (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	
	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	
	GXSetLineWidth((u8)(mLineWidth>>1),GX_TO_ZERO);
	
	GXSetDither      (GX_FALSE);
	
	u32		col;
	
//	if( GetSelected()==false ){
//		col = mCurrentSelectedColor;
//	}else{
//		col = mCurrentNormalColor;
//	}
//	col = (col & 0xffffff00)|GetBackAlpha();
//	if( GetHighLighted()==true ){
//		col = HIGHLIGHTTED_COLOR;
//	}else{
//		col = 0xffffff00;
//	}
//	col |= GetBackAlpha();
	col = (mCurrentBackColor & 0xffffff00)|GetBackAlpha();
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,4);
	
	for(i=0 ; i<4 ; i++){
		GXPosition3f32(mFrame.mParallelFramePosition[i].x,mFrame.mParallelFramePosition[i].y,mRenderZ + 10.0f);
		GXColor1u32   (col);
	}
	
	GXEnd();
	
	return true;
}

bool WK7SelectItem::RenderRate(void)
{
	
	// _`
	
	Vec		dir,vdir;
	f32		len;
static	f32	size  = 20.0f;
	f32		width;
	Vec		pos,pos0;
	
	if( GetHighLighted()==true ){
		width = size_max;
	}else{
		width = size_min;
	}
	
	VECAdd     (&mFrame.mCutFramePosition[3],&mFrame.mCutFramePosition[0],&pos0);
	VECScale   (&pos0,&pos0,0.5f);
	VECSubtract(&mFrame.mCutFramePosition[1],&mFrame.mCutFramePosition[0],&dir);
	dir.z = 0.0f;
	len = VECMag(&dir);
	VECNormalize(&dir,&dir);
	vdir.x =  dir.y;
	vdir.y = -dir.x;
	vdir.z = 0.0f;
	VECScale(&vdir,&vdir,size);
	
	VECScale(&dir,&pos,mRate * len);
	VECAdd  (&pos,&pos0,&pos);
	
	VECScale(&dir,&dir,width);
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);

	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode     (GX_CULL_NONE);
	GXSetDither       (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	
	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	GXSetDither       (GX_FALSE);
	
	u32		col;
	if( GetSelected()==true ){
//		col = COLOR_WHITE;
		col = COLOR_RED;
	}else{
//		col = COLOR_BLUE;
		col = COLOR_WHITE;
	}
	col = (col & 0xffffff00)|GetBackAlpha();
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,4);
	
	GXPosition3f32(pos.x + vdir.x,pos.y + vdir.y,mRenderZ + 10.0f);
	GXColor1u32   (col);
	GXPosition3f32(pos.x - vdir.x,pos.y - vdir.y,mRenderZ + 10.0f);
	GXColor1u32   (col);
	
	pos.x -= dir.x;
	pos.y -= dir.y;
	
	GXPosition3f32(pos.x + vdir.x,pos.y + vdir.y,mRenderZ + 10.0f);
	GXColor1u32   (col);
	GXPosition3f32(pos.x - vdir.x,pos.y - vdir.y,mRenderZ + 10.0f);
	GXColor1u32   (col);
	
	GXEnd();
	
//	if( mpRatePrim==NULL ){
//		return false;
//	}
//	int		i;
//	u32		col = (mCurrentBackColor & 0xffffff00)|GetBackAlpha();
//	for(i=0 ; i<RATEPRIM_NUM ; i++){
//		mpRatePrim[i].mSetRGBA(col);
//	}
	return true;
}

bool WK7SelectItem::SetupRenderProcess(void)
{
	f32			rate,_rate;
	GXColor		selectedColor = GetSelectedColor();
	GXColor		normalColor   = GetNormalColor();
	 rate = (f32)mColorR.GetValue() / COLOR_RANGE;
	_rate = 1.0f - rate;
	mCurrentSelectedColor = ((u32)(normalColor.r * _rate + selectedColor.r * rate) << 24)|
	                        ((u32)(normalColor.g * _rate + selectedColor.g * rate) << 16)|
	                        ((u32)(normalColor.b * _rate + selectedColor.b * rate) <<  8)|
	                         mAlpha.GetValue();
	mCurrentNormalColor   = ((u32)normalColor.r << 24)|
	                        ((u32)normalColor.g << 16)|
	                        ((u32)normalColor.b <<  8)|
	                         mAlpha.GetValue();
	
	mCurrentMaskColor      = 0xffffff00|mAlpha.GetValue();
	return true;
}

stFontParam* GetFontParam(char letter,int type)
{
	if( letter < '!' || letter > '~' ){
		return NULL;
	}
	stFontParam		*pFontParam = gpTypoMng->mGetFontParamPtr();
	switch( type ){
	case	FONTTYPE_ARIALBLACK:
		pFontParam += 94;
		break;
	case	FONTTYPE_ARIAL:
	default:
		break;
	}
	pFontParam += (letter - '!');
	return pFontParam;
}

bool WK7SelectItem::SetType(int type)
{
	switch( type ){
	case	TYPE_CUT:
		GHM_SAFE_DELETE_ARRAY(mpRatePrim);
		break;
	case	TYPE_LINE:
//		if( mpRatePrim==NULL ){
//			if( (mpRatePrim = new mK7Sprite[RATEPRIM_NUM])==NULL ){
//				break;
//			}
//			stTexture	*pTexture   = gpTypoMng->mGetTexStrPtr();
//			int			i;
//			f32			x = 320.0f,y = 240.0f;
//			for(i=0 ; i<RATEPRIM_NUM ; i++){
//				mpRatePrim[i].mRelateTex(pTexture);
//				mpRatePrim[i].mSetWH(0,0);
//				mpRatePrim[i].mSetDispFlag(false);
//				mpRatePrim[i].mSetRGBA(0xffffffff);
//			}
//			stFontParam		*pFontParam;
//			if( (pFontParam=GetFontParam('0',FONTTYPE_ARIALBLACK))!=NULL ){
//				for(i=0 ; i<3 ; i++){
//					mpRatePrim[i].mSetWH ((s16)pFontParam->w,(s16)pFontParam->h);
//					mpRatePrim[i].mSetUV0((s16)pFontParam->u,(s16)pFontParam->v);
//					mpRatePrim[i].mSetXY0(x,y);
//					mpRatePrim[i].mSetScale(eScaleWH,RATEPRIM_SCALE,1);
//					x += (f32)pFontParam->w * RATEPRIM_SCALE;
//				}
//			}
//			if( (pFontParam=GetFontParam('%',FONTTYPE_ARIALBLACK))!=NULL ){
//				mpRatePrim[RATEPRIM_NUM-1].mSetWH ((s16)pFontParam->w,(s16)pFontParam->h);
//				mpRatePrim[RATEPRIM_NUM-1].mSetUV0((s16)pFontParam->u,(s16)pFontParam->v);
//				mpRatePrim[RATEPRIM_NUM-1].mSetScale(eScaleWH,RATEPRIM_SCALE,1);
//				mpRatePrim[RATEPRIM_NUM-1].mSetXY0(x,y);
//			}
//		}
		GHM_SAFE_DELETE_ARRAY(mpRatePrim);
		break;
	default:
		GHM_SAFE_DELETE_ARRAY(mpRatePrim);
		return false;
	}
	mType = type;
	return true;
}

bool WK7SelectItem::RenderProcess(void)
{
	
	int		i;
	
//	PRINTF("STC (%.3f,%.3f) FP  (%.3f,%.3f)\n",mScreenTargetCenter.x,mScreenTargetCenter.y,mFocusPos.x,mFocusPos.y);
//	PRINTF("CIP (%.3f,%.3f) CID (%.3f,%.3f)\n",mCutInPosition.x,mCutInPosition.y,mCutInDirection.x,mCutInDirection.y);
//	PRINTF("TP  (%.3f,%.3f) (%.3f,%.3f)\n",tp[0].x,tp[0].y,tp[1].x,tp[1].y);
//	for(i=0 ; i<6 ; i++){
//		PRINTF("%d:(%.3f,%.3f)\n",i,mFrame.mCutFramePosition[i].x,mFrame.mCutFramePosition[i].y);
//	}
	
	if( !mVisible || !mRender ){
		if( mpRatePrim!=NULL ){
			for(i=0 ; i<RATEPRIM_NUM ; i++){
				mpRatePrim[i].mSetDispFlag(false);
			}
		}
//		WK7Letter	*pLetter;
//		for(i=0,pLetter=mpLetter ; i<SELECTITEM_LETTER_NUM ; i++,pLetter++){
//			pLetter->SetDisp(false);
//		}
//05/09/2004			mTypos.mSetDispFlag(false);
		return true;
	}
	
//	if( mpRatePrim!=NULL ){
//		for(i=0 ; i<RATEPRIM_NUM ; i++){
//			mpRatePrim[i].mSetDispFlag(true);
//		}
//	}
//05/09/2004	mTypos.mSetDispFlag(true);
	
	RenderBack();
	
//	if( HSD_PadGetTrigger(1)==HSD_PAD_X ){
//		if( mType==TYPE_LINE ){
//			mType = TYPE_CUT;
//		}else{
//			mType = TYPE_LINE;
//		}
//	}
	
	switch( mType ){
	case	TYPE_LINE:
//		RenderItemLine();
//		RenderRate();
//		break;
	case	TYPE_CUT:
	default:
//		RenderBackColor();
		RenderFrame();
#ifdef	_DEBUG
		RenderTargetCircle();
#endif
		break;
	}
	
	if( mpLockOn != NULL ){
		mpLockOn->SetColor(mCurrentBackColor|GetBackAlpha());
		mpLockOn->RenderProcess();
	}
	
	return true;
}

bool WK7SelectItem::RenderMask(void)
{
	if( !mVisible || !mRender ){
		return true;
	}
//	switch( mType ){
//	case	TYPE_LINE:
//		return true;
//	case	TYPE_CUT:
//	default:
//		break;
//	}
	
	u32		mask_color;
	int		i;
	mask_color = GetBackAlpha();
	if( mask_color==0 ){
		return true;
	}
	mask_color |= 0xffffff00;
	f32		z = WK7SELECTIF->GetWindowMaskZ();
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);
	
	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate (GX_FALSE);
	GXSetAlphaUpdate (GX_TRUE);
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXSetDither      (GX_TRUE);
	
	GXBegin(GX_TRIANGLEFAN,GX_VTXFMT0,4);
	
	for(i=0 ; i<4 ; i++){
		GXPosition3f32(mFrame.mCurrentFramePosition[i].x,mFrame.mCurrentFramePosition[i].y,z);
		GXColor1u32   (mask_color);
	}
	
	GXEnd();
	
	return true;
}

bool WK7SelectItem::CalcParallelFramePosition(f32 dist)
{
	return ::CalcParallelFramePosition(mFrame.mParallelFramePosition,
	                                   mFrame.mCurrentFramePosition,sizeof(mFrame.mCurrentFramePosition) / sizeof(Vec),dist);
}

bool CalcParallelFramePosition(Vec *pParallelFramePosition,Vec *pFramePosition,int framePositionNum,f32 dist)
{
	if( framePositionNum <= 1 ){
		return false;
	}
	
	Vec		dir[2],vdir[2],mdir;
	f32		ip;
	int		i;
	
	VECSubtract(pFramePosition + 1,pFramePosition + 0,dir + 0);
	dir[0].z = 0.0f;
	
	vdir[0].x =  dir[0].y;
	vdir[0].y = -dir[0].x;
	vdir[0].z = 0.0f;
	VECNormalize(vdir + 0,vdir + 0);
	VECScale    (vdir + 0,vdir + 0,dist);
	
	VECAdd(pFramePosition + 0,vdir + 0,pParallelFramePosition + 0);
	
	for(i=1 ; i<framePositionNum-1 ; i++){
	
		VECSubtract(pFramePosition + (i+1),pFramePosition + i,dir + 1);
		dir[1].z = 0.0f;
		
		if( VECMag(dir + 1) < FLOAT_ERROR ){
			
			VECAdd(vdir + 0,pFramePosition + i,pParallelFramePosition + i);
			
		}else{
			
			vdir[1].x =  dir[1].y;
			vdir[1].y = -dir[1].x;
			vdir[1].z = 0.0f;
			VECNormalize(vdir + 1,vdir + 1);
			VECScale    (vdir + 1,vdir + 1,dist);
			VECAdd      (vdir + 0,vdir + 1,&mdir);
			
			if( VECMag(&mdir) < FLOAT_ERROR ){
				
				VECAdd(vdir + 1,pFramePosition + i,pParallelFramePosition + i);
				
			}else{
			
				VECNormalize(&mdir,&mdir);
				VECNormalize(vdir + 0,vdir + 0);
				
				ip = VECDotProduct(&mdir,vdir + 0);
				if( ip >= 0.0f && ip < FLOAT_ERROR ){
					ip = FLOAT_ERROR;
				}else if( ip <= 0.0f && ip > -FLOAT_ERROR ){
					ip = -FLOAT_ERROR;
				}
				
				VECScale(&mdir,&mdir,dist / ip);
				VECAdd  (&mdir,pFramePosition + i,pParallelFramePosition + i);
				
			}
			
			vdir[0] = vdir[1];
			 dir[0] =  dir[1];
		
		}
		
	}
	
	VECAdd(pFramePosition + framePositionNum - 1,vdir + 1,pParallelFramePosition + framePositionNum - 1);
	
	return true;
}

//
// pParallelFramePositionframePositionNum~Q̗̈悪Kv
//
bool CalcParallelFramePosition2(Vec *pParallelFramePosition,Vec *pFramePosition,int framePositionNum,f32 dist)
{
	if( framePositionNum <= 1 ){
		return false;
	}
	
	Vec		dir[2],vdir[2],mdir;
	f32		ip;
	int		i;
	
	VECSubtract(pFramePosition + 1,pFramePosition + 0,dir + 0);
	
	vdir[0].x =  dir[0].y;
	vdir[0].y = -dir[0].x;
	vdir[0].z = 0.0f;
	VECNormalize(vdir + 0,vdir + 0);
	VECScale    (vdir + 0,vdir + 0,dist);
	vdir[1]   = vdir[0];
	
	VECAdd     (pFramePosition + 0,vdir + 0,pParallelFramePosition + 0);
	VECSubtract(pFramePosition + 0,vdir + 0,pParallelFramePosition + 1);
	
	for(i=1,pParallelFramePosition+=2,pFramePosition++ ; i<framePositionNum-1 ; i++,pParallelFramePosition+=2,pFramePosition++){
	
		VECSubtract(pFramePosition + 1,pFramePosition,dir + 1);
		vdir[1].x =  dir[1].y;
		vdir[1].y = -dir[1].x;
		vdir[1].z = 0.0f;
		VECNormalize(vdir + 1,vdir + 1);
		
		VECScale    (vdir + 1,vdir + 1,dist);
		VECAdd      (vdir + 0,vdir + 1,&mdir);
		VECNormalize(&mdir,&mdir);
		VECNormalize(vdir + 0,vdir + 0);
		
		ip = VECDotProduct(&mdir,vdir + 0);
		if( ip >= 0.0f && ip < FLOAT_ERROR ){
			ip = FLOAT_ERROR;
		}else if( ip <= 0.0f && ip > -FLOAT_ERROR ){
			ip = -FLOAT_ERROR;
		}
		
		VECScale   (&mdir,&mdir,dist / ip);
		VECAdd     (pFramePosition,&mdir,pParallelFramePosition + 0);
		VECSubtract(pFramePosition,&mdir,pParallelFramePosition + 1);
		
		vdir[0] = vdir[1];
		 dir[0] =  dir[1];
		
	}
	
	VECAdd     (pFramePosition,vdir + 1,pParallelFramePosition + 0);
	VECSubtract(pFramePosition,vdir + 1,pParallelFramePosition + 1);
	
	return true;
}

bool WK7SelectItem::RenderLine(f32 backz)
{
	
	if( !mVisible || !mRender ){
		return true;
	}
//	switch( mType ){
//	case	TYPE_LINE:
//		return true;
//	case	TYPE_CUT:
//	default:
//		break;
//	}
	
	int			i;
	u32			color;
	u32			color2;
	f32			rate,_rate;
//	GXColor		selectedColor = GetSelectedColor();
//	GXColor		normalColor   = GetNormalColor();
	
	 rate = (f32)mColorR.GetValue() / COLOR_RANGE;
	_rate = 1.0f - rate;
//	color = ((u32)(normalColor.r * _rate + selectedColor.r * rate) << 24)|
//	        ((u32)(normalColor.g * _rate + selectedColor.g * rate) << 16)|
//	        ((u32)(normalColor.b * _rate + selectedColor.b * rate) <<  8)|
//	        mAlpha.GetValue();
//	color2 = ((u32)normalColor.r << 24)|
//	         ((u32)normalColor.g << 16)|
//	         ((u32)normalColor.b <<  8)|
//	          mAlpha.GetValue();
	
//	if( GetHighLighted()==true ){
//		color = HIGHLIGHTTED_COLOR;
//	}else{
//		color = 0xffffff00;
//	}
	if( GetSelected()==true ){
//		if( mType==TYPE_LINE ){
//			color = COLOR_BLUE;
//		}else{
			color = WK7SELECTIF->GetSelectedFrameColor();
//			color = COLOR_RED;
//		}
	}else{
		color = COLOR_WHITE;
	}
	color |= GetBackAlpha();
	
//	color  = mCurrentBackColor|mAlpha.GetValue();
	color2 = 0xffffff00|mAlpha.GetValue();
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_NONE   );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);

	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevOp       (GX_TEVSTAGE0,GX_PASSCLR);
	
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
//	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_INVDSTALPHA,GX_BL_DSTALPHA,GX_LO_CLEAR);
	
	GXSetLineWidth   ((u8)(mLineWidth>>1),GX_TO_ZERO);
//	GXSetLineWidth   (mLineWidth,GX_TO_ZERO);
	
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
//	GXSetZMode       (GX_TRUE,GX_GEQUAL,GX_FALSE);
//	GXSetZMode       (GX_TRUE,GX_LEQUAL,GX_TRUE);
//	GXSetZMode       (GX_TRUE,GX_EQUAL,GX_TRUE);
	
	GXSetDither      (GX_FALSE);
//	GXSetDither      (GX_TRUE);
	
	Vec		parallelFramePosition[4];
	::CalcParallelFramePosition(parallelFramePosition,
	                            mFrame.mCurrentFramePosition,sizeof(parallelFramePosition) / sizeof(Vec),
	                            WK7SELECTIF->GetLineSize() * WK7_SCREEN_WIDTH / WK7SELECTIF->GetMaskW());
//	CalcParallelFramePosition(WK7SELECTIF->GetLineSize() * WK7_SCREEN_WIDTH / WK7SELECTIF->GetMaskW());
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,8);
		
	for(i=0 ; i<4 ; i++){
		u32		col;
		switch( i ){
		case	1:
		case	2:
			col = color;
			break;
		default:
			col = color2;
			break;
		}
		GXPosition3f32(mFrame.mCurrentFramePosition [i].x,mFrame.mCurrentFramePosition [i].y,backz);
		GXColor1u32   (col);
		GXPosition3f32(parallelFramePosition[i].x,parallelFramePosition[i].y,backz);
		GXColor1u32   (col);
	}
	
	GXEnd();
	
	return true;
}

bool WK7SelectItem::Appear(u32 tick)
{
	mAlpha.Play(0xff,tick);
//	mTypos.mSetFade(0xff,(s32)tick);
	mState = FADEIN;
	return true;
}

bool WK7SelectItem::AppearTypo(u32 tick)
{
#pragma unused(tick)
	mTypos.mDisp();
//	mTypos.mSetFade(0xff,(s32)tick);
	return true;
}

bool WK7SelectItem::Disappear(u32 tick)
{
	mAlpha.Play(0x00,tick);
	mTypos.mUnDisp(eTypoUnselect);
//	mTypos.mSetFade(0x00,(s32)tick);
	mState = FADEOUT;
	mEnable = 0;
	return true;
}

bool WK7SelectItem::Select(u32 tick)
{
	if( mpObj!=NULL ){
		mpObj->SetEnable(false);
		if( mpObj->CallSelectCallback()==false ){
			return false;
		}
	}
	if( mpBranchPointObj!=NULL ){
		WK7PathBranchPointObjJObj	*pBranchPointObjJObj;
		switch( mpBranchPointObj->GetType() ){
		case	WK7PathBranchPointObj::TYPE_JOBJ:
		case	WK7PathBranchPointObj::TYPE_JOBJDOOR:
			pBranchPointObjJObj = (WK7PathBranchPointObjJObj *)mpBranchPointObj;
			break;
		default:
			pBranchPointObjJObj = NULL;
			break;
		}
		bool		callevent = true;
		bool		move      = false;
		
		if( pBranchPointObjJObj               != NULL       &&
		    pBranchPointObjJObj->GetEventNo() != 0xffffffff ){
			
			WK7PathBranchPoint	*pBranchPoint;
			WK7Stage			*pStage;
			
			callevent = false;
			
			if( (pBranchPoint=mpBranchPointObj->GetBranchPoint())!=NULL &&
			    (pStage=pBranchPoint->GetStage())!=NULL ){
				
				switch( pBranchPointObjJObj->GetEventNo() & K7EV_TYPE_MASK ){
				case	K7EV_TYPE_NO:{
					WK7EventInfo		eventinfo;
					eventinfo.Initialize();
					eventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					eventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallEventCallBackTask    (eventinfo);
					if( WK7SELECTIF->IsValid(this)==false ){
						return true;
					}
					if( mpBranchPointObj!=NULL && mpBranchPointObj->GetEventID()!=0xffffffff ){
						callevent = true;
					}
					}break;
				case	K7EV_TYPE_KEY:{
					WK7KeyEventInfo		keyeventinfo;
					if( mpBranchPointObj->GetEventID()!=WK7Stage::GetDefKeyEventID() ){
						callevent = true;
						break;
					}
					keyeventinfo.Initialize();
					keyeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					keyeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallKeyEventCallBackTask (keyeventinfo);
					}break;
				case	K7EV_TYPE_MAP:{
					WK7MapEventInfo		mapeventinfo;
					if( mpBranchPointObj->GetEventID()!=WK7Stage::GetDefMapEventID() ){
						callevent = true;
						break;
					}
					mapeventinfo.Initialize();
					mapeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					mapeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallMapEventCallBackTask (mapeventinfo);
					}break;
				case	K7EV_TYPE_MEMO:{
					WK7MemoEventInfo	memoeventinfo;
					if( mpBranchPointObj->GetEventID()!=WK7Stage::GetDefMemoEventID() ){
						callevent = true;
						break;
					}
					memoeventinfo.Initialize();
					memoeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					memoeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallMemoEventCallBackTask(memoeventinfo);
					}break;
				case	K7EV_TYPE_VMEMO:{
					WK7VoiceMemoEventInfo	vmemoeventinfo;
					vmemoeventinfo.Initialize();
					vmemoeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					vmemoeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallVoiceMemoEventCallBackTask(vmemoeventinfo);
					}break;
				case	K7EV_TYPE_RAKU:{
					WK7RakuEventInfo		rakueventinfo;
					rakueventinfo.Initialize();
					rakueventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					rakueventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallRakuEventCallBackTask(rakueventinfo);
					}break;
				case	K7EV_TYPE_PLAY:{
					WK7PlayEventInfo		playeventinfo;
					playeventinfo.Initialize();
					playeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					playeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallPlayEventCallBackTask(playeventinfo);
					}break;
				case	K7EV_TYPE_DOOR:{
					WK7DoorEventInfo				dooreventinfo;
					WK7PathBranchPointObjJObjDoor	*pBranchPointObjJObjDoor;
					if( mpBranchPointObj->GetEventID()!=WK7Stage::GetDefDoorEventID() ){
						callevent = true;
						break;
					}
					WK7PathBranchPoint				*pBranchPoint = mpBranchPointObj->GetBranchPoint();
					WK7Stage						*pStage;
					dooreventinfo.Initialize();
					dooreventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					dooreventinfo.SetSelectObj(pBranchPointObjJObj);
					if( pBranchPointObjJObj->GetType()==WK7PathBranchPointObj::TYPE_JOBJDOOR ){
						pBranchPointObjJObjDoor = (WK7PathBranchPointObjJObjDoor *)pBranchPointObjJObj;
						dooreventinfo.mNextStageID = pBranchPointObjJObjDoor->GetNextStageID();
						dooreventinfo.SetNextBpName(pBranchPointObjJObjDoor->GetNextPoint());
						WK7Stage::SetCurrentStageBpName(pBranchPointObjJObjDoor->GetNextPoint());
						WK7Stage::SetCurrentStageParam(pBranchPointObjJObjDoor->GetParam());
					}
					if( pBranchPoint!=NULL && (pStage=pBranchPoint->GetStage())!=NULL ){
						ghmGcMemSetAllocName("SIBPEVNT");
						WK7StageBpEvent		*pBpEvent = new WK7StageBpEvent(pBranchPoint->GetName());
						dooreventinfo.mCurrStageID = pStage->GetID();
						WK7Stage::SetPrevStageID    (dooreventinfo.mCurrStageID);
						if( pBpEvent!=NULL ){
							WK7Stage::SetPrevStageBpName(pBpEvent->GetShortBpName());
						}else{
							WK7Stage::SetPrevStageBpName(pBranchPoint->GetName());
						}
						WK7Stage::SetPrevStagePathName(NULL);
						GHM_SAFE_DELETE(pBpEvent);
					}
					pStage->CallDoorEventCallBackTask(dooreventinfo);
					}break;
				case	K7EV_TYPE_TV:{
					WK7TvEventInfo		tveventinfo;
					tveventinfo.Initialize();
					tveventinfo.SetSelectObj(pBranchPointObjJObj);
					tveventinfo.SetStageID  (pStage->GetID());
					pStage->CallTvEventCallBackTask(tveventinfo);
					}break;
				case	K7EV_TYPE_LTV:{
					WK7TvEventInfo		ltveventinfo;
					ltveventinfo.Initialize();
					ltveventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallLtvEventCallBackTask(ltveventinfo);
					}break;
				case	K7EV_TYPE_ZAN:{
					WK7ZanEventInfo		zaneventinfo;
					zaneventinfo.Initialize();
					zaneventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					zaneventinfo.SetSelectObj(pBranchPointObjJObj);
					zaneventinfo.mpStage     = pStage;
					pStage->CallZanEventCallBackTask(zaneventinfo);
					}break;
				case	K7EV_TYPE_TAMA:{
					WK7TamaEventInfo	tamaeventinfo;
					tamaeventinfo.Initialize();
					tamaeventinfo.mEventNo    = pBranchPointObjJObj->GetEventNo();
					tamaeventinfo.SetSelectObj(pBranchPointObjJObj);
					pStage->CallTamaEventCallBackTask(tamaeventinfo);
					}break;
				default:
					move = true;
					break;
				}
			}else{
				move = true;
			}
		}else{
			move = true;
		}
		if( WK7SELECTIF->IsValid(this)==false ){
			return true;
		}
		if( callevent==true ){
			if( mpBranchPointObj!=NULL && mpBranchPointObj->GetEventID()!=0xffffffff ){
				WK7EVENTCONTROLLER->Call(mpBranchPointObj->GetEventID(),(u32)mpBranchPointObj);
			}else{
				move = true;
			}
		}
		if( move==true ){
			GameInfo.SetMode(WK7GameInfo::MOVE);
		}
		if( mpBranchPointObj!=NULL ){
			mpBranchPointObj->Select(0);
		}
	}
	mExpandRate.Play(WK7SELECTITEM_EXPAND_RATE_MAX,tick);
	mTypos.mUnDisp(eTypoSelect);
//	mTypos.mSetFade(0x00,(s32)tick);
	mState = EXPAND;
	mAlpha.Play(0x00,tick);
	mEnable = 0;
	return true;
}

void WK7SelectItem::SetVisible(bool flag)
{
	switch( mState ){
	case	FADEIN:
	case	CUTFRAME:
		break;
	case	FADEOUT:
	case	EXPAND:
//		if( flag==false ){
			break;
//		}
	case	VANISHED:
	case	EXPANDED:
	default:
		return;
	}
	mVisible = (flag==true)? 1 : 0;
	mResetTypoPos = 1;
	mTypos.mSetDispFlag(flag);
//05/09/2004		mTypos.mSetDispFlag(flag);
	mTargetCenterSet = 0;
//	WK7Letter				*pLetter;
//	int						i;
//	if( mpLetter!=NULL ){
//		for(i=0,pLetter=mpLetter ; i<SELECTITEM_LETTER_NUM ; i++,pLetter++){
//			pLetter->SetDisp(false);
//		}
//	}
}

void WK7SelectItem::SetDeformRate(f32 rate)
{
	if( rate < 0.0f ){
		rate = 0.0f;
	}else if( rate > 1.0f ){
		rate = 1.0f;
	}
static u32	rate_tick = WK7_TICK_PER_SEC / 4;
	mDeformRate.Play(rate,rate_tick);
}

WK7SelectIF::WK7SelectIF()
{
	mFlag          = 0;
	mEnable        = 1;
	mOperateEnable = 1;
	mVisible       = 1;
	mState         = IDLE;
	mTypoVisible   = false;
	mItemNum       = 0;
	
	// \n
	mBackVisible   = true;
//	mBackScale     = 1.05f;
//	mBackScale     = 0.95f;
	mBackScale     = 0.80f;
	mBackZ         = 0.0f;
	mBackColor     = 0x000000ff;
	mBackAlpha.SetValue(0x00);
//	mBackTexColor  = 0xff0000ff;
//	mBackTexColor  = 0xffc000ff;
//	mBackTexColor  = 0xffffffff;
//	mBackTexColor  = 0x808080ff;
	mBackTexColor  = 0xff0032ff;
	mCinemaScopeColor = 0x000000ff;
//	mSelectBackColor  = 0xf0c87800;
//	mSelectBackColor  = 0x3200ff00;
	mSelectBackColor  = 0xffffff00;
	mHighLightedBackColor = 0x3200ff00;
//	mHighLightedBackColor = 0x8c8cff00;
//	mHighLightedBackColor = 0x3200ff00;
//	mHighLightedBackColor = 0xff000000;
	mSemiHighLightedBackColor = 0xffffff00;
//	mSemiHighLightedBackColor = 0x00c0c000;
//	mSemiHighLightedBackColor = 0x32ffff00;
//	mSelectedFrameColor   = 0xe6b43c00;
	mSelectedFrameColor   = 0xff640000;
//	mSelectedFrameColor   = 0xe8100800;
//	mSelectBackDeltaColor = 0x40404000;
	mSelectBackDeltaColor = 0x2d1e0000;
//	mSelectBackDeltaColor = 0xb9966400;
//	mSelectBackDeltaColor = 0x00000000;
	mHighLightedBackDeltaColor = 0x00000000;
	mSemiHighLightedBackDeltaColor = 0x00000000;
	
	mMaskX = 410.0f;
	mMaskY = 300.0f;
	mMaskW = 240.0f;
	mMaskH = 180.0f;
	mLineSize = WK7SELECTIF_LINESIZE;
	mpFullColorScreenTexObj = NULL;
	mpMonoColorScreenTexObj = NULL;
}

WK7SelectIF::~WK7SelectIF()
{
}

bool WK7SelectIF::Initialize(void)
{
	return true;
}

bool WK7SelectIF::Terminate(void)
{
	WK7SelectItem	*pItem;
	WK7SelectItem	*pNext;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pNext){
		pNext = pItem->GetNext();
		delete pItem;
	}
	mBackAlpha.SetValue(0x00);
	mpFullColorScreenTexObj = NULL;
	mpMonoColorScreenTexObj = NULL;
	return true;
}

bool WK7SelectIF::SetSelectedItemAll(bool flag,WK7SelectItem *pExceptItem)
{
	WK7SelectItem	*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext() ){
		if( pItem==pExceptItem ){
			continue;
		}
		if( pItem->GetEnable()==false ){
			continue;
		}
		pItem->SetSelected(flag);
	}
	return true;
}

#define	WK7SELECTIF_STICK_LEVEL		(0.8f)
static WK7SelectItem	*pOldSelectedItem = NULL;
static bool				selectedPadRelease = true;
static Vec				selectedDir;
bool WK7SelectIF::CalcSelectedItem(void)
{
	// L[
	
	if( !mEnable ){
		pOldSelectedItem = NULL;
		SetSelectedItemAll(false);
		return false;
	}
	
	Vec				dir;
	f32				len;
	dir.x =  HSD_PadGetNormalStickX(0);
	dir.y = -HSD_PadGetNormalStickY(0) * WK7SELECTIF_ASPECT;		// L[ũhbgcl
	dir.z =  0.0f;
	len   = VECMag(&dir);
	
	if( selectedPadRelease==false ){
//		if( HSD_PadGetButton(0) & HSD_PAD_A ){
//			SetSelectedItemAll(false);
//			pOldSelectedItem = NULL;
//			return true;
//		}
		if( len > WK7SELECTIF_STICK_LEVEL ){
			Vec		ndir;
			VECNormalize(&dir,&ndir);
			if( VECDotProduct(&ndir,&selectedDir) > 0.9f ){
				SetSelectedItemAll(false);
				pOldSelectedItem = NULL;
				return true;
			}
		}
		selectedPadRelease = true;
	}
	
	if( !mOperateEnable ){
		pOldSelectedItem = NULL;
		return true;
	}
	
	WK7SelectItem	*pSelectedItem = NULL;
	
	if( len > WK7SELECTIF_STICK_LEVEL ){
		
		// AiO|ꂽ
		
#define	SELECTRATE_THRESHOLD		(0.9f)
		WK7SelectItem	*pItem;
		f32				maxrate = SELECTRATE_THRESHOLD;
		f32				rate;
		int				count = 0;
		
		VECNormalize(&dir,&dir);
		
		for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext() ){
			
			if( pItem->GetEnable()==false ){
				continue;
			}
			
			if( (rate = pItem->GetSelectRate(dir)) > maxrate ){
				pSelectedItem = pItem;
				maxrate       = rate;
			}
			
			count++;
			
		}
		
		if( !count ){
			pOldSelectedItem = NULL;
		}
		
		if( pSelectedItem!=NULL ){
			pSelectedItem->SetSelected(true);
			if( pOldSelectedItem != pSelectedItem ){
				gpSound->mPlaySE(SFX_SYS_HOUKOU);
				pOldSelectedItem  = pSelectedItem;
				selectedDir = dir;
				VECNormalize(&dir,&selectedDir);
			}
//		}else{
//			pOldSelectedItem  = NULL;
		}
		
	}else{
		if( WK7PLAYERCONTROL->GetPathControl()->GetState()!=WK7PathControl::PC_PATHSELECT ){
			pOldSelectedItem = NULL;
		}
	}
	
	if( WK7PLAYERCONTROL->GetPathControl()->GetState()!=WK7PathControl::PC_PATHSELECT ){
		SetSelectedItemAll(false,pSelectedItem);
	}else{
		if( pSelectedItem!=NULL ){
			SetSelectedItemAll(false,pSelectedItem);
		}
	}
	
//	if( HSD_PadGetTrigger(0) & HSD_PAD_A ){
//		pSelectedItem = GetSelectedItem();
//		if( pSelectedItem!=NULL && (pSelectedItem->GetObj()!=NULL || pSelectedItem->GetPathBranchPointObj()!=NULL) ){
//			SelectItem(pSelectedItem);
//		}
//	}
	
	return true;
}

bool WK7SelectIF::CheckSelectObj(void)
{
	WK7SelectItem		*pItem;
	WK7SelectObj		*pObj;
	ghmGcViewVolume		vv;
	f32					n,f,aspect,fov;
	Vec					campos,camdir;
	Vec					playerpos,objpos;
	
	switch( GameInfo.GetMode() ){
	case	WK7GameInfo::MOVE:
		break;
	case	WK7GameInfo::PLAY:
	case	WK7GameInfo::BATTLE:
	default:
		return false;
	}
	
	n      = WK7CAM->GetNear  ();
	f      = WK7CAM->GetFar   ();
	fov    = WK7CAM->GetFov   ();
	aspect = WK7CAM->GetAspect();
	WK7CAM->GetPosition      (campos);
	WK7CAM->GetTargetPosition(camdir);
	VECSubtract(&camdir,&campos,&camdir);
	
	WK7PLAYERCONTROL->GetPosition(playerpos);
	
	vv.SetCurrent(campos,camdir,n,f,aspect,fov);
	
	for(pObj=(WK7SelectObj *)mObjAnchor.GetNext() ; pObj!=NULL ; pObj=pObj->GetNext()){
		
		pObj->FrameProcess();
		
		if( pObj->GetEnable()==false ){
			continue;
		}
		pItem = GetItem(pObj);
		switch( pObj->GetCrossState(vv) ){
		case	ghmGcCollObj::CROSSOVER:
		case	ghmGcCollObj::INSIDE:
			pObj->GetPosition(objpos);
			if( GHM_VEC_GET_DIST(&playerpos,&objpos) < 30.0f ){
				if( pItem!=NULL ){
					continue;
				}
				break;
			}
		case	ghmGcCollObj::OUTSIDE:
		default:
			if( pItem!=NULL && pItem->GetEnable()==true ){
				pItem->Disappear();
				if( GetEnableItemNum()==0 ){
					Disappear();
				}
			}
			continue;
		}
		ghmGcMemSetAllocName("SIFI");
		if( (pItem = new WK7SelectItem(&mAnchor))!=NULL ){
			const char		*pName[2];
			if( (pName[0] = pObj->GetObjName(0))==NULL ){
				pName[0] = "nameless";
			}
			if( (pName[1] = pObj->GetObjName(1))==NULL ){
				pName[1] = "NPC";
			}
			mItemNum++;
			pItem->SetObj(pObj);
			pItem->SetTypo(pName[0],pName[1]);
			pItem->SetVisible(true);
			pItem->Appear();
			pItem->AppearTypo();
			switch( mState ){
			case	IDLE:
			case	DISAPPEAR:
				Appear();
				break;
			case	APPEAR:
			case	EXECUTE:
			default:
				break;
			}
		}
	}
	
	return true;
}

bool WK7SelectIF::FrameProcess(void)
{
	WK7SelectItem		*pItem;
	WK7SelectItem		*pNext;
//	C_Paper				paper(32,32);
	
	CheckSelectObj();
	
	WK7SelectItem::CalcPadDirection();
	
	CalcSelectedItem();
	
	mBackAlpha.FrameProcess();
	if( mBackAlpha.GetValue()==0xff ){
		if( !mBackAlphaMax ){
			gpSound->mPlaySE(SFX_SYS_HOUKOU_TOMARU);
			mBackAlphaMax = 1;
		}
	}else{
		mBackAlphaMax = 0;
	}
	
	WK7SelectItem::CalcCenterPosition();
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem!=NULL ){
		pNext = pItem->GetNext();
		if( pItem->FrameProcess()==false ){
			mItemNum--;
			delete pItem;
		}
		pItem = pNext;
	}
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem!=NULL ){
		pItem->CalcTypoOverlap();
		pItem = pItem->GetNext();
	}
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem!=NULL ){
		pItem->CalcTypoPosition();
		pItem = pItem->GetNext();
	}
	
	return true;
}

void WK7SelectIF::SetVisible(bool flag)
{
	WK7SelectItem	*pItem;
	mVisible = (flag==true)? 1 : 0;
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem != NULL ){
		pItem->SetVisible(flag);
		pItem = pItem->GetNext();
	}
}

bool WK7SelectIF::SetupContrastTev(void)
{
	GXTevStageID		tevstageid = (GXTevStageID)((int)GX_TEVSTAGE0 - 1);
	
	// ((texc>0.5? texc : 0) - 0.5) -> r0
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXColor				cmpcolor = { 0x7f,0x7f,0x7f,0x7f };
	
	GXSetTevKColor    (GX_KCOLOR0,cmpcolor);
	GXSetTevKColorSel (tevstageid,GX_TEV_KCSEL_K0);
	GXSetTevColorIn   (tevstageid,GX_CC_TEXC,GX_CC_KONST,GX_CC_TEXC,GX_CC_ZERO);
//	GXSetTevColorIn   (tevstageid,GX_CC_TEXC,GX_CC_HALF,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorOp   (tevstageid,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVREG0);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD0,SetupContrastTev_texmapID,GX_COLOR0A0);
	
	// r0 - 0.5 -> r0
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_HALF,GX_CC_ZERO,GX_CC_ZERO,GX_CC_C0);
	GXSetTevColorOp   (tevstageid,GX_TEV_SUB,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVREG0);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	// (r0 * r0) * 2
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_ZERO,GX_CC_C0,GX_CC_C0,GX_CC_ZERO);
	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_CPREV,GX_CC_ZERO,GX_CC_ZERO,GX_CC_C0);
	GXSetTevColorOp   (tevstageid,GX_TEV_SUB,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVREG0);
//	GXSetTevColorOp   (tevstageid,GX_TEV_SUB,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	// 
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevKColorSel (tevstageid,GX_TEV_KCSEL_K0);
	GXSetTevColorIn   (tevstageid,GX_CC_TEXC,GX_CC_KONST,GX_CC_HALF,GX_CC_C0);
//	GXSetTevColorIn   (tevstageid,GX_CC_TEXC,GX_CC_HALF,GX_CC_HALF,GX_CC_C0);
//	GXSetTevColorOp   (tevstageid,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevColorOp   (tevstageid,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVREG0);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD0,SetupContrastTev_texmapID,GX_COLOR0A0);
	
//	tevstageid = (GXTevStageID)((int)tevstageid + 1);
//	GXSetTevColorIn   (tevstageid,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_CPREV);
//	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVPREV);
//	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
//	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
//	GXSetTevColorIn   (tevstageid,GX_CC_KONST,GX_CC_TEXC,GX_CC_TEXC,GX_CC_ZERO);
	GXSetTevColorIn   (tevstageid,GX_CC_HALF,GX_CC_TEXC,GX_CC_TEXC,GX_CC_ZERO);
//	GXSetTevColorOp   (tevstageid,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevColorOp   (tevstageid,GX_TEV_COMP_RGB8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVREG1);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD0,SetupContrastTev_texmapID,GX_COLOR0A0);
	
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_ZERO,GX_CC_C1,GX_CC_C1,GX_CC_ZERO);
	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVREG1);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
//	tevstageid = (GXTevStageID)((int)tevstageid + 1);
//	GXSetTevColorIn   (tevstageid,GX_CC_CPREV,GX_CC_ZERO,GX_CC_ZERO,GX_CC_C1);
//	GXSetTevColorOp   (tevstageid,GX_TEV_SUB,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVREG1);
//	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
//	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_C0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_C1);
	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_2,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
	tevstageid = (GXTevStageID)((int)tevstageid + 1);
	GXSetTevColorIn   (tevstageid,GX_CC_ZERO,GX_CC_CPREV,GX_CC_RASC,GX_CC_ZERO);
	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (tevstageid,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	
//static	bool	col = false;
//if( col==true ){
//	tevstageid = (GXTevStageID)((int)tevstageid + 1);
//	GXSetTevColorIn   (tevstageid,GX_CC_ZERO,GX_CC_CPREV,GX_CC_TEXC,GX_CC_ZERO);
//	GXSetTevColorOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevAlphaIn   (tevstageid,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
//	GXSetTevAlphaOp   (tevstageid,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
//	GXSetTevOrder     (tevstageid,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR_NULL);
//}
	
	GXSetNumTevStages ((u8)((int)tevstageid + 1));
	
	return true;
}

bool WK7SelectIF::RenderBack(void)
{
	if( mBackAlpha.GetValue()==0x00 ){
		return true;
	}
		
	if( mBackVisible==false || mpMonoColorScreenTexObj==NULL ){
		return true;
	}
	
#if 0
	// yNA
	
	GXClearVtxDesc    ();
	GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
	
	GXSetColorUpdate  (GX_FALSE);
	GXSetAlphaUpdate  (GX_TRUE);
	GXSetDstAlpha     (GX_TRUE,(u8)(0xff-mBackAlpha.GetValue()));
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
	
	GXSetZMode        (GX_TRUE,GX_ALWAYS,GX_TRUE);
	
	GXSetNumTevStages (1);
	GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC,GX_CC_ZERO);
	GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA,GX_CA_ZERO);
	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR_NULL);
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	
	GXBegin(GX_QUADS,GX_VTXFMT0,4);
	
		GXPosition3f32(0.0f            ,0.0f             ,mBackZ);
		GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mBackZ);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mBackZ);
		GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mBackZ);
		
	GXEnd();
#endif
	
		u32			backcolor;
	
		// wimg[\
		backcolor = (mBackTexColor&0xffffff00)|mBackAlpha.GetValue();
//		backcolor = 0xff000000|mBackAlpha.GetValue();
		
		GXSetDstAlpha     (GX_TRUE,(u8)(0xff-mBackAlpha.GetValue()));
		
		GXSetZMode        (GX_TRUE,GX_ALWAYS,GX_TRUE);
		GXSetColorUpdate  (GX_TRUE);
		GXSetAlphaUpdate  (GX_TRUE);
	//	GXSetAlphaUpdate  (GX_FALSE);
		
		GXClearVtxDesc    ();
		GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_NRM ,GX_NONE  );
		GXSetVtxDesc      (GX_VA_CLR0,GX_DIRECT);
		GXSetVtxDesc      (GX_VA_TEX0,GX_DIRECT);
		
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS ,GX_POS_XYZ ,GX_F32  ,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_CLR0,GX_CLR_RGBA,GX_RGBA8,0);
		GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_TEX0,GX_TEX_ST  ,GX_F32  ,0);
		
		GXSetNumTexGens   (1);
		GXSetTexCoordGen  (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
		
		SetupContrastTev();
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4);
		
			GXPosition3f32(0.0f            ,0.0f             ,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f - mBackScale));
			
			GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f - mBackScale));
			
			GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f + mBackScale));
			
			GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f + mBackScale));
			
		GXEnd();
		
		// v[[ƓGtJ[\
	static	GXColor		col = { 0x00,0x00,0x00,0x00 };
		
		col.a = WK7RenderObj::ConvRenderIDToAlpha(WK7RENDEROBJ_RENDERID_PLAYER);
	//	GXColor		col = { 0x00,0x00,0x00,0x08 };
		GXSetTevKColor   (GX_KCOLOR0,col);
		GXSetTevKAlphaSel(GX_TEVSTAGE0,GX_TEV_KASEL_K0_A);
		
		GXSetNumTevStages (1);
		
		GXSetTevColorIn   (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_TEXC);
		GXSetTevColorOp   (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevAlphaIn   (GX_TEVSTAGE0,GX_CA_TEXA,GX_CA_KONST,GX_CA_RASA,GX_CA_ZERO);
		GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_COMP_A8_GT,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	//	GXSetTevAlphaOp   (GX_TEVSTAGE0,GX_TEV_COMP_A8_EQ,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
		GXSetTevOrder     (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
		GXSetAlphaCompare (GX_GREATER,0x00,GX_AOP_AND,GX_ALWAYS,0xff);
		
		GXBegin(GX_QUADS,GX_VTXFMT0,4);
		
			GXPosition3f32(0.0f            ,0.0f             ,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f - mBackScale));
			
			GXPosition3f32(WK7_SCREEN_WIDTH,0.0f             ,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f - mBackScale));
			
			GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f + mBackScale),0.5f * (1.0f + mBackScale));
			
			GXPosition3f32(0.0f            ,WK7_SCREEN_HEIGHT,mBackZ);
			GXColor1u32   (backcolor);
			GXTexCoord2f32(0.5f * (1.0f - mBackScale),0.5f * (1.0f + mBackScale));
			
		GXEnd();
		
		GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);

	return true;
}

bool WK7SelectIF::RenderWindowMask(void)
{
	WK7SelectItem	*pItem;
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem != NULL ){
		pItem->RenderMask();
		pItem = pItem->GetNext();
	}
	
	return true;
}

bool WK7SelectIF::RenderWindowFrame(void)
{
	
//static bool	frame_visible = true;
//	
//	if( frame_visible==false ){
//		return true;
//	}
	
	WK7SelectItem	*pItem;
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem != NULL ){
		pItem->RenderLine(pItem->mRenderZ + 10.0f);
		pItem = pItem->GetNext();
	}
	
	u32			color;
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_NONE   );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);

	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
//		GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_INVDSTALPHA,GX_BL_DSTALPHA,GX_LO_CLEAR);
	
//	GXSetZMode       (GX_TRUE,GX_ALWAYS,GX_TRUE);
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetDither      (GX_TRUE);
	
	color = *((u32 *)&WK7SelectItem::mNormalColor);
	color |= 0xff;
//		color = 0xffffffff;
	
	f32		size = mLineSize * WK7_SCREEN_WIDTH / mMaskW;
	
	GXBegin(GX_TRIANGLESTRIP,GX_VTXFMT0,10);
		
	GXPosition3f32(0.0f,0.0f,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(size,size,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(WK7_SCREEN_WIDTH,0.0f,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(WK7_SCREEN_WIDTH-size,size,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(WK7_SCREEN_WIDTH-size,WK7_SCREEN_HEIGHT-size,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(0.0f,WK7_SCREEN_HEIGHT,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(size,WK7_SCREEN_HEIGHT-size,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(0.0f,0.0f,mWindowZ);
	GXColor1u32   (color);
	GXPosition3f32(size,size,mWindowZ);
	GXColor1u32   (color);
		
	GXEnd();
	
	return true;
}

bool WK7SelectIF::RenderWindowBack(void)
{
	
	u32		wincolor = 0xffffff00|mBackAlpha.GetValue();
	
	GXClearVtxDesc   ();
	GXSetVtxDesc     (GX_VA_POS , GX_DIRECT );
	GXSetVtxDesc     (GX_VA_NRM , GX_NONE   );
	GXSetVtxDesc     (GX_VA_CLR0, GX_DIRECT );
	GXSetVtxDesc     (GX_VA_TEX0, GX_DIRECT );
	
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_POS  , GX_POS_XYZ  , GX_F32   , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_CLR0 , GX_CLR_RGBA , GX_RGBA8 , 0);
	GXSetVtxAttrFmt  (GX_VTXFMT0 , GX_VA_TEX0 , GX_TEX_ST   , GX_F32   , 0);

	GXSetNumTexGens  (1);
	GXSetTexCoordGen (GX_TEXCOORD0,GX_TG_MTX2x4,GX_TG_TEX0,GX_IDENTITY);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP1,GX_COLOR0A0);
//	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD0,GX_TEXMAP0,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_TEXC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
	GXSetColorUpdate (GX_TRUE);
	GXSetAlphaUpdate (GX_FALSE);
//	GXSetBlendMode   (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetBlendMode   (GX_BM_BLEND,GX_BL_INVDSTALPHA,GX_BL_DSTALPHA,GX_LO_CLEAR);
//	GXSetBlendMode   (GX_BM_BLEND,GX_BL_DSTALPHA,GX_BL_INVDSTALPHA,GX_LO_CLEAR);
	
//	GXSetZMode       (GX_TRUE,GX_ALWAYS,GX_TRUE);
	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetDither      (GX_TRUE);
	
	GXBegin(GX_TRIANGLEFAN,GX_VTXFMT0,4);
	
	GXPosition3f32(0.0f,0.0f,mWindowZ);
	GXColor1u32   (wincolor);
	GXTexCoord2f32(0.0f,0.0f);
	GXPosition3f32(WK7_SCREEN_WIDTH,0.0f,mWindowZ);
	GXColor1u32   (wincolor);
	GXTexCoord2f32(1.0f,0.0f);
	GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,mWindowZ);
	GXColor1u32   (wincolor);
	GXTexCoord2f32(1.0f,1.0f);
	GXPosition3f32(0.0f,WK7_SCREEN_HEIGHT,mWindowZ);
	GXColor1u32   (wincolor);
	GXTexCoord2f32(0.0f,1.0f);
		
	GXEnd();
	
	return true;
}

bool WK7SelectIF::SetupWindowViewport(void)
{
#ifdef	DBGPAD
	
#define	LINESIZE_MOVE_SPEED		(0.125f)
	f32		pr = HSD_PadGetNormalAnalogR(1) * LINESIZE_MOVE_SPEED;
	f32		pl = HSD_PadGetNormalAnalogL(1) * LINESIZE_MOVE_SPEED;
	mLineSize += pr;
	mLineSize -= pl;
	if( mLineSize < 0.0f ){
		mLineSize = 0.0f;
	}
	
#if 1
	
#define	MASK_MOVE_SPEED		(16.0f)
	f32		px  = HSD_PadGetNormalStickX(1) * MASK_MOVE_SPEED;
	f32		py  = HSD_PadGetNormalStickY(1) * MASK_MOVE_SPEED;
	mMaskX += px;
	mMaskY -= py;
	
#else
	
	int			num = 0;
	f32			cx,cy;
	cx = 0.0f;
	cy = 0.0f;
	mMaskX = 0.0f;
	mMaskY = 0.0f;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( !pItem->mVisible || !pItem->mRender || !pItem->mEnable ){
			continue;
		}
		cx += pItem->mScreenTargetCenter.x;
		cy += pItem->mScreenTargetCenter.y;
		mMaskX -= pItem->mScreenTargetCenter.x;
		mMaskY -= pItem->mScreenTargetCenter.y;
//		mMaskX += pItem->mScreenTargetCenter.x - WK7_SCREEN_WIDTH  * 0.5f;
//		mMaskY += pItem->mScreenTargetCenter.y - WK7_SCREEN_HEIGHT * 0.5f;
		num++;
	}
	
	if( num > 0 ){
		mMaskX -= cx;
		mMaskY -= cy;
		cx /= num;
		cy /= num;
	}
	
	mMaskX += cx;
	mMaskY += cy;
//	mMaskX += (WK7_SCREEN_WIDTH  - mMaskW) * 0.5f;
//	mMaskY += (WK7_SCREEN_HEIGHT - mMaskH) * 0.5f;
	
#endif
	
#endif
	
	f32			c_mask_x,c_mask_y,c_mask_w,c_mask_h;
	
	c_mask_x = mMaskX;
	c_mask_y = mMaskY;
	c_mask_w = mMaskW;
	c_mask_h = mMaskH;
	
	if( c_mask_x + c_mask_w > WK7_SCREEN_WIDTH ){
		c_mask_w -= c_mask_x + c_mask_w - WK7_SCREEN_WIDTH;
	}
	if( c_mask_x > WK7_SCREEN_WIDTH ){
		c_mask_x = WK7_SCREEN_WIDTH;
		c_mask_w = 0.0f;
	}else if( c_mask_x < 0.0f ){
		c_mask_w += c_mask_x;
		c_mask_x  = 0.0f;
	}
	if( c_mask_y + c_mask_h > WK7_SCREEN_HEIGHT ){
		c_mask_h -= c_mask_y + c_mask_h - WK7_SCREEN_HEIGHT;
	}
	if( c_mask_y > WK7_SCREEN_HEIGHT ){
		c_mask_y = WK7_SCREEN_HEIGHT;
		c_mask_h = 0.0f;
	}else if( c_mask_y < 0.0f ){
		c_mask_h += c_mask_y;
		c_mask_y  = 0.0f;
	}
	
	GXSetScissor    ((u32)c_mask_x,(u32)c_mask_y,(u32)c_mask_w,(u32)c_mask_h);
	GXSetViewport   (mMaskX,mMaskY,mMaskW,mMaskH,0.0f,1.0f);
	GXSetDstAlpha   (GX_FALSE,0);
	
	return true;
}

bool WK7SelectIF::RenderWindow(void)
{
	if( mBackAlpha.GetValue()==0x00 ){
		return true;
	}
	
	SetupWindowViewport();
	
//	GXSetZMode       (GX_FALSE,GX_ALWAYS,GX_FALSE);
//	GXSetColorUpdate (GX_FALSE);
//	GXSetAlphaUpdate (GX_TRUE);
	RenderWindowMask();
	
	RenderWindowBack();
	
//	GXSetZMode       (GX_TRUE,GX_ALWAYS,GX_TRUE);
//	GXSetColorUpdate (GX_FALSE);
//	GXSetAlphaUpdate (GX_FALSE);
//	RenderWindowMask();
	
	RenderWindowFrame();
	
	GXSetScissor    (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport   (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	return true;
}

bool WK7SelectIF::AdjustColor(void)
{
	if( mainMenu.GetJunctionColorEdit()==false ){
		return true;
	}
static	u32					rgbid   = 0;
static	u32					paramid = 0;
		u32					rgb;
		s32					r,g,b,a,d;
	
	if( HSD_PadGetRepeat(1) & HSD_PAD_A ){
		paramid = (paramid+1) % 9;
	}
	
	switch( paramid ){
	case	0: rgb = WK7SELECTIF->GetBackTexColor();                  break;
	case	1: rgb = WK7SELECTIF->GetSelectBackColor();               break;
	case	2: rgb = WK7SELECTIF->GetSelectBackDeltaColor();          break;
	case	3: rgb = WK7SELECTIF->GetHighLightedBackColor();          break;
	case	4: rgb = WK7SELECTIF->GetHighLightedBackDeltaColor();     break;
	case	5: rgb = WK7SELECTIF->GetSemiHighLightedBackColor();      break;
	case	6: rgb = WK7SELECTIF->GetSemiHighLightedBackDeltaColor(); break;
	case	7: rgb = WK7SELECTIF->GetSelectedFrameColor();            break;
	case	8: rgb = WK7SELECTIF->GetCinemaScopeColor();              break;
	}
	
	r   = (s32)(rgb >> 24);
	g   = (s32)(rgb >> 16)& 0xff;
	b   = (s32)(rgb >>  8)& 0xff;
	a   = (s32)(rgb & 0xff);
	
	if( HSD_PadGetButton(1) & HSD_PAD_B ){
		d = 4;
	}else{
		d = 1;
	}
	if( HSD_PadGetRepeat(1) & (HSD_PAD_UP|HSD_PAD_STICK_UP) ){
		if( rgbid==0 ){
			rgbid = 2;
		}else{
			rgbid--;
		}
	}
	if( HSD_PadGetRepeat(1) & (HSD_PAD_DOWN|HSD_PAD_STICK_DOWN) ){
		if( rgbid==2 ){
			rgbid = 0;
		}else{
			rgbid++;
		}
	}
	
	switch( rgbid ){
	case	0:
		if( HSD_PadGetRepeat(1) & (HSD_PAD_RIGHT|HSD_PAD_STICK_RIGHT) ){
			r += d;
		}
		if( HSD_PadGetRepeat(1) & (HSD_PAD_LEFT |HSD_PAD_STICK_LEFT ) ){
			r -= d;
		}
		break;
	case	1:
		if( HSD_PadGetRepeat(1) & (HSD_PAD_RIGHT|HSD_PAD_STICK_RIGHT) ){
			g += d;
		}
		if( HSD_PadGetRepeat(1) & (HSD_PAD_LEFT |HSD_PAD_STICK_LEFT ) ){
			g -= d;
		}
		break;
	case	2:
		if( HSD_PadGetRepeat(1) & (HSD_PAD_RIGHT|HSD_PAD_STICK_RIGHT) ){
			b += d;
		}
		if( HSD_PadGetRepeat(1) & (HSD_PAD_LEFT |HSD_PAD_STICK_LEFT ) ){
			b -= d;
		}
		break;
	}
	
	GHM_LIMIT_RANGE(r,255,0);
	GHM_LIMIT_RANGE(g,255,0);
	GHM_LIMIT_RANGE(b,255,0);
	
	rgb = (u32)((r<<24)|(g<<16)|(b<<8)|a);
	
	switch( paramid ){
	case	0: WK7SELECTIF->SetBackTexColor(rgb);                  break;
	case	1: WK7SELECTIF->SetSelectBackColor(rgb);               break;
	case	2: WK7SELECTIF->SetSelectBackDeltaColor(rgb);          break;
	case	3: WK7SELECTIF->SetHighLightedBackColor(rgb);          break;
	case	4: WK7SELECTIF->SetHighLightedBackDeltaColor(rgb);     break;
	case	5: WK7SELECTIF->SetSemiHighLightedBackColor(rgb);      break;
	case	6: WK7SELECTIF->SetSemiHighLightedBackDeltaColor(rgb); break;
	case	7: WK7SELECTIF->SetSelectedFrameColor(rgb);            break;
	case	8: WK7SELECTIF->SetCinemaScopeColor(rgb);              break;
	}
	
	C_Paper			paper(32,360);
	
	switch( paramid ){
	case	0: PRINTF("Back Color\n");                 break;
	case	1: PRINTF("Select Color\n");               break;
	case	2: PRINTF("Select Delta Color\n");         break;
	case	3: PRINTF("HighLight Color\n");            break;
	case	4: PRINTF("HighLight Delta Color\n");      break;
	case	5: PRINTF("Semi HighLight Color\n");       break;
	case	6: PRINTF("Semi HighLight Delta Color\n"); break;
	case	7: PRINTF("Selected Frame Color\n");       break;
	case	8: PRINTF("Cinema Scope Color\n");         break;
	}
	
	PRINTF("%c R : %02x(%3d)\n",rgbid==0? '>':' ',r,r);
	PRINTF("%c G : %02x(%3d)\n",rgbid==1? '>':' ',g,g);
	PRINTF("%c B : %02x(%3d)\n",rgbid==2? '>':' ',b,b);
	
	return true;
}

bool WK7SelectIF::SetupRenderProcess(void)
{
	Mtx				view;
	Mtx44			proj;
	
	WK7Cam::Set2DMatrix(view,proj,0.0f,1024.0f);
	
	mpFullColorScreenTexObj = screenImageFullColor.GetTexObj();
	mpMonoColorScreenTexObj = screenImageMonoColor.GetTexObj();
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm (view,GX_PNMTX0);
	GXLoadNrmMtxImm (view,GX_PNMTX0);
	GXSetCurrentMtx (GX_PNMTX0);
	GXSetProjection (proj,GX_ORTHOGRAPHIC);
	
	HSD_RectU16		sc_rect;
	WK7CAM->GetScissor(sc_rect);
	GXSetScissor    (sc_rect.xmin,sc_rect.ymin,(u32)(sc_rect.xmax-sc_rect.xmin),(u32)(sc_rect.ymax-sc_rect.ymin));
	GXSetViewport   (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	if( mpFullColorScreenTexObj != NULL ){
		GXLoadTexObj(mpFullColorScreenTexObj,GX_TEXMAP0);
	}
	if( mpMonoColorScreenTexObj != NULL ){
		GXLoadTexObj(mpMonoColorScreenTexObj,GX_TEXMAP1);
	}
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode     (GX_CULL_NONE);
	GXSetDither       (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	
	WK7SelectItem	*pItem = (WK7SelectItem *)mAnchor.GetNext();
	f32				z = mBackZ + 50.0f;
	
	mWindowZ     = z; z += 50.0f;
	mWindowMaskZ = z;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext(),z += 50.0f ; pItem != NULL ; pItem=pItem->GetNext(),z += 50.0f){
		pItem->mRenderZ = z;
		pItem->SetupRenderProcess();
	}
//	mWindowZ     = z; z += 50.0f;
//	mWindowMaskZ = z;
	
	return true;
}

static	f32		cinemaScopeSize = 64.0f;
bool WK7SelectIF::RenderCinemaScope(void)
{
	Mtx				view;
	Mtx44			proj;
	
	WK7Cam::Set2DMatrix(view,proj,0.0f,1024.0f);
	
	// [Jr[ϊ}gNXݒ
	GXLoadPosMtxImm   (view,GX_PNMTX0);
	GXLoadNrmMtxImm   (view,GX_PNMTX0);
	GXSetCurrentMtx   (GX_PNMTX0);
	GXSetProjection   (proj,GX_ORTHOGRAPHIC);
	
	GXSetScissor      (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport     (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	GXSetAlphaCompare (GX_ALWAYS,0xff,GX_AOP_OR,GX_ALWAYS,0xff);
	GXSetNumChans     (1);
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_REG,GX_SRC_REG,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	GXSetBlendMode    (GX_BM_BLEND,GX_BL_SRCALPHA,GX_BL_INVSRCALPHA,GX_LO_CLEAR);
	GXSetCullMode     (GX_CULL_NONE);
	GXSetDither       (GX_FALSE);
	GXSetDstAlpha     (GX_FALSE,0);
	// VlXRORgɁ
	GXSetZMode        (GX_FALSE,GX_ALWAYS,GX_FALSE);
	GXSetColorUpdate  (GX_TRUE);
	GXSetAlphaUpdate  (GX_FALSE);
	
	GXClearVtxDesc    ();
	GXSetVtxDesc      (GX_VA_POS ,GX_DIRECT);
	GXSetVtxAttrFmt   (GX_VTXFMT0,GX_VA_POS,GX_POS_XYZ,GX_F32,0);
	
	GXSetNumTexGens  (0);
	
	GXSetNumTevStages(1);
	GXSetTevOrder    (GX_TEVSTAGE0,GX_TEXCOORD_NULL,GX_TEXMAP_NULL,GX_COLOR0A0);
	GXSetTevColorIn  (GX_TEVSTAGE0,GX_CC_ZERO,GX_CC_ZERO,GX_CC_ZERO,GX_CC_RASC);
	GXSetTevColorOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	GXSetTevAlphaIn  (GX_TEVSTAGE0,GX_CA_ZERO,GX_CA_ZERO,GX_CA_ZERO,GX_CA_RASA);
	GXSetTevAlphaOp  (GX_TEVSTAGE0,GX_TEV_ADD,GX_TB_ZERO,GX_CS_SCALE_1,GX_TRUE,GX_TEVPREV);
	
#define		CINEMA_SCOPE_Z	(1000.0f)
//#define		CINEMA_SCOPE_Z	(1.0f)
//static	u32	CINEMA_SCOPE_COLOR = (0x00000000);
//static	u32	CINEMA_SCOPE_COLOR = (0x000000ff);
static	f32	z = CINEMA_SCOPE_Z;
	
//	GXSetZMode        (GX_TRUE,GX_LEQUAL,GX_TRUE);
//	GXSetZMode        (GX_TRUE,GX_GEQUAL,GX_TRUE);
//	z = mWindowZ - 10.0f;
	
	f32		ty,dy;
	
	ty = cinemaScopeSize * mBackAlpha.GetValue() / 255.0f;
	dy = WK7_SCREEN_HEIGHT - ty;
	
	GXSetChanMatColor (GX_COLOR0A0,*((GXColor *)&mCinemaScopeColor));
//	GXSetChanMatColor (GX_COLOR0A0,*((GXColor *)&CINEMA_SCOPE_COLOR));
	
	GXBegin(GX_QUADS,GX_VTXFMT0,8);
	
		GXPosition3f32(            0.0f,             0.0f,z);
		GXPosition3f32(WK7_SCREEN_WIDTH,             0.0f,z);
		GXPosition3f32(WK7_SCREEN_WIDTH,               ty,z);
		GXPosition3f32(            0.0f,               ty,z);
		
		GXPosition3f32(            0.0f,               dy,z);
		GXPosition3f32(WK7_SCREEN_WIDTH,               dy,z);
		GXPosition3f32(WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,z);
		GXPosition3f32(            0.0f,WK7_SCREEN_HEIGHT,z);
		
	GXEnd();
	
	GXSetChanCtrl     (GX_COLOR0A0,GX_FALSE,GX_SRC_VTX,GX_SRC_VTX,GX_LIGHT_NULL,GX_DF_NONE,GX_AF_NONE);
	
	return true;
}

bool WK7SelectIF::TermRenderProcess(void)
{
	GXSetScissor    (0,0,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT);
	GXSetViewport   (0.0f,0.0f,WK7_SCREEN_WIDTH,WK7_SCREEN_HEIGHT,0.0f,1.0f);
	
	mpFullColorScreenTexObj = NULL;
	mpMonoColorScreenTexObj = NULL;
	return true;
}

bool WK7SelectIF::RenderBackProcess(void)
{
	WK7SelectItem	*pItem;
	int				visible = 0;
	
	mRender = 0;
	
#ifdef	_DEBUG
	AdjustColor();
#endif
	
	if( !mVisible ){
		return true;
	}
	
	if( mBackAlpha.GetValue()!=0x00 ){
		visible++;
	}
	
	pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem != NULL ){
		if( pItem->GetVisible()==true ){
			visible++;
		}
		pItem = pItem->GetNext();
	}
	
	if( visible==0 ){
		return true;
	}
	
	mRender = 1;
	
	SetupRenderProcess();
	
	RenderBack();
	
	return true;
}

bool WK7SelectIF::RenderProcess(void)
{
	WK7SelectItem	*pItem;
//	C_Paper			paper(32,32);
	
	RenderBackProcess();
	
	if( !mRender ){
		return true;
	}
	mRender = 0;
	
	RenderCinemaScope();
	
	RenderWindow();
	
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		pItem->RenderProcess();
	}
	
//	RenderCinemaScope();
	
	TermRenderProcess();
	
	return true;
}

bool WK7SelectIF::SelectItem(WK7SelectItem *pSelectItem,u32 tick)
{
	WK7SelectItem		*pItem = (WK7SelectItem *)mAnchor.GetNext();
	while( pItem!=NULL ){
		if( pItem==pSelectItem ){
			if( pItem->Select(tick)==false ){
				pItem->Disappear(tick);
			}
		}else{
			pItem->Disappear(tick);
		}
		pItem = pItem->GetNext();
	}
	selectedPadRelease = false;
	mBackAlpha.Play(0x00,tick);
	mState = DISAPPEAR;
	mTypoVisible = false;
	AllocInit();
//	GameInfo.InitOptionWindow();
//	GameInfo.OptionDemoClose();
	return true;
}

bool WK7SelectIF::ReleaseItemAll(void)
{
	WK7SelectItem		*pItem,*pNext;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pNext){
		pNext = pItem->GetNext();
		delete pItem;
	}
	return true;
}

bool WK7SelectIF::AppearTypo(u32 tick)
{
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetEnable()==false ){
			continue;
		}
		pItem->AppearTypo(tick);
	}
	mTypoVisible = true;
	return true;
}

bool WK7SelectIF::Appear(u32 tick)
{
	WK7SelectItem		*pItem = (WK7SelectItem *)mAnchor.GetNext();
	bool				normalSE = false;
	bool				eventSE  = false;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem = pItem->GetNext()){
		if( pItem->GetEnable()==false ){
			continue;
		}
		switch( pItem->GetType() ){
		case	WK7SelectItem::TYPE_CUT:
			normalSE = true;
			break;
		case	WK7SelectItem::TYPE_LINE:
			eventSE  = true;
			break;
		default:
			break;
		}
		pItem->Appear(tick);
	}
	
	if( normalSE==true ){
		gpSound->mPlaySE(SFX_SYS_HOUKOU_DERU);
	}
	if( eventSE==true ){
		gpSound->mPlaySE(SFX_SYS_HOUKOU_DERU_EVENT);
	}
	
	AppearTypo();
	
	mBackAlpha.Play(0xff,tick);
	mState = APPEAR;
	
	return true;
}

bool WK7SelectIF::Disappear(u32 tick,WK7SelectItem *pExceptItem)
{
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem = pItem->GetNext()){
		if( pItem==pExceptItem ){
			continue;
		}
		if( pItem->GetEnable()==false ){
			continue;
		}
		pItem->Disappear(tick);
	}
	pOldSelectedItem = NULL;
	mBackAlpha.Play(0x00,tick);
	mState = DISAPPEAR;
	AllocInit();
	wK7SelectItemRouteID = 0;
	mTypoVisible = false;
	return true;
}

void WK7SelectIF::SetDeformRateAll(f32 rate)
{
	if( rate < 0.0f ){
		rate = 0.0f;
	}else if( rate > 1.0f ){
		rate = 1.0f;
	}
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem = pItem->GetNext()){
		pItem->SetDeformRate(rate);
	}
}

#include "..\Fujita\C_Font.h"

Vec	WK7SelectDirControl::mDirection[8] = 
{
	{  1.0f ,  0.0f ,  0.0f },		// 
	{  0.8f ,  0.6f ,  0.0f },		// 
	{  0.0f ,  1.0f ,  0.0f },		// 
	{ -0.8f ,  0.6f ,  0.0f },		// E
	{ -1.0f ,  0.0f ,  0.0f },		// E
	{ -0.8f , -0.6f ,  0.0f },		// E
	{  0.0f , -1.0f ,  0.0f },		// 
	{  0.8f , -0.6f ,  0.0f },		// 
};

Vec	WK7SelectDirControl::mPosition [8] = 
{
	{  0.0f                                   ,  WK7_SCREEN_HEIGHT/2.0f                  ,  0.0f },
	{  0.0f                                   ,  0.0f                                    ,  0.0f },
	{  WK7_SCREEN_WIDTH/2.0f                  ,  0.0f                                    ,  0.0f },
	{  WK7_SCREEN_WIDTH                       ,  0.0f                                    ,  0.0f },
	{  WK7_SCREEN_WIDTH                       ,  WK7_SCREEN_HEIGHT/2.0f                  ,  0.0f },
	{  WK7_SCREEN_WIDTH                       ,  WK7_SCREEN_HEIGHT                       ,  0.0f },
	{  WK7_SCREEN_WIDTH/2.0f                  ,  WK7_SCREEN_HEIGHT                       ,  0.0f },
	{  0.0f                                   ,  WK7_SCREEN_HEIGHT                       ,  0.0f },
};


WK7SelectDirControl::WK7SelectDirControl()
{
	Initialize();
}

WK7SelectDirControl::~WK7SelectDirControl()
{
	
}

void WK7SelectDirControl::Initialize(void)
{
	mUseNum = 0;
	memset(mFlag,0,sizeof(mFlag));
}

bool WK7SelectDirControl::GetPosition(int dir,Vec *pPosition)
{
	if( dir < 0 || dir >= 8 || pPosition==NULL ){
		return false;
	}
	pPosition->x = mPosition[dir].x;
	pPosition->y = mPosition[dir].y;
	pPosition->z = mPosition[dir].z;
	return true;
}

bool WK7SelectDirControl::GetDirection(int dir,Vec *pDirection)
{
	if( dir < 0 || dir >= 8 || pDirection==NULL ){
		return false;
	}
	pDirection->x = mDirection[dir].x;
	pDirection->y = mDirection[dir].y;
	pDirection->z = mDirection[dir].z;
	return true;
}

bool WK7SelectDirControl::FreeDirection(int dir)
{
	if( dir < 0 || dir >=8 ){
		return false;
	}
	if( !mFlag[dir] ){
		return false;
	}
	mUseNum--;
	mFlag[dir] = 0;
	return true;
}

int WK7SelectDirControl::AllocDirection(const Vec *pTargetPoint,const Vec *pBasePoint)
{
	if( mUseNum >= 8 ){
		return -1;
	}
	
	Vec		tdir,odir;
	int		i,dir = -1;
	f32		ip,op,ipmin,invlen;
	f32		top;
	int		flag;
	
	tdir.x = pTargetPoint->x - pBasePoint->x;
	tdir.y = pTargetPoint->y - pBasePoint->y;
	op     = 1.0f / sqrtf(tdir.x *tdir.x + tdir.y * tdir.y);
	tdir.x *= op;
	tdir.y *= op;
	
	ipmin = -1.0f;
	for(i=0 ; i<8 ; i++){
		odir.x = mPosition[i].x - pBasePoint->x;
		odir.y = mPosition[i].y - pBasePoint->y;
		invlen = 1.0f / sqrtf(odir.x * odir.x + odir.y * odir.y);
		odir.x *= invlen;
		odir.y *= invlen;
		top    = odir.x * tdir.y - odir.y * tdir.x;
		top    = fabs(top);
		invlen = 1.0f / sqrtf(tdir.x * tdir.x + tdir.y * tdir.y);
		if( dir < 0 ){
			ip = odir.x * tdir.x + odir.y * tdir.y;
			ip *= invlen;
			if( ip > 0.0f || ip > ipmin ){
				op = top;
				dir = i;
				ipmin = ip;
			}
		}else if( top < op ){
			ip = odir.x * tdir.x + odir.y * tdir.y;
			ip *= invlen;
			if( ip > 0.0f || ip > ipmin ){
				op = top;
				dir = i;
				ipmin = ip;
			}
		}
	}
	
	flag = 1;
	
	for(i=1 ; i<=8 ; i++){
		
		if( !mFlag[dir] ){
			mFlag[dir] = 1;
			mUseNum++;
			return dir;
		}
		
		dir  += i * flag;
		dir  &= 7;
		flag  = -flag;
		
	}
	
	return -1;
}


bool WK7SelectIF::AllocInit(void)
{
	mSelectDirControl.Initialize();
	return true;
}

bool WK7SelectIF::CreateNextBranchSelectItem(WK7PathInfo *pInfo,bool pathselect)
{
	if( pInfo==NULL ){
		return false;
	}
	
	WK7PathBranch	*pBranch;
	WK7PathNurbs	*pCurrPath = pInfo->GetPath();
	WK7PathNurbs	*pNextPath;
	
	if( pathselect==true ){
		bool			backpath = false;
		if( pCurrPath->GetStageJump()==true ){
			if( pInfo->GetNextBranchPoint()==NULL ){
//			if( pInfo->GetRestDistance()==pInfo->GetTotalDistance() ){
				backpath = true;
			}
		}
		
		if( backpath==true ){
			WK7PathBranchPoint	*pPrevBranchPoint = pInfo->GetPrevBranchPoint();
			if( pPrevBranchPoint==NULL || pCurrPath->GetEnableEvent()==false ){
				return false;
			}
			for(pBranch=pPrevBranchPoint->GetTopBranch() ; pBranch!=NULL ; pBranch=pBranch->GetNext() ){
				if( (pNextPath=pBranch->GetPath())==pCurrPath ){
					CreateBranchSelectItem(pPrevBranchPoint,pBranch);
					WK7Stage::SetBpCamInfo(pPrevBranchPoint);
					return true;
				}
			}
			return false;
		}
	}
	
	WK7PathBranchPoint	*pNextBranchPoint = pInfo->GetNextBranchPoint();
	if( pNextBranchPoint==NULL ){
		return false;
	}
	
	WK7Stage::SetBpCamInfo(pNextBranchPoint);
	
	int				bcnt,ocnt,pcnt;
	for(pBranch=pNextBranchPoint->GetTopBranch(),bcnt=0,pcnt=0 ; pBranch!=NULL ; pBranch=pBranch->GetNext() ){
		if( (pNextPath=pBranch->GetPath())==pCurrPath ){
			continue;
		}
		if( pNextPath->GetEnable()==false ){
			continue;
		}
		if( pNextPath->GetStageJump()==true ){
			if( pNextPath->GetNextStageID()!=0xffffffff && pNextPath->GetEnableEvent()==true ){
				pcnt++;
			}
			continue;
		}
//		if( pNextPath->GetEnableEx()==false ){
//			continue;
//		}
		bcnt++;
	}
	WK7PathBranchPointObj	*pObj;
	for(pObj=pNextBranchPoint->GetTopObj(),ocnt=0 ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetEnable()==false ){
			continue;
		}
		ocnt++;
	}
	
	if( bcnt<=1 && ocnt<=0 && pcnt<=0 ){
		return false;
	}
	
	// _ݒ
	pNextBranchPoint->GetPosition(&WK7SelectItem::mWorldItemCenter);
	WK7SelectItem::mWorldItemCenter.y += tgth;
	
	for(pBranch=pNextBranchPoint->GetTopBranch() ; pBranch!=NULL ; pBranch=pBranch->GetNext() ){
		if( (pNextPath=pBranch->GetPath())==pCurrPath ){
			continue;
		}
		if( pNextPath==NULL ){
			continue;
		}
		if( pNextPath->GetEnable()==true ){
			if( pNextPath->GetStageJump()==false ){
				CreateBranchSelectItem(pNextBranchPoint,pBranch);
			}else{
				if( pNextPath->GetNextStageID()!=0xffffffff && pNextPath->GetEnableEvent()==true ){
					CreateBranchSelectItem(pNextBranchPoint,pBranch);
				}
			}
		}
	}
	
	for(pObj=pNextBranchPoint->GetTopObj() ; pObj!=NULL ; pObj=pObj->GetNext()){
		if( pObj->GetEnable()==false ){
			continue;
		}
		CreateBranchSelectItem(pObj);
	}
	
//	WK7Stage::SetBpCamInfo(pNextBranchPoint);
	
	return true;
}

static	const char *pDirNameTable[] = {
	"N",
	"NNW",
	"NW",
	"WNW",
	"W",
	"WSW",
	"SW",
	"SSW",
	"S",
	"SSE",
	"SE",
	"ESE",
	"E",
	"ENE",
	"NE",
	"NNE",
};
#define	WK7SELECTIF_ROUTENAME_MAX		(256)

int WK7SelectIF::CalcDir(WK7PathBranch *pBranch)
{
	if( pBranch==NULL ){
		return -1;
	}
	Vec		dir;
	f32		ang;
	int		dir_index;
	pBranch->GetStartDeltaPosition(&dir);
	VECNormalize(&dir,&dir);
	ang = atan2f(dir.x,dir.z) + ((PI * 17.0f) / 16.0f);
	ang += WK7SelectIF::GetStageDirOffset();
	dir_index = (int)(ang / (TWO_PI / 16.0f));
	dir_index &= 16-1;
	return dir_index;
}

const char* WK7SelectIF::GetRouteName(char *pRouteName,const char *pOrgName,WK7PathBranch *pBranch)
{
	int				i;
	const char		*pDirName;
	const char		*psrc;
	char			*pdst;
	psrc = pOrgName;
	pdst = pRouteName;
	for(i=0 ; i<WK7SELECTIF_ROUTENAME_MAX-1 ; i++){
		if( *psrc=='\0' ){
			break;
		}
		if( *psrc=='|' ){
			if( psrc[1]!='|' ){
				break;
			}
			psrc++;
		}
		*pdst++ = *psrc++;
	}
	*pdst = '\0';
	if( *psrc=='|' ){
		for(psrc++ ; *psrc!='\0' ; psrc++){
			switch( *psrc ){
			case	'L':
			case	'l':
				if( pBranch->GetDir() ){
					pDirName = "counterclockwise";
				}else{
					pDirName = "clockwise";
				}
				break;
			default:
				break;
			}
		}
	}else{
		int		dir_index = CalcDir(pBranch);
		if( dir_index < 0 ){
			pDirName = pDirNameTable[0];
		}else{
			pDirName = pDirNameTable[dir_index];
		}
	}
	return pDirName;
}

bool WK7SelectIF::CreateBranchSelectItem(WK7PathBranchPoint *pBranchPoint,WK7PathBranch *pBranch)
{
	ghmGcMemSetAllocName("SIFI");
	WK7SelectItem	*pItem = new WK7SelectItem(&mAnchor);
	
	if( pItem==NULL ){
		return false;
	}
	
	WK7PathNurbs	*pPath = pBranch->GetPath();
	ghmScenePathSet	*pPathSet;
	WK7PathDataDesc	*pPathDataDesc;
	
	mItemNum++;
	
	pItem->SetPathBranch(pBranch);
	pItem->SetParam0    (0xffffffff);
	
	if( pBranch->HasSelected()==true ){
		pItem->SetHighLighted    (true );
		pItem->SetSemiHighLighted(false);
	}else if( pBranch->HasPassed(true)==true ){
		pItem->SetHighLighted    (true );
		pItem->SetSemiHighLighted(true );
	}else{
		pItem->SetHighLighted    (false);
		pItem->SetSemiHighLighted(false);
	}
//	if( pPath->HasPassed()==true ){
//		pItem->SetHighLighted(true );
//		if( pBranch->HasPassed()==false ){
//			pItem->SetSemiHighLighted(true );
//		}else{
//			pItem->SetSemiHighLighted(false);
//		}
//	}else{
//		pItem->SetHighLighted(false);
//	}
	
	pPathDataDesc = NULL;
	if( (pPathSet = pPath->GetPathSet())!=NULL ){
		pPathDataDesc = (WK7PathDataDesc *)pPathSet->mpData;
	}
	
	char			routeName[256];
	const char		*pRouteName,*pDirName;
	u32				icon = (u32)TF_MAX;
	
	pDirName   = NULL;
	if( pPathDataDesc!=NULL && pPathDataDesc->mpRouteName!=NULL ){
		pRouteName = pPathDataDesc->mpRouteName;
	}else{
		pRouteName = "===";
	}
	
	if( pPath->GetNextStageID()!=0xffffffff ){
		icon = (u32)TF_DOOR;
		pItem->SetType(WK7SelectItem::TYPE_LINE);
	}else{
		if( pRouteName != NULL ){
			pDirName = GetRouteName(routeName,pRouteName,pBranch);
			pRouteName = routeName;
			pDirName = NULL;
			int		dir_index = CalcDir(pBranch);
			switch( dir_index ){
			case	WK7SelectItem::DIR_N:   icon = (u32)TF_N;   break;
			case	WK7SelectItem::DIR_NNW: icon = (u32)TF_NNW; break;
			case	WK7SelectItem::DIR_NW:  icon = (u32)TF_NW;  break;
			case	WK7SelectItem::DIR_WNW: icon = (u32)TF_WNW; break;
			case	WK7SelectItem::DIR_W:   icon = (u32)TF_W;   break;
			case	WK7SelectItem::DIR_WSW: icon = (u32)TF_WSW; break;
			case	WK7SelectItem::DIR_SW:  icon = (u32)TF_SW;  break;
			case	WK7SelectItem::DIR_SSW: icon = (u32)TF_SSW; break;
			case	WK7SelectItem::DIR_S:   icon = (u32)TF_S;   break;
			case	WK7SelectItem::DIR_SSE: icon = (u32)TF_SSE; break;
			case	WK7SelectItem::DIR_SE:  icon = (u32)TF_SE;  break;
			case	WK7SelectItem::DIR_ESE: icon = (u32)TF_ESE; break;
			case	WK7SelectItem::DIR_E:   icon = (u32)TF_E;   break;
			case	WK7SelectItem::DIR_ENE: icon = (u32)TF_ENE; break;
			case	WK7SelectItem::DIR_NE:  icon = (u32)TF_NE;  break;
			case	WK7SelectItem::DIR_NNE: icon = (u32)TF_NNE; break;
			default:
				break;
			}
		}
	}
	
	pItem->SetTypo(pRouteName,pDirName,icon);
	
	Vec			gpos[2];
	if( pBranch->GetDir() ){
		pPath->GetEndPoint(gpos[0]);
	}else{
		pPath->GetStartPoint(gpos[0]);
	}
	gpos[1].x = gpos[0].x;
	gpos[1].y = gpos[0].y + 10.0f;
	gpos[1].z = gpos[0].z;
	pItem->SetTargetPoints(gpos,sizeof(gpos) / sizeof(Vec));
	
	Vec					pos,dir;
	HSD_CObj			*pCObj;
	Mtx					view;
	f32					pm[GX_PROJECTION_SZ];
	f32					vp[GX_VIEWPORT_SZ  ];
	Vec					bp,sbp;					// _
	const Vec			*pTargetPoint;
	int					i,targetPointNum;
	f32					invnum;
	
	pBranchPoint->GetPosition(&bp);
	
	WK7CAM->SetCurrent();
	if( (pCObj = HSD_CObjGetCurrent())==NULL ){
		MTXIdentity(view);
	}else{
		HSD_CObjGetViewingMtx(pCObj,view);
	}
	
	GXGetProjectionv(pm);
	GXGetViewportv  (vp);
	
	pTargetPoint   = pItem->GetTargetPoint();
	targetPointNum = pItem->GetTargetPointNum();
	
	pos.x = 0.0f;
	pos.y = 0.0f;
	pos.z = 0.0f;
	for(i=0 ; i<targetPointNum ; i++){
		GXProject(pTargetPoint->x,
		          pTargetPoint->y,
		          pTargetPoint->z,
		          view,
		          pm,
		          vp,
		          &dir.x,
		          &dir.y,
		          &dir.z);
		pos.x += dir.x;
		pos.y += dir.y;
		pos.z += dir.z;
	}
	invnum  = 1.0f / targetPointNum;
	pos.x  *= invnum;
	pos.y  *= invnum;
	pos.z  *= invnum;
	dir.x   = pos.x - (WK7_SCREEN_WIDTH /2.0f);
	dir.y   = pos.y - (WK7_SCREEN_HEIGHT/2.0f);
	dir.z   = 0.0f;
	invnum  = 1.0f / sqrtf(dir.x * dir.x + dir.y * dir.y);
	dir.x  *= invnum;
	dir.y  *= invnum;
	dir.z  *= invnum;
	
	GXProject(bp.x,bp.y,bp.z,view,pm,vp,&sbp.x,&sbp.y,&sbp.z);
	
	bp.x = pos.x;
	bp.y = pos.y;
	bp.z = pos.z;
	
	int		direction = mSelectDirControl.AllocDirection(&bp,&sbp);
	WK7SelectDirControl::GetPosition (direction,&pos);
	WK7SelectDirControl::GetDirection(direction,&dir);
	
	pItem->SetCutInParam(&pos,&dir);
	pItem->Appear();
	pItem->SetVisible(true);
	pItem->SetDirection(direction);
	
	return true;
}

bool WK7SelectIF::CreateBranchSelectItem(WK7PathBranchPointObj *pBranchPointObj)
{
	ghmGcMemSetAllocName("SIFI");
	WK7SelectItem	*pItem = new WK7SelectItem(&mAnchor);
	
	if( pItem==NULL ){
		return false;
	}
	
	const char		*pObjName[2];
	u32				icon;
	
	if( (pObjName[0]=pBranchPointObj->GetObjName(0))==NULL ){
		pObjName[0] = "nameless";
	}
	if( (pObjName[1]=pBranchPointObj->GetObjName(1))==NULL ){
		pObjName[1] = "Bp Object";
	}
	
	mItemNum++;
	
	switch( pBranchPointObj->GetType() ){
	case	WK7PathBranchPointObj::TYPE_JOBJ:
	case	WK7PathBranchPointObj::TYPE_JOBJDOOR:{
		WK7PathBranchPointObjJObj	*pBranchPointObjJObj;
		u32							evno;
		pBranchPointObjJObj = (WK7PathBranchPointObjJObj *)pBranchPointObj;
		evno                = pBranchPointObjJObj->GetEventNo();
		if( evno!=0xffffffff ){
			switch( pBranchPointObjJObj->GetEventNo() & K7EV_TYPE_MASK ){
			case	K7EV_TYPE_DOOR:  icon = (u32)TF_DOOR;  break;
			case	K7EV_TYPE_NO:    icon = (u32)TF_EVENT; break;
			case	K7EV_TYPE_KEY:   icon = (u32)TF_KEY;   break;
			case	K7EV_TYPE_MAP:   icon = (u32)TF_MAP;   break;
			case	K7EV_TYPE_MEMO:  icon = (u32)TF_MEMO;  break;
			case	K7EV_TYPE_VMEMO: icon = (u32)TF_VREC;  break;
			case	K7EV_TYPE_TV:
			case	K7EV_TYPE_LTV:   icon = (u32)TF_TV;    break;
			case	K7EV_TYPE_PLAY:  icon = (u32)TF_EVENT; break;
			case	K7EV_TYPE_RAKU:  icon = (u32)TF_GRFF;  break;
			default:                 icon = (u32)TF_MAX;   break;
			}
		}else{
			u32	evid = pBranchPointObj->GetEventID();
			if( evid==0xffffffff ){
				icon = (u32)TF_MAX;
			}else if( evid==WK7Stage::GetDefBranchMoveEventID() ){
				icon = (u32)TF_DOOR;
			}else if( evid==WK7Stage::GetDefTvEventID() ){
				icon = (u32)TF_TV;
			}else if( evid==WK7Stage::GetDefLtvEventID() ){
				icon = (u32)TF_TV;
			}else if( evid==WK7Stage::GetDefDoorEventID() ){
				icon = (u32)TF_DOOR;
			}else if( evid==WK7Stage::GetDefKeyEventID() ){
				icon = (u32)TF_KEY;
			}else if( evid==WK7Stage::GetDefMapEventID() ){
				icon = (u32)TF_MAP;
			}else if( evid==WK7Stage::GetDefMemoEventID() ){
				icon = (u32)TF_MEMO;
			}else{
				icon = (u32)TF_MAX;
			}
		}
		}break;
	default:
		icon = (u32)TF_MAX;
		break;
	}
	
	pItem->SetPathBranchPointObj(pBranchPointObj);
	pItem->SetParam0            (0xffffffff);
	pItem->SetTypo              (pObjName[0],pObjName[1],icon);
	pItem->Appear();
	pItem->SetVisible(true);
	if( pBranchPointObj->HasSelected()==true ){
		pItem->SetHighLighted(true );
		if( pBranchPointObj->HasSelected(0)==false ){
			pItem->SetSemiHighLighted(true );
		}else{
			pItem->SetSemiHighLighted(false);
		}
	}else{
		pItem->SetHighLighted(false);
	}
	pItem->SetType(WK7SelectItem::TYPE_LINE);
	
	return true;
}

WK7SelectIF*	WK7SelectIF::mpSelectIFInstance = NULL;

WK7SelectIF* WK7SelectIF::GetInstance(void)
{
	if( mpSelectIFInstance==NULL ){
		ghmGcMemSetAllocName("SelectIF");
		mpSelectIFInstance = new WK7SelectIF;
	}
	return mpSelectIFInstance;
}

WK7SelectItem* WK7SelectIF::GetSelectedItem(void)
{
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetEnable()==false ){
			continue;
		}
		if( pItem->GetSelected()==true ){
			return pItem;
		}
	}
	return NULL;
}

int WK7SelectIF::ReleaseBranch(WK7PathBranch *pBranch)
{
	if( pBranch==NULL ){
		return 0;
	}
	WK7SelectItem		*pItem,*pNext;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pNext){
		pNext = pItem->GetNext();
		if( pItem->GetPathBranch()!=pBranch ){
			continue;
		}
		pItem->mpBranch = NULL;
		pItem->mState   = WK7SelectItem::VANISHED;
//		delete pItem;
//		mItemNum--;
		count++;
	}
	return count;
}

int WK7SelectIF::ReleaseObj(WK7SelectObj *pObj)
{
	if( pObj==NULL ){
		return 0;
	}
	WK7SelectItem		*pItem,*pNext;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pNext){
		pNext = pItem->GetNext();
		if( pItem->GetObj()!=pObj ){
			continue;
		}
		delete pItem;
		mItemNum--;
		count++;
	}
	return count;
}

int WK7SelectIF::ReleasePathBranchPointObj(WK7PathBranchPointObj *pObj)
{
	if( pObj==NULL ){
		return 0;
	}
	WK7SelectItem		*pItem,*pNext;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pNext){
		pNext = pItem->GetNext();
		if( pItem->GetPathBranchPointObj()!=pObj ){
			continue;
		}
		pItem->mpBranchPointObj = NULL;
		pItem->mState           = WK7SelectItem::VANISHED;
//		delete pItem;
//		mItemNum--;
		count++;
	}
	return count;
}

WK7SelectItem* WK7SelectIF::GetItem(WK7PathBranch *pBranch)
{
	if( pBranch==NULL ){
		return NULL;
	}
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetPathBranch()!=pBranch ){
			return pItem;
		}
	}
	return NULL;
}

WK7SelectItem* WK7SelectIF::GetItem(WK7SelectObj *pObj)
{
	if( pObj==NULL ){
		return NULL;
	}
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetObj()==pObj ){
			return pItem;
		}
	}
	return NULL;
}

int WK7SelectIF::GetEnableItemNum(void)
{
	WK7SelectItem		*pItem;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetEnable()==false ){
			continue;
		}
		count++;
	}
	return count;
}

int WK7SelectIF::GetBranchItemNum(int enable)
{
	WK7SelectItem		*pItem;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetPathBranch()==NULL && pItem->GetPathBranchPointObj()==NULL ){
			continue;
		}
		if( enable >= 0 && pItem->GetEnable()!=enable ){
			continue;
		}
		count++;
	}
	return count;
}

int WK7SelectIF::GetObjItemNum(int enable)
{
	WK7SelectItem		*pItem;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetObj()==NULL ){
			continue;
		}
		if( enable >= 0 && pItem->GetEnable()!=enable ){
			continue;
		}
		count++;
	}
	return count;
}

int WK7SelectIF::GetItemNum(int type)
{
	WK7SelectItem		*pItem;
	int					count = 0;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem->GetType()!=type ){
			continue;
		}
		if( pItem->GetEnable()==false ){
			continue;
		}
		count++;
	}
	return count;
}

bool WK7SelectIF::IsValid(WK7SelectItem *pTgtItem,u32 id)
{
	WK7SelectItem		*pItem;
	for(pItem=(WK7SelectItem *)mAnchor.GetNext() ; pItem!=NULL ; pItem=pItem->GetNext()){
		if( pItem==pTgtItem ){
			if( id!=0xffffffff && pItem->GetID()!=id ){
				continue;
			}
			return true;
		}
	}
	return false;
}

struct WK7SelectIFStageDirOffset{
	u32			mStageID;
	f32			mDirOffset;
	int			mMultiID;
};
//////////////////////////////////////////////////////////////////////////
// Xe[W̕pItZbge[u
static WK7SelectIFStageDirOffset	stageDirOffset[] = {
	// tNV}ALXMXe[W
	{ K7STGID(115,0),  PI      , -1 },
	// prAciNELoj
	{ K7STGID(160,0), -HALF_PI , -1 },
	// eLTXAZXQe
	{ K7STGID(210,0),  PI      ,  2 },
	{ K7STGID(210,0),  PI      ,  3 },
	{ K7STGID(210,0),  PI      ,  4 },
	{ K7STGID(212,0),  PI      , -1 },
	// h~jJAt
	{ K7STGID(420,0),  PI      , -1 },
	{ K7STGID(420,1),  PI      , -1 },
	{ K7STGID(421,0),  PI      , -1 },
	{ K7STGID(422,0),  PI      , -1 },
	{ K7STGID(423,0),  PI      , -1 },
	{ K7STGID(425,0),  PI      , -1 },
	// wZÄ
	{ K7STGID(553,0),  PI      , -1 },
	// wZA
	{ K7STGID(559,0),  PI      , -1 },
	{ 0xffffffff    ,  0.0f    , -1 },
};
// Xe[W̕pItZbge[u
//////////////////////////////////////////////////////////////////////////
f32 WK7SelectIF::GetStageDirOffset(void)
{
	WK7SelectIFStageDirOffset	*pDirOffset;
	for(pDirOffset=stageDirOffset ; pDirOffset->mStageID!=0xffffffff ; pDirOffset++){
		if( pDirOffset->mStageID==WK7Stage::GetCurrentStageID () ||
		    pDirOffset->mStageID==WK7Stage::GetReleasedStageID() ){
			if( pDirOffset->mMultiID >= 0 ){
				WK7Stage	*pStage;
				if( (pStage=WK7RenderObj::GetStage(pDirOffset->mStageID))!=NULL &&
				    pStage->GetMultiID()==pDirOffset->mMultiID ){
					return pDirOffset->mDirOffset;
				}
			}else{
				return pDirOffset->mDirOffset;
			}
		}
	}
	return 0.0f;
}

